@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -25,34 +25,27 @@
 
 #include "utils.hpp"
 
+#define VDISPATCH_CONCAT(cond, msg, ...) \
+    VCONDCHECK(primitive, create, dispatch, concat, (cond), \
+            status::unimplemented, "%s," msg, this->info(engine), \
+            ##__VA_ARGS__)
+
+#define VDISPATCH_CONCAT_SC(f, msg, ...) \
+    VCHECK(primitive, create, dispatch, concat, (f), "%s," msg, \
+            this->info(engine), ##__VA_ARGS__)
+
 namespace dnnl {
 namespace impl {
 
+// NOLINTBEGIN(google-default-arguments)
 struct concat_pd_t : public primitive_desc_t {
-    concat_pd_t(const primitive_attr_t *attr, const memory_desc_t *dst_md,
-            int n, int concat_dim, const memory_desc_t *src_mds)
-        : primitive_desc_t(attr, primitive_kind::concat)
-        , n_(n)
-        , concat_dim_(concat_dim)
-        , dst_md_(*dst_md) {
-        src_mds_.reserve(n_);
-        for (int i = 0; i < n_; ++i)
-            src_mds_.push_back(src_mds[i]);
-
-        // Fill a desc that is intended for internal use only
-        desc_ = concat_desc_t();
-        desc_.primitive_kind = primitive_kind::concat;
-        desc_.dst_md = dst_md_;
-        desc_.n = n_;
-        desc_.concat_dimension = concat_dim_;
-        desc_.src_mds = src_mds_;
-    }
-
     const concat_desc_t *desc() const { return &desc_; }
     const op_desc_t *op_desc() const override {
         return reinterpret_cast<const op_desc_t *>(this->desc());
     }
 
+    ~concat_pd_t() override = default;
+
     arg_usage_t arg_usage(int arg) const override {
         if (arg >= DNNL_ARG_MULTIPLE_SRC
                 && arg < DNNL_ARG_MULTIPLE_SRC + n_inputs())
@@ -63,18 +56,24 @@ struct concat_pd_t : public primitive_desc_t {
         return primitive_desc_t::arg_usage(arg);
     }
 
-    const memory_desc_t *arg_md(int arg) const override {
+    const memory_desc_t *arg_md(
+            int arg, bool user_input = false) const override {
         int src_index = arg - DNNL_ARG_MULTIPLE_SRC;
         if (src_index >= 0 && src_index < n_inputs()) return src_md(src_index);
-        if (arg == DNNL_ARG_DST) return dst_md(0);
+        if (arg == DNNL_ARG_DST) return dst_md(0, user_input);
         return primitive_desc_t::arg_md(arg);
     }
 
-    const memory_desc_t *src_md(int index = 0) const override {
-        return index < n_inputs() ? &src_mds_[index] : &glob_zero_md;
+    const memory_desc_t *src_md(
+            int index = 0, bool user_input = false) const override {
+        if (index < n_inputs())
+            return user_input ? desc()->src_mds[index] : &src_mds_[index];
+        return &glob_zero_md;
     }
-    const memory_desc_t *dst_md(int index = 0) const override {
-        return index == 0 ? &dst_md_ : &glob_zero_md;
+    const memory_desc_t *dst_md(
+            int index = 0, bool user_input = false) const override {
+        if (index == 0) return user_input ? desc()->dst_md : &dst_md_;
+        return &glob_zero_md;
     }
 
     int n_inputs() const override { return n_; }
@@ -89,6 +88,7 @@ struct concat_pd_t : public primitive_desc_t {
 protected:
     int n_, concat_dim_;
     memory_desc_t dst_md_;
+    memory_desc_t original_dst_;
     std::vector<memory_desc_t> src_mds_;
 
     /* contains images of srcs in the dst memory (if possible)
@@ -96,9 +96,46 @@ protected:
      * use this auxiliary array iff init() returned success */
     std::vector<memory_desc_t> src_image_mds_;
 
-protected:
     concat_desc_t desc_;
 
+    concat_pd_t(const primitive_attr_t *attr, const memory_desc_t *dst_md,
+            int n, int concat_dim, const memory_desc_t *const *src_mds)
+        : primitive_desc_t(attr, primitive_kind::concat)
+        , n_(n)
+        , concat_dim_(concat_dim)
+        , dst_md_(*dst_md)
+        , original_dst_(*dst_md) {
+        src_mds_.reserve(n_);
+        for (int i = 0; i < n_; ++i)
+            src_mds_.push_back(*src_mds[i]);
+
+        init_desc();
+    }
+
+    concat_pd_t(const concat_pd_t &other)
+        : primitive_desc_t(other)
+        , n_(other.n_)
+        , concat_dim_(other.concat_dim_)
+        , dst_md_(other.dst_md_)
+        , original_dst_(other.original_dst_)
+        , src_mds_(other.src_mds_)
+        , src_image_mds_(other.src_image_mds_) {
+        init_desc();
+    }
+
+    concat_pd_t &operator=(const concat_pd_t &other) {
+        DNNL_SHORT_CIRCUIT_SELF_ASSIGN(other);
+        n_ = other.n_;
+        concat_dim_ = other.concat_dim_;
+        dst_md_ = other.dst_md_;
+        original_dst_ = other.original_dst_;
+        src_mds_ = other.src_mds_;
+        src_image_mds_ = other.src_image_mds_;
+
+        init_desc();
+        return *this;
+    }
+
     /* inits src_image_mds_ and dst_md_ in simple cases. It is possible to
      * override dst_md_ by using force_dst_md.
      * Rationale: if user forces particular dst_md, that cannot be used to
@@ -107,7 +144,7 @@ protected:
      *
      * @warning The call may fail. */
     status_t init(const memory_desc_t *force_dst_md = nullptr) {
-        bool ok = attr()->has_default_values();
+        bool ok = true;
         if (force_dst_md == nullptr)
             ok = ok && set_default_params() == status::success;
         if (!ok) return status::unimplemented;
@@ -122,17 +159,17 @@ protected:
         }
 
         const int ndims = force_dst_md->ndims;
-        int current_concat_dim_offset = 0;
+        dim_t current_concat_dim_offset = 0;
         for (int i = 0; i < n_; ++i) {
-            const int dim = src_mds_[i].dims[concat_dim_];
+            const dim_t dim = src_mds_[i].dims[concat_dim_];
             dims_t dims, offsets = {};
             utils::array_copy(dims, force_dst_md->dims, ndims);
             dims[concat_dim_] = dim;
             offsets[concat_dim_] = current_concat_dim_offset;
 
             memory_desc_t src_img_d;
-            status_t status = dnnl_memory_desc_init_submemory(
-                    &src_img_d, force_dst_md, dims, offsets);
+            status_t status = memory_desc_init_submemory(
+                    src_img_d, *force_dst_md, dims, offsets);
             if (status != status::success) {
                 src_image_mds_.clear();
                 return status;
@@ -168,22 +205,21 @@ protected:
         if (status == status::success) {
             /* check if we can create a sub-memory for the dst */
             bool desired_format_ok = true;
-            int current_concat_dim_offset = 0;
+            dims_t dims {}, offsets {};
+            utils::array_copy(dims, dst_md_.dims, ndims);
+
             for (int i = 0; i < n_; ++i) {
-                const int dim = src_mds_[i].dims[concat_dim_];
-                dims_t dims, offsets = {};
-                utils::array_copy(dims, dst_md_.dims, ndims);
+                const auto dim = src_mds_[i].dims[concat_dim_];
                 dims[concat_dim_] = dim;
-                offsets[concat_dim_] = current_concat_dim_offset;
 
                 memory_desc_t src_img_d;
-                status_t status = dnnl_memory_desc_init_submemory(
-                        &src_img_d, &dst_md_, dims, offsets);
+                status_t status = memory_desc_init_submemory(
+                        src_img_d, dst_md_, dims, offsets);
                 if (status != status::success) {
                     desired_format_ok = false;
                     break;
                 }
-                current_concat_dim_offset += dim;
+                offsets[concat_dim_] += dim;
             }
 
             if (!desired_format_ok) status = status::unimplemented;
@@ -193,8 +229,18 @@ protected:
         if (status != status::success) {
             for (int i = 0; i < n_; ++i) {
                 const memory_desc_wrapper src_d(src_mds_[i]);
-                if (src_d.is_blocking_desc() && src_d.is_plain()
-                        && src_d.nelems() > 0) {
+                // Dim of `1` may tweak a destination format leading to
+                // sub-optimal performance. Limit it to an axis case to allow
+                // case like a:a->ab or a:ab->ab to work properly.
+                // TODO: update the whole logic to getting string tags of
+                // sources but discarding dims of one. If ndims of any source
+                // coincides with dst ndims, use that tag (if they are same).
+                // If dst has +1 ndim (due to concat dim), use slices as dense
+                // layers inside a dst, which means axis should be the least
+                // dense dimension.
+                const bool axis_dim_has_one = src_d.dims()[concat_dim()] == 1;
+                if (!axis_dim_has_one && src_d.is_blocking_desc()
+                        && src_d.is_plain() && src_d.nelems() > 0) {
                     status = memory_desc_init_by_blocking_desc(dst_md_,
                             memory_desc_wrapper(src_mds_[i]).blocking_desc());
                     if (status == status::success) return status;
@@ -208,35 +254,46 @@ protected:
 
         return status;
     }
+
+private:
+    void init_desc() {
+        desc_ = concat_desc_t();
+        desc_.primitive_kind = primitive_kind::concat;
+        desc_.dst_md = &original_dst_;
+        desc_.n = n_;
+        desc_.concat_dimension = concat_dim_;
+        for (const auto &md : src_mds_)
+            desc_.src_mds.push_back(&md);
+    }
 };
+// NOLINTEND(google-default-arguments)
 
 #define DECLARE_CONCAT_PD_t(impl_name, ...) \
-    static status_t create(concat_pd_t **concat_pd, engine_t *engine, \
-            const primitive_attr_t *attr, const memory_desc_t *dst_md, int n, \
-            int concat_dim, const memory_desc_t *src_mds) { \
+    static status_t create(concat_pd_t **concat_pd, \
+            dnnl::impl::engine_t *engine, const primitive_attr_t *attr, \
+            const memory_desc_t *dst_md, int n, int concat_dim, \
+            const memory_desc_t *const *src_mds) { \
         using namespace status; \
-        auto _pd = new pd_t(attr, dst_md, n, concat_dim, src_mds); \
+        auto _pd = make_unique_pd<pd_t>(attr, dst_md, n, concat_dim, src_mds); \
         if (_pd == nullptr) return out_of_memory; \
-        if (_pd->init(engine) != success) { \
-            delete _pd; \
-            return unimplemented; \
-        } \
-        _pd->init_scratchpad_md(); \
-        return safe_ptr_assign(*concat_pd, _pd); \
+        CHECK(_pd->init(engine)); \
+        CHECK(_pd->init_scratchpad_md()); \
+        return safe_ptr_assign(*concat_pd, _pd.release()); \
     } \
     status_t create_primitive( \
-            std::pair<std::shared_ptr<primitive_t>, bool> &primitive, \
-            engine_t *engine) const override { \
+            std::pair<std::shared_ptr<impl::primitive_t>, cache_state_t> \
+                    &primitive, \
+            dnnl::impl::engine_t *engine, const cache_blob_t &cache_blob) \
+            const override { \
         return primitive_t::create_primitive_common<__VA_ARGS__, pd_t>( \
-                primitive, this, engine, false); \
+                primitive, this, engine, false, cache_blob); \
     } \
     pd_t *clone() const override { \
         auto new_pd = utils::make_unique<pd_t>(*this); \
         if (!new_pd->is_initialized()) return nullptr; \
         return new_pd.release(); \
     } \
-    const char *name() const override { return impl_name; } \
-    std::type_index impl_id() const override { return typeid(pd_t); }
+    const char *name() const override { return impl_name; }
 
 #define DECLARE_CONCAT_PD_T(impl_name, ...) \
     DECLARE_CONCAT_PD_t(impl_name, __VA_ARGS__)
