@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -23,6 +23,19 @@
 #include "primitive_desc.hpp"
 #include "utils.hpp"
 
+#define VDISPATCH_CONV(cond, msg, ...) \
+    VCONDCHECK(primitive, create, dispatch, convolution, (cond), \
+            status::unimplemented, "%s," msg, this->info(engine), \
+            ##__VA_ARGS__)
+
+#define VDISPATCH_CONV_SC(f, msg, ...) \
+    VCHECK(primitive, create, dispatch, convolution, (f), "%s," msg, \
+            this->info(engine), ##__VA_ARGS__)
+
+#define VDISPATCH_CONV_IC(cond, msg, ...) \
+    VCONDCHECK(primitive, create, dispatch, convolution, (cond), \
+            status::unimplemented, msg, ##__VA_ARGS__);
+
 namespace dnnl {
 namespace impl {
 
@@ -46,13 +59,6 @@ struct convolution_fwd_pd_t;
 struct convolution_pd_t : public primitive_desc_t {
     static constexpr auto base_pkind = primitive_kind::convolution;
 
-    convolution_pd_t(const convolution_desc_t *adesc,
-            const primitive_attr_t *attr,
-            const convolution_fwd_pd_t *hint_fwd_pd)
-        : primitive_desc_t(attr, base_pkind)
-        , desc_(*adesc)
-        , hint_fwd_pd_(hint_fwd_pd) {}
-
     const convolution_desc_t *desc() const { return &desc_; }
     const op_desc_t *op_desc() const override {
         return reinterpret_cast<const op_desc_t *>(this->desc());
@@ -63,8 +69,20 @@ struct convolution_pd_t : public primitive_desc_t {
             case query::prop_kind:
                 *(prop_kind_t *)result = desc()->prop_kind;
                 break;
-            case pkind_traits<base_pkind>::query_d:
-                *(const convolution_desc_t **)result = desc();
+            case query::alg_kind:
+                *(alg_kind_t *)result = desc()->alg_kind;
+                break;
+            case query::strides:
+                *(const dims_t **)result = &desc()->strides;
+                break;
+            case query::dilations:
+                *(const dims_t **)result = &desc()->dilates;
+                break;
+            case query::padding_l:
+                *(const dims_t **)result = &desc()->padding[0];
+                break;
+            case query::padding_r:
+                *(const dims_t **)result = &desc()->padding[1];
                 break;
             default: return primitive_desc_t::query(what, idx, result);
         }
@@ -114,7 +132,7 @@ struct convolution_pd_t : public primitive_desc_t {
     dim_t KSW() const { return desc_.strides[ndims() - 3]; }
 
     dim_t KDD() const { return ndims() >= 5 ? desc_.dilates[ndims() - 5] : 0; }
-    dim_t KDH() const { return ndims() >= 4 ? desc_.dilates[ndims() - 4] : 1; }
+    dim_t KDH() const { return ndims() >= 4 ? desc_.dilates[ndims() - 4] : 0; }
     dim_t KDW() const { return desc_.dilates[ndims() - 3]; }
 
     dim_t padFront() const {
@@ -135,10 +153,7 @@ struct convolution_pd_t : public primitive_desc_t {
     int ndims() const { return invariant_src_md()->ndims; }
 
     bool with_bias() const {
-        auto *bia_d = desc()->prop_kind == prop_kind::backward_weights
-                ? &desc()->diff_bias_desc
-                : &desc()->bias_desc;
-        return !memory_desc_wrapper(bia_d).is_zero();
+        return !memory_desc_wrapper(invariant_bia_md()).is_zero();
     }
     bool with_groups() const {
         return invariant_wei_md()->ndims == ndims() + 1;
@@ -163,30 +178,20 @@ struct convolution_pd_t : public primitive_desc_t {
         return s_d.has_zero_dim() || d_d.has_zero_dim();
     }
 
-    const memory_desc_t *invariant_src_md() const {
-        return desc()->prop_kind == prop_kind::backward_data ? diff_src_md()
-                                                             : src_md();
-    }
-    const memory_desc_t *invariant_wei_md(int index = 0) const {
-        return desc()->prop_kind == prop_kind::backward_weights
-                ? diff_weights_md(index)
-                : weights_md(index);
-    }
-    const memory_desc_t *invariant_bia_md() const {
-        return invariant_wei_md(1);
-    }
-    const memory_desc_t *invariant_dst_md() const {
-        return is_fwd() ? dst_md() : diff_dst_md();
-    }
-
 protected:
     convolution_desc_t desc_;
     const convolution_fwd_pd_t *hint_fwd_pd_;
 
+    convolution_pd_t(const op_desc_t *adesc, const primitive_attr_t *attr,
+            const convolution_fwd_pd_t *hint_fwd_pd)
+        : primitive_desc_t(attr, base_pkind)
+        , desc_(*op_desc_t::to_desc<convolution_desc_t>(adesc))
+        , hint_fwd_pd_(hint_fwd_pd) {}
+
     bool set_default_formats_common_template(memory_desc_t &src_md,
             format_tag_t src_tag, memory_desc_t &wei_md, format_tag_t wei_tag,
             memory_desc_t &dst_md, format_tag_t dst_tag,
-            memory_desc_t &bia_md) {
+            memory_desc_t &bia_md) const {
         using namespace format_tag;
 
 #define IS_OK(f) \
@@ -232,57 +237,74 @@ protected:
             ok = ok && invariant_bia_md()->data_type == bia_dt;
         return ok;
     }
+
+    bool attr_scales_ok(const std::vector<int> &supported_args
+            = {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST}) const {
+        bool ok = attr()->scales_.has_default_values(supported_args);
+        for (int arg : supported_args) {
+            if (attr()->scales_.has_default_values(arg)) continue;
+
+            const auto &mask = attr()->scales_.get_mask(arg);
+            if (arg == DNNL_ARG_WEIGHTS)
+                ok = ok && (mask == 0 || mask == (with_groups() ? 3 : 1));
+            else if (arg == DNNL_ARG_DST)
+                ok = ok && (mask == 0 || mask == 2);
+            else
+                ok = ok && (mask == 0);
+        }
+        return ok;
+    }
 };
 
+// NOLINTBEGIN(google-default-arguments)
 struct convolution_fwd_pd_t : public convolution_pd_t {
-    typedef convolution_fwd_pd_t base_class;
-    typedef convolution_fwd_pd_t hint_class;
-
-    convolution_fwd_pd_t(const convolution_desc_t *adesc,
-            const primitive_attr_t *attr,
-            const convolution_fwd_pd_t *hint_fwd_pd)
-        : convolution_pd_t(adesc, attr, hint_fwd_pd)
-        , src_md_(desc_.src_desc)
-        , weights_md_(desc_.weights_desc)
-        , bias_md_(desc_.bias_desc)
-        , dst_md_(desc_.dst_desc) {}
+    using base_class = convolution_fwd_pd_t;
+    using hint_class = convolution_fwd_pd_t;
 
     arg_usage_t arg_usage(int arg) const override {
         if (utils::one_of(arg, DNNL_ARG_SRC, DNNL_ARG_WEIGHTS))
             return arg_usage_t::input;
 
-        if (arg == DNNL_ARG_BIAS && with_bias()) return arg_usage_t::input;
+        if (arg == DNNL_ARG_BIAS)
+            return with_bias() ? arg_usage_t::input : arg_usage_t::unused;
 
         if (arg == DNNL_ARG_DST) return arg_usage_t::output;
 
         return primitive_desc_t::arg_usage(arg);
     }
 
-    const memory_desc_t *arg_md(int arg) const override {
+    const memory_desc_t *arg_md(
+            int arg, bool user_input = false) const override {
         switch (arg) {
             case DNNL_ARG_SRC: return src_md(0);
             case DNNL_ARG_WEIGHTS: return weights_md(0);
             case DNNL_ARG_BIAS: return weights_md(1);
-            case DNNL_ARG_DST: return dst_md(0);
+            case DNNL_ARG_DST: return dst_md(0, user_input);
             default: return convolution_pd_t::arg_md(arg);
         }
     }
 
-    const memory_desc_t *src_md(int index = 0) const override {
-        return index == 0 ? &src_md_ : &glob_zero_md;
+    const memory_desc_t *src_md(
+            int index = 0, bool user_input = false) const override {
+        if (index == 0) return user_input ? &desc()->src_desc : &src_md_;
+        return &glob_zero_md;
     }
-    const memory_desc_t *dst_md(int index = 0) const override {
-        return index == 0 ? &dst_md_ : &glob_zero_md;
+    const memory_desc_t *dst_md(
+            int index = 0, bool user_input = false) const override {
+        if (index == 0) return user_input ? &desc()->dst_desc : &dst_md_;
+        return &glob_zero_md;
     }
-    const memory_desc_t *weights_md(int index = 0) const override {
-        if (index == 0) return &weights_md_;
-        if (index == 1 && with_bias()) return &bias_md_;
+    const memory_desc_t *weights_md(
+            int index = 0, bool user_input = false) const override {
+        if (index == 0)
+            return user_input ? &desc()->weights_desc : &weights_md_;
+        if (index == 1) return user_input ? &desc()->bias_desc : &bias_md_;
         return &glob_zero_md;
     }
 
     int n_inputs() const override {
-        return 2 + with_bias() + attr_post_op_dw_inputs()
-                + n_binary_po_inputs();
+        return 2 + with_bias() + attr_post_op_dw_inputs() + n_binary_po_inputs()
+                + n_prelu_po_inputs();
     }
 
     int n_outputs() const override { return 1; }
@@ -293,6 +315,14 @@ protected:
     memory_desc_t bias_md_;
     memory_desc_t dst_md_;
 
+    convolution_fwd_pd_t(const op_desc_t *adesc, const primitive_attr_t *attr,
+            const convolution_fwd_pd_t *hint_fwd_pd)
+        : convolution_pd_t(adesc, attr, hint_fwd_pd)
+        , src_md_(desc_.src_desc)
+        , weights_md_(desc_.weights_desc)
+        , bias_md_(desc_.bias_desc)
+        , dst_md_(desc_.dst_desc) {}
+
     bool set_default_formats_common(
             format_tag_t src_tag, format_tag_t wei_tag, format_tag_t dst_tag) {
         return set_default_formats_common_template(src_md_, src_tag,
@@ -307,19 +337,12 @@ protected:
                                                                           : 2;
     }
 };
+// NOLINTEND(google-default-arguments)
 
+// NOLINTBEGIN(google-default-arguments)
 struct convolution_bwd_data_pd_t : public convolution_pd_t {
-    typedef convolution_bwd_data_pd_t base_class;
-    typedef convolution_fwd_pd_t hint_class;
-
-    convolution_bwd_data_pd_t(const convolution_desc_t *adesc,
-            const primitive_attr_t *attr,
-            const convolution_fwd_pd_t *hint_fwd_pd)
-        : convolution_pd_t(adesc, attr, hint_fwd_pd)
-        , diff_src_md_(desc_.diff_src_desc)
-        , weights_md_(desc_.weights_desc)
-        , bias_md_(desc_.bias_desc)
-        , diff_dst_md_(desc_.diff_dst_desc) {}
+    using base_class = convolution_bwd_data_pd_t;
+    using hint_class = convolution_fwd_pd_t;
 
     arg_usage_t arg_usage(int arg) const override {
         if (utils::one_of(arg, DNNL_ARG_WEIGHTS, DNNL_ARG_DIFF_DST))
@@ -330,25 +353,34 @@ struct convolution_bwd_data_pd_t : public convolution_pd_t {
         return primitive_desc_t::arg_usage(arg);
     }
 
-    const memory_desc_t *arg_md(int arg) const override {
+    const memory_desc_t *arg_md(
+            int arg, bool user_input = false) const override {
         switch (arg) {
             case DNNL_ARG_DIFF_SRC: return diff_src_md(0);
             case DNNL_ARG_WEIGHTS: return weights_md(0);
             case DNNL_ARG_BIAS: return weights_md(1);
-            case DNNL_ARG_DIFF_DST: return diff_dst_md(0);
+            case DNNL_ARG_DIFF_DST: return diff_dst_md(0, user_input);
             default: return convolution_pd_t::arg_md(arg);
         }
     }
 
-    const memory_desc_t *diff_src_md(int index = 0) const override {
-        return index == 0 ? &diff_src_md_ : &glob_zero_md;
+    const memory_desc_t *diff_src_md(
+            int index = 0, bool user_input = false) const override {
+        if (index == 0)
+            return user_input ? &desc()->diff_src_desc : &diff_src_md_;
+        return &glob_zero_md;
     }
-    const memory_desc_t *diff_dst_md(int index = 0) const override {
-        return index == 0 ? &diff_dst_md_ : &glob_zero_md;
+    const memory_desc_t *diff_dst_md(
+            int index = 0, bool user_input = false) const override {
+        if (index == 0)
+            return user_input ? &desc()->diff_dst_desc : &diff_dst_md_;
+        return &glob_zero_md;
     }
-    const memory_desc_t *weights_md(int index = 0) const override {
-        if (index == 0) return &weights_md_;
-        if (index == 1 && with_bias()) return &bias_md_;
+    const memory_desc_t *weights_md(
+            int index = 0, bool user_input = false) const override {
+        if (index == 0)
+            return user_input ? &desc()->weights_desc : &weights_md_;
+        if (index == 1) return user_input ? &desc()->bias_desc : &bias_md_;
         return &glob_zero_md;
     }
 
@@ -363,18 +395,29 @@ protected:
     memory_desc_t bias_md_;
     memory_desc_t diff_dst_md_;
 
+    convolution_bwd_data_pd_t(const op_desc_t *adesc,
+            const primitive_attr_t *attr,
+            const convolution_fwd_pd_t *hint_fwd_pd)
+        : convolution_pd_t(adesc, attr, hint_fwd_pd)
+        , diff_src_md_(desc_.diff_src_desc)
+        , weights_md_(desc_.weights_desc)
+        , bias_md_(desc_.bias_desc)
+        , diff_dst_md_(desc_.diff_dst_desc) {}
+
     bool set_default_formats_common(format_tag_t diff_src_tag,
             format_tag_t wei_tag, format_tag_t diff_dst_tag) {
         return set_default_formats_common_template(diff_src_md_, diff_src_tag,
                 weights_md_, wei_tag, diff_dst_md_, diff_dst_tag, bias_md_);
     }
 };
+// NOLINTEND(google-default-arguments)
 
+// NOLINTBEGIN(google-default-arguments)
 struct convolution_bwd_weights_pd_t : public convolution_pd_t {
-    typedef convolution_bwd_weights_pd_t base_class;
-    typedef convolution_fwd_pd_t hint_class;
+    using base_class = convolution_bwd_weights_pd_t;
+    using hint_class = convolution_fwd_pd_t;
 
-    convolution_bwd_weights_pd_t(const convolution_desc_t *adesc,
+    convolution_bwd_weights_pd_t(const op_desc_t *adesc,
             const primitive_attr_t *attr,
             const convolution_fwd_pd_t *hint_fwd_pd)
         : convolution_pd_t(adesc, attr, hint_fwd_pd)
@@ -389,31 +432,40 @@ struct convolution_bwd_weights_pd_t : public convolution_pd_t {
 
         if (arg == DNNL_ARG_DIFF_WEIGHTS) return arg_usage_t::output;
 
-        if (arg == DNNL_ARG_DIFF_BIAS && with_bias())
-            return arg_usage_t::output;
+        if (arg == DNNL_ARG_DIFF_BIAS)
+            return with_bias() ? arg_usage_t::output : arg_usage_t::unused;
 
         return primitive_desc_t::arg_usage(arg);
     }
 
-    const memory_desc_t *arg_md(int arg) const override {
+    const memory_desc_t *arg_md(
+            int arg, bool user_input = false) const override {
         switch (arg) {
             case DNNL_ARG_SRC: return src_md(0);
             case DNNL_ARG_DIFF_WEIGHTS: return diff_weights_md(0);
             case DNNL_ARG_DIFF_BIAS: return diff_weights_md(1);
-            case DNNL_ARG_DIFF_DST: return diff_dst_md(0);
+            case DNNL_ARG_DIFF_DST: return diff_dst_md(0, user_input);
             default: return convolution_pd_t::arg_md(arg);
         }
     }
 
-    const memory_desc_t *src_md(int index = 0) const override {
-        return index == 0 ? &src_md_ : &glob_zero_md;
+    const memory_desc_t *src_md(
+            int index = 0, bool user_input = false) const override {
+        if (index == 0) return user_input ? &desc()->src_desc : &src_md_;
+        return &glob_zero_md;
     }
-    const memory_desc_t *diff_dst_md(int index = 0) const override {
-        return index == 0 ? &diff_dst_md_ : &glob_zero_md;
+    const memory_desc_t *diff_dst_md(
+            int index = 0, bool user_input = false) const override {
+        if (index == 0)
+            return user_input ? &desc()->diff_dst_desc : &diff_dst_md_;
+        return &glob_zero_md;
     }
-    const memory_desc_t *diff_weights_md(int index = 0) const override {
-        if (index == 0) return &diff_weights_md_;
-        if (index == 1 && with_bias()) return &diff_bias_md_;
+    const memory_desc_t *diff_weights_md(
+            int index = 0, bool user_input = false) const override {
+        if (index == 0)
+            return user_input ? &desc()->diff_weights_desc : &diff_weights_md_;
+        if (index == 1)
+            return user_input ? &desc()->diff_bias_desc : &diff_bias_md_;
         return &glob_zero_md;
     }
 
@@ -433,6 +485,7 @@ protected:
                 diff_bias_md_);
     }
 };
+// NOLINTEND(google-default-arguments)
 
 } // namespace impl
 } // namespace dnnl
