@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -22,14 +22,19 @@
 #include "c_types_map.hpp"
 #include "nstl.hpp"
 #include "utils.hpp"
+#include "verbose.hpp"
 
 #include "type_helpers.hpp"
 
+#define VCHECK_MEMORY(cond, stat, msg, ...) \
+    VCONDCHECK(common, create, check, memory, (cond), stat, msg, ##__VA_ARGS__)
+
 namespace dnnl {
 namespace impl {
 
 /** thin wrapper class over \struct memory_desc_t which allows easy
  * manipulations with underlying C structure, which is taken by reference */
+// NOLINTNEXTLINE(readability-identifier-naming)
 struct memory_desc_wrapper : public c_compatible {
     const memory_desc_t *md_;
 
@@ -53,14 +58,25 @@ struct memory_desc_wrapper : public c_compatible {
     bool is_blocking_desc() const {
         return format_kind() == format_kind::blocked;
     }
+
+    bool is_sparse_packed_desc() const {
+        return is_sparse_desc()
+                && sparse_desc().encoding == sparse_encoding::packed;
+    }
+
     bool is_wino_desc() const { return format_kind() == format_kind::wino; }
     bool is_rnn_packed_desc() const {
         return format_kind() == format_kind::rnn_packed;
     }
+    bool is_cublaslt_blocked_desc() const {
+        return format_kind() == format_kind::cublaslt_blocked;
+    }
+    bool is_sparse_desc() const { return format_kind() == format_kind::sparse; }
 
     const blocking_desc_t &blocking_desc() const {
-        assert(is_blocking_desc());
-        return md_->format_desc.blocking;
+        assert(is_blocking_desc() || is_sparse_packed_desc());
+        if (!is_sparse_desc()) return md_->format_desc.blocking;
+        return sparse_desc().packed_desc;
     }
     const wino_desc_t &wino_desc() const {
         assert(is_wino_desc());
@@ -70,6 +86,32 @@ struct memory_desc_wrapper : public c_compatible {
         assert(is_rnn_packed_desc());
         return md_->format_desc.rnn_packed_desc;
     }
+    const cublaslt_blocked_desc_t &cublaslt_blocked_desc() const {
+        assert(is_cublaslt_blocked_desc());
+        return md_->format_desc.cublaslt_blocked_desc;
+    }
+
+    const sparse_desc_t &sparse_desc() const {
+        assert(is_sparse_desc());
+        return md_->format_desc.sparse_desc;
+    }
+
+    data_type_t metadata_type(int idx = 0) const {
+        assert(is_sparse_desc() && idx < sparse_desc_t::max_metadata_types);
+        return sparse_desc().metadata_types[idx];
+    }
+
+    sparse_encoding_t encoding() const {
+        assert(is_sparse_desc());
+        return sparse_desc().encoding;
+    }
+
+    dim_t nnz() const {
+        assert(is_sparse_desc());
+        return sparse_desc().nnz;
+    }
+
+    const dims_t &strides() const { return blocking_desc().strides; }
 
     const memory_extra_desc_t &extra() const { return md_->extra; }
 
@@ -97,13 +139,21 @@ struct memory_desc_wrapper : public c_compatible {
     /** return the size of data type (a shortcut) */
     size_t data_type_size() const { return types::data_type_size(data_type()); }
 
+    /** For sub-byte data types returns number of elements per byte.
+     * For the rest data types returns 1. */
+    size_t sub_byte_data_type_multiplier() const {
+        if (utils::one_of(data_type(), data_type::s4, data_type::u4)) return 2;
+        return 1;
+    }
+
     /** return the size of data type of additional buffer */
     size_t additional_buffer_data_size(uint64_t flag_select) const {
-        if (flag_select & memory_extra_flags::compensation_conv_s8s8)
+        using namespace memory_extra_flags;
+        if (flag_select & compensation_conv_s8s8) return sizeof(int32_t);
+        if (flag_select & rnn_u8s8_compensation) return sizeof(float);
+        if (flag_select & compensation_conv_asymmetric_src)
             return sizeof(int32_t);
-        if (flag_select & memory_extra_flags::rnn_u8s8_compensation)
-            return sizeof(float);
-        if (flag_select & memory_extra_flags::compensation_conv_asymmetric_src)
+        if (flag_select & compensation_gpu_conv_asymmetric_src)
             return sizeof(int32_t);
         return 0;
     }
@@ -111,9 +161,53 @@ struct memory_desc_wrapper : public c_compatible {
     /** return true if memory format has additional buffer */
     bool is_additional_buffer() const {
         using namespace memory_extra_flags;
-        return (extra().flags
+        return extra().flags
                 & (compensation_conv_s8s8 | rnn_u8s8_compensation
-                        | compensation_conv_asymmetric_src));
+                        | compensation_gpu_conv_asymmetric_src
+                        | compensation_conv_asymmetric_src);
+    }
+
+    /** returns the size required for a particular extra memory buffer */
+    size_t additional_buffer_size(memory_extra_flags_t flag) const {
+        using namespace memory_extra_flags;
+        const auto flags = extra().flags;
+        if (!(flags & flag)) return 0;
+
+        const auto ndims = this->ndims();
+        const auto &pdims = padded_dims();
+
+        auto calculate_size
+                = [ndims, &pdims](int cmask, size_t buff_data_size) {
+                      assert(utils::one_of(cmask, 1, 2, 3, 5, 13, 27));
+                      dim_t prod = 1;
+                      for (int d = 0; d < ndims; ++d)
+                          if (cmask & (1 << d)) { prod *= pdims[d]; }
+                      return (size_t)prod * buff_data_size;
+                  };
+
+        if (flag == compensation_conv_s8s8) {
+            return calculate_size(extra().compensation_mask,
+                    additional_buffer_data_size(flag));
+        }
+        if (flag == rnn_u8s8_compensation) {
+            return calculate_size(extra().compensation_mask,
+                    additional_buffer_data_size(flag));
+        }
+        if (flag == compensation_conv_asymmetric_src) {
+            return calculate_size(extra().asymm_compensation_mask,
+                    additional_buffer_data_size(flag));
+        }
+        if (flag == compensation_gpu_conv_asymmetric_src) {
+            return extra().dst_size;
+        }
+
+        return 0;
+    }
+
+    dim_t blk_size() const {
+        assert(is_blocking_desc() || is_sparse_packed_desc());
+        const auto &bd = blocking_desc();
+        return utils::array_product(bd.inner_blks, bd.inner_nblks);
     }
 
     /** returns the size of the appended buffer when the memory descriptor
@@ -121,45 +215,39 @@ struct memory_desc_wrapper : public c_compatible {
     size_t additional_buffer_size() const {
         using namespace memory_extra_flags;
 
-        auto calculate_size = [=](int cmask, size_t buff_data_size) {
-            assert(cmask == 1 || cmask == 3 || cmask == 13 || cmask == 27);
-            dim_t prod = 1;
-            for (int d = 0; d < ndims(); ++d)
-                if (cmask & (1 << d)) prod *= padded_dims()[d];
-            return (size_t)prod * buff_data_size;
-        };
-
         size_t buff_size = 0;
-        const uint64_t comp_flags
-                = compensation_conv_s8s8 | rnn_u8s8_compensation;
-        if (extra().flags & comp_flags) {
-            buff_size += calculate_size(extra().compensation_mask,
-                    additional_buffer_data_size(comp_flags));
-        }
-        if (extra().flags & compensation_conv_asymmetric_src) {
-            buff_size += calculate_size(extra().asymm_compensation_mask,
-                    additional_buffer_data_size(
-                            compensation_conv_asymmetric_src));
-        }
+        buff_size += additional_buffer_size(compensation_conv_s8s8);
+        buff_size += additional_buffer_size(rnn_u8s8_compensation);
+        buff_size += additional_buffer_size(compensation_conv_asymmetric_src);
+        buff_size
+                += additional_buffer_size(compensation_gpu_conv_asymmetric_src);
         return buff_size;
     }
 
-    /** returns the size required to store described memory
-     * note: if offset0 != 0 returns 0 (need to specify the behavior) */
-    size_t size() const {
+    /** returns the size required to store described memory note: does not
+        include offset0 by default */
+    size_t size(int index = 0, bool include_additional_size = true,
+            bool include_offset0 = false) const {
         if (utils::one_of(format_kind(), format_kind::undef, format_kind::any)
                 || is_zero() || has_zero_dim())
             return 0;
 
+        if (utils::one_of(format_kind(), format_kind::blocked,
+                    format_kind::wino, format_kind::rnn_packed,
+                    format_kind::cublaslt_blocked)
+                && index != 0) {
+            return 0;
+        }
+
         if (has_runtime_dims_or_strides()) return DNNL_RUNTIME_SIZE_VAL;
 
-        if (format_kind() == format_kind::wino) {
+        if (is_wino_desc()) {
             return wino_desc().size;
-        } else if (format_kind() == format_kind::rnn_packed) {
+        } else if (is_rnn_packed_desc()) {
             return rnn_packed_desc().size;
-        } else {
-            if (offset0() != 0) return 0;
-
+        } else if (is_cublaslt_blocked_desc()) {
+            return cublaslt_blocked_desc().size;
+        } else if (is_blocking_desc()) {
             dims_t blocks = {0};
             compute_blocks(blocks);
 
@@ -174,10 +262,77 @@ struct memory_desc_wrapper : public c_compatible {
             }
 
             if (max_size == 1 && bd.inner_nblks != 0) {
-                max_size = utils::array_product(bd.inner_blks, bd.inner_nblks);
+                max_size = static_cast<size_t>(blk_size());
             }
 
-            return max_size * data_type_size() + additional_buffer_size();
+            // `div_up` guarantees a spot in memory for odd number of half-byte
+            // elements. Crucial case is `1` when simple division returns 0.
+            size_t data_size = utils::div_up(max_size * data_type_size(),
+                    sub_byte_data_type_multiplier());
+            if (is_additional_buffer()) {
+                // The additional buffers, typically of data type int32_t, float
+                // are stored at the end of data. Pad the data, so that the
+                // buffers are properly aligned to their data type.
+                const size_t alignment_in_bytes = 4;
+                data_size = utils::rnd_up(data_size, alignment_in_bytes);
+            }
+            return data_size
+                    + (include_additional_size ? additional_buffer_size() : 0)
+                    + (include_offset0 ? data_type_size() * offset0() : 0);
+        } else if (is_sparse_desc()) {
+            if (sparse_desc().encoding == sparse_encoding::csr) {
+                switch (index) {
+                    // Return size for values.
+                    case 0: return nnz() * data_type_size();
+                    // Return size for indices.
+                    case 1: {
+                        const auto idx_dt = metadata_type(0);
+                        return nnz() * types::data_type_size(idx_dt);
+                    }
+                    // Return size for pointers.
+                    case 2: {
+                        const auto ptr_dt = metadata_type(1);
+                        return (dims()[0] + 1) * types::data_type_size(ptr_dt);
+                    }
+                    default: assert(!"unknown index"); return 0;
+                }
+            } else if (sparse_desc().encoding == sparse_encoding::coo) {
+                // Return size for values.
+                if (index == 0) {
+                    return nnz() * data_type_size();
+                } else if (index > 0 && index <= ndims()) {
+                    // Return size for index buffers.
+                    const auto idx_dt = metadata_type(0);
+                    return nnz() * types::data_type_size(idx_dt);
+                } else {
+                    assert(!"unknown index");
+                    return 0;
+                }
+            } else if (sparse_desc().encoding == sparse_encoding::packed) {
+                // If the size if queried from a user-created memory descriptor.
+                if (blocking_desc().strides[0] == 0) return 0;
+
+                switch (index) {
+                    case 0:
+                        // Return size for values.
+                        return nnz() * data_type_size();
+                    case 1: {
+                        // Return size for offsets.
+                        return (nelems(true) / blk_size()) * sizeof(int64_t);
+                    }
+                    case 2:
+                        // Return size for bitmask. The bitmask has 1 bit
+                        // per each value.
+                        return utils::div_up(nelems(true), CHAR_BIT);
+                    default: assert(!"unknown index"); return 0;
+                }
+            } else {
+                assert(!"unknown sparse encoding");
+                return 0;
+            }
+        } else {
+            assert(!"unknown format kind");
+            return 0;
         }
     }
 
@@ -189,12 +344,23 @@ struct memory_desc_wrapper : public c_compatible {
         return false;
     }
 
+    /** returns true if number of non unit dims is <= `n`. */
+    bool count_non_unit_dims(int n) const {
+        int non_unit_dims = 0;
+        for (int d = 0; d < ndims(); d++) {
+            if (dims()[d] != 1) non_unit_dims++;
+        }
+        return non_unit_dims <= n;
+    }
+
     /** returns true if data is dense in memory */
     bool is_dense(bool with_padding = false) const {
         if (utils::one_of(format_kind(), format_kind::undef, format_kind::any))
             return false;
         if (has_runtime_dims_or_strides() || has_broadcast()) return false;
-        return nelems(with_padding) * data_type_size() == size();
+        return nelems(with_padding) * data_type_size()
+                / sub_byte_data_type_multiplier()
+                == size(0, /* include_additional_size = */ false);
     }
 
     /** returns true if format is set to `any` */
@@ -282,8 +448,7 @@ struct memory_desc_wrapper : public c_compatible {
     /** returns true if one memory can be reordered to another */
     bool consistent_with(const memory_desc_wrapper &rhs) const;
 
-    /** returns true if the memory desc corresponds to the given format tag and
-     * strides.
+    /** returns true if the memory desc corresponds to the given format tag.
      * @sa memory_desc_matches_tag */
     bool matches_tag(format_tag_t tag, const dims_t strides = nullptr) const {
         return memory_desc_matches_tag(*md_, tag, strides);
@@ -299,13 +464,23 @@ struct memory_desc_wrapper : public c_compatible {
         return format_tag::undef;
     }
 
+    template <typename... Tags>
+    format_tag_t mb_stride_relaxed_match(Tags... tags) const {
+        dims_t skip_mb_stride {};
+        // See `memory_desc_matches_tag` comment.
+        skip_mb_stride[0] = -1;
+        for (const auto &tag : {tags...})
+            if (matches_tag(tag, skip_mb_stride)) return tag;
+        return format_tag::undef;
+    }
+
     /* offset section */
 
     /** returns physical offset by logical one. logical offset is represented by
      * an array \param pos. if \param is_pos_padded is true \param pos
      * represents the position in already padded area */
     dim_t off_v(const dims_t pos, bool is_pos_padded = false) const {
-        assert(is_blocking_desc());
+        assert(is_blocking_desc() || is_sparse_packed_desc());
         const blocking_desc_t &blk = blocking_desc();
 
         dims_t pos_copy = {0};
@@ -317,7 +492,7 @@ struct memory_desc_wrapper : public c_compatible {
         if (blk.inner_nblks > 0) {
             dim_t blk_stride = 1;
             for (int iblk = blk.inner_nblks - 1; iblk >= 0; --iblk) {
-                const int d = blk.inner_idxs[iblk];
+                const dim_t d = blk.inner_idxs[iblk];
 
                 dim_t p;
                 /* switch to faster 32-bit division when possible.
@@ -349,6 +524,7 @@ struct memory_desc_wrapper : public c_compatible {
      * a scalar \param l_offset. if \param is_pos_padded is true, \param
      * l_offset represents logical offset in already padded area */
     dim_t off_l(dim_t l_offset, bool is_pos_padded = false) const {
+        if (l_offset == 0) return offset0();
         dims_t dims_pos;
         const auto &cur_dims = is_pos_padded ? padded_dims() : dims();
         utils::l_dims_by_l_offset(dims_pos, l_offset, cur_dims, ndims());
@@ -388,6 +564,16 @@ struct memory_desc_wrapper : public c_compatible {
                           : blk_off<T, Args...>(xn, args...);
     }
 
+    /** returns physical offset by logical one. Logical offset is represented by
+     * a tuple of block indices (\param bn, ..., \param b1, \param b0). It is a
+     * user responsibility to adjust the result to get offset within blocks.
+     * If @tparam sub_off0 is true, then offset0() will be subtracted
+     * from result.*/
+    template <bool skip_first, bool sub_off0, typename T, typename... Args>
+    dim_t blk_off(T xn, Args... args) const {
+        return blk_off<skip_first, Args...>(xn, args...) - sub_off0 * offset0();
+    }
+
     /* static functions section */
     /* TODO: replace with non-static, once md_ becomes non-const ref */
 
@@ -415,7 +601,7 @@ private:
 
     template <int ORIG_LEN, typename T, typename... Args>
     dim_t _blk_off(T xc, Args... args) const {
-        assert(is_blocking_desc());
+        assert(is_blocking_desc() || is_sparse_packed_desc());
         constexpr int dc = ORIG_LEN - sizeof...(args) - 1;
         return xc * blocking_desc().strides[dc]
                 + _blk_off<ORIG_LEN, Args...>(args...);
@@ -428,7 +614,8 @@ inline bool memory_desc_wrapper::similar_to(const memory_desc_wrapper &rhs,
 
     if (one_of(format_kind(), format_kind::undef, format_kind::any))
         return false;
-    if (is_wino_desc() || is_rnn_packed_desc()) return false;
+    if (is_wino_desc() || is_rnn_packed_desc() || is_cublaslt_blocked_desc())
+        return false;
 
     const int ds = dim_start;
     const auto &blk = blocking_desc();
