@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,13 +17,14 @@
 #ifndef COMMON_PRIMITIVE_HASHING_HPP
 #define COMMON_PRIMITIVE_HASHING_HPP
 
+#include <thread>
 #include <typeindex>
 #include <type_traits>
 
-#include "c_types_map.hpp"
-#include "oneapi/dnnl/dnnl.h"
-#include "primitive_attr.hpp"
-#include "type_helpers.hpp"
+#include "common/c_types_map.hpp"
+#include "common/engine_id.hpp"
+#include "common/type_helpers.hpp"
+#include "common/verbose.hpp"
 
 namespace dnnl {
 namespace impl {
@@ -32,30 +33,32 @@ struct primitive_desc_t;
 namespace primitive_hashing {
 
 struct key_t {
-    key_t(const primitive_desc_t *pd, const engine_t *engine, int impl_nthr);
+    key_t(const engine_t *engine, const op_desc_t *op_desc,
+            const primitive_attr_t *attr, int pd_iterator_offset,
+            const std::vector<memory_desc_t> &hint_mds, int skip_idx);
+
+    key_t(const primitive_desc_t *pd, const engine_t *engine);
 
     bool operator==(const key_t &other) const;
     const std::thread::id &thread_id() const { return thread_id_; }
+    bool has_runtime_dependencies() const {
+        return !(engine_id_.kind() == engine_kind::cpu
+                && is_native_runtime(engine_id_.runtime_kind()));
+    }
 
     primitive_kind_t primitive_kind_;
     // Make these data fields mutable to be able to update them without removing
     // and adding a key (extract is available in C++17 only).
     mutable const op_desc_t *op_desc_;
     mutable const primitive_attr_t *attr_;
-    std::type_index impl_id_;
+    int pd_iterator_offset_;
     int impl_nthr_;
-    std::vector<memory_desc_t> mds;
-    engine_kind_t engine_kind_;
-    runtime_kind_t runtime_kind_;
-    device_id_t device_id_;
+    int skip_idx_;
+    std::vector<memory_desc_t> hint_mds_;
+    engine_id_t engine_id_;
 
 private:
-    void init_mds(const primitive_desc_t *pd);
-
-    template <typename desc_t>
-    static const desc_t &cast_to_desc(const void *p) {
-        return *(reinterpret_cast<const desc_t *>(p));
-    }
+    static primitive_kind_t get_pkind(primitive_kind_t pkind);
 
     // Thread ID is not used as part of the key, it's only used to get
     // information about what thread inserted the key and the corresponding
@@ -71,16 +74,18 @@ size_t get_desc_hash(const binary_desc_t &desc);
 size_t get_desc_hash(const convolution_desc_t &desc);
 size_t get_desc_hash(const eltwise_desc_t &desc);
 size_t get_desc_hash(const gemm_desc_t &desc);
+size_t get_desc_hash(const group_normalization_desc_t &desc);
 size_t get_desc_hash(const inner_product_desc_t &desc);
 size_t get_desc_hash(const layer_normalization_desc_t &desc);
 size_t get_desc_hash(const lrn_desc_t &desc);
 size_t get_desc_hash(const matmul_desc_t &desc);
-size_t get_desc_hash(const pooling_v2_desc_t &desc);
+size_t get_desc_hash(const pooling_desc_t &desc);
 size_t get_desc_hash(const prelu_desc_t &desc);
 size_t get_desc_hash(const reduction_desc_t &desc);
 size_t get_desc_hash(const reorder_desc_t &desc);
 size_t get_desc_hash(const resampling_desc_t &desc);
 size_t get_desc_hash(const rnn_desc_t &desc);
+size_t get_desc_hash(const sdpa_desc_t &desc);
 size_t get_desc_hash(const shuffle_desc_t &desc);
 size_t get_desc_hash(const softmax_desc_t &desc);
 size_t get_desc_hash(const sum_desc_t &desc);
@@ -103,10 +108,18 @@ inline size_t get_array_hash<memory_desc_t>(
     return seed;
 }
 
+inline size_t get_array_hash(
+        size_t seed, const std::vector<const memory_desc_t *> &mds) {
+    for (const auto *md : mds)
+        seed = hash_combine(seed, get_md_hash(*md));
+    return seed;
+}
+
 template <>
-inline size_t get_array_hash<float>(size_t seed, const float *v, int size) {
+inline size_t get_array_hash<data_type_t>(
+        size_t seed, const data_type_t *v, int size) {
     for (int i = 0; i < size; i++) {
-        seed = hash_combine(seed, float2int(v[i]));
+        seed = hash_combine(seed, static_cast<size_t>(v[i]));
     }
     return seed;
 }
@@ -129,20 +142,24 @@ struct hash<dnnl::impl::primitive_hashing::key_t> {
         seed = hash_combine(seed,
                 hash_combine(0, static_cast<size_t>(key.primitive_kind_)));
         seed = hash_combine(seed, get_attr_hash(*key.attr_));
-        seed = hash_combine(seed, hash_combine(0, key.impl_id_));
+        seed = hash_combine(seed, hash_combine(0, key.pd_iterator_offset_));
         seed = hash_combine(seed, hash_combine(0, key.impl_nthr_));
-        seed = hash_combine(
-                seed, hash_combine(0, static_cast<size_t>(key.engine_kind_)));
-        seed = hash_combine(
-                seed, hash_combine(0, static_cast<size_t>(key.runtime_kind_)));
-        seed = hash_combine(seed, hash_combine(0, std::get<0>(key.device_id_)));
-        seed = hash_combine(seed, hash_combine(0, std::get<1>(key.device_id_)));
-        seed = hash_combine(seed, hash_combine(0, std::get<2>(key.device_id_)));
+        seed = hash_combine(seed, hash_combine(0, key.skip_idx_));
+
+        seed = hash_combine(seed, key.engine_id_.hash());
+
+        seed = get_array_hash(
+                seed, key.hint_mds_.data(), (int)key.hint_mds_.size());
+
+        const result_type verb_seed_before_desc = seed;
+        UNUSED(verb_seed_before_desc);
+
         // Combine hash for op_desc with the computed hash
 #define CASE(pkind) \
     case primitive_kind::pkind: \
-        seed = hash_combine( \
-                seed, get_desc_hash(*(pkind##_desc_t *)key.op_desc_)); \
+        seed = hash_combine(seed, \
+                get_desc_hash( \
+                        *op_desc_t::to_desc<pkind##_desc_t>(key.op_desc_))); \
         break;
 
         // clang-format off
@@ -154,16 +171,18 @@ struct hash<dnnl::impl::primitive_hashing::key_t> {
             CASE(deconvolution)
             CASE(eltwise)
             CASE(gemm)
+            CASE(group_normalization)
             CASE(inner_product)
             CASE(layer_normalization)
             CASE(lrn)
             CASE(matmul)
-            CASE(pooling_v2)
+            CASE(pooling)
             CASE(prelu)
             CASE(reduction)
             CASE(reorder)
             CASE(resampling)
             CASE(rnn)
+            CASE(sdpa)
             CASE(shuffle)
             CASE(softmax)
             CASE(sum)
@@ -172,7 +191,13 @@ struct hash<dnnl::impl::primitive_hashing::key_t> {
         }
             // clang-format on
 #undef CASE
-        seed = get_array_hash(seed, key.mds.data(), (int)key.mds.size());
+
+        // Note: `16` is just a random number, as debuginfo hasn't received a
+        // single command center for levels across layers of the library.
+        // ANCHOR: HASHING_DEBUGINFO_16.
+        VDEBUGINFO(16, primitive, hashing,
+                "operator(),seed_before_desc=%zu seed_after_desc=%zu",
+                verb_seed_before_desc, seed);
 
         return seed;
     }
