@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -20,24 +20,32 @@
 #include <assert.h>
 
 #include "c_types_map.hpp"
+#include "cache_hit_types.hpp"
 #include "engine.hpp"
 #include "primitive.hpp"
 #include "primitive_attr.hpp"
-#include "primitive_desc.hpp"
+#include "primitive_desc_iface.hpp"
+#include "primitive_iface.hpp"
 #include "type_helpers.hpp"
 #include "utils.hpp"
 
+#define VDISPATCH_REORDER(cond, msg, ...) \
+    VCONDCHECK(primitive, create, dispatch, reorder, (cond), \
+            status::unimplemented, "%s," msg, this->info(engine), \
+            ##__VA_ARGS__)
+
+#define VDISPATCH_REORDER_IC(cond, msg, ...) \
+    VCONDCHECK(primitive, create, dispatch, reorder, (cond), \
+            status::unimplemented, msg, ##__VA_ARGS__)
+
+#define VDISPATCH_REORDER_SC(f, msg, ...) \
+    VCHECK(primitive, create, dispatch, reorder, (f), "%s," msg, \
+            this->info(engine), ##__VA_ARGS__)
+
 namespace dnnl {
 namespace impl {
 
 struct reorder_primitive_desc_iface_t : public dnnl_primitive_desc {
-    reorder_primitive_desc_iface_t(primitive_desc_t *pd, engine_t *engine,
-            engine_t *src_engine, engine_t *dst_engine)
-        : dnnl_primitive_desc(pd, engine)
-        , src_engine_(src_engine)
-        , dst_engine_(dst_engine)
-        , scratchpad_engine_(nullptr) {}
-
     reorder_primitive_desc_iface_t(const std::shared_ptr<primitive_desc_t> &pd,
             engine_t *engine, engine_t *src_engine, engine_t *dst_engine)
         : dnnl_primitive_desc(pd, engine)
@@ -68,11 +76,11 @@ struct reorder_primitive_desc_iface_t : public dnnl_primitive_desc {
     }
 
     status_t create_primitive_iface(
-            std::pair<primitive_iface_t *, bool> &primitive_iface)
-            const override {
+            std::pair<primitive_iface_t *, cache_state_t> &primitive_iface,
+            const cache_blob_t &cache_blob) const override {
         // Step 1: create impl::primitive_t or get it from primitive cache
-        std::pair<std::shared_ptr<primitive_t>, bool> p;
-        auto status = pd_->create_primitive(p, engine());
+        std::pair<std::shared_ptr<primitive_t>, cache_state_t> p;
+        auto status = pd_->create_primitive(p, engine(), cache_blob);
         if (status != status::success) return status;
         // Step 2: create primitive_iface_t, init and return it to user
         primitive_iface_t *p_iface = nullptr;
@@ -94,23 +102,8 @@ private:
     dnnl::impl::engine_t *scratchpad_engine_;
 };
 
+// NOLINTBEGIN(google-default-arguments)
 struct reorder_pd_t : public primitive_desc_t {
-    reorder_pd_t(const primitive_attr_t *attr, engine_kind_t src_engine_kind,
-            const memory_desc_t *src_md, engine_kind_t dst_engine_kind,
-            const memory_desc_t *dst_md)
-        : primitive_desc_t(attr, primitive_kind::reorder)
-        , src_md_(*src_md)
-        , dst_md_(*dst_md) {
-
-        // Fill a desc that is intended for internal use only
-        desc_ = reorder_desc_t();
-        desc_.primitive_kind = primitive_kind::reorder;
-        desc_.src_md = src_md_;
-        desc_.dst_md = dst_md_;
-        desc_.src_engine_kind = src_engine_kind;
-        desc_.dst_engine_kind = dst_engine_kind;
-    }
-
     const reorder_desc_t *desc() const { return &desc_; }
     const op_desc_t *op_desc() const override {
         return reinterpret_cast<const op_desc_t *>(this->desc());
@@ -124,25 +117,29 @@ struct reorder_pd_t : public primitive_desc_t {
         return primitive_desc_t::arg_usage(arg);
     }
 
-    const memory_desc_t *arg_md(int arg) const override {
+    const memory_desc_t *arg_md(
+            int arg, bool user_input = false) const override {
         switch (arg) {
             case DNNL_ARG_FROM: return src_md(0);
-            case DNNL_ARG_TO: return dst_md(0);
+            case DNNL_ARG_TO: return dst_md(0, user_input);
             default: return primitive_desc_t::arg_md(arg);
         }
     }
 
-    const memory_desc_t *src_md(int index = 0) const override {
-        return index == 0 ? &src_md_ : &glob_zero_md;
+    const memory_desc_t *src_md(
+            int index = 0, bool user_input = false) const override {
+        if (index == 0) return user_input ? desc()->src_md : &src_md_;
+        return &glob_zero_md;
     }
-    const memory_desc_t *dst_md(int index = 0) const override {
-        return index == 0 ? &dst_md_ : &glob_zero_md;
+    const memory_desc_t *dst_md(
+            int index = 0, bool user_input = false) const override {
+        if (index == 0) return user_input ? desc()->dst_md : &dst_md_;
+        return &glob_zero_md;
     }
 
     int n_inputs() const override { return 1; }
     int n_outputs() const override { return 1; }
 
-    float alpha() const { return attr()->output_scales_.scales_[0]; }
     float beta() const {
         const int sum_idx = attr()->post_ops_.find(primitive_kind::sum);
         return sum_idx == -1 ? 0 : attr()->post_ops_.entry_[sum_idx].sum.scale;
@@ -152,7 +149,47 @@ protected:
     reorder_desc_t desc_;
     memory_desc_t src_md_;
     memory_desc_t dst_md_;
+
+    reorder_pd_t(const primitive_attr_t *attr, engine_kind_t src_engine_kind,
+            const memory_desc_t *src_md, engine_kind_t dst_engine_kind,
+            const memory_desc_t *dst_md)
+        : primitive_desc_t(attr, primitive_kind::reorder)
+        , src_md_(*src_md)
+        , dst_md_(*dst_md) {
+
+        init_desc(src_engine_kind, dst_engine_kind, false);
+    }
+
+    reorder_pd_t(const reorder_pd_t &other)
+        : primitive_desc_t(other)
+        , src_md_(other.src_md_)
+        , dst_md_(other.dst_md_) {
+        init_desc(other.desc_.src_engine_kind, other.desc_.dst_engine_kind,
+                other.desc_.is_cross_engine);
+    }
+
+    reorder_pd_t &operator=(const reorder_pd_t &other) {
+        DNNL_SHORT_CIRCUIT_SELF_ASSIGN(other);
+        src_md_ = other.src_md_;
+        dst_md_ = other.dst_md_;
+
+        init_desc(other.desc_.src_engine_kind, other.desc_.dst_engine_kind,
+                other.desc_.is_cross_engine);
+        return *this;
+    }
+
+    void init_desc(engine_kind_t src_engine_kind, engine_kind_t dst_engine_kind,
+            bool is_cross_engine) {
+        desc_ = reorder_desc_t();
+        desc_.primitive_kind = primitive_kind::reorder;
+        desc_.src_md = &src_md_;
+        desc_.dst_md = &dst_md_;
+        desc_.src_engine_kind = src_engine_kind;
+        desc_.dst_engine_kind = dst_engine_kind;
+        desc_.is_cross_engine = is_cross_engine;
+    }
 };
+// NOLINTEND(google-default-arguments)
 
 } // namespace impl
 } // namespace dnnl
