@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2024 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -52,17 +52,17 @@ namespace impl {
 
 #define DNNL_SHORT_CIRCUIT_SELF_ASSIGN(other) \
     do { \
-        if (this == &other) return *this; \
+        if (this == &(other)) return *this; \
     } while (0)
 
 #define DNNL_SHORT_CIRCUIT_SELF_COMPARISON(other) \
     do { \
-        if (this == &other) return true; \
+        if (this == &(other)) return true; \
     } while (0)
 
 #define DNNL_DISALLOW_COPY_AND_ASSIGN(T) \
     T(const T &) = delete; \
-    T &operator=(const T &) = delete;
+    void operator=(const T &) = delete;
 
 // Sanity check for 64 bits
 static_assert(sizeof(void *) == 8, "oneDNN supports 64-bit architectures only");
@@ -104,41 +104,50 @@ namespace utils {
 
 /* SFINAE helper -- analogue to std::enable_if */
 template <bool expr, class T = void>
-struct enable_if {};
+struct enable_if {}; // NOLINT(readability-identifier-naming)
+
 template <class T>
 struct enable_if<true, T> {
-    typedef T type;
+    using type = T;
 };
 
+// Replacement implementation of std::enable_if_t from C++14, included here for
+// interoperability with C++11
+template <bool B, class T = void>
+using enable_if_t = typename enable_if<B, T>::type;
+
+template <typename T>
+using is_vector = std::is_same<T, typename std::vector<typename T::value_type>>;
+
 /* analogue std::conditional */
 template <bool, typename, typename>
-struct conditional {};
+struct conditional {}; // NOLINT(readability-identifier-naming)
 template <typename T, typename F>
 struct conditional<true, T, F> {
-    typedef T type;
+    using type = T;
 };
 template <typename T, typename F>
 struct conditional<false, T, F> {
-    typedef F type;
+    using type = F;
 };
 
 template <bool, typename, bool, typename, typename>
-struct conditional3 {};
+struct conditional3 {}; // NOLINT(readability-identifier-naming)
 template <typename T, typename FT, typename FF>
 struct conditional3<true, T, false, FT, FF> {
-    typedef T type;
+    using type = T;
 };
 template <typename T, typename FT, typename FF>
 struct conditional3<false, T, true, FT, FF> {
-    typedef FT type;
+    using type = FT;
 };
 template <typename T, typename FT, typename FF>
 struct conditional3<false, T, false, FT, FF> {
-    typedef FF type;
+    using type = FF;
 };
 
 template <bool, typename U, U, U>
-struct conditional_v {};
+struct conditional_v {}; // NOLINT(readability-identifier-naming)
 template <typename U, U t, U f>
 struct conditional_v<true, U, t, f> {
     static constexpr U value = t;
@@ -149,16 +158,16 @@ struct conditional_v<false, U, t, f> {
 };
 
 template <typename T>
-struct remove_reference {
-    typedef T type;
+struct remove_reference { // NOLINT(readability-identifier-naming)
+    using type = T;
 };
 template <typename T>
 struct remove_reference<T &> {
-    typedef T type;
+    using type = T;
 };
 template <typename T>
 struct remove_reference<T &&> {
-    typedef T type;
+    using type = T;
 };
 
 template <typename T>
@@ -181,6 +190,7 @@ std::unique_ptr<T> make_unique(Args &&...args) {
     return std::unique_ptr<T>(new T(std::forward<Args>(args)...));
 }
 
+// NOLINTBEGIN(performance-unnecessary-value-param)
 template <typename T, typename P>
 constexpr bool everyone_is(T val, P item) {
     return val == item;
@@ -189,7 +199,9 @@ template <typename T, typename P, typename... Args>
 constexpr bool everyone_is(T val, P item, Args... item_others) {
     return val == item && everyone_is(val, item_others...);
 }
+// NOLINTEND(performance-unnecessary-value-param)
 
+// NOLINTBEGIN(performance-unnecessary-value-param)
 template <typename T, typename P>
 constexpr bool one_of(T val, P item) {
     return val == item;
@@ -198,6 +210,7 @@ template <typename T, typename P, typename... Args>
 constexpr bool one_of(T val, P item, Args... item_others) {
     return val == item || one_of(val, item_others...);
 }
+// NOLINTEND(performance-unnecessary-value-param)
 
 template <typename T, typename P>
 constexpr P map(T pat, P def) {
@@ -251,7 +264,7 @@ inline void array_set(T *arr, const U &val, size_t size) {
 
 namespace product_impl {
 template <size_t>
-struct int2type {};
+struct int2type {}; // NOLINT(readability-identifier-naming)
 
 template <typename T>
 constexpr int product_impl(const T *arr, int2type<0>) {
@@ -478,11 +491,10 @@ T pick_by_prop_kind(prop_kind_t prop_kind, const T &val_fwd, const T &val_bwd_d,
 }
 
 template <typename Telem, size_t Tdims>
-struct array_offset_calculator {
+struct array_offset_calculator { // NOLINT(readability-identifier-naming)
     template <typename... Targs>
-    array_offset_calculator(Telem *base, Targs... Fargs) : _dims {Fargs...} {
-        _base_ptr = base;
-    }
+    array_offset_calculator(Telem *base, Targs... Fargs)
+        : _base_ptr(base), _dims {Fargs...} {}
 
     template <typename... Targs>
     array_offset_calculator(std::nullptr_t, Targs... Fargs) = delete;
@@ -721,7 +733,7 @@ public:
     constexpr setting_t(const T init) : value_ {init}, initialized_ {false} {}
     bool initialized() { return initialized_; }
     T get() { return value_; }
-    void set(T new_value) {
+    void set(const T &new_value) {
         value_ = new_value;
         initialized_ = true;
     }
@@ -823,16 +835,10 @@ using maybe_unique_ptr = std::unique_ptr<T, nop_deleter_t>;
 struct nibble2_t {
 
     // constructs a nibble pair from a pair of uint8_t values
-    nibble2_t(uint8_t low_, uint8_t high_) {
-        low = low_;
-        high = high_;
-    }
+    nibble2_t(uint8_t low_, uint8_t high_) : low(low_), high(high_) {}
 
     // constructs a nibble pairs from an uin8_t, taking its low and high part
-    nibble2_t(uint8_t pack_) {
-        low = pack_ & 0xf;
-        high = (pack_ >> 4) & 0xf;
-    }
+    nibble2_t(uint8_t pack_) : low(pack_ & 0xf), high((pack_ >> 4) & 0xf) {}
 
     // sets low (idx=0) or high (idx=1)  nibble.
     inline void set(uint8_t val, int idx) {
@@ -868,7 +874,7 @@ static_assert(sizeof(nibble2_t) == 1, "nibble2_t must be 1 byte");
 ///     printf("%d\t", idx);
 /// }
 /// output: 0  2  3
-class mask_iterator {
+class mask_iterator { // NOLINT(readability-identifier-naming)
     int mask_;
     int index_;
 
