@@ -1,5 +1,6 @@
 /*******************************************************************************
-* Copyright 2018-2021 Intel Corporation
+* Copyright 2018-2024 Intel Corporation
+* Copyright 2022 IBM Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -15,10 +16,6 @@
 *******************************************************************************/
 
 #include "oneapi/dnnl/dnnl.h"
-#if DNNL_CPU_RUNTIME == DNNL_RUNTIME_THREADPOOL
-#include "oneapi/dnnl/dnnl_threadpool.hpp"
-#include "oneapi/dnnl/dnnl_threadpool_iface.hpp"
-#endif
 
 #include "common/bfloat16.hpp"
 #include "common/c_types_map.hpp"
@@ -31,6 +28,7 @@
 #include "cpu/gemm/gemm_msan_unpoison.hpp"
 #include "cpu/gemm/os_blas.hpp"
 
+#include "cpu/gemm/bf16/ref_gemm_bf16.hpp"
 #include "cpu/gemm/f32/ref_gemm_f32.hpp"
 #include "cpu/gemm/s8x8s32/ref_gemm_s8x8s32.hpp"
 #include "cpu/gemm/s8x8s32/simple_gemm_s8s8s32.hpp"
@@ -44,6 +42,11 @@
 #include "cpu/x64/gemm/gemm_driver.hpp"
 
 using namespace dnnl::impl::cpu::x64;
+#elif DNNL_PPC64
+#include "cpu/ppc64/ppc64_gemm_driver.hpp"
+using namespace dnnl::impl::cpu::ppc64;
+#elif DNNL_S390X
+#include "cpu/s390x/gemm.h"
 #endif
 
 namespace dnnl {
@@ -76,6 +79,13 @@ dnnl_status_t check_gemm_input(const char *transa, const char *transb,
             && (is_packed_b || *ldb >= nstl::max(dim_t(1), nrow_b))
             && *ldc >= nstl::max(dim_t(1), *M);
     if (!consistency) return dnnl_invalid_arguments;
+#if DNNL_PPC64
+#ifdef __MMA__
+    if (!(utils::one_of(*transa, 'n', 'N', 't', 'T')
+                && utils::one_of(*transb, 'n', 'N', 't', 'T')))
+        return dnnl_unimplemented;
+#endif
+#endif
 
     return dnnl_success;
 }
@@ -124,13 +134,14 @@ dnnl_status_t extended_sgemm(const char *transa, const char *transb,
     }
 #endif
 
-#if DNNL_X64
+#if DNNL_X64 && !__BUILD_GEMM_NONE
     if (mayiuse(sse41)) {
         float *dummy_ao = nullptr;
         float *dummy_bo = nullptr;
-        return gemm_driver(transa, transb, bias ? "C" : nullptr, M, N, K, alpha,
-                A, lda, dummy_ao, B, ldb, dummy_bo, beta, C, ldc, bias,
+        auto status = gemm_driver(transa, transb, bias ? "C" : nullptr, M, N, K,
+                alpha, A, lda, dummy_ao, B, ldb, dummy_bo, beta, C, ldc, bias,
                 force_jit_nocopy_gemm);
+        if (status != status::unimplemented) return status;
     }
 #endif
 
@@ -174,12 +185,12 @@ dnnl_status_t try_cblas_gemm_s8u8s32(const char *transa, const char *transb,
 #endif
 }
 
-template <>
-dnnl_status_t gemm_s8x8s32(const char *transa, const char *transb,
+dnnl_status_t gemm_s8u8s32(const char *transa, const char *transb,
         const char *offsetc, const dim_t *M, const dim_t *N, const dim_t *K,
         const float *alpha, const int8_t *A, const dim_t *LDA, const int8_t *ao,
         const uint8_t *B, const dim_t *LDB, const uint8_t *bo,
         const float *beta, int32_t *C, const dim_t *LDC, const int32_t *co) {
+
     dnnl_status_t status = check_gemm_x8x8x32_input(offsetc, transa, transb, M,
             N, K, A, LDA, B, LDB, C, LDC, alpha, beta, false);
     if (status != dnnl_success) return status;
@@ -188,42 +199,84 @@ dnnl_status_t gemm_s8x8s32(const char *transa, const char *transb,
 
     status = try_cblas_gemm_s8u8s32(transa, transb, offsetc, M, N, K, alpha, A,
             LDA, ao, B, LDB, bo, beta, C, LDC, co);
-    if (status == dnnl_success) return status;
+    if (status != status::unimplemented) return status;
 
-#if DNNL_X64
-    if (mayiuse(sse41) && !mayiuse(avx512_mic))
-        return gemm_driver(transa, transb, offsetc, M, N, K, alpha, A, LDA, ao,
-                B, LDB, bo, beta, C, LDC, co, false);
+#if DNNL_X64 && !__BUILD_GEMM_NONE
+    if (mayiuse(sse41)) {
+        auto status = gemm_driver(transa, transb, offsetc, M, N, K, alpha, A,
+                LDA, ao, B, LDB, bo, beta, C, LDC, co, false);
+        if (status != status::unimplemented) return status;
+    }
+#elif DNNL_PPC64
+#ifdef __MMA__
+    int ATflag = (*transa == 'T') || (*transa == 't');
+    int BTflag = (*transb == 'T') || (*transb == 't');
+
+    return cblas_gemm_s8x8s32_ppc64(ATflag, BTflag, offsetc, *M, *N, *K, *alpha,
+            A, *LDA, ao, B, *LDB, bo, C, *beta, *LDC, co, 0);
+#endif
+#elif DNNL_S390X
+#if defined(__VX__)
+    return s390x::gemmx8x8s32(transa, transb, offsetc, *M, *N, *K, *alpha, A,
+            *LDA, ao, B, *LDB, bo, *beta, C, *LDC, co);
+#endif
 #endif
 
     return ref_gemm_s8x8s32(transa, transb, offsetc, M, N, K, alpha, A, LDA, ao,
             B, LDB, bo, beta, C, LDC, co);
 }
 
-template <>
-dnnl_status_t gemm_s8x8s32(const char *transa, const char *transb,
+dnnl_status_t gemm_s8s8s32(const char *transa, const char *transb,
         const char *offsetc, const dim_t *M, const dim_t *N, const dim_t *K,
         const float *alpha, const int8_t *A, const dim_t *LDA, const int8_t *ao,
         const int8_t *B, const dim_t *LDB, const int8_t *bo, const float *beta,
         int32_t *C, const dim_t *LDC, const int32_t *co) {
+
     dnnl_status_t status = check_gemm_x8x8x32_input(offsetc, transa, transb, M,
             N, K, A, LDA, B, LDB, C, LDC, alpha, beta, false);
     if (status != dnnl_success) return status;
 
     if (*M == 0 || *N == 0 || *K == 0) return dnnl_success;
 
-#if DNNL_X64
-    bool use_jit = mayiuse(avx512_core);
+#if DNNL_X64 && !__BUILD_GEMM_NONE
+    bool use_jit = mayiuse(avx512_core) && __BUILD_GEMM_AVX512;
     bool use_s8u8 = true
             && utils::everyone_is(0, *ao, *bo) // so far a requirement
             && IMPLICATION(USE_MKL_IGEMM == 0, mayiuse(sse41));
 
-    if (use_jit)
-        return gemm_driver(transa, transb, offsetc, M, N, K, alpha, A, LDA, ao,
-                B, LDB, bo, beta, C, LDC, co, false);
-    else if (use_s8u8)
-        return simple_gemm_s8s8s32(transa, transb, offsetc, M, N, K, alpha, A,
-                LDA, ao, B, LDB, bo, beta, C, LDC, co);
+    if (use_jit) {
+        auto status = gemm_driver(transa, transb, offsetc, M, N, K, alpha, A,
+                LDA, ao, B, LDB, bo, beta, C, LDC, co, false);
+        if (status != status::unimplemented) return status;
+    }
+
+    if (use_s8u8) {
+        auto status = simple_gemm_s8s8s32(transa, transb, offsetc, M, N, K,
+                alpha, A, LDA, ao, B, LDB, bo, beta, C, LDC, co);
+        if (status != status::unimplemented) return status;
+    }
+#endif
+
+#if DNNL_PPC64
+#ifdef __MMA__
+    int ATflag = (*transa == 'T') || (*transa == 't');
+    int BTflag = (*transb == 'T') || (*transb == 't');
+
+    // Note please that the coercion of "B" and "bo" from int8_t to uint8_t is
+    // accompanied by the last parameter being set to "1" instead of "0", as
+    // in the analogous call in the previous routine above.
+    // This last parameter flags the fact of the coercion, so the called routine
+    // can process "B" and "bo" appropriately.
+
+    return cblas_gemm_s8x8s32_ppc64(ATflag, BTflag, offsetc, *M, *N, *K, *alpha,
+            A, *LDA, ao, (const uint8_t *)B, *LDB, (const uint8_t *)bo, C,
+            *beta, *LDC, co, 1);
+#endif
+#elif DNNL_S390X
+#if defined(__VX__)
+    return s390x::gemmx8x8s32(transa, transb, offsetc, *M, *N, *K, *alpha, A,
+            *LDA, ao, B, *LDB, bo, *beta, C, *LDC, co);
+#endif
 #endif
 
     return ref_gemm_s8x8s32(transa, transb, offsetc, M, N, K, alpha, A, LDA, ao,
@@ -238,116 +291,36 @@ dnnl_status_t gemm_bf16bf16f32(const char *transa, const char *transb,
             ldb, C, ldc, alpha, beta, false);
     if (status != dnnl_success) return status;
 
-#if DNNL_X64
+#if DNNL_X64 && !__BUILD_GEMM_NONE
     char *dummyOffsetC = nullptr;
     bfloat16_t *dummy_ao = nullptr;
     bfloat16_t *dummy_bo = nullptr;
     float *dummy_co = nullptr;
 
-    if (mayiuse(avx512_core))
-        return gemm_driver(transa, transb, dummyOffsetC, M, N, K, alpha,
+    if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
+        auto status = gemm_driver(transa, transb, dummyOffsetC, M, N, K, alpha,
                 (const bfloat16_t *)A, lda, dummy_ao, (const bfloat16_t *)B,
                 ldb, dummy_bo, beta, (float *)C, ldc, dummy_co, false);
+        if (status != status::unimplemented) return status;
+    }
+#elif DNNL_PPC64
+#if defined(USE_CBLAS) && defined(BLAS_HAS_SBGEMM) && defined(__MMA__)
+    bool trA = *transa == 't' || *transa == 'T';
+    bool trB = *transb == 't' || *transb == 'T';
+    CBLAS_TRANSPOSE Cblas_trA = trA ? CblasTrans : CblasNoTrans;
+    CBLAS_TRANSPOSE Cblas_trB = trB ? CblasTrans : CblasNoTrans;
+
+    cblas_sbgemm(CblasColMajor, Cblas_trA, Cblas_trB, *M, *N, *K, *alpha,
+            (const bfloat16 *)A, *lda, (const bfloat16 *)B, *ldb, *beta, C,
+            *ldc);
+    return dnnl_success;
+#endif
 #endif
 
-    return dnnl_unimplemented;
+    return ref_gemm_bf16bf16f32(
+            transa, transb, M, N, K, alpha, A, lda, B, ldb, beta, C, ldc);
 }
 
 } // namespace cpu
 } // namespace impl
 } // namespace dnnl
-
-using namespace dnnl::impl;
-using namespace dnnl::impl::cpu;
-
-dnnl_status_t dnnl_sgemm(char transa, char transb, dim_t M, dim_t N, dim_t K,
-        float alpha, const float *A, dim_t lda, const float *B, const dim_t ldb,
-        float beta, float *C, dim_t ldc) {
-    return extended_sgemm(&transb, &transa, &N, &M, &K, &alpha, B, &ldb, A,
-            &lda, &beta, C, &ldc);
-}
-
-namespace {
-const char *c2f_offsetC(const char *offC) {
-    if (offC) {
-        if (offC[0] == 'R' || offC[0] == 'r') return "C";
-        if (offC[0] == 'C' || offC[0] == 'c') return "R";
-    }
-    return offC;
-}
-} // namespace
-
-dnnl_status_t dnnl_gemm_u8s8s32(char transa, char transb, char offsetc, dim_t M,
-        dim_t N, dim_t K, float alpha, const uint8_t *A, dim_t lda, uint8_t ao,
-        const int8_t *B, dim_t ldb, int8_t bo, float beta, int32_t *C,
-        dim_t ldc, const int32_t *co) {
-    return gemm_s8x8s32(&transb, &transa, c2f_offsetC(&offsetc), &N, &M, &K,
-            &alpha, B, &ldb, &bo, A, &lda, &ao, &beta, C, &ldc, co);
-}
-
-dnnl_status_t dnnl_gemm_s8s8s32(char transa, char transb, char offsetc, dim_t M,
-        dim_t N, dim_t K, float alpha, const int8_t *A, dim_t lda, int8_t ao,
-        const int8_t *B, dim_t ldb, int8_t bo, float beta, int32_t *C,
-        dim_t ldc, const int32_t *co) {
-    return gemm_s8x8s32<int8_t>(&transb, &transa, c2f_offsetC(&offsetc), &N, &M,
-            &K, &alpha, B, &ldb, &bo, A, &lda, &ao, &beta, C, &ldc, co);
-}
-
-extern "C" dnnl_status_t DNNL_API dnnl_gemm_bf16bf16f32(char transa,
-        char transb, dim_t M, dim_t N, dim_t K, float alpha,
-        const bfloat16_t *A, dim_t lda, const bfloat16_t *B, dim_t ldb,
-        float beta, float *C, dim_t ldc) {
-    return gemm_bf16bf16f32(&transb, &transa, &N, &M, &K, &alpha, B, &ldb, A,
-            &lda, &beta, C, &ldc);
-}
-
-#if DNNL_CPU_RUNTIME == DNNL_RUNTIME_THREADPOOL
-dnnl_status_t dnnl_threadpool_interop_sgemm(char transa, char transb, dim_t M,
-        dim_t N, dim_t K, float alpha, const float *A, dim_t lda,
-        const float *B, const dim_t ldb, float beta, float *C, dim_t ldc,
-        void *th) {
-    threadpool_utils::activate_threadpool(
-            (dnnl::threadpool_interop::threadpool_iface *)th);
-    status_t status = extended_sgemm(&transb, &transa, &N, &M, &K, &alpha, B,
-            &ldb, A, &lda, &beta, C, &ldc, nullptr, false);
-    threadpool_utils::deactivate_threadpool();
-    return status;
-}
-
-dnnl_status_t dnnl_threadpool_interop_gemm_u8s8s32(char transa, char transb,
-        char offsetc, dim_t M, dim_t N, dim_t K, float alpha, const uint8_t *A,
-        dim_t lda, uint8_t ao, const int8_t *B, dim_t ldb, int8_t bo,
-        float beta, int32_t *C, dim_t ldc, const int32_t *co, void *th) {
-    threadpool_utils::activate_threadpool(
-            (dnnl::threadpool_interop::threadpool_iface *)th);
-    status_t status = gemm_s8x8s32(&transb, &transa, c2f_offsetC(&offsetc), &N,
-            &M, &K, &alpha, B, &ldb, &bo, A, &lda, &ao, &beta, C, &ldc, co);
-    threadpool_utils::deactivate_threadpool();
-    return status;
-}
-
-dnnl_status_t dnnl_threadpool_interop_gemm_s8s8s32(char transa, char transb,
-        char offsetc, dim_t M, dim_t N, dim_t K, float alpha, const int8_t *A,
-        dim_t lda, int8_t ao, const int8_t *B, dim_t ldb, int8_t bo, float beta,
-        int32_t *C, dim_t ldc, const int32_t *co, void *th) {
-    threadpool_utils::activate_threadpool(
-            (dnnl::threadpool_interop::threadpool_iface *)th);
-    status_t status = gemm_s8x8s32<int8_t>(&transb, &transa,
-            c2f_offsetC(&offsetc), &N, &M, &K, &alpha, B, &ldb, &bo, A, &lda,
-            &ao, &beta, C, &ldc, co);
-    threadpool_utils::deactivate_threadpool();
-    return status;
-}
-
-extern "C" dnnl_status_t DNNL_API dnnl_threadpool_interop_gemm_bf16bf16f32(
-        char transa, char transb, dim_t M, dim_t N, dim_t K, float alpha,
-        const bfloat16_t *A, dim_t lda, const bfloat16_t *B, dim_t ldb,
-        float beta, float *C, dim_t ldc, void *th) {
-    threadpool_utils::activate_threadpool(
-            (dnnl::threadpool_interop::threadpool_iface *)th);
-    status_t status = gemm_bf16bf16f32(&transb, &transa, &N, &M, &K, &alpha, B,
-            &ldb, A, &lda, &beta, C, &ldc);
-    threadpool_utils::deactivate_threadpool();
-    return status;
-}
-#endif
