@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -44,15 +44,31 @@ struct ref_eltwise_fwd_t : public primitive_t {
             using namespace utils;
             using sm = primitive_attr_t::skip_mask_t;
 
-            bool ok = is_fwd() && data_type == desc()->data_desc.data_type
-                    && platform::has_data_type_support(data_type)
-                    && attr()->has_default_values(sm::post_ops);
-            if (!ok) return status::unimplemented;
+            const memory_desc_wrapper src_d(src_md());
+            const memory_desc_wrapper dst_d(dst_md());
 
-            auto src_d = memory_desc_wrapper(data_md());
+            VDISPATCH_ELTWISE(is_fwd(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_ELTWISE(utils::everyone_is(data_type, src_md()->data_type,
+                                      dst_md()->data_type),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_ELTWISE(platform::has_data_type_support(data_type),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_ELTWISE(attr()->has_default_values(sm::post_ops),
+                    VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_ELTWISE(
+                    ref_post_ops_t::primitive_kind_ok(attr()->post_ops_),
+                    VERBOSE_UNSUPPORTED_POSTOP);
+            VDISPATCH_ELTWISE(
+                    set_default_formats_common(), VERBOSE_UNSUPPORTED_TAG);
+            VDISPATCH_ELTWISE(
+                    src_d == dst_d, VERBOSE_INCONSISTENT_MDS, "src", "dst");
+            VDISPATCH_ELTWISE(
+                    attr_.set_default_formats(dst_md(0)) == status::success,
+                    VERBOSE_UNSUPPORTED_POSTOP);
 
-            use_dense_ = src_d.is_dense(true)
-                    && IMPLICATION(!src_d.is_dense(), is_zero_preserved());
+            use_dense_ = src_d.is_dense(true) && dst_d.is_dense(true)
+                    && IMPLICATION(!src_d.is_dense() || !dst_d.is_dense(),
+                            is_zero_preserved());
 
             use_nCspBc_padded_ = !use_dense_
                     && src_d.blocking_desc().inner_nblks == 1
@@ -76,10 +92,11 @@ struct ref_eltwise_fwd_t : public primitive_t {
         ref_post_ops
                 = utils::make_unique<ref_post_ops_t>(pd()->attr()->post_ops_);
         if (!ref_post_ops) return status::out_of_memory;
+        CHECK(ref_post_ops->init(pd()->dst_md()));
         return status::success;
     }
 
-    using data_t = typename prec_traits<data_type>::type;
+    using data_t = typename prec_traits_t<data_type>::type;
 
     status_t execute(const exec_ctx_t &ctx) const override {
         if (pd()->use_dense_)
@@ -107,17 +124,25 @@ struct ref_eltwise_bwd_t : public primitive_t {
 
         status_t init(engine_t *engine) {
             using namespace utils;
+            using namespace data_type;
 
-            bool ok = !is_fwd()
-                    && everyone_is(data_type, desc()->data_desc.data_type,
-                            desc()->diff_data_desc.data_type)
-                    && platform::has_data_type_support(data_type)
-                    && set_default_formats_common()
-                    && attr()->has_default_values();
-            if (!ok) return status::unimplemented;
-
+            const memory_desc_wrapper diff_src_d(diff_src_md());
             const memory_desc_wrapper diff_dst_d(diff_dst_md());
 
+            VDISPATCH_ELTWISE(!is_fwd(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_ELTWISE(
+                    utils::everyone_is(data_type, data_md()->data_type,
+                            diff_src_md()->data_type, diff_dst_md()->data_type),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_ELTWISE(platform::has_data_type_support(data_type),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_ELTWISE(
+                    attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_ELTWISE(
+                    set_default_formats_common(), VERBOSE_UNSUPPORTED_TAG);
+            VDISPATCH_ELTWISE(diff_dst_d == diff_src_d,
+                    VERBOSE_INCONSISTENT_MDS, "diff_src", "diff_dst");
+
             use_dense_ = diff_dst_d.is_dense()
                     || (diff_dst_d.is_dense(true) && is_zero_preserved());
 
@@ -125,7 +150,7 @@ struct ref_eltwise_bwd_t : public primitive_t {
             if (diff_dst_d != memory_desc_wrapper(data_md()))
                 use_dense_ = false;
 
-            if (data_type == data_type::bf16) init_scratchpad();
+            if (utils::one_of(data_type, bf16, f16)) init_scratchpad();
 
             return status::success;
         }
@@ -135,10 +160,10 @@ struct ref_eltwise_bwd_t : public primitive_t {
     private:
         void init_scratchpad() {
             const memory_desc_wrapper data_d(data_md());
-            const memory_desc_wrapper diff_data_d(diff_dst_md());
+            const memory_desc_wrapper diff_dst_d(diff_dst_md());
             using namespace memory_tracking::names;
             auto scratchpad = scratchpad_registry().registrar();
-            const auto diff_dst_size = diff_data_d.nelems(true);
+            const auto diff_dst_size = diff_dst_d.nelems(true);
             scratchpad.template book<float>(
                     key_eltwise_src, data_d.nelems(true));
             scratchpad.template book<float>(
@@ -147,7 +172,7 @@ struct ref_eltwise_bwd_t : public primitive_t {
     };
 
     ref_eltwise_bwd_t(const pd_t *apd) : primitive_t(apd) {}
-    typedef typename prec_traits<data_type>::type data_t;
+    using data_t = typename prec_traits_t<data_type>::type;
 
     status_t execute(const exec_ctx_t &ctx) const override {
         if (pd()->use_dense_)
