@@ -1,5 +1,6 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
+* Copyright 2022 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -18,7 +19,8 @@
 #define CPU_REF_FUSED_CONVOLUTION_HPP
 
 #include "common/primitive.hpp"
-#include "common/primitive_iterator.hpp"
+#include "common/primitive_desc_iterator.hpp"
+#include "common/reorder.hpp"
 #include "common/stream.hpp"
 
 #include "cpu/cpu_convolution_pd.hpp"
@@ -72,48 +74,84 @@ struct ref_fused_convolution_fwd_t : public primitive_t {
     };
 
     struct pd_t : public cpu_convolution_fwd_pd_t {
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const typename pd_t::base_class *hint_fwd_pd)
-            : cpu_convolution_fwd_pd_t(adesc, attr, hint_fwd_pd) {
-            name_ = "ref_fused_convolution:any";
-        }
-
-        pd_t(const pd_t &other) : cpu_convolution_fwd_pd_t(other) {
-            copy_from(other);
-        }
+        using cpu_convolution_fwd_pd_t::cpu_convolution_fwd_pd_t;
 
         DECLARE_COMMON_PD_T(name_.c_str(), ref_fused_convolution_fwd_t);
 
         virtual status_t init(engine_t *engine) {
-            bool ok = true && is_fwd()
-                    && (attr()->post_ops_.find(primitive_kind::sum) == -1);
+            using namespace primitive_kind;
 
-            if (!ok) return status::unimplemented;
+            VDISPATCH_CONV(is_fwd(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_CONV(attr()->post_ops_.has_default_values(
+                                   {binary, eltwise, convolution}),
+                    VERBOSE_UNSUPPORTED_ATTR);
 
             CHECK(init_ops(engine));
             init_name();
             return status::success;
         }
 
-        const memory_desc_t *src_md(int index = 0) const override {
-            return op_pds_.front()->src_md(index);
+        const memory_desc_t *src_md(
+                int index = 0, bool user_input = false) const override {
+            if (op_pds_.empty())
+                return cpu_convolution_fwd_pd_t::src_md(index, user_input);
+            return op_pds_.front()->src_md(index, user_input);
         }
 
-        const memory_desc_t *dst_md(int index = 0) const override {
-            return op_pds_.back()->dst_md(index);
+        const memory_desc_t *dst_md(
+                int index = 0, bool user_input = false) const override {
+            if (op_pds_.empty())
+                return cpu_convolution_fwd_pd_t::dst_md(index, user_input);
+            return op_pds_.back()->dst_md(index, user_input);
         }
 
-        const memory_desc_t *weights_md(int index = 0) const override {
-            return op_pds_.front()->weights_md(index); // for now
+        const memory_desc_t *weights_md(
+                int index = 0, bool user_input = false) const override {
+            if (op_pds_.empty())
+                return cpu_convolution_fwd_pd_t::weights_md(index, user_input);
+            return op_pds_.front()->weights_md(index, user_input); // for now
         }
 
-        const memory_desc_t *arg_md(int index = 0) const override {
-            switch (index) { // for now
+        const memory_desc_t *arg_md(
+                int arg, bool user_input = false) const override {
+            if (op_pds_.empty())
+                return cpu_convolution_fwd_pd_t::arg_md(arg, user_input);
+            // Binary post-op:
+            // format_tag::any should be supported here since output dst_md
+            // may be different from the intermediate one and they should be
+            // initialized and queried separately.
+            if (arg >= DNNL_ARG_ATTR_MULTIPLE_POST_OP(0)
+                    && arg < DNNL_ARG_ATTR_MULTIPLE_POST_OP(
+                               post_ops_t::post_ops_limit)) {
+                const auto &po = attr()->post_ops_;
+                auto dw_idx = po.find(primitive_kind::convolution);
+                for (int idx = 0; idx < po.len(); ++idx) {
+                    if (arg
+                            != (DNNL_ARG_ATTR_MULTIPLE_POST_OP(idx)
+                                    | DNNL_ARG_SRC_1))
+                        continue;
+
+                    if (dw_idx > idx)
+                        return &op_pds_.front()
+                                        ->attr()
+                                        ->post_ops_.entry_[idx]
+                                        .binary.src1_desc;
+                    else
+                        return &op_pds_.back()
+                                        ->attr()
+                                        ->post_ops_.entry_[idx - (dw_idx + 1)]
+                                        .binary.src1_desc;
+                }
+            }
+
+            switch (arg) { // for now
+                case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_SRC:
+                    return op_pds_.front()->dst_md(0, user_input);
                 case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS:
                     return op_pds_.back()->weights_md(0);
                 case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS:
                     return op_pds_.back()->weights_md(1);
-                default: return convolution_fwd_pd_t::arg_md(index);
+                default: return convolution_fwd_pd_t::arg_md(arg, user_input);
             }
         }
 
@@ -121,42 +159,38 @@ struct ref_fused_convolution_fwd_t : public primitive_t {
             if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS))
                 return arg_usage_t::input;
 
-            if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS)
-                    && attr_post_op_dw_inputs() > 1)
+            if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS))
+                return attr_post_op_dw_inputs() > 1 ? arg_usage_t::input
+                                                    : arg_usage_t::unused;
+
+            if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_SRC))
+                return arg_usage_t::input;
+
+            if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_DST))
                 return arg_usage_t::input;
 
             return convolution_fwd_pd_t::arg_usage(arg);
         }
 
         size_t user_scratchpad_size_;
-        std::vector<std::unique_ptr<primitive_desc_t>> op_pds_;
+        std::vector<std::shared_ptr<primitive_desc_t>> op_pds_;
         std::vector<arg_cache_t> args_;
 
     private:
-        std::string name_;
+        std::string name_ = "ref_fused_convolution:any";
         const unsigned int max_fusions_ = 1;
 
-        status_t append_op(primitive_desc_t *op_pd, size_t &sp_begin,
-                size_t &sp_end, engine_t *engine) {
+        status_t append_op(std::shared_ptr<primitive_desc_t> &op_pd,
+                size_t &sp_begin, size_t &sp_end, engine_t *engine) {
             auto from_md = op_pds_.back()->dst_md();
             auto to_md = op_pd->src_md();
 
             if (*from_md != *to_md) {
                 //TODO: Find a test-case for this
-                reorder_pd_t *r_pd = nullptr;
-                auto r_impls = engine->get_reorder_implementation_list(
-                        from_md, to_md);
-                for (auto r = r_impls; *r; ++r) {
-                    primitive_attr_t attr;
-                    attr.set_scratchpad_mode(scratchpad_mode::user);
-                    if ((*r)(&r_pd, engine, &attr, engine, from_md, engine,
-                                to_md)
-                            == status::success) {
-                        op_pds_.emplace_back((primitive_desc_t *)r_pd);
-                        break;
-                    }
-                }
-                if (!r_pd) return status::unimplemented;
+                std::shared_ptr<primitive_desc_t> pd;
+                CHECK(reorder_primitive_desc_create(
+                        pd, engine, from_md, to_md));
+                op_pds_.emplace_back(std::move(pd));
 
                 arg_cache_t arg_cache;
                 arg_cache.append_inout_arg(
@@ -169,12 +203,13 @@ struct ref_fused_convolution_fwd_t : public primitive_t {
                 sp_end += memory_desc_wrapper(to_md).size();
 
                 user_scratchpad_size_ = nstl::max<size_t>(user_scratchpad_size_,
-                        op_pds_.back()->scratchpad_size(scratchpad_mode::user));
+                        op_pds_.back()->scratchpad_size(
+                                attr()->scratchpad_mode_));
             }
 
-            op_pds_.emplace_back(op_pd);
+            op_pds_.emplace_back(std::move(op_pd));
             user_scratchpad_size_ = nstl::max<size_t>(user_scratchpad_size_,
-                    op_pds_.back()->scratchpad_size(scratchpad_mode::user));
+                    op_pds_.back()->scratchpad_size(attr()->scratchpad_mode_));
             return status::success;
         }
 
@@ -182,23 +217,25 @@ struct ref_fused_convolution_fwd_t : public primitive_t {
             using namespace data_type;
             primitive_attr_t root_attr(*attr());
             if (!root_attr.is_initialized()) return status::out_of_memory;
-            root_attr.set_scratchpad_mode(scratchpad_mode::user);
             auto po_op_iter
                     = attr()->post_ops_.find(primitive_kind::convolution);
             if (po_op_iter == -1) return status::unimplemented;
 
             primitive_attr_t attr_1x1(*attr());
+            // erase dw_conv post-op scales
+            for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST}) {
+                if (!attr_1x1.scales_.has_default_values(
+                            DNNL_ARG_ATTR_POST_OP_DW | arg))
+                    CHECK(attr_1x1.scales_.set(DNNL_ARG_ATTR_POST_OP_DW | arg,
+                            default_quant_entry()));
+            }
             // erase post-ops after fusion as they will be handled separately
             auto &e = attr_1x1.post_ops_.entry_;
             e.erase(e.begin() + po_op_iter, e.end());
 
-            attr_1x1.set_scratchpad_mode(scratchpad_mode::user);
-
-            dnnl_primitive_desc_iterator it(
-                    engine, op_desc(), &attr_1x1, nullptr);
+            primitive_desc_iterator_t it(engine, op_desc(), &attr_1x1, nullptr);
             if (!it.is_initialized()) return status::out_of_memory;
-            ++it;
-            primitive_desc_t *root_pd = it.fetch_once();
+            std::shared_ptr<primitive_desc_t> root_pd = *(++it);
             if (!root_pd) return status::unimplemented;
             op_pds_.emplace_back(root_pd);
             // Scratchpad offsets. Simulate offset computation so that offset
@@ -206,16 +243,21 @@ struct ref_fused_convolution_fwd_t : public primitive_t {
             size_t inout_sp_offset_begin = 0;
             size_t inout_sp_offset_end = 0;
             user_scratchpad_size_
-                    = root_pd->scratchpad_size(scratchpad_mode::user);
+                    = root_pd->scratchpad_size(attr()->scratchpad_mode_);
 
             // Create arg cache for the root pd
             arg_cache_t arg_cache;
             arg_cache.append_ctx_arg(DNNL_ARG_SRC);
             arg_cache.append_ctx_arg(DNNL_ARG_WEIGHTS);
+            for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST})
+                if (!attr_1x1.scales_.has_default_values(arg))
+                    arg_cache.append_ctx_arg(DNNL_ARG_ATTR_SCALES | arg);
             if (desc()->bias_desc.data_type != data_type::undef)
                 arg_cache.append_ctx_arg(DNNL_ARG_BIAS);
             arg_cache.append_inout_arg(DNNL_ARG_DST, inout_sp_offset_end,
                     root_pd->dst_md(), false);
+            // Initialize binary post_op.
+            CHECK(attr_1x1.set_default_formats(root_pd->dst_md()));
             for (int idx = 0; idx < attr_1x1.post_ops_.len(); ++idx) {
                 if (attr_1x1.post_ops_.contain(primitive_kind::binary, idx))
                     arg_cache.append_ctx_arg(DNNL_ARG_ATTR_MULTIPLE_POST_OP(idx)
@@ -256,19 +298,15 @@ struct ref_fused_convolution_fwd_t : public primitive_t {
                 primitive_attr_t attr_dw;
                 CHECK(get_depthwise_conv_desc(cd_dw, *(conv_pd->dst_md()),
                         root_attr, attr_dw, po_op_iter));
-                dnnl_primitive_desc_iterator it(
+                primitive_desc_iterator_t it(
                         engine, (op_desc_t *)&cd_dw, &attr_dw, nullptr);
                 if (!it.is_initialized()) return status::out_of_memory;
 
-                primitive_desc_t *append_conv_pd = (++it).fetch_once();
+                std::shared_ptr<primitive_desc_t> append_conv_pd = *(++it);
                 if (!append_conv_pd) return status::unimplemented;
 
-                auto status = append_op(append_conv_pd, inout_sp_offset_begin,
-                        inout_sp_offset_end, engine);
-                if (status != status::success) {
-                    delete append_conv_pd;
-                    return status;
-                }
+                CHECK(append_op(append_conv_pd, inout_sp_offset_begin,
+                        inout_sp_offset_end, engine));
 
                 const auto &op = op_pds_.back();
                 arg_cache_t arg_cache;
@@ -277,9 +315,21 @@ struct ref_fused_convolution_fwd_t : public primitive_t {
                 arg_cache.append_ctx_arg(DNNL_ARG_DST);
                 arg_cache.append_ctx_arg(DNNL_ARG_WEIGHTS,
                         DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS);
+                for (auto arg : {DNNL_ARG_WEIGHTS, DNNL_ARG_DST})
+                    if (!attr_dw.scales_.has_default_values(arg))
+                        arg_cache.append_ctx_arg(DNNL_ARG_ATTR_SCALES | arg,
+                                DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_ATTR_SCALES
+                                        | arg);
+                // dw_conv src_scale = 1x1_conv dst_scale
+                if (!attr_1x1.scales_.has_default_values(DNNL_ARG_DST))
+                    arg_cache.append_ctx_arg(
+                            DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC,
+                            DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
                 if (op->weights_md(1)->data_type != data_type::undef)
                     arg_cache.append_ctx_arg(DNNL_ARG_BIAS,
                             DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS);
+                // Initialize binary post_op.
+                CHECK(attr_dw.set_default_formats(op->dst_md()));
                 for (int idx = 0; idx < attr_dw.post_ops_.len(); ++idx) {
                     if (attr_dw.post_ops_.contain(primitive_kind::binary, idx))
                         arg_cache.append_ctx_arg(
@@ -325,16 +375,6 @@ struct ref_fused_convolution_fwd_t : public primitive_t {
             }
             return;
         }
-
-        void copy_from(const pd_t &other) {
-            user_scratchpad_size_ = other.user_scratchpad_size_;
-            op_pds_.clear();
-            for (const auto &other_op_pd : other.op_pds_)
-                op_pds_.emplace_back(other_op_pd->clone());
-            args_ = other.args_;
-            name_ = other.name_;
-            return;
-        }
     };
 
     ref_fused_convolution_fwd_t(const pd_t *apd) : primitive_t(apd) {}
@@ -349,6 +389,16 @@ struct ref_fused_convolution_fwd_t : public primitive_t {
         return status::success;
     }
 
+#if DNNL_AARCH64 && DNNL_AARCH64_USE_ACL
+    status_t create_resource(
+            engine_t *engine, resource_mapper_t &mapper) const override {
+        for (auto &p : primitives_) {
+            CHECK(p->create_resource(engine, mapper));
+        }
+        return status::success;
+    }
+#endif
+
     status_t execute(const exec_ctx_t &ctx) const override {
         engine_t *engine = ctx.stream()->engine();
         const auto scratchpad = ctx.get_scratchpad_grantor();
@@ -358,7 +408,7 @@ struct ref_fused_convolution_fwd_t : public primitive_t {
 
         const auto &ctx_args = ctx.args();
         const auto op_count = primitives_.size();
-        std::vector<std::unique_ptr<memory_t>> inout_memory;
+        std::vector<std::unique_ptr<memory_t, memory_deleter_t>> inout_memory;
 
         for (size_t i = 0; i < op_count; ++i) {
             const auto &op = primitives_[i];
