@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2021 Intel Corporation
+* Copyright 2018-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -42,18 +42,29 @@ struct ref_shuffle_t : public primitive_t {
         status_t init(engine_t *engine) {
             using namespace format_tag;
 
-            const data_type_t data_type = data_md()->data_type;
-            bool ok = platform::has_data_type_support(data_type)
-                    && attr()->has_default_values()
-                    && IMPLICATION(!is_fwd(), set_default_formats_common());
-            if (!ok) return status::unimplemented;
+            const memory_desc_wrapper src_d(
+                    is_fwd() ? src_md() : diff_src_md());
+            const memory_desc_wrapper dst_d(
+                    is_fwd() ? dst_md() : diff_dst_md());
+
+            VDISPATCH_SHUFFLE(src_d.data_type() == dst_d.data_type(),
+                    VERBOSE_INCONSISTENT_DT, "src", "dst");
+            VDISPATCH_SHUFFLE(
+                    platform::has_data_type_support(src_d.data_type()),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_SHUFFLE(
+                    attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_SHUFFLE(
+                    set_default_formats_common(), VERBOSE_UNSUPPORTED_TAG);
+            VDISPATCH_SHUFFLE(
+                    src_d == dst_d, VERBOSE_INCONSISTENT_MDS, "src", "dst");
 
             if (ndims() == 5) {
                 dat_tag_ = memory_desc_matches_one_of_tag(
-                        *data_md(), nCdhw16c, nCdhw8c, nCdhw4c, ncdhw, ndhwc);
+                        *src_d.md_, nCdhw16c, nCdhw8c, nCdhw4c, ncdhw, ndhwc);
             } else if (ndims() == 4) {
                 dat_tag_ = memory_desc_matches_one_of_tag(
-                        *data_md(), nChw16c, nChw8c, nChw4c, nchw, nhwc);
+                        *src_d.md_, nChw16c, nChw8c, nChw4c, nchw, nhwc);
             } else
                 dat_tag_ = any;
 
@@ -66,26 +77,27 @@ struct ref_shuffle_t : public primitive_t {
     ref_shuffle_t(const pd_t *apd) : primitive_t(apd) {}
 
     status_t init(engine_t *engine) override {
-        const int axis_size = pd()->axis_size();
-        const int group_size = pd()->group_size();
-        const int transpose_row
+        const dim_t axis_size = pd()->axis_size();
+        const dim_t group_size = pd()->group_size();
+        const dim_t transpose_row
                 = pd()->is_fwd() ? group_size : axis_size / group_size;
-        const int transpose_col
+        const dim_t transpose_col
                 = pd()->is_fwd() ? axis_size / group_size : group_size;
-        rev_transposed_ = (int *)malloc(
-                axis_size * sizeof(int), platform::get_cache_line_size());
+        rev_transposed_ = (dim_t *)malloc(
+                axis_size * sizeof(dim_t), platform::get_cache_line_size());
         if (rev_transposed_ == nullptr) return dnnl_out_of_memory;
-        parallel_nd(transpose_col, transpose_row, [&](int i, int j) {
+        parallel_nd(transpose_col, transpose_row, [&](dim_t i, dim_t j) {
             rev_transposed_[j * transpose_col + i] = i * transpose_row + j;
         });
         return dnnl_success;
     }
 
-    ~ref_shuffle_t() { free(rev_transposed_); }
+    ~ref_shuffle_t() override { free(rev_transposed_); }
 
     status_t execute(const exec_ctx_t &ctx) const override {
-        const data_type_t data_type = pd()->data_md()->data_type;
-        switch (types::data_type_size(data_type)) {
+        const memory_desc_wrapper src_d(
+                pd()->is_fwd() ? pd()->src_md() : pd()->diff_src_md());
+        switch (types::data_type_size(src_d.data_type())) {
             case sizeof(float): return execute_<sizeof(float)>(ctx); break;
             case sizeof(bfloat16_t):
                 return execute_<sizeof(bfloat16_t)>(ctx);
@@ -100,7 +112,7 @@ private:
     template <int data_type_size>
     status_t execute_(const exec_ctx_t &ctx) const;
     const pd_t *pd() const { return (const pd_t *)primitive_t::pd().get(); }
-    int *rev_transposed_ = nullptr;
+    dim_t *rev_transposed_ = nullptr;
 };
 
 } // namespace cpu
