@@ -1,5 +1,6 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
+* Copyright 2022 FUJITSU LIMITED
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -22,21 +23,28 @@ namespace cpu {
 
 // clang-format off
 
-const impl_list_map_t regular_f32_s32_impl_list_map {
-    // f32 -> s32
-    {{f32, s32, 0}, {
-        REG_FAST_DIRECT_COPY_COMMA(f32, s32)
+const impl_list_map_t &regular_f32_s32_impl_list_map() {
+    static const impl_list_map_t the_map = REG_REORDER_P({
+        // f32 -> s32
+        {{f32, s32, 0}, {
+            DNNL_X64_ONLY(CPU_REORDER_INSTANCE(x64::jit_uni_reorder_direct_copy_t))
+            DNNL_X64_ONLY(CPU_REORDER_INSTANCE(x64::jit_blk_reorder_t))
+            DNNL_X64_ONLY(CPU_REORDER_INSTANCE(x64::jit_uni_reorder_t))
 
-        DNNL_X64_ONLY(x64::jit_uni_reorder_create,)
-        DNNL_AARCH64_ONLY(aarch64::jit_uni_reorder_create,)
+            DNNL_AARCH64_ONLY(CPU_REORDER_INSTANCE(aarch64::jit_blk_reorder_t))
+            DNNL_AARCH64_ONLY(CPU_REORDER_INSTANCE(aarch64::jit_uni_reorder_t))
 
-        REG_SR_BIDIR(f32, any, s32, nChw16c),
+            REG_FAST_DIRECT_COPY(f32, s32)
 
-        REG_SR(f32, any, s32, any, fmt_order::any, spec::reference),
+            DNNL_NON_X64_ONLY(REG_SR_BIDIR(f32, any, s32, nChw16c))
 
-        nullptr,
-    }},
-};
+            REG_SR(f32, any, s32, any, fmt_order::any, spec::reference)
+
+            nullptr,
+        }},
+    });
+    return the_map;
+}
 
 // clang-format on
 
