@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -47,7 +47,7 @@ static bool check_dims_contiguous_except_one(const memory_desc_wrapper &mdw,
     dim_t expect_stride = 1;
     for (int idx = mdw.ndims() - 1; idx >= 0; --idx) {
         const int permuted_idx = *(perm.begin() + idx);
-        bool ok = (idx == idx_with_arbitrary_stride)
+        const bool ok = (idx == idx_with_arbitrary_stride)
                 ? expect_stride <= blk.strides[permuted_idx]
                 : expect_stride == blk.strides[permuted_idx];
         if (!ok) return false;
@@ -75,20 +75,28 @@ bool rnn_utils::is_ldoi(const memory_desc_wrapper &mdw) {
 
 bool rnn_utils::is_ldigo_blocked(const memory_desc_wrapper &mdw) {
     format_tag_t md_format_tag = mdw.matches_one_of_tag(format_tag::ldgOi32o,
-            format_tag::ldgOI32o2i, format_tag::ldgOI32o4i);
+            format_tag::ldgOI32o2i, format_tag::ldgOI32o4i,
+            format_tag::ldgOI16o4i, format_tag::ldgOI64o2i,
+            format_tag::ldgOI64o4i, format_tag::ldgOi16o);
+    return md_format_tag != format_tag::undef;
+}
+
+bool rnn_utils::is_ldgoi_blocked(const memory_desc_wrapper &mdw) {
+    format_tag_t md_format_tag = mdw.matches_one_of_tag(
+            format_tag::ldgIo32i, format_tag::ldgIO32i2o, format_tag::ldgIo16i);
     return md_format_tag != format_tag::undef;
 }
 
 bool rnn_utils::is_ldio_blocked(const memory_desc_wrapper &mdw) {
-    format_tag_t md_format_tag = mdw.matches_one_of_tag(
-            format_tag::ldOi32o, format_tag::ldOI32o4i);
+    format_tag_t md_format_tag = mdw.matches_one_of_tag(format_tag::ldOi32o,
+            format_tag::ldOI32o4i, ldOI16o4i, format_tag::ldOi16o);
     return md_format_tag != format_tag::undef;
 }
 
 int rnn_utils::get_good_ld(int dim, int sizeof_dt) {
     // we want matrices leading dimentions to be 64-byte aligned,
     // and not divisible by 256 to avoid 4K aliasing effects
-    int ld = rnd_up(dim, 64 / sizeof_dt);
+    const int ld = rnd_up(dim, 64 / sizeof_dt);
     return (ld % 256 == 0) ? ld + 64 / sizeof_dt : ld;
 }
 
@@ -144,7 +152,10 @@ void rnn_utils::set_offsets(const rnn_conf_t &rnn, size_t &ws_gates_offset,
     register_space(scratch_ht);
     register_space(scratch_diff_ht);
     register_space(scratch_cell);
-    if (rnn.copy_bias) register_space(ws_bias);
+    if (rnn.copy_bias)
+        register_space(ws_bias);
+    else
+        ws_bias_offset = 0;
 
     scratchpad_size = current_offset;
 #undef register_space
@@ -169,7 +180,7 @@ void rnn_utils::get_scratchpad_and_workspace_sizes(const rnn_conf_t &rnn,
 status_t rnn_utils::set_good_strides(
         memory_desc_t &weights_md, format_tag_t tag) {
     auto &strides = weights_md.format_desc.blocking.strides;
-    auto dims = weights_md.dims;
+    const auto dims = weights_md.dims;
 
     int ld_dim_idx = 0;
     switch (tag) {
@@ -217,7 +228,9 @@ status_t rnn_utils::set_expected_desc(rnn_conf_t &rnn,
         rnn_packed_desc_t &rnn_pdata = weights_md.format_desc.rnn_packed_desc;
         switch (weights_type) {
             case weights_type_t::iter:
-                rnn_pdata.format = rnn.is_fwd ? dnnl_ldigo_p : dnnl_ldgoi_p;
+                rnn_pdata.format = rnn.is_fwd
+                        ? rnn_packed_memory_format_t::ldigo_p
+                        : rnn_packed_memory_format_t::ldgoi_p;
                 rnn_pdata.ldb = rnn.ws_states_iter_ld;
                 rnn_pdata.n = rnn.mb;
                 rnn_pdata.n_parts = rnn.n_parts_weights_iter;
@@ -229,7 +242,9 @@ status_t rnn_utils::set_expected_desc(rnn_conf_t &rnn,
                 rnn_pdata.size = rnn.weights_iter_pack_size;
                 break;
             case weights_type_t::layer:
-                rnn_pdata.format = rnn.is_fwd ? dnnl_ldigo_p : dnnl_ldgoi_p;
+                rnn_pdata.format = rnn.is_fwd
+                        ? rnn_packed_memory_format_t::ldigo_p
+                        : rnn_packed_memory_format_t::ldgoi_p;
                 rnn_pdata.ldb = rnn.ws_states_layer_ld;
                 rnn_pdata.n
                         = rnn.merge_gemm_layer ? rnn.n_iter * rnn.mb : rnn.mb;
@@ -243,7 +258,7 @@ status_t rnn_utils::set_expected_desc(rnn_conf_t &rnn,
                 break;
             case weights_type_t::projection:
                 // TODO: add ldoi_p for bwd?
-                rnn_pdata.format = dnnl_ldio_p;
+                rnn_pdata.format = rnn_packed_memory_format_t::ldio_p;
                 rnn_pdata.ldb = rnn.proj_ht_ld;
                 rnn_pdata.n = rnn.mb;
                 rnn_pdata.n_parts = rnn.n_parts_weights_projection;
@@ -258,34 +273,69 @@ status_t rnn_utils::set_expected_desc(rnn_conf_t &rnn,
                 break;
             default: assert(!"unsupported weights type");
         }
+        if (rnn.is_signed_int8_conf()) {
+            weights_md.extra.flags
+                    = 0 | memory_extra_flags::rnn_s8s8_compensation;
+            weights_md.extra.compensation_mask = 0;
+        }
     } else {
         using namespace format_tag;
         if (rnn.is_brgemm) {
+            const int n_block = rnn.n_block;
             format_tag_t tag = format_tag::undef;
-            tag = (weights_type == weights_type_t::projection)
-                    ? (rnn.is_int8()) ? format_tag::ldOI32o4i
-                                      : format_tag::ldOi32o
-                    : (rnn.is_int8()) ? format_tag::ldgOI32o4i
-                                      : (rnn.is_bf16()) ? format_tag::ldgOI32o2i
-                                                        : format_tag::ldgOi32o;
+
+            if (weights_type == weights_type_t::projection) {
+                if (rnn.is_int8_conf())
+                    tag = utils::map(n_block, format_tag::undef, 32,
+                            format_tag::ldOI32o4i, 16, format_tag::ldOI16o4i);
+                else
+                    tag = utils::map(n_block, format_tag::undef, 32,
+                            format_tag::ldOi32o, 16, format_tag::ldOi16o);
+            } else if (rnn.is_fwd) {
+                if (rnn.is_int8_conf())
+                    tag = utils::map(n_block, format_tag::undef, 64,
+                            format_tag::ldgOI64o4i, 32, ldgOI32o4i, 16,
+                            ldgOI16o4i);
+                else if (rnn.is_xf16_conf())
+                    tag = utils::map(n_block, format_tag::undef, 64,
+                            format_tag::ldgOI64o2i, 32, ldgOI32o2i);
+                else {
+                    assert(IMPLICATION(n_block == 64, rnn.is_bf32()));
+                    tag = utils::map(n_block, format_tag::undef, 64,
+                            format_tag::ldgOi32o /*bf32*/, 32,
+                            format_tag::ldgOi32o, 16, ldgOi16o);
+                }
+            } else {
+                if (rnn.is_xf16_conf())
+                    tag = format_tag::ldgIO32i2o;
+                else
+                    tag = utils::map(n_block, format_tag::undef, 32,
+                            format_tag::ldgIo32i, 16, format_tag::ldgIo16i);
+            }
+
             if (tag != format_tag::undef) {
                 CHECK(memory_desc_init_by_tag(weights_md, tag));
-                if (rnn.is_int8()) {
+                if (rnn.is_unsigned_int8_conf()) {
                     weights_md.extra.flags
                             = 0 | memory_extra_flags::rnn_u8s8_compensation;
                     weights_md.extra.compensation_mask
                             = (weights_type == weights_type_t::projection)
                             ? 13 // 1101
                             : 27; // 11011
+                } else if (rnn.is_signed_int8_conf()) {
+                    weights_md.extra.flags
+                            = 0 | memory_extra_flags::rnn_s8s8_compensation;
+                    weights_md.extra.compensation_mask = 0;
                 }
                 return status::success;
             } else {
                 return status::unimplemented;
             }
         } else {
-            format_tag_t tag = weights_type == weights_type_t::projection
+            const format_tag_t tag = weights_type == weights_type_t::projection
                     ? rnn.is_fwd ? ldio : ldoi
-                    : rnn.is_fwd ? ldigo : ldgoi;
+                    : rnn.is_fwd ? ldigo
+                                 : ldgoi;
             CHECK(memory_desc_init_by_tag(weights_md, tag));
             // Adjust strides for good leading dimension in GEMM
             CHECK(set_good_strides(weights_md, tag));
@@ -294,6 +344,33 @@ status_t rnn_utils::set_expected_desc(rnn_conf_t &rnn,
     return status::success;
 }
 
+float rnn_utils::to_float(const void *data, const data_type_t dt) {
+    if (dt == data_type::f32)
+        return *static_cast<const float *>(data);
+    else if (dt == data_type::bf16)
+        return float(*static_cast<const bfloat16_t *>(data));
+    else if (dt == data_type::f16)
+        return float(*static_cast<const float16_t *>(data));
+    return 0.0;
+}
+
+const void *rnn_utils::inc_ptr(
+        const void *data, data_type_t data_type, int offset) {
+    if (data_type == data_type::f32)
+        return static_cast<const float *>(data) + offset;
+    else if (data_type == data_type::bf16)
+        return static_cast<const bfloat16_t *>(data) + offset;
+    else if (data_type == data_type::f16)
+        return static_cast<const float16_t *>(data) + offset;
+    else
+        return data;
+}
+
+void *rnn_utils::inc_ptr(void *data, data_type_t data_type, int offset) {
+    return const_cast<void *>(
+            inc_ptr(static_cast<const void *>(data), data_type, offset));
+}
+
 } // namespace cpu
 } // namespace impl
 } // namespace dnnl
