@@ -252,6 +252,21 @@ status_t brgemm_desc_init(brgemm_desc_t *brg, cpu_isa_t isa,
 
     if (M <= 0 || N <= 0 || K <= 0) return status::invalid_arguments;
 
+    // Upper bound, this can likely be improved by accounting for blocking
+    dim_t max_a_stride = brg->LDA * types::data_type_size(brg->dt_a)
+            * (brg->layout == brgemm_col_major ? K : M);
+    dim_t max_b_stride = brg->LDB * types::data_type_size(brg->dt_b)
+            * (brg->layout == brgemm_col_major ? N : K);
+    dim_t max_c_stride = brg->LDC * types::data_type_size(brg->dt_c)
+            * (brg->layout == brgemm_col_major ? N : M);
+
+    // Required for EVEX encoding for offsets
+    const dim_t max_stride = std::numeric_limits<int32_t>::max();
+    if ((max_a_stride > max_stride && !brg->is_runtime_lda)
+            || (max_b_stride > max_stride && !brg->is_runtime_ldb)
+            || (max_c_stride >= max_stride && !brg->is_runtime_ldc))
+        return status::unimplemented;
+
     if (utils::everyone_is(false, brg->is_int8, brg->is_bf16, brg->is_f32,
                 brg->is_f16, brg->is_fp8))
         return status::unimplemented;
@@ -423,46 +438,47 @@ status_t brgemm_desc_set_postops(brgemm_desc_t *brg,
     if (brg->with_scales) {
         // Note. the current version supports only two different output scale
         // types:
-        //     1) common (mask_ = 0)
+        //     1) common (mask = 0)
         //     2) per_n_dim_scale - broadcast across n dimension;
         //        for convolution and inner product promitives it corresponds
-        //        to "per_oc" mask_ = 1 << 1; for matmul - to
-        //        mask_ = (1 << (ndims - 1))), where ndims is number of
+        //        to "per_oc" mask = 1 << 1; for matmul - to
+        //        mask = (1 << (ndims - 1))), where ndims is number of
         //        dimensions for original matmul problem
-        // So if wei_scales.mask_ != 0 (not common) it's assumed here that scale
-        // type is per_n_dim_scale and driver which calls brgemm kernel checked
-        // that mask has correct value for this case
-        brg->is_oc_scale = wei_scales.mask_ != 0;
+        // So if wei_scales.get_mask() > 0 (not common) it's assumed here that
+        // scale type is per_n_dim_scale and driver which calls brgemm kernel
+        // checked that mask has correct value for this case
+        brg->is_oc_scale = wei_scales.get_mask() > 0;
     }
 
     const auto &dst_scales = attr->scales_.get(DNNL_ARG_DST);
     brg->with_dst_scales = !dst_scales.has_default_values();
-    const bool scales_ok = src_scales.mask_ == 0 && dst_scales.mask_ == 0
-            && attr->scales_.has_default_values(
-                    {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST});
+    const bool scales_ok = attr->scales_.has_default_values({DNNL_ARG_SRC,
+                                   DNNL_ARG_WEIGHTS, DNNL_ARG_DST})
+            && IMPLICATION(!src_scales.has_default_values(),
+                    src_scales.get_mask() == 0)
+            && IMPLICATION(!dst_scales.has_default_values(),
+                    dst_scales.get_mask() == 0);
     if (!scales_ok) return status::unimplemented;
 
     auto init_zp_type
             = [&](brgemm_broadcast_t &zp_type, int mem_arg) -> status_t {
-        auto zero_points = attr->zero_points_;
-
-        // common zero point type is supported for now
-        const bool is_per_dim_1_bcast = zero_points.per_dim_1(mem_arg);
-        const bool is_common_bcast = zero_points.common(mem_arg);
-        if (!is_common_bcast && !is_per_dim_1_bcast)
-            return status::unimplemented;
+        const auto &zp = attr->zero_points_;
+        // Always init a default value;
+        zp_type = brgemm_broadcast_t::none;
 
         const bool skip_zero_point
                 = mem_arg == DNNL_ARG_WEIGHTS && brg->skip_zp_b_compensation;
+        if (skip_zero_point) return status::success;
 
-        zp_type = brgemm_broadcast_t::none;
-        const bool is_any_bcast
-                = !(zero_points.has_default_values(mem_arg) || skip_zero_point);
-        if (is_any_bcast) {
-            if (is_common_bcast)
+        if (!zp.has_default_values(mem_arg)) {
+            int mask = zp.get_mask(mem_arg);
+            if (mask == 0) {
                 zp_type = brgemm_broadcast_t::per_tensor;
-            else if (is_per_dim_1_bcast)
+            } else if (mask == (1 << 1)) {
                 zp_type = brgemm_broadcast_t::per_n;
+            } else {
+                return status::unimplemented;
+            }
         }
 
         return status::success;
