@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -59,7 +59,7 @@ namespace x64 {
 #define TILED(X) dword[rsp + ((X) + 0xc0)]
 #define TILEQ(X) qword[rsp + ((X) + 0xc0)]
 
-void jit_avx512_core_amx_gemm_kern::generate() {
+void jit_avx512_core_amx_gemm_kern_t::generate() {
 
     int kerneltype = ((typea << 1) | typeb);
 
@@ -115,8 +115,8 @@ void jit_avx512_core_amx_gemm_kern::generate() {
     mov(K, qword[r8]);
 
     mov(rcx, ARG_X(0x28));
-    mov(B, ARG_X(0x30));
-    mov(C, ARG_X(0x38));
+    mov(r8, ARG_X(0x30));
+    mov(r9, ARG_X(0x38));
     mov(LDC, ARG_X(0x40));
 #else
 
@@ -455,9 +455,9 @@ void jit_avx512_core_amx_gemm_kern::generate() {
     ret();
 }
 
-jit_avx512_core_amx_gemm_kern::jit_avx512_core_amx_gemm_kern(
+jit_avx512_core_amx_gemm_kern_t::jit_avx512_core_amx_gemm_kern_t(
         int typea, int typeb, int typec, int betaZero)
-    : jit_generator(nullptr, 2 * 1024)
+    : jit_generator_t(jit_name(), avx512_core_amx)
     , typea(typea)
     , typeb(typeb)
     , typec(typec)
