@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -139,7 +139,7 @@ gemm_info_t<a_t, b_t, c_t>::gemm_info_t(const char *transA, const char *transB,
     }
 
     constexpr bool is_int8 = utils::one_of(
-            data_traits<a_t>::data_type, data_type::s8, data_type::u8);
+            data_traits_t<a_t>::data_type, data_type::s8, data_type::u8);
     if (is_int8) this->ao = oa ? *oa : a_t(0);
     prepare_bo<b_t>(this->bo, ob);
 
@@ -155,7 +155,7 @@ gemm_info_t<a_t, b_t, c_t>::gemm_info_t(const char *transA, const char *transB,
         this->co = oc;
     }
 
-    bool is_sgemm = data_traits<a_t>::data_type == data_type::f32;
+    bool is_sgemm = data_traits_t<a_t>::data_type == data_type::f32;
     bool is_gemv = this->m == 1 || this->n == 1;
 
     // Copy-based sgemm doesn't support force-nocopy for ISAs older
@@ -213,7 +213,8 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
     // TODO: Add dispatching for 1-fma SKUs with support to bf16
     // instructions for AMX kernel.
     {
-        constexpr bool is_bf16 = data_traits<a_t>::data_type == data_type::bf16;
+        constexpr bool is_bf16
+                = data_traits_t<a_t>::data_type == data_type::bf16;
         const bool max_isa_supports_bf16_ymm = mayiuse(avx512_core_bf16_ymm)
                 && __BUILD_GEMM_AVX512
                 && !(mayiuse(avx512_core_amx) && __BUILD_GEMM_AMX);
@@ -221,7 +222,7 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
         use_bf16_ymm = is_bf16 && max_isa_supports_bf16_ymm;
     }
 
-    switch (data_traits<a_t>::data_type) {
+    switch (data_traits_t<a_t>::data_type) {
         case data_type::s8:
             if (false) {
                 // dummy if
@@ -391,21 +392,22 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
     static std::once_flag initialized;
     static std::atomic<dnnl_status_t> st(dnnl_success);
     std::call_once(initialized, [&, um] {
-        const bool b_is_s8 = data_traits<b_t>::data_type == data_type::s8;
+        const bool b_is_s8 = data_traits_t<b_t>::data_type == data_type::s8;
         UNUSED(b_is_s8);
         constexpr bool is_int8 = utils::one_of(
-                data_traits<a_t>::data_type, data_type::s8, data_type::u8);
-        constexpr bool is_bf16 = data_traits<a_t>::data_type == data_type::bf16;
+                data_traits_t<a_t>::data_type, data_type::s8, data_type::u8);
+        constexpr bool is_bf16
+                = data_traits_t<a_t>::data_type == data_type::bf16;
         bool is_int8_amx
                 = is_int8 && mayiuse(avx512_core_amx) && __BUILD_GEMM_AMX;
         bool is_bf16_amx
                 = is_bf16 && mayiuse(avx512_core_amx) && __BUILD_GEMM_AMX;
         bool is_amx = is_int8_amx || is_bf16_amx;
 
-        static maybe_unique_ptr<jit_generator> copy_a[2][2] = {{nullptr}};
-        static maybe_unique_ptr<jit_generator> copy_b[2][2] = {{nullptr}};
+        static maybe_unique_ptr<jit_generator_t> copy_a[2][2] = {{nullptr}};
+        static maybe_unique_ptr<jit_generator_t> copy_b[2][2] = {{nullptr}};
 
-        switch (data_traits<a_t>::data_type) {
+        switch (data_traits_t<a_t>::data_type) {
             case data_type::s8:
                 if (false) {
                     // dummy if
@@ -413,123 +415,123 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
                 } else if (mayiuse(amx_int8)) {
                     for (int isTrans : {no_trans, do_trans}) {
                         copy_a[isTrans][no_sum].reset(
-                                new jit_avx512_core_amx_copy_kern(
+                                new jit_avx512_core_amx_copy_kern_t(
                                         true, !isTrans, sizeof(a_t)));
 
                         copy_b[isTrans][no_sum].reset(
-                                new jit_avx512_core_amx_copy_kern(
+                                new jit_avx512_core_amx_copy_kern_t(
                                         false, isTrans, sizeof(b_t)));
                     }
 #endif
 #if __BUILD_GEMM_AVX512
                 } else if (mayiuse(avx512_core)) {
                     copy_a[no_trans][no_sum].reset(
-                            new jit_avx512_core_u8_copy_an_kern());
+                            new jit_avx512_core_u8_copy_an_kern_t());
                     copy_a[do_trans][no_sum].reset(
-                            new jit_avx512_core_u8_copy_at_kern());
+                            new jit_avx512_core_u8_copy_at_kern_t());
 
                     copy_b[no_trans][no_sum].reset(
-                            new jit_avx512_core_u8_copy_bn_kern(b_is_s8));
+                            new jit_avx512_core_u8_copy_bn_kern_t(b_is_s8));
                     copy_b[do_trans][no_sum].reset(
-                            new jit_avx512_core_u8_copy_bt_kern(b_is_s8));
+                            new jit_avx512_core_u8_copy_bt_kern_t(b_is_s8));
 
                     copy_a[no_trans][do_sum].reset(
-                            new jit_avx512_core_u8_copy_sum_an_kern());
+                            new jit_avx512_core_u8_copy_sum_an_kern_t());
                     copy_a[do_trans][do_sum].reset(
-                            new jit_avx512_core_u8_copy_sum_at_kern());
+                            new jit_avx512_core_u8_copy_sum_at_kern_t());
 
                     copy_b[no_trans][do_sum].reset(
-                            new jit_avx512_core_u8_copy_sum_bn_kern(b_is_s8));
+                            new jit_avx512_core_u8_copy_sum_bn_kern_t(b_is_s8));
                     copy_b[do_trans][do_sum].reset(
-                            new jit_avx512_core_u8_copy_sum_bt_kern(b_is_s8));
+                            new jit_avx512_core_u8_copy_sum_bt_kern_t(b_is_s8));
 #endif
 #if __BUILD_GEMM_AVX2
                 } else if (mayiuse(avx2_vnni)) {
                     copy_a[no_trans][no_sum].reset(
-                            new jit_avx2_vnni_u8_copy_an_kern());
+                            new jit_avx2_vnni_u8_copy_an_kern_t());
                     copy_a[do_trans][no_sum].reset(
-                            new jit_avx2_vnni_u8_copy_at_kern());
+                            new jit_avx2_vnni_u8_copy_at_kern_t());
 
                     copy_b[no_trans][no_sum].reset(
-                            new jit_avx2_vnni_u8_copy_bn_kern());
+                            new jit_avx2_vnni_u8_copy_bn_kern_t());
                     copy_b[do_trans][no_sum].reset(
-                            new jit_avx2_vnni_u8_copy_bt_kern());
+                            new jit_avx2_vnni_u8_copy_bt_kern_t());
 
                     copy_a[no_trans][do_sum].reset(
-                            new jit_avx2_vnni_u8_copy_sum_an_kern());
+                            new jit_avx2_vnni_u8_copy_sum_an_kern_t());
                     copy_a[do_trans][do_sum].reset(
-                            new jit_avx2_vnni_u8_copy_sum_at_kern());
+                            new jit_avx2_vnni_u8_copy_sum_at_kern_t());
 
                     copy_b[no_trans][do_sum].reset(
-                            new jit_avx2_vnni_u8_copy_sum_bn_kern());
+                            new jit_avx2_vnni_u8_copy_sum_bn_kern_t());
                     copy_b[do_trans][do_sum].reset(
-                            new jit_avx2_vnni_u8_copy_sum_bt_kern());
+                            new jit_avx2_vnni_u8_copy_sum_bt_kern_t());
 #endif
 #if __BUILD_GEMM_AVX2
                 } else if (mayiuse(avx2)) {
                     copy_a[no_trans][no_sum].reset(
-                            new jit_avx2_u8_copy_an_kern());
+                            new jit_avx2_u8_copy_an_kern_t());
                     copy_a[do_trans][no_sum].reset(
-                            new jit_avx2_u8_copy_at_kern());
+                            new jit_avx2_u8_copy_at_kern_t());
 
                     copy_b[no_trans][no_sum].reset(
-                            new jit_avx2_u8_copy_bn_kern());
+                            new jit_avx2_u8_copy_bn_kern_t());
                     copy_b[do_trans][no_sum].reset(
-                            new jit_avx2_u8_copy_bt_kern());
+                            new jit_avx2_u8_copy_bt_kern_t());
 
                     copy_a[no_trans][do_sum].reset(
-                            new jit_avx2_u8_copy_sum_an_kern());
+                            new jit_avx2_u8_copy_sum_an_kern_t());
                     copy_a[do_trans][do_sum].reset(
-                            new jit_avx2_u8_copy_sum_at_kern());
+                            new jit_avx2_u8_copy_sum_at_kern_t());
 
                     copy_b[no_trans][do_sum].reset(
-                            new jit_avx2_u8_copy_sum_bn_kern());
+                            new jit_avx2_u8_copy_sum_bn_kern_t());
                     copy_b[do_trans][do_sum].reset(
-                            new jit_avx2_u8_copy_sum_bt_kern());
+                            new jit_avx2_u8_copy_sum_bt_kern_t());
 #endif
 #if __BUILD_GEMM_AVX2
                 } else if (mayiuse(avx)) {
                     copy_a[no_trans][no_sum].reset(
-                            new jit_avx_u8_copy_an_kern());
+                            new jit_avx_u8_copy_an_kern_t());
                     copy_a[do_trans][no_sum].reset(
-                            new jit_avx_u8_copy_at_kern());
+                            new jit_avx_u8_copy_at_kern_t());
 
                     copy_b[no_trans][no_sum].reset(
-                            new jit_avx_u8_copy_bn_kern());
+                            new jit_avx_u8_copy_bn_kern_t());
                     copy_b[do_trans][no_sum].reset(
-                            new jit_avx_u8_copy_bt_kern());
+                            new jit_avx_u8_copy_bt_kern_t());
 
                     copy_a[no_trans][do_sum].reset(
-                            new jit_avx_u8_copy_sum_an_kern());
+                            new jit_avx_u8_copy_sum_an_kern_t());
                     copy_a[do_trans][do_sum].reset(
-                            new jit_avx_u8_copy_sum_at_kern());
+                            new jit_avx_u8_copy_sum_at_kern_t());
 
                     copy_b[no_trans][do_sum].reset(
-                            new jit_avx_u8_copy_sum_bn_kern());
+                            new jit_avx_u8_copy_sum_bn_kern_t());
                     copy_b[do_trans][do_sum].reset(
-                            new jit_avx_u8_copy_sum_bt_kern());
+                            new jit_avx_u8_copy_sum_bt_kern_t());
 #endif
 #if __BUILD_GEMM_SSE41
                 } else if (mayiuse(sse41)) {
                     copy_a[no_trans][no_sum].reset(
-                            new jit_sse41_u8_copy_an_kern());
+                            new jit_sse41_u8_copy_an_kern_t());
                     copy_a[do_trans][no_sum].reset(
-                            new jit_sse41_u8_copy_at_kern());
+                            new jit_sse41_u8_copy_at_kern_t());
 
                     copy_b[no_trans][no_sum].reset(
-                            new jit_sse41_u8_copy_bn_kern());
+                            new jit_sse41_u8_copy_bn_kern_t());
                     copy_b[do_trans][no_sum].reset(
-                            new jit_sse41_u8_copy_bt_kern());
+                            new jit_sse41_u8_copy_bt_kern_t());
 
                     copy_a[no_trans][do_sum].reset(
-                            new jit_sse41_u8_copy_sum_an_kern());
+                            new jit_sse41_u8_copy_sum_an_kern_t());
                     copy_a[do_trans][do_sum].reset(
-                            new jit_sse41_u8_copy_sum_at_kern());
+                            new jit_sse41_u8_copy_sum_at_kern_t());
 
                     copy_b[no_trans][do_sum].reset(
-                            new jit_sse41_u8_copy_sum_bn_kern());
+                            new jit_sse41_u8_copy_sum_bn_kern_t());
                     copy_b[do_trans][do_sum].reset(
-                            new jit_sse41_u8_copy_sum_bt_kern());
+                            new jit_sse41_u8_copy_sum_bt_kern_t());
 #endif
                 }
                 break;
@@ -541,37 +543,37 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
                 } else if (mayiuse(amx_bf16)) {
                     for (int isTrans : {no_trans, do_trans}) {
                         copy_a[isTrans][no_sum].reset(
-                                new jit_avx512_core_amx_copy_kern(
+                                new jit_avx512_core_amx_copy_kern_t(
                                         true, !isTrans, sizeof(a_t)));
 
                         copy_b[isTrans][no_sum].reset(
-                                new jit_avx512_core_amx_copy_kern(
+                                new jit_avx512_core_amx_copy_kern_t(
                                         false, isTrans, sizeof(b_t)));
                     }
 #endif
 #if __BUILD_GEMM_AVX512
                 } else if (mayiuse(avx512_core) && !use_bf16_ymm) {
                     copy_a[no_trans][no_sum].reset(
-                            new jit_avx512_core_s16_48x8_copy_an_kern());
+                            new jit_avx512_core_s16_48x8_copy_an_kern_t());
                     copy_a[do_trans][no_sum].reset(
-                            new jit_avx512_core_s16_48x8_copy_at_kern());
+                            new jit_avx512_core_s16_48x8_copy_at_kern_t());
 
                     copy_b[no_trans][no_sum].reset(
-                            new jit_avx512_core_s16_48x8_copy_bn_kern());
+                            new jit_avx512_core_s16_48x8_copy_bn_kern_t());
                     copy_b[do_trans][no_sum].reset(
-                            new jit_avx512_core_s16_48x8_copy_bt_kern());
+                            new jit_avx512_core_s16_48x8_copy_bt_kern_t());
 #endif
 #if __BUILD_GEMM_AVX512
                 } else if (mayiuse(avx512_core) && use_bf16_ymm) {
                     copy_a[no_trans][no_sum].reset(
-                            new jit_avx512_core_s16_24x8_copy_an_kern());
+                            new jit_avx512_core_s16_24x8_copy_an_kern_t());
                     copy_a[do_trans][no_sum].reset(
-                            new jit_avx512_core_s16_24x8_copy_at_kern());
+                            new jit_avx512_core_s16_24x8_copy_at_kern_t());
 
                     copy_b[no_trans][no_sum].reset(
-                            new jit_avx512_core_s16_24x8_copy_bn_kern());
+                            new jit_avx512_core_s16_24x8_copy_bn_kern_t());
                     copy_b[do_trans][no_sum].reset(
-                            new jit_avx512_core_s16_24x8_copy_bt_kern());
+                            new jit_avx512_core_s16_24x8_copy_bt_kern_t());
 #endif
                 }
                 break;
@@ -582,50 +584,50 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
 #if __BUILD_GEMM_AVX512
                 } else if (mayiuse(avx512_core)) {
                     copy_a[no_trans][no_sum].reset(
-                            new jit_avx512_core_f32_copy_an_kern());
+                            new jit_avx512_core_f32_copy_an_kern_t());
                     copy_a[do_trans][no_sum].reset(
-                            new jit_avx512_core_f32_copy_at_kern());
+                            new jit_avx512_core_f32_copy_at_kern_t());
 
                     copy_b[no_trans][no_sum].reset(
-                            new jit_avx512_core_f32_copy_bn_kern());
+                            new jit_avx512_core_f32_copy_bn_kern_t());
                     copy_b[do_trans][no_sum].reset(
-                            new jit_avx512_core_f32_copy_bt_kern());
+                            new jit_avx512_core_f32_copy_bt_kern_t());
 #endif
 #if __BUILD_GEMM_AVX2
                 } else if (mayiuse(avx2)) {
                     copy_a[no_trans][no_sum].reset(
-                            new jit_avx2_f32_copy_an_kern());
+                            new jit_avx2_f32_copy_an_kern_t());
                     copy_a[do_trans][no_sum].reset(
-                            new jit_avx2_f32_copy_at_kern());
+                            new jit_avx2_f32_copy_at_kern_t());
 
                     copy_b[no_trans][no_sum].reset(
-                            new jit_avx2_f32_copy_bn_kern());
+                            new jit_avx2_f32_copy_bn_kern_t());
                     copy_b[do_trans][no_sum].reset(
-                            new jit_avx2_f32_copy_bt_kern());
+                            new jit_avx2_f32_copy_bt_kern_t());
 #endif
 #if __BUILD_GEMM_AVX2
                 } else if (mayiuse(avx)) {
                     copy_a[no_trans][no_sum].reset(
-                            new jit_avx_f32_copy_an_kern());
+                            new jit_avx_f32_copy_an_kern_t());
                     copy_a[do_trans][no_sum].reset(
-                            new jit_avx_f32_copy_at_kern());
+                            new jit_avx_f32_copy_at_kern_t());
 
                     copy_b[no_trans][no_sum].reset(
-                            new jit_avx_f32_copy_bn_kern());
+                            new jit_avx_f32_copy_bn_kern_t());
                     copy_b[do_trans][no_sum].reset(
-                            new jit_avx_f32_copy_bt_kern());
+                            new jit_avx_f32_copy_bt_kern_t());
 #endif
 #if __BUILD_GEMM_SSE41
                 } else if (mayiuse(sse41)) {
                     copy_a[no_trans][no_sum].reset(
-                            new jit_sse41_f32_copy_an_kern());
+                            new jit_sse41_f32_copy_an_kern_t());
                     copy_a[do_trans][no_sum].reset(
-                            new jit_sse41_f32_copy_at_kern());
+                            new jit_sse41_f32_copy_at_kern_t());
 
                     copy_b[no_trans][no_sum].reset(
-                            new jit_sse41_f32_copy_bn_kern());
+                            new jit_sse41_f32_copy_bn_kern_t());
                     copy_b[do_trans][no_sum].reset(
-                            new jit_sse41_f32_copy_bt_kern());
+                            new jit_sse41_f32_copy_bt_kern_t());
 #endif
                 }
                 break;
@@ -633,16 +635,17 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
             default: break;
         }
 
-        constexpr bool is_a_s8 = data_traits<a_t>::data_type == data_type::s8;
-        constexpr bool is_b_s8 = data_traits<b_t>::data_type == data_type::s8;
-        constexpr bool is_c_s32 = data_traits<c_t>::data_type == data_type::s32;
+        constexpr bool is_a_s8 = data_traits_t<a_t>::data_type == data_type::s8;
+        constexpr bool is_b_s8 = data_traits_t<b_t>::data_type == data_type::s8;
+        constexpr bool is_c_s32
+                = data_traits_t<c_t>::data_type == data_type::s32;
         UNUSED(is_a_s8);
         UNUSED(is_b_s8);
         UNUSED(is_c_s32);
 
-        static maybe_unique_ptr<jit_generator> kernel[2][2][2][2]
+        static maybe_unique_ptr<jit_generator_t> kernel[2][2][2][2]
                 = {{{{nullptr}}}};
-        switch (data_traits<a_t>::data_type) {
+        switch (data_traits_t<a_t>::data_type) {
             case data_type::s8:
                 if (false) {
                     // dummy if
@@ -650,7 +653,7 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
                 } else if (mayiuse(avx512_core_amx)) {
                     for (int isBeta0 : {no_beta0, do_beta0}) {
                         kernel[isBeta0][do_alpha1][no_sum][no_sum].reset(
-                                new jit_avx512_core_amx_gemm_kern(
+                                new jit_avx512_core_amx_gemm_kern_t(
                                         is_a_s8, is_b_s8, is_c_s32, isBeta0));
                     }
 #endif
@@ -660,7 +663,7 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
                         for (int doColSum : {no_sum, do_sum})
                             for (int doRowSum : {no_sum, do_sum}) {
                                 kernel[isBeta0][do_alpha1][doColSum][doRowSum].reset(
-                                        new jit_avx512_core_gemm_s8u8s32_kern(
+                                        new jit_avx512_core_gemm_s8u8s32_kern_t(
                                                 isBeta0, doColSum, doRowSum));
                             }
 #endif
@@ -670,7 +673,7 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
                         for (int doColSum : {no_sum, do_sum})
                             for (int doRowSum : {no_sum, do_sum}) {
                                 kernel[isBeta0][do_alpha1][doColSum][doRowSum]
-                                        .reset(new jit_avx2_gemm_s8u8s32_kern(
+                                        .reset(new jit_avx2_gemm_s8u8s32_kern_t(
                                                 isBeta0, doColSum, doRowSum,
                                                 um));
                             }
@@ -678,42 +681,42 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
 #if __BUILD_GEMM_AVX2
                 } else if (mayiuse(avx)) {
                     kernel[no_beta0][do_alpha1][no_sum][no_sum].reset(
-                            new jit_avx_kernel_gemm_s8u8s32_kern());
+                            new jit_avx_kernel_gemm_s8u8s32_kern_t());
                     kernel[no_beta0][do_alpha1][do_sum][no_sum].reset(
-                            new jit_avx_kernel_c_gemm_s8u8s32_kern());
+                            new jit_avx_kernel_c_gemm_s8u8s32_kern_t());
                     kernel[no_beta0][do_alpha1][no_sum][do_sum].reset(
-                            new jit_avx_kernel_r_gemm_s8u8s32_kern());
+                            new jit_avx_kernel_r_gemm_s8u8s32_kern_t());
                     kernel[no_beta0][do_alpha1][do_sum][do_sum].reset(
-                            new jit_avx_kernel_b_gemm_s8u8s32_kern());
+                            new jit_avx_kernel_b_gemm_s8u8s32_kern_t());
 
                     kernel[do_beta0][do_alpha1][no_sum][no_sum].reset(
-                            new jit_avx_kernel_b0_gemm_s8u8s32_kern());
+                            new jit_avx_kernel_b0_gemm_s8u8s32_kern_t());
                     kernel[do_beta0][do_alpha1][do_sum][no_sum].reset(
-                            new jit_avx_kernel_b0_c_gemm_s8u8s32_kern());
+                            new jit_avx_kernel_b0_c_gemm_s8u8s32_kern_t());
                     kernel[do_beta0][do_alpha1][no_sum][do_sum].reset(
-                            new jit_avx_kernel_b0_r_gemm_s8u8s32_kern());
+                            new jit_avx_kernel_b0_r_gemm_s8u8s32_kern_t());
                     kernel[do_beta0][do_alpha1][do_sum][do_sum].reset(
-                            new jit_avx_kernel_b0_b_gemm_s8u8s32_kern());
+                            new jit_avx_kernel_b0_b_gemm_s8u8s32_kern_t());
 #endif
 #if __BUILD_GEMM_SSE41
                 } else if (mayiuse(sse41)) {
                     kernel[no_beta0][do_alpha1][no_sum][no_sum].reset(
-                            new jit_sse41_kernel_gemm_s8u8s32_kern());
+                            new jit_sse41_kernel_gemm_s8u8s32_kern_t());
                     kernel[no_beta0][do_alpha1][do_sum][no_sum].reset(
-                            new jit_sse41_kernel_c_gemm_s8u8s32_kern());
+                            new jit_sse41_kernel_c_gemm_s8u8s32_kern_t());
                     kernel[no_beta0][do_alpha1][no_sum][do_sum].reset(
-                            new jit_sse41_kernel_r_gemm_s8u8s32_kern());
+                            new jit_sse41_kernel_r_gemm_s8u8s32_kern_t());
                     kernel[no_beta0][do_alpha1][do_sum][do_sum].reset(
-                            new jit_sse41_kernel_b_gemm_s8u8s32_kern());
+                            new jit_sse41_kernel_b_gemm_s8u8s32_kern_t());
 
                     kernel[do_beta0][do_alpha1][no_sum][no_sum].reset(
-                            new jit_sse41_kernel_b0_gemm_s8u8s32_kern());
+                            new jit_sse41_kernel_b0_gemm_s8u8s32_kern_t());
                     kernel[do_beta0][do_alpha1][do_sum][no_sum].reset(
-                            new jit_sse41_kernel_b0_c_gemm_s8u8s32_kern());
+                            new jit_sse41_kernel_b0_c_gemm_s8u8s32_kern_t());
                     kernel[do_beta0][do_alpha1][no_sum][do_sum].reset(
-                            new jit_sse41_kernel_b0_r_gemm_s8u8s32_kern());
+                            new jit_sse41_kernel_b0_r_gemm_s8u8s32_kern_t());
                     kernel[do_beta0][do_alpha1][do_sum][do_sum].reset(
-                            new jit_sse41_kernel_b0_b_gemm_s8u8s32_kern());
+                            new jit_sse41_kernel_b0_b_gemm_s8u8s32_kern_t());
 #endif
                 }
                 break;
@@ -725,7 +728,7 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
                 } else if (mayiuse(avx512_core_amx)) {
                     for (int isBeta0 : {no_beta0, do_beta0}) {
                         kernel[isBeta0][do_alpha1][no_sum][no_sum].reset(
-                                new jit_avx512_core_amx_gemm_kern(
+                                new jit_avx512_core_amx_gemm_kern_t(
                                         is_a_s8, is_b_s8, is_c_s32, isBeta0));
                     }
 #endif
@@ -734,7 +737,7 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
                     for (int isBeta0 : {no_beta0, do_beta0})
                         for (int isAlpha1 : {no_alpha1, do_alpha1}) {
                             kernel[isBeta0][isAlpha1][no_sum][no_sum].reset(
-                                    new jit_avx512_core_gemm_bf16bf16f32_kern(
+                                    new jit_avx512_core_gemm_bf16bf16f32_kern_t(
                                             isBeta0, isAlpha1, !use_bf16_ymm));
                         }
 #endif
@@ -748,22 +751,22 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
                 } else if (mayiuse(avx2)) {
                     for (int isBeta0 : {no_beta0, do_beta0}) {
                         kernel[isBeta0][do_alpha1][no_sum][no_sum].reset(
-                                new jit_avx2_kernel_sgemm_kern(isBeta0));
+                                new jit_avx2_kernel_sgemm_kern_t(isBeta0));
                     }
 #endif
 #if __BUILD_GEMM_AVX2
                 } else if (mayiuse(avx)) {
                     kernel[no_beta0][do_alpha1][no_sum][no_sum].reset(
-                            new jit_avx_kernel_sgemm_kern());
+                            new jit_avx_kernel_sgemm_kern_t());
                     kernel[do_beta0][do_alpha1][no_sum][no_sum].reset(
-                            new jit_avx_kernel_b0_sgemm_kern());
+                            new jit_avx_kernel_b0_sgemm_kern_t());
 #endif
 #if __BUILD_GEMM_SSE41
                 } else if (mayiuse(sse41)) {
                     kernel[no_beta0][do_alpha1][no_sum][no_sum].reset(
-                            new jit_sse41_kernel_sgemm_kern());
+                            new jit_sse41_kernel_sgemm_kern_t());
                     kernel[do_beta0][do_alpha1][no_sum][no_sum].reset(
-                            new jit_sse41_kernel_b0_sgemm_kern());
+                            new jit_sse41_kernel_b0_sgemm_kern_t());
 #endif
                 }
                 break;
@@ -771,22 +774,25 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
             default: break;
         }
 
-        static maybe_unique_ptr<jit_generator> gemv_kernel[2] = {nullptr};
-        static maybe_unique_ptr<jit_generator> gemv_s8s8s32_kernel = nullptr;
-        static maybe_unique_ptr<jit_generator> gemv_s8u8s32_kernel = nullptr;
-        static maybe_unique_ptr<jit_generator> gemv_u8s8s32_kernel = nullptr;
-        switch (data_traits<a_t>::data_type) {
+        static maybe_unique_ptr<jit_generator_t> gemv_kernel[2] = {nullptr};
+        static maybe_unique_ptr<jit_generator_t> gemv_s8s8s32_kernel = nullptr;
+        static maybe_unique_ptr<jit_generator_t> gemv_s8u8s32_kernel = nullptr;
+        static maybe_unique_ptr<jit_generator_t> gemv_u8s8s32_kernel = nullptr;
+        switch (data_traits_t<a_t>::data_type) {
             case data_type::s8:
                 if (false) {
                     // dummy if
 #if __BUILD_GEMM_AVX512
                 } else if (mayiuse(avx512_core)) {
                     gemv_s8s8s32_kernel.reset(
-                            new jit_avx512_core_gemv_s8x8s32_kern(ver_t::s8s8));
+                            new jit_avx512_core_gemv_s8x8s32_kern_t(
+                                    ver_t::s8s8));
                     gemv_s8u8s32_kernel.reset(
-                            new jit_avx512_core_gemv_s8x8s32_kern(ver_t::s8u8));
+                            new jit_avx512_core_gemv_s8x8s32_kern_t(
+                                    ver_t::s8u8));
                     gemv_u8s8s32_kernel.reset(
-                            new jit_avx512_core_gemv_s8x8s32_kern(ver_t::u8s8));
+                            new jit_avx512_core_gemv_s8x8s32_kern_t(
+                                    ver_t::u8s8));
 #endif
                 }
                 break;
@@ -798,7 +804,7 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
                 } else if (mayiuse(avx512_core)) {
                     for (int isTrans : {no_trans, do_trans})
                         gemv_kernel[isTrans].reset(
-                                new jit_avx512_core_gemv_bf16bf16f32_kern(
+                                new jit_avx512_core_gemv_bf16bf16f32_kern_t(
                                         isTrans));
 #endif
                 }
@@ -810,15 +816,16 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
 #if __BUILD_GEMM_AVX2
                 } else if (mayiuse(avx)) {
                     gemv_kernel[no_trans].reset(
-                            new jit_sse41_gemv_n_f32_kern());
-                    gemv_kernel[do_trans].reset(new jit_avx_gemv_t_f32_kern());
+                            new jit_sse41_gemv_n_f32_kern_t());
+                    gemv_kernel[do_trans].reset(
+                            new jit_avx_gemv_t_f32_kern_t());
 #endif
 #if __BUILD_GEMM_SSE41
                 } else if (mayiuse(sse41)) {
                     gemv_kernel[no_trans].reset(
-                            new jit_sse41_gemv_n_f32_kern());
+                            new jit_sse41_gemv_n_f32_kern_t());
                     gemv_kernel[do_trans].reset(
-                            new jit_sse41_gemv_t_f32_kern());
+                            new jit_sse41_gemv_t_f32_kern_t());
 #endif
                 }
                 break;
@@ -882,7 +889,7 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
         }
 
         // Set gemv floating point kernels
-        if (utils::one_of(data_traits<a_t>::data_type, data_type::f32,
+        if (utils::one_of(data_traits_t<a_t>::data_type, data_type::f32,
                     data_type::bf16)) {
             for (int isTrans : {no_trans, do_trans}) {
                 auto *p_gemv_kernel = gemv_kernel[isTrans].get();
@@ -895,7 +902,7 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
         }
 
         // Set gemv integer gemm kernels
-        if (data_traits<a_t>::data_type == data_type::s8) {
+        if (data_traits_t<a_t>::data_type == data_type::s8) {
             if (gemv_s8s8s32_kernel != nullptr) {
                 auto *kern = gemv_s8s8s32_kernel.get();
                 st = kern->create_kernel();
@@ -927,7 +934,7 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
     int copy_trans_a = (this->transa == do_trans) ? do_trans : no_trans;
     int copy_trans_b = (this->transb == do_trans) ? do_trans : no_trans;
 
-    constexpr bool is_bf16 = data_traits<a_t>::data_type == data_type::bf16;
+    constexpr bool is_bf16 = data_traits_t<a_t>::data_type == data_type::bf16;
     bool doAlpha1 = this->alpha != 1.0f && is_bf16 ? no_alpha1 : do_alpha1;
 
     {
@@ -950,7 +957,7 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
     this->gemv_s8s8s32_kernel = nullptr;
     this->gemv_s8u8s32_kernel = nullptr;
     this->gemv_u8s8s32_kernel = nullptr;
-    if (data_traits<a_t>::data_type == data_type::s8) {
+    if (data_traits_t<a_t>::data_type == data_type::s8) {
         this->gemv_s8s8s32_kernel = gemv_s8s8s32_kern;
         this->gemv_s8u8s32_kernel = gemv_s8u8s32_kern;
         this->gemv_u8s8s32_kernel = gemv_u8s8s32_kern;
@@ -965,7 +972,7 @@ void gemm_info_t<a_t, b_t, c_t>::jit_init(void) {
 template <typename a_t, typename b_t, typename c_t>
 bool gemm_info_t<a_t, b_t, c_t>::hasKernels(void) {
 
-    switch (data_traits<a_t>::data_type) {
+    switch (data_traits_t<a_t>::data_type) {
         case data_type::s8:
             if (mayiuse(sse41)) {
                 for (int isBeta0 : {no_beta0, do_beta0})
