@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -23,10 +23,10 @@ namespace impl {
 namespace cpu {
 namespace x64 {
 
-jit_avx512_core_u8_copy_sum_at_kern::jit_avx512_core_u8_copy_sum_at_kern()
-    : jit_generator(nullptr, U8_COPY_KERNEL_CODE_SIZE) {}
+jit_avx512_core_u8_copy_sum_at_kern_t::jit_avx512_core_u8_copy_sum_at_kern_t()
+    : jit_generator_t(jit_name()) {}
 
-void jit_avx512_core_u8_copy_sum_at_kern::generate() {
+void jit_avx512_core_u8_copy_sum_at_kern_t::generate() {
 
 #ifndef _WIN32
 #define M rdi
@@ -65,62 +65,7 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
 
     inLocalLabel();
     {
-
-        Xbyak::Label l1750;
-        Xbyak::Label l1b6c;
-        Xbyak::Label l1e14;
-        Xbyak::Label l20;
-        Xbyak::Label l2068;
-        Xbyak::Label l226c;
-        Xbyak::Label l22b8;
-        Xbyak::Label l22c4;
-        Xbyak::Label l22f4;
-        Xbyak::Label l26b4;
-        Xbyak::Label l28cc;
-        Xbyak::Label l2a2c;
-        Xbyak::Label l2b5c;
-        Xbyak::Label l2c64;
-        Xbyak::Label l2c94;
-        Xbyak::Label l2ca0;
-        Xbyak::Label l2cc8;
-        Xbyak::Label l2eac;
-        Xbyak::Label l2fc0;
-        Xbyak::Label l3078;
-        Xbyak::Label l3118;
-        Xbyak::Label l319c;
-        Xbyak::Label l31c0;
-        Xbyak::Label l31cc;
-        Xbyak::Label l31ec;
-        Xbyak::Label l32e4;
-        Xbyak::Label l3378;
-        Xbyak::Label l33dc;
-        Xbyak::Label l3434;
-        Xbyak::Label l347c;
-        Xbyak::Label l349c;
-        Xbyak::Label l34a8;
-        Xbyak::Label l34c8;
-        Xbyak::Label l3558;
-        Xbyak::Label l35b0;
-        Xbyak::Label l35f4;
-        Xbyak::Label l3638;
-        Xbyak::Label l366c;
-        Xbyak::Label l368a;
-        Xbyak::Label l3694;
-        Xbyak::Label l36a8;
-        Xbyak::Label l36ec;
-        Xbyak::Label l3728;
-        Xbyak::Label l3760;
-        Xbyak::Label l3794;
-        Xbyak::Label l37b8;
-        Xbyak::Label l37d8;
-        Xbyak::Label l5cc;
-        Xbyak::Label l6c;
-        Xbyak::Label l968;
-        Xbyak::Label lc80;
-        Xbyak::Label lf1c;
-        Xbyak::Label lf64;
-        Xbyak::Label lf70;
-        Xbyak::Label lfb4;
+        std::vector<Xbyak::Label> labels(55);
 
         preamble();
         auto stacksize = get_size_of_abi_save_regs();
@@ -150,10 +95,10 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -128);
         lea(LDA3, ptr[LDA + LDA * 2]);
         cmp(N, 0x30);
-        jl(lf64, T_NEAR);
+        jl(labels[52], T_NEAR);
         align(4);
 
-        L(l20);
+        L(labels[4]);
         mov(A1, A);
         mov(I, LDA);
         shl(I, 0x5);
@@ -168,10 +113,10 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         vxorps(ymm13, ymm13, ymm13);
         mov(I, M);
         sar(I, 0x3);
-        jle(l5cc, T_NEAR);
+        jle(labels[47], T_NEAR);
         align(4);
 
-        L(l6c);
+        L(labels[48]);
         vmovq(xmm0, qword[A1 - 0x80]);
         vmovq(xmm1, qword[A1 + LDA * 1 - 0x80]);
         vmovq(xmm2, qword[A1 + LDA * 2 - 0x80]);
@@ -391,12 +336,12 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(A1, -8);
         sub(B, -384);
         dec(I);
-        jg(l6c, T_NEAR);
+        jg(labels[48], T_NEAR);
         align(4);
 
-        L(l5cc);
+        L(labels[47]);
         test(M, 0x4);
-        jle(l968, T_NEAR);
+        jle(labels[49], T_NEAR);
         vmovd(xmm0, dword[A1 - 0x80]);
         vmovd(xmm1, dword[A1 + LDA * 1 - 0x80]);
         vmovd(xmm2, dword[A1 + LDA * 2 - 0x80]);
@@ -575,9 +520,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -192);
         align(4);
 
-        L(l968);
+        L(labels[49]);
         test(M, 0x2);
-        jle(lc80, T_NEAR);
+        jle(labels[50], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         vpinsrw(xmm0, xmm0, eax, 0x0);
         mov(ax, word[A1 + LDA * 1 - 0x80]);
@@ -732,9 +677,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -96);
         align(4);
 
-        L(lc80);
+        L(labels[50]);
         test(M, 0x1);
-        jle(lf1c, T_NEAR);
+        jle(labels[51], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         vpinsrb(xmm0, xmm0, eax, 0x0);
         mov(al, byte[A1 + LDA * 1 - 0x80]);
@@ -864,7 +809,7 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -48);
         align(4);
 
-        L(lf1c);
+        L(labels[51]);
         mov(A1, qword[ARG_BIAS]);
         vmovdqu(yword[A1], ymm8);
         vmovdqu(yword[A1 + 0x20], ymm9);
@@ -875,16 +820,16 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         add(qword[ARG_BIAS], 0xc0);
         sub(N, 0x30);
         cmp(N, 0x30);
-        jge(l20, T_NEAR);
+        jge(labels[4], T_NEAR);
         vzeroupper();
         align(4);
 
-        L(lf64);
+        L(labels[52]);
         cmp(N, 0x20);
-        jl(l22b8, T_NEAR);
+        jl(labels[6], T_NEAR);
         align(4);
 
-        L(lf70);
+        L(labels[53]);
         mov(A1, A);
         mov(I, LDA);
         shl(I, 0x5);
@@ -899,10 +844,10 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         pxor(xmm15, xmm15);
         mov(I, M);
         sar(I, 0x4);
-        jle(l1750, T_NEAR);
+        jle(labels[0], T_NEAR);
         align(4);
 
-        L(lfb4);
+        L(labels[54]);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         movdqu(xmm2, xword[A1 + LDA * 2 - 0x80]);
@@ -1298,12 +1243,12 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(A1, -16);
         sub(B, -512);
         dec(I);
-        jg(lfb4, T_NEAR);
+        jg(labels[54], T_NEAR);
         align(4);
 
-        L(l1750);
+        L(labels[0]);
         test(M, 0x8);
-        jle(l1b6c, T_NEAR);
+        jle(labels[1], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         movq(xmm1, qword[A1 + LDA * 1 - 0x80]);
         movq(xmm2, qword[A1 + LDA * 2 - 0x80]);
@@ -1515,9 +1460,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -256);
         align(4);
 
-        L(l1b6c);
+        L(labels[1]);
         test(M, 0x4);
-        jle(l1e14, T_NEAR);
+        jle(labels[2], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         movd(xmm1, dword[A1 + LDA * 1 - 0x80]);
         movd(xmm2, dword[A1 + LDA * 2 - 0x80]);
@@ -1650,9 +1595,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l1e14);
+        L(labels[2]);
         test(M, 0x2);
-        jle(l2068, T_NEAR);
+        jle(labels[3], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         pinsrw(xmm0, eax, 0x0);
         mov(ax, word[A1 + LDA * 1 - 0x80]);
@@ -1769,9 +1714,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l2068);
+        L(labels[3]);
         test(M, 0x1);
-        jle(l226c, T_NEAR);
+        jle(labels[5], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         pinsrb(xmm0, eax, 0x0);
         mov(al, byte[A1 + LDA * 1 - 0x80]);
@@ -1871,7 +1816,7 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l226c);
+        L(labels[5]);
         mov(A1, qword[ARG_BIAS]);
         movdqu(xword[A1], xmm8);
         movdqu(xword[A1 + 0x10], xmm9);
@@ -1884,15 +1829,15 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         add(qword[ARG_BIAS], 0x80);
         sub(N, 0x20);
         cmp(N, 0x20);
-        jge(lf70, T_NEAR);
+        jge(labels[53], T_NEAR);
         align(4);
 
-        L(l22b8);
+        L(labels[6]);
         cmp(N, 0x10);
-        jl(l2c94, T_NEAR);
+        jl(labels[14], T_NEAR);
         align(4);
 
-        L(l22c4);
+        L(labels[7]);
         mov(A1, A);
         mov(I, LDA);
         shl(I, 0x4);
@@ -1903,10 +1848,10 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         pxor(xmm11, xmm11);
         mov(I, M);
         sar(I, 0x4);
-        jle(l26b4, T_NEAR);
+        jle(labels[9], T_NEAR);
         align(4);
 
-        L(l22f4);
+        L(labels[8]);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         movdqu(xmm2, xword[A1 + LDA * 2 - 0x80]);
@@ -2106,12 +2051,12 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(A1, -16);
         sub(B, -256);
         dec(I);
-        jg(l22f4, T_NEAR);
+        jg(labels[8], T_NEAR);
         align(4);
 
-        L(l26b4);
+        L(labels[9]);
         test(M, 0x8);
-        jle(l28cc, T_NEAR);
+        jle(labels[10], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         movq(xmm1, qword[A1 + LDA * 1 - 0x80]);
         movq(xmm2, qword[A1 + LDA * 2 - 0x80]);
@@ -2219,9 +2164,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l28cc);
+        L(labels[10]);
         test(M, 0x4);
-        jle(l2a2c, T_NEAR);
+        jle(labels[11], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         movd(xmm1, dword[A1 + LDA * 1 - 0x80]);
         movd(xmm2, dword[A1 + LDA * 2 - 0x80]);
@@ -2290,9 +2235,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l2a2c);
+        L(labels[11]);
         test(M, 0x2);
-        jle(l2b5c, T_NEAR);
+        jle(labels[12], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         pinsrw(xmm0, eax, 0x0);
         mov(ax, word[A1 + LDA * 1 - 0x80]);
@@ -2352,9 +2297,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l2b5c);
+        L(labels[12]);
         test(M, 0x1);
-        jle(l2c64, T_NEAR);
+        jle(labels[13], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         pinsrb(xmm0, eax, 0x0);
         mov(al, byte[A1 + LDA * 1 - 0x80]);
@@ -2405,7 +2350,7 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l2c64);
+        L(labels[13]);
         mov(A1, qword[ARG_BIAS]);
         movdqu(xword[A1], xmm8);
         movdqu(xword[A1 + 0x10], xmm9);
@@ -2414,15 +2359,15 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         add(qword[ARG_BIAS], 0x40);
         sub(N, 0x10);
         cmp(N, 0x10);
-        jge(l22c4, T_NEAR);
+        jge(labels[7], T_NEAR);
         align(4);
 
-        L(l2c94);
+        L(labels[14]);
         cmp(N, 0x8);
-        jl(l31c0, T_NEAR);
+        jl(labels[22], T_NEAR);
         align(4);
 
-        L(l2ca0);
+        L(labels[15]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         lea(I, ptr[A1 + LDA * 8]);
@@ -2431,10 +2376,10 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         pxor(xmm9, xmm9);
         mov(I, M);
         sar(I, 0x4);
-        jle(l2eac, T_NEAR);
+        jle(labels[17], T_NEAR);
         align(4);
 
-        L(l2cc8);
+        L(labels[16]);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         movdqu(xmm2, xword[A1 + LDA * 2 - 0x80]);
@@ -2535,12 +2480,12 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         movdqu(xword[B - 0x10], xmm3);
         sub(B, -128);
         dec(I);
-        jg(l2cc8, T_NEAR);
+        jg(labels[16], T_NEAR);
         align(4);
 
-        L(l2eac);
+        L(labels[17]);
         test(M, 0x8);
-        jle(l2fc0, T_NEAR);
+        jle(labels[18], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         movq(xmm1, qword[A1 + LDA * 1 - 0x80]);
         movq(xmm2, qword[A1 + LDA * 2 - 0x80]);
@@ -2596,9 +2541,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l2fc0);
+        L(labels[18]);
         test(M, 0x4);
-        jle(l3078, T_NEAR);
+        jle(labels[19], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         movd(xmm1, dword[A1 + LDA * 1 - 0x80]);
         movd(xmm2, dword[A1 + LDA * 2 - 0x80]);
@@ -2634,9 +2579,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l3078);
+        L(labels[19]);
         test(M, 0x2);
-        jle(l3118, T_NEAR);
+        jle(labels[20], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         pinsrw(xmm0, eax, 0x0);
         mov(ax, word[A1 + LDA * 1 - 0x80]);
@@ -2668,9 +2613,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l3118);
+        L(labels[20]);
         test(M, 0x1);
-        jle(l319c, T_NEAR);
+        jle(labels[21], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         pinsrb(xmm0, eax, 0x0);
         mov(al, byte[A1 + LDA * 1 - 0x80]);
@@ -2696,22 +2641,22 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l319c);
+        L(labels[21]);
         mov(A1, qword[ARG_BIAS]);
         movdqu(xword[A1], xmm8);
         movdqu(xword[A1 + 0x10], xmm9);
         add(qword[ARG_BIAS], 0x20);
         sub(N, 0x8);
         cmp(N, 0x8);
-        jge(l2ca0, T_NEAR);
+        jge(labels[15], T_NEAR);
         align(4);
 
-        L(l31c0);
+        L(labels[22]);
         cmp(N, 0x4);
-        jl(l349c, T_NEAR);
+        jl(labels[30], T_NEAR);
         align(4);
 
-        L(l31cc);
+        L(labels[23]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 2]);
         lea(I, ptr[A1 + LDA * 4]);
@@ -2719,10 +2664,10 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         pxor(xmm7, xmm7);
         mov(I, M);
         sar(I, 0x4);
-        jle(l32e4, T_NEAR);
+        jle(labels[25], T_NEAR);
         align(4);
 
-        L(l31ec);
+        L(labels[24]);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         sub(A1, -16);
@@ -2775,12 +2720,12 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         movdqu(xword[B - 0x50], xmm3);
         sub(B, -64);
         dec(I);
-        jg(l31ec, T_NEAR);
+        jg(labels[24], T_NEAR);
         align(4);
 
-        L(l32e4);
+        L(labels[25]);
         test(M, 0x8);
-        jle(l3378, T_NEAR);
+        jle(labels[26], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         movq(xmm1, qword[A1 + LDA * 1 - 0x80]);
         sub(A1, -8);
@@ -2811,9 +2756,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l3378);
+        L(labels[26]);
         test(M, 0x4);
-        jle(l33dc, T_NEAR);
+        jle(labels[27], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         movd(xmm1, dword[A1 + LDA * 1 - 0x80]);
         sub(A1, -4);
@@ -2834,9 +2779,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l33dc);
+        L(labels[27]);
         test(M, 0x2);
-        jle(l3434, T_NEAR);
+        jle(labels[28], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         pinsrw(xmm0, eax, 0x0);
         mov(ax, word[A1 + LDA * 1 - 0x80]);
@@ -2855,9 +2800,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l3434);
+        L(labels[28]);
         test(M, 0x1);
-        jle(l347c, T_NEAR);
+        jle(labels[29], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         pinsrb(xmm0, eax, 0x0);
         mov(al, byte[A1 + LDA * 1 - 0x80]);
@@ -2872,21 +2817,21 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -4);
         align(4);
 
-        L(l347c);
+        L(labels[29]);
         mov(A1, qword[ARG_BIAS]);
         movdqu(xword[A1], xmm7);
         add(qword[ARG_BIAS], 0x10);
         sub(N, 0x4);
         cmp(N, 0x4);
-        jge(l31cc, T_NEAR);
+        jge(labels[23], T_NEAR);
         align(4);
 
-        L(l349c);
+        L(labels[30]);
         cmp(N, 0x2);
-        jl(l368a, T_NEAR);
+        jl(labels[38], T_NEAR);
         align(4);
 
-        L(l34a8);
+        L(labels[31]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 1]);
         lea(I, ptr[A1 + LDA * 2]);
@@ -2894,10 +2839,10 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         pxor(xmm7, xmm7);
         mov(I, M);
         sar(I, 0x4);
-        jle(l3558, T_NEAR);
+        jle(labels[33], T_NEAR);
         align(4);
 
-        L(l34c8);
+        L(labels[32]);
         movdqu(xmm0, xword[A1 - 0x80]);
         sub(A1, -16);
         movdqu(xmm1, xword[A2 - 0x80]);
@@ -2927,12 +2872,12 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         movdqu(xword[B - 0x70], xmm2);
         sub(B, -32);
         dec(I);
-        jg(l34c8, T_NEAR);
+        jg(labels[32], T_NEAR);
         align(4);
 
-        L(l3558);
+        L(labels[33]);
         test(M, 0x8);
-        jle(l35b0, T_NEAR);
+        jle(labels[34], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         sub(A1, -8);
         movq(xmm1, qword[A2 - 0x80]);
@@ -2951,9 +2896,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l35b0);
+        L(labels[34]);
         test(M, 0x4);
-        jle(l35f4, T_NEAR);
+        jle(labels[35], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         sub(A1, -4);
         movd(xmm1, dword[A2 - 0x80]);
@@ -2968,9 +2913,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l35f4);
+        L(labels[35]);
         test(M, 0x2);
-        jle(l3638, T_NEAR);
+        jle(labels[36], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         sub(A1, -2);
         pinsrw(xmm0, eax, 0x0);
@@ -2985,9 +2930,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -4);
         align(4);
 
-        L(l3638);
+        L(labels[36]);
         test(M, 0x1);
-        jle(l366c, T_NEAR);
+        jle(labels[37], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         pinsrb(xmm0, eax, 0x0);
         mov(byte[B - 0x80], al);
@@ -2999,30 +2944,30 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         paddd(xmm7, xmm5);
         align(4);
 
-        L(l366c);
+        L(labels[37]);
         mov(A1, qword[ARG_BIAS]);
         movq(qword[A1], xmm7);
         add(qword[ARG_BIAS], 0x8);
         sub(N, 0x2);
         cmp(N, 0x2);
-        jge(l34a8, T_NEAR);
+        jge(labels[31], T_NEAR);
         align(4);
 
-        L(l368a);
+        L(labels[38]);
         cmp(N, 0x1);
-        jl(l37d8, T_NEAR);
+        jl(labels[46], T_NEAR);
         align(4);
 
-        L(l3694);
+        L(labels[39]);
         mov(A1, A);
         add(A, LDA);
         pxor(xmm7, xmm7);
         mov(I, M);
         sar(I, 0x4);
-        jle(l36ec, T_NEAR);
+        jle(labels[41], T_NEAR);
         align(4);
 
-        L(l36a8);
+        L(labels[40]);
         movdqu(xmm0, xword[A1 - 0x80]);
         sub(A1, -16);
         pmovsxbw(xmm5, xmm0);
@@ -3037,12 +2982,12 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         movdqu(xword[B - 0x80], xmm0);
         sub(B, -16);
         dec(I);
-        jg(l36a8, T_NEAR);
+        jg(labels[40], T_NEAR);
         align(4);
 
-        L(l36ec);
+        L(labels[41]);
         test(M, 0x8);
-        jle(l3728, T_NEAR);
+        jle(labels[42], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         sub(A1, -8);
         pmovsxbw(xmm5, xmm0);
@@ -3055,9 +3000,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l3728);
+        L(labels[42]);
         test(M, 0x4);
-        jle(l3760, T_NEAR);
+        jle(labels[43], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         sub(A1, -4);
         pmovsxbw(xmm5, xmm0);
@@ -3069,9 +3014,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -4);
         align(4);
 
-        L(l3760);
+        L(labels[43]);
         test(M, 0x2);
-        jle(l3794, T_NEAR);
+        jle(labels[44], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         pinsrw(xmm0, eax, 0x0);
         pmovsxbw(xmm5, xmm0);
@@ -3083,9 +3028,9 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -2);
         align(4);
 
-        L(l3794);
+        L(labels[44]);
         test(M, 0x1);
-        jle(l37b8, T_NEAR);
+        jle(labels[45], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         pinsrb(xmm0, eax, 0x0);
         pmovsxbd(xmm5, xmm0);
@@ -3094,16 +3039,16 @@ void jit_avx512_core_u8_copy_sum_at_kern::generate() {
         sub(B, -1);
         align(4);
 
-        L(l37b8);
+        L(labels[45]);
         mov(A1, qword[ARG_BIAS]);
         movd(dword[A1], xmm7);
         add(qword[ARG_BIAS], 0x4);
         sub(N, 0x1);
         cmp(N, 0x1);
-        jge(l3694, T_NEAR);
+        jge(labels[39], T_NEAR);
         align(4);
 
-        L(l37d8);
+        L(labels[46]);
 
         postamble();
     }
