@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -32,32 +32,28 @@ namespace injector_utils {
 
 using vmm_index_set_t = typename std::set<size_t>;
 using vmm_index_set_iterator_t = typename std::set<size_t>::iterator;
-template <typename Vmm>
-struct vmm_size_t;
 
-template <>
-struct vmm_size_t<Xbyak::Zmm> {
-    static constexpr std::size_t bytes = 64u;
-};
+enum class layout_t { ncsp, c_blocked, nspc, cspn, unsupported };
 
-template <>
-struct vmm_size_t<Xbyak::Ymm> {
-    static constexpr std::size_t bytes = 32u;
-};
-
-template <>
-struct vmm_size_t<Xbyak::Xmm> {
-    static constexpr std::size_t bytes = 16u;
-};
+inline layout_t get_layout_type(const memory_desc_wrapper &dst_d) {
+    const auto strides = dst_d.blocking_desc().strides;
+    if (!dst_d.is_plain()) return layout_t::c_blocked;
+    if (strides[0] >= strides[1]
+            && IMPLICATION(dst_d.ndims() >= 3, strides[1] >= strides[2]))
+        return layout_t::ncsp;
+    if (strides[1] == 1) return layout_t::nspc;
+    if (strides[0] == 1) return layout_t::cspn;
+    return layout_t::unsupported;
+}
 
 /*
- * Scope guard for general purpouse register and vector registers preservation.
+ * Scope guard for general purpose register and vector registers preservation.
  * Pushes registers to stack during construction and pops during destruction.
  */
 class register_preserve_guard_t {
 
 public:
-    register_preserve_guard_t(jit_generator *host,
+    register_preserve_guard_t(jit_generator_t *host,
             std::initializer_list<Xbyak::Reg64> reg64_to_preserve,
             std::initializer_list<Xbyak::Xmm> vmm_to_preserve = {});
     register_preserve_guard_t(register_preserve_guard_t &&other) = default;
@@ -68,7 +64,7 @@ public:
     size_t stack_space_occupied() const;
 
 private:
-    jit_generator *host_;
+    jit_generator_t *host_;
     std::stack<Xbyak::Reg64> reg64_stack_;
     std::stack<Xbyak::Xmm> vmm_stack_;
     size_t vmm_to_preserve_size_bytes_;
@@ -77,7 +73,7 @@ private:
 class conditional_register_preserve_guard_t : public register_preserve_guard_t {
 public:
     conditional_register_preserve_guard_t(bool condition_to_be_met,
-            jit_generator *host,
+            jit_generator_t *host,
             std::initializer_list<Xbyak::Reg64> reg64_to_preserve,
             std::initializer_list<Xbyak::Xmm> vmm_to_preserve = {});
     DNNL_DISALLOW_COPY_AND_ASSIGN(conditional_register_preserve_guard_t);
