@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@
 #define CPU_X64_JIT_UNI_ELTWISE_INJECTOR_HPP
 
 #include <assert.h>
+#include <type_traits>
 
 #include "common/c_types_map.hpp"
 #include "common/primitive_attr.hpp"
@@ -36,58 +37,47 @@ namespace eltwise_injector {
 struct static_params_t {
 
     static_params_t(bool save_state = true,
-            Xbyak::Reg64 p_table = Xbyak::util::rax,
+            Xbyak::Reg64 p_table = Xbyak::Reg64(Xbyak::Operand::RAX),
             Xbyak::Opmask k_mask = Xbyak::Opmask(1), bool is_fwd = true,
-            bool use_dst = false)
+            bool use_dst = false, bool preserve_vmm = true,
+            bool preserve_p_table = true)
         : save_state(save_state)
-        , p_table(p_table)
-        , k_mask(k_mask)
+        , p_table_(p_table)
+        , k_mask_(k_mask)
         , is_fwd(is_fwd)
-        , use_dst(use_dst) {}
+        , use_dst(use_dst)
+        , preserve_vmm(preserve_vmm)
+        , preserve_p_table(preserve_p_table) {}
 
     bool save_state;
-    Xbyak::Reg64 p_table;
-    Xbyak::Opmask k_mask;
+    Xbyak::Reg64 p_table_;
+    Xbyak::Opmask k_mask_;
     bool is_fwd;
     bool use_dst;
+    bool preserve_vmm;
+    bool preserve_p_table;
 };
 
 /*
- * Checks if isa is supported by binary injector.
+ * Checks if isa is supported by eltwise injector.
  */
-constexpr bool is_isa_supported(cpu_isa_t isa) {
-    return utils::one_of(isa, sse41, avx, avx2, avx512_common, avx512_core);
-}
+bool is_isa_supported(cpu_isa_t isa);
 
 /*
  * Checks if eltwise algorithm is supported by eltwise injector.
  */
-constexpr bool is_alg_supported(alg_kind_t alg) {
-    using namespace alg_kind;
-    return utils::one_of(alg, eltwise_relu, eltwise_tanh, eltwise_elu,
-            eltwise_square, eltwise_abs, eltwise_sqrt, eltwise_linear,
-            eltwise_bounded_relu, eltwise_soft_relu, eltwise_logistic,
-            eltwise_logsigmoid, eltwise_mish, eltwise_exp, eltwise_gelu_tanh,
-            eltwise_hardswish, eltwise_swish, eltwise_log, eltwise_clip,
-            eltwise_clip_v2, eltwise_pow, eltwise_gelu_erf, eltwise_round,
-            eltwise_relu_use_dst_for_bwd, eltwise_tanh_use_dst_for_bwd,
-            eltwise_elu_use_dst_for_bwd, eltwise_sqrt_use_dst_for_bwd,
-            eltwise_logistic_use_dst_for_bwd, eltwise_exp_use_dst_for_bwd,
-            eltwise_clip_v2_use_dst_for_bwd);
-}
+bool is_alg_supported(alg_kind_t alg);
 
 /*
  * Checks if eltwise injection for given args is supported.
  */
-constexpr bool is_supported(cpu_isa_t isa, alg_kind_t alg) {
-    return is_isa_supported(isa) && is_alg_supported(alg);
-}
+bool is_supported(cpu_isa_t isa, alg_kind_t alg, data_type_t dt);
 
 } // namespace eltwise_injector
 
-template <cpu_isa_t isa>
-struct jit_uni_eltwise_injector_f32 {
-    using Vmm = typename cpu_isa_traits<isa>::Vmm;
+template <cpu_isa_t isa, typename Wmm = typename cpu_isa_traits_t<isa>::Vmm>
+struct jit_uni_eltwise_injector_t {
+    using Vmm = Wmm;
 
     // Arguments description:
     // host - jit generator which is filled with instructions
@@ -101,102 +91,131 @@ struct jit_uni_eltwise_injector_f32 {
     //   - algorithm derivative.
     // use_dst - defines whether source or destination point is passed to alg
     //   code. Depends on algorithm. See `_use_dst_for_bwd` algs definition.
-    jit_uni_eltwise_injector_f32(jit_generator *host, alg_kind_t alg,
-            float alpha, float beta, float scale, bool save_state = true,
-            Xbyak::Reg64 p_table = Xbyak::util::rax,
+    jit_uni_eltwise_injector_t(jit_generator_t *host, alg_kind_t alg,
+            float alpha, float beta, float scale,
+            data_type_t dt = data_type::f32, bool save_state = true,
+            Xbyak::Reg64 p_table = Xbyak::Reg64(Xbyak::Operand::RAX),
             Xbyak::Opmask k_mask = Xbyak::Opmask(1), bool is_fwd = true,
-            bool use_dst = false)
+            bool use_dst = false, bool preserve_vmm = true,
+            bool preserve_p_table = true)
         : alg_(alg)
         , alpha_(alpha)
         , beta_(beta)
         , scale_(scale)
+        , dt_(dt)
         , h(host)
         , save_state_(save_state)
-        , p_table(p_table)
-        , k_mask(k_mask)
+        , p_table_(p_table)
+        , k_mask_(k_mask)
         , is_fwd_(is_fwd)
-        , use_dst_(use_dst) {
-        assert(eltwise_injector::is_supported(isa, alg_));
+        , use_dst_(use_dst)
+        , preserve_vmm_(preserve_vmm)
+        , preserve_p_table_(preserve_p_table)
+        , n_vregs_to_preserve_(aux_vecs_count(alg_, is_fwd_, alpha_)) {
+        assert(eltwise_injector::is_supported(isa, alg_, dt_));
 
         register_table_entries();
     }
 
-    jit_uni_eltwise_injector_f32(jit_generator *host,
+    jit_uni_eltwise_injector_t(jit_generator_t *host,
             const post_ops_t::entry_t::eltwise_t &eltwise,
-            bool save_state = true, Xbyak::Reg64 p_table = Xbyak::util::rax,
+            data_type_t dt = data_type::f32, bool save_state = true,
+            Xbyak::Reg64 p_table = Xbyak::Reg64(Xbyak::Operand::RAX),
             Xbyak::Opmask k_mask = Xbyak::Opmask(1), bool is_fwd = true,
-            bool use_dst = false)
-        : jit_uni_eltwise_injector_f32(host, eltwise.alg, eltwise.alpha,
-                eltwise.beta, eltwise.scale, save_state, p_table, k_mask,
-                is_fwd, use_dst) {}
+            bool use_dst = false, bool preserve_vmm = true,
+            bool preserve_p_table = true)
+        : jit_uni_eltwise_injector_t(host, eltwise.alg, eltwise.alpha,
+                eltwise.beta, eltwise.scale, dt, save_state, p_table, k_mask,
+                is_fwd, use_dst, preserve_vmm, preserve_p_table) {}
 
-    void compute_vector_range(size_t start_idx, size_t end_idx);
-    void compute_vector_range(const injector_utils::vmm_index_set_t &vmm_idxs);
-    void compute_vector(size_t idx) { compute_vector_range({idx}); }
+    void compute_vector_range(size_t start_compute_idx, size_t end_compute_idx,
+            const injector_utils::vmm_index_set_t &vmm_aux_indices = {});
+    void compute_vector_range(
+            const injector_utils::vmm_index_set_t &vmm_compute_idxs,
+            const injector_utils::vmm_index_set_t &vmm_aux_indices = {});
+    void compute_vector(size_t compute_idx,
+            const injector_utils::vmm_index_set_t &vmm_aux_indices = {}) {
+        compute_vector_range({compute_idx}, vmm_aux_indices);
+    }
     void prepare_table(bool gen_table = true);
-    void load_table_addr() { h->mov(p_table, l_table); }
+    void load_table_addr() { h->lea(p_table_, h->ptr[h->rip + l_table_]); }
+
+    // This call is `static` and `public` to make a decision on the injector's
+    // saving state if the caller can supply the necessary number of vmms. The
+    // decision must be made BEFORE constructing the injector, thus, `static`.
+    static size_t aux_vecs_count(alg_kind_t alg, bool is_fwd, float alpha);
 
 private:
     const alg_kind_t alg_;
     const float alpha_;
     const float beta_;
     const float scale_;
+    const data_type_t dt_;
 
-    jit_generator *const h;
+    jit_generator_t *const h;
 
     const bool save_state_;
-    const Xbyak::Reg64 p_table;
-    const Xbyak::Opmask k_mask;
+    const Xbyak::Reg64 p_table_;
+    Xbyak::Reg64 reg_vmm_stack_ptr_;
+    const Xbyak::Opmask k_mask_;
     const bool is_fwd_;
     const bool use_dst_;
+    const bool preserve_vmm_;
+    const bool preserve_p_table_;
 
-    Xbyak::Label l_table;
+    Xbyak::Label l_table_;
 
-    // if only the injector was inherited from jit_generator...
+    // if only the injector was inherited from jit_generator_t...
     enum {
-        _cmp_eq_oq = jit_generator::_cmp_eq_oq,
-        _cmp_lt_os = jit_generator::_cmp_lt_os,
-        _cmp_le_os = jit_generator::_cmp_le_os,
-        _cmp_ge_os = jit_generator::_cmp_nlt_us,
-        _cmp_gt_os = jit_generator::_cmp_nle_us,
-        _op_floor = jit_generator::_op_floor,
-        _op_mxcsr = jit_generator::_op_mxcsr
+        _cmp_eq_oq = jit_generator_t::_cmp_eq_oq,
+        _cmp_neq_uq = jit_generator_t::_cmp_neq_uq,
+        _cmp_lt_os = jit_generator_t::_cmp_lt_os,
+        _cmp_le_os = jit_generator_t::_cmp_le_os,
+        _cmp_ge_os = jit_generator_t::_cmp_nlt_us,
+        _cmp_gt_os = jit_generator_t::_cmp_nle_us,
+        _op_floor = jit_generator_t::_op_floor,
+        _op_mxcsr = jit_generator_t::_op_mxcsr
     };
 
-    static constexpr bool has_avx512() {
-        return utils::one_of(isa, avx512_common, avx512_core);
-    }
+    const bool is_avx512_ = is_superset(isa, avx512_core);
 
-    static constexpr size_t vlen = cpu_isa_traits<isa>::vlen;
-    static constexpr size_t preserved_vecs_max = 6;
-    static constexpr size_t preserved_gprs_max = 5;
-    static constexpr size_t vecs_count = has_avx512() ? 32 : 16;
-    static constexpr int n_mantissa_bits = 23;
-    static constexpr int k_mask_size = 8;
+    static constexpr size_t vlen_ = vreg_traits_t<Vmm>::vlen;
+    static constexpr size_t preserved_vecs_max_ = 6;
+    static constexpr size_t preserved_gprs_max_ = 5;
+    static constexpr size_t n_vregs_ = cpu_isa_traits_t<isa>::n_vregs;
+    static constexpr int n_mantissa_bits_ = 23;
 
-    bool preserve_vec_for_avx = false;
+    const size_t n_vregs_to_preserve_;
+    size_t n_vregs_preserved_ = 0;
+    bool need_vmm_mask_register_ = false;
+    // Default initialization will put zeros. Putting any value to trigger a
+    // potential error to Xbyak is not working as Xbyak cycles vmm indices
+    // over 32 value.
+    size_t preserved_vmm_indices_[preserved_vecs_max_] = {};
+    size_t preserved_vmm_tail_indices_[preserved_vecs_max_] = {};
+    size_t preserved_gpr_indices_[preserved_gprs_max_] = {};
 
-    size_t vecs_to_preserve = 0;
-    size_t preserved_vecs_count = 0;
-    size_t preserved_vec_idxs[preserved_vecs_max] = {0};
-    size_t preserved_gpr_idxs[preserved_gprs_max] = {0};
-    injector_utils::vmm_index_set_iterator_t start_idx_tail;
+    Vmm vmm_mask_;
+    Vmm vmm_tmp_;
+    Xbyak::Ymm ymm_tmp_;
+    Xbyak::Xmm xmm_tmp_;
 
-    Vmm vmm_mask, vmm_aux0, vmm_aux1, vmm_aux2, vmm_aux3, vmm_aux4, vmm_tmp;
-    Xbyak::Ymm ymm_tmp;
-    Xbyak::Xmm xmm_tmp;
-
-    size_t aux_vecs_count();
-    size_t aux_gprs_count();
+    static bool need_mask_register(alg_kind_t alg, bool is_fwd, float alpha);
+    static size_t aux_gprs_count(alg_kind_t alg, bool is_fwd, float alpha);
+    static bool need_vmm_stack_ptr(alg_kind_t alg, bool is_fwd, float alpha);
+    static size_t op_vecs_count(alg_kind_t alg, bool is_fwd);
+    size_t get_stack_vmm_space();
 
     void compute_body(
             const injector_utils::vmm_index_set_iterator_t &start_idx_it,
             const injector_utils::vmm_index_set_iterator_t &end_idx_it);
-    void injector_preamble(const injector_utils::vmm_index_set_t &vmm_idxs);
-    void injector_preamble_tail(
-            const injector_utils::vmm_index_set_iterator_t start_idx_it);
+    void injector_preamble(const injector_utils::vmm_index_set_t &vmm_idxs,
+            injector_utils::vmm_index_set_iterator_t &start_idx_tail_it,
+            const injector_utils::vmm_index_set_t &vmm_aux_indices);
+    void injector_preamble_tail(size_t n_vregs_not_preserved);
     void injector_postamble();
     void assign_regs();
+    Wmm vmm_aux(size_t idx);
     void vec_shift(const Vmm &vmm_dst, const Vmm &vmm_src, bool shift_left,
             const int imm);
     void compute_cmp_mask(const Vmm &vmm_src,
@@ -213,9 +232,7 @@ private:
     void abs_compute_vector_fwd(const Vmm &vmm_src);
     void sqrt_compute_vector_fwd(const Vmm &vmm_src);
     void linear_compute_vector_fwd(const Vmm &vmm_src);
-    void bounded_relu_compute_vector_fwd(const Vmm &vmm_src);
     void soft_relu_compute_vector_fwd(const Vmm &vmm_src);
-    void logsigmoid_compute_vector_fwd(const Vmm &vmm_src);
     void mish_compute_vector_fwd(const Vmm &vmm_src);
     void logistic_compute_vector_fwd(const Vmm &vmm_src);
     void gelu_tanh_compute_vector_fwd(const Vmm &vmm_src);
@@ -224,8 +241,10 @@ private:
     void clip_compute_vector_fwd(const Vmm &vmm_src);
     void pow_compute_vector_fwd(const Vmm &vmm_src);
     void gelu_erf_compute_vector_fwd(const Vmm &vmm_src);
+    void gelu_erf_minimax_approx_compute_vector_fwd(const Vmm &vmm_src);
     void round_compute_vector_fwd(const Vmm &vmm_src);
     void hardswish_compute_vector_fwd(const Vmm &vmm_src);
+    void hardsigmoid_compute_vector_fwd(const Vmm &vmm_src);
 
     void exp_compute_vector_bwd(const Vmm &vmm_src);
     void relu_compute_vector_bwd(const Vmm &vmm_src);
@@ -235,10 +254,8 @@ private:
     void abs_compute_vector_bwd(const Vmm &vmm_src);
     void sqrt_compute_vector_bwd(const Vmm &vmm_src);
     void linear_compute_vector_bwd(const Vmm &vmm_src);
-    void bounded_relu_compute_vector_bwd(const Vmm &vmm_src);
     void soft_relu_compute_vector_bwd(const Vmm &vmm_src);
     void logistic_compute_vector_bwd(const Vmm &vmm_src);
-    void logsigmoid_compute_vector_bwd(const Vmm &vmm_src);
     void mish_compute_vector_bwd(const Vmm &vmm_src);
     void gelu_tanh_compute_vector_bwd(const Vmm &vmm_src);
     void swish_compute_vector_bwd(const Vmm &vmm_src);
@@ -247,6 +264,7 @@ private:
     void pow_compute_vector_bwd(const Vmm &vmm_src);
     void gelu_erf_compute_vector_bwd(const Vmm &vmm_src);
     void hardswish_compute_vector_bwd(const Vmm &vmm_src);
+    void hardsigmoid_compute_vector_bwd(const Vmm &vmm_src);
 
     enum key_t {
         scale = 0, // scale argument
@@ -284,10 +302,20 @@ private:
         gelu_tanh_fitting_const, // 0.044715f
         gelu_tanh_fitting_const_times_three, // 0.134145f
         gelu_tanh_sqrt_two_over_pi, // sqrtf(2.f/pi) = 0.797884f
-        gelu_erf_approx_const, // 0.3275911f - implementation based for approx
-        gelu_erf_one_over_sqrt_two, // 1.f / sqrtf(2.f)
-        gelu_erf_one_over_sqrt_pi, // 1.f / sqrtf(pi) = 0.564190f
-        gelu_erf_pol, // see correspondent table for float values
+        // 0.3275911f - implementation based for approx
+        gelu_erf_Abramowitz_Stegun_approx_const,
+        gelu_erf_Abramowitz_Stegun_one_over_sqrt_two, // 1.f / sqrtf(2.f)
+        // 1.f / sqrtf(pi) = 0.564190f
+        gelu_erf_Abramowitz_Stegun_one_over_sqrt_pi,
+        // see correspondent table for float values
+        gelu_erf_Abramowitz_Stegun_pol,
+        gelu_erf_minimax_pol, // see correspondent table for float values
+        gelu_erf_idx_bias, // bias applied to compute table index
+        gelu_erf_rbound, // upper bound at which we clamp erf at 1
+        gelu_erf_one, // just the integer value 1, used for index clamping
+        gelu_erf_twenty_three, // just the integer value 23, used for index clamping
+        gelu_erf_twenty_four, // just the integer value 24, used for index clamping
+        log_inf, // inf
         log_minus_inf, // -inf
         log_qnan, // qnan
         log_mantissa_mask, // gets mantissa bits
@@ -304,19 +332,22 @@ private:
         // share their broadcast property
         // TODO: enforce through data structure
         const auto it = entry_map_.find(key); // search an entry for a key
-        assert(it != entry_map_.end());
+        if (it == entry_map_.end()) {
+            assert(!"Non-existent key");
+            return 0;
+        }
         const auto &te = (*it).second;
-        const auto scale = te.bcast ? vlen : sizeof(table_entry_val_t);
+        const auto scale = te.bcast ? vlen_ : sizeof(table_entry_val_t);
         return te.off + key_off_val_shift * scale;
     }
     Xbyak::Address table_val(key_t key, size_t key_off_val_shift = 0) {
         auto off = table_off(key, key_off_val_shift);
-        return h->ptr[p_table + off];
+        return h->ptr[p_table_ + off];
     }
 
     // we accept only 32bit hexadecimal table values to avoid any rounding
     using table_entry_val_t = uint32_t;
-    using table_entry_offset_t = size_t; // offsets are in bytes wrt p_table
+    using table_entry_offset_t = size_t; // offsets are in bytes wrt p_table_
     using table_entry_bcast_t = bool; // true => bcast value
 
     struct table_entry_t {
