@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
 #ifndef CPU_X64_JIT_AVX512_COMMON_CONV_KERNEL_HPP
 #define CPU_X64_JIT_AVX512_COMMON_CONV_KERNEL_HPP
 
+#include <memory>
 #include "common/c_types_map.hpp"
 #include "common/memory_tracking.hpp"
 
@@ -30,7 +31,7 @@ namespace cpu {
 namespace x64 {
 
 template <typename Vmm>
-struct _jit_avx512_common_conv_fwd_kernel : public jit_generator {
+struct _jit_avx512_common_conv_fwd_kernel : public jit_generator_t {
 
     _jit_avx512_common_conv_fwd_kernel(const jit_conv_conf_t &ajcp,
             const primitive_attr_t &attr, const memory_desc_t &dst_md);
@@ -42,11 +43,10 @@ struct _jit_avx512_common_conv_fwd_kernel : public jit_generator {
 
 private:
     constexpr static int isa_simd_width_
-            = cpu_isa_traits<avx512_common>::vlen / sizeof(float);
+            = cpu_isa_traits_t<avx512_core>::vlen / sizeof(float);
     using reg64_t = const Xbyak::Reg64;
     enum {
         typesize = sizeof(float),
-        unroll_4fma = 4,
         ker_reg_base_idx = 28,
     };
 
@@ -55,24 +55,16 @@ private:
     reg64_t reg_ker = r9;
     reg64_t reg_out = r10;
 
-    reg64_t reg_inp_prf = r11;
-    reg64_t reg_ker_prf = r12;
-    reg64_t reg_out_prf = r13;
     reg64_t reg_owb = r12;
 
     reg64_t aux_reg_inp = r14;
     reg64_t aux_reg_ker = r15;
 
-    reg64_t aux_reg_inp_prf = rsi;
-    reg64_t aux_reg_ker_prf = rdx;
-
     reg64_t reg_channel = rsi;
     reg64_t reg_bias = rdx;
 
     reg64_t aux_reg_ker_d = r9;
     reg64_t aux_reg_inp_d = rbx;
-    reg64_t aux_reg_inp_d_prf = r13;
-    reg64_t aux_reg_ker_d_prf = abi_not_param1;
     reg64_t reg_ki = r10;
 
     reg64_t reg_kj = rax;
@@ -87,6 +79,10 @@ private:
     reg64_t reg_ker_long_offt = r11;
     reg64_t reg_tail = aux_reg_ker;
     reg64_t reg_load_work = reg_tail;
+
+    /* binary post-ops operand */
+    reg64_t temp_offset_reg = r12;
+
     Xbyak::Opmask k_oc_tail_mask = Xbyak::Opmask(2);
     const Xbyak::Opmask postops_mask = Xbyak::Opmask(3);
 
@@ -114,7 +110,7 @@ private:
     Xbyak::Reg64 imm_addr64 = r15;
     Vmm vmm_wei = Vmm(31);
 
-    std::unique_ptr<injector::jit_uni_postops_injector_t<avx512_common>>
+    std::unique_ptr<injector::jit_uni_postops_injector_t<avx512_core>>
             postops_injector_;
 
     inline void prepare_output(int ur_w);
@@ -122,8 +118,6 @@ private:
     inline void store_output(int ur_w);
     inline void compute_loop_fma(int ur_w, int pad_l, int pad_r);
     inline void compute_loop_fma_core(int ur_w, int pad_l, int pad_r);
-    inline void compute_loop_4fma(int ur_w, int pad_l, int pad_r);
-    inline void compute_loop_4fma_1st(int ur_w, int pad_l, int pad_r);
     inline void compute_loop(int ur_w, int pad_l, int pad_r);
 
     void generate() override;
@@ -187,31 +181,37 @@ struct jit_avx512_common_conv_fwd_kernel {
         : kernel_(nullptr) {
         switch (ajcp.oc_block) {
             case 16:
-                kernel_ = new _jit_avx512_common_conv_fwd_kernel<Xbyak::Zmm>(
+                kernel_ = utils::make_unique<
+                        _jit_avx512_common_conv_fwd_kernel<Xbyak::Zmm>>(
                         ajcp, attr, dst_md);
                 return;
             case 8:
-                kernel_ = new _jit_avx512_common_conv_fwd_kernel<Xbyak::Ymm>(
+                kernel_ = utils::make_unique<
+                        _jit_avx512_common_conv_fwd_kernel<Xbyak::Ymm>>(
                         ajcp, attr, dst_md);
                 return;
             case 4:
-                kernel_ = new _jit_avx512_common_conv_fwd_kernel<Xbyak::Xmm>(
+                kernel_ = utils::make_unique<
+                        _jit_avx512_common_conv_fwd_kernel<Xbyak::Xmm>>(
                         ajcp, attr, dst_md);
                 return;
             default: assert(!"invalid channel blocking");
         }
     }
 
-    status_t create_kernel() { return kernel_->create_kernel(); }
+    status_t create_kernel() {
+        if (kernel_) return kernel_->create_kernel();
+        return status::out_of_memory;
+    }
 
-    ~jit_avx512_common_conv_fwd_kernel() { delete kernel_; }
+    ~jit_avx512_common_conv_fwd_kernel() = default;
 
     enum { typesize = sizeof(float) };
 
     static status_t init_conf(jit_conv_conf_t &jcp,
             const convolution_desc_t &cd, memory_desc_t &src_pd,
             memory_desc_t &weights_pd, memory_desc_t &dst_pd,
-            memory_desc_t &bias_pd, const primitive_attr_t &attr, int nthreads);
+            memory_desc_t &bias_pd, primitive_attr_t &attr, int nthreads);
     static void init_scratchpad(memory_tracking::registrar_t &scratchpad,
             const jit_conv_conf_t &jcp);
 
@@ -221,14 +221,14 @@ struct jit_avx512_common_conv_fwd_kernel {
 
 private:
     DNNL_DISALLOW_COPY_AND_ASSIGN(jit_avx512_common_conv_fwd_kernel);
-    jit_generator *kernel_;
+    std::unique_ptr<jit_generator_t> kernel_;
 };
 
 template <typename Vmm>
-struct _jit_avx512_common_conv_bwd_data_kernel_f32 : public jit_generator {
+struct _jit_avx512_common_conv_bwd_data_kernel_f32 : public jit_generator_t {
 
     _jit_avx512_common_conv_bwd_data_kernel_f32(const jit_conv_conf_t &ajcp)
-        : jcp(ajcp) {}
+        : jit_generator_t(jit_name()), jcp(ajcp) {}
 
     DECLARE_CPU_JIT_AUX_FUNCTIONS(_jit_avx512_common_conv_bwd_data_kernel_f32)
     jit_conv_conf_t jcp;
@@ -245,20 +245,12 @@ private:
     reg64_t reg_ker = r9;
     reg64_t reg_src = r10;
 
-    reg64_t reg_dst_prf = r11;
-    reg64_t reg_ker_prf = r12;
-    reg64_t reg_src_prf = r13;
     reg64_t reg_iwb = r14;
 
     reg64_t aux_reg_dst = r14;
     reg64_t aux_reg_ker = r15;
 
-    reg64_t aux_reg_dst_prf = rsi;
-    reg64_t aux_reg_ker_prf = rdx;
-
-    reg64_t aux_reg_dst_d_prf = r13;
     reg64_t aux_reg_dst_d = rbx;
-    reg64_t aux_reg_ker_d_prf = abi_not_param1;
     reg64_t aux_reg_ker_d = r9;
     reg64_t reg_ki = r10;
 
@@ -295,7 +287,6 @@ private:
 
     inline void prepare_output(int ur_w);
     inline void store_output(int ur_w);
-    inline void compute_loop_4fma(int ur_w, int l_overflow, int r_overflow);
     inline void compute_loop_fma(int ur_w, int l_overflow, int r_overflow);
     inline void compute_loop_fma_core(
             int ur_w, int l_overflow, int r_overflow, int k_offset);
@@ -337,8 +328,9 @@ private:
     inline ptrdiff_t get_dst_offset(int iw, int oc, int kw) {
         ptrdiff_t ow
                 = (iw + jcp.l_pad - kw * (jcp.dilate_w + 1)) / jcp.stride_w;
-        ptrdiff_t ow_str
-                = is_ddst_layout_nxc() ? jcp.ngroups * jcp.oc : jcp.oc_block;
+        ptrdiff_t ow_str = is_ddst_layout_nxc()
+                ? static_cast<ptrdiff_t>(jcp.ngroups) * jcp.oc
+                : jcp.oc_block;
 
         return typesize * (ow * ow_str + oc);
     };
@@ -359,24 +351,30 @@ struct jit_avx512_common_conv_bwd_data_kernel_f32 {
         : kernel_(nullptr) {
         switch (ajcp.ic_block) {
             case 16:
-                kernel_ = new _jit_avx512_common_conv_bwd_data_kernel_f32<
-                        Xbyak::Zmm>(ajcp);
+                kernel_ = utils::make_unique<
+                        _jit_avx512_common_conv_bwd_data_kernel_f32<
+                                Xbyak::Zmm>>(ajcp);
                 return;
             case 8:
-                kernel_ = new _jit_avx512_common_conv_bwd_data_kernel_f32<
-                        Xbyak::Ymm>(ajcp);
+                kernel_ = utils::make_unique<
+                        _jit_avx512_common_conv_bwd_data_kernel_f32<
+                                Xbyak::Ymm>>(ajcp);
                 return;
             case 4:
-                kernel_ = new _jit_avx512_common_conv_bwd_data_kernel_f32<
-                        Xbyak::Xmm>(ajcp);
+                kernel_ = utils::make_unique<
+                        _jit_avx512_common_conv_bwd_data_kernel_f32<
+                                Xbyak::Xmm>>(ajcp);
                 return;
             default: assert(!"invalid channel blocking");
         }
     }
 
-    status_t create_kernel() { return kernel_->create_kernel(); }
+    status_t create_kernel() {
+        if (kernel_) return kernel_->create_kernel();
+        return status::out_of_memory;
+    }
 
-    ~jit_avx512_common_conv_bwd_data_kernel_f32() { delete kernel_; }
+    ~jit_avx512_common_conv_bwd_data_kernel_f32() = default;
 
     enum { typesize = sizeof(float) };
 
@@ -391,13 +389,13 @@ struct jit_avx512_common_conv_bwd_data_kernel_f32 {
 
 private:
     DNNL_DISALLOW_COPY_AND_ASSIGN(jit_avx512_common_conv_bwd_data_kernel_f32);
-    jit_generator *kernel_;
+    std::unique_ptr<jit_generator_t> kernel_;
 };
 
-struct jit_avx512_common_conv_bwd_weights_kernel_f32 : public jit_generator {
+struct jit_avx512_common_conv_bwd_weights_kernel_f32 : public jit_generator_t {
 
     jit_avx512_common_conv_bwd_weights_kernel_f32(const jit_conv_conf_t &ajcp)
-        : jcp(ajcp) {}
+        : jit_generator_t(jit_name()), jcp(ajcp) {}
 
     void generate() override {
         if (jcp.harness != harness_nxc)
@@ -466,24 +464,18 @@ private:
     inline void compute_ic_block_step_fma_expl(int ur_w, int pad_l, int pad_r,
             int ic_block_step, int input_offset, int kernel_offset,
             int output_offset, bool input_wraparound);
-    inline void compute_ic_block_step_4fma(int ur_w, int pad_l, int pad_r,
-            int ic_block_step, int input_offset, int kernel_offset,
-            int output_offset, bool input_wraparound);
     inline void compute_oh_step_common(int ic_block_step, int max_ur_w);
     inline void compute_oh_step_disp();
     inline void compute_oh_loop_common();
     inline void compute_oh_loop_partial();
     inline void compute_od_loop_partial();
 
-    inline bool compute_full_spat_loop();
-    inline bool flat_4ops_compute();
-
     inline void compute_loop();
-    inline bool is_src_layout_nxc() {
+    inline bool is_src_layout_nxc() const {
         return utils::one_of(jcp.src_tag, format_tag::ndhwc, format_tag::nhwc,
                 format_tag::nwc);
     }
-    inline bool is_ddst_layout_nxc() {
+    inline bool is_ddst_layout_nxc() const {
         return utils::one_of(jcp.dst_tag, format_tag::ndhwc, format_tag::nhwc,
                 format_tag::nwc);
     }
@@ -493,20 +485,16 @@ private:
         const bool is_nxc_layout = is_src_layout_nxc();
         const size_t w_shift_st = (jcp.is_hw_transp ? jcp.iw : 1)
                 * (jcp.is_1stconv ? 1 : jcp.ic_block);
-        ptrdiff_t w_shift = is_nxc_layout
-                ? jcp.ngroups * jcp.ic
-                : jcp.ver == ver_4fma ? 1 : w_shift_st;
-        ptrdiff_t ic_shift = jcp.ver == ver_4fma
-                ? jcp.tr_iw
-                : (jcp.is_1stconv && !is_nxc_layout
-                                ? (ptrdiff_t)jcp.ih * jcp.iw * jcp.id
-                                : 1);
+        ptrdiff_t w_shift = is_nxc_layout ? jcp.ngroups * jcp.ic : w_shift_st;
+        ptrdiff_t ic_shift = jcp.is_1stconv && !is_nxc_layout
+                ? (ptrdiff_t)jcp.ih * jcp.iw * jcp.id
+                : 1;
 
         ptrdiff_t local_input_offset = i_iw * w_shift + i_ic * ic_shift;
         return input_offset + typesize * local_input_offset;
     };
 
-    inline int get_iw_idx(int ow, int kw, int l_pad) {
+    inline int get_iw_idx(int ow, int kw, int l_pad) const {
         return ow * jcp.stride_w + kw * (jcp.dilate_w + 1) - l_pad;
     }
 
