@@ -45,7 +45,7 @@ using namespace Xbyak;
 jit_avx512_core_bf16_1x1_conv_kernel::jit_avx512_core_bf16_1x1_conv_kernel(
         const jit_1x1_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(jit_name(), avx512_core_bf16), jcp(ajcp), attr_(attr) {
+    : jit_generator_t(jit_name(), avx512_core_bf16), jcp(ajcp), attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary) {
         using namespace binary_injector;
         static constexpr bool preserve_gpr = true;
@@ -1193,7 +1193,7 @@ status_t jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
         const memory_desc_wrapper &dst_d, primitive_attr_t &attr, int nthreads,
         bool reduce_src) {
     const bool with_groups = weights_d.ndims() == src_d.ndims() + 1;
-    const int simd_w = cpu_isa_traits<avx512_core>::vlen / sizeof(float);
+    const int simd_w = cpu_isa_traits_t<avx512_core>::vlen / sizeof(float);
     const int ndims = src_d.ndims();
 
     // Big int (> INT_MAX) values are unsupported and jcp fields may overflow
@@ -1335,7 +1335,7 @@ status_t jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
         jcp.dst_dt = src_d.data_type();
     } else if (jcp.prop_kind == backward_weights) {
         jcp.typesize_in = types::data_type_size(src_d.data_type());
-        jcp.typesize_out = sizeof(prec_traits<data_type::f32>::type);
+        jcp.typesize_out = sizeof(prec_traits_t<data_type::f32>::type);
         jcp.dst_dt = weights_d.data_type();
     }
 
