@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -30,7 +30,7 @@
 
 #include "cpu/x64/cpu_reducer.hpp"
 #include "cpu/x64/jit_avx512_core_bf16_1x1_conv_kernel.hpp"
-#include "cpu/x64/jit_transpose_src_utils.hpp"
+#include "cpu/x64/jit_transpose_utils.hpp"
 #include "cpu/x64/jit_uni_1x1_conv_utils.hpp"
 #include "cpu/x64/jit_uni_dw_convolution.hpp"
 
@@ -42,13 +42,7 @@ namespace x64 {
 template <impl::data_type_t dst_type>
 struct jit_avx512_core_bf16_1x1_convolution_fwd_t : public primitive_t {
     struct pd_t : public cpu_convolution_fwd_pd_t {
-        using dw_conv_pd_type = cpu_convolution_fwd_pd_t;
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const typename pd_t::base_class *hint_fwd_pd)
-            : cpu_convolution_fwd_pd_t(adesc, attr, hint_fwd_pd)
-            , jcp_()
-            , rtus_()
-            , jcp_dw_(nullptr) {}
+        using cpu_convolution_fwd_pd_t::cpu_convolution_fwd_pd_t;
 
         pd_t(const pd_t &other) : cpu_convolution_fwd_pd_t(other) {
             if (copy(other) != status::success) is_initialized_ = false;
@@ -58,50 +52,70 @@ struct jit_avx512_core_bf16_1x1_convolution_fwd_t : public primitive_t {
                 jit_avx512_core_bf16_1x1_convolution_fwd_t);
 
         status_t init(engine_t *engine) {
-            bool ok = true && mayiuse(avx512_core) && is_fwd()
-                    && set_default_alg_kind(alg_kind::convolution_direct)
-                    && expect_data_types(data_type::bf16, data_type::bf16,
-                            data_type::undef, dst_type, data_type::undef)
-                    && IMPLICATION(with_bias(),
-                            utils::one_of(weights_md(1)->data_type,
-                                    data_type::f32, data_type::bf16))
-                    && attr()->has_default_values(
-                            primitive_attr_t::skip_mask_t::post_ops, dst_type)
-                    && !has_zero_dim_memory() && set_default_formats();
+            using namespace data_type;
+            // Disabling verbose dispatch messages for unsupported isa for
+            // better readability.
+            if (!mayiuse(avx512_core)) return status::unimplemented;
 
-            if (!ok) return status::unimplemented;
+            VDISPATCH_CONV(is_fwd(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_CONV(expect_data_types(bf16, bf16, data_type::undef,
+                                   dst_type, data_type::undef),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(set_default_alg_kind(alg_kind::convolution_direct),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_CONV(
+                    IMPLICATION(with_bias(),
+                            utils::one_of(weights_md(1)->data_type, f32, bf16)),
+                    VERBOSE_UNSUPPORTED_BIAS_CFG);
+            VDISPATCH_CONV(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+            VDISPATCH_CONV(
+                    attr()->has_default_values(
+                            primitive_attr_t::skip_mask_t::post_ops, dst_type),
+                    VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_CONV(set_default_formats(), VERBOSE_UNSUPPORTED_TAG);
+            VDISPATCH_CONV(
+                    attr_.set_default_formats(dst_md(0)) == status::success,
+                    VERBOSE_UNSUPPORTED_POSTOP);
 
             const convolution_desc_t *conv_d = desc();
             const memory_desc_t *src_d = src_md();
+
+            // TODO: make `rtus_prepare` assign initialized object to `rtus_`
             rtus_prepare(this, conv_d, src_d, dst_md(), weights_md());
 
-            status_t status = jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
-                    jcp_, *conv_d, *src_d, *weights_md(), *dst_md(), *attr(),
-                    dnnl_get_max_threads(), rtus_.reduce_src_);
-            if (status != status::success) return status;
+            // TODO: make `init_conf` assign initialized object to `jcp_`
+            CHECK(jit_avx512_core_bf16_1x1_conv_kernel::init_conf(jcp_, *conv_d,
+                    *src_d, *weights_md(), *dst_md(), attr_,
+                    dnnl_get_max_threads(), rtus_.reduce_src_));
 
-            if (jcp_.with_dw_conv) {
-                status = depthwise_po_init(engine);
-                if (status != status::success) return status;
-            }
+            if (jcp_.with_dw_conv) CHECK(depthwise_po_init(engine));
 
             auto scratchpad = scratchpad_registry().registrar();
-            status = jit_avx512_core_bf16_1x1_conv_kernel::init_scratchpad(
-                    scratchpad, jcp_);
-            if (status != status::success) return status;
+            CHECK(jit_avx512_core_bf16_1x1_conv_kernel::init_scratchpad(
+                    scratchpad, jcp_));
 
             rtus_prepare_space_info(this, scratchpad, jcp_.nthr);
 
             return status::success;
         }
 
-        const memory_desc_t *dst_md(int index = 0) const override {
-            return jcp_.with_dw_conv ? dw_conv_pd_->dst_md(index) : &dst_md_;
+        const memory_desc_t *dst_1x1_md(int index = 0) const {
+            return cpu_convolution_fwd_pd_t::dst_md(index);
         }
 
-        const memory_desc_t *arg_md(int index = 0) const override {
-            if (jcp_.with_dw_conv) {
-                switch (index) {
+        const memory_desc_t *dst_md(
+                int index = 0, bool user_input = false) const override {
+            return dw_conv_pd_ && jcp_.with_dw_conv
+                    ? dw_conv_pd_->dst_md(index, user_input)
+                    : cpu_convolution_fwd_pd_t::dst_md(index, user_input);
+        }
+
+        const memory_desc_t *arg_md(
+                int arg, bool user_input = false) const override {
+            if (dw_conv_pd_ && jcp_.with_dw_conv) {
+                switch (arg) {
+                    case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_SRC:
+                        return cpu_convolution_fwd_pd_t::dst_md(0, user_input);
                     case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS:
                         return dw_conv_pd_->weights_md(0);
                     case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS:
@@ -109,23 +123,23 @@ struct jit_avx512_core_bf16_1x1_convolution_fwd_t : public primitive_t {
                     default: break;
                 }
             }
-            return convolution_fwd_pd_t::arg_md(index);
+            return convolution_fwd_pd_t::arg_md(arg, user_input);
         }
 
         arg_usage_t arg_usage(int arg) const override {
             if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS))
                 return arg_usage_t::input;
 
-            if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS)
-                    && attr_post_op_dw_inputs() > 1)
-                return arg_usage_t::input;
+            if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS))
+                return attr_post_op_dw_inputs() > 1 ? arg_usage_t::input
+                                                    : arg_usage_t::unused;
 
             return convolution_fwd_pd_t::arg_usage(arg);
         }
 
-        jit_1x1_conv_conf_t jcp_;
-        reduce_to_unit_stride_t rtus_;
-        jit_conv_conf_t *jcp_dw_; // doesn't own a resource
+        jit_1x1_conv_conf_t jcp_ = utils::zero<decltype(jcp_)>();
+        reduce_to_unit_stride_t rtus_ = utils::zero<decltype(rtus_)>();
+        jit_conv_conf_t *jcp_dw_ = nullptr; // doesn't own a resource
         std::unique_ptr<cpu_convolution_fwd_pd_t> dw_conv_pd_;
 
     protected:
@@ -135,7 +149,25 @@ struct jit_avx512_core_bf16_1x1_convolution_fwd_t : public primitive_t {
 
         bool set_default_formats() {
             using namespace format_tag;
-            auto dat_tag = utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c);
+
+            const memory_desc_wrapper src_d(&src_md_);
+            const memory_desc_wrapper dst_d(&dst_md_);
+
+            const auto dat_tag_nxc = utils::pick(ndims() - 3, nwc, nhwc, ndhwc);
+            const auto dat_tag_nCx16c
+                    = utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c);
+            const auto curr_src_tag
+                    = src_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx16c);
+            const auto curr_dst_tag
+                    = dst_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx16c);
+            const auto is_data_layout_nxc
+                    = IMPLICATION(curr_src_tag != dat_tag_nxc,
+                              src_d.format_kind() == format_kind::any)
+                    && IMPLICATION(curr_dst_tag != dat_tag_nxc,
+                            dst_d.format_kind() == format_kind::any)
+                    && utils::one_of(dat_tag_nxc, curr_src_tag, curr_dst_tag);
+
+            auto dat_tag = is_data_layout_nxc ? dat_tag_nxc : dat_tag_nCx16c;
             auto wei_tag = utils::pick(2 * ndims() - 6 + with_groups(),
                     OIw8i16o2i, gOIw8i16o2i, OIhw8i16o2i, gOIhw8i16o2i,
                     OIdhw8i16o2i, gOIdhw8i16o2i);
@@ -177,7 +209,6 @@ struct jit_avx512_core_bf16_1x1_convolution_fwd_t : public primitive_t {
             jit_conv_conf_t *jcp_dw = nullptr;
             primitive_attr_t attr_1x1(*attr());
             if (!attr_1x1.is_initialized()) return status::out_of_memory;
-            attr_1x1.set_scratchpad_mode(scratchpad_mode::user);
 
             const auto &src_md = dst_md_;
             const memory_desc_wrapper src_d(src_md);
@@ -192,16 +223,22 @@ struct jit_avx512_core_bf16_1x1_convolution_fwd_t : public primitive_t {
             // for 1x1: Check that no better ISA is available.
             // for dw: Always fuse with same ISA.
             // Caveat: May be a better dw conv exists.
+            VDISPATCH_CONV_IC(!mayiuse(avx512_core_amx),
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL, "higher ISA is supported");
 
-            bool ok = !mayiuse(avx512_core_bf16_amx_bf16)
-                    && (attr_1x1.post_ops_.find(primitive_kind::sum) == -1)
-                    // TODO: Below may be further tuned.
-                    && (l2_cache * 2 < src_d.size())
-                    // load_grp_count check can be redundant due to l2 check
-                    // above. Adding it explicitly as the current driver doesn't
-                    // work if this condition fails.
-                    && (jcp_1x1.load_grp_count < 2);
-            if (!ok) return status::unimplemented;
+            VDISPATCH_CONV_IC(
+                    attr_1x1.post_ops_.find(primitive_kind::sum) == -1,
+                    VERBOSE_UNSUPPORTED_FEATURE, "unsupported sum post-op");
+
+            // TODO: Below may be further tuned.
+            VDISPATCH_CONV_IC(l2_cache * 2 < src_d.size(),
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL, "cache size check failed");
+
+            // load_grp_count check can be redundant due to l2 check
+            // above. Adding it explicitly as the current driver doesn't
+            // work if this condition fails.
+            VDISPATCH_CONV_IC(jcp_1x1.load_grp_count < 2,
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL, "load group count > 1");
 
             int dw_po_index
                     = attr_1x1.post_ops_.find(primitive_kind::convolution);
@@ -214,9 +251,9 @@ struct jit_avx512_core_bf16_1x1_convolution_fwd_t : public primitive_t {
             auto dw_dst_dt = cd_dw.dst_desc.data_type;
 
 #define CASE(dt) \
-    case dt: { \
-        std::unique_ptr<dw_pd_t<dt>> fusable_pd( \
-                new dw_pd_t<dt>(&cd_dw, &attr_dw, nullptr)); \
+    case (dt): { \
+        auto fusable_pd \
+                = make_unique_pd<dw_pd_t<(dt)>>(&cd_dw, &attr_dw, nullptr); \
         CHECK(fusable_pd->init(engine)); \
         jcp_dw = &(fusable_pd->jcp_); \
         dw_conv_pd_ = std::move(fusable_pd); \
@@ -232,12 +269,17 @@ struct jit_avx512_core_bf16_1x1_convolution_fwd_t : public primitive_t {
                 return status::unimplemented;
 #undef CASE
 
-            ok = true
-                    && (dnnl_memory_desc_equal(&src_md, dw_conv_pd_->src_md(0)))
-                    && (jcp_1x1.oc_without_padding % jcp_1x1.oc_block == 0)
-                    && IMPLICATION(
-                            jcp_dw->ow_block, jcp_dw->ow_block == jcp_dw->ow);
-            if (!ok) return status::unimplemented;
+            VDISPATCH_CONV_IC(
+                    dnnl_memory_desc_equal(&src_md, dw_conv_pd_->src_md(0)),
+                    VERBOSE_INCONSISTENT_MDS, "src_md", "dw_conv_pd_->src_md");
+            VDISPATCH_CONV_IC(
+                    jcp_1x1.oc_without_padding % jcp_1x1.oc_block == 0,
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL,
+                    "output-channel is not an exact multiple of oc_block");
+            VDISPATCH_CONV_IC(IMPLICATION(jcp_dw->ow_block,
+                                      jcp_dw->ow_block == jcp_dw->ow),
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL,
+                    "ow_block does not equal output-width");
 
             assert(dw_conv_pd_->dst_md(0)->format_kind != format_kind::any);
             assert(dw_conv_pd_->weights_md(0)->format_kind != format_kind::any);
@@ -280,17 +322,17 @@ struct jit_avx512_core_bf16_1x1_convolution_fwd_t : public primitive_t {
     jit_avx512_core_bf16_1x1_convolution_fwd_t(const pd_t *apd)
         : primitive_t(apd) {}
 
-    typedef typename prec_traits<data_type::bf16>::type src_data_t;
-    typedef typename prec_traits<data_type::bf16>::type wei_data_t;
-    typedef typename prec_traits<dst_type>::type dst_data_t;
+    using src_data_t = typename prec_traits_t<data_type::bf16>::type;
+    using wei_data_t = typename prec_traits_t<data_type::bf16>::type;
+    using dst_data_t = typename prec_traits_t<dst_type>::type;
     // Note: In case of fused depthwise convolution, the final output datatype
     // may not be dst_data_t.
-    typedef typename prec_traits<dst_type>::type dw_wei_data_t;
+    using dw_wei_data_t = typename prec_traits_t<dst_type>::type;
 
     status_t init(engine_t *engine) override {
         CHECK(safe_ptr_assign(kernel_,
                 new jit_avx512_core_bf16_1x1_conv_kernel(
-                        pd()->jcp_, *pd()->attr(), *pd()->dst_md(0))));
+                        pd()->jcp_, *pd()->attr(), *pd()->dst_1x1_md(0))));
         CHECK(kernel_->create_kernel());
 
         if (pd()->jcp_.with_dw_conv) {
@@ -299,7 +341,7 @@ struct jit_avx512_core_bf16_1x1_convolution_fwd_t : public primitive_t {
             CHECK(kernel_dw_->create_kernel());
         }
 
-        CHECK(init_rtus_driver<avx512_common>(this));
+        CHECK(init_rtus_driver<avx512_core>(this));
         return status::success;
     }
 
@@ -319,7 +361,7 @@ private:
     const pd_t *pd() const { return (const pd_t *)primitive_t::pd().get(); }
 
     std::unique_ptr<jit_avx512_core_bf16_1x1_conv_kernel> kernel_;
-    std::unique_ptr<rtus_driver_t<avx512_common>> rtus_driver_;
+    std::unique_ptr<rtus_driver_t<avx512_core>> rtus_driver_;
     using dw_conv_kernel_t
             = jit_uni_dw_conv_fwd_kernel<avx512_core, data_type::bf16>;
     std::unique_ptr<dw_conv_kernel_t> kernel_dw_;
@@ -328,51 +370,70 @@ private:
 template <impl::data_type_t diff_src_type>
 struct jit_avx512_core_bf16_1x1_convolution_bwd_data_t : public primitive_t {
     struct pd_t : public cpu_convolution_bwd_data_pd_t {
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const convolution_fwd_pd_t *hint_fwd_pd)
-            : cpu_convolution_bwd_data_pd_t(adesc, attr, hint_fwd_pd)
-            , jcp_()
-            , rtus_() {}
+        using cpu_convolution_bwd_data_pd_t::cpu_convolution_bwd_data_pd_t;
 
         DECLARE_COMMON_PD_T(JIT_IMPL_NAME_HELPER("jit_bf16_1x1:", jcp_.isa, ""),
                 jit_avx512_core_bf16_1x1_convolution_bwd_data_t);
 
         status_t init(engine_t *engine) {
-            bool ok = true && mayiuse(avx512_core) && is_bwd_d()
-                    && set_default_alg_kind(alg_kind::convolution_direct)
-                    && expect_data_types(diff_src_type, data_type::bf16,
-                            data_type::undef, data_type::bf16, data_type::undef)
-                    && attr()->has_default_values() && !has_zero_dim_memory()
-                    && set_default_formats();
-            if (!ok) return status::unimplemented;
+            using namespace data_type;
+            // disabling verbose dispatch messages for unsupported isa for better readability
+            if (!mayiuse(avx512_core)) return status::unimplemented;
+
+            VDISPATCH_CONV(is_bwd_d(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_CONV(expect_data_types(diff_src_type, bf16,
+                                   data_type::undef, bf16, data_type::undef),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(set_default_alg_kind(alg_kind::convolution_direct),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_CONV(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+            VDISPATCH_CONV(
+                    attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_CONV(set_default_formats(), VERBOSE_UNSUPPORTED_TAG);
 
             const convolution_desc_t *conv_d = desc();
             const memory_desc_t *diff_src_d = diff_src_md();
+            // TODO: make `rtus_prepare` assign initialized object to `rtus_`
             rtus_prepare(this, conv_d, diff_src_d, diff_dst_md(), weights_md());
 
-            status_t status = jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
-                    jcp_, *conv_d, *diff_src_d, *weights_md(), *diff_dst_md(),
-                    *attr(), dnnl_get_max_threads(), rtus_.reduce_src_);
-            if (status != status::success) return status;
+            // TODO: make `init_conf` assign initialized object to `jcp_`
+            CHECK(jit_avx512_core_bf16_1x1_conv_kernel::init_conf(jcp_, *conv_d,
+                    *diff_src_d, *weights_md(), *diff_dst_md(), attr_,
+                    dnnl_get_max_threads(), rtus_.reduce_src_));
 
             auto scratchpad = scratchpad_registry().registrar();
-            status = jit_avx512_core_bf16_1x1_conv_kernel::init_scratchpad(
-                    scratchpad, jcp_);
-            if (status != status::success) return status;
+            CHECK(jit_avx512_core_bf16_1x1_conv_kernel::init_scratchpad(
+                    scratchpad, jcp_));
             rtus_prepare_space_info(this, scratchpad, jcp_.nthr);
 
             return status::success;
         }
 
         // TODO (Roma): structs conf header cleanup
-        jit_1x1_conv_conf_t jcp_;
-        reduce_to_unit_stride_t rtus_;
+        jit_1x1_conv_conf_t jcp_ = utils::zero<decltype(jcp_)>();
+        reduce_to_unit_stride_t rtus_ = utils::zero<decltype(rtus_)>();
 
     protected:
         bool set_default_formats() {
             using namespace format_tag;
 
-            auto dat_tag = utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c);
+            const memory_desc_wrapper diff_src_d(&diff_src_md_);
+            const memory_desc_wrapper diff_dst_d(&diff_dst_md_);
+
+            const auto dat_tag_nxc = utils::pick(ndims() - 3, nwc, nhwc, ndhwc);
+            const auto dat_tag_nCx16c
+                    = utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c);
+            const auto curr_src_tag = diff_src_d.matches_one_of_tag(
+                    dat_tag_nxc, dat_tag_nCx16c);
+            const auto curr_dst_tag = diff_dst_d.matches_one_of_tag(
+                    dat_tag_nxc, dat_tag_nCx16c);
+            const auto is_data_layout_nxc
+                    = IMPLICATION(curr_src_tag != dat_tag_nxc,
+                              diff_src_d.format_kind() == format_kind::any)
+                    && IMPLICATION(curr_dst_tag != dat_tag_nxc,
+                            diff_dst_d.format_kind() == format_kind::any)
+                    && utils::one_of(dat_tag_nxc, curr_src_tag, curr_dst_tag);
+            auto dat_tag = is_data_layout_nxc ? dat_tag_nxc : dat_tag_nCx16c;
             auto wei_tag = utils::pick(2 * ndims() - 6 + with_groups(),
                     IOw8o16i2o, gIOw8o16i2o, IOhw8o16i2o, gIOhw8o16i2o,
                     IOdhw8o16i2o, gIOdhw8o16i2o);
@@ -387,16 +448,16 @@ struct jit_avx512_core_bf16_1x1_convolution_bwd_data_t : public primitive_t {
     jit_avx512_core_bf16_1x1_convolution_bwd_data_t(const pd_t *apd)
         : primitive_t(apd) {}
 
-    typedef typename prec_traits<data_type::bf16>::type diff_dst_data_t;
-    typedef typename prec_traits<data_type::bf16>::type wei_data_t;
-    typedef typename prec_traits<diff_src_type>::type diff_src_data_t;
+    using diff_dst_data_t = typename prec_traits_t<data_type::bf16>::type;
+    using wei_data_t = typename prec_traits_t<data_type::bf16>::type;
+    using diff_src_data_t = typename prec_traits_t<diff_src_type>::type;
 
     status_t init(engine_t *engine) override {
         CHECK(safe_ptr_assign(kernel_,
                 new jit_avx512_core_bf16_1x1_conv_kernel(
                         pd()->jcp_, *pd()->attr(), *pd()->dst_md(0))));
         CHECK(kernel_->create_kernel());
-        CHECK(init_rtus_driver<avx512_common>(this));
+        CHECK(init_rtus_driver<avx512_core>(this));
         return status::success;
     }
 
@@ -414,49 +475,54 @@ private:
 
     std::unique_ptr<jit_avx512_core_bf16_1x1_conv_kernel> kernel_;
     /* reduction to unit stride */
-    std::unique_ptr<rtus_driver_t<avx512_common>> rtus_driver_;
+    std::unique_ptr<rtus_driver_t<avx512_core>> rtus_driver_;
 };
 
 template <impl::data_type_t diff_weights_type>
 struct jit_avx512_core_bf16_1x1_convolution_bwd_weights_t : public primitive_t {
     struct pd_t : public cpu_convolution_bwd_weights_pd_t {
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const convolution_fwd_pd_t *hint_fwd_pd)
-            : cpu_convolution_bwd_weights_pd_t(adesc, attr, hint_fwd_pd)
-            , jcp_()
-            , rtus_() {}
+        using cpu_convolution_bwd_weights_pd_t::
+                cpu_convolution_bwd_weights_pd_t;
 
         DECLARE_COMMON_PD_T(JIT_IMPL_NAME_HELPER("jit_bf16_1x1:", jcp_.isa, ""),
                 jit_avx512_core_bf16_1x1_convolution_bwd_weights_t);
 
         status_t init(engine_t *engine) {
             using namespace prop_kind;
+            using namespace data_type;
             assert(engine->kind() == engine_kind::cpu);
-            bool ok = true && mayiuse(avx512_core) && is_bwd_w()
-                    && set_default_alg_kind(alg_kind::convolution_direct)
-                    && expect_data_types(data_type::bf16, diff_weights_type,
-                            data_type::undef, data_type::bf16, data_type::undef)
-                    && IMPLICATION(with_bias(),
-                            utils::one_of(diff_weights_md(1)->data_type,
-                                    data_type::f32, data_type::bf16))
-                    && attr()->has_default_values() && !has_zero_dim_memory()
-                    && set_default_formats();
-            if (!ok) return status::unimplemented;
+            // disabling verbose dispatch messages for unsupported isa for better readability
+            if (!mayiuse(avx512_core)) return status::unimplemented;
+
+            VDISPATCH_CONV(is_bwd_w(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_CONV(expect_data_types(bf16, diff_weights_type,
+                                   data_type::undef, bf16, data_type::undef),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(set_default_alg_kind(alg_kind::convolution_direct),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_CONV(IMPLICATION(with_bias(),
+                                   utils::one_of(diff_weights_md(1)->data_type,
+                                           f32, bf16)),
+                    VERBOSE_UNSUPPORTED_BIAS_CFG);
+            VDISPATCH_CONV(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+            VDISPATCH_CONV(
+                    attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_CONV(set_default_formats(), VERBOSE_UNSUPPORTED_TAG);
 
             const convolution_desc_t *conv_d = desc();
             const memory_desc_t *src_d = src_md();
-            rtus_prepare(
-                    this, conv_d, src_d, diff_dst_md(), diff_weights_md(0));
 
-            status_t status = jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
-                    jcp_, *conv_d, *src_d, *diff_weights_md(0), *diff_dst_md(),
-                    *attr(), dnnl_get_max_threads(), rtus_.reduce_src_);
-            if (status != status::success) return status;
+            // TODO: make `rtus_prepare` assign initialized object to `rtus_`
+            rtus_prepare(this, conv_d, src_d, diff_dst_md(), diff_weights_md());
+
+            // TODO: make `init_conf` assign initialized object to `jcp_`
+            CHECK(jit_avx512_core_bf16_1x1_conv_kernel::init_conf(jcp_, *conv_d,
+                    *src_d, *diff_weights_md(0), *diff_dst_md(), attr_,
+                    dnnl_get_max_threads(), rtus_.reduce_src_));
 
             auto scratchpad = scratchpad_registry().registrar();
-            status = jit_avx512_core_bf16_1x1_conv_kernel::init_scratchpad(
-                    scratchpad, jcp_);
-            if (status != status::success) return status;
+            CHECK(jit_avx512_core_bf16_1x1_conv_kernel::init_scratchpad(
+                    scratchpad, jcp_));
 
             rtus_prepare_space_info(this, scratchpad, jcp_.nthr);
 
@@ -464,14 +530,30 @@ struct jit_avx512_core_bf16_1x1_convolution_bwd_weights_t : public primitive_t {
         }
 
         // TODO (Roma): structs conf header cleanup
-        jit_1x1_conv_conf_t jcp_;
-        reduce_to_unit_stride_t rtus_;
+        jit_1x1_conv_conf_t jcp_ = utils::zero<decltype(jcp_)>();
+        reduce_to_unit_stride_t rtus_ = utils::zero<decltype(rtus_)>();
 
     protected:
         bool set_default_formats() {
             using namespace format_tag;
 
-            auto dat_tag = utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c);
+            const memory_desc_wrapper src_d(&src_md_);
+            const memory_desc_wrapper diff_dst_d(&diff_dst_md_);
+
+            const auto dat_tag_nxc = utils::pick(ndims() - 3, nwc, nhwc, ndhwc);
+            const auto dat_tag_nCx16c
+                    = utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c);
+            const auto curr_src_tag
+                    = src_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx16c);
+            const auto curr_dst_tag = diff_dst_d.matches_one_of_tag(
+                    dat_tag_nxc, dat_tag_nCx16c);
+            const auto is_data_layout_nxc
+                    = IMPLICATION(curr_src_tag != dat_tag_nxc,
+                              src_d.format_kind() == format_kind::any)
+                    && IMPLICATION(curr_dst_tag != dat_tag_nxc,
+                            diff_dst_d.format_kind() == format_kind::any)
+                    && utils::one_of(dat_tag_nxc, curr_src_tag, curr_dst_tag);
+            auto dat_tag = is_data_layout_nxc ? dat_tag_nxc : dat_tag_nCx16c;
             auto wei_tag = utils::pick(2 * ndims() - 6 + with_groups(),
                     OIw16i16o, gOIw16i16o, OIhw16i16o, gOIhw16i16o, OIdhw16i16o,
                     gOIdhw16i16o);
@@ -494,10 +576,10 @@ struct jit_avx512_core_bf16_1x1_convolution_bwd_weights_t : public primitive_t {
         return status::success;
     }
 
-    typedef typename prec_traits<data_type::bf16>::type src_data_t;
-    typedef typename prec_traits<data_type::bf16>::type diff_dst_data_t;
+    using src_data_t = typename prec_traits_t<data_type::bf16>::type;
+    using diff_dst_data_t = typename prec_traits_t<data_type::bf16>::type;
 
-    typedef typename prec_traits<diff_weights_type>::type diff_wei_data_t;
+    using diff_wei_data_t = typename prec_traits_t<diff_weights_type>::type;
 
 private:
     void execute_backward_weights(const exec_ctx_t &ctx) const;
@@ -507,7 +589,7 @@ private:
     std::unique_ptr<cpu_accumulator_1d_t<data_type::f32>> acc_ker_;
 
     /* reduction to unit stride */
-    std::unique_ptr<rtus_driver_t<avx512_common>> rtus_driver_;
+    std::unique_ptr<rtus_driver_t<avx512_core>> rtus_driver_;
 
     std::unique_ptr<jit_avx512_core_bf16_reorder_s16c_to_S16c2s_t> tr_reorder_;
     std::unique_ptr<jit_avx512_core_bf16_reorder_s16c_to_S16c2s_t>
