@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
 #ifndef CPU_X64_JIT_AVX512_CORE_BF16_CONV_KERNEL_HPP
 #define CPU_X64_JIT_AVX512_CORE_BF16_CONV_KERNEL_HPP
 
+#include <memory>
 #include "common/c_types_map.hpp"
 #include "common/memory_tracking.hpp"
 
@@ -31,7 +32,7 @@ namespace cpu {
 namespace x64 {
 
 template <typename Vmm>
-struct _jit_avx512_core_bf16_fwd_kernel : public jit_generator {
+struct _jit_avx512_core_bf16_fwd_kernel : public jit_generator_t {
     _jit_avx512_core_bf16_fwd_kernel(const jit_conv_conf_t &ajcp,
             const primitive_attr_t &attr, const memory_desc_t &dst_md);
 
@@ -42,14 +43,13 @@ struct _jit_avx512_core_bf16_fwd_kernel : public jit_generator {
 
 private:
     constexpr static int isa_simd_width_
-            = cpu_isa_traits<avx512_core>::vlen / sizeof(float);
+            = cpu_isa_traits_t<avx512_core>::vlen / sizeof(float);
     using Vmm_down_t =
             typename utils::conditional<std::is_same<Vmm, Xbyak::Zmm>::value,
                     Xbyak::Ymm, Xbyak::Xmm>::type;
     using reg64_t = const Xbyak::Reg64;
     enum {
         ker_reg_base_idx = 28,
-        ker_code_size = 1024 * 1024,
     };
 
     reg64_t param = abi_param1; //L: RDI, W: RCX
@@ -73,6 +73,9 @@ private:
 
     reg64_t reg_long_offt = r14;
 
+    /* binary post-ops operand */
+    reg64_t temp_offset_reg = r12;
+
     int vmm_dst_idx(const int i_ur, const int i_oc) const;
     Vmm vmm_dst(const int i_ur, const int i_oc) const;
 
@@ -124,7 +127,7 @@ private:
     constexpr static int off_reg_ker_ = 8;
     constexpr static int stack_space_needed_ = 16;
 
-    std::unique_ptr<injector::jit_uni_postops_injector_t<avx512_core>>
+    std::unique_ptr<injector::jit_uni_postops_injector_t<avx512_core, Vmm>>
             postops_injector_;
     std::unique_ptr<bf16_emulation_t> bf16_emu_;
 
@@ -137,27 +140,29 @@ private:
 
     inline dim_t get_dst_offset(dim_t sp_idx, int ocb) {
         const bool is_layout_nxc = is_dst_layout_nxc();
-        dim_t sp_str = is_layout_nxc ? jcp.ngroups * jcp.oc : jcp.oc_block;
+        dim_t sp_str = is_layout_nxc ? static_cast<dim_t>(jcp.ngroups) * jcp.oc
+                                     : jcp.oc_block;
         dim_t ocb_str = jcp.oc_block
                 * (is_layout_nxc ? 1 : (dim_t)jcp.od * jcp.oh * jcp.ow);
         return jcp.typesize_out * (ocb_str * ocb + sp_str * sp_idx);
     }
 
     inline dim_t filter_w_to_src(int kw, int ow = 0, int pad_l = 0) {
-        return kw * (jcp.dilate_w + 1) + ow * jcp.stride_w - pad_l;
+        return static_cast<dim_t>(kw) * (jcp.dilate_w + 1) + ow * jcp.stride_w
+                - pad_l;
     }
     inline dim_t filter_h_to_src(int kh) {
-        return kh * (jcp.dilate_h + 1) * jcp.iw;
+        return static_cast<dim_t>(kh) * (jcp.dilate_h + 1) * jcp.iw;
     }
     inline dim_t filter_d_to_src(int kd) {
-        return kd * (jcp.dilate_d + 1) * jcp.iw * jcp.ih;
+        return static_cast<dim_t>(kd) * (jcp.dilate_d + 1) * jcp.iw * jcp.ih;
     }
 
     inline dim_t get_src_offset(dim_t ic_idx, dim_t isp) {
         int icb = ic_idx / jcp.ic_block;
         int ic = ic_idx % jcp.ic_block;
         dim_t isp_str = is_src_layout_nxc()
-                ? jcp.ngroups * jcp.ic
+                ? static_cast<dim_t>(jcp.ngroups) * jcp.ic
                 : (jcp.is_1stconv ? 1 : jcp.ic_block);
         dim_t full_spatial_size = (dim_t)jcp.iw * jcp.ih * jcp.id;
         dim_t ic_str = jcp.is_1stconv && !is_src_layout_nxc()
@@ -174,13 +179,15 @@ private:
         int rnd_ic_block = utils::rnd_up(jcp.ic_block, scale);
         int icb = ic_idx / jcp.ic_block;
         int ic = ic_idx % jcp.ic_block;
-        dim_t ksp_str = rnd_ic_block * jcp.oc_block;
-        dim_t ksp_idx = kd * jcp.kh * jcp.kw + kh * jcp.kw + kw;
+        dim_t ksp_str = static_cast<dim_t>(rnd_ic_block) * jcp.oc_block;
+        dim_t ksp_idx
+                = static_cast<dim_t>(kd) * jcp.kh * jcp.kw + kh * jcp.kw + kw;
 
-        dim_t icb_str = jcp.kd * jcp.kh * jcp.kw * ksp_str;
-        dim_t ocb_str = jcp.nb_ic * icb_str;
-        dim_t ic_off = (ic / scale) * jcp.oc_block * scale + (ic % scale);
-        return jcp.typesize_in
+        dim_t icb_str = static_cast<dim_t>(jcp.kd) * jcp.kh * jcp.kw * ksp_str;
+        dim_t ocb_str = static_cast<dim_t>(jcp.nb_ic) * icb_str;
+        dim_t ic_off = static_cast<dim_t>(ic / scale) * jcp.oc_block * scale
+                + (ic % scale);
+        return static_cast<dim_t>(jcp.typesize_in)
                 * (ocb * ocb_str + icb * icb_str + ksp_idx * ksp_str + ic_off);
     }
 
@@ -212,29 +219,35 @@ struct jit_avx512_core_bf16_fwd_kernel {
         : kernel_(nullptr) {
         switch (ajcp.oc_block) {
             case 16:
-                kernel_ = new _jit_avx512_core_bf16_fwd_kernel<Xbyak::Zmm>(
+                kernel_ = utils::make_unique<
+                        _jit_avx512_core_bf16_fwd_kernel<Xbyak::Zmm>>(
                         ajcp, attr, dst_md);
                 return;
             case 8:
-                kernel_ = new _jit_avx512_core_bf16_fwd_kernel<Xbyak::Ymm>(
+                kernel_ = utils::make_unique<
+                        _jit_avx512_core_bf16_fwd_kernel<Xbyak::Ymm>>(
                         ajcp, attr, dst_md);
                 return;
             case 4:
-                kernel_ = new _jit_avx512_core_bf16_fwd_kernel<Xbyak::Xmm>(
+                kernel_ = utils::make_unique<
+                        _jit_avx512_core_bf16_fwd_kernel<Xbyak::Xmm>>(
                         ajcp, attr, dst_md);
                 return;
             default: assert(!"invalid channel blocking");
         }
     }
 
-    status_t create_kernel() { return kernel_->create_kernel(); }
+    status_t create_kernel() {
+        if (kernel_) return kernel_->create_kernel();
+        return status::out_of_memory;
+    }
 
-    ~jit_avx512_core_bf16_fwd_kernel() { delete kernel_; }
+    ~jit_avx512_core_bf16_fwd_kernel() = default;
 
     static status_t init_conf(jit_conv_conf_t &jcp,
             const convolution_desc_t &cd, memory_desc_t &src_pd,
             memory_desc_t &weights_pd, memory_desc_t &dst_pd,
-            memory_desc_t &bias_pd, const primitive_attr_t &attr, int nthreads);
+            memory_desc_t &bias_pd, primitive_attr_t &attr, int nthreads);
     static void init_scratchpad(memory_tracking::registrar_t &scratchpad,
             const jit_conv_conf_t &jcp);
 
@@ -243,14 +256,14 @@ struct jit_avx512_core_bf16_fwd_kernel {
 
 private:
     DNNL_DISALLOW_COPY_AND_ASSIGN(jit_avx512_core_bf16_fwd_kernel);
-    jit_generator *kernel_;
+    std::unique_ptr<jit_generator_t> kernel_;
 };
 
 template <typename Vmm>
-struct _jit_avx512_core_bf16_bwd_data_kernel : public jit_generator {
+struct _jit_avx512_core_bf16_bwd_data_kernel : public jit_generator_t {
 
     _jit_avx512_core_bf16_bwd_data_kernel(const jit_conv_conf_t &ajcp)
-        : jit_generator(nullptr, ker_code_size, true, avx512_core_bf16)
+        : jit_generator_t(jit_name(), avx512_core_bf16)
         , jcp(ajcp)
         , bf16_emu_(nullptr) {
         if (!isa_has_bf16(jcp.isa))
@@ -270,7 +283,6 @@ private:
     using reg64_t = const Xbyak::Reg64;
     enum {
         ker_reg_base_idx = 31,
-        ker_code_size = 1024 * 1024,
     };
 
     reg64_t param = abi_param1;
@@ -423,24 +435,30 @@ struct jit_avx512_core_bf16_bwd_data_kernel {
         : kernel_(nullptr) {
         switch (ajcp.ic_block) {
             case 16:
-                kernel_ = new _jit_avx512_core_bf16_bwd_data_kernel<Xbyak::Zmm>(
+                kernel_ = utils::make_unique<
+                        _jit_avx512_core_bf16_bwd_data_kernel<Xbyak::Zmm>>(
                         ajcp);
                 return;
             case 8:
-                kernel_ = new _jit_avx512_core_bf16_bwd_data_kernel<Xbyak::Ymm>(
+                kernel_ = utils::make_unique<
+                        _jit_avx512_core_bf16_bwd_data_kernel<Xbyak::Ymm>>(
                         ajcp);
                 return;
             case 4:
-                kernel_ = new _jit_avx512_core_bf16_bwd_data_kernel<Xbyak::Xmm>(
+                kernel_ = utils::make_unique<
+                        _jit_avx512_core_bf16_bwd_data_kernel<Xbyak::Xmm>>(
                         ajcp);
                 return;
             default: assert(!"invalid channel blocking");
         }
     }
 
-    status_t create_kernel() { return kernel_->create_kernel(); }
+    status_t create_kernel() {
+        if (kernel_) return kernel_->create_kernel();
+        return status::out_of_memory;
+    }
 
-    ~jit_avx512_core_bf16_bwd_data_kernel() { delete kernel_; }
+    ~jit_avx512_core_bf16_bwd_data_kernel() = default;
 
     static status_t init_conf(jit_conv_conf_t &jcp,
             const convolution_desc_t &cd, memory_desc_t &diff_src_md,
@@ -451,14 +469,15 @@ struct jit_avx512_core_bf16_bwd_data_kernel {
 
 private:
     DNNL_DISALLOW_COPY_AND_ASSIGN(jit_avx512_core_bf16_bwd_data_kernel);
-    jit_generator *kernel_;
+    std::unique_ptr<jit_generator_t> kernel_;
 };
 
-struct jit_avx512_core_bf16_conv_bwd_weights_kernel_f32 : public jit_generator {
+struct jit_avx512_core_bf16_conv_bwd_weights_kernel_f32
+    : public jit_generator_t {
 
     jit_avx512_core_bf16_conv_bwd_weights_kernel_f32(
             const jit_conv_conf_t &ajcp)
-        : jit_generator(nullptr, ker_code_size, true, avx512_core_bf16)
+        : jit_generator_t(jit_name(), avx512_core_bf16)
         , jcp(ajcp)
         , bf16_emu_(nullptr) {
         if (!isa_has_bf16(jcp.isa)) {
@@ -467,7 +486,7 @@ struct jit_avx512_core_bf16_conv_bwd_weights_kernel_f32 : public jit_generator {
         }
     }
 
-    ~jit_avx512_core_bf16_conv_bwd_weights_kernel_f32() = default;
+    ~jit_avx512_core_bf16_conv_bwd_weights_kernel_f32() override = default;
 
     DECLARE_CPU_JIT_AUX_FUNCTIONS(
             jit_avx512_core_bf16_conv_bwd_weights_kernel_f32)
@@ -506,7 +525,6 @@ private:
         sizeof_cacheline = 64,
         full_spat_opt_working_set_size = 48 * 1024,
         full_spat_max_working_set_size = 128 * 1024,
-        ker_code_size = 1024 * 1024,
     };
     static const int max_ur_w;
 
@@ -546,7 +564,7 @@ private:
     reg64_t scratch = r11;
 
     inline void maybe_zero_kernel();
-    inline void get_ur_w(int &ur_w, int &ur_w_tail, int &ur_w_trips);
+    inline void get_ur_w(int &ur_w, int &ur_w_tail, int &ur_w_trips) const;
     inline void compute_oh_step_unroll_ow_icblock(int ic_block_step);
     inline void od_step_comeback_pointers();
     inline void oh_step_comeback_pointers();
@@ -579,12 +597,12 @@ private:
     inline void compute_ic_block_step_vpermw_expl(int ur_w, int pad_l,
             int pad_r, int ic_block_step, int src_offset, int kernel_offset,
             int ddst_offset, bool is_tail = false);
-    inline bool is_src_layout_nxc() {
+    inline bool is_src_layout_nxc() const {
         return jcp.uses_permw_transposition
                 && utils::one_of(jcp.src_tag, format_tag::ndhwc,
                         format_tag::nhwc, format_tag::nwc);
     }
-    inline bool is_ddst_layout_nxc() {
+    inline bool is_ddst_layout_nxc() const {
         return jcp.uses_permw_transposition
                 && utils::one_of(jcp.dst_tag, format_tag::ndhwc,
                         format_tag::nhwc, format_tag::nwc);
@@ -596,8 +614,8 @@ private:
             int &nthr_g, int &nthr_oc_b, int &nthr_ic_b);
 
     void get_w_positions(int ur_w, int pad_l, int pad_r, int i_ur, int i_kw,
-            int &iw_0, int &iw_1) {
-        auto get_w_position = [=](int idx) {
+            int &iw_0, int &iw_1) const {
+        auto get_w_position = [&](int idx) {
             int iw = i_ur + idx;
             if (iw >= ur_w) return -1;
             iw += i_kw;
@@ -632,13 +650,16 @@ private:
         return rt;
     }
 
-    inline dim_t filter_w_to_src(int kw, int ow = 0, int pad_l = 0) {
+    inline dim_t filter_w_to_src(int kw, int ow = 0, int pad_l = 0) const {
         int stride_w = jcp.transpose_src ? 1 : jcp.stride_w;
-        return kw * (jcp.dilate_w + 1) + ow * stride_w - pad_l;
+        return static_cast<dim_t>(kw) * (jcp.dilate_w + 1) + ow * stride_w
+                - pad_l;
     }
-    inline dim_t filter_h_to_src(int kh) { return kh * (jcp.dilate_h + 1); }
-    inline dim_t filter_d_to_src(int kd) {
-        return kd * (jcp.dilate_d + 1) * jcp.ih;
+    inline dim_t filter_h_to_src(int kh) const {
+        return static_cast<dim_t>(kh) * (jcp.dilate_h + 1);
+    }
+    inline dim_t filter_d_to_src(int kd) const {
+        return static_cast<dim_t>(kd) * (jcp.dilate_d + 1) * jcp.ih;
     }
 
     inline dim_t get_src_offset(dim_t ic_idx, dim_t w_idx, dim_t hd_idx = 0) {
@@ -650,9 +671,11 @@ private:
         dim_t ic = is_src_layout_nxc() ? ic_idx % jcp.ic_block : ic_idx;
         dim_t iw_str = jcp.is_1stconv || jcp.transpose_src
                 ? 1
-                : (is_src_layout_nxc() ? jcp.ngroups * jcp.ic : jcp.ic_block);
-        dim_t ihid_str
-                = jcp.tr_iw * (jcp.transpose_src ? jcp.ic_block : iw_str);
+                : (is_src_layout_nxc()
+                                ? static_cast<dim_t>(jcp.ngroups) * jcp.ic
+                                : jcp.ic_block);
+        dim_t ihid_str = static_cast<dim_t>(jcp.tr_iw)
+                * (jcp.transpose_src ? jcp.ic_block : iw_str);
         // jcp.transpose_src w_idx might be greater than jcp.tr_iw as right zero
         // padding memory is shared with left zero padding of the next block
         dim_t isp_off = hd_idx * ihid_str + w_idx * iw_str;
@@ -660,9 +683,10 @@ private:
         dim_t ic_str = jcp.transpose_src
                 ? jcp.tr_iw
                 : (jcp.is_1stconv ? full_spatial_size : 1);
-        dim_t icb_str
-                = jcp.ic_block * (is_src_layout_nxc() ? 1 : full_spatial_size);
-        return jcp.typesize_in * (isp_off + icb_str * icb + ic_str * ic);
+        dim_t icb_str = static_cast<dim_t>(jcp.ic_block)
+                * (is_src_layout_nxc() ? 1 : full_spatial_size);
+        return static_cast<dim_t>(jcp.typesize_in)
+                * (isp_off + icb_str * icb + ic_str * ic);
     }
 
     inline dim_t get_ddst_offset(dim_t w_idx, dim_t hd_idx = 0) {
@@ -675,7 +699,7 @@ private:
         return jcp.typesize_in * (w_off + hd_off);
     }
 
-    inline dim_t get_kernel_offset(int ic_idx, dim_t ksp_idx) {
+    inline dim_t get_kernel_offset(int ic_idx, dim_t ksp_idx) const {
         // Only the ic_idx index inside the block is supported,
         // ic_idx == jcp.ic_block is considered as a shift inside one block
         // and not as moving to the next ic block.
@@ -685,7 +709,7 @@ private:
                 * (ksp_idx * jcp.ic_block + ic_idx);
     }
 
-    Xbyak::Zmm get_perm_reg() {
+    Xbyak::Zmm get_perm_reg() const {
         int idx = !(jcp.uses_permw_transposition
                           && jcp.kernel_kind == expl_bcast)
                 ? 24
@@ -694,10 +718,10 @@ private:
     }
     std::unique_ptr<bf16_emulation_t> bf16_emu_;
 
-    inline int interleave_w_reorder_size(int ur_w);
+    inline int interleave_w_reorder_size(int ur_w) const;
     inline int interleave_w_reorder_bytes(int ur_w);
     inline int interleave_stack_size(int ur_w, int ic_block_step);
-    inline int permw_stack_size(int ur_w) {
+    inline int permw_stack_size(int ur_w) const {
         return (ur_w + jcp.kw - 1) * sizeof_cacheline;
     }
 
