@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -75,7 +75,8 @@ void jit_avx512_core_bf16_convolution_fwd_t::execute_forward_1d(
     // TODO: experiment with g_blocking for perf fine tuning
     int g_blocking = 1;
     int nb_groups = jcp.ngroups / g_blocking;
-    dim_t work_amount = jcp.mb * nb_groups * oc_chunks * jcp.nb_ow;
+    dim_t work_amount
+            = static_cast<dim_t>(jcp.mb) * nb_groups * oc_chunks * jcp.nb_ow;
 
     int nthr = jcp.aligned_threads ? jcp.aligned_threads : jcp.nthr;
     parallel(nthr, [&](const int ithr, const int nthr) {
@@ -128,7 +129,6 @@ void jit_avx512_core_bf16_convolution_fwd_t::execute_forward_1d(
             par_conv.bias = bias_w;
             par_conv.owb = owb;
 
-            par_conv.oc_l_off = oc_idx * (is_dst_layout_nxc ? 1 : jcp.oc_block);
             par_conv.dst_orig = dst;
             par_conv.post_ops_binary_rhs_arg_vec
                     = post_ops_binary_rhs_arg_vec.data();
@@ -176,7 +176,8 @@ void jit_avx512_core_bf16_convolution_fwd_t::execute_forward_2d(
     // TODO: experiment with g_blocking for perf fine tuning
     int g_blocking = 1;
     int nb_groups = jcp.ngroups / g_blocking;
-    dim_t work_amount = jcp.mb * nb_groups * oc_chunks * jcp.oh * jcp.nb_ow;
+    dim_t work_amount = static_cast<dim_t>(jcp.mb) * nb_groups * oc_chunks
+            * jcp.oh * jcp.nb_ow;
 
     int nthr = jcp.aligned_threads ? jcp.aligned_threads : jcp.nthr;
     parallel(nthr, [&](const int ithr, const int nthr) {
@@ -184,8 +185,10 @@ void jit_avx512_core_bf16_convolution_fwd_t::execute_forward_2d(
         balance211(work_amount, nthr, ithr, start, end);
         auto par_conv = jit_conv_call_s();
 
-        size_t src_h_stride = src_d.blk_off(0, 0, 1);
-        size_t dst_h_stride = dst_d.blk_off(0, 0, 1);
+        // The second arg in template means sub_offset0 = true
+        // See `blk_off` method definition.
+        size_t src_h_stride = src_d.blk_off<false, true>(0, 0, 1);
+        size_t dst_h_stride = dst_d.blk_off<false, true>(0, 0, 1);
         size_t wht_h_stride = wht_blk_off(weights_d, 0, 0, 0, 1);
 
         int n {0}, gg {0}, occ {0}, oh_s {0}, owb {0};
@@ -251,8 +254,6 @@ void jit_avx512_core_bf16_convolution_fwd_t::execute_forward_2d(
                 par_conv.kh_padding = kh_padding;
                 par_conv.owb = owb;
 
-                par_conv.oc_l_off
-                        = oc_idx * (is_dst_layout_nxc ? 1 : jcp.oc_block);
                 par_conv.dst_orig = dst;
                 par_conv.post_ops_binary_rhs_arg_vec
                         = post_ops_binary_rhs_arg_vec.data();
@@ -301,8 +302,8 @@ void jit_avx512_core_bf16_convolution_fwd_t::execute_forward_3d(
     // TODO: experiment with g_blocking for perf fine tuning
     int g_blocking = 1;
     int nb_groups = jcp.ngroups / g_blocking;
-    dim_t work_amount
-            = jcp.mb * nb_groups * oc_chunks * jcp.od * jcp.oh * jcp.nb_ow;
+    dim_t work_amount = static_cast<dim_t>(jcp.mb) * nb_groups * oc_chunks
+            * jcp.od * jcp.oh * jcp.nb_ow;
 
     int nthr = jcp.aligned_threads ? jcp.aligned_threads : jcp.nthr;
     parallel(nthr, [&](const int ithr, const int nthr) {
@@ -310,9 +311,11 @@ void jit_avx512_core_bf16_convolution_fwd_t::execute_forward_3d(
         balance211(work_amount, nthr, ithr, start, end);
         auto par_conv = jit_conv_call_s();
 
-        size_t src_d_stride = src_d.blk_off(0, 0, 1);
-        size_t src_h_stride = src_d.blk_off(0, 0, 0, 1);
-        size_t dst_h_stride = dst_d.blk_off(0, 0, 0, 1);
+        // The second arg in template means sub_offset0 = true
+        // See `blk_off` method definition.
+        size_t src_d_stride = src_d.blk_off<false, true>(0, 0, 1);
+        size_t src_h_stride = src_d.blk_off<false, true>(0, 0, 0, 1);
+        size_t dst_h_stride = dst_d.blk_off<false, true>(0, 0, 0, 1);
         size_t wht_d_stride = wht_blk_off(weights_d, 0, 0, 0, 1);
         size_t wht_h_stride = wht_blk_off(weights_d, 0, 0, 0, 0, 1);
 
@@ -390,8 +393,6 @@ void jit_avx512_core_bf16_convolution_fwd_t::execute_forward_3d(
                 par_conv.kd_padding = kd_padding;
                 par_conv.owb = owb;
 
-                par_conv.oc_l_off
-                        = oc_idx * (is_dst_layout_nxc ? 1 : jcp.oc_block);
                 par_conv.dst_orig = dst;
                 par_conv.post_ops_binary_rhs_arg_vec
                         = post_ops_binary_rhs_arg_vec.data();
@@ -439,8 +440,10 @@ void jit_avx512_core_bf16_convolution_bwd_data_t ::execute_backward_data_3d(
 
         auto par_conv = jit_conv_call_s();
 
-        size_t diff_src_h_stride = diff_src_d.blk_off(0, 0, 0, 1);
-        size_t diff_dst_h_stride = diff_dst_d.blk_off(0, 0, 0, 1);
+        // The second arg in template means sub_offset0 = true
+        // See `blk_off` method definition.
+        size_t diff_src_h_stride = diff_src_d.blk_off<false, true>(0, 0, 0, 1);
+        size_t diff_dst_h_stride = diff_dst_d.blk_off<false, true>(0, 0, 0, 1);
         size_t wht_h_stride = wht_blk_off(weights_d, 0, 0, 0, 0, 1);
 
         bool is_fast_path_d = jcp.dilate_d == 0 && jcp.stride_d == 1;
@@ -604,8 +607,11 @@ void jit_avx512_core_bf16_convolution_bwd_data_t ::execute_backward_data(
         balance211(work_amount, nthr, ithr, start, end);
 
         auto par_conv = jit_conv_call_s();
-        size_t diff_src_h_stride = diff_src_d.blk_off(0, 0, 1);
-        size_t diff_dst_h_stride = diff_dst_d.blk_off(0, 0, 1);
+
+        // The second arg in template means sub_offset0 = true
+        // See `blk_off` method definition.
+        size_t diff_src_h_stride = diff_src_d.blk_off<false, true>(0, 0, 1);
+        size_t diff_dst_h_stride = diff_dst_d.blk_off<false, true>(0, 0, 1);
         size_t wht_h_stride = wht_blk_off(weights_d, 0, 0, 0, 1);
 
         bool is_fast_path = jcp.dilate_h == 0 && jcp.stride_h == 1;
@@ -779,10 +785,11 @@ struct jit_avx512_core_bf16_convolution_bwd_weights_t ::thread_info_t {
 
     thread_info_t(const jit_avx512_core_bf16_convolution_bwd_weights_t *self,
             const exec_ctx_t &ctx, int ithr)
-        : scratchpad(ctx.get_scratchpad_grantor()), ithr(ithr) {
-        diff_dst = CTX_IN_MEM(const diff_dst_data_t *, DNNL_ARG_DIFF_DST);
-        src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
-        diff_weights = CTX_OUT_MEM(void *, DNNL_ARG_DIFF_WEIGHTS);
+        : src(CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC))
+        , diff_dst(CTX_IN_MEM(const diff_dst_data_t *, DNNL_ARG_DIFF_DST))
+        , diff_weights(CTX_OUT_MEM(void *, DNNL_ARG_DIFF_WEIGHTS))
+        , scratchpad(ctx.get_scratchpad_grantor())
+        , ithr(ithr) {
 
         const auto &jcp = self->kernel_->jcp;
         diff_bias = self->pd()->with_bias()
@@ -1217,8 +1224,9 @@ void jit_avx512_core_bf16_convolution_bwd_weights_t ::compute_diff_weights_2d(
             }
 
             p.filt = diff_wei + wht_blk_off(diff_weights_d, g, oc_b, ic_b);
-            p.bias = diff_bias + g * rnd_up(jcp.oc, jcp.oc_block)
-                    + oc_b * jcp.oc_block;
+            p.bias = diff_bias ? diff_bias + g * rnd_up(jcp.oc, jcp.oc_block)
+                            + oc_b * jcp.oc_block
+                               : nullptr;
             p.channel = (start == ti->img_start) && (ohb_s == oh_s);
             p.reduce_work = ic_to_compute;
             p.load_work = oc_to_compute;
@@ -1464,8 +1472,9 @@ void jit_avx512_core_bf16_convolution_bwd_weights_t ::compute_diff_weights_3d(
             }
 
             p.filt = diff_wei + wht_blk_off(diff_weights_d, g, oc_b, ic_b);
-            p.bias = diff_bias + g * rnd_up(jcp.oc, jcp.oc_block)
-                    + oc_b * jcp.oc_block;
+            p.bias = diff_bias ? diff_bias + g * rnd_up(jcp.oc, jcp.oc_block)
+                            + oc_b * jcp.oc_block
+                               : nullptr;
             p.channel = (start == ti->img_start) && (odb_s == od_s);
             p.reduce_work = ic_to_compute;
             p.load_work = oc_to_compute;
@@ -1492,8 +1501,9 @@ void jit_avx512_core_bf16_convolution_bwd_weights_t ::compute_diff_weights(
             jcp.src_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
     const bool is_ddst_layout_nxc = utils::one_of(
             jcp.dst_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
-    const int wei_size = jcp.ngroups * jcp.nb_oc * jcp.oc_block * jcp.nb_ic
-            * jcp.ic_block * jcp.kh * jcp.kw * jcp.kd;
+    const size_t wei_size = static_cast<size_t>(jcp.ngroups) * jcp.nb_oc
+            * jcp.oc_block * jcp.nb_ic * jcp.ic_block * jcp.kh * jcp.kw
+            * jcp.kd;
     const int bias_buf_size = jcp.ngroups * jcp.nb_oc * jcp.oc_block;
 
     float *diff_wei;
@@ -1545,7 +1555,7 @@ void jit_avx512_core_bf16_convolution_bwd_weights_t ::compute_diff_weights(
     auto uker_trans = [&](int img, int g = 0, int ic_b = 0) {
         int j {0}, d {0};
         int my_work = jcp.ih * jcp.id;
-        int ic;
+        dim_t ic;
         int icb_start = ic_b;
         if (jcp.global_transpose) {
             const int work_amount = is_src_layout_nxc
@@ -1573,11 +1583,13 @@ void jit_avx512_core_bf16_convolution_bwd_weights_t ::compute_diff_weights(
             g += ti->g_start;
             ic_b += ti->ic_b_start;
             icb_start = ic_b;
-            ic = is_src_layout_nxc ? g * jcp.ic + ic_b * jcp.ic_block
-                                   : g * jcp.nb_ic + ic_b;
+            ic = is_src_layout_nxc ? static_cast<dim_t>(g) * jcp.ic
+                            + static_cast<dim_t>(ic_b) * jcp.ic_block
+                                   : static_cast<dim_t>(g) * jcp.nb_ic + ic_b;
         } else {
-            ic = is_src_layout_nxc ? g * jcp.ic + ic_b * jcp.ic_block
-                                   : g * jcp.nb_ic + ic_b;
+            ic = is_src_layout_nxc ? static_cast<dim_t>(g) * jcp.ic
+                            + static_cast<dim_t>(ic_b) * jcp.ic_block
+                                   : static_cast<dim_t>(g) * jcp.nb_ic + ic_b;
             g = 0;
             ic_b = 0;
         }
@@ -1585,7 +1597,9 @@ void jit_avx512_core_bf16_convolution_bwd_weights_t ::compute_diff_weights(
         const auto local_gwork = need_local_gwork ? ti->g_work : 1;
 
         for (int gg = g; gg < g + local_gwork; ++gg) {
-            if (need_local_gwork) ic = gg * jcp.ic + ic_b * jcp.ic_block;
+            if (need_local_gwork)
+                ic = static_cast<dim_t>(gg) * jcp.ic
+                        + static_cast<dim_t>(ic_b) * jcp.ic_block;
             src_data_t *tr_src = (jcp.ndims == 5)
                     ? &ti->tr_src[tr_src_off_3d(gg, ic_b, d, j)]
                     : &ti->tr_src[tr_src_off(gg, ic_b, j)];
@@ -1757,8 +1771,9 @@ void jit_avx512_core_bf16_convolution_bwd_weights_t ::compute_diff_weights(
             }
 
             p.filt = diff_wei + wht_blk_off(diff_weights_d, g, oc_b, ic_b);
-            p.bias = diff_bias + g * rnd_up(jcp.oc, jcp.oc_block)
-                    + oc_b * jcp.oc_block;
+            p.bias = diff_bias ? diff_bias + g * rnd_up(jcp.oc, jcp.oc_block)
+                            + oc_b * jcp.oc_block
+                               : nullptr;
             p.channel = (img == ti->img_start);
             p.flags = 0 | (ic_b == 0 ? FLAG_IC_FIRST : 0);
             p.reduce_work = ic_to_compute;
@@ -1774,8 +1789,9 @@ void jit_avx512_core_bf16_convolution_bwd_weights_t ::
     const memory_desc_wrapper diff_weights_d(pd()->diff_weights_md(0));
 
     const auto &jcp = kernel_->jcp;
-    const int wei_size = jcp.ngroups * jcp.nb_oc * jcp.oc_block * jcp.nb_ic
-            * jcp.ic_block * jcp.kh * jcp.kw * ((jcp.ndims == 5) ? jcp.kd : 1);
+    const size_t wei_size = static_cast<size_t>(jcp.ngroups) * jcp.nb_oc
+            * jcp.oc_block * jcp.nb_ic * jcp.ic_block * jcp.kh * jcp.kw
+            * ((jcp.ndims == 5) ? jcp.kd : 1);
 
     const bool is_bf16_out = diff_weights_d.data_type() == data_type::bf16;
     const bool is_bf16_bias = jcp.with_bias && jcp.bia_dt == data_type::bf16;
