@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -28,7 +28,7 @@
 #include "cpu/x64/cpu_reducer.hpp"
 
 #include "cpu/x64/jit_avx512_core_bf16_conv_kernel.hpp"
-#include "cpu/x64/jit_transpose_src_utils.hpp"
+#include "cpu/x64/jit_transpose_utils.hpp"
 
 namespace dnnl {
 namespace impl {
@@ -37,35 +37,39 @@ namespace x64 {
 
 struct jit_avx512_core_bf16_convolution_fwd_t : public primitive_t {
     struct pd_t : public cpu_convolution_fwd_pd_t {
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const typename pd_t::base_class *hint_fwd_pd)
-            : cpu_convolution_fwd_pd_t(adesc, attr, hint_fwd_pd), jcp_() {}
+        using cpu_convolution_fwd_pd_t::cpu_convolution_fwd_pd_t;
 
         DECLARE_COMMON_PD_T(JIT_IMPL_NAME_HELPER("jit_bf16:", jcp_.isa, ""),
                 jit_avx512_core_bf16_convolution_fwd_t);
 
         status_t init(engine_t *engine) {
-            bool ok = true && mayiuse(avx512_core) && is_fwd()
-                    && set_default_alg_kind(alg_kind::convolution_direct)
-                    && (expect_data_types(data_type::bf16, data_type::bf16,
-                                data_type::undef, data_type::bf16,
-                                data_type::undef)
-                            || expect_data_types(data_type::bf16,
-                                    data_type::bf16, data_type::undef,
-                                    data_type::f32, data_type::undef))
-                    && IMPLICATION(with_bias(),
-                            utils::one_of(weights_md(1)->data_type,
-                                    data_type::f32, data_type::bf16))
-                    && attr()->has_default_values(
-                            primitive_attr_t::skip_mask_t::post_ops,
-                            dst_md()->data_type)
-                    && !has_zero_dim_memory();
-            if (!ok) return status::unimplemented;
+            using namespace data_type;
+            // disabling verbose dispatch messages for unsupported isa for better readability
+            if (!mayiuse(avx512_core)) return status::unimplemented;
 
-            status_t status = jit_avx512_core_bf16_fwd_kernel::init_conf(jcp_,
-                    *desc(), src_md_, weights_md_, dst_md_, bias_md_, *attr(),
-                    dnnl_get_max_threads());
-            if (status != status::success) return status::unimplemented;
+            VDISPATCH_CONV(is_fwd(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_CONV(
+                    (expect_data_types(bf16, bf16, data_type::undef, bf16,
+                             data_type::undef)
+                            || expect_data_types(bf16, bf16, data_type::undef,
+                                    f32, data_type::undef)),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(set_default_alg_kind(alg_kind::convolution_direct),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_CONV(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+            VDISPATCH_CONV(attr()->has_default_values(
+                                   primitive_attr_t::skip_mask_t::post_ops,
+                                   dst_md()->data_type),
+                    VERBOSE_UNSUPPORTED_POSTOP);
+            VDISPATCH_CONV(
+                    IMPLICATION(with_bias(),
+                            utils::one_of(weights_md(1)->data_type, f32, bf16)),
+                    VERBOSE_UNSUPPORTED_BIAS_CFG);
+
+            // TODO: make `init_conf` assign initialized object to `jcp_`
+            CHECK(jit_avx512_core_bf16_fwd_kernel::init_conf(jcp_, *desc(),
+                    src_md_, weights_md_, dst_md_, bias_md_, attr_,
+                    dnnl_get_max_threads()));
 
             auto scratchpad = scratchpad_registry().registrar();
             jit_avx512_core_bf16_fwd_kernel::init_scratchpad(scratchpad, jcp_);
@@ -73,14 +77,14 @@ struct jit_avx512_core_bf16_convolution_fwd_t : public primitive_t {
             return status::success;
         }
 
-        jit_conv_conf_t jcp_;
+        jit_conv_conf_t jcp_ = utils::zero<decltype(jcp_)>();
     };
 
     jit_avx512_core_bf16_convolution_fwd_t(const pd_t *apd)
         : primitive_t(apd) {}
 
-    typedef typename prec_traits<data_type::bf16>::type src_data_t;
-    typedef typename prec_traits<data_type::bf16>::type wei_data_t;
+    using src_data_t = typename prec_traits_t<data_type::bf16>::type;
+    using wei_data_t = typename prec_traits_t<data_type::bf16>::type;
 
     status_t init(engine_t *engine) override {
         CHECK(safe_ptr_assign(kernel_,
@@ -116,40 +120,45 @@ private:
 
 struct jit_avx512_core_bf16_convolution_bwd_data_t : public primitive_t {
     struct pd_t : public cpu_convolution_bwd_data_pd_t {
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const convolution_fwd_pd_t *hint_fwd_pd)
-            : cpu_convolution_bwd_data_pd_t(adesc, attr, hint_fwd_pd), jcp_() {}
+        using cpu_convolution_bwd_data_pd_t::cpu_convolution_bwd_data_pd_t;
 
         DECLARE_COMMON_PD_T(JIT_IMPL_NAME_HELPER("jit_bf16:", jcp_.isa, ""),
                 jit_avx512_core_bf16_convolution_bwd_data_t);
 
         status_t init(engine_t *engine) {
             using namespace prop_kind;
-            bool ok = true && mayiuse(avx512_core) && is_bwd_d()
-                    && set_default_alg_kind(alg_kind::convolution_direct)
-                    && (expect_data_types(data_type::f32, data_type::bf16,
-                                data_type::undef, data_type::bf16,
-                                data_type::undef)
-                            || expect_data_types(data_type::bf16,
-                                    data_type::bf16, data_type::undef,
-                                    data_type::bf16, data_type::undef))
-                    && attr()->has_default_values() && !has_zero_dim_memory();
-            if (!ok) return status::unimplemented;
+            using namespace data_type;
+            // disabling verbose dispatch messages for unsupported isa for better readability
+            if (!mayiuse(avx512_core)) return status::unimplemented;
 
-            status_t status = jit_avx512_core_bf16_bwd_data_kernel::init_conf(
-                    jcp_, *desc(), diff_src_md_, weights_md_, diff_dst_md_,
-                    dnnl_get_max_threads());
-            return status;
+            VDISPATCH_CONV(is_bwd_d(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_CONV(
+                    (expect_data_types(f32, bf16, data_type::undef, bf16,
+                             data_type::undef)
+                            || expect_data_types(bf16, bf16, data_type::undef,
+                                    bf16, data_type::undef)),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(set_default_alg_kind(alg_kind::convolution_direct),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_CONV(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+            VDISPATCH_CONV(
+                    attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
+
+            // TODO: make `init_conf` assign initialized object to `jcp_`
+            CHECK(jit_avx512_core_bf16_bwd_data_kernel::init_conf(jcp_, *desc(),
+                    diff_src_md_, weights_md_, diff_dst_md_,
+                    dnnl_get_max_threads()));
+            return status::success;
         }
 
-        jit_conv_conf_t jcp_;
+        jit_conv_conf_t jcp_ = utils::zero<decltype(jcp_)>();
     };
 
     jit_avx512_core_bf16_convolution_bwd_data_t(const pd_t *apd)
         : primitive_t(apd) {}
 
-    typedef typename prec_traits<data_type::bf16>::type diff_dst_data_t;
-    typedef typename prec_traits<data_type::bf16>::type wei_data_t;
+    using diff_dst_data_t = typename prec_traits_t<data_type::bf16>::type;
+    using wei_data_t = typename prec_traits_t<data_type::bf16>::type;
 
     status_t init(engine_t *engine) override {
         CHECK(safe_ptr_assign(
@@ -177,50 +186,54 @@ private:
 
 struct jit_avx512_core_bf16_convolution_bwd_weights_t : public primitive_t {
     struct pd_t : public cpu_convolution_bwd_weights_pd_t {
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const convolution_fwd_pd_t *hint_fwd_pd)
-            : cpu_convolution_bwd_weights_pd_t(adesc, attr, hint_fwd_pd)
-            , jcp_() {}
+        using cpu_convolution_bwd_weights_pd_t::
+                cpu_convolution_bwd_weights_pd_t;
 
         DECLARE_COMMON_PD_T(JIT_IMPL_NAME_HELPER("jit_bf16:", jcp_.isa, ""),
                 jit_avx512_core_bf16_convolution_bwd_weights_t);
 
         status_t init(engine_t *engine) {
-            bool ok = true && mayiuse(avx512_core) && is_bwd_w()
-                    && set_default_alg_kind(alg_kind::convolution_direct)
-                    && (expect_data_types(data_type::bf16, data_type::bf16,
-                                data_type::undef, data_type::bf16,
-                                data_type::undef)
-                            || expect_data_types(data_type::bf16,
-                                    data_type::f32, data_type::undef,
-                                    data_type::bf16, data_type::undef))
-                    && IMPLICATION(with_bias(),
-                            utils::one_of(diff_bias_md_.data_type,
-                                    data_type::f32, data_type::bf16))
-                    && attr()->has_default_values() && !has_zero_dim_memory();
-            if (!ok) return status::unimplemented;
+            using namespace data_type;
+            // disabling verbose dispatch messages for unsupported isa for better readability
+            if (!mayiuse(avx512_core)) return status::unimplemented;
 
-            status_t status = jit_avx512_core_bf16_conv_bwd_weights_kernel_f32::
-                    init_conf(jcp_, *desc(), src_md_, diff_weights_md_,
-                            diff_bias_md_, diff_dst_md_,
-                            dnnl_get_max_threads());
-            if (status != status::success) return status;
+            VDISPATCH_CONV(is_bwd_w(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_CONV(
+                    (expect_data_types(bf16, bf16, data_type::undef, bf16,
+                             data_type::undef)
+                            || expect_data_types(bf16, f32, data_type::undef,
+                                    bf16, data_type::undef)),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(set_default_alg_kind(alg_kind::convolution_direct),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_CONV(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+            VDISPATCH_CONV(
+                    IMPLICATION(with_bias(),
+                            utils::one_of(diff_bias_md_.data_type, f32, bf16)),
+                    VERBOSE_UNSUPPORTED_BIAS_CFG);
+            VDISPATCH_CONV(
+                    attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
+
+            // TODO: make `init_conf` assign initialized object to `jcp_`
+            CHECK(jit_avx512_core_bf16_conv_bwd_weights_kernel_f32::init_conf(
+                    jcp_, *desc(), src_md_, diff_weights_md_, diff_bias_md_,
+                    diff_dst_md_, dnnl_get_max_threads()));
 
             auto scratchpad = scratchpad_registry().registrar();
             jit_avx512_core_bf16_conv_bwd_weights_kernel_f32::init_scratchpad(
                     scratchpad, jcp_);
 
-            return status;
+            return status::success;
         }
 
-        jit_conv_conf_t jcp_;
+        jit_conv_conf_t jcp_ = utils::zero<decltype(jcp_)>();
     };
 
     jit_avx512_core_bf16_convolution_bwd_weights_t(const pd_t *apd)
         : primitive_t(apd) {}
 
-    typedef typename prec_traits<data_type::bf16>::type src_data_t;
-    typedef typename prec_traits<data_type::bf16>::type diff_dst_data_t;
+    using src_data_t = typename prec_traits_t<data_type::bf16>::type;
+    using diff_dst_data_t = typename prec_traits_t<data_type::bf16>::type;
 
     status_t init(engine_t *engine) override;
 
