@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -37,60 +37,78 @@ namespace impl {
 namespace cpu {
 namespace x64 {
 
-template <impl::data_type_t src_type, impl::data_type_t dst_type>
 struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
     struct pd_t : public cpu_convolution_fwd_pd_t {
-        using dw_conv_pd_type = cpu_convolution_fwd_pd_t;
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const typename pd_t::base_class *hint_fwd_pd)
-            : cpu_convolution_fwd_pd_t(adesc, attr, hint_fwd_pd)
-            , jcp_()
-            , rtus_()
-            , jcp_dw_(nullptr) {}
+        using cpu_convolution_fwd_pd_t::cpu_convolution_fwd_pd_t;
 
         pd_t(const pd_t &other) : cpu_convolution_fwd_pd_t(other) {
             if (copy(other) != status::success) is_initialized_ = false;
         }
 
-        DECLARE_COMMON_PD_T(JIT_IMPL_NAME_HELPER("jit_int8_1x1:",
-                                    ((jcp_.ver == ver_vnni) ? avx512_core_vnni
-                                                            : avx512_core),
-                                    ""),
+        DECLARE_COMMON_PD_T(
+                JIT_IMPL_NAME_HELPER("jit_int8_1x1:",
+                        ((jcp_.has_vnni) ? avx512_core_vnni : avx512_core), ""),
                 jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t);
 
         status_t init(engine_t *engine) {
+            using namespace data_type;
             using smask_t = primitive_attr_t::skip_mask_t;
-            bool ok = true && is_fwd()
-                    && set_default_alg_kind(alg_kind::convolution_direct)
-                    && expect_data_types(src_type, data_type::s8,
-                            data_type::undef, dst_type, data_type::s32)
-                    && IMPLICATION(with_bias(),
-                            utils::one_of(desc()->bias_desc.data_type,
-                                    data_type::f32, data_type::s32,
-                                    data_type::s8, data_type::u8))
-                    && attr()->has_default_values(smask_t::oscale
-                                    | smask_t::zero_points_runtime
-                                    | smask_t::post_ops,
-                            dst_type)
-                    && !has_zero_dim_memory() && zero_points_ok()
-                    && set_default_formats_common(
-                            dat_tag(), format_tag::any, dat_tag());
+            VDISPATCH_CONV(is_fwd(), VERBOSE_BAD_PROPKIND);
+
+            VDISPATCH_CONV(utils::one_of(src_md(0)->data_type, s8, u8),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(
+                    weights_md(0)->data_type == s8, VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(
+                    IMPLICATION(with_bias(), weights_md(1)->data_type == f32),
+                    VERBOSE_UNSUPPORTED_BIAS_CFG);
+            VDISPATCH_CONV(
+                    utils::one_of(dst_md(0)->data_type, f32, s32, s8, u8, bf16),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(
+                    desc()->accum_data_type == s32, VERBOSE_UNSUPPORTED_DT);
+
+            VDISPATCH_CONV(set_default_alg_kind(alg_kind::convolution_direct),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_CONV(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+
+            VDISPATCH_CONV(
+                    attr()->has_default_values(smask_t::scales
+                                    | smask_t::zero_points | smask_t::post_ops
+                                    | smask_t::sum_dt,
+                            dst_md(0)->data_type),
+                    VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_CONV(attr()->scales_.has_default_values({DNNL_ARG_SRC,
+                                   DNNL_ARG_WEIGHTS, DNNL_ARG_DST,
+                                   DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS,
+                                   DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_DST}),
+                    VERBOSE_UNSUPPORTED_ATTR);
+
+            VDISPATCH_CONV(set_default_formats_common(
+                                   dat_tag(), format_tag::any, dat_tag()),
+                    VERBOSE_UNSUPPORTED_TAG);
+            VDISPATCH_CONV(
+                    attr_.set_default_formats(dst_md(0)) == status::success,
+                    VERBOSE_UNSUPPORTED_POSTOP);
+            VDISPATCH_CONV(attr()->post_ops_.check_sum_consistency(
+                                   dst_md(0)->data_type,
+                                   /* is_int8 */ true),
+                    VERBOSE_UNSUPPORTED_POSTOP);
+
+            VDISPATCH_CONV(attr_scales_ok(), VERBOSE_UNSUPPORTED_SCALES_CFG);
+            VDISPATCH_CONV(zero_points_ok(), VERBOSE_UNSUPPORTED_ZP_CFG);
 
-            if (!ok) return status::unimplemented;
             const convolution_desc_t *conv_d = desc();
             const memory_desc_t *src_d = src_md();
+
+            // TODO: make `rtus_prepare` assign initialized object to `rtus_`
             rtus_prepare(this, conv_d, src_d, dst_md(), weights_md());
 
-            status_t status
-                    = jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(jcp_,
-                            *conv_d, src_d, weights_md_, dst_md_, bias_md_,
-                            *attr(), dnnl_get_max_threads(), rtus_.reduce_src_);
-            if (status != status::success) return status;
-
-            if (jcp_.with_dw_conv) {
-                status = depthwise_po_init(engine);
-                if (status != status::success) return status;
-            }
+            // TODO: make `init_conf` assign initialized object to `jcp_`
+            CHECK(jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(jcp_,
+                    *conv_d, src_d, weights_md_, dst_md_, bias_md_, *attr(),
+                    dnnl_get_max_threads(), rtus_.reduce_src_));
+            if (jcp_.with_dw_conv) CHECK(depthwise_po_init(engine));
 
             auto scratchpad = scratchpad_registry().registrar();
             jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_scratchpad(
@@ -101,13 +119,23 @@ struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
             return status::success;
         }
 
-        const memory_desc_t *dst_md(int index = 0) const override {
-            return jcp_.with_dw_conv ? dw_conv_pd_->dst_md(index) : &dst_md_;
+        const memory_desc_t *dst_1x1_md(int index = 0) const {
+            return cpu_convolution_fwd_pd_t::dst_md(index);
         }
 
-        const memory_desc_t *arg_md(int index = 0) const override {
-            if (jcp_.with_dw_conv) {
-                switch (index) {
+        const memory_desc_t *dst_md(
+                int index = 0, bool user_input = false) const override {
+            return dw_conv_pd_ && jcp_.with_dw_conv
+                    ? dw_conv_pd_->dst_md(index, user_input)
+                    : cpu_convolution_fwd_pd_t::dst_md(index, user_input);
+        }
+
+        const memory_desc_t *arg_md(
+                int arg, bool user_input = false) const override {
+            if (dw_conv_pd_ && jcp_.with_dw_conv) {
+                switch (arg) {
+                    case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_SRC:
+                        return cpu_convolution_fwd_pd_t::dst_md(0, user_input);
                     case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS:
                         return dw_conv_pd_->weights_md(0);
                     case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS:
@@ -115,27 +143,26 @@ struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
                     default: break;
                 }
             }
-            return convolution_fwd_pd_t::arg_md(index);
+            return convolution_fwd_pd_t::arg_md(arg, user_input);
         }
 
         arg_usage_t arg_usage(int arg) const override {
             if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS))
                 return arg_usage_t::input;
 
-            if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS)
-                    && attr_post_op_dw_inputs() > 1)
-                return arg_usage_t::input;
+            if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS))
+                return attr_post_op_dw_inputs() > 1 ? arg_usage_t::input
+                                                    : arg_usage_t::unused;
 
             return convolution_fwd_pd_t::arg_usage(arg);
         }
 
-        jit_1x1_conv_conf_t jcp_;
-        reduce_to_unit_stride_t rtus_;
-        jit_conv_conf_t *jcp_dw_; // doesn't own a resource
+        jit_1x1_conv_conf_t jcp_ = utils::zero<decltype(jcp_)>();
+        reduce_to_unit_stride_t rtus_ = utils::zero<decltype(rtus_)>();
+        jit_conv_conf_t *jcp_dw_ = nullptr; // doesn't own a resource
         std::unique_ptr<cpu_convolution_fwd_pd_t> dw_conv_pd_;
-        template <data_type_t sdt, data_type_t ddt>
-        using dw_pd_t = typename jit_avx512_core_x8s8s32x_convolution_fwd_t<sdt,
-                ddt>::pd_t;
+        using dw_pd_t =
+                typename jit_avx512_core_x8s8s32x_convolution_fwd_t::pd_t;
 
     protected:
         format_tag_t dat_tag() const {
@@ -144,13 +171,20 @@ struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
         }
 
         bool zero_points_ok() const {
-            using namespace data_type;
-            int mask_src = 0, mask_dst = 0;
-            attr()->zero_points_.get(DNNL_ARG_SRC, nullptr, &mask_src, nullptr);
-            attr()->zero_points_.get(DNNL_ARG_DST, nullptr, &mask_dst, nullptr);
-            return attr()->zero_points_.has_default_values(DNNL_ARG_WEIGHTS)
-                    && utils::one_of(mask_src, 0, 0x1, 0x3)
-                    && utils::one_of(mask_dst, 0, 0x1, 0x3);
+            const auto &zp = attr()->zero_points_;
+
+            if (!zp.has_default_values(DNNL_ARG_SRC)) {
+                int mask_src = zp.get_mask(DNNL_ARG_SRC);
+                const bool ok = mask_src == 0;
+                if (!ok) return false;
+            }
+            if (!zp.has_default_values(DNNL_ARG_DST)) {
+                int mask_dst = zp.get_mask(DNNL_ARG_DST);
+                const bool ok = mask_dst == 0;
+                if (!ok) return false;
+            }
+
+            return zp.has_default_values(DNNL_ARG_WEIGHTS);
         }
 
         status_t copy(const pd_t &other) {
@@ -162,34 +196,8 @@ struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
                 dw_conv_pd_.reset(static_cast<cpu_convolution_fwd_pd_t *>(
                         other.dw_conv_pd_->clone()));
                 if (!dw_conv_pd_) return status::out_of_memory;
-#define CASE(sdt, ddt) \
-    case ddt: \
-        jcp_dw_ = &( \
-                static_cast<dw_pd_t<sdt, ddt> *>(dw_conv_pd_.get())->jcp_); \
-        break;
 
-                auto dw_dst_dt = dw_conv_pd_->dst_md()->data_type;
-                if (jcp_.dst_dt == data_type::u8) {
-                    switch (dw_dst_dt) {
-                        CASE(data_type::u8, data_type::u8);
-                        CASE(data_type::u8, data_type::s8);
-                        CASE(data_type::u8, data_type::s32);
-                        CASE(data_type::u8, data_type::f32);
-                        default: assert(!"unreachable");
-                    }
-                } else if (jcp_.dst_dt == data_type::s8) {
-                    switch (dw_dst_dt) {
-                        CASE(data_type::s8, data_type::u8);
-                        CASE(data_type::s8, data_type::s8);
-                        CASE(data_type::s8, data_type::s32);
-                        CASE(data_type::s8, data_type::f32);
-                        default: assert(!"unreachable");
-                    }
-                } else {
-                    assert(!"unreachable");
-                }
-
-#undef CASE
+                jcp_dw_ = &(static_cast<dw_pd_t *>(dw_conv_pd_.get())->jcp_);
             }
             return status::success;
         }
@@ -199,7 +207,6 @@ struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
             auto &jcp_1x1 = jcp_;
             primitive_attr_t attr_1x1(*attr());
             if (!attr_1x1.is_initialized()) return status::out_of_memory;
-            attr_1x1.set_scratchpad_mode(scratchpad_mode::user);
 
             const auto &src_md = dst_md_;
             const memory_desc_wrapper src_d(src_md);
@@ -214,16 +221,22 @@ struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
             // for 1x1: Check that no better ISA is available.
             // for dw: Always fuse with same ISA.
             // Caveat: May be a better dw conv exists.
+            VDISPATCH_CONV_IC(!mayiuse(avx512_core_amx),
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL, "higher ISA is supported");
 
-            bool ok = !mayiuse(avx512_core_bf16_amx_int8)
-                    && (attr_1x1.post_ops_.find(primitive_kind::sum) == -1)
-                    // TODO: Below may be further tuned.
-                    && (l2_cache < src_d.size())
-                    // load_grp_count check can be redundant due to l2 check
-                    // above. Adding it explicitly as the current driver doesn't
-                    // work if this condition fails.
-                    && (jcp_1x1.load_grp_count < 2);
-            if (!ok) return status::unimplemented;
+            VDISPATCH_CONV_IC(
+                    attr_1x1.post_ops_.find(primitive_kind::sum) == -1,
+                    VERBOSE_UNSUPPORTED_FEATURE, "unsupported sum post-op");
+
+            // TODO: Below may be further tuned.
+            VDISPATCH_CONV_IC(l2_cache * 2 < src_d.size(),
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL, "cache size check failed");
+
+            // load_grp_count check can be redundant due to l2 check
+            // above. Adding it explicitly as the current driver doesn't
+            // work if this condition fails.
+            VDISPATCH_CONV_IC(jcp_1x1.load_grp_count < 2,
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL, "load group count > 1");
 
             int dw_po_index
                     = attr_1x1.post_ops_.find(primitive_kind::convolution);
@@ -233,43 +246,23 @@ struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
             CHECK(get_depthwise_conv_desc(
                     cd_dw, src_md, attr_1x1, attr_dw, dw_po_index));
 
-            auto dw_dst_dt = cd_dw.dst_desc.data_type;
+            auto fusable_pd
+                    = make_unique_pd<dw_pd_t>(&cd_dw, &attr_dw, nullptr);
+            CHECK(fusable_pd->init(engine));
+            jcp_dw_ = &(fusable_pd->jcp_);
+            dw_conv_pd_ = std::move(fusable_pd);
 
-#define CASE(sdt, ddt) \
-    case ddt: { \
-        std::unique_ptr<dw_pd_t<sdt, ddt>> fusable_pd( \
-                new dw_pd_t<sdt, ddt>(&cd_dw, &attr_dw, nullptr)); \
-        CHECK(fusable_pd->init(engine)); \
-        jcp_dw_ = &(fusable_pd->jcp_); \
-        dw_conv_pd_ = std::move(fusable_pd); \
-        break; \
-    }
-            if (jcp_1x1.dst_dt == data_type::u8) {
-                switch (dw_dst_dt) {
-                    CASE(data_type::u8, data_type::u8);
-                    CASE(data_type::u8, data_type::s8);
-                    CASE(data_type::u8, data_type::s32);
-                    CASE(data_type::u8, data_type::f32);
-                    default: return status::unimplemented;
-                }
-            } else if (jcp_1x1.dst_dt == data_type::s8) {
-                switch (dw_dst_dt) {
-                    CASE(data_type::s8, data_type::u8);
-                    CASE(data_type::s8, data_type::s8);
-                    CASE(data_type::s8, data_type::s32);
-                    CASE(data_type::s8, data_type::f32);
-                    default: return status::unimplemented;
-                }
-            } else
-                return status::unimplemented;
-#undef CASE
-
-            ok = true
-                    && (dnnl_memory_desc_equal(&src_md, dw_conv_pd_->src_md(0)))
-                    && (jcp_1x1.oc_without_padding % jcp_1x1.oc_block == 0)
-                    && IMPLICATION(jcp_dw_->ow_block,
-                            jcp_dw_->ow_block == jcp_dw_->ow);
-            if (!ok) return status::unimplemented;
+            VDISPATCH_CONV_IC(
+                    dnnl_memory_desc_equal(&src_md, dw_conv_pd_->src_md(0)),
+                    VERBOSE_INCONSISTENT_MDS, "src_md", "dw_conv_pd_->src_md");
+            VDISPATCH_CONV_IC(
+                    jcp_1x1.oc_without_padding % jcp_1x1.oc_block == 0,
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL,
+                    "output-channel is not an exact multiple of oc_block");
+            VDISPATCH_CONV_IC(IMPLICATION(jcp_dw_->ow_block,
+                                      jcp_dw_->ow_block == jcp_dw_->ow),
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL,
+                    "ow_block does not equal output-width");
 
             assert(jcp_dw_);
             assert(dw_conv_pd_->dst_md(0)->format_kind != format_kind::any);
@@ -316,17 +309,14 @@ struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
     jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t(const pd_t *apd)
         : primitive_t(apd) {}
 
-    typedef typename prec_traits<src_type>::type src_data_t;
-    typedef typename prec_traits<data_type::s8>::type wei_data_t;
-    typedef typename prec_traits<dst_type>::type dst_data_t;
-    // Note: In case of fused depthwise convolution, the final output datatype
-    // after fusion may not be dst_data_t.
-    typedef typename prec_traits<data_type::s32>::type acc_data_t;
+    // Note: In case of fused depthwise convolution, the final output data type
+    // after fusion may not be same as for dst.
+    using acc_data_t = typename prec_traits_t<data_type::s32>::type;
 
     status_t init(engine_t *engine) override {
         CHECK(safe_ptr_assign(kernel_,
                 new jit_avx512_core_x8s8s32x_1x1_conv_kernel(
-                        pd()->jcp_, *pd()->attr(), *pd()->dst_md(0))));
+                        pd()->jcp_, *pd()->attr(), *pd()->dst_1x1_md(0))));
         CHECK(kernel_->create_kernel());
 
         if (pd()->jcp_.with_dw_conv) {
@@ -336,7 +326,7 @@ struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
             CHECK(kernel_dw_->create_kernel());
         }
 
-        CHECK(init_rtus_driver<avx512_common>(this));
+        CHECK(init_rtus_driver<avx512_core>(this));
         return status::success;
     }
 
@@ -346,16 +336,24 @@ struct jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
 
 private:
     status_t execute_forward(const exec_ctx_t &ctx) const;
-    void execute_forward_thr(const int ithr, const int nthr,
-            const src_data_t *src, const wei_data_t *weights, const char *bias,
-            const wei_data_t *weights_dw, const char *bias_dw, dst_data_t *dst,
-            const int32_t *src_zero_point, const int32_t *dst_zero_point,
+    void execute_forward_thr(const int ithr, const int nthr, const char *src,
+            const char *weights, const char *bias, const char *weights_dw,
+            const char *bias_dw, char *dst, const float *oscales,
+            const float *dst_scales, const float *dw_oscales,
+            const float *dw_dst_scales, const int32_t *src_zero_point,
+            const int32_t *dst_zero_point,
             const memory_tracking::grantor_t &scratchpad,
             const void *post_ops_binary_rhs_arg_vec,
             const void *post_ops_binary_rhs_arg_vec_dw) const;
     const pd_t *pd() const { return (const pd_t *)primitive_t::pd().get(); }
+    const float *adjust_oscales(const memory_tracking::grantor_t &scratchpad,
+            const float *src_scales, const float *wei_scales) const;
+    const float *maybe_adjust_dw_oscales(
+            const memory_tracking::grantor_t &scratchpad,
+            const float *dst_scales, const float *dw_wei_scales) const;
+
     std::unique_ptr<jit_avx512_core_x8s8s32x_1x1_conv_kernel> kernel_;
-    std::unique_ptr<rtus_driver_t<avx512_common>> rtus_driver_;
+    std::unique_ptr<rtus_driver_t<avx512_core>> rtus_driver_;
     using dw_conv_kernel_t = jit_avx512_core_x8s8s32x_fwd_kernel;
     std::unique_ptr<dw_conv_kernel_t> kernel_dw_;
 };
