@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,10 +17,12 @@
 #ifndef CPU_X64_JIT_AVX512_CORE_X8S8S32X_CONV_KERNEL_HPP
 #define CPU_X64_JIT_AVX512_CORE_X8S8S32X_CONV_KERNEL_HPP
 
+#include <memory>
 #include "common/c_types_map.hpp"
 #include "common/memory_tracking.hpp"
 
 #include "cpu/x64/injectors/jit_uni_postops_injector.hpp"
+#include "cpu/x64/jit_avx512_core_bf16cvt.hpp"
 #include "cpu/x64/jit_generator.hpp"
 #include "cpu/x64/jit_primitive_conf.hpp"
 
@@ -30,7 +32,7 @@ namespace cpu {
 namespace x64 {
 
 template <typename Vmm>
-struct _jit_avx512_core_x8s8s32x_fwd_kernel : public jit_generator {
+struct _jit_avx512_core_x8s8s32x_fwd_kernel : public jit_generator_t {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(_jit_avx512_core_x8s8s32x_conv_fwd_ker_t)
 
     enum { STATE_FIRST_DST_LOAD = 0x1U };
@@ -43,9 +45,12 @@ struct _jit_avx512_core_x8s8s32x_fwd_kernel : public jit_generator {
 
 private:
     constexpr static int isa_simd_width_
-            = cpu_isa_traits<avx512_core>::vlen / sizeof(float);
+            = cpu_isa_traits_t<avx512_core>::vlen / sizeof(float);
+    using Vmm_down_t =
+            typename utils::conditional<std::is_same<Vmm, Xbyak::Zmm>::value,
+                    Xbyak::Ymm, Xbyak::Xmm>::type;
     const int ic_sub_step = 4;
-    std::unique_ptr<injector::jit_uni_postops_injector_t<avx512_core>>
+    std::unique_ptr<injector::jit_uni_postops_injector_t<avx512_core, Vmm>>
             postops_injector_;
 
     enum {
@@ -68,6 +73,7 @@ private:
     const Xbyak::Reg64 reg_out = r10;
     const Xbyak::Reg64 aux_reg_inp = r11;
     const Xbyak::Reg64 reg_ptr_sum_scale = r11;
+    const Xbyak::Reg64 reg_ptr_sum_zp = abi_not_param1;
     const Xbyak::Reg64 aux_reg_ker = r12;
     const Xbyak::Reg64 reg_compensation = r14;
     const Xbyak::Reg64 aux_reg_inp_d = r13;
@@ -81,8 +87,10 @@ private:
     const Xbyak::Reg64 reg_src_zero_point = aux_reg_ker_d;
     const Xbyak::Reg64 reg_dst_zero_point = reg_src_zero_point;
 
+    // dst scale
+    const Xbyak::Reg64 reg_dst_scale = reg_src_zero_point;
+
     /* counter regs */
-    const Xbyak::Reg64 reg_bias_alpha = abi_not_param1;
     const Xbyak::Reg64 reg_oi = rbx;
     const Xbyak::Reg64 reg_bias = rdx;
     const Xbyak::Reg64 reg_oc_blocks = rsi;
@@ -94,9 +102,13 @@ private:
     const Xbyak::Reg64 reg_icb = reg_bias;
     const Xbyak::Reg64 reg_jmp_tbl_base = reg_kj;
 
+    /* binary post-op operand */
+    const Xbyak::Reg64 temp_offset_reg = r12;
+
     const Xbyak::Opmask ktail_mask = Xbyak::Opmask(2);
     const Xbyak::Opmask kblend_mask = Xbyak::Opmask(3);
     const Xbyak::Opmask postops_mask = Xbyak::Opmask(4);
+    const Xbyak::Opmask ktail_mask_extended = Xbyak::Opmask(5);
 
     const Vmm vmm_wei = Vmm(31);
     /* used during bias section of store_output */
@@ -106,6 +118,7 @@ private:
     const Vmm vmm_prev_dst = Vmm(31);
     /* used during write-out section of store_output */
     const Vmm vmm_saturation = Vmm(30);
+    const Vmm vmm_sum_zp = Vmm(30);
     const Vmm vmm_zero = Vmm(31);
 
     /* used in compute_ker (but set during prepare_output) */
@@ -119,6 +132,17 @@ private:
     const Vmm vmm_zp_one = Vmm(26);
     const Vmm vmm_zp_tmp = vmm_zp;
 
+    const Vmm vmm_dst_scale = Vmm(31);
+
+    /* bf16 emulation */
+    Xbyak::Zmm bf16_emu_reserv_1 = Xbyak::Zmm(26);
+    Xbyak::Zmm bf16_emu_reserv_2 = Xbyak::Zmm(27);
+    Xbyak::Zmm bf16_emu_reserv_3 = Xbyak::Zmm(28);
+    Xbyak::Zmm bf16_emu_reserv_4 = Xbyak::Zmm(30);
+    // bf16_emu_reserv_5 not required when only computing vcvtneps2bf16()
+    const Xbyak::Reg64 bf16_emu_scratch = aux_reg_ker;
+    std::unique_ptr<bf16_emulation_t> bf16_emu_;
+
     /* registers use only for depthwise
        groups are always blocked by 16(padded if needed),
        hence use only Zmm registers */
@@ -132,8 +156,7 @@ private:
         const int nb_x_blocking
                 = jcp.is_depthwise ? jcp.nb_ch_blocking : jcp.nb_oc_blocking;
         const int idx = i_ur * nb_x_blocking + i_oc;
-        assert(idx < (jcp.is_depthwise
-                               ? ker_dw_reg_base_idx
+        assert(idx < (jcp.is_depthwise              ? ker_dw_reg_base_idx
                                : jcp.src_zero_point ? ker_zp_reg_base_idx
                                                     : ker_reg_base_idx));
         return idx;
@@ -160,16 +183,6 @@ private:
 
         return Xbyak::Zmm(idx);
     }
-    Vmm vmm_bias_alpha() {
-        int nb_c_block
-                = jcp.is_depthwise ? jcp.nb_ch_blocking : jcp.nb_oc_blocking;
-        return Vmm(nb_c_block * jcp.ur_w);
-    }
-    Xbyak::Xmm xmm_bias_alpha() {
-        int nb_c_block
-                = jcp.is_depthwise ? jcp.nb_ch_blocking : jcp.nb_oc_blocking;
-        return Xbyak::Xmm(nb_c_block * jcp.ur_w);
-    }
     int get_ow_start(int ki, int pad_l) {
         return nstl::max(0,
                 utils::div_up(pad_l - ki * (jcp.dilate_w + 1), jcp.stride_w));
@@ -182,11 +195,37 @@ private:
                                 jcp.stride_w));
     }
 
+    // bf16 utils
+    int get_src_down_idx(int nb_x_blocking) {
+        int idx = nb_x_blocking * jcp.ur_w;
+        assert(idx < 31);
+        return idx;
+    }
+    inline Vmm maybe_mask_vmm(Vmm vmm, bool mask_flag) {
+        return mask_flag ? vmm | ktail_mask_extended : vmm;
+    }
+    inline Vmm_down_t maybe_mask_vmm_down(Vmm_down_t vmm, bool mask_flag) {
+        return (mask_flag) ? vmm | ktail_mask : vmm;
+    }
+    inline void store_bf16(Xbyak::Address addr, int vmm_dst_idx,
+            int vmm_down_idx, bool mask_flag) {
+        auto vmm_down = Vmm_down_t(vmm_down_idx);
+        bf16_emu_->vcvtneps2bf16(
+                Xbyak::Ymm(vmm_down_idx), Xbyak::Zmm(vmm_dst_idx));
+
+        // for xmm, upper half is zero after conversion to
+        // bf16, so mask always & mask for tails
+        vmovdqu16(addr,
+                maybe_mask_vmm_down(vmm_down, mask_flag || jcp.simd_w == 4));
+    }
+
     void prepare_output(int ur_w);
     void apply_sum(int ur_w, bool last_oc_block_flag, const int nb_oc_block,
-            const int oc_block, const float *p_sum_scale);
+            const int oc_block, const float *p_sum_scale,
+            const int32_t *p_sum_zp);
     void apply_postops(int ur_w, bool last_oc_block_flag, const int nb_oc_block,
-            const int oc_block, const float *p_sum_scale);
+            const int oc_block, const float *p_sum_scale,
+            const int32_t *p_sum_zp);
     void store_output(int ur_w, bool last_oc_block_flag);
     void compute_ker_dw(int ur_w, int pad_l, int pad_r,
             ic_block_t last_ic_block_flag, bool h_padded);
@@ -203,41 +242,48 @@ private:
 struct jit_avx512_core_x8s8s32x_fwd_kernel {
 
     jit_avx512_core_x8s8s32x_fwd_kernel(const jit_conv_conf_t &ajcp,
-            const primitive_attr_t &attr, const dnnl_memory_desc_t &dst_md)
+            const primitive_attr_t &attr, const memory_desc_t &dst_md)
         : kernel_(nullptr) {
         int ch_block = ajcp.is_depthwise ? ajcp.ch_block : ajcp.ic_block;
         switch (ch_block) {
             case 16:
-                kernel_ = new _jit_avx512_core_x8s8s32x_fwd_kernel<Xbyak::Zmm>(
+                kernel_ = utils::make_unique<
+                        _jit_avx512_core_x8s8s32x_fwd_kernel<Xbyak::Zmm>>(
                         ajcp, attr, dst_md);
                 return;
             case 8:
-                kernel_ = new _jit_avx512_core_x8s8s32x_fwd_kernel<Xbyak::Ymm>(
+                kernel_ = utils::make_unique<
+                        _jit_avx512_core_x8s8s32x_fwd_kernel<Xbyak::Ymm>>(
                         ajcp, attr, dst_md);
                 return;
             case 4:
-                kernel_ = new _jit_avx512_core_x8s8s32x_fwd_kernel<Xbyak::Xmm>(
+                kernel_ = utils::make_unique<
+                        _jit_avx512_core_x8s8s32x_fwd_kernel<Xbyak::Xmm>>(
                         ajcp, attr, dst_md);
                 return;
             default: assert(!"invalid channel blocking");
         }
     }
 
-    status_t create_kernel() { return kernel_->create_kernel(); }
+    status_t create_kernel() {
+        if (kernel_) return kernel_->create_kernel();
+        return status::out_of_memory;
+    }
 
-    ~jit_avx512_core_x8s8s32x_fwd_kernel() { delete kernel_; }
+    ~jit_avx512_core_x8s8s32x_fwd_kernel() = default;
 
     static status_t init_conf(jit_conv_conf_t &jcp,
             const convolution_desc_t &cd, memory_desc_t &src_pd,
             memory_desc_t &weights_pd, memory_desc_t &dst_pd,
-            memory_desc_t &bias_pd, const primitive_attr_t &attr, int nthreads);
+            memory_desc_t &bias_pd, primitive_attr_t &attr, int nthreads);
     static void init_scratchpad(memory_tracking::registrar_t &scratchpad,
             const jit_conv_conf_t &jcp, const primitive_attr_t &attr);
     void operator()(const jit_conv_call_s *p) const { (*kernel_)(p); }
     const Xbyak::uint8 *jit_ker() const { return kernel_->jit_ker(); }
 
 private:
-    jit_generator *kernel_;
+    DNNL_DISALLOW_COPY_AND_ASSIGN(jit_avx512_core_x8s8s32x_fwd_kernel)
+    std::unique_ptr<jit_generator_t> kernel_;
 };
 
 } // namespace x64
