@@ -280,9 +280,16 @@ inline bool has_large_size(const convolution_desc_t &cd,
         const memory_desc_wrapper &src_d, const memory_desc_wrapper &weights_d,
         const memory_desc_wrapper &dst_d) {
     auto is_large = [](const dim_t val) { return val > INT_MAX; };
+    auto img_size = [](const memory_desc_wrapper &mem_d) {
+        // assuming that first dimension for src and dst is minibatch
+        const auto mb = mem_d.dims()[0] != 0 ? mem_d.dims()[0] : 1;
+        return mem_d.nelems() / mb;
+    };
 
-    if (utils::one_of(true, is_large(src_d.nelems()),
-                is_large(weights_d.nelems()), is_large(dst_d.nelems())))
+    // Check if numbers of elements (excepting minibatch) for any memory
+    // descriptor is greater than INT_MAX
+    if (is_large(img_size(src_d)) || is_large(weights_d.nelems())
+            || is_large(img_size(dst_d)))
         return true;
 
     const int ndims = src_d.ndims();
@@ -545,6 +552,8 @@ struct jit_pool_conf_t {
     bool with_binary;
     int nthr;
     memory_desc_t tmp_md;
+    bool needs_f32_accum_for_bf16;
+    dim_t f32_accum_block_size;
 };
 
 struct jit_pool_call_s {
@@ -607,7 +616,7 @@ struct jit_resampling_conf_t {
 
     cpu_isa_t isa = isa_undef;
 
-    post_ops_t post_ops = post_ops_t();
+    post_ops_t post_ops;
     bool with_postops = false;
     bool with_eltwise = false;
     bool with_binary = false;
@@ -815,6 +824,8 @@ struct jit_brgemm_conv_conf_t {
     bool is_bf32;
     bool is_fp8 {false};
     bool is_fp8_convert {false};
+    bool is_f32_f16 {false};
+    bool is_f32_bf16 {false};
     bool comp_with_vpads;
 
     int nthr_mb, nthr_g, nthr_oc_b, nthr_ic_b, nthr_oh;
@@ -940,7 +951,7 @@ struct jit_reduction_conf_t {
 
     bool is_saturation_needed = false;
 
-    post_ops_t post_ops = post_ops_t();
+    post_ops_t post_ops;
     bool with_postops = false;
     bool with_eltwise = false;
     bool with_binary = false;
