@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -29,7 +29,7 @@ namespace impl {
 namespace cpu {
 namespace x64 {
 
-struct jit_sse41_1x1_conv_kernel_f32 : public jit_generator {
+struct jit_sse41_1x1_conv_kernel_f32 : public jit_generator_t {
     jit_sse41_1x1_conv_kernel_f32(const jit_1x1_conv_conf_t &ajcp,
             const primitive_attr_t &attr, const memory_desc_t &dst_md);
 
@@ -45,7 +45,8 @@ struct jit_sse41_1x1_conv_kernel_f32 : public jit_generator {
     const primitive_attr_t &attr_;
 
 private:
-    static constexpr auto simd_w_ = cpu_isa_traits<sse41>::vlen / sizeof(float);
+    static constexpr auto simd_w_
+            = cpu_isa_traits_t<sse41>::vlen / sizeof(float);
     using reg64_t = const Xbyak::Reg64;
     using xmm_t = const Xbyak::Xmm;
 
@@ -70,8 +71,8 @@ private:
 
     constexpr static int reg64_size_ = sizeof(int64_t);
     constexpr static int reg_diff_bias_data_stack_offt = 0;
-    constexpr static int reg_binary_post_op_acc_off = 1 * reg64_size_;
-    constexpr static int reg_abi_param1_backup = 2 * reg64_size_;
+    constexpr static int reg_abi_param1_backup = 1 * reg64_size_;
+    constexpr static int reg_dw_binary_output_off = 2 * reg64_size_;
     constexpr static int stack_space_needed = 3 * reg64_size_;
 
     xmm_t reg_bcast = xmm_t(15);
@@ -86,7 +87,8 @@ private:
     void generate() override;
 
     void apply_postops(const int load_loop_blk, const int ur);
-    size_t get_fwd_output_ptr_l_off(int i, int j, int n);
+    size_t get_fwd_output_ptr_l_off(
+            int i, int j, int n, bool ignore_dw_conv) const;
 };
 
 } // namespace x64
