@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2021 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -69,8 +69,9 @@ void jit_sse41_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
         const void *post_ops_binary_rhs_arg_vec_dw) const {
 
     const memory_desc_wrapper src_d(pd()->src_md());
-    const memory_desc_wrapper dst_d(pd()->dst_md());
+    const memory_desc_wrapper dst_d(pd()->dst_1x1_md());
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
+    const memory_desc_wrapper dw_dst_d(pd()->dst_md());
     const memory_desc_wrapper dw_weights_d(
             pd()->arg_md(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS));
     const memory_desc_wrapper dw_bias_d(
@@ -98,12 +99,15 @@ void jit_sse41_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
     const int nb_load_blocking_max = jcp.with_dw_conv
             ? jcp.nb_load_blocking
             : jcp.nb_load_blocking_max;
+    const bool is_dst_layout_nxc = utils::one_of(
+            jcp.dst_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
+    const bool is_src_layout_nxc = utils::one_of(
+            jcp.src_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
 
     // Begin: declare Variables needed for dw conv.
     data_t *pbuf {nullptr};
     size_t row_offset {};
     const int nb_buffer = jcp.nb_load_blocking;
-    const int jcp_dw_kh = 3;
     std::vector<data_t *> addrs;
 
     auto step = [](int default_step, int remaining, int tail_step) {
@@ -142,13 +146,11 @@ void jit_sse41_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
         const size_t _ocb = g * nb_oc + ocb;
         const size_t _icb = g * nb_ic + icb;
 
-        const bool is_dst_layout_nxc = utils::one_of(jcp.dst_tag,
-                format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
         const int oc_off_idx = (is_dst_layout_nxc ? jcp.oc_block : 1) * _ocb;
-
+        const size_t dst_off = data_blk_off(dst_d, n, oc_off_idx, oh, ow);
         par_conv.output_data = jcp.with_dw_conv
-                ? pbuf + (oh % jcp_dw_kh) * row_offset
-                : &dst[data_blk_off(dst_d, n, oc_off_idx, oh, ow)];
+                ? pbuf + (oh % pd()->dw_conv_pd_->jcp_.kh) * row_offset
+                : &dst[dst_off];
 
         par_conv.bias_data = &bias[_ocb * jcp.oc_block];
 
@@ -158,8 +160,6 @@ void jit_sse41_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
         par_conv.reduce_dim = this_block_size(
                 icb * jcp.ic_block, jcp.ic, nb_ic_blocking * jcp.ic_block);
 
-        const bool is_src_layout_nxc = utils::one_of(jcp.src_tag,
-                format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
         const int ic_off_idx = (is_src_layout_nxc ? jcp.ic_block : 1) * _icb;
 
         const size_t src_off = data_blk_off(src_d, n, ic_off_idx, ih, iw);
@@ -169,9 +169,9 @@ void jit_sse41_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
                 = &weights[pd()->with_groups() ? weights_d.blk_off(g, ocb, icb)
                                                : weights_d.blk_off(ocb, icb)];
 
-        par_conv.oc_l_off = _ocb * jcp.oc_block;
         par_conv.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec;
-        par_conv.dst_orig = dst;
+        par_conv.dst_orig
+                = static_cast<const float *>(par_conv.output_data) - dst_off;
 
         (*kernel_)(&par_conv);
     };
@@ -205,8 +205,8 @@ void jit_sse41_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
             addrs[i] = pbuf + ((oh_1x1++) % jcp_dw.kh) * row_offset;
 
         const auto ocb_end = ocb_start + load_step;
-        const auto wch_stride
-                = jcp_dw.iw * jcp_dw.nb_ch_blocking * jcp_dw.ch_block;
+        const auto wch_stride = (is_src_layout_nxc ? 1 : jcp_dw.iw)
+                * jcp_dw.nb_ch_blocking * jcp_dw.ch_block;
         const int dil_h = jcp_dw.dilate_h + 1;
         const int str_h = jcp_dw.stride_h;
         const int ch_num = jcp_dw.nb_ch_blocking;
@@ -230,8 +230,12 @@ void jit_sse41_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
             jit_conv_call_s par_conv_dw;
 
             par_conv_dw.src = addrs.data();
-            par_conv_dw.dst = &dst[dst_d.blk_off(n, ch, dw_oh, ow)];
 
+            const size_t ch_step = is_dst_layout_nxc
+                    ? jcp_dw.ch_block
+                    : dw_dst_d.blk_off(0, 1, 0, 0);
+            par_conv_dw.dst
+                    = &dst[dw_dst_d.blk_off(n, 0, dw_oh, ow) + ch * ch_step];
             par_conv_dw.filt
                     = &weights_dw[dw_weights_d.blk_off(ch, 0, 0, kh, kw)];
             if (bias)
@@ -243,7 +247,6 @@ void jit_sse41_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
             par_conv_dw.load_work = (nstl::min(ch + ch_num, jcp_dw.nb_ch) - ch)
                     * jcp_dw.ch_block;
 
-            par_conv_dw.oc_l_off = ch;
             par_conv_dw.post_ops_binary_rhs_arg_vec
                     = post_ops_binary_rhs_arg_vec_dw;
             par_conv_dw.dst_orig = dst;
