@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -39,10 +39,7 @@ namespace x64 {
 
 struct jit_sse41_1x1_convolution_fwd_t : public primitive_t {
     struct pd_t : public cpu_convolution_fwd_pd_t {
-        using dw_conv_pd_type = jit_sse41_dw_convolution_fwd_t::pd_t;
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const typename pd_t::base_class *hint_fwd_pd)
-            : cpu_convolution_fwd_pd_t(adesc, attr, hint_fwd_pd), jcp_() {}
+        using cpu_convolution_fwd_pd_t::cpu_convolution_fwd_pd_t;
 
         pd_t(const pd_t &other) : cpu_convolution_fwd_pd_t(other) {
             if (copy(other) != status::success) is_initialized_ = false;
@@ -52,36 +49,48 @@ struct jit_sse41_1x1_convolution_fwd_t : public primitive_t {
                 jit_sse41_1x1_convolution_fwd_t);
 
         status_t init(engine_t *engine) {
-            bool ok = true && is_fwd()
-                    && set_default_alg_kind(alg_kind::convolution_direct)
-                    && expect_data_types(data_type::f32, data_type::f32,
-                            data_type::f32, data_type::f32, data_type::f32)
-                    && attr()->has_default_values(
-                            primitive_attr_t::skip_mask_t::post_ops,
-                            data_type::f32)
-                    && !has_zero_dim_memory() && set_default_formats();
-            if (!ok) return status::unimplemented;
+            using namespace data_type;
+            VDISPATCH_CONV(is_fwd(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_CONV(expect_data_types(f32, f32, f32, f32, f32),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(set_default_alg_kind(alg_kind::convolution_direct),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_CONV(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+            VDISPATCH_CONV(
+                    attr()->has_default_values(
+                            primitive_attr_t::skip_mask_t::post_ops, f32),
+                    VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_CONV(set_default_formats(), VERBOSE_UNSUPPORTED_TAG);
+            VDISPATCH_CONV(
+                    attr_.set_default_formats(dst_md(0)) == status::success,
+                    VERBOSE_UNSUPPORTED_POSTOP);
 
-            status_t status = jit_sse41_1x1_conv_kernel_f32::init_conf(jcp_,
-                    *desc(), *src_md(), *weights_md(), *dst_md(), *attr(),
-                    dnnl_get_max_threads());
-            if (status != status::success) return status;
+            // TODO: make `init_conf` assign initialized object to `jcp_`
+            CHECK(jit_sse41_1x1_conv_kernel_f32::init_conf(jcp_, *desc(),
+                    *src_md(), *weights_md(), *dst_md(), *attr(),
+                    dnnl_get_max_threads()));
+            if (jcp_.with_dw_conv) CHECK(depthwise_po_init(engine));
 
-            if (jcp_.with_dw_conv) {
-                status = depthwise_po_init(engine);
-                if (status != status::success) return status;
-            }
-
-            return status;
+            return status::success;
         }
 
-        const memory_desc_t *dst_md(int index = 0) const override {
-            return jcp_.with_dw_conv ? dw_conv_pd_->dst_md(index) : &dst_md_;
+        const memory_desc_t *dst_1x1_md(int index = 0) const {
+            return cpu_convolution_fwd_pd_t::dst_md(index);
         }
 
-        const memory_desc_t *arg_md(int index = 0) const override {
-            if (jcp_.with_dw_conv) {
-                switch (index) {
+        const memory_desc_t *dst_md(
+                int index = 0, bool user_input = false) const override {
+            return dw_conv_pd_ && jcp_.with_dw_conv
+                    ? dw_conv_pd_->dst_md(index, user_input)
+                    : cpu_convolution_fwd_pd_t::dst_md(index, user_input);
+        }
+
+        const memory_desc_t *arg_md(
+                int arg, bool user_input = false) const override {
+            if (dw_conv_pd_ && jcp_.with_dw_conv) {
+                switch (arg) {
+                    case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_SRC:
+                        return cpu_convolution_fwd_pd_t::dst_md(0, user_input);
                     case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS:
                         return dw_conv_pd_->weights_md(0);
                     case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS:
@@ -89,21 +98,21 @@ struct jit_sse41_1x1_convolution_fwd_t : public primitive_t {
                     default: break;
                 }
             }
-            return convolution_fwd_pd_t::arg_md(index);
+            return convolution_fwd_pd_t::arg_md(arg, user_input);
         }
 
         arg_usage_t arg_usage(int arg) const override {
             if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS))
                 return arg_usage_t::input;
 
-            if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS)
-                    && attr_post_op_dw_inputs() > 1)
-                return arg_usage_t::input;
+            if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_BIAS))
+                return attr_post_op_dw_inputs() > 1 ? arg_usage_t::input
+                                                    : arg_usage_t::unused;
 
             return convolution_fwd_pd_t::arg_usage(arg);
         }
 
-        jit_1x1_conv_conf_t jcp_;
+        jit_1x1_conv_conf_t jcp_ = utils::zero<decltype(jcp_)>();
         using dw_pd_t = jit_sse41_dw_convolution_fwd_t::pd_t;
         std::unique_ptr<dw_pd_t> dw_conv_pd_;
 
@@ -111,7 +120,23 @@ struct jit_sse41_1x1_convolution_fwd_t : public primitive_t {
         bool set_default_formats() {
             using namespace format_tag;
 
-            auto dat_tag = utils::pick(ndims() - 3, nCw8c, nChw8c, nCdhw8c);
+            const memory_desc_wrapper src_d(&src_md_);
+            const memory_desc_wrapper dst_d(&dst_md_);
+
+            const auto dat_tag_nxc = utils::pick(ndims() - 3, nwc, nhwc, ndhwc);
+            const auto dat_tag_nCx8c
+                    = utils::pick(ndims() - 3, nCw8c, nChw8c, nCdhw8c);
+            const auto curr_src_tag
+                    = src_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx8c);
+            const auto curr_dst_tag
+                    = dst_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx8c);
+            const auto is_data_layout_nxc
+                    = IMPLICATION(curr_src_tag != dat_tag_nxc,
+                              src_d.format_kind() == format_kind::any)
+                    && IMPLICATION(curr_dst_tag != dat_tag_nxc,
+                            dst_d.format_kind() == format_kind::any)
+                    && utils::one_of(dat_tag_nxc, curr_src_tag, curr_dst_tag);
+            auto dat_tag = is_data_layout_nxc ? dat_tag_nxc : dat_tag_nCx8c;
             auto wei_tag = with_groups()
                     ? utils::pick(ndims() - 3, gOIw8i8o, gOIhw8i8o)
                     : utils::pick(ndims() - 3, OIw8i8o, OIhw8i8o);
@@ -134,7 +159,6 @@ struct jit_sse41_1x1_convolution_fwd_t : public primitive_t {
             auto &jcp_1x1 = jcp_;
             primitive_attr_t attr_1x1(*attr());
             if (!attr_1x1.is_initialized()) return status::out_of_memory;
-            attr_1x1.set_scratchpad_mode(scratchpad_mode::user);
             const auto &src_md = dst_md_;
             const memory_desc_wrapper src_d(src_md);
             const auto nthr = dnnl_get_max_threads();
@@ -148,16 +172,19 @@ struct jit_sse41_1x1_convolution_fwd_t : public primitive_t {
             // for 1x1: Check that no better ISA is available.
             // for dw: Always fuse with same ISA.
             // Caveat: May be a better dw conv exists.
-
-            bool ok = true && (!mayiuse(avx))
-                    && (attr_1x1.post_ops_.find(primitive_kind::sum) == -1)
-                    // TODO: Below may be further tuned.
-                    && (l2_cache * 2 < src_d.size())
-                    // load_grp_count check can be redundant due to l2 check
-                    // above. Adding it explicitly as the current driver doesn't
-                    // work if this condition fails.
-                    && (jcp_1x1.load_grp_count < 2);
-            if (!ok) return status::unimplemented;
+            VDISPATCH_CONV_IC(!mayiuse(avx), VERBOSE_1x1CONV_HEURISTIC_FAIL,
+                    "higher isa is supported");
+            VDISPATCH_CONV_IC(
+                    attr_1x1.post_ops_.find(primitive_kind::sum) == -1,
+                    VERBOSE_UNSUPPORTED_FEATURE, "unsupported sum post-op");
+            // TODO: Below may be further tuned.
+            VDISPATCH_CONV_IC(l2_cache * 2 < src_d.size(),
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL, "cache check size failed");
+            // load_grp_count check can be redundant due to l2 check
+            // above. Adding it explicitly as the current driver doesn't
+            // work if this condition fails.
+            VDISPATCH_CONV_IC(jcp_1x1.load_grp_count < 2,
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL, "load grp count > 1");
 
             int dw_po_index
                     = attr_1x1.post_ops_.find(primitive_kind::convolution);
@@ -168,17 +195,51 @@ struct jit_sse41_1x1_convolution_fwd_t : public primitive_t {
             CHECK(get_depthwise_conv_desc(
                     cd_dw, src_md, attr_1x1, attr_dw, dw_po_index));
 
+            // The code below doesn't work because currently it requires `jcp_`
+            // member which is not available from the common interface. In turn,
+            // this means the common pd creation interface through an iterator
+            // can't be used and a specific convolution implementation's pd is
+            // required here. It restricts the usage of inherited
+            // `convolution_pd_t` constructor.
+            // ANCHOR: USING_INHERITED_IS_IMPOSSIBLE.
+            //
+            // ```cpp
+            // primitive_desc_iterator_t it(
+            //         engine, (op_desc_t *)&cd_dw, &attr_dw, nullptr);
+            // if (!it.is_initialized()) return status::out_of_memory;
+            // while (++it != it.end()) {
+            //     dw_conv_pd_ = *it;
+            //     break;
+            // }
+            // VDISPATCH_CONV_IC(dw_conv_pd_, "dw_conv_pd hasn't been created");
+            // ```
+            //
+            // ```compiler output
+            // error: ‘using element_type = struct dnnl::impl::primitive_desc_t’
+            // {aka ‘struct dnnl::impl::primitive_desc_t’} has no member named
+            // ‘jcp_’
+            // auto &jcp_dw = dw_conv_pd_->jcp_;
+            //                             ^~~~
+            // ```
+            //
+            // TODO: figure out the way to initialize fused conv through a
+            // normal interface without hacks accessing specific members.
             CHECK(safe_ptr_assign(
                     dw_conv_pd_, new dw_pd_t(&cd_dw, &attr_dw, nullptr)));
             CHECK(dw_conv_pd_->init(engine));
             auto &jcp_dw = dw_conv_pd_->jcp_;
 
-            ok = true
-                    && (dnnl_memory_desc_equal(&src_md, dw_conv_pd_->src_md(0)))
-                    && (jcp_1x1.oc_without_padding % jcp_1x1.oc_block == 0)
-                    && IMPLICATION(
-                            jcp_dw.ow_block, jcp_dw.ow_block == jcp_dw.ow);
-            if (!ok) return status::unimplemented;
+            VDISPATCH_CONV_IC(
+                    dnnl_memory_desc_equal(&src_md, dw_conv_pd_->src_md(0)),
+                    VERBOSE_INCONSISTENT_MDS, "src_md", "dw_conv_pd_->src_md");
+            VDISPATCH_CONV_IC(
+                    jcp_1x1.oc_without_padding % jcp_1x1.oc_block == 0,
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL,
+                    "output-channel is not an exact multiple of oc_block");
+            VDISPATCH_CONV_IC(
+                    IMPLICATION(jcp_dw.ow_block, jcp_dw.ow_block == jcp_dw.ow),
+                    VERBOSE_1x1CONV_HEURISTIC_FAIL,
+                    "ow_block does not equal output-width");
 
             assert(dw_conv_pd_->dst_md(0)->format_kind != format_kind::any);
             assert(dw_conv_pd_->weights_md(0)->format_kind != format_kind::any);
@@ -199,8 +260,14 @@ struct jit_sse41_1x1_convolution_fwd_t : public primitive_t {
 
             jcp_dw.dw_conv_buffer_oc
                     = jcp_1x1.nb_load_blocking * jcp_1x1.oc_block;
-            jcp_1x1.bcast_loop_output_step
-                    = jcp_1x1.ur * jcp_1x1.load_block * jcp_1x1.typesize_out;
+
+            const auto dat_tag_nxc = utils::pick(ndims() - 3, format_tag::nwc,
+                    format_tag::nhwc, format_tag::ndhwc);
+            const bool is_data_nxc = utils::everyone_is(
+                    dat_tag_nxc, jcp_1x1.src_tag, jcp_1x1.dst_tag);
+            if (!is_data_nxc)
+                jcp_1x1.bcast_loop_output_step = jcp_1x1.ur * jcp_1x1.load_block
+                        * jcp_1x1.typesize_out;
 
             registrar_t scratchpad(scratchpad_registry_);
             registrar_t dw_scratchpad(scratchpad, names::prefix_fusion);
@@ -221,12 +288,12 @@ struct jit_sse41_1x1_convolution_fwd_t : public primitive_t {
 
     jit_sse41_1x1_convolution_fwd_t(const pd_t *apd) : primitive_t(apd) {}
 
-    typedef typename prec_traits<data_type::f32>::type data_t;
+    using data_t = typename prec_traits_t<data_type::f32>::type;
 
     status_t init(engine_t *engine) override {
         CHECK(safe_ptr_assign(kernel_,
                 new jit_sse41_1x1_conv_kernel_f32(
-                        pd()->jcp_, *pd()->attr(), *pd()->dst_md(0))));
+                        pd()->jcp_, *pd()->attr(), *pd()->dst_1x1_md(0))));
         CHECK(kernel_->create_kernel());
         if (pd()->jcp_.with_dw_conv) {
             CHECK(safe_ptr_assign(kernel_dw_,
