@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@
 #include "common/dnnl_thread.hpp"
 #include "common/memory_tracking.hpp"
 #include "common/nstl.hpp"
-#include "common/primitive_iterator.hpp"
+#include "common/primitive_desc_iterator.hpp"
 #include "common/type_helpers.hpp"
 #include "common/utils.hpp"
 
@@ -125,7 +125,7 @@ inline void rtus_prepare_space_info(conv_pd_t *self,
 }
 
 template <cpu_isa_t isa>
-struct rtus_driver_t : public jit_generator {
+struct rtus_driver_t : public jit_generator_t {
 
     struct call_params_t {
         const void *ws; /* reduced image (w/ strides = 1) */
@@ -169,7 +169,7 @@ struct rtus_driver_t : public jit_generator {
     rtus_driver_t(int iw, int stride_w, int src_step_h, int src_step_icb,
             int ws_step_icb, bool src_to_ws, size_t typesize, int ic,
             bool is_nspc = false)
-        : jit_generator(nullptr, MAX_CODE_SIZE, true, isa)
+        : jit_generator_t(jit_name(), isa)
         , iw_(iw)
         , stride_w_(stride_w)
         , src_step_h_(src_step_h)
@@ -189,15 +189,13 @@ struct rtus_driver_t : public jit_generator {
          * fail to work on reg_v, reg_zero because of this
          * data_type change, e.g. uni_vpxor doen't
          * work on reg_zero now*/
-        auto Vmm = [=](int idx, size_t typesize) {
+        auto Vmm = [this](int idx, size_t typesize) {
             Xmm res;
             if (is_nspc_) {
                 switch (isa) {
                     case sse41: res = Xmm(idx); break;
                     case avx2: res = Ymm(idx); break;
-                    case avx512_common:
-                    case avx512_core:
-                    case avx512_mic: res = Zmm(idx); break;
+                    case avx512_core: res = Zmm(idx); break;
                     default: assert(!"Not supported isa"); res = Xmm(idx);
                 }
                 return res;
@@ -218,9 +216,7 @@ struct rtus_driver_t : public jit_generator {
                             res = Ymm(idx);
                     }
                     break;
-                case avx512_common:
                 case avx512_core:
-                case avx512_mic:
                     switch (typesize) {
                         case 4: res = Zmm(idx); break;
                         case 2: res = Ymm(idx); break;
@@ -277,7 +273,7 @@ struct rtus_driver_t : public jit_generator {
             Label skip_h_step;
             add(reg_cur_iw, stride_w_);
             cmp(reg_cur_iw, iw_);
-            jl(skip_h_step);
+            jl(skip_h_step, T_NEAR);
 
             if (src_to_ws_) {
                 add(reg_cur_src, (src_step_h_ - iw_) * vlen_);
@@ -292,14 +288,14 @@ struct rtus_driver_t : public jit_generator {
 
                 add(reg_cur_src, stride_w_ * vlen_);
                 cmp(reg_cur_src, reg_cur_src_fin);
-                jl(ih_loop);
+                jl(ih_loop, T_NEAR);
             }
             xor_(reg_cur_iw, reg_cur_iw);
             L(skip_h_step);
         }
 
         sub(reg_cur_os, vlen_);
-        jnz(is_loop);
+        jnz(is_loop, T_NEAR);
 
         /* restore dst */
         sub(reg_ws, reg_os);
@@ -313,7 +309,7 @@ struct rtus_driver_t : public jit_generator {
         mov(reg_cur_src, reg_src);
         mov(reg_cur_iw, reg_iw_start);
 
-        if (isa == avx512_common) {
+        if (isa == avx512_core) {
             push(rcx); // preserve rcx, used for shift
             mov(reg_icb_remainder, reg_icb);
             and_(reg_icb_remainder,
@@ -331,9 +327,9 @@ struct rtus_driver_t : public jit_generator {
             }
         }
 
-        auto load_reg = [=](const Xmm &vreg, const Reg64 &reg,
+        auto load_reg = [this](const Xmm &vreg, const Reg64 &reg,
                                 const int64_t offset, const int load_size) {
-            if (isa == avx512_common) {
+            if (isa == avx512_core) {
                 const Address &addr = ptr[reg + offset];
                 switch (typesize_) {
                     case 4: vmovups(vreg, addr); break;
@@ -348,13 +344,13 @@ struct rtus_driver_t : public jit_generator {
                 if (is_ymm)
                     load_bytes(Ymm(vreg.getIdx()), reg, offset, load_size);
                 else
-                    load_bytes(vreg, reg, offset, load_size);
+                    load_bytes(Xmm(vreg.getIdx()), reg, offset, load_size);
             }
         };
 
-        auto store_reg = [=](const Reg64 &reg, const Xmm &vreg,
+        auto store_reg = [this](const Reg64 &reg, const Xmm &vreg,
                                  const int64_t offset, const int store_size) {
-            if (isa == avx512_common) {
+            if (isa == avx512_core) {
                 const Address &addr = ptr[reg + offset];
                 switch (typesize_) {
                     case 4: vmovups(addr, vreg); break;
@@ -379,9 +375,10 @@ struct rtus_driver_t : public jit_generator {
         const size_t w_step_factor = ic_ * typesize_;
         const size_t max_load_store_bytes = isa == sse41
                 ? typesize_ == 4 ? 16 : 8
-                : typesize_ == 4 ? 32 : 16;
+                : typesize_ == 4 ? 32
+                                 : 16;
         const size_t load_store_size
-                = isa == avx512_common ? vlen_ : max_load_store_bytes;
+                = isa == avx512_core ? vlen_ : max_load_store_bytes;
         size_t load_store_tail_size = (typesize_ == 1 ? max_load_store_bytes
                                                       : ic_tail_ * typesize_);
 
@@ -395,7 +392,7 @@ struct rtus_driver_t : public jit_generator {
             L(ic_loop);
             {
                 cmp(reg_cur_icb, load_store_size);
-                jl(ic_loop_tail);
+                jl(ic_loop_tail, T_NEAR);
 
                 if (src_to_ws_) {
                     load_reg(reg_v, reg_cur_src, 0, load_store_size);
@@ -411,13 +408,13 @@ struct rtus_driver_t : public jit_generator {
                 add(reg_cur_src, load_store_size);
 
                 sub(reg_cur_icb, load_store_size);
-                jmp(ic_loop);
+                jmp(ic_loop, T_NEAR);
             }
 
             L(ic_loop_tail);
             {
                 cmp(reg_cur_icb, 0);
-                je(ic_loop_finish);
+                je(ic_loop_finish, T_NEAR);
 
                 if (src_to_ws_) {
                     load_reg(reg_v | tail_mask, reg_cur_src, 0,
@@ -460,7 +457,7 @@ struct rtus_driver_t : public jit_generator {
                     mov(reg_cur_icb, reg_icb);
                     L(ic_ih_loop_nhwc);
                     cmp(reg_cur_icb, load_store_size);
-                    jl(ic_tail_ih_loop_nhwc);
+                    jl(ic_tail_ih_loop_nhwc, T_NEAR);
 
                     for (int w = 0; w < stride_w_; ++w)
                         store_reg(reg_cur_src, reg_zero, w * w_step_factor,
@@ -468,11 +465,11 @@ struct rtus_driver_t : public jit_generator {
 
                     add(reg_cur_src, load_store_size);
                     sub(reg_cur_icb, load_store_size);
-                    jnz(ic_ih_loop_nhwc);
+                    jnz(ic_ih_loop_nhwc, T_NEAR);
 
                     L(ic_tail_ih_loop_nhwc);
                     cmp(reg_cur_icb, 0);
-                    jle(ic_finish_ih_loop_nhwc);
+                    jle(ic_finish_ih_loop_nhwc, T_NEAR);
 
                     for (int w = 0; w < stride_w_; ++w)
                         store_reg(reg_cur_src, reg_zero | tail_mask,
@@ -482,21 +479,20 @@ struct rtus_driver_t : public jit_generator {
 
                     add(reg_src, stride_w_ * w_step_factor);
                     cmp(reg_src, reg_cur_src_fin);
-                    jl(ih_loop_nhwc);
+                    jl(ih_loop_nhwc, T_NEAR);
                 }
                 xor_(reg_cur_iw, reg_cur_iw);
                 L(skip_h_step);
             }
 
             sub(reg_os, 1);
-            jnz(is_loop);
+            jnz(is_loop, T_NEAR);
         }
     }
 
     void generate() override {
         using namespace Xbyak;
-        assert(utils::one_of(
-                isa, sse41, avx2, avx512_common, avx512_core, avx512_mic));
+        assert(utils::one_of(isa, sse41, avx2, avx512_core));
 
         preamble();
 #define READ_PARAM(what) \
@@ -604,7 +600,7 @@ inline int best_divider(int value, int min_divider, int max_divider,
     return x_divider;
 }
 
-typedef jit_1x1_conv_conf_t jcp_t;
+using jcp_t = jit_1x1_conv_conf_t;
 
 inline bool is_bcast_layout_nxc(const jcp_t &jcp) {
     switch (jcp.prop_kind) {
@@ -680,11 +676,13 @@ inline size_t get_load_bwd_w_offset(const jcp_t &jcp, int i, int u0) {
     }
 }
 
-inline size_t get_output_i_offset(const jcp_t &jcp) {
+inline size_t get_output_i_offset(
+        const jcp_t &jcp, bool ignore_dw_conv = false) {
     if (is_out_layout_nxc(jcp)) {
         return jcp.load_block;
     } else {
-        return (jcp.with_dw_conv ? jcp.ow : jcp.bcast_dim) * jcp.load_block;
+        return (!ignore_dw_conv && jcp.with_dw_conv ? jcp.ow : jcp.bcast_dim)
+                * jcp.load_block;
     }
 }
 
@@ -693,10 +691,10 @@ inline size_t get_output_j_offset(const jcp_t &jcp) {
 }
 
 inline size_t get_load_loop_output_fwd_offset(
-        const jcp_t &jcp, int load_loop_blk) {
+        const jcp_t &jcp, int load_loop_blk, bool ignore_dw_conv = false) {
     size_t offset = load_loop_blk * jcp.oc_block * sizeof(float);
     if (!is_out_layout_nxc(jcp)) {
-        offset *= jcp.with_dw_conv ? jcp.ow : jcp.os;
+        offset *= !ignore_dw_conv && jcp.with_dw_conv ? jcp.ow : jcp.os;
     }
     return offset;
 }
