@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2023-2024 Intel Corporation
+* Copyright 2023-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -61,12 +61,12 @@ const bcast_set_t &get_supported_bcast_strategies() {
 
 template <cpu_isa_t isa>
 struct kernel_t : public jit_uni_group_normalization_fwd_t::kernel_base_t,
-                  public jit_generator {
+                  public jit_generator_t {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_uni_group_normalization_fwd_t::kernel_t);
 
     kernel_t(const group_normalization_pd_t *pd)
         : jit_uni_group_normalization_fwd_t::kernel_base_t(pd)
-        , jit_generator(jit_name(), isa)
+        , jit_generator_t(jit_name(), isa)
         , src_d_(pd->src_md())
         , dst_d_(pd->dst_md())
         , C_(pd->C())
@@ -84,8 +84,8 @@ struct kernel_t : public jit_uni_group_normalization_fwd_t::kernel_base_t,
         with_eltwise_ = post_ops.find(primitive_kind::eltwise) != -1;
 
         const auto &attr_scales = pd->attr()->scales_;
-        with_src_scales_ = !attr_scales.get(DNNL_ARG_SRC).has_default_values();
-        with_dst_scales_ = !attr_scales.get(DNNL_ARG_DST).has_default_values();
+        with_src_scales_ = !attr_scales.has_default_values(DNNL_ARG_SRC);
+        with_dst_scales_ = !attr_scales.has_default_values(DNNL_ARG_DST);
 
         io::io_conf_t io_conf;
         io::io_tail_conf_t io_tail_conf(simd_w_, axis_simd_tail_,
@@ -112,7 +112,9 @@ struct kernel_t : public jit_uni_group_normalization_fwd_t::kernel_base_t,
                 axis_simd_tail_, use_scale_, use_shift_);
     }
 
-    status_t create_kernel() override { return jit_generator::create_kernel(); }
+    status_t create_kernel() override {
+        return jit_generator_t::create_kernel();
+    }
     void generate() override {
         const size_t c_src_size
                 = C_ * types::data_type_size(src_d_.data_type());
@@ -216,15 +218,15 @@ struct kernel_t : public jit_uni_group_normalization_fwd_t::kernel_base_t,
         args.eps = eps_;
         args.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec;
 
-        jit_generator::operator()(&args);
+        jit_generator_t::operator()(&args);
     }
 
 protected:
-    using Vmm = typename cpu_isa_traits<isa>::Vmm;
+    using Vmm = typename cpu_isa_traits_t<isa>::Vmm;
     const Xbyak::AddressFrame &vmmword = (isa == sse41) ? xword
             : (isa == avx2)                             ? yword
                                                         : zword;
-    const int vlen = cpu_isa_traits<isa>::vlen;
+    const int vlen = cpu_isa_traits_t<isa>::vlen;
 
     struct ker_args_t {
         const void *src;
@@ -385,12 +387,12 @@ template struct kernel_t<avx512_core>;
 template <cpu_isa_t isa>
 struct kernel_stat_t
     : public jit_uni_group_normalization_fwd_t::kernel_stat_base_t,
-      public jit_generator {
+      public jit_generator_t {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(
             jit_uni_group_normalization_fwd_t::kernel_stat_t);
 
     kernel_stat_t(const group_normalization_pd_t *pd, bool compute_var = false)
-        : jit_generator(jit_name())
+        : jit_generator_t(jit_name())
         , src_d_(pd->src_md())
         , compute_var_(compute_var)
         , C_(pd->C())
@@ -433,7 +435,9 @@ struct kernel_stat_t
                 unroll_c_tail_);
     }
 
-    status_t create_kernel() override { return jit_generator::create_kernel(); }
+    status_t create_kernel() override {
+        return jit_generator_t::create_kernel();
+    }
 
     void generate() override {
         preamble();
@@ -552,7 +556,7 @@ struct kernel_stat_t
         args.block_size
                 = block_size * C_ * types::data_type_size(src_d_.data_type());
 
-        jit_generator::operator()(&args);
+        jit_generator_t::operator()(&args);
     }
 
     void operator()(const void *src, const float *mean, float *var,
@@ -564,15 +568,15 @@ struct kernel_stat_t
         args.block_size
                 = block_size * C_ * types::data_type_size(src_d_.data_type());
 
-        jit_generator::operator()(&args);
+        jit_generator_t::operator()(&args);
     }
 
 protected:
-    using Vmm = typename cpu_isa_traits<isa>::Vmm;
+    using Vmm = typename cpu_isa_traits_t<isa>::Vmm;
     const Xbyak::AddressFrame &vmmword = (isa == sse41) ? xword
             : (isa == avx2)                             ? yword
                                                         : zword;
-    const int vlen = cpu_isa_traits<isa>::vlen;
+    const int vlen = cpu_isa_traits_t<isa>::vlen;
 
     struct ker_args_t {
         const void *src;
@@ -804,8 +808,8 @@ status_t jit_uni_group_normalization_fwd_t::pd_t::init(engine_t *engine) {
                                         dst_md()->data_type),
                             mayiuse(avx512_core_fp16) || mayiuse(avx2_vnni_2)),
             VERBOSE_ISA_DT_MISMATCH);
-    VDISPATCH_GNORM(attr()->has_default_values(skip_mask_t::scales_runtime
-                            | skip_mask_t::post_ops),
+    VDISPATCH_GNORM(attr()->has_default_values(
+                            skip_mask_t::scales | skip_mask_t::post_ops),
             VERBOSE_UNSUPPORTED_ATTR);
     VDISPATCH_GNORM(attr_scales_ok(), VERBOSE_UNSUPPORTED_SCALES_CFG);
     VDISPATCH_GNORM(set_default_formats_common(), VERBOSE_UNSUPPORTED_TAG);
