@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -78,7 +78,7 @@ static bcast_set_t get_supported_bcast_strategies(int ndims) {
 
 template <cpu_isa_t isa>
 struct jit_stat_and_data_base_kernel_t : stat_and_data_kernel_t,
-                                         public jit_generator {
+                                         public jit_generator_t {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_lnorm_stat_and_data_kernel_t);
 
     void operator()(const void *src, void *dst, const float *scale,
@@ -99,14 +99,16 @@ struct jit_stat_and_data_base_kernel_t : stat_and_data_kernel_t,
                 = block_size * C_ * types::data_type_size(src_d_.data_type());
         args.eps = eps_;
         args.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec;
-        jit_generator::operator()(&args);
+        jit_generator_t::operator()(&args);
     }
 
-    status_t create_kernel() override { return jit_generator::create_kernel(); }
+    status_t create_kernel() override {
+        return jit_generator_t::create_kernel();
+    }
 
     jit_stat_and_data_base_kernel_t(const layer_normalization_pd_t *pd)
         : stat_and_data_kernel_t(pd)
-        , jit_generator(jit_name(), isa)
+        , jit_generator_t(jit_name(), isa)
         , src_d_(pd_->src_md())
         , dst_d_(pd_->dst_md())
         , simd_w_(vlen / sizeof(float))
@@ -128,8 +130,8 @@ struct jit_stat_and_data_base_kernel_t : stat_and_data_kernel_t,
         with_eltwise_ = post_ops.find(primitive_kind::eltwise) != -1;
 
         const auto &attr_scales = pd_->attr()->scales_;
-        with_src_scales_ = !attr_scales.get(DNNL_ARG_SRC).has_default_values();
-        with_dst_scales_ = !attr_scales.get(DNNL_ARG_DST).has_default_values();
+        with_src_scales_ = !attr_scales.has_default_values(DNNL_ARG_SRC);
+        with_dst_scales_ = !attr_scales.has_default_values(DNNL_ARG_DST);
 
         io::io_conf_t io_conf;
         io::io_tail_conf_t io_tail_conf(simd_w_, axis_simd_tail_,
@@ -150,11 +152,11 @@ struct jit_stat_and_data_base_kernel_t : stat_and_data_kernel_t,
 
 protected:
     static constexpr int unroll_factor_ = 4;
-    using Vmm = typename cpu_isa_traits<isa>::Vmm;
+    using Vmm = typename cpu_isa_traits_t<isa>::Vmm;
     const AddressFrame &vmmword = (isa == sse41) ? xword
             : (isa == avx2)                      ? yword
                                                  : zword;
-    const int vlen = cpu_isa_traits<isa>::vlen;
+    const int vlen = cpu_isa_traits_t<isa>::vlen;
 
     struct ker_args_t {
         const void *src;
@@ -708,7 +710,7 @@ stat_and_data_kernel_t *stat_and_data_kernel_t::create(
 }
 
 template <cpu_isa_t isa>
-struct jit_diff_ss_kernel_t : diff_ss_kernel_t, public jit_generator {
+struct jit_diff_ss_kernel_t : diff_ss_kernel_t, public jit_generator_t {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_lnorm_diff_ss_kernel_t);
 
     void operator()(const void *src, const void *diff_dst, float *diff_scale,
@@ -733,14 +735,16 @@ struct jit_diff_ss_kernel_t : diff_ss_kernel_t, public jit_generator {
         args.inv_sqrtvar = inv_sqrtvar;
         args.block_size
                 = block_size * C_ * types::data_type_size(src_d_.data_type());
-        jit_generator::operator()(&args);
+        jit_generator_t::operator()(&args);
     }
 
-    status_t create_kernel() override { return jit_generator::create_kernel(); }
+    status_t create_kernel() override {
+        return jit_generator_t::create_kernel();
+    }
 
     jit_diff_ss_kernel_t(const layer_normalization_pd_t *pd)
         : diff_ss_kernel_t(pd)
-        , jit_generator(jit_name())
+        , jit_generator_t(jit_name())
         , src_d_(pd_->src_md())
         , d_dst_d_(pd_->diff_dst_md())
         , simd_w_(vlen / sizeof(float))
@@ -764,11 +768,11 @@ struct jit_diff_ss_kernel_t : diff_ss_kernel_t, public jit_generator {
     }
 
 protected:
-    using Vmm = typename cpu_isa_traits<isa>::Vmm;
+    using Vmm = typename cpu_isa_traits_t<isa>::Vmm;
     const AddressFrame &vmmword = (isa == sse41) ? xword
             : (isa == avx2)                      ? yword
                                                  : zword;
-    const int vlen = cpu_isa_traits<isa>::vlen;
+    const int vlen = cpu_isa_traits_t<isa>::vlen;
 
     struct ker_args_t {
         const void *src;
@@ -909,7 +913,8 @@ diff_ss_kernel_t *diff_ss_kernel_t::create(const layer_normalization_pd_t *pd) {
 }
 
 template <cpu_isa_t isa>
-struct jit_diff_data_base_kernel_t : diff_data_kernel_t, public jit_generator {
+struct jit_diff_data_base_kernel_t : diff_data_kernel_t,
+                                     public jit_generator_t {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_lnorm_diff_data_kernel_t);
 
     void operator()(const void *src, const void *diff_dst, void *diff_src,
@@ -924,14 +929,16 @@ struct jit_diff_data_base_kernel_t : diff_data_kernel_t, public jit_generator {
         args.inv_sqrtvar = inv_sqrtvar;
         args.block_size
                 = block_size * C_ * types::data_type_size(src_d_.data_type());
-        jit_generator::operator()(&args);
+        jit_generator_t::operator()(&args);
     }
 
-    status_t create_kernel() override { return jit_generator::create_kernel(); }
+    status_t create_kernel() override {
+        return jit_generator_t::create_kernel();
+    }
 
     jit_diff_data_base_kernel_t(const layer_normalization_pd_t *pd)
         : diff_data_kernel_t(pd)
-        , jit_generator(jit_name())
+        , jit_generator_t(jit_name())
         , src_d_(pd_->src_md())
         , d_dst_d_(pd_->diff_dst_md())
         , d_src_d_(pd_->diff_src_md())
@@ -962,11 +969,11 @@ struct jit_diff_data_base_kernel_t : diff_data_kernel_t, public jit_generator {
 
 protected:
     static constexpr int unroll_factor_ = 4;
-    using Vmm = typename cpu_isa_traits<isa>::Vmm;
+    using Vmm = typename cpu_isa_traits_t<isa>::Vmm;
     const AddressFrame &vmmword = (isa == sse41) ? xword
             : (isa == avx2)                      ? yword
                                                  : zword;
-    const int vlen = cpu_isa_traits<isa>::vlen;
+    const int vlen = cpu_isa_traits_t<isa>::vlen;
 
     struct ker_args_t {
         const void *src;
@@ -1215,8 +1222,8 @@ status_t jit_uni_layer_normalization_fwd_t::pd_t::init(engine_t *engine) {
     VDISPATCH_LNORM(stat_md()->data_type == f32, VERBOSE_UNSUPPORTED_DT);
     VDISPATCH_LNORM(check_scale_shift_data_type(), VERBOSE_UNSUPPORTED_FEATURE,
             "unsupported scale or shift data type");
-    VDISPATCH_LNORM(attr()->has_default_values(skip_mask_t::scales_runtime
-                            | skip_mask_t::post_ops),
+    VDISPATCH_LNORM(attr()->has_default_values(
+                            skip_mask_t::scales | skip_mask_t::post_ops),
             VERBOSE_UNSUPPORTED_ATTR);
     VDISPATCH_LNORM(attr_scales_ok(), VERBOSE_UNSUPPORTED_SCALES_CFG);
     VDISPATCH_LNORM(set_default_formats_common(), VERBOSE_UNSUPPORTED_TAG);
