@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 * Copyright 2018 YANDEX LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -27,27 +27,29 @@
 #include "cpu/x64/injectors/jit_uni_postops_injector.hpp"
 #include "cpu/x64/jit_generator.hpp"
 #include "cpu/x64/jit_primitive_conf.hpp"
+#include "cpu/x64/utils/jit_io_helper.hpp"
 
 namespace dnnl {
 namespace impl {
 namespace cpu {
 namespace x64 {
 
-struct bf16_emulation_t;
-
 template <cpu_isa_t isa>
-struct jit_uni_pool_kernel : public jit_generator {
+struct jit_uni_pool_kernel_t : public jit_generator_t {
 
-    jit_uni_pool_kernel(
+    jit_uni_pool_kernel_t(
             const jit_pool_conf_t &ajpp, const memory_desc_t *dst_md);
     jit_pool_conf_t jpp;
-    ~jit_uni_pool_kernel();
 
-    DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_uni_pool_kernel)
+    ~jit_uni_pool_kernel_t() override;
 
-    static status_t init_conf(jit_pool_conf_t &jbp,
-            memory_tracking::registrar_t &scratchpad, const pooling_pd_t *ppd,
-            int nthreads);
+    DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_uni_pool_kernel_t)
+
+    static status_t init_conf(jit_pool_conf_t &jpp, primitive_attr_t &attr,
+            const pooling_pd_t *ppd);
+
+    static void init_scratchpad(const jit_pool_conf_t &jpp,
+            memory_tracking::registrar_t &scratchpad);
 
 private:
     using Xmm = Xbyak::Xmm;
@@ -57,10 +59,10 @@ private:
     using Reg32 = Xbyak::Reg32;
     using Reg64 = Xbyak::Reg64;
 
-    using Vmm = typename cpu_isa_traits<isa>::Vmm;
+    using Vmm = typename cpu_isa_traits_t<isa>::Vmm;
 
     int vmm_idx_upper_bound() const noexcept {
-        return utils::one_of(isa, avx512_common, avx512_core) ? 31 : 15;
+        return is_superset(isa, avx512_core) ? 31 : 15;
     }
 
     int reg_idx(int idx) const noexcept { return vmm_idx_upper_bound() - idx; }
@@ -70,69 +72,56 @@ private:
     Zmm zreg(int idx) const noexcept { return Zmm(reg_idx(idx)); }
     Vmm vreg(int idx) const noexcept { return Vmm(reg_idx(idx)); }
 
-    const Xbyak::AddressFrame &vmmword = (isa == sse41)
-            ? xword
-            : (isa == avx || isa == avx2) ? yword : zword;
+    const Xbyak::AddressFrame &vmmword = (isa == sse41)  ? xword
+            : utils::one_of(isa, avx, avx2, avx2_vnni_2) ? yword
+                                                         : zword;
 
     Xmm vmm_mask = Xmm(0);
+    Xmm xmm_tmp_1 = Xmm(0);
     Ymm ymm_tmp_1 = Ymm(0);
     Vmm vmm_tmp_1 = Vmm(0);
 
-    // Used only for avx and if c tail is present
+    // Used only for avx and if c tail is present; is shared with jit_io_multi_dt_helper_t
     Vmm vmm_c_tail_mask = Vmm(2);
 
-    Xmm xmm_ker_area_h = Xmm(2);
-    Xmm xmm_one = Xmm(2);
-    Xmm xmm_tmp = Xmm(3);
-
     Vmm vmm_ker_area_h = Vmm(2);
     Vmm vmm_one = Vmm(2);
     Vmm vmm_tmp = Vmm(3);
-    Ymm ymm_tmp = Ymm(3);
+    Xmm xmm_tmp = Xmm(3);
 
     Vmm vmm_k_offset = Vmm(1);
 
-    // Used only for avx512 when bf16 is present
-    inline Vmm vmm_idx() {
-        if (!jpp.is_backward) {
-            return (jpp.is_training) ? Vmm(4) : Vmm(1);
-        } else
-            return Vmm(4);
-    }
-
     Zmm bf16_emu_reserv_1 = Zmm(5);
     Zmm bf16_emu_reserv_2 = Zmm(6);
     Zmm bf16_emu_reserv_3 = Zmm(7);
     Reg64 bf16_emu_reserv_4 = r11;
     Zmm bf16_emu_reserv_5 = Zmm(8);
 
-    Opmask k_c_tail_mask = Opmask(4);
-    Opmask k_mask_cvt = Opmask(5);
-    Opmask k_store_mask = Opmask(6);
+    Zmm fp8_emu_reserv_1 = Zmm(5);
+    Zmm fp8_emu_reserv_2 = Zmm(6);
+    Zmm fp8_emu_reserv_3 = Zmm(7);
+    Zmm fp8_emu_reserv_4 = Zmm(8);
+    Zmm fp8_emu_reserv_5 = Zmm(9);
+    Reg64 fp8_emu_reg64 = bf16_emu_reserv_4;
+    Xbyak::Opmask fp8_tmp_mask = Xbyak::Opmask(3);
 
-    // Here be some (tame) dragons. This kernel does not follow the regular
-    // OS-agnostic ABI pattern because when isa is sse41 it uses maskmovdqu
-    // instruction which has its destination hardcoded in rdi. Therefore:
-    // - all registers are hardcoded
-    // - on Windows rdi and rcx are swapped to mimic the Unix x86_64 ABI
-    //
-    // While this is only required by the backward pass, the quirk above
-    // is applied to the forward pass as well to keep things simpler.
+    // k_c_tail_mask is shared with jit_io_multi_dt_helper_t and jit_uni_postops_injector_t
+    Opmask k_c_tail_mask = Opmask(4);
+    Opmask k_store_mask = Opmask(5);
 
     using reg64_t = const Reg64;
-    reg64_t reg_param = rdi; // Always mimic the Unix ABI
+    reg64_t reg_param = abi_param1;
     reg64_t reg_input = r8;
     reg64_t aux_reg_input = r9;
     reg64_t reg_index = r10;
     reg64_t reg_output = r12;
     reg64_t reg_kd_pad_shift = r13;
-    reg64_t dst_ptr = rdi; // Must be rdi due to maskmovdqu
 
     reg64_t kj = r14;
     reg64_t oi_iter = r15;
     reg64_t reg_kh = rax;
     reg64_t reg_k_shift = rbx;
-    reg64_t tmp_gpr = rcx; // Must be rcx because rdi is used above
+    reg64_t tmp_gpr = abi_not_param1;
     reg64_t reg_ker_area_h = rdx;
     reg64_t reg_nbc = rsi;
 
@@ -150,15 +139,18 @@ private:
 
     int prev_kw;
 
-    void prepare_tail_mask();
     void put_one_in_vmm();
     void uni_broadcast_reg_val(const int reg_idx, const int vmm_idx);
     void push_vmm_val(const int idx);
     void pop_vmm_val(const int idx);
-    void load(const int idx, const reg64_t &reg_ptr, const int offset,
-            const bool is_c_tail_proccessing);
-    void store(const int idx, const reg64_t &reg_ptr, const int offset,
-            const bool is_c_tail_proccessing);
+    void load(const data_type_t dt, const int idx, const reg64_t &reg_ptr,
+            const int offset, const bool is_c_tail_proccessing);
+    void store(const data_type_t dt, const int idx, const reg64_t &reg_ptr,
+            const int offset, const bool is_c_tail_proccessing);
+    void pad_with_zeros(int idx);
+    void load_indices(int indr_i, int step_index, bool is_c_tail_processing);
+    void store_indices(int indr_i, int step_index, bool is_c_tail_processing,
+            bool is_first_w_block);
 
     void maybe_recalculate_divisor(int jj, int ur_w, int pad_l, int pad_r,
             bool with_c_tail_proccessing);
@@ -189,8 +181,13 @@ private:
         add(reg_input, sizeof(float) * 4);
         add(reg_output, sizeof(float) * 4);
         if (jpp.alg == alg_kind::pooling_max
-                && (jpp.is_training || jpp.is_backward))
+                && (jpp.is_training || jpp.is_backward)) {
+            if (jpp.ind_dt == data_type::undef) {
+                assert(!"Unknown data type for indices");
+                return;
+            }
             add(reg_index, types::data_type_size(jpp.ind_dt) * 4);
+        }
 
         step(ur_w, ur_bc, pad_l, pad_r, with_c_tail_processing);
     }
@@ -250,9 +247,21 @@ private:
     static bool post_ops_ok(jit_pool_conf_t &jpp, const primitive_attr_t &attr,
             const memory_desc_wrapper &dst_d);
 
-    std::unique_ptr<bf16_emulation_t> bf16_emu_;
+    inline bool use_bf16_emulation() const {
+        return jpp.is_bf16 && !isa_has_bf16(jpp.isa) && isa != avx2_vnni_2;
+    }
+
+    inline bool use_fp8_emulation() const {
+        return jpp.is_fp8 && is_superset(isa, avx512_core_fp16);
+    }
+
+    static bool has_large_buffers(const pooling_pd_t *ppd);
+
+    std::unique_ptr<fp8_emulation_e5m2_t> f8_e5m2_emu_;
+    std::unique_ptr<fp8_emulation_e4m3_t> f8_e4m3_emu_;
     std::unique_ptr<injector::jit_uni_postops_injector_t<isa>>
             postops_injector_;
+    io::jit_io_multi_dt_helper_t<Vmm> io_;
 };
 
 } // namespace x64
