@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
 #ifndef CPU_X64_JIT_UNI_X8S8S32X_1X1_CONV_KERNEL_HPP
 #define CPU_X64_JIT_UNI_X8S8S32X_1X1_CONV_KERNEL_HPP
 
+#include <memory>
 #include "common/c_types_map.hpp"
 #include "common/memory_tracking.hpp"
 
@@ -30,7 +31,7 @@ namespace cpu {
 namespace x64 {
 
 template <cpu_isa_t isa, typename Vmm>
-struct _jit_uni_x8s8s32x_1x1_conv_kernel : public jit_generator {
+struct _jit_uni_x8s8s32x_1x1_conv_kernel : public jit_generator_t {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(_jit_uni_x8s8s32x_1x1_conv_kernel)
     _jit_uni_x8s8s32x_1x1_conv_kernel(const jit_1x1_conv_conf_t &ajcp,
             const primitive_attr_t &attr, const memory_desc_t &dst_md);
@@ -52,9 +53,11 @@ private:
     const Xbyak::Reg64 reg_output_data = r9;
     const Xbyak::Reg64 reg_load_data = r10;
     const Xbyak::Reg64 reg_ptr_sum_scale = r10;
+    const Xbyak::Reg64 reg_ptr_sum_zp = rdx;
     const Xbyak::Reg64 reg_reduce_loop_work = r11;
     const Xbyak::Reg64 reg_bias_data = r12;
     const Xbyak::Reg64 reg_comp_data = r12;
+    const Xbyak::Reg64 reg_ptr_dst_scale = r12;
     const Xbyak::Reg64 reg_init_bcast = r13;
     const Xbyak::Reg64 reg_store_bcast = r13;
     const Xbyak::Reg64 reg_reduce_loop_iter = r13;
@@ -88,6 +91,8 @@ private:
     /* zero-point */
     const Vmm vmm_zp = Vmm(1);
     const Vmm vmm_zp_comp = Vmm(2);
+    /* dst scale */
+    const Vmm vmm_dst_scale = Vmm(1);
 
     constexpr static int simd_w = isa == avx2 ? 8 : 4;
     constexpr static int reg64_size = sizeof(int64_t);
@@ -96,12 +101,13 @@ private:
     constexpr static int reg_bcast_data_off = 2 * reg64_size;
     constexpr static int reg_load_data_off = 3 * reg64_size;
     constexpr static int reg_ptr_sum_scale_off = 4 * reg64_size;
-    constexpr static int reg_comp_data_off = 5 * reg64_size;
-    constexpr static int reg_zp_compensation_off = 6 * reg64_size;
-    constexpr static int reg_src_zero_point_off = 7 * reg64_size;
-    constexpr static int reg_dst_zero_point_off = 8 * reg64_size;
-    constexpr static int reg_binary_post_op_acc_off = 9 * reg64_size;
-    constexpr static int stack_space_needed = 10 * reg64_size;
+    constexpr static int reg_bcast_loop_iter_off = 5 * reg64_size;
+    constexpr static int reg_comp_data_off = 6 * reg64_size;
+    constexpr static int reg_zp_compensation_off = 7 * reg64_size;
+    constexpr static int reg_src_zero_point_off = 8 * reg64_size;
+    constexpr static int reg_dst_zero_point_off = 9 * reg64_size;
+    constexpr static int reg_dst_scale_off = 10 * reg64_size;
+    constexpr static int stack_space_needed = 11 * reg64_size;
 
     int vreg_accum_idx(
             const int load_loop_blk, const int i_load, const int i_ur);
@@ -109,10 +115,12 @@ private:
     int output_ptr(const int i_load, const int i_ur);
     void bcast_loop(int load_loop_blk);
     void apply_sum(const int ur, const int load_loop_blk,
-            const bool mask_flag_in, const float *p_sum_scale);
+            const bool mask_flag_in, const float *p_sum_scale,
+            const int32_t *p_sum_zp);
     void apply_postops(const int ur, const int load_loop_blk,
-            const bool mask_flag_in, const float *p_sum_scale);
-    void reduce_loop(int load_loop_blk, int ur, int substep, bool wraparound);
+            const bool mask_flag_in, const float *p_sum_scale,
+            const int32_t *p_sum_zp);
+    void reduce_loop(int load_loop_blk, int ur, bool wraparound);
 
     void generate() override;
     void cvt2ps(data_type_t type_in, const Vmm &vmm_in, const Xbyak::Reg64 &reg,
@@ -128,20 +136,23 @@ struct jit_uni_x8s8s32x_1x1_conv_kernel {
 
         switch (isa) {
             case avx2:
-                kernel_ = new jit_avx2_x8s8s32x_1x1_conv_kernel(
+                kernel_ = utils::make_unique<jit_avx2_x8s8s32x_1x1_conv_kernel>(
                         ajcp, attr, dst_md);
                 return;
             case sse41:
-                kernel_ = new jit_sse41_x8s8s32x_1x1_conv_kernel(
-                        ajcp, attr, dst_md);
+                kernel_ = utils::make_unique<
+                        jit_sse41_x8s8s32x_1x1_conv_kernel>(ajcp, attr, dst_md);
                 return;
             default: assert(!"Current ISA is not supported!");
         }
     }
 
-    status_t create_kernel() { return kernel_->create_kernel(); }
+    status_t create_kernel() {
+        if (kernel_) return kernel_->create_kernel();
+        return status::out_of_memory;
+    }
 
-    ~jit_uni_x8s8s32x_1x1_conv_kernel() { delete kernel_; }
+    ~jit_uni_x8s8s32x_1x1_conv_kernel() = default;
 
     void operator()(const jit_1x1_conv_call_s *p) const { (*kernel_)(p); }
 
@@ -149,7 +160,7 @@ struct jit_uni_x8s8s32x_1x1_conv_kernel {
             const convolution_desc_t &cd, const memory_desc_wrapper &src_d,
             const memory_desc_wrapper &weights_d,
             const memory_desc_wrapper &dst_d, const memory_desc_wrapper &bias_d,
-            const primitive_attr_t &attr, int nthreads, bool reduce_src);
+            primitive_attr_t &attr, int nthreads, bool reduce_src);
 
     static void init_scratchpad(memory_tracking::registrar_t &scratchpad,
             const jit_1x1_conv_conf_t &jcp, const primitive_attr_t &attr);
@@ -163,7 +174,7 @@ struct jit_uni_x8s8s32x_1x1_conv_kernel {
 
 private:
     DNNL_DISALLOW_COPY_AND_ASSIGN(jit_uni_x8s8s32x_1x1_conv_kernel);
-    jit_generator *kernel_;
+    std::unique_ptr<jit_generator_t> kernel_;
 };
 
 } // namespace x64
