@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
 #ifndef CPU_X64_JIT_UNI_X8S8S32X_CONV_KERNEL_HPP
 #define CPU_X64_JIT_UNI_X8S8S32X_CONV_KERNEL_HPP
 
+#include <memory>
 #include "common/c_types_map.hpp"
 #include "common/memory_tracking.hpp"
 
@@ -30,7 +31,7 @@ namespace cpu {
 namespace x64 {
 
 template <cpu_isa_t isa, typename Vmm>
-struct _jit_uni_x8s8s32x_fwd_kernel : public jit_generator {
+struct _jit_uni_x8s8s32x_fwd_kernel : public jit_generator_t {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(_jit_uni_x8s8s32x_conv_fwd_ker_t_)
 
     _jit_uni_x8s8s32x_fwd_kernel(const jit_conv_conf_t &ajcp,
@@ -41,7 +42,7 @@ struct _jit_uni_x8s8s32x_fwd_kernel : public jit_generator {
 
 private:
     constexpr static int isa_simd_width_
-            = cpu_isa_traits<isa>::vlen / sizeof(float);
+            = cpu_isa_traits_t<isa>::vlen / sizeof(float);
     std::unique_ptr<injector::jit_uni_postops_injector_t<isa>>
             postops_injector_;
     enum {
@@ -66,6 +67,7 @@ private:
     const Xbyak::Reg64 reg_out = r10;
     const Xbyak::Reg64 aux_reg_inp = r11;
     const Xbyak::Reg64 reg_ptr_sum_scale = r11;
+    const Xbyak::Reg64 reg_ptr_sum_zp = rdx;
     const Xbyak::Reg64 aux_reg_ker = r12;
     const Xbyak::Reg64 aux_reg_inp_d = r13;
     const Xbyak::Reg64 reg_compensation = r14;
@@ -73,7 +75,6 @@ private:
     const Xbyak::Reg64 reg_ker_long_offt = r13;
 
     /* counter regs */
-    const Xbyak::Reg64 reg_bias_alpha = abi_not_param1;
     const Xbyak::Reg64 reg_oi = rbx;
     const Xbyak::Reg64 reg_bias = rdx;
     const Xbyak::Reg64 reg_oc_blocks = rsi;
@@ -91,6 +92,11 @@ private:
     const Xbyak::Reg64 reg_zp_compensation = aux_reg_inp;
     const Xbyak::Reg64 reg_src_zero_point = aux_reg_ker_d;
     const Xbyak::Reg64 reg_dst_zero_point = reg_src_zero_point;
+    // dst scale
+    const Xbyak::Reg64 reg_dst_scale = reg_dst_zero_point;
+
+    /* binary post-ops operand */
+    const Xbyak::Reg64 temp_offset_reg = r12;
 
     const Vmm vmm_wei = Vmm(0);
     /* used during bias/comp/scale section of store_output */
@@ -107,6 +113,8 @@ private:
     const Vmm vmm_zp_one = Vmm(5);
     const Vmm vmm_zp_comp = vmm_zp_one;
     const Vmm vmm_zp_dw_tmp = vmm_zp_one;
+    /* dst scale */
+    const Vmm vmm_dst_scale = Vmm(0);
 
     /* used in compute_ker (but set during prepare_output) */
     const Vmm vmm_shift = Vmm(1); // only for signed input
@@ -119,8 +127,7 @@ private:
     Vmm vmm_dw_src;
 
     int vmm_out_idx(int i_ur, int i_oc) {
-        const int idx_limit = jcp.src_zero_point
-                ? ker_zp_reg_base_idx
+        const int idx_limit = jcp.src_zero_point ? ker_zp_reg_base_idx
                 : jcp.is_depthwise ? ker_dw_reg_base_idx - jcp.signed_input
                                    : ker_reg_base_idx;
         const int nb_x_blocking
@@ -141,16 +148,6 @@ private:
         assert(idx < ker_max_reg);
         return Vmm(ker_max_reg - idx);
     }
-    Vmm vmm_bias_alpha() {
-        int nb_c_block
-                = jcp.is_depthwise ? jcp.nb_ch_blocking : jcp.nb_oc_blocking;
-        return Vmm(ker_max_reg - nb_c_block * jcp.ur_w);
-    }
-    Xbyak::Xmm xmm_bias_alpha() {
-        int nb_c_block
-                = jcp.is_depthwise ? jcp.nb_ch_blocking : jcp.nb_oc_blocking;
-        return Xbyak::Xmm(ker_max_reg - nb_c_block * jcp.ur_w);
-    }
     int get_ow_start(int ki, int pad_l) {
         return nstl::max(0,
                 utils::div_up(pad_l - ki * (jcp.dilate_w + 1), jcp.stride_w));
@@ -184,10 +181,10 @@ private:
             int offset, int load_size);
     void apply_sum(const int nb_oc_block, const int ur_w,
             const bool last_oc_block_flag, const int oc_block,
-            const float *p_sum_scale);
+            const float *p_sum_scale, const int32_t *p_sum_zp);
     void apply_postops(const int nb_oc_block, const int ur_w,
             const bool last_oc_block_flag, const int oc_block,
-            const float *p_sum_scale);
+            const float *p_sum_scale, const int32_t *p_sum_zp);
 };
 
 template <cpu_isa_t isa>
@@ -200,37 +197,42 @@ struct jit_uni_x8s8s32x_fwd_kernel {
         switch (ch_block) {
             case 8:
                 if (utils::one_of(isa, avx2)) {
-                    kernel_ = new _jit_uni_x8s8s32x_fwd_kernel<isa, Xbyak::Ymm>(
+                    kernel_ = utils::make_unique<
+                            _jit_uni_x8s8s32x_fwd_kernel<isa, Xbyak::Ymm>>(
                             ajcp, attr, dst_md);
                 } else
                     assert(!"invalid channel blocking for current ISA");
                 return;
             case 4:
-                kernel_ = new _jit_uni_x8s8s32x_fwd_kernel<isa, Xbyak::Xmm>(
+                kernel_ = utils::make_unique<
+                        _jit_uni_x8s8s32x_fwd_kernel<isa, Xbyak::Xmm>>(
                         ajcp, attr, dst_md);
                 return;
             default: assert(!"invalid channel blocking");
         }
     }
 
-    status_t create_kernel() { return kernel_->create_kernel(); }
+    status_t create_kernel() {
+        if (kernel_) return kernel_->create_kernel();
+        return status::out_of_memory;
+    }
 
-    ~jit_uni_x8s8s32x_fwd_kernel() { delete kernel_; }
+    ~jit_uni_x8s8s32x_fwd_kernel() = default;
 
     void operator()(const jit_conv_call_s *p) const { (*kernel_)(p); }
 
     static status_t init_conf(jit_conv_conf_t &jcp,
             const convolution_desc_t &cd, memory_desc_t &src_pd,
             memory_desc_t &weights_pd, memory_desc_t &dst_pd,
-            memory_desc_t &bias_pd, const primitive_attr_t &attr, int nthreads);
+            memory_desc_t &bias_pd, primitive_attr_t &attr, int nthreads);
     static void init_scratchpad(memory_tracking::registrar_t &scratchpad,
             const jit_conv_conf_t &jcp, const primitive_attr_t &attr);
 
     void (*jit_ker)(jit_conv_call_s *);
 
 private:
-    DNNL_DISALLOW_COPY_AND_ASSIGN(jit_uni_x8s8s32x_fwd_kernel<isa>);
-    jit_generator *kernel_;
+    DNNL_DISALLOW_COPY_AND_ASSIGN(jit_uni_x8s8s32x_fwd_kernel);
+    std::unique_ptr<jit_generator_t> kernel_;
 };
 
 } // namespace x64
