@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -32,70 +32,83 @@ namespace impl {
 namespace cpu {
 namespace x64 {
 
-template <cpu_isa_t isa, impl::data_type_t src_type, impl::data_type_t dst_type>
+template <cpu_isa_t isa>
 struct jit_uni_x8s8s32x_convolution_fwd_t : public primitive_t {
     struct pd_t : public cpu_convolution_fwd_pd_t {
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const typename pd_t::base_class *hint_fwd_pd)
-            : cpu_convolution_fwd_pd_t(adesc, attr, hint_fwd_pd), jcp_() {}
+        using cpu_convolution_fwd_pd_t::cpu_convolution_fwd_pd_t;
 
         DECLARE_COMMON_PD_T(
                 JIT_IMPL_NAME_HELPER("jit_uni_int8:",
-                        isa == avx2 && jcp_.ver == ver_vnni ? avx2_vnni : isa,
-                        ""),
+                        isa == avx2 && jcp_.has_vnni ? avx2_vnni : isa, ""),
                 jit_uni_x8s8s32x_convolution_fwd_t);
 
         status_t init(engine_t *engine) {
+            using namespace data_type;
             using smask_t = primitive_attr_t::skip_mask_t;
-            const bool args_ok = true && is_fwd()
-                    && set_default_alg_kind(alg_kind::convolution_direct)
-                    && expect_data_types(src_type, data_type::s8,
-                            data_type::undef, dst_type, data_type::s32)
-                    && IMPLICATION(with_bias(),
-                            utils::one_of(bias_md_.data_type, data_type::f32,
-                                    data_type::s32, data_type::s8,
-                                    data_type::u8))
-                    && attr()->has_default_values(smask_t::oscale
-                                    | smask_t::zero_points_runtime
-                                    | smask_t::post_ops,
-                            dst_type)
-                    && !has_zero_dim_memory() && zero_points_ok();
-            if (!args_ok) return status::unimplemented;
+            VDISPATCH_CONV(is_fwd(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_CONV(utils::one_of(src_md(0)->data_type, s8, u8),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(
+                    weights_md(0)->data_type == s8, VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(
+                    utils::one_of(dst_md(0)->data_type, f32, s32, s8, u8),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(IMPLICATION(with_bias(),
+                                   utils::one_of(weights_md(1)->data_type, f32,
+                                           s32, s8, u8)),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(
+                    desc()->accum_data_type == s32, VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_CONV(set_default_alg_kind(alg_kind::convolution_direct),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_CONV(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+            VDISPATCH_CONV(
+                    attr()->has_default_values(smask_t::scales
+                                    | smask_t::zero_points | smask_t::post_ops
+                                    | smask_t::sum_dt,
+                            dst_md(0)->data_type),
+                    VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_CONV(attr()->post_ops_.check_sum_consistency(
+                                   dst_md(0)->data_type, /* is_int8 */ true),
+                    VERBOSE_UNSUPPORTED_POSTOP);
+            VDISPATCH_CONV(attr_scales_ok(), VERBOSE_UNSUPPORTED_SCALES_CFG);
+            VDISPATCH_CONV(zero_points_ok(), VERBOSE_UNSUPPORTED_ZP_CFG);
 
-            const auto status = jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(
-                    jcp_, *desc(), src_md_, weights_md_, dst_md_, bias_md_,
-                    *attr(), dnnl_get_max_threads());
-            if (status != status::success) return status;
+            // TODO: make `init_conf` assign initialized object to `jcp_`
+            CHECK(jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jcp_, *desc(),
+                    src_md_, weights_md_, dst_md_, bias_md_, attr_,
+                    dnnl_get_max_threads()));
 
             auto scratchpad = scratchpad_registry().registrar();
             jit_uni_x8s8s32x_fwd_kernel<isa>::init_scratchpad(
                     scratchpad, jcp_, *attr());
 
-            return status;
+            return attr_.set_default_formats(dst_md(0));
         }
 
-        jit_conv_conf_t jcp_;
+        jit_conv_conf_t jcp_ = utils::zero<decltype(jcp_)>();
 
     protected:
         bool zero_points_ok() const {
-            using namespace data_type;
-            int mask_src = 0, mask_dst = 0;
-            const int c_mask = 0x1,
-                      g_mask = 0x3; // mask for i/o-channel and ngroups
-            attr()->zero_points_.get(DNNL_ARG_SRC, nullptr, &mask_src, nullptr);
-            attr()->zero_points_.get(DNNL_ARG_DST, nullptr, &mask_dst, nullptr);
-            return attr()->zero_points_.has_default_values(DNNL_ARG_WEIGHTS)
-                    && utils::one_of(mask_src, 0, c_mask, g_mask)
-                    && utils::one_of(mask_dst, 0, c_mask, g_mask);
+            const auto &zp = attr()->zero_points_;
+
+            if (!zp.has_default_values(DNNL_ARG_SRC)) {
+                int mask_src = zp.get_mask(DNNL_ARG_SRC);
+                const bool ok = mask_src == 0;
+                if (!ok) return false;
+            }
+            if (!zp.has_default_values(DNNL_ARG_DST)) {
+                int mask_dst = zp.get_mask(DNNL_ARG_DST);
+                const bool ok = mask_dst == 0;
+                if (!ok) return false;
+            }
+
+            return zp.has_default_values(DNNL_ARG_WEIGHTS);
         }
     };
 
     jit_uni_x8s8s32x_convolution_fwd_t(const pd_t *apd) : primitive_t(apd) {}
 
-    typedef typename prec_traits<src_type>::type src_data_t;
-    typedef typename prec_traits<data_type::s8>::type wei_data_t;
-    typedef typename prec_traits<dst_type>::type dst_data_t;
-
     status_t init(engine_t *engine) override {
         CHECK(safe_ptr_assign(kernel_,
                 new jit_uni_x8s8s32x_fwd_kernel<isa>(
@@ -126,6 +139,8 @@ private:
     const pd_t *pd() const {
         return static_cast<const pd_t *>(primitive_t::pd().get());
     }
+    const float *adjust_oscales(const memory_tracking::grantor_t &scratchpad,
+            const float *src_scales, const float *wei_scales) const;
 
     std::unique_ptr<jit_uni_x8s8s32x_fwd_kernel<isa>> kernel_;
 };
