@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2024 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -50,7 +50,7 @@ template <template <cpu_isa_t isa, data_type_t d_type> class Derived,
         cpu_isa_t isa, data_type_t d_type>
 jit_uni_lrn_kernel_t<Derived<isa, d_type>>::jit_uni_lrn_kernel_t(
         const char *name)
-    : jit_generator(name, isa)
+    : jit_generator_t(name, isa)
     , emulate_bfloat_(d_type == data_type::bf16 && !mayiuse(avx512_core_bf16)
               && is_superset(isa, avx512_core))
     , bf16_emu_(
@@ -71,7 +71,7 @@ jit_uni_lrn_kernel_t<Derived<isa, d_type>>::jit_uni_lrn_kernel_t(
     : jit_uni_lrn_kernel_t(name) {
     if (config.dat_tag == nhwc)
         single_pixel_offset_
-                = config.C * sizeof(typename prec_traits<d_type>::type);
+                = config.C * sizeof(typename prec_traits_t<d_type>::type);
 }
 
 template <template <cpu_isa_t isa, data_type_t d_type> class Derived,
@@ -625,7 +625,7 @@ void jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::generate(
     if (pk_ != prop_kind::forward_inference) add(scratch_, 32);
     this->dec(hw);
     this->cmp(hw, 0);
-    this->jne(lrn_loop, this->T_NEAR);
+    this->jne(lrn_loop, jit_generator_t::T_NEAR);
 
     this->add(t, 64);
     this->postamble();
@@ -722,7 +722,7 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::generate(const nhwc_across_t &J) {
 
     this->dec(c);
     this->cmp(c, 0);
-    this->jne(lrn_loop, this->T_NEAR);
+    this->jne(lrn_loop, jit_generator_t::T_NEAR);
 
     this->vmovups(yc, this->ptr[src_]);
     this->vfmadd231ps(ysum, yc, yc);
@@ -912,7 +912,7 @@ void jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::generate(
 
     this->dec(c);
     this->cmp(c, 0);
-    this->jne(lrn_loop, this->T_NEAR);
+    this->jne(lrn_loop, jit_generator_t::T_NEAR);
 
     /* compute last 3 blocks of channels:
      * block:       | -- low -- | -- hi --  |
@@ -1233,7 +1233,7 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::generate(const nchw_across_t &J) {
     if (pk_ != prop_kind::forward_inference) this->add(scratch_, J.HW * 4);
     this->dec(c);
     this->cmp(c, 0);
-    this->jne(lrn_loop, this->T_NEAR);
+    this->jne(lrn_loop, jit_generator_t::T_NEAR);
 
     this->vxorps(ye, ye, ye);
 
@@ -1308,7 +1308,7 @@ void jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::generate(
     const Xbyak::Xmm &xe_lo = this->xmm14;
     const Xbyak::Xmm &xe_hi = this->xmm15;
 
-    const int vlen = cpu_isa_traits<sse41>::vlen / sizeof(float);
+    const int vlen = cpu_isa_traits_t<sse41>::vlen / sizeof(float);
 
     bool compute_tail = J.tail != 0;
     bool load_lo = J.tail == 0 || J.tail > 4;
@@ -1410,7 +1410,7 @@ void jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::generate(
     if (pk_ != prop_kind::forward_inference) add(scratch_, J.HW * 4);
     this->dec(c);
     this->cmp(c, 0);
-    this->jne(lrn_loop, this->T_NEAR);
+    this->jne(lrn_loop, jit_generator_t::T_NEAR);
 
     this->xorps(xe_lo, xe_lo);
     this->xorps(xe_hi, xe_hi);
