@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -67,7 +67,7 @@ public:
         }
     }
 
-    using data_t = typename prec_traits<d_type>::type;
+    using data_t = typename prec_traits_t<d_type>::type;
 
     status_t create_kernel() override {
         CHECK(ker_->create_kernel());
@@ -109,8 +109,8 @@ public:
                             d_type>::jit_args_fwd_t args;
                     args.src = &src[offset];
                     args.dst = &dst[offset];
-                    args.ws0 = &ws[ws_offset0];
-                    args.ws1 = &ws[ws_offset1];
+                    args.ws0 = ws ? &ws[ws_offset0] : nullptr;
+                    args.ws1 = ws ? &ws[ws_offset1] : nullptr;
 
                     if (C16 == 1)
                         (*ker)(&args);
@@ -136,8 +136,8 @@ public:
                             d_type>::jit_args_fwd_t args;
                     args.src = &src[offset];
                     args.dst = &dst[offset];
-                    args.ws0 = &ws[ws_offset0];
-                    args.ws1 = &ws[ws_offset1];
+                    args.ws0 = ws ? &ws[ws_offset0] : nullptr;
+                    args.ws1 = ws ? &ws[ws_offset1] : nullptr;
 
                     if (C16 == 1)
                         (*ker)(&args);
@@ -205,7 +205,7 @@ public:
         }
     }
 
-    using data_t = typename prec_traits<d_type>::type;
+    using data_t = typename prec_traits_t<d_type>::type;
 
     status_t create_kernel() override {
         CHECK(ker_->create_kernel());
@@ -248,8 +248,8 @@ public:
                             d_type>::jit_args_bwd_t args;
                     args.src = &src[offset];
                     args.diff_dst = &diff_dst[offset];
-                    args.ws0 = &ws[ws_offset0];
-                    args.ws1 = &ws[ws_offset1];
+                    args.ws0 = ws ? &ws[ws_offset0] : nullptr;
+                    args.ws1 = ws ? &ws[ws_offset1] : nullptr;
                     args.diff_src = &diff_src[offset];
 
                     if (C16 == 1)
@@ -276,8 +276,8 @@ public:
                             d_type>::jit_args_bwd_t args;
                     args.src = &src[offset];
                     args.diff_dst = &diff_dst[offset];
-                    args.ws0 = &ws[ws_offset0];
-                    args.ws1 = &ws[ws_offset1];
+                    args.ws0 = ws ? &ws[ws_offset0] : nullptr;
+                    args.ws1 = ws ? &ws[ws_offset1] : nullptr;
                     args.diff_src = &diff_src[offset];
 
                     if (C16 == 1)
@@ -314,4 +314,4 @@ private:
 } // namespace impl
 } // namespace dnnl
 
-#endif
\ No newline at end of file
+#endif
