@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -42,7 +42,7 @@ public:
         , H_(pd->H())
         , W_(pd->W()) {}
 
-    using data_t = typename prec_traits<d_type>::type;
+    using data_t = typename prec_traits_t<d_type>::type;
 
     status_t create_kernel() override { return ker_->create_kernel(); }
 
@@ -55,7 +55,7 @@ public:
         CHECK(status);
 
         const auto ker = ker_.get();
-        parallel_nd(N_, H_ * W_, [&](int n, int pixel_id) {
+        parallel_nd(N_, H_ * W_, [&](dim_t n, dim_t pixel_id) {
             typename lrn::jit_avx512_common_lrn_kernel_fwd_t<
                     d_type>::jit_args_fwd_t args;
             const auto offset = n * C_ * H_ * W_ + pixel_id * C_;
@@ -64,8 +64,8 @@ public:
 
             args.src = &src[offset];
             args.dst = &dst[offset];
-            args.ws0 = &ws[ws_offset0];
-            args.ws1 = &ws[ws_offset1];
+            args.ws0 = ws ? &ws[ws_offset0] : nullptr;
+            args.ws1 = ws ? &ws[ws_offset1] : nullptr;
 
             (*ker)(&args);
         });
@@ -73,14 +73,14 @@ public:
         return status::success;
     }
 
-    virtual ~lrn_avx512_nhwc_executor_fwd_t() = default;
+    ~lrn_avx512_nhwc_executor_fwd_t() override = default;
 
 private:
     std::unique_ptr<jit_avx512_common_lrn_kernel_fwd_nhwc_t<d_type>> ker_;
-    const int N_;
-    const int C_;
-    const int H_;
-    const int W_;
+    const dim_t N_;
+    const dim_t C_;
+    const dim_t H_;
+    const dim_t W_;
 };
 template <::dnnl::impl::data_type_t d_type, typename PD_T>
 class lrn_avx512_nhwc_executor_bwd_t : public i_lrn_executor_t {
@@ -94,7 +94,7 @@ public:
         , C_(pd->C())
         , H_(pd->H())
         , W_(pd->W()) {}
-    using data_t = typename prec_traits<d_type>::type;
+    using data_t = typename prec_traits_t<d_type>::type;
 
     status_t create_kernel() override { return ker_->create_kernel(); }
 
@@ -107,7 +107,7 @@ public:
         auto ws = CTX_IN_MEM(data_t *, DNNL_ARG_WORKSPACE);
 
         const auto ker = ker_.get();
-        parallel_nd(N_, H_ * W_, [&](int n, int pixel_id) {
+        parallel_nd(N_, H_ * W_, [&](dim_t n, dim_t pixel_id) {
             typename lrn::jit_avx512_common_lrn_kernel_bwd_nhwc_t<
                     d_type>::jit_args_bwd_t args;
             const auto offset = n * C_ * H_ * W_ + pixel_id * C_;
@@ -126,14 +126,14 @@ public:
         return status::success;
     }
 
-    virtual ~lrn_avx512_nhwc_executor_bwd_t() = default;
+    ~lrn_avx512_nhwc_executor_bwd_t() override = default;
 
 private:
     std::unique_ptr<jit_avx512_common_lrn_kernel_bwd_nhwc_t<d_type>> ker_;
-    const int N_;
-    const int C_;
-    const int H_;
-    const int W_;
+    const dim_t N_;
+    const dim_t C_;
+    const dim_t H_;
+    const dim_t W_;
 };
 
 } // namespace lrn
@@ -142,4 +142,4 @@ private:
 } // namespace impl
 } // namespace dnnl
 
-#endif
\ No newline at end of file
+#endif
