@@ -109,8 +109,9 @@ io_gather_conf_t::io_gather_conf_t(const std::size_t simd_w,
     , vmm_tmp_idx_(vmm_tmp_idx) {}
 
 template <typename Vmm>
-jit_io_helper_t<Vmm>::jit_io_helper_t(jit_generator *host, const cpu_isa_t &isa,
-        const data_type_t &data_type, const io_conf_t &io_conf,
+jit_io_helper_t<Vmm>::jit_io_helper_t(jit_generator_t *host,
+        const cpu_isa_t &isa, const data_type_t &data_type,
+        const io_conf_t &io_conf,
         const utils::optional_t<io_tail_conf_t> &tail_conf,
         const utils::optional_t<io_emu_bf16_conf_t> &bf16_conf,
         const utils::optional_t<io_saturation_conf_t> &saturation_conf,
@@ -206,7 +207,7 @@ bool jit_io_helper_t<Vmm>::is_data_type_supported(const data_type_t dt) {
         case data_type::f16:
             return is_superset(isa_, avx512_core_fp16) || isa_ == avx2_vnni_2;
         case data_type::f8_e4m3:
-        case data_type::f8_e5m2: return is_superset(isa_, avx512_core_amx);
+        case data_type::f8_e5m2: return is_superset(isa_, avx512_core_fp16);
         default: assert(!"Unsupported data type");
     }
     return false;
@@ -244,7 +245,7 @@ void jit_io_helper_t<Vmm>::prepare_vmm_mask(
                 reinterpret_cast<size_t>(&mask_f32[7 - how_many_bits_to_set]));
         host_->uni_vmovups(mask, host_->ptr[reg_tmp]);
     } else if (how_many_bits_to_set == simd_w) {
-        host_->uni_vcmpps(mask, mask, mask, jit_generator::_cmp_eq_oq);
+        host_->uni_vcmpps(mask, mask, mask, jit_generator_t::_cmp_eq_oq);
     } else {
         assert(!"Can't set so many bits.");
     }
@@ -278,12 +279,10 @@ void jit_io_helper_t<Vmm>::prepare_i8_data_to_store(const Vmm &i8_vmm) {
 template <typename Vmm>
 void jit_io_helper_t<Vmm>::prepare_xf16_data_to_store(const Vmm &vmm) {
     const auto &cvt_lower_vmm =
-            typename vreg_traits<Vmm>::Vmm_lower_t(vmm.getIdx());
+            typename vreg_traits_t<Vmm>::Vmm_lower_t(vmm.getIdx());
 
     if (data_type_ == data_type::bf16)
-        host_->vcvtneps2bf16(cvt_lower_vmm, vmm,
-                mayiuse(avx512_core) ? Xbyak::EvexEncoding
-                                     : Xbyak::VexEncoding);
+        host_->vcvtneps2bf16(cvt_lower_vmm, vmm, host_->get_encoding());
     else
         host_->uni_vcvtps2phx(cvt_lower_vmm, vmm);
 }
@@ -780,7 +779,7 @@ void jit_io_helper_t<Vmm>::store(const Vmm &src_raw_vmm,
         // TODO: Consider adding opmask to store xf16 data from Xmm.
         // This could allow to use store_bf16/store_f16 functions for isa >= avx512_core.
         const size_t xmm_length
-                = vreg_traits<Xbyak::Xmm>::vlen / sizeof(int32_t);
+                = vreg_traits_t<Xbyak::Xmm>::vlen / sizeof(int32_t);
         const size_t store_size = (tail ? tail_conf_->tail_size_ : xmm_length)
                 * types::data_type_size(data_type_);
         store_byte_by_byte(src_vmm, dst_addr, store_size);
@@ -819,7 +818,7 @@ void jit_io_helper_t<Vmm>::store_byte_by_byte(const Vmm &src_vmm,
     const bool is_xf16
             = utils::one_of(data_type_, data_type::bf16, data_type::f16);
     const auto &cvt_lower_vmm =
-            typename vreg_traits<Vmm>::Vmm_lower_t(src_vmm.getIdx());
+            typename vreg_traits_t<Vmm>::Vmm_lower_t(src_vmm.getIdx());
 
     if (is_i8) prepare_i8_data_to_store(src_vmm);
     if (is_xf16) prepare_xf16_data_to_store(src_vmm);
@@ -847,14 +846,12 @@ void jit_io_helper_t<Vmm>::store_bf16(
             && "Store operation for bf16 is not supported for Xmms.");
 
     const auto &cvt_lower_vmm =
-            typename vreg_traits<Vmm>::Vmm_lower_t(src_vmm.getIdx());
+            typename vreg_traits_t<Vmm>::Vmm_lower_t(src_vmm.getIdx());
 
     if (bf16_emu_)
         bf16_emu_->vcvtneps2bf16(cvt_lower_vmm, src_vmm);
     else
-        host_->vcvtneps2bf16(cvt_lower_vmm, src_vmm,
-                mayiuse(avx512_core) ? Xbyak::EvexEncoding
-                                     : Xbyak::VexEncoding);
+        host_->vcvtneps2bf16(cvt_lower_vmm, src_vmm, host_->get_encoding());
 
     if (io_conf_.nt_stores_enabled_)
         host_->uni_vmovntps(dst_addr, cvt_lower_vmm);
@@ -870,7 +867,7 @@ void jit_io_helper_t<Vmm>::store_f16(
             && "Store operation for f16 is not supported for Xmms.");
 
     const auto &cvt_lower_vmm =
-            typename vreg_traits<Vmm>::Vmm_lower_t(src_vmm.getIdx());
+            typename vreg_traits_t<Vmm>::Vmm_lower_t(src_vmm.getIdx());
 
     host_->uni_vcvtps2phx(cvt_lower_vmm, src_vmm);
 
@@ -914,8 +911,8 @@ void jit_io_helper_t<Vmm>::store_i8(
         static constexpr bool is_zmm = std::is_same<Vmm, Xbyak::Zmm>::value;
 
         auto store_i8_fn = data_type_ == data_type::s8
-                ? std::bind(&jit_generator::vpmovsdb, host_, _1, _2)
-                : std::bind(&jit_generator::vpmovusdb, host_, _1, _2);
+                ? std::bind(&jit_generator_t::vpmovsdb, host_, _1, _2)
+                : std::bind(&jit_generator_t::vpmovusdb, host_, _1, _2);
 
         if (io_conf_.nt_stores_enabled_ && is_zmm) {
             Xbyak::Xmm src_xmm(src_vmm.getIdx());
@@ -1025,7 +1022,7 @@ template <typename Vmm>
 jit_io_multi_dt_helper_t<Vmm>::jit_io_multi_dt_helper_t() = default;
 
 template <typename Vmm>
-jit_io_multi_dt_helper_t<Vmm>::jit_io_multi_dt_helper_t(jit_generator *host,
+jit_io_multi_dt_helper_t<Vmm>::jit_io_multi_dt_helper_t(jit_generator_t *host,
         const cpu_isa_t &isa, const data_types_t &data_types,
         const io_conf_t &io_conf,
         const utils::optional_t<io_tail_conf_t> &tail_conf,
