@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,27 +16,36 @@
 
 #include "gpu/gpu_impl_list.hpp"
 
-#include "gpu/ocl/ref_concat.hpp"
-#include "gpu/ocl/simple_concat.hpp"
+#include "gpu/generic/ref_concat.hpp"
+
+#if DNNL_GPU_VENDOR == DNNL_VENDOR_INTEL
+#include "gpu/intel/ocl/gen9_concat.hpp"
+#include "gpu/intel/ocl/multi_concat.hpp"
+#include "gpu/intel/ocl/reusable_simple_concat.hpp"
+#include "gpu/intel/ocl/simple_concat.hpp"
+#endif
 
 namespace dnnl {
 namespace impl {
 namespace gpu {
 
-using cpd_create_f = dnnl::impl::engine_t::concat_primitive_desc_create_f;
-
 namespace {
-#define INSTANCE(...) __VA_ARGS__::pd_t::create
-const cpd_create_f concat_impl_list[] = {
-        INSTANCE(ocl::simple_concat_t),
-        INSTANCE(ocl::ref_concat_t),
+
+// clang-format off
+constexpr impl_list_item_t impl_list[] = REG_CONCAT_P({
+        GPU_CONCAT_INSTANCE_INTEL(intel::ocl::reusable_simple_concat_t)
+        GPU_CONCAT_INSTANCE_INTEL(intel::ocl::simple_concat_t)
+        GPU_CONCAT_INSTANCE_INTEL(intel::ocl::gen9_concat_t)
+        GPU_CONCAT_INSTANCE_INTEL(intel::ocl::multi_concat_t)
+        GPU_CONCAT_INSTANCE_GENERIC(generic::ref_concat_t)
         nullptr,
-};
-#undef INSTANCE
+});
+// clang-format on
+
 } // namespace
 
-const cpd_create_f *gpu_impl_list_t::get_concat_implementation_list() {
-    return concat_impl_list;
+const impl_list_item_t *get_concat_impl_list() {
+    return impl_list;
 }
 
 } // namespace gpu
