@@ -31,6 +31,7 @@
 #include "gpu/intel/jit/codegen/register_scope.hpp"
 #include "gpu/intel/jit/codegen/reorder.hpp"
 #include "gpu/intel/jit/emulation.hpp"
+#include "gpu/intel/jit/generator.hpp"
 #include "gpu/intel/jit/ir/ir.hpp"
 #include "gpu/intel/jit/ir/ir_builder.hpp"
 #include "gpu/intel/jit/ir/kernel_desc.hpp"
@@ -38,9 +39,8 @@
 #include "gpu/intel/jit/ir/message.hpp"
 #include "gpu/intel/jit/ir/tensor.hpp"
 #include "gpu/intel/jit/ir/walk_order.hpp"
-#include "gpu/intel/jit/jit_generator.hpp"
-#include "gpu/intel/jit/ngen/ngen.hpp"
-#include "gpu/intel/jit/ngen/ngen_register_allocator.hpp"
+#include "ngen.hpp"
+#include "ngen_register_allocator.hpp"
 #include "xpu/utils.hpp"
 
 namespace dnnl {
@@ -50,18 +50,19 @@ namespace intel {
 namespace jit {
 
 template <template <ngen::HW> class KernelT>
-struct ir_generator_t : public jit_generator_base {
+struct ir_generator_t : public generator_base_t {
     ir_generator_t(const kernel_desc_base_t &kernel_desc)
         : kernel_name_(kernel_desc.kernel_name()), kernel_desc_(kernel_desc) {}
 
     const char *kernel_name() const override { return kernel_name_.c_str(); }
 
-    xpu::binary_t get_binary(const ocl::ocl_gpu_engine_t *engine) override {
+    status_t get_kernel(compute::kernel_t &kernel,
+            const compute::compute_engine_t *engine) override {
         try {
 #define CASE(hw) \
     case ngen::HW::hw: { \
-        KernelT<ngen::HW::hw> kernel(kernel_desc_, engine); \
-        return kernel.getBinary(engine->context(), engine->device()); \
+        KernelT<ngen::HW::hw> _kernel(kernel_desc_, engine); \
+        return _kernel.get_kernel(kernel, engine); \
     }
             auto *device_info = engine->device_info();
             auto hw = convert_dnnl_arch_to_ngen(device_info->gpu_arch());
@@ -77,8 +78,10 @@ struct ir_generator_t : public jit_generator_base {
                 default: gpu_assert(false) << "Unexpected GPU architecture";
             }
 #undef CASE
-        } catch (ngen::out_of_registers_exception &) { return xpu::binary_t(); }
-        return xpu::binary_t();
+        } catch (ngen::out_of_registers_exception &) {
+            return status::runtime_error;
+        }
+        return status::runtime_error;
     }
 
 private:
@@ -92,7 +95,7 @@ public:
 
     ~expr_binding_t() {
         if (!cpp_compat::uncaught_exceptions()) {
-            ir_assert(expr2dst_.empty()) << "Detected missing unbind_dst().";
+            gpu_assert(expr2dst_.empty()) << "Detected missing unbind_dst().";
         }
     }
 
@@ -101,20 +104,20 @@ public:
     }
 
     ngen_operand_t get_dst(const expr_t &expr) const {
-        ir_assert(is_dst_bound(expr)) << "Destination is not bound: " << expr;
+        gpu_assert(is_dst_bound(expr)) << "Destination is not bound: " << expr;
         return expr2dst_.at(expr);
     }
 
     void bind_dst(const expr_t &expr, const ngen_operand_t &operand) {
-        ir_assert(!expr.is_empty());
+        gpu_assert(!expr.is_empty());
         auto ret = expr2dst_.insert({expr, operand});
-        ir_assert(ret.second) << "Already bound: " << expr;
+        gpu_assert(ret.second) << "Already bound: " << expr;
     }
 
     void unbind_dst(const expr_t &expr) {
-        ir_assert(!expr.is_empty());
+        gpu_assert(!expr.is_empty());
         auto it = expr2dst_.find(expr);
-        ir_assert(it != expr2dst_.end());
+        gpu_assert(it != expr2dst_.end());
         expr2dst_.erase(it);
     }
 
@@ -126,7 +129,7 @@ public:
         if (expr.is_empty()) return ngen_operand_t();
         if (!is_bound(expr)) {
             if (!allow_empty)
-                ir_assert(false) << "Operand is not bound: " << expr;
+                gpu_assert(false) << "Operand is not bound: " << expr;
             return ngen_operand_t();
         }
         return expr2operand_.at(expr);
@@ -147,24 +150,23 @@ public:
         int esize = operand.mod().getExecSize();
         if (esize == 0) esize = 1;
         if (esize != expr.type().elems() && !expr.type().is_bool()) {
-            ir_assert(expr.type().is_scalar() || esize == 1)
+            gpu_assert(expr.type().is_scalar() || esize == 1)
                     << "Expected broadcast.";
             if (operand.is_reg_buf_data() && esize != 1) {
                 // Bind scalar expression to the first vector element.
-                op_to_bind = operand.reg_buf_data().format(
-                        0, ngen::DataType::invalid, 1);
+                op_to_bind = operand.reg_buf_data().format(0, 1);
             }
         }
 
         auto ret = expr2operand_.insert({expr, op_to_bind});
-        ir_assert(ret.second) << "Already bound: " << expr;
+        gpu_assert(ret.second) << "Already bound: " << expr;
     }
 
     void unbind(const expr_t &expr) {
-        ir_assert(!expr.is_empty());
+        gpu_assert(!expr.is_empty());
 
         auto it = expr2operand_.find(expr);
-        ir_assert(it != expr2operand_.end());
+        gpu_assert(it != expr2operand_.end());
         expr2operand_.erase(it);
     }
 
@@ -174,107 +176,58 @@ private:
     object_map_t<expr_t, ngen_operand_t> expr2operand_;
 };
 
-template <ngen::HW hw>
+template <typename ngen_generator_t>
 class expr_evaluator_t;
 
-template <ngen::HW hw>
+template <typename ngen_generator_t>
 class ir_to_ngen_t;
 
-template <ngen::HW hw>
-class ir_kernel_t : public jit_generator<hw> {
+template <typename ngen_generator_t>
+class ir_kernel_base_t : public ngen_generator_t {
 public:
-    NGEN_FORWARD_OPENCL(hw);
+    NGEN_FORWARD_SCOPE(ngen_generator_t)
 
-    friend class expr_evaluator_t<hw>;
-    friend class ir_to_ngen_t<hw>;
     friend class send_impl_t;
 
-    ir_kernel_t(const kernel_desc_base_t &desc, const impl::engine_t *engine,
-            const debug_config_t &debug_config)
-        : jit_generator<hw>(debug_config)
-        , kernel_name_(desc.kernel_name())
+    template <typename... ngen_generator_args>
+    ir_kernel_base_t(const kernel_desc_base_t &desc,
+            const impl::engine_t *engine, ngen_generator_args... args)
+        : ngen_generator_t(std::forward<ngen_generator_args>(args)...)
         , exec_cfg_(desc.exec_cfg(engine))
-        , local_range_(desc.local_range())
-        , require_dpas_(desc.with_dpas())
-        , regs_(exec_cfg_.regs())
-        , ra_(hw, desc.kernel_name())
-        , emu_strategy(hw, exec_cfg_.hw().stepping_id()) {
+        , ra_(getHardware())
+        , emu_strategy(getHardware(), exec_cfg_.hw().stepping_id()) {
         desc.init_kernel_iface(kernel_iface_);
-        setStepping(exec_cfg_.hw().stepping_id());
-        ra_.setRegisterCount(regs_);
+        ra_.setRegisterCount(exec_cfg_.regs());
     }
 
-    ir_kernel_t(const std::string &kernel_name, const exec_config_t &exec_cfg,
-            const compute::range_t &local_range, bool require_dpas,
-            const debug_config_t &debug_config)
-        : jit_generator<hw>(debug_config)
-        , kernel_name_(kernel_name)
+    template <typename... ngen_generator_args>
+    ir_kernel_base_t(const exec_config_t &exec_cfg,
+            const kernel_iface_t &kernel_iface, ngen_generator_args... args)
+        : ngen_generator_t(std::forward<ngen_generator_args>(args)...)
+        , kernel_iface_(kernel_iface)
         , exec_cfg_(exec_cfg)
-        , local_range_(local_range)
-        , require_dpas_(require_dpas)
-        , regs_(exec_cfg.regs())
-        , ra_(hw, kernel_name)
-        , emu_strategy(hw, exec_cfg.hw().stepping_id()) {
-        setStepping(exec_cfg.hw().stepping_id());
-        ra_.setRegisterCount(regs_);
+        , ra_(getHardware())
+        , emu_strategy(getHardware(), exec_cfg.hw().stepping_id()) {
+        ngen_generator_t::setStepping(exec_cfg.hw().stepping_id());
+        ra_.setRegisterCount(exec_cfg_.regs());
     }
 
-    const exec_config_t &exec_cfg() { return exec_cfg_; }
     const kernel_iface_t &kernel_iface() const { return kernel_iface_; }
-    void set_kernel_iface(const kernel_iface_t &kernel_iface) {
-        kernel_iface_ = kernel_iface;
-    }
-
-    void setup_interface(const stmt_t &kernel_body = stmt_t()) {
-        externalName(kernel_name_);
-        requireLocalID(3);
-        requireLocalSize();
-        requireGRF(regs_);
-        requireSIMD(exec_cfg_.simd());
-        requireBarrier();
-        if (require_dpas_) requireDPAS();
-        if (has_send_atomics(kernel_body)) requireGlobalAtomics();
-
-        for (int i = 0; i < kernel_iface_.nargs(); i++) {
-            auto &name = kernel_iface_.arg_name(i);
-            auto &type = kernel_iface_.arg_type(i);
-            if (type.is_ptr()) {
-                newArgument(name, ngen::ExternalArgumentType::GlobalPtr,
-                        ngen::GlobalAccessType::Stateless);
-            } else {
-                newArgument(name, to_ngen(type));
-            }
-        }
-
-        if (!kernel_body.is_empty() && local_range_) {
-            int slm_size = alloc_manager_t(kernel_body)
-                                   .total_size(alloc_kind_t::slm);
-            int max_slm_size = compute::device_info_t::max_slm_size_per_tg(
-                    convert_ngen_arch_to_dnnl(hw), thread_group_size(),
-                    regs_ > 128);
-            if (slm_size > max_slm_size) {
-                // TODO: Use status code for this check.
-                ir_except_not_implemented("SLM size limit is exceeded.");
-            }
-            requireSLM(slm_size);
-        }
-
-        finalizeInterface();
-    }
+    const exec_config_t &exec_cfg() const { return exec_cfg_; }
 
     void generate_prologue() {
-        setDefaultNoMask();
-        setDefaultAutoSWSB(true);
+        ngen_generator_t::setDefaultNoMask();
+        ngen_generator_t::setDefaultAutoSWSB(true);
 
-        prologue();
+        ngen_generator_t::prologue();
 
         // Claim registers.
-        ra_.claim(r0);
+        ra_.claim(ngen_generator_t::r0);
         for (int i = 0; i < 3; i++)
-            ra_.claim(getLocalID(i));
+            ra_.claim(ngen_generator_t::getLocalID(i));
 
         for (int i = 0; i < kernel_iface_.nargs(); i++) {
-            ra_.claim(getArgument(kernel_iface_.arg_name(i)));
+            ra_.claim(ngen_generator_t::getArgument(kernel_iface_.arg_name(i)));
         }
 
         if (emu_strategy.emulate64) {
@@ -282,22 +235,15 @@ public:
             emu_state.temp[1] = ra_.alloc();
         }
         // Enable IEEE f32 -> s32 rounding and f64/f32/f16 denormals.
-        or_(1, cr0, cr0, uint16_t(0x14C0));
+        or_(1, ngen_generator_t::cr0, ngen_generator_t::cr0, uint16_t(0x14C0));
 
         // Allocate and initialize signal header for future use.
-        if (require_signal_header_) {
+        if (exec_cfg_.require_signal_header()) {
             signal_header_ = ra_.alloc();
-            barrierheader(signal_header_);
+            ngen_generator_t::barrierheader(signal_header_);
         }
     }
 
-    void bind_external_vars(const stmt_t &kernel_body,
-            const walk_order_t &kernel_grid_walk_order,
-            expr_binding_t &expr_binding) {
-        bind_external_vars(kernel_body, expr_binding);
-        bind_kernel_grid_walk_order(kernel_grid_walk_order, expr_binding);
-    }
-
     void bind_external_vars(
             const stmt_t &kernel_body, expr_binding_t &expr_binding) {
         alloc_manager_t alloc_mgr(kernel_body);
@@ -308,7 +254,7 @@ public:
             auto tg_idx = alloc_mgr.find_let(ir_builder_t::tg_idx(i), true);
             if (!tg_idx.is_empty()) {
                 auto tmp = ra_.template alloc_sub<int32_t>();
-                mov(1, tmp, r0.ud(r0_sub_idxs[i]));
+                mov(1, tmp, ngen_generator_t::r0.ud(r0_sub_idxs[i]));
                 expr_binding.bind(tg_idx, tmp);
             }
         }
@@ -317,7 +263,8 @@ public:
         for (int i = 0; i < 3; i++) {
             auto local_id = alloc_mgr.find_let(ir_builder_t::local_id(i), true);
             if (!local_id.is_empty()) {
-                expr_binding.bind(local_id, getLocalID(i).uw(0));
+                expr_binding.bind(
+                        local_id, ngen_generator_t::getLocalID(i).uw(0));
             }
         }
 
@@ -329,12 +276,12 @@ public:
                 auto alloc_buf
                         = alloc_mgr.find_buffer(name, /*allow_empty=*/true);
                 if (alloc_buf.is_empty()) {
-                    ir_warning() << "Unused argument: " << arg_var << std::endl;
+                    gpu_warning() << "Unused argument: " << arg_var;
                     continue;
                 }
-                ir_assert(alloc_buf.is_same(arg_var));
+                gpu_assert(alloc_buf.is_same(arg_var));
             }
-            expr_binding.bind(arg_var, getArgument(name));
+            expr_binding.bind(arg_var, ngen_generator_t::getArgument(name));
         }
 
         // Bind SLM buffer (SLM loads/stores use 0-based offsets).
@@ -362,7 +309,7 @@ public:
                                const ngen::Subregister &src1, uint32_t src2) {
             bool is_src2_16_bit
                     = (src2 <= std::numeric_limits<uint16_t>::max());
-            if (hw >= ngen::HW::XeLP && is_src2_16_bit && false) {
+            if (getHardware() >= ngen::HW::XeLP && is_src2_16_bit && false) {
                 mad(1, dst, src0, src1, src2);
             } else {
                 auto tmp = ra_.alloc_sub<uint64_t>();
@@ -420,7 +367,7 @@ public:
             const std::vector<expr_t> &grid_vars,
             expr_binding_t &expr_binding) {
         int nblocks = (int)blocks.size();
-        ir_assert((int)grid_vars.size() == nblocks);
+        gpu_assert((int)grid_vars.size() == nblocks);
         if (nblocks == 1) {
             expr_binding.bind(grid_vars[0], id);
             return;
@@ -473,7 +420,7 @@ public:
     }
 
     void generate_epilogue() {
-        epilogue();
+        ngen_generator_t::epilogue();
         pad_kernel();
     }
 
@@ -504,7 +451,7 @@ public:
             }
         } else {
             // dst is a flag register.
-            ir_assert(!dst.is_negated());
+            gpu_assert(!dst.is_negated());
             auto _mod = mod;
             _mod.setExecSize(1);
             if (src0.is_reg_data()) {
@@ -518,7 +465,7 @@ public:
     void eadd(const ngen::InstructionModifier &mod, const ngen_operand_t &dst,
             const ngen_operand_t &src0, const ngen_operand_t &src1) {
         if (src0.is_immediate()) {
-            ir_assert(src1.is_reg_data());
+            gpu_assert(src1.is_reg_data());
             eadd(mod, dst, src1, src0);
             return;
         }
@@ -548,7 +495,7 @@ public:
         auto dst = dst_;
         auto mod = mod_;
         if (src0.is_immediate()) {
-            ir_assert(src1.is_reg_data());
+            gpu_assert(src1.is_reg_data());
             emul(mod, dst, src1, src0);
             return;
         }
@@ -565,7 +512,7 @@ public:
                 };
                 src0 = subreg(src0_);
                 src1 = subreg(src1_);
-                dst = dst_.sub_reg_data(i, 1);
+                dst = dst_.sub_reg_data(i, esize);
                 if (ngen_is_dw(src1.type()) && ngen_is_w(src0.type())) {
                     emul(mod, dst.reg_data(), src1.reg_data(), src0.reg_data());
                 } else {
@@ -579,7 +526,7 @@ public:
                 return;
             }
             if (ngen_is_dw(src1_imm.getType())) {
-                ir_assert(mod.getExecSize() == 1);
+                gpu_assert(mod.getExecSize() == 1);
                 auto tmp = ra_.alloc_sub<int64_t>();
                 if (ngen_is_w(src0.type())) {
                     auto tmp_src1 = ra_.alloc_sub<int32_t>();
@@ -606,7 +553,7 @@ public:
         auto scope = ngen_register_scope_t(ra_);
         align_src_dst_offset(this, scope, mod, dst, src0);
         align_src_dst_offset(this, scope, mod, dst, src1);
-        if (hw >= ngen::HW::XeHP) {
+        if (getHardware() >= ngen::HW::XeHP) {
             if (src2.is_reg_data()) {
                 align_src_dst_offset(this, scope, mod, dst, src2);
                 add3(mod, dst.reg_data(), fixup_ternary_rgn(src0.reg_data()),
@@ -639,7 +586,7 @@ public:
             align_src_dst_offset(this, scope, mod, dst, src2);
             mad(mod, dst.reg_data(), fixup_ternary_rgn(src0.reg_data()),
                     fixup_ternary_rgn(src1.reg_data()), src2.reg_data());
-        } else if (hw < ngen::HW::XeLP) {
+        } else if (getHardware() < ngen::HW::XeLP) {
             align_src_dst_offset(this, scope, mod, dst, src0);
             mul(mod, dst.reg_data(), src1.reg_data(), src2.immediate());
             add(mod, dst.reg_data(), dst.reg_data(), src0.reg_data());
@@ -662,10 +609,11 @@ public:
             const ngen_operand_t &src0, const ngen_operand_t &src1) {
         if (!src1.is_immediate()) {
             // Immediate src0 is not supported with fdiv_ieee.
-            if (src0.is_immediate() && hw >= ngen::HW::XeHPC) {
+            if (src0.is_immediate() && getHardware() >= ngen::HW::XeHPC) {
                 auto tmp_src0 = ra_.alloc_sub(src0.type());
                 mov(mod, tmp_src0, src0.immediate());
-                efdiv(mod, dst, ngen_operand_t(reg_buf_data_t(hw, tmp_src0)),
+                efdiv(mod, dst,
+                        ngen_operand_t(reg_buf_data_t(getHardware(), tmp_src0)),
                         src1);
                 ra_.safeRelease(tmp_src0);
             } else {
@@ -674,11 +622,14 @@ public:
         } else {
             auto &src1_imm = src1.immediate();
             if (to_ir(src0.type()).is_fp()) {
-                ngen::Immediate src1_inv_value(1.f / to_cpp<float>(src1_imm));
+                constexpr float inf = std::numeric_limits<float>::infinity();
+                float f = to_cpp<float>(src1_imm);
+                float f_inv = f ? 1.f / f : std::signbit(f) ? -inf : inf;
+                ngen::Immediate src1_inv_value(f_inv);
                 emul(mod, dst, src0, src1_inv_value);
             } else {
                 int32_t src1_value = to_cpp<int32_t>(src1_imm);
-                ir_assert(0 < src1_value && src1_value <= INT32_MAX)
+                gpu_assert(0 < src1_value && src1_value <= INT32_MAX)
                         << src1_value;
                 eidiv(mod, dst.reg_data(), ngen::Subregister(), src0.reg_data(),
                         src1_value);
@@ -689,12 +640,12 @@ public:
     void efdiv(const ngen::InstructionModifier &mod, const ngen_operand_t &dst,
             const ngen_operand_t &src0, const ngen_operand_t &src1) {
         int esize = mod.getExecSize();
-        int grf_size = ngen::GRF::bytes(hw);
+        int grf_size = ngen::GRF::bytes(getHardware());
         int div_esize = std::min(esize, grf_size / int(sizeof(float)));
 
-        ir_assert(dst.type() == ngen::DataType::f);
-        ir_assert(src0.type() == ngen::DataType::f);
-        ir_assert(src1.type() == ngen::DataType::f);
+        gpu_assert(dst.type() == ngen::DataType::f);
+        gpu_assert(src0.type() == ngen::DataType::f);
+        gpu_assert(src1.type() == ngen::DataType::f);
 
         if (src1.reg_data().getHS() != 0) {
             int nregs = std::max(1, (mod.getExecSize() * 4) / grf_size);
@@ -710,10 +661,11 @@ public:
         }
 
         // fdiv_ieee() is not supported in XeHPG so we use a less precise, inv-based sequence.
-        if (hw < ngen::HW::XeHPC) {
+        if (getHardware() < ngen::HW::XeHPC) {
             auto tmp = ra_.alloc_sub<float>();
             inv(1, tmp, src1.reg_data());
-            emul(mod, dst, src0, ngen_operand_t(reg_buf_data_t(hw, tmp)));
+            emul(mod, dst, src0,
+                    ngen_operand_t(reg_buf_data_t(getHardware(), tmp)));
             ra_.safeRelease(tmp);
             return;
         }
@@ -744,10 +696,10 @@ public:
             auto src0_rd = r_spill(s0, div_esize, s0_spill);
             auto src1_rd = r_spill(s1, div_esize, s1_spill);
             // Enable mask as fdiv_ieee relies on masked if/endif flow.
-            setDefaultNoMask(false);
+            ngen_generator_t::setDefaultNoMask(false);
             fdiv_ieee(div_mod, f0[0], dst_rd(), src0_rd(), src1_rd(), zero, one,
                     tmp);
-            setDefaultNoMask(true);
+            ngen_generator_t::setDefaultNoMask(true);
         }
 
         ra_.safeRelease(one);
@@ -757,10 +709,10 @@ public:
 
     void emod(const ngen::InstructionModifier &mod, const ngen_operand_t &dst,
             const ngen_operand_t &src0, const ngen_operand_t &src1) {
-        ir_assert(src1.is_immediate());
+        gpu_assert(src1.is_immediate());
         auto &src1_imm = src1.immediate();
         int32_t src1_value = to_cpp<int32_t>(src1_imm);
-        ir_assert(0 < src1_value && src1_value <= INT32_MAX) << src1_value;
+        gpu_assert(0 < src1_value && src1_value <= INT32_MAX) << src1_value;
         eidiv(mod, ngen::Subregister(), dst.reg_data(), src0.reg_data(),
                 src1_value);
     }
@@ -860,10 +812,10 @@ public:
     void eidiv(const ngen::InstructionModifier &mod, const ngen::RegData &qot,
             const ngen::RegData &rem, const ngen::RegData &x,
             const ngen::RegData &_y, const ngen::RegData &_magic) {
-        ir_assert(x.getHS() == 0);
-        ir_assert(_y.getType() == ngen::DataType::ud);
-        ir_assert(_magic.getHS() == 0);
-        ir_assert(_magic.getType() == ngen::DataType::uq);
+        gpu_assert(x.getHS() == 0);
+        gpu_assert(_y.getType() == ngen::DataType::ud);
+        gpu_assert(_magic.getHS() == 0);
+        gpu_assert(_magic.getType() == ngen::DataType::uq);
 
         bool x_signed = utils::one_of(x.getType(), ngen::DataType::b,
                 ngen::DataType::w, ngen::DataType::d);
@@ -919,8 +871,8 @@ public:
     void eidiv(const ngen::InstructionModifier &mod, const ngen::RegData &_qot,
             const ngen::RegData &rem, const ngen::RegData &x,
             const ngen::RegData &_y, bool update_cr0_fp_to_int_rtz = true) {
-        ir_assert(mod.getExecSize() == 1);
-        ir_assert(_y.getType() == ngen::DataType::ud);
+        gpu_assert(mod.getExecSize() == 1);
+        gpu_assert(_y.getType() == ngen::DataType::ud);
         auto cr0_save = ra_.alloc_sub<uint32_t>();
         auto f_tmp = ra_.alloc_sub<float>();
         auto x_tmp = ra_.alloc_sub<float>();
@@ -965,7 +917,7 @@ public:
         bool x_signed = utils::one_of(x.getType(), ngen::DataType::b,
                 ngen::DataType::w, ngen::DataType::d);
         auto div_type = (x_signed ? ngen::DataType::d : ngen::DataType::ud);
-        ir_assert(x.getHS() == 0);
+        gpu_assert(x.getHS() == 0);
         if (ngen::utils::is_zero_or_pow2(y)) {
             auto _x = get_subregister(x);
             if (x.getNeg() || (x == qot) || (x == rem)) {
@@ -989,24 +941,16 @@ public:
         auto _qot = qot_tmp[0];
         mov(1, _x, x);
 
-        // qot = (x * m) >> p
-        bool use_mach = true;
-        if (use_mach) {
-            auto acc = acc0.retype(div_type);
-            mul(1, acc[0], _x, m & 0xFFFF);
-            mach(1, _qot, _x, m);
-            shr<uint32_t>(1, _qot, _qot, p - 32);
-        } else {
-            auto q_tmp = qot_tmp.retype(ngen::DataType::q);
-            emul(1, q_tmp[0], _x, m);
-            eshr(1, q_tmp.uq(0), q_tmp.uq(0), p);
-        }
+        auto acc = acc0.retype(div_type);
+        mul(1, acc[0], _x, m & 0xFFFF);
+        mach(1, _qot, _x, m);
+        shr<uint32_t>(1, _qot, _qot, p - 32);
 
         if (!rem.isInvalid()) {
             // rem = x - qot * y
             bool y_is_16_bit = (y <= static_cast<uint32_t>(
                                         std::numeric_limits<int16_t>::max()));
-            if (hw >= ngen::HW::XeLP && y_is_16_bit) {
+            if (getHardware() >= ngen::HW::XeLP && y_is_16_bit) {
                 mad(mod, rem, x, _qot, -int16_t(y));
             } else {
                 auto tmp = ra_.alloc_sub<uint64_t>();
@@ -1079,6 +1023,20 @@ public:
                 *this, mod, dst, src0, src1, emu_strategy, emu_state);
     }
 
+    void esel(const ngen::InstructionModifier &mod, const ngen_operand_t &dst,
+            const ngen_operand_t &src0, const ngen_operand_t &src1) {
+        if (ngen_is_qw(dst.type())) {
+            auto neg_mod = mod;
+            neg_mod.setPredInv(!mod.isPredInv());
+            emov(mod, dst, src0);
+            emov(neg_mod, dst, src1);
+        } else if (src1.is_reg_data()) {
+            sel(mod, dst.reg_data(), src0.reg_data(), src1.reg_data());
+        } else {
+            sel(mod, dst.reg_data(), src0.reg_data(), src1.immediate());
+        }
+    }
+
 protected:
     // Helper RAII class allocating a temporary GRF buffer aligned at a
     // register boundary for instructions that require aligned operands.
@@ -1089,7 +1047,7 @@ protected:
         // read - whether operand is to be used as input (needs pre-copy)
         // write - whether operand is to be used as output (needs post-copy)
         // force_copy - always copy the region (even if it's aligned)
-        spiller_t(ir_kernel_t<hw> *host, const ngen::RegData &rd, int esize,
+        spiller_t(ir_kernel_base_t *host, const ngen::RegData &rd, int esize,
                 bool read, bool write, bool force_copy)
             : host_(host), rd_(rd), esize_(esize), read_(read), write_(write) {
             if (rd.getOffset() == 0 && !force_copy) return;
@@ -1097,7 +1055,7 @@ protected:
             int w = rd.getWidth();
             int hs = rd.getHS();
             int vs = rd.getVS();
-            int grf_size = ngen::GRF::bytes(hw);
+            int grf_size = ngen::GRF::bytes(host->getHardware());
             int regs = utils::div_up(
                     std::max(esize * hs, 1) * rd.getBytes(), grf_size);
             tmp_range_ = host_->ra_.alloc_range(regs);
@@ -1129,12 +1087,12 @@ protected:
                 case 1: ret.setType(ngen::DataType::ub); break;
                 case 2: ret.setType(ngen::DataType::uw); break;
                 case 4: ret.setType(ngen::DataType::ud); break;
-                default: ir_error_not_expected();
+                default: gpu_error_not_expected();
             }
             return ret;
         }
 
-        ir_kernel_t<hw> *host_ = nullptr;
+        ir_kernel_base_t *host_ = nullptr;
         ngen::RegData rd_;
         int esize_;
         bool read_ = false;
@@ -1158,9 +1116,9 @@ protected:
         return spill(rd, esize, false, true, force_copy);
     }
 
-    static bool overlaps(
-            int esize, const ngen::RegData &a, const ngen::RegData &b) {
-        int grf_size = ngen::GRF::bytes(hw);
+    bool overlaps(
+            int esize, const ngen::RegData &a, const ngen::RegData &b) const {
+        int grf_size = ngen::GRF::bytes(getHardware());
         int a_beg = a.getBase() * grf_size + a.getByteOffset();
         int b_beg = b.getBase() * grf_size + b.getByteOffset();
         int a_end = a_beg + std::max(esize * a.getHS(), 1) * a.getBytes() - 1;
@@ -1174,15 +1132,6 @@ protected:
         return false;
     }
 
-    int thread_group_size() const {
-        ir_assert(local_range_);
-        int local_size = 1;
-        for (int i = 0; i < (int)local_range_.ndims(); i++) {
-            local_size *= (int)local_range_[i];
-        }
-        return ir_utils::safe_divide(local_size, exec_cfg_.simd());
-    }
-
     static ngen::RegData fixup_ternary_rgn(const ngen::RegData &r) {
         ngen::RegData retn = r;
         return ((retn.getHS() == 1) && (retn.getVS() == retn.getWidth()))
@@ -1191,12 +1140,7 @@ protected:
     }
 
     kernel_iface_t kernel_iface_;
-    std::string kernel_name_;
     exec_config_t exec_cfg_;
-    compute::range_t local_range_;
-    bool require_dpas_;
-    bool require_signal_header_ = false;
-    int regs_;
     reg_allocator_t ra_;
     ngen::GRF signal_header_;
 
@@ -1204,6 +1148,91 @@ protected:
     EmulationState emu_state;
 };
 
+template <ngen::HW hw>
+class ir_kernel_t : public ir_kernel_base_t<generator_t<hw>> {
+public:
+    using base = ir_kernel_base_t<generator_t<hw>>;
+    using elf_generator_t = ngen::ELFCodeGenerator<hw>;
+    friend class expr_evaluator_t<ir_kernel_t>;
+    friend class ir_to_ngen_t<ir_kernel_t>;
+
+    ir_kernel_t(const kernel_desc_base_t &desc, const impl::engine_t *engine,
+            const debug_config_t &debug_config)
+        : base(desc, engine, debug_config)
+        , kernel_name_(desc.kernel_name())
+        , require_dpas_(desc.with_dpas())
+        , local_range_(desc.local_range()) {}
+
+    ir_kernel_t(const std::string &kernel_name, const exec_config_t &exec_cfg,
+            const compute::range_t &local_range, bool require_dpas,
+            const debug_config_t &debug_config)
+        : base(exec_cfg, {}, debug_config)
+        , kernel_name_(kernel_name)
+        , require_dpas_(require_dpas)
+        , local_range_(local_range) {}
+
+    const ngen::NEOInterfaceHandler &neo_interface() const {
+        return elf_generator_t::interface_;
+    }
+
+    void set_kernel_iface(const kernel_iface_t &kernel_iface) {
+        base::kernel_iface_ = kernel_iface;
+    }
+
+    void setup_interface(const stmt_t &kernel_body = stmt_t()) {
+        elf_generator_t::externalName(kernel_name_);
+        elf_generator_t::requireLocalID(3);
+        elf_generator_t::requireLocalSize();
+        elf_generator_t::requireGRF(base::exec_cfg().regs());
+        elf_generator_t::requireSIMD(base::exec_cfg().simd());
+        elf_generator_t::requireBarrier();
+        if (require_dpas_) elf_generator_t::requireDPAS();
+        if (has_send_atomics(kernel_body))
+            elf_generator_t::requireGlobalAtomics();
+
+        for (int i = 0; i < base::kernel_iface().nargs(); i++) {
+            auto &name = base::kernel_iface().arg_name(i);
+            auto &type = base::kernel_iface().arg_type(i);
+            if (type.is_ptr()) {
+                elf_generator_t::newArgument(name,
+                        ngen::ExternalArgumentType::GlobalPtr,
+                        ngen::GlobalAccessType::Stateless);
+            } else {
+                elf_generator_t::newArgument(name, to_ngen(type));
+            }
+        }
+
+        if (!kernel_body.is_empty() && local_range_) {
+            int slm_size = alloc_manager_t(kernel_body)
+                                   .total_size(alloc_kind_t::slm);
+            int max_slm_size = compute::device_info_t::max_slm_size_per_tg(
+                    convert_ngen_arch_to_dnnl(hw), thread_group_size(),
+                    base::exec_cfg().regs() > 128);
+            if (slm_size > max_slm_size) {
+                // TODO: Use status code for this check.
+                gpu_except_not_implemented("SLM size limit is exceeded.");
+            }
+            elf_generator_t::requireSLM(slm_size);
+        }
+
+        elf_generator_t::finalizeInterface();
+    }
+
+    int thread_group_size() const {
+        gpu_assert(local_range_);
+        int local_size = 1;
+        for (int i = 0; i < (int)local_range_.ndims(); i++) {
+            local_size *= (int)local_range_[i];
+        }
+        return ir_utils::safe_divide(local_size, base::exec_cfg_.simd());
+    }
+
+private:
+    std::string kernel_name_;
+    bool require_dpas_;
+    compute::range_t local_range_;
+};
+
 #define IR_KERNEL_EMULATION_FORWARD(hw) \
     using ir_kernel_t<hw>::emov; \
     using ir_kernel_t<hw>::eadd; \
@@ -1212,7 +1241,7 @@ protected:
     using ir_kernel_t<hw>::eshr;
 
 #define IR_KERNEL_FORWARD(hw) \
-    NGEN_FORWARD_OPENCL(hw) \
+    NGEN_FORWARD_ELF(hw) \
     IR_KERNEL_EMULATION_FORWARD(hw) \
     using ir_kernel_t<hw>::exec_cfg; \
     using ir_kernel_t<hw>::kernel_iface; \
@@ -1224,6 +1253,59 @@ protected:
     using ir_kernel_t<hw>::emu_strategy; \
     using ir_kernel_t<hw>::ra_;
 
+#ifdef NGEN_ASM
+class ir_asm_generator_t : public ngen::AsmCodeGenerator {
+public:
+    template <typename ngen_generator_t>
+    ir_asm_generator_t(const ngen_generator_t &k)
+        : ngen::AsmCodeGenerator(k.getProduct())
+        , interface_(k.neo_interface()) {}
+
+    NGEN_FORWARD_SCOPE(ngen::AsmCodeGenerator)
+
+    int getSIMD() const { return interface_.getSIMD(); }
+    void prologue() { interface_.generatePrologue(*this); }
+    void epilogue() {
+        int GRFCount = interface_.getGRFCount();
+        bool hasSLM = (interface_.getSLMSize() > 0);
+        epilogue(GRFCount, hasSLM, r0);
+    }
+
+    ngen::Subregister getArgument(const std::string &name) const {
+        return interface_.getArgument(name);
+    }
+    ngen::GRF getLocalID(int dim) const { return interface_.getLocalID(dim); }
+    std::string str() {
+        std::ostringstream oss;
+        getCode(oss);
+        return oss.str();
+    }
+
+private:
+    ngen::NEOInterfaceHandler interface_;
+};
+
+class ir_asm_kernel_t : public ir_kernel_base_t<ir_asm_generator_t> {
+public:
+    using base = ir_kernel_base_t<ir_asm_generator_t>;
+
+    friend class expr_evaluator_t<ir_asm_kernel_t>;
+    friend class ir_to_ngen_t<ir_asm_kernel_t>;
+
+    template <ngen::HW hw>
+    ir_asm_kernel_t(const ir_kernel_t<hw> &k)
+        : base(k.exec_cfg(), k.kernel_iface(), k) {}
+};
+#else
+class ir_asm_kernel_t {
+public:
+    template <ngen::HW hw>
+    ir_asm_kernel_t(const ir_kernel_t<hw> &k) {
+        MAYBE_UNUSED(k);
+    }
+};
+#endif
+
 } // namespace jit
 } // namespace intel
 } // namespace gpu
