@@ -27,9 +27,9 @@
 #include "gpu/intel/jit/conv/plan.hpp"
 #include "gpu/intel/jit/conv/problem.hpp"
 #include "gpu/intel/jit/conv/tiler.hpp"
+#include "gpu/intel/jit/eltwise_injector.hpp"
 #include "gpu/intel/jit/ir/gemm_schedule.hpp"
 #include "gpu/intel/jit/ir/tensor_config.hpp"
-#include "gpu/intel/jit/jit_eltwise_injector.hpp"
 
 #define VDISPATCH_CHECK(pd, engine, cond, msg, ...) \
     VCONDCHECK(primitive, create, dispatch, convolution, (cond), \
@@ -114,11 +114,12 @@ std::string prepend_groups_to_tag(const std::string &tag) {
 
 int get_default_mad_block(const type_t &type) {
     switch (type.size()) {
-        case 1: return 32;
+        // fp4 gets upconverted to f16 for mad.
+        case 1: return (type.is_fp4() ? 16 : 32);
         case 2:
         case 4: return 16;
         case 8: return 8;
-        default: ir_error_not_expected() << type;
+        default: gpu_error_not_expected() << type;
     }
     return 1;
 }
@@ -288,9 +289,9 @@ std::string build_tag(const std::vector<int> &inner_blocks,
         const std::vector<int> &outer_blocks, const std::vector<char> &letters,
         const std::vector<int> &idxs) {
     dim_idx_t n = into<dim_idx_t>(letters.size());
-    ir_assert(inner_blocks.size() == n);
-    ir_assert(outer_blocks.size() == n);
-    ir_assert(idxs.size() == n);
+    gpu_assert(inner_blocks.size() == n);
+    gpu_assert(outer_blocks.size() == n);
+    gpu_assert(idxs.size() == n);
 
     std::string tag;
     std::vector<bool> seen(n);
@@ -322,6 +323,7 @@ std::string build_tag(const std::vector<int> &inner_blocks,
             char c = letters[i];
             if (c == ' ') continue;
             if (seen[i]) c = static_cast<char>(std::toupper(c));
+            // NOLINTNEXTLINE(performance-inefficient-string-concatenation)
             tag = c + tag;
         }
     }
@@ -355,7 +357,7 @@ int pick_block(dim_t dim, int b0, int b1 = 0, int b2 = 0) {
 int get_default_block(fma_kind_t fma, const type_t &type, dim_t elems) {
     if (is_dp_fma(fma)) {
         if (is_small(type, elems)) {
-            int packed_dword_elems = 4 / type.size();
+            int packed_dword_elems = 32 / type.bitsize();
             return std::max(
                     utils::rnd_up_pow2(into<int>(elems)), packed_dword_elems);
         }
@@ -407,13 +409,17 @@ struct nc_block_t {
             auto default_gc_blk
                     = get_default_block(get_default_fma(hw, type), type, g * c);
             if (c_block != default_gc_blk) {
-                if (default_gc_blk % c == 0 && g % (default_gc_blk / c) == 0) {
+                if ((default_gc_blk % c == 0
+                            && g % (default_gc_blk / c) == 0)) {
                     c_block = default_gc_blk;
                 }
             }
         }
         auto default_n_blk = (type.size() <= 2) ? 32 : 16;
-        int n_block = (c_block == 1) ? 1 : pick_block(n, 16, default_n_blk);
+        int n_block = (c_block == 1)
+                ? 1
+                : (c_block < 8 ? pick_block(n, default_n_blk)
+                               : pick_block(n, 16, default_n_blk));
         return nc_block_t(n_block, c_block);
     }
 
@@ -449,8 +455,8 @@ struct goi_block_t {
     }
 
     static goi_block_t get_default_blocking(type_t type, int vec_size,
-            fma_kind_t fma_kind, bool is_bwd_d, dim_t g, dim_t o, dim_t i,
-            bool ab_transpose) {
+            fma_kind_t fma_kind, bool is_fwd, bool is_bwd_d, dim_t g, dim_t o,
+            dim_t i, bool ab_transpose) {
         dim_t x = o;
         dim_t y = i;
         int g_block = 1;
@@ -468,25 +474,23 @@ struct goi_block_t {
             std::swap(x_block, y_block);
             std::swap(x_block_outer, y_block_outer);
         }
-        get_default_blocking(type, vec_size, fma_kind, is_bwd_d, g, x, y,
-                g_block, *x_block, *y_block, *y_block_outer, ab_transpose);
+        get_default_blocking(type, vec_size, fma_kind, is_fwd, is_bwd_d, g, x,
+                y, g_block, *x_block, *y_block, *y_block_outer, ab_transpose);
         return goi_block_t(fma_kind, is_dw(g, o, i), is_bwd_d, g_block, o_block,
                 i_block, o_block_outer, i_block_outer);
     }
 
     static void get_default_blocking(type_t type, int vec_size,
-            fma_kind_t fma_kind, bool is_bwd_d, dim_t g, dim_t x, dim_t y,
-            int &g_block, int &x_block, int &y_block, int &y_block_outer,
-            bool ab_transpose = false) {
+            fma_kind_t fma_kind, bool is_fwd, bool is_bwd_d, dim_t g, dim_t x,
+            dim_t y, int &g_block, int &x_block, int &y_block,
+            int &y_block_outer, bool ab_transpose = false) {
         if (is_dw(g, x, y)) {
             g_block = vec_size;
         } else if (fma_kind == fma_kind_t::mad) {
-            x_block = (ab_transpose && is_bwd_d)
-                    ? into<int>(utils::rnd_up_pow2(x))
-                    : vec_size;
-            y_block = get_default_block(fma_kind, type, y);
+            x_block = (ab_transpose && (is_fwd || is_bwd_d)) ? 1 : vec_size;
+            y_block = (x_block == 1 ? 1 : get_default_block(fma_kind, type, y));
         } else {
-            int packed_dword_elems = 4 / type.size();
+            int packed_dword_elems = 32 / type.bitsize();
             x_block = ab_transpose ? into<int>(utils::rnd_up_pow2(x))
                                    : vec_size;
             y_block = packed_dword_elems;
@@ -531,7 +535,7 @@ std::string get_plain_user_tag(
                 = {"abcx", "abxc", "axcb"};
         auto &plain_wei_tags = (prb.with_groups ? plain_group_wei_tags
                                                 : plain_non_group_wei_tags);
-        ir_assert(
+        gpu_assert(
                 plain_non_group_wei_tags.size() == plain_group_wei_tags.size());
         for (size_t i = 0; i < plain_wei_tags.size(); i++) {
             if (matches_tag(md, plain_wei_tags[i])) {
@@ -564,7 +568,7 @@ std::string maybe_fixup_1st_conv_wei_tag(
         auto ret = tag;
         return ret.replace(pos, std::strlen(*p), *(p + 1));
     }
-    ir_error_not_expected() << tag;
+    gpu_error_not_expected() << tag;
     return tag;
 }
 
@@ -617,8 +621,8 @@ void init_data_tags(const conv_config_t &cfg, const memory_desc_t &src_md,
             dst_compute_type, prb.is_dw, prb.mb, prb.oc, prb.g,
             /*is_output=*/prb.is_fwd);
     auto wei_blk = goi_block_t::get_default_blocking(wei_compute_type,
-            cfg.vec_size(), cfg.fma_kind(), prb.is_bwd_d, prb.g, prb.oc, prb.ic,
-            prb.ab_swap_transpose);
+            cfg.vec_size(), cfg.fma_kind(), prb.is_fwd, prb.is_bwd_d, prb.g,
+            prb.oc, prb.ic, prb.ab_swap_transpose);
 
     src_tag = src_blk.tag();
     wei_tag = wei_blk.tag();
@@ -647,9 +651,9 @@ void init_data_tags(const conv_config_t &cfg, const memory_desc_t &src_md,
 
     // Use plain tags for user-facing activations for small-channel tensors.
     if (!matches_tag(src_md, src_tag) && is_small_ic_g1)
-        user_src_tag = (user_src_req.empty() ? "axb" : std::move(user_src_req));
+        user_src_tag = (user_src_req.empty() ? "axb" : user_src_req);
     if (!matches_tag(dst_md, dst_tag) && is_small_oc_g1)
-        user_dst_tag = (user_dst_req.empty() ? "axb" : std::move(user_dst_req));
+        user_dst_tag = (user_dst_req.empty() ? "axb" : user_dst_req);
 
     // Avoid reorder for small shapes
     if (!user_src_tag.empty() && !user_dst_tag.empty() && prb.g == 1
@@ -719,7 +723,7 @@ void prepare_zp_precompute_conv(const conv_problem_t &prb, dim_t *idhw,
 
     for (int i = off; i < int(K.size()); i++) {
         const auto KD = (K[i] - 1) * (D[i] + 1) + 1;
-        ir_assert(w->dims[2 + i + prb.with_groups - off] == K[i]);
+        gpu_assert(w->dims[2 + i + prb.with_groups - off] == K[i]);
         O[i] = ir_utils::max_unique_pad_states(
                 O[i], I[i], KD, P[i], S[i], true);
         I[i] = std::min(KD, I[i]);
@@ -842,6 +846,23 @@ status_t init_tensor_layouts(
             prb.dhw_map,
             /*add_groups=*/true);
 
+    // Disable cases that cannot generate valid fp4 tiling.
+    if (src_layout.type().is_fp4())
+        for (auto &b : src_layout.blocks()) {
+            if (b.stride == stride_t(1) && b.block % 8)
+                return status::unimplemented;
+        }
+    if (wei_layout.type().is_fp4())
+        for (auto &b : wei_layout.blocks()) {
+            if (b.stride == stride_t(1) && b.block % 8)
+                return status::unimplemented;
+        }
+    if (dst_layout.type().is_fp4())
+        for (auto &b : dst_layout.blocks()) {
+            if (b.stride == stride_t(1) && b.block % 8)
+                return status::unimplemented;
+        }
+
     src.set_compute(src_layout);
     src.set_user(user_src_layout);
     wei.set_compute(wei_layout);
@@ -896,6 +917,8 @@ bool data_types_ok(
     auto bia = prb.bia_data_type;
     bool is_fp8 = utils::one_of(data_type::f8_e5m2, src, wei, dst, bia)
             || utils::one_of(data_type::f8_e4m3, src, wei, dst, bia);
+    bool is_fp4 = utils::one_of(data_type::f4_e2m1, src, wei, dst, bia)
+            || utils::one_of(data_type::f4_e3m0, src, wei, dst, bia);
     if (!prb.is_f64_accumulator()
             && utils::one_of(data_type::f64, src, wei, dst, bia))
         return false;
@@ -904,8 +927,7 @@ bool data_types_ok(
     auto *device_info = compute_engine->device_info();
     if (prb.is_f64_accumulator() && !device_info->has_native(data_type::f64))
         return false;
-    if (is_fp8
-            && !(utils::one_of(hw, ngen::HW::XeHPC) && hw.systolic_support()))
+    if ((is_fp8 || is_fp4) && !(hw >= ngen::HW::XeHPC && hw.systolic_support()))
         return false;
     if (prb.is_fwd) return true;
     if (prb.is_bwd_d) return true;
@@ -914,13 +936,15 @@ bool data_types_ok(
         data_type_t default_acc_type
                 = src == data_type::f64 ? data_type::f64 : data_type::f32;
         ok &= utils::one_of(src, data_type::f8_e5m2, data_type::f8_e4m3,
-                data_type::bf16, data_type::f16, data_type::f32,
-                data_type::f64);
+                data_type::f4_e3m0, data_type::f4_e2m1, data_type::bf16,
+                data_type::f16, data_type::f32, data_type::f64);
         ok &= (dst == src);
         ok &= (utils::one_of(wei, src, default_acc_type)
-                || (utils::one_of(src, data_type::f8_e4m3, data_type::f8_e5m2)
+                || (utils::one_of(src, data_type::f8_e4m3, data_type::f8_e5m2,
+                            data_type::f4_e2m1, data_type::f4_e2m1)
                         && utils::one_of(wei, data_type::f8_e4m3,
-                                data_type::f8_e5m2, data_type::f32,
+                                data_type::f8_e5m2, data_type::f4_e2m1,
+                                data_type::f4_e2m1, data_type::f32,
                                 data_type::bf16, data_type::f16)));
 
         if (prb.with_bias) { ok &= utils::one_of(bia, src, data_type::f32); }
@@ -932,29 +956,38 @@ bool data_types_ok(
 bool zero_points_ok(const conv_problem_t &prb) {
     auto *pd = prb.conv_pd;
     auto *attr = pd->attr();
+    const auto &zp = attr->zero_points_;
 
     using namespace data_type;
     const auto input_type = (prb.is_fwd) ? pd->invariant_src_md()->data_type
                                          : pd->invariant_dst_md()->data_type;
-    int mask_wei = 0, mask_src = 0, mask_dst = 0;
-    if (attr->zero_points_.get(DNNL_ARG_WEIGHTS, &mask_wei) != status::success)
-        return false;
-    if (attr->zero_points_.get(DNNL_ARG_SRC, &mask_src) != status::success)
-        return false;
-    if (attr->zero_points_.get(DNNL_ARG_DST, &mask_dst) != status::success)
-        return false;
 
-    if (!attr->zero_points_.has_default_values(DNNL_ARG_WEIGHTS)) {
-        if (attr->zero_points_.get_data_type(DNNL_ARG_WEIGHTS) != s8)
-            return false;
+    bool ok = IMPLICATION(
+            !utils::one_of(input_type, s8, u8), zp.has_default_values());
+    if (!ok) return false;
+
+    if (!zp.has_default_values(DNNL_ARG_SRC)) {
+        int mask_src = zp.get_mask(DNNL_ARG_SRC);
+        ok = utils::one_of(mask_src, 0, (1 << 1));
+        if (!ok) return false;
+    }
+    if (!zp.has_default_values(DNNL_ARG_WEIGHTS)) {
+        int mask_wei = zp.get_mask(DNNL_ARG_WEIGHTS);
+        ok = mask_wei == 0;
+        if (!ok) return false;
+
+        if (zp.get_data_type(DNNL_ARG_WEIGHTS) != s8) return false;
         if (prb.with_groups) return false;
-        if (mask_src != 0) return false; // zp_wei implies scalar zp_src
+        // zp_wei implies scalar zp_src
+        if (zp.get_mask(DNNL_ARG_SRC) > 0) return false;
+    }
+    if (!zp.has_default_values(DNNL_ARG_DST)) {
+        int mask_dst = zp.get_mask(DNNL_ARG_DST);
+        ok = utils::one_of(mask_dst, 0, (1 << 1));
+        if (!ok) return false;
     }
 
-    return IMPLICATION(!utils::one_of(input_type, s8, u8),
-                   attr->zero_points_.has_default_values())
-            && (mask_wei == 0) && (mask_src == 0 || mask_src == 1 << 1)
-            && (mask_dst == 0 || mask_dst == 1 << 1);
+    return true;
 }
 
 bool post_ops_ok(const conv_problem_t &prb, const hw_t &hw) {
@@ -967,10 +1000,8 @@ bool post_ops_ok(const conv_problem_t &prb, const hw_t &hw) {
     using sm = primitive_attr_t::skip_mask_t;
     auto attr_skip_mask = sm::fpmath_mode | sm::accumulation_mode;
     if (prb.is_fwd || prb.is_bwd_d) {
-        attr_skip_mask |= sm::post_ops | sm::sum_dt | sm::zero_points_runtime
-                | sm::zero_points_runtime_data_type | sm::scales_runtime
-                | sm::rounding_mode | sm::scales_runtime_groups
-                | sm::scales_runtime_data_type;
+        attr_skip_mask |= sm::post_ops | sm::sum_dt | sm::zero_points_data_type
+                | sm::rounding_mode | sm::scales_data_type;
         if (!attr->has_default_values(attr_skip_mask)) return false;
     } else {
         if (!attr->has_default_values(attr_skip_mask)) return false;
@@ -991,7 +1022,9 @@ bool post_ops_ok(const conv_problem_t &prb, const hw_t &hw) {
         scales[i] = scale_args[i].second;
     if (!attr->scales_.has_default_values(scales)) return false;
     for (int arg : scales) {
-        int mask = attr->scales_.get(arg).mask_;
+        if (attr->scales_.has_default_values(arg)) continue;
+
+        int mask = attr->scales_.get(arg).get_mask();
         // XXX: per_oc for BWD_D is treated as per_ic assuming it's called from
         // deconvolution.
         if (arg == DNNL_ARG_WEIGHTS) {
@@ -1006,7 +1039,7 @@ bool post_ops_ok(const conv_problem_t &prb, const hw_t &hw) {
     for (int i = 0; i < attr->post_ops_.len(); i++) {
         auto &po = attr->post_ops_.entry_[i];
         if (po.is_eltwise()) {
-            if (!jit_eltwise_injector_f32_is_supported(po.eltwise.alg))
+            if (!eltwise_injector_f32_is_supported(po.eltwise.alg))
                 return false;
             else if (po.eltwise.alg == alg_kind::eltwise_tanh
                     && hw == ngen::HW::XeHPG
@@ -1024,9 +1057,14 @@ bool post_ops_ok(const conv_problem_t &prb, const hw_t &hw) {
 }
 
 bool should_use_mad(const conv_problem_t &prb) {
-    bool small_ic_oc = prb.ic < 3 && prb.oc < 3 && prb.mb < 8;
-    bool grouped_small_ic_oc = prb.ic < 4 && prb.oc < 4 && prb.g > 1;
-    return prb.is_dw || small_ic_oc || grouped_small_ic_oc;
+    if (prb.is_dw) return true;
+    if (prb.is_bwd_w) return false;
+    dim_t kw_xc = prb.kw * (prb.is_fwd ? prb.ic : prb.oc);
+    bool small_ic_oc = (prb.oc <= 3 && prb.ic <= 3 && kw_xc <= 10)
+            || (prb.oc <= 2 && prb.ic <= 2);
+    bool small_mb_ic_oc = prb.mb < 8 && small_ic_oc;
+    bool grouped_small_ic_oc = prb.g > 1 && small_ic_oc;
+    return small_mb_ic_oc || grouped_small_ic_oc;
 }
 
 status_t init_fma_kind(
@@ -1105,7 +1143,7 @@ bool post_op_layouts_ok(const conv_problem_t &prb) {
                             po.binary.src1_desc.dims, prb.ndims, true);
             // These cases don't have message-related limitations.
             if ((mask & (1 << 1)) == 0 || mask == (1 << 1)) continue;
-            auto rhs_layout = po.is_prelu()
+            const auto &rhs_layout = po.is_prelu()
                     ? layout_t(type_t::f32(), 0,
                             get_prelu_weights_dims(po.prelu.mask, output_md))
                     : layout_t(po.binary.src1_desc);
@@ -1170,6 +1208,7 @@ status_t init_pd_time_cfg(const conv_problem_t &prb, conv_config_t &cfg,
     cfg.set_exec_cfg(exec_config_t(hw));
     cfg.maybe_override_from_env();
 
+    cfg.set_require_signal_header(true);
     CHECK(init_fma_kind(cfg, pd, engine));
     CHECK(init_simd(cfg));
     CHECK(init_vec_size(cfg));
@@ -1278,7 +1317,7 @@ send_pattern_t<pvar_t> validate_blocking(const conv_config_t &cfg,
         return all_hints;
     }();
     if (hints.empty()) {
-        ir_suggestion() << "No hints generated! ";
+        gpu_suggestion() << "No hints generated!";
         return send_pattern();
     }
 
@@ -1286,11 +1325,11 @@ send_pattern_t<pvar_t> validate_blocking(const conv_config_t &cfg,
         if (is_match(h)) { return send_pattern(h); }
     }
 
-    ir_suggestion() << "blocking disables " << send_pattern(hints[0])
-                    << " load of the " << tensor
-                    << " tensor. Try a multiple of:\n";
+    gpu_suggestion() << "blocking disables " << send_pattern(hints[0])
+                     << " load of the " << tensor
+                     << " tensor. Try a multiple of:";
     for (auto &hint : hints) {
-        ir_suggestion() << "\t" << hint.str() << "\n";
+        gpu_suggestion() << "\t" << hint.str();
     }
 
     return send_pattern();
@@ -1338,7 +1377,7 @@ pvar_tile_3 get_thread_group_grid_conv_dims(const conv_config_t &cfg) {
     if (prb.is_fwd) return (prb.ab_swap_transpose) ? t_fwd : fwd;
     if (prb.is_bwd_d) return (prb.ab_swap_transpose) ? t_bwd_d : bwd_d;
     if (prb.is_bwd_w) return (prb.ab_swap_transpose) ? t_bwd_w : bwd_w;
-    ir_error_not_expected();
+    gpu_error_not_expected();
     return fwd;
 }
 
@@ -1374,9 +1413,9 @@ void get_layout_and_dims(tensor_kind_t ab_kind, const conv_config_t &cfg,
             dims = prb.pick_b<const std::vector<pvar_t> &>(
                     src_dims, wei_dims, dst_dims);
             break;
-        default: ir_error_not_expected();
+        default: gpu_error_not_expected();
     }
-    ir_assert(layout.ndims() == dims.size());
+    gpu_assert(layout.ndims() == dims.size());
 }
 
 // Calculates the size of the range for spatial dimensions within a tile.
@@ -1396,7 +1435,7 @@ dim_t map_spatial(
     dim_t stride[] = {prb.sd, prb.sh, prb.sw};
     dim_t dilation[] = {prb.dd, prb.dh, prb.dw};
     int idx = dim.spatial_index();
-    ir_assert(idx != -1);
+    gpu_assert(idx != -1);
     dim_t O = tile.get(osp_dims[idx], 1);
     dim_t I = tile.get(isp_dims[idx], 1);
     dim_t K = tile.get(ksp_dims[idx], 1);
@@ -1405,13 +1444,13 @@ dim_t map_spatial(
     dim_t D = dilation[idx];
     if (is_isp) {
         // Source tensor, map ox, kx to ix.
-        ir_assert(prb.is_fwd || prb.is_bwd_w);
+        gpu_assert(prb.is_fwd || prb.is_bwd_w);
         dim_t i_min = -P;
         dim_t i_max = (O - 1) * S - P + (K - 1) * (1 + D);
         return std::min(isp[idx], i_max - i_min + 1);
     }
     // Destination tensor, map ix, kx to ox.
-    ir_assert(is_osp && prb.is_bwd_d);
+    gpu_assert(is_osp && prb.is_bwd_d);
     dim_t os_min = P - (K - 1) * (1 + D);
     dim_t os_max = (I - 1) + P;
     return std::min(osp[idx], utils::div_up(os_max - os_min + 1, S));
@@ -1440,7 +1479,7 @@ size_t get_memory_footprint(const tensor_kind_t &ab_kind,
         tile[d] = d_size;
         elems *= std::min(d_size, layout.dim(i));
     }
-    ir_assert(elems >= 1);
+    gpu_assert(elems >= 1);
     return (size_t)layout.type().size() * elems;
 }
 
@@ -1555,12 +1594,13 @@ public:
         : prb_(prb) {
         for (auto &d : tile) {
             auto bmnk = to_gemm(d, prb);
-            entry_t e;
+            if (!utils::one_of(bmnk, pvars::m, pvars::n)) continue;
+
+            entries_.emplace_back();
+            entry_t &e = entries_.back();
             e.dim = d;
             e.tile_size = tile[d];
-            if (!utils::one_of(bmnk, pvars::m, pvars::n)) continue;
             e.mn_kind = (bmnk == pvars::m ? 'm' : 'n');
-            entries_.push_back(e);
         }
         // Put through spatial dimensions first and order spatial accordingly
         // (WHD, width is first).
@@ -1600,7 +1640,7 @@ public:
                 }
             }
         }
-        ir_error_not_expected();
+        gpu_error_not_expected();
         return entry_t();
     }
 
@@ -1766,7 +1806,7 @@ void init_slm(conv_config_t &cfg) {
         }
         bufs = fixup_slm_bufs(prb, bufs, cfg.zp_cfg().do_src_compensation,
                 enable_a, enable_b, cfg.pipeline().do_unroll());
-        ir_assert(bufs > 0);
+        gpu_assert(bufs > 0);
         gmem_bufs = (cfg.is_dp_fma() && cfg.pipeline().do_unroll()) ? 2 : 1;
     }
     gmem_bufs = std::min(cfg.plan().max_gmem_bufs, gmem_bufs);
@@ -1820,7 +1860,7 @@ void validate_config_and_plan(conv_config_t &cfg) {
                   for (auto &tile : grid)
                       for (auto &d : tile)
                           if (d == dim) return;
-                  ir_error_not_expected() << dim.name();
+                  gpu_error_not_expected() << dim.name();
               };
     const auto &tg_dims = get_thread_group_grid_conv_dims(cfg);
     const auto &grid_dims = get_kernel_grid_conv_dims(cfg);
@@ -1830,9 +1870,9 @@ void validate_config_and_plan(conv_config_t &cfg) {
     }
 
     auto &plan = cfg.plan();
-    ir_assert(cfg.slm().a() == plan.slm.has_a());
-    ir_assert(cfg.slm().b() == plan.slm.has_b());
-    ir_assert(cfg.pipeline().reuse_headers() == plan.reuse_headers);
+    gpu_assert(cfg.slm().a() == plan.slm.has_a());
+    gpu_assert(cfg.slm().b() == plan.slm.has_b());
+    gpu_assert(cfg.pipeline().reuse_headers() == plan.reuse_headers);
 
 #ifdef DNNL_DEV_MODE
     using send_pattern = send_pattern_t<pvar_t>;
@@ -1861,13 +1901,13 @@ void validate_config_and_plan(conv_config_t &cfg) {
     auto dummy_reg(var_t::make(type_t::byte_ptr(), "reg"));
     if (!a_load_pattern.matches(
                 plan.x2r.a_load.create_stmt(dummy_mem, dummy_reg))) {
-        ir_warning() << "Generated load for tensor A does not match "
-                     << a_load_pattern << " load idiom\n";
+        gpu_warning() << "Generated load for tensor A does not match "
+                      << a_load_pattern << " load idiom";
     }
     if (!b_load_pattern.matches(
                 plan.x2r.b_load.create_stmt(dummy_mem, dummy_reg))) {
-        ir_warning() << "Generated load for tensor B does not match "
-                     << a_load_pattern << " load idiom\n";
+        gpu_warning() << "Generated load for tensor B does not match "
+                      << a_load_pattern << " load idiom";
     }
 #endif
 }
