@@ -29,7 +29,7 @@
 #include "gpu/intel/jit/ir/kernel_info.hpp"
 #include "gpu/intel/jit/reorder/reorder_kernel.hpp"
 #include "gpu/intel/jit/utils/utils.hpp"
-#include "gpu/intel/ocl/ocl_utils.hpp"
+#include "gpu/intel/ocl/utils.hpp"
 
 #include "gpu/intel/jit/conv/config.hpp"
 #include "gpu/intel/jit/conv/conv_kernel.hpp"
@@ -84,8 +84,7 @@ public:
                     || pd->data->pd_cfg.zp_cfg().needs_src_conv_precalc) {
                 primitive_attr_t attr;
                 if (pd->data->pd_cfg.zp_cfg().needs_src_conv_precalc) {
-                    int mask = 0;
-                    CHECK(pd->attr_.zero_points_.get(DNNL_ARG_SRC, &mask));
+                    int mask = pd->attr_.zero_points_.get_mask(DNNL_ARG_SRC);
                     attr.zero_points_.set(DNNL_ARG_SRC, mask);
                     attr.post_ops_.append_eltwise(
                             1.f, alg_kind::eltwise_linear, -1.f, 0.f);
@@ -154,8 +153,8 @@ public:
 
                 if (!tiler->is_grf_limit_ok(cfg)) continue;
 
-                ir_info() << "Configuration:" << std::endl;
-                ir_info() << cfg;
+                gpu_info() << "Configuration:";
+                gpu_info() << cfg;
 
                 init_nd_ranges(primitive, cfg);
                 auto &kernel_infos = data.kernel_infos;
@@ -164,7 +163,7 @@ public:
                 // since it adds its own version mark to the cache blob
                 for (int i = 0; i < int(kernel_infos.size()); i++)
                     if (kernel_infos[i].id() == kernel_id_t::zp_precalc) {
-                        ir_assert(data.zp_pd);
+                        gpu_assert(data.zp_pd);
                         CHECK(primitive->create_nested_primitive(
                                 zp_prim_, data.zp_pd, engine));
                     }
@@ -220,7 +219,7 @@ public:
                             tmp_kernels.emplace_back();
                             continue;
 
-                        default: ir_error_not_expected();
+                        default: gpu_error_not_expected();
                     }
                     if (!tmp_kernels[i]) return status::runtime_error;
                 }
@@ -242,7 +241,7 @@ public:
             }
         }
         if (!ok) return status::runtime_error;
-        ir_assert(kernels_.size() == data.kernel_infos.size());
+        gpu_assert(kernels_.size() == data.kernel_infos.size());
         CHECK(primitive->register_kernels(kernels_));
 
         conv_tiler_t::after_create_hook(cfg, primitive);
@@ -284,7 +283,7 @@ public:
                                 new memory_t(ctx.stream()->engine(), md,
                                         std::move(s)));
                     };
-                    ir_assert(zp_prim_);
+                    gpu_assert(zp_prim_);
                     std::unique_ptr<memory_t, memory_deleter_t> zp_src, zp_dst;
                     CHECK(scratchpad_arg(
                             zp_src, "src_zero_points", zp_conv_md_in(data)));
@@ -328,7 +327,7 @@ private:
     template <typename T>
     static kernel_info_t &create_kernel_info(T *pd, kernel_id_t kernel_id) {
         auto &infos = pd->data->kernel_infos;
-        ir_assert((int)infos.size() + 1 <= max_kernels);
+        gpu_assert((int)infos.size() + 1 <= max_kernels);
         infos.emplace_back();
         auto &ret = infos.back();
         ret.set_id(kernel_id);
@@ -360,9 +359,9 @@ private:
             int compute_arg_key = t.arg_key;
 
             if (compute_arg_key == DNNL_ARG_UNDEF) {
-                ir_assert(!t.needs_reorder);
-                ir_assert(!t.needs_zero_out);
-                ir_error_not_expected();
+                gpu_assert(!t.needs_reorder);
+                gpu_assert(!t.needs_zero_out);
+                gpu_error_not_expected();
                 continue;
             }
 
@@ -400,8 +399,9 @@ private:
                     reorder_info.register_user_arg(user_buf, user_arg_key,
                             /*is_input=*/true);
                     add_compute_arg(reorder_info, compute_buf, false);
-                    reorder_info.set_nd_range(reorder_kernel_t<>::nd_range(
-                            cfg.exec_cfg(), t.user_layout, t.compute_layout));
+                    reorder_config_t reorder_cfg(
+                            cfg.exec_cfg(), t.user_layout, t.compute_layout);
+                    reorder_info.set_nd_range(reorder_cfg.nd_range());
                 }
                 if (!src_conv_precalc && t.is_output) {
                     auto &reorder_info
@@ -409,8 +409,9 @@ private:
                     add_compute_arg(reorder_info, compute_buf, true);
                     reorder_info.register_user_arg(user_buf, user_arg_key,
                             /*is_input=*/false);
-                    reorder_info.set_nd_range(reorder_kernel_t<>::nd_range(
-                            cfg.exec_cfg(), t.compute_layout, t.user_layout));
+                    reorder_config_t reorder_cfg(
+                            cfg.exec_cfg(), t.compute_layout, t.user_layout);
+                    reorder_info.set_nd_range(reorder_cfg.nd_range());
                 }
                 if (src_conv_precalc) {
                     scratchpad_book(++scratchpad_key);
@@ -471,14 +472,14 @@ private:
                     nd_ranges_[i] = info.nd_range();
                     break;
                 case kernel_id_t::zp_precalc: break;
-                default: ir_error_not_expected();
+                default: gpu_error_not_expected();
             }
         }
     }
 
     static bool can_skip_zero_out(
             const kernel_info_t &info, const conv_config_t &cfg) {
-        ir_assert(info.id() == kernel_id_t::zero_out);
+        gpu_assert(info.id() == kernel_id_t::zero_out);
         auto &buf_name = info.arg_var(1).as<var_t>().name;
         if (buf_name == "wei") return cfg.can_skip_wei_zero_out();
         if (buf_name == "bia") return cfg.can_skip_bia_zero_out();
