@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2023-2024 Intel Corporation
+* Copyright 2023-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -42,10 +42,10 @@ enum class hw_t {
     xehpc,
 };
 
-hw_t to_hw(const std::string &s) {
+inline hw_t to_hw(const std::string &s) {
     if (s == "xehpg") return hw_t::xehpg;
     if (s == "xehpc") return hw_t::xehpc;
-    ir_assert(false);
+    gpu_assert(false);
     return hw_t::undef;
 }
 
@@ -55,10 +55,10 @@ enum class fma_t {
     mad,
 };
 
-fma_t to_fma(const std::string &s) {
+inline fma_t to_fma(const std::string &s) {
     if (s == "mad") return fma_t::mad;
     if (s == "dpas") return fma_t::dpas;
-    ir_assert(false);
+    gpu_assert(false);
     return fma_t::undef;
 }
 
@@ -69,18 +69,18 @@ enum class prop_t {
     bwd_w,
 };
 
-prop_t to_prop(const std::string &s) {
+inline prop_t to_prop(const std::string &s) {
     if (s == "fwd") return prop_t::fwd;
     if (s == "bwd_d") return prop_t::bwd_d;
     if (s == "bwd_w") return prop_t::bwd_w;
-    ir_assert(false);
+    gpu_assert(false);
     return prop_t::undef;
 }
 
 enum class type_t { undef, d8, d16, d32, d64 };
 
-void get_types(const std::string &type_cfg, prop_t prop, type_t &src_type,
-        type_t &dst_type) {
+inline void get_types(const std::string &type_cfg, prop_t prop,
+        type_t &src_type, type_t &dst_type) {
     const std::pair<const char *, type_t> all_types[] = {
             std::make_pair("bf16", type_t::d16),
             std::make_pair("f16", type_t::d16),
@@ -113,26 +113,28 @@ void get_types(const std::string &type_cfg, prop_t prop, type_t &src_type,
     }
     if (pos != type_cfg.length() || idx != ntypes) {
         std::cout << type_cfg << std::endl;
-        ir_assert(false);
+        gpu_assert(false);
     }
     switch (prop) {
         case prop_t::fwd: break;
         case prop_t::bwd_d: std::swap(types[0], types[2]); break;
         case prop_t::bwd_w: std::swap(types[1], types[2]); break;
-        default: ir_assert(false);
+        default: gpu_assert(false);
     }
     src_type = types[0];
     dst_type = types[2];
 }
 
-type_t to_src_type(const std::string &type_cfg, prop_t prop = prop_t::fwd) {
+inline type_t to_src_type(
+        const std::string &type_cfg, prop_t prop = prop_t::fwd) {
     type_t src_type;
     type_t dst_type;
     get_types(type_cfg, prop, src_type, dst_type);
     return src_type;
 }
 
-type_t to_dst_type(const std::string &type_cfg, prop_t prop = prop_t::fwd) {
+inline type_t to_dst_type(
+        const std::string &type_cfg, prop_t prop = prop_t::fwd) {
     type_t src_type;
     type_t dst_type;
     get_types(type_cfg, prop, src_type, dst_type);
@@ -170,7 +172,7 @@ struct hw_config_t {
                 s8_dpas_ops_per_clock = 1024;
                 f32_mad_ops_per_clock = 32;
                 break;
-            default: ir_assert(false); break;
+            default: gpu_assert(false); break;
         }
         bool is_dpas = (fma == fma_t::dpas);
         switch (src_type) {
@@ -184,7 +186,7 @@ struct hw_config_t {
                 ops_per_clock = (is_dpas ? s8_dpas_ops_per_clock / 8
                                          : f32_mad_ops_per_clock / 2);
                 break;
-            default: ir_assert(false); break;
+            default: gpu_assert(false); break;
         }
         threads_per_eu = (is_dpas ? 4 : 8);
     }
@@ -209,10 +211,10 @@ enum class metric_t {
     msre, // Mean squared relative error.
 };
 
-metric_t to_metric(const std::string &s) {
+inline metric_t to_metric(const std::string &s) {
     if (s == "mse") return metric_t::mse;
     if (s == "msre") return metric_t::msre;
-    ir_assert(false);
+    gpu_assert(false);
     return metric_t::undef;
 }
 
@@ -224,11 +226,11 @@ enum class score_t {
     mape, // Mean absolute percentage error.
 };
 
-score_t to_score(const std::string &s) {
+inline score_t to_score(const std::string &s) {
     if (s == "r2") return score_t::r2;
     if (s == "mae") return score_t::mae;
     if (s == "mape") return score_t::mape;
-    ir_assert(false);
+    gpu_assert(false);
     return score_t::undef;
 }
 
@@ -507,7 +509,7 @@ struct conv_sample_t {
                 n = t.oc;
                 k = t.mb * t.od * t.oh * t.ow;
                 break;
-            default: ir_assert(false);
+            default: gpu_assert(false);
         }
         if (transpose) std::swap(m, n);
     }
@@ -529,7 +531,7 @@ struct conv_sample_t {
                 dims.push_back(&ret.ih);
                 dims.push_back(&ret.iw);
                 break;
-            default: ir_assert(false);
+            default: gpu_assert(false);
         }
         for (auto *d : dims)
             if (*d == -1) *d = 1;
@@ -567,7 +569,7 @@ public:
                     buckets_[i].push_back(kv.first);
                 }
             }
-            ir_assert((int)buckets_[i].size() <= bucket_count);
+            gpu_assert((int)buckets_[i].size() <= bucket_count);
         }
     }
 
@@ -582,7 +584,7 @@ public:
 
     template <typename T>
     vec1d<T> to(const vec1d<float> &x) const {
-        ir_assert(x.size() == buckets_.size());
+        gpu_assert(x.size() == buckets_.size());
         vec1d<T> ret(x.size());
         for (int i = 0; i < (int)x.size(); i++)
             ret[i] = to_bucket<T>(x[i], i);
@@ -597,7 +599,7 @@ public:
         return ret;
     }
 
-    void serialize(serialized_data_t &s) const { s.append(buckets_); }
+    void serialize(serialization_stream_t &s) const { s.append(buckets_); }
 
     static histogram_t deserialize(deserializer_t &d) {
         histogram_t h;
@@ -609,7 +611,8 @@ private:
     vec2d<float> buckets_;
 };
 
-float r2_score(const std::vector<float> &y, const std::vector<float> &y_pred) {
+inline float r2_score(
+        const std::vector<float> &y, const std::vector<float> &y_pred) {
     float u = 0;
     float v = 0;
     float y_mean = 0;
@@ -624,7 +627,8 @@ float r2_score(const std::vector<float> &y, const std::vector<float> &y_pred) {
     return 1 - u / v;
 }
 
-float mae_score(const std::vector<float> &y, const std::vector<float> &y_pred) {
+inline float mae_score(
+        const std::vector<float> &y, const std::vector<float> &y_pred) {
     int n = (int)y.size();
     float err = 0;
     for (int i = 0; i < n; i++) {
@@ -633,7 +637,7 @@ float mae_score(const std::vector<float> &y, const std::vector<float> &y_pred) {
     return -err / n;
 }
 
-float mape_score(
+inline float mape_score(
         const std::vector<float> &y, const std::vector<float> &y_pred) {
     float eps = std::numeric_limits<float>::epsilon();
     int n = (int)y.size();
@@ -644,13 +648,13 @@ float mape_score(
     return -err / n;
 }
 
-float score(const std::vector<float> &y, const std::vector<float> &y_pred,
-        score_t score) {
+inline float score(const std::vector<float> &y,
+        const std::vector<float> &y_pred, score_t score) {
     switch (score) {
         case score_t::r2: return r2_score(y, y_pred);
         case score_t::mae: return mae_score(y, y_pred);
         case score_t::mape: return mape_score(y, y_pred);
-        default: ir_assert(false);
+        default: gpu_assert(false);
     }
     return 0;
 }
@@ -688,7 +692,7 @@ public:
     int feature_count() const { return nfeatures_; }
 
     float predict(const vec1d<x_type> &x) const {
-        ir_assert((int)x.size() == (int)nfeatures_);
+        gpu_assert((int)x.size() == (int)nfeatures_);
         return predict_impl(x, 0);
     }
 
@@ -715,7 +719,7 @@ public:
             walk(node.right);
         };
         walk(0);
-        ir_assert(non_leaf_nodes * 2 + 1 == node_count());
+        gpu_assert(non_leaf_nodes * 2 + 1 == node_count());
         for (auto &c : count)
             c /= non_leaf_nodes;
         return count;
@@ -728,7 +732,7 @@ public:
         std::cout << "  Nodes:      " << node_count() << std::endl;
     }
 
-    void serialize(serialized_data_t &s) const {
+    void serialize(serialization_stream_t &s) const {
         s.append(nfeatures_);
         s.append(max_depth_);
         s.append(subsamples_);
@@ -761,7 +765,7 @@ private:
     }
 
     const tree_node_t &get_node(int idx) const {
-        ir_assert(idx < reserved_nodes_);
+        gpu_assert(idx < reserved_nodes_);
         return nodes_[idx];
     }
 
@@ -895,7 +899,7 @@ private:
                     err += weight * val * val;
                 }
                 break;
-            default: ir_assert(false);
+            default: gpu_assert(false);
         }
         return err / total;
     }
@@ -919,15 +923,15 @@ private:
             data.resize(off + sizeof(node.value));
             std::memcpy(&data[off], &node.value, sizeof(node.value));
         } else {
-            ir_assert(node.feature_idx >= 0 && node.feature_idx <= u8_max);
-            ir_assert(node.value >= 0 && node.value <= u8_max);
+            gpu_assert(node.feature_idx >= 0 && node.feature_idx <= u8_max);
+            gpu_assert(node.value >= 0 && node.value <= u8_max);
             data.push_back((uint8_t)node.feature_idx);
             data.push_back((uint8_t)node.value);
             size_t right_off_idx = data.size();
             data.push_back(0);
             data.push_back(0);
             size_t right_off = serialize_node(data, node.left);
-            ir_assert(right_off <= u16_max);
+            gpu_assert(right_off <= u16_max);
             std::memcpy(&data[right_off_idx], &right_off, sizeof(uint16_t));
             serialize_node(data, node.right);
         }
@@ -943,10 +947,10 @@ private:
         int left = -1;
         int right = -1;
         if (is_leaf) {
-            ir_assert(off + 1 + sizeof(value) <= data.size());
+            gpu_assert(off + 1 + sizeof(value) <= data.size());
             std::memcpy(&value, &data[off + 1], sizeof(value));
         } else {
-            ir_assert(off + 3 < data.size());
+            gpu_assert(off + 3 < data.size());
             value = (float)data[off + 1];
             uint8_t right_off_u8[2] = {data[off + 2], data[off + 3]};
             uint16_t right_off;
@@ -1001,7 +1005,7 @@ public:
     int tree_count() const { return (int)trees_.size(); }
 
     int feature_count() const {
-        ir_assert(!trees_.empty());
+        gpu_assert(!trees_.empty());
         return trees_[0].feature_count();
     }
 
@@ -1025,14 +1029,14 @@ public:
     }
 
     float score(const vec2d<float> &X, const vec1d<float> &y, score_t score,
-            int max_trees = std::numeric_limits<int>::max()) {
+            int max_trees = std::numeric_limits<int>::max()) const {
         auto y_pred = predict(X, max_trees);
         return model::score(y, y_pred, score);
     }
 
     std::vector<std::pair<std::string, float>> feature_importances(
             const std::vector<const char *> &feature_names) const {
-        ir_assert((int)feature_names.size() == feature_count());
+        gpu_assert((int)feature_names.size() == feature_count());
         vec1d<float> fi(feature_count());
         for (auto &tree : trees_) {
             auto tree_fi = tree.feature_importances();
@@ -1045,6 +1049,7 @@ public:
         }
         using entry_t = std::pair<std::string, float>;
         std::vector<entry_t> ret;
+        ret.reserve(feature_count());
         for (int i = 0; i < feature_count(); i++) {
             ret.emplace_back(feature_names[i], fi[i]);
         }
@@ -1056,7 +1061,7 @@ public:
     }
 
     void print_info(const std::vector<const char *> &feature_names,
-            const std::string prefix = "") const {
+            const std::string &prefix = "") const {
         std::cout << prefix << "Gradient boost regressor" << std::endl;
         std::cout << prefix << "  Features:   " << feature_count() << std::endl;
         std::cout << prefix << "  Trees:      " << tree_count() << std::endl;
@@ -1070,12 +1075,12 @@ public:
     }
 
     int serialized_size() const {
-        serialized_data_t s;
+        serialization_stream_t s;
         serialize(s);
         return (int)s.get_data().size();
     }
 
-    void serialize(serialized_data_t &s) const {
+    void serialize(serialization_stream_t &s) const {
         s.append(learning_rate_);
         s.append(hist_);
         s.append(f0_);
