@@ -51,9 +51,25 @@ bool is_reduction_dim(const pvar_t &d, const conv_problem_t &prb) {
     return to_gemm(d, prb) == pvars::k;
 }
 
-bool is_vectorized_dim(const pvar_t &d, const conv_problem_t &prb) {
-    if (prb.is_dw) return d == pvars::g;
-    return to_gemm(d, prb) == pvars::n;
+pvar_t vectorized_dim(const conv_problem_t &prb, const pvar_tile_t &tile) {
+    pvar_t vec_dim;
+    if (prb.is_dw) {
+        vec_dim = pvars::g;
+    } else {
+        for (auto &d : tile) {
+            if (to_gemm(d, prb) != pvars::n) continue;
+            gpu_assert(vec_dim.is_undef()) << "Found 2+ N dimensions: " << tile;
+            vec_dim = d;
+        }
+    }
+    gpu_assert(!vec_dim.is_undef())
+            << "Cannot find vectorized dimension: " << tile;
+    return vec_dim;
+}
+
+bool is_vectorized_dim(
+        const pvar_t &d, const conv_problem_t &prb, const pvar_tile_t &tile) {
+    return d == vectorized_dim(prb, tile);
 }
 
 int tensor_conv_dim_index(const pvar_t &d, tensor_kind_t t) {
@@ -66,7 +82,7 @@ int tensor_conv_dim_index(const pvar_t &d, tensor_kind_t t) {
         case tensor_kind_t::src: pvars = &src_dims; break;
         case tensor_kind_t::wei: pvars = &wei_dims; break;
         case tensor_kind_t::dst: pvars = &dst_dims; break;
-        default: ir_error_not_expected();
+        default: gpu_error_not_expected();
     }
     auto it = std::find(pvars->begin(), pvars->end(), d);
     if (it == pvars->end()) return -1;
@@ -78,7 +94,7 @@ struct x2_tile_info_t {
     x2_tile_info_t() = default;
     x2_tile_info_t(const pvar_t &dim0, const pvar_t &dim1)
         : dim0(dim0), dim1(dim1) {
-        ir_assert(dim0 != dim1);
+        gpu_assert(dim0 != dim1);
     }
     void add(tile_flags_t f) { flags = flags | f; }
     void set_iter_unit(int unit) { d.set_iter_unit(unit); }
@@ -107,19 +123,19 @@ struct x2_tile_info_t {
             }
             if (!ret.empty()) return ret;
         }
-        ir_error_not_expected();
+        gpu_error_not_expected();
         return ret;
     }
 
     std::vector<std::pair<int, int>> thread_group_blocks() const {
-        if (any(flags & tile_flags_t::thread_group)) ir_error_not_expected();
+        if (any(flags & tile_flags_t::thread_group)) gpu_error_not_expected();
         return {std::make_pair(1, 1)};
     }
 
     std::vector<std::pair<dim_t, dim_t>> loop_blocks(
             dim_t size0, dim_t size1) const {
         if (!any(flags & tile_flags_t::loop)) return {std::make_pair(1, 1)};
-        if (!any(flags & tile_flags_t::loop_span)) ir_error_not_expected();
+        if (!any(flags & tile_flags_t::loop_span)) gpu_error_not_expected();
         return {std::make_pair(size0, size1)};
     }
 
@@ -136,7 +152,7 @@ const layout_t &compute_layout(const conv_config_t &cfg, tensor_kind_t kind) {
         case tensor_kind_t::src: return cfg.src_layout().compute();
         case tensor_kind_t::wei: return cfg.wei_layout().compute();
         case tensor_kind_t::dst: return cfg.dst_layout().compute();
-        default: ir_error_not_expected();
+        default: gpu_error_not_expected();
     }
     return cfg.src_layout().compute();
 }
@@ -239,7 +255,7 @@ public:
         auto to_idx = [&](const pvar_t &d) {
             for (int i = 0; i < ndims; i++)
                 if (all_dims[i] == d) return i;
-            ir_error_not_expected();
+            gpu_error_not_expected();
             return -1;
         };
 
@@ -249,7 +265,7 @@ public:
             int idx1 = to_idx(info.dim1);
             get_level_tiles(padded_shape[info.dim0], padded_shape[info.dim1],
                     info, tiles[idx0], tiles[idx1]);
-            ir_assert(!seen[idx0] && !seen[idx1]);
+            gpu_assert(!seen[idx0] && !seen[idx1]);
             seen[idx0] = seen[idx1] = true;
             deps[std::max(idx0, idx1)] = std::min(idx0, idx1);
         }
@@ -273,7 +289,7 @@ private:
         for (auto &d : iter_) {
             auto &info = tile_info(d);
             int unit = 1;
-            if (is_vectorized_dim(d, prb)) unit = cfg.vec_size();
+            if (is_vectorized_dim(d, prb, iter_)) unit = cfg.vec_size();
             if (is_reduction_dim(d, prb)) {
                 // This is to ensure that reduction-related address shifts are
                 // constant. For example with a_blk = 8 and Ax16a layout there are two
@@ -328,15 +344,15 @@ private:
             (d0.is_undef() ? d0 : d1) = d;
         }
         if (rdims == 1) return;
-        ir_assert(rdims == 2) << "Can't fuse more than two dimensions.";
+        gpu_assert(rdims == 2) << "Can't fuse more than two dimensions.";
         auto &info0 = tile_info(d0);
         auto &info1 = tile_info(d1);
         tile_flags_t flags = tile_flags_t::iter | tile_flags_t::loop
                 | tile_flags_t::loop_span;
-        ir_assert(info0.flags == flags);
-        ir_assert(info1.flags == flags);
-        ir_assert(info0.min_iter_blk == tile_info_t::default_min_iter_blk);
-        ir_assert(info1.min_iter_blk == tile_info_t::default_min_iter_blk);
+        gpu_assert(info0.flags == flags);
+        gpu_assert(info1.flags == flags);
+        gpu_assert(info0.min_iter_blk == tile_info_t::default_min_iter_blk);
+        gpu_assert(info1.min_iter_blk == tile_info_t::default_min_iter_blk);
 
         int unit = 32 / prb.a_data_type_size;
         x2_tile_info_t x2_info(d0, d1);
@@ -344,12 +360,19 @@ private:
         x2_info.set_iter_unit(unit);
         x2_info.d0 = info0.div_info;
         x2_info.d1 = info1.div_info;
-        x2_tile_infos_.push_back(x2_info);
+        x2_tile_infos_.push_back(std::move(x2_info));
     }
 
     void finalize_loop_dims(const conv_config_t &cfg) {
         auto &prb = cfg.prb();
         if (prb.is_bwd_w) {
+            if (!cfg.allow_global_reduction()) {
+                for (auto &d : {pvars::mb, pvars::od, pvars::oh, pvars::ow}) {
+                    loop_[d] = 1;
+                    tile_info(d).add(tile_flags_t::loop);
+                    tile_info(d).add(tile_flags_t::loop_span);
+                }
+            }
             struct loop_dim_t {
                 pvar_t dim;
                 dim_t size = 0;
@@ -372,7 +395,7 @@ private:
                 ld.size = shape.get(d, 1);
                 if (iter_.has(d))
                     ld.size = utils::div_up(ld.size, iter_dim_hint);
-                loop_dims.push_back(ld);
+                loop_dims.push_back(std::move(ld));
             }
             std::sort(loop_dims.begin(), loop_dims.end(),
                     [&](const loop_dim_t &a, const loop_dim_t &b) {
@@ -406,7 +429,7 @@ protected:
 int inner_block(const conv_config_t &cfg, tensor_kind_t tensor_kind,
         const pvar_t &dim) {
     int dim_idx = tensor_conv_dim_index(dim, tensor_kind);
-    ir_assert(dim_idx != -1);
+    gpu_assert(dim_idx != -1);
     auto &layout = compute_layout(cfg, tensor_kind);
     return into<int>(layout.inner_block(
             dim_idx, /*skip_outer=*/true, /*inner_only=*/false));
@@ -425,7 +448,7 @@ int inner_block(const conv_config_t &cfg, const pvar_t &dim) {
 dim_t inner_stride(const conv_config_t &cfg, tensor_kind_t tensor_kind,
         const pvar_t &dim) {
     dim_idx_t dim_idx = tensor_conv_dim_index(dim, tensor_kind);
-    ir_assert(dim_idx != dim_idx::invalid);
+    gpu_assert(dim_idx != dim_idx::invalid);
     auto &layout = compute_layout(cfg, tensor_kind);
     for (auto &b : layout.blocks()) {
         if (b.dim_idx == dim_idx) return (dim_t)b.stride;
@@ -520,7 +543,7 @@ public:
     }
 
     void reset_checks() override {
-        ir_assert((int)check_kind_t::_max < 64);
+        gpu_assert((int)check_kind_t::_max < 64);
 
         check_mask_ = 0;
         optional_check_mask_ = 0;
@@ -558,6 +581,7 @@ public:
     bool is_ok(const blocking_t &blk) const override {
         context_t ctx(blk, cfg_);
         if (!check_vec_ok(ctx)) return false;
+        if (!check_fp4_ok(ctx)) return false;
         if (!check_tg_size_ok(ctx)) return false;
         if (!check_dpas_ok(ctx)) return false;
         if (!check_grf_usage_ok(ctx)) return false;
@@ -574,25 +598,28 @@ public:
 
 private:
     struct context_t {
-        context_t(const blocking_t &blk, const conv_config_t &cfg) : blk(blk) {
-            auto &prb = cfg.prb();
-            auto gemm_iter = to_gemm(blk.iter(), prb);
-            auto gemm_loop = to_gemm(blk.loop(), prb);
-            auto gemm_tg = to_gemm(blk.thread_group(), prb);
-            b_iter = gemm_iter.get(pvars::b, 1);
-            m_iter = gemm_iter.get(pvars::m, 1);
-            n_iter = gemm_iter.get(pvars::n, 1);
-            k_iter = gemm_iter.get(pvars::k, 1);
-            k_loop = gemm_loop.get(pvars::k, 1);
-            b_tg = gemm_tg.get(pvars::b, 1);
-            m_tg = gemm_tg.get(pvars::m, 1);
-            n_tg = gemm_tg.get(pvars::n, 1);
-            k_tg = gemm_tg.get(pvars::k, 1);
-            dpas_2x_depth = get_dpas_2x_depth(blk, cfg);
-        }
+        context_t(const blocking_t &blk, const conv_config_t &cfg)
+            : context_t(blk, cfg, to_gemm(blk.iter(), cfg.prb()),
+                    to_gemm(blk.loop(), cfg.prb()),
+                    to_gemm(blk.thread_group(), cfg.prb())) {}
 
-        bool get_dpas_2x_depth(
-                const blocking_t &blk, const conv_config_t &cfg) const {
+        context_t(const blocking_t &blk, const conv_config_t &cfg,
+                const pvar_tile_t &iter, const pvar_tile_t &loop,
+                const pvar_tile_t &tg)
+            : blk(blk)
+            , b_iter(iter.get(pvars::b, 1))
+            , m_iter(iter.get(pvars::m, 1))
+            , n_iter(iter.get(pvars::n, 1))
+            , k_iter(iter.get(pvars::k, 1))
+            , k_loop(loop.get(pvars::k, 1))
+            , b_tg(tg.get(pvars::b, 1))
+            , m_tg(tg.get(pvars::m, 1))
+            , n_tg(tg.get(pvars::n, 1))
+            , k_tg(tg.get(pvars::k, 1))
+            , dpas_2x_depth(get_dpas_2x_depth(blk, cfg, m_iter * n_iter)) {}
+
+        static bool get_dpas_2x_depth(
+                const blocking_t &blk, const conv_config_t &cfg, dim_t mn) {
             if (!cfg.is_dp_fma() || cfg.regs() <= 128) return false;
 
             // Use 2x reduction when the reduction dimension is dense to avoid
@@ -602,7 +629,6 @@ private:
                     if (is_inner_non_blocked(cfg, d)) return true;
 
             // Use larger reduction when M/N are small.
-            dim_t mn = m_iter * n_iter;
             if (mn <= 128) return true;
 
             return false;
@@ -672,11 +698,25 @@ private:
 
         int vec_ndims = 0;
         for (auto &d : ctx.blk.iter()) {
-            if (is_vectorized_dim(d, cfg_.prb())) vec_ndims++;
+            if (is_vectorized_dim(d, cfg_.prb(), ctx.blk.iter())) vec_ndims++;
         }
         return vec_ndims == 1;
     }
 
+    bool check_fp4_ok(const context_t &ctx) const {
+        auto prb = cfg_.prb();
+        if (prb.is_fp4_conv()) {
+            for (auto &d : ctx.blk.iter())
+                for (auto t : input_tensors(cfg_.prb())) {
+                    if (tensor_conv_dim_index(d, t) == -1) continue;
+                    if (inner_stride(cfg_, t, d) == 1
+                            && inner_block(cfg_, d) % 8)
+                        return false;
+                }
+        }
+        return true;
+    }
+
     bool check_tg_size_ok(const context_t &ctx) const {
         if (!is_enabled(check_kind_t::check_tg_size)) return true;
 
@@ -753,7 +793,7 @@ private:
                 if ((prb.iw / prb.sw) % iw_tg != 0) return false;
                 return true;
             }
-            default: ir_error_not_expected();
+            default: gpu_error_not_expected();
         }
         return false;
     }
@@ -947,21 +987,16 @@ namespace conv_schemes {
 //   #dim - remove minimum block restriction (minimum is 1)
 conv_blocking_scheme_t fwd_T_wo_I_noi("ls:[ic,kd,kh,kw],T:[oc,ow],i:[mb,oc,ic]");
 conv_blocking_scheme_t fwd_T_no_I_noi("ls:[ic,kd,kh,kw],T:[oc,mb],i:[mb,oc,ic]");
-conv_blocking_scheme_t fwd_T_wn_I_wnoi("ls:[ic,kd,kh,kw],T:[ow,mb],i:[ow,mb,oc,ic]");
 conv_blocking_scheme_t fwd_T_i_I_noi("ls:[ic,kd,kh,kw],T:[ic],i:[mb,oc,ic]");
-conv_blocking_scheme_t fwd_T_iw_I_wnoi("ls:[ic,kd,kh,kw],T:[ic,ow],i:[ow,mb,oc,ic]");
 conv_blocking_scheme_t fwd_T_wo_I_woi("ls:[ic,kd,kh,kw],T:[oc,ow],i:[ow,oc,ic]");
 conv_blocking_scheme_t fwd_T_i_I_woi("ls:[ic,kd,kh,kw],T:[ic],i:[ow,oc,ic]");
 conv_blocking_scheme_t fwd_T_wo_I_woki("ls:[ic,kd,kh,kw],T:[oc,ow],i:[ow,oc,kw,ic]");
-conv_blocking_scheme_t fwd_T_w_I_woki("ls:[ic,kd,kh,kw],T:[ow],i:[ow,oc,kw,ic]");
-conv_blocking_scheme_t fwd_T_w_I_noki("ls:[ic,kd,kh,kw],T:[ow],i:[mb,ow,oc,kw,ic]");
 conv_blocking_scheme_t fwd_T_wo_I_noki("ls:[ic,kd,kh,kw],T:[oc,ow],i:[mb,oc,kw,ic]");
 conv_blocking_scheme_t fwd_dw_T_w_I_wgk("ls:[kd,kh,kw],T:[ow],i:[ow,g,#kw]");
 conv_blocking_scheme_t fwd_dw_T_w_I_ngk("ls:[kd,kh,kw],T:[ow],i:[mb,g,#kw]");
 conv_blocking_scheme_t bwd_d_T_wi_I_nio("ls:[oc,kd,kh,kw],T:[ic,iw],i:[mb,ic,oc]");
 conv_blocking_scheme_t bwd_d_T_ni_I_nio("ls:[oc,kd,kh,kw],T:[ic,mb],i:[mb,ic,oc]");
 conv_blocking_scheme_t bwd_d_T_o_I_nio("ls:[oc,kd,kh,kw],T:[oc],i:[mb,ic,oc]");
-conv_blocking_scheme_t bwd_d_T_w_I_on("ls:[oc,kd,kh,kw],T:[iw],i:[oc,mb]");
 conv_blocking_scheme_t bwd_d_T_wi_I_wio("ls:[oc,kd,kh,kw],T:[ic,iw],i:[iw,ic,oc]");
 conv_blocking_scheme_t bwd_d_T_o_I_wio("ls:[oc,kd,kh,kw],T:[oc],i:[iw,ic,oc]");
 conv_blocking_scheme_t bwd_d_dw_T_w_I_wg("ls:[kd,kh,kw],T:[iw],i:[iw,g]");
@@ -989,7 +1024,7 @@ double get_iter_dim_score(
     } else if (dim == pvars::mb) {
         return dim_size;
     } else {
-        ir_error_not_expected() << "Unknown dimension: " << dim;
+        gpu_error_not_expected() << "Unknown dimension: " << dim;
     }
     return 0;
 }
@@ -1015,7 +1050,7 @@ pvar_t select_iter_dim(
         if (is_bwd_d_w_opt && d == pvars::iw) continue;
         dims.push_back(d);
     }
-    ir_assert(!dims.empty());
+    gpu_assert(!dims.empty());
     if (dims.size() == 1) return dims[0];
 
     std::vector<int> dim_blocks;
@@ -1028,7 +1063,7 @@ pvar_t select_iter_dim(
     for (int i = 0; i < (int)dims.size(); i++) {
         if (dim_blocks[i] == max_block) return dims[i];
     }
-    ir_error_not_expected();
+    gpu_error_not_expected();
     return *dims.begin();
 }
 
@@ -1074,45 +1109,33 @@ conv_blocking_scheme_list_t get_blocking_schemes_bwd_w_dw(
 
 conv_blocking_scheme_list_t get_blocking_schemes_fwd(const conv_config_t &cfg) {
     conv_blocking_scheme_list_t ret(conv_tune_level());
-    auto m_iter_dim = cfg.prb().ab_swap_transpose
-            ? pvars::oc
-            : select_iter_dim(cfg, {pvars::mb, pvars::ow});
-    bool m_is_mb = (m_iter_dim == pvars::mb);
-    bool m_is_ow = (m_iter_dim == pvars::ow);
-    bool m_is_oc = (m_iter_dim == pvars::oc);
+    auto m_iter_dim = select_iter_dim(cfg, {pvars::mb, pvars::ow});
+    bool mb_iter = (m_iter_dim == pvars::mb);
+    bool ow_iter = (m_iter_dim == pvars::ow);
     bool ge_xelp = (cfg.hw() >= ngen::HW::XeLP);
     bool small_ic = (is_small_ic(cfg.prb()) && cfg.prb().kw > 1);
-    ret.add(m_is_mb, conv_schemes::fwd_T_wo_I_noi);
-    ret.add(m_is_mb, conv_schemes::fwd_T_no_I_noi);
-    ret.add(m_is_mb && ge_xelp, conv_schemes::fwd_T_i_I_noi);
-    ret.add(m_is_oc, conv_schemes::fwd_T_wn_I_wnoi);
-    ret.add(m_is_oc && ge_xelp, conv_schemes::fwd_T_i_I_noi);
-    ret.add(m_is_oc && ge_xelp, conv_schemes::fwd_T_iw_I_wnoi);
-    ret.add(m_is_ow, conv_schemes::fwd_T_wo_I_woi);
-    ret.add(m_is_ow && ge_xelp, conv_schemes::fwd_T_i_I_woi);
-    ret.add(m_is_mb && small_ic, conv_schemes::fwd_T_wo_I_noki);
-    ret.add(m_is_oc && small_ic, conv_schemes::fwd_T_w_I_woki);
-    ret.add(m_is_oc && small_ic, conv_schemes::fwd_T_w_I_noki);
-    ret.add(m_is_ow && small_ic, conv_schemes::fwd_T_wo_I_woki);
+    ret.add(mb_iter, conv_schemes::fwd_T_wo_I_noi);
+    ret.add(mb_iter, conv_schemes::fwd_T_no_I_noi);
+    ret.add(mb_iter && ge_xelp, conv_schemes::fwd_T_i_I_noi);
+    ret.add(ow_iter, conv_schemes::fwd_T_wo_I_woi);
+    ret.add(ow_iter && ge_xelp, conv_schemes::fwd_T_i_I_woi);
+    ret.add(mb_iter && small_ic, conv_schemes::fwd_T_wo_I_noki);
+    ret.add(ow_iter && small_ic, conv_schemes::fwd_T_wo_I_woki);
     return ret;
 }
 
 conv_blocking_scheme_list_t get_blocking_schemes_bwd_d(
         const conv_config_t &cfg) {
     conv_blocking_scheme_list_t ret(conv_tune_level());
-    auto m_iter_dim = cfg.prb().ab_swap_transpose
-            ? pvars::ic
-            : select_iter_dim(cfg, {pvars::mb, pvars::iw});
-    bool m_is_mb = (m_iter_dim == pvars::mb);
-    bool m_is_iw = (m_iter_dim == pvars::iw);
-    bool m_is_ic = (m_iter_dim == pvars::ic);
+    auto m_iter_dim = select_iter_dim(cfg, {pvars::mb, pvars::iw});
+    bool mb_iter = (m_iter_dim == pvars::mb);
+    bool iw_iter = (m_iter_dim == pvars::iw);
     bool ge_xelp = (cfg.hw() >= ngen::HW::XeLP);
-    ret.add(m_is_mb, conv_schemes::bwd_d_T_ni_I_nio);
-    ret.add(m_is_mb, conv_schemes::bwd_d_T_wi_I_nio);
-    ret.add(m_is_mb && ge_xelp, conv_schemes::bwd_d_T_o_I_nio);
-    ret.add(m_is_iw, conv_schemes::bwd_d_T_wi_I_wio);
-    ret.add(m_is_iw && ge_xelp, conv_schemes::bwd_d_T_o_I_wio);
-    ret.add(m_is_ic, conv_schemes::bwd_d_T_w_I_on);
+    ret.add(mb_iter, conv_schemes::bwd_d_T_ni_I_nio);
+    ret.add(mb_iter, conv_schemes::bwd_d_T_wi_I_nio);
+    ret.add(mb_iter && ge_xelp, conv_schemes::bwd_d_T_o_I_nio);
+    ret.add(iw_iter, conv_schemes::bwd_d_T_wi_I_wio);
+    ret.add(iw_iter && ge_xelp, conv_schemes::bwd_d_T_o_I_wio);
     return ret;
 }
 
@@ -1140,7 +1163,7 @@ conv_blocking_scheme_list_t get_blocking_schemes_dw_impl(
     if (prb.is_fwd) return get_blocking_schemes_fwd_dw(cfg);
     if (prb.is_bwd_d) return get_blocking_schemes_bwd_d_dw(cfg);
     if (prb.is_bwd_w) return get_blocking_schemes_bwd_w_dw(cfg);
-    ir_error_not_expected();
+    gpu_error_not_expected();
     return conv_blocking_scheme_list_t();
 }
 
@@ -1151,7 +1174,7 @@ conv_blocking_scheme_list_t get_blocking_schemes_impl(
     if (prb.is_fwd) return get_blocking_schemes_fwd(cfg);
     if (prb.is_bwd_d) return get_blocking_schemes_bwd_d(cfg);
     if (prb.is_bwd_w) return get_blocking_schemes_bwd_w(cfg);
-    ir_error_not_expected();
+    gpu_error_not_expected();
     return conv_blocking_scheme_list_t();
 }
 
@@ -1207,7 +1230,7 @@ void sort_by_model_scores(params_generator_t &params_gen,
         table << p.str() << (int)(score * 1000) / 1000.0 << eff << regs
               << slm_size << std::endl;
     }
-    ir_trace() << table.str() << std::endl;
+    gpu_trace() << table.str();
 #endif
     MAYBE_UNUSED(&slm_usage_bytes_for_params);
 }
@@ -1383,9 +1406,8 @@ private:
         const int nbest = 5;
         auto best_ids = tune_data_.best_ids(nbest);
         std::unordered_map<int, float> dists;
-        ir_trace() << "[Tuning] Rescoring: " << (end - beg) << " configs left"
-                   << std::endl;
-        ir_trace() << "  Best config: " << best_params_dbg_ << std::endl;
+        gpu_trace() << "[Tuning] Rescoring: " << (end - beg) << " configs left";
+        gpu_trace() << "  Best config: " << best_params_dbg_;
         for (int i = beg; i < end; i++) {
             auto &p = params_gen_.at(i);
             dists[p.id()] = std::numeric_limits<float>::max();
@@ -1399,8 +1421,7 @@ private:
 
         for (int i = beg; i < end; i++) {
             auto &p = params_gen_.at(i);
-            ir_trace() << "  " << p << " [dist:" << dists[p.id()] << "]"
-                       << std::endl;
+            gpu_trace() << "  " << p << " [dist:" << dists[p.id()] << "]";
         }
     }
 
@@ -1486,7 +1507,7 @@ public:
     }
 
     void set_cur_version(int32_t version) {
-        ir_assert(!is_tuning_mode());
+        gpu_assert(!is_tuning_mode());
         int idx;
         unpack_version(version, idx, grf_mode_policy_);
         params_gen_.set_cur_index(idx);
@@ -1565,8 +1586,7 @@ private:
             case tiler_mode_t::lookup: {
                 const auto params = const_conv_lookup_table().find(cfg.key());
                 if (!params.is_empty() && chk.is_ok(params.blocking())) {
-                    ir_info() << "[INFO] Using lookup table config: "
-                              << params.str() << std::endl;
+                    gpu_info() << "Using lookup table config: " << params.str();
                     params_gen_ = params_generator_t(tune_level, simd_size, chk,
                             level_tile_sets, params);
                 } else {
@@ -1591,7 +1611,7 @@ private:
                 break;
         }
         if (!is_tuning_mode()) {
-            ir_assert(!params_gen_.is_empty()) << "No configurations found.";
+            gpu_assert(!params_gen_.is_empty()) << "No configurations found.";
             sort_by_model_scores(params_gen_, cfg, mode_);
         }
         if (tiler_params().do_list) print_all();
@@ -1616,10 +1636,10 @@ private:
     }
 
     void print_info(double init_time_ms) {
-        ir_info() << "Convolution tiler:" << std::endl;
-        ir_info() << "  Mode:              " << to_string(mode_) << std::endl;
-        ir_info() << "  Filtered configs:  " << configs() << std::endl;
-        ir_info() << "  Init time (ms):    " << init_time_ms << std::endl;
+        gpu_info() << "Convolution tiler:";
+        gpu_info() << "  Mode:              " << to_string(mode_);
+        gpu_info() << "  Filtered configs:  " << configs();
+        gpu_info() << "  Init time (ms):    " << init_time_ms;
     }
 
     tiler_mode_t mode_ = tiler_mode_t::undef;
