@@ -57,7 +57,7 @@ struct gen_gemm_t : public gpu_gemm_t {
 
             // LIMITATIONS:
             // - runtime dims are not supported
-            auto attr_skip_mask = smask_t::scales_runtime | smask_t::post_ops
+            auto attr_skip_mask = smask_t::scales | smask_t::post_ops
                     | smask_t::fpmath_mode | smask_t::accumulation_mode
                     | smask_t::rounding_mode;
             auto &attr_zps = attr()->zero_points_;
@@ -73,15 +73,24 @@ struct gen_gemm_t : public gpu_gemm_t {
                             d->c_type(), f8_e5m2, f8_e4m3, f16, bf16, f32));
             wei_decomp_ = (utils::one_of(d->c_type(), f32, f16, bf16, f8_e5m2,
                                    f8_e4m3)
-                                  && utils::one_of(d->a_type(), u8, s8, s4, u4)
+                                  && utils::one_of(d->a_type(), u8, s8, s4, u4,
+                                          f8_e5m2, f8_e4m3)
                                   && utils::one_of(d->b_type(), f16, f32, bf16,
                                           f8_e5m2, f8_e4m3))
+                    && types::data_type_bits(d->a_type())
+                            < types::data_type_bits(d->b_type())
                     && attr()->mayiconvert(d->a_type(), f32);
             dy_quant_enabled_
                     = (utils::one_of(d->c_type(), f32, f16, bf16)
                               && utils::one_of(d->a_type(), u8, s8, s4, u4)
                               && utils::one_of(d->b_type(), u8, s8))
                     || all_f8;
+
+            VDISPATCH_GEMM(
+                    IMPLICATION(utils::one_of(d->a_type(), f8_e5m2, f8_e4m3),
+                            arch_ >= arch_t::xe_hpc),
+                    VERBOSE_UNSUPPORTED_DT); /* temporary; pending gemmstone pulldown */
+
             quant_enabled_ = wei_decomp_ || dy_quant_enabled_;
             CHECK(set_default_formats(false));
 
@@ -125,17 +134,15 @@ struct gen_gemm_t : public gpu_gemm_t {
 
             if (quant_enabled_) {
                 attr_skip_mask |= smask_t::fpmath_mode
-                        | smask_t::scales_runtime_data_type
-                        | smask_t::scales_runtime_groups
-                        | smask_t::zero_points_runtime_data_type
-                        | smask_t::zero_points_runtime_groups;
+                        | smask_t::scales_data_type | smask_t::scales_groups
+                        | smask_t::zero_points_data_type
+                        | smask_t::zero_points_groups;
             }
 
             const int mask_scalar = 1 << 0;
             const int mask_per_oc = 1 << 1;
             const int mask_per_ic = 1 << 2;
 
-            bool src_zp_2d = false;
             bool wei_zp_2d = false;
             auto wei_scales_type = data_type::undef;
             auto src_scales_type = data_type::undef;
@@ -145,33 +152,37 @@ struct gen_gemm_t : public gpu_gemm_t {
             int b_ndims = desc()->b_desc.ndims;
 
             // Check parameters.
-            if (utils::one_of(d->c_type(), s32, f16, f32, u8, s8)
+            if (utils::one_of(d->c_type(), s32, f16, bf16, f32, u8, s8)
                     && utils::one_of(d->a_type(), u8, s8, u4, s4)) {
                 VDISPATCH_GEMM(
                         (utils::one_of(d->b_type(), u8, s8) || wei_decomp_),
                         VERBOSE_UNSUPPORTED_DT);
-                attr_skip_mask |= smask_t::zero_points_runtime;
+                attr_skip_mask |= smask_t::zero_points;
 
                 VDISPATCH_GEMM(IMPLICATION(utils::one_of(d->c_type(), f32, s8,
-                                                   u8, f16),
+                                                   u8, f16, bf16),
                                        arch_ >= arch_t::xe_hp),
                         VERBOSE_ISA_DT_MISMATCH);
-            } else if (d->a_type() == bf16) {
-                VDISPATCH_GEMM(
-                        d->b_type() == bf16, VERBOSE_INCONSISTENT_DT, "a", "b");
-                VDISPATCH_GEMM(
-                        utils::one_of(d->c_type(), bf16, f32, f8_e5m2, f8_e4m3),
+            } else if (utils::one_of(d->a_type(), f16, bf16)) {
+                VDISPATCH_GEMM(d->b_type() == d->a_type(),
+                        VERBOSE_INCONSISTENT_DT, "a", "b");
+                VDISPATCH_GEMM(utils::one_of(d->c_type(), d->a_type(), f32,
+                                       f8_e5m2, f8_e4m3),
                         VERBOSE_INCONSISTENT_DT, "a", "c");
-                VDISPATCH_GEMM(utils::one_of(d->acc_type, bf16, f32),
+                VDISPATCH_GEMM(utils::one_of(d->acc_type, d->a_type(), f32),
                         VERBOSE_INCONSISTENT_DT, "a", "acc");
             } else if (!wei_decomp_) {
-                VDISPATCH_GEMM(utils::one_of(d->a_type(), f64, f32, f16,
-                                       f8_e5m2, f8_e4m3),
+                VDISPATCH_GEMM(utils::one_of(d->a_type(), f64, f32, f16, bf16,
+                                       f8_e5m2, f8_e4m3, f4_e2m1, f4_e3m0),
                         VERBOSE_UNSUPPORTED_DT);
-                VDISPATCH_GEMM(d->b_type() == d->a_type()
+                VDISPATCH_GEMM(
+                        (d->b_type() == d->a_type()
                                 || (utils::one_of(d->a_type(), f8_e5m2, f8_e4m3)
                                         && utils::one_of(
-                                                d->b_type(), f8_e5m2, f8_e4m3)),
+                                                d->b_type(), f8_e5m2, f8_e4m3))
+                                || (utils::one_of(d->a_type(), f4_e2m1, f4_e3m0)
+                                        && utils::one_of(d->b_type(), f4_e2m1,
+                                                f4_e3m0))),
                         VERBOSE_INCONSISTENT_DT, "a", "b");
                 VDISPATCH_GEMM(utils::one_of(d->acc_type, d->a_type(), f32),
                         VERBOSE_UNSUPPORTED_DT);
@@ -219,99 +230,110 @@ struct gen_gemm_t : public gpu_gemm_t {
                 return utils::one_of(mask, (1 << (ndims - 1)),
                         (1 << (ndims - 1)) + (1 << (ndims - 2)));
             };
-
-            if (!attr()->zero_points_.has_default_values()) {
-                bool a_zp = !attr_zps.has_default_values(DNNL_ARG_A);
-                bool b_zp = !attr_zps.has_default_values(DNNL_ARG_B);
-
-                int cmask_a = 0, cmask_b = 0, cmask_c = 0;
-                CHECK(attr_zps.get(DNNL_ARG_A, &cmask_a));
-                CHECK(attr_zps.get(DNNL_ARG_B, &cmask_b));
-                CHECK(attr_zps.get(DNNL_ARG_C, &cmask_c));
-
-                src_zp_2d = attr_zps.get_groups_ndims(DNNL_ARG_B) > 1;
-                wei_zp_2d = attr_zps.get_groups_ndims(DNNL_ARG_A) > 1;
-                VDISPATCH_GEMM(
-                        (utils::one_of(cmask_a, 0, mask_per_oc, mask_per_ic)
-                                || (wei_zp_2d
-                                        && valid_2d_mask(cmask_a, a_ndims)))
-                                && utils::one_of(
-                                        cmask_c, 0, mask_scalar, mask_per_oc),
-                        VERBOSE_UNSUPPORTED_ZP_CFG);
-                VDISPATCH_GEMM(utils::one_of(cmask_b, 0, mask_scalar,
-                                       mask_per_oc | mask_per_ic)
-                                || (src_zp_2d
-                                        && valid_2d_mask(cmask_b, b_ndims)),
-                        VERBOSE_UNSUPPORTED_ZP_CFG);
-
-                ao_dims_ = a_zp ? (cmask_a != 0 ? 1 : 0) : -1;
-                bo_dims_ = b_zp ? (cmask_b != 0 ? 1 : 0) : -1;
-                if (wei_zp_2d) ao_dims_ = 2;
-                if (src_zp_2d) bo_dims_ = 2;
-                if (swap_ab_) std::swap(ao_dims_, bo_dims_);
-
-                if (wei_zp_2d) {
-                    const auto idx = DNNL_ARG_WEIGHTS;
-                    auto zp_group_k = attr_zps.get_groups_ndims(idx) > 0
-                            ? attr_zps.get_groups(idx)[0]
-                            : 1;
-                    if (zp_group_k >= d->k()) {
-                        wei_zp_2d = false;
-                        ao_dims_ = 1;
-                    } else {
-                        wei_q2d_group_k = zp_group_k;
-                    }
-                    const auto wei_q2d_group_n
-                            = attr_zps.get_groups_ndims(idx) > 0
-                            ? attr_zps.get_groups(idx)[1]
-                            : 1;
-                    // Non-trivial N group unsupported.
-                    VDISPATCH_GEMM(
-                            wei_q2d_group_n == 1, VERBOSE_UNSUPPORTED_ZP_CFG);
-                }
-                if (src_zp_2d) {
-                    const auto idx = DNNL_ARG_SRC;
-                    auto zp_group_k = attr_zps.get_groups_ndims(idx) > 0
-                            ? attr_zps.get_groups(idx)[1]
-                            : 1;
-                    if (zp_group_k >= d->k()) {
-                        src_zp_2d = false;
-                        bo_dims_ = 1;
-                    } else {
-                        src_q2d_group_k = zp_group_k;
-                    }
-                    const auto src_q2d_group_m
-                            = attr_zps.get_groups_ndims(idx) > 0
-                            ? attr_zps.get_groups(idx)[0]
-                            : 1;
-                    // Non-trivial M group unsupported.
-                    VDISPATCH_GEMM(
-                            src_q2d_group_m == 1, VERBOSE_UNSUPPORTED_ZP_CFG);
-                }
-                // Zero points with non-trivial groups only supported when
-                // target tensor is being dequantized.
-                VDISPATCH_GEMM(
-                        IMPLICATION(wei_zp_2d,
-                                !dy_quant_enabled_
-                                        || utils::one_of(d->a_type(), s4, u4)
-                                        || wei_q2d_group_k == desc()->k()),
-                        VERBOSE_UNSUPPORTED_ZP_CFG);
-                VDISPATCH_GEMM(
-                        IMPLICATION(src_zp_2d,
-                                !dy_quant_enabled_
-                                        || utils::one_of(d->b_type(), s4, u4)
-                                        || src_q2d_group_k == desc()->k()),
-                        VERBOSE_UNSUPPORTED_ZP_CFG);
+            auto c_kernel_type
+                    = convert_dnnl_to_kernel_type(desc_.c_desc.data_type);
+            for (int i = 0; i < desc_.c_desc.ndims; i++) {
+                auto c_stride = desc_.c_desc.format_desc.blocking.strides[i];
+                VDISPATCH_GEMM(IMPLICATION((c_kernel_type.isInt4()
+                                                   || c_kernel_type.isFP4()),
+                                       c_stride == 1 || c_stride % 2 == 0),
+                        VERBOSE_SHAPE_RESTRICTION);
             }
 
             auto &wei_scales = attr()->scales_.get(DNNL_ARG_WEIGHTS);
             auto &src_scales = attr()->scales_.get(DNNL_ARG_SRC);
 
-            if (quant_enabled_ && wei_scales.ndims_ > 1) wei_scales_2d_ = true;
-            if (quant_enabled_ && src_scales.ndims_ > 1) src_scales_2d_ = true;
+            if (quant_enabled_ && !wei_scales.has_default_groups())
+                wei_scales_2d_ = true;
+            if (quant_enabled_ && !src_scales.has_default_groups())
+                src_scales_2d_ = true;
+
+            if (!attr()->zero_points_.has_default_values()) {
+                if (!attr_zps.has_default_values(DNNL_ARG_A)) {
+                    const int cmask_a = attr_zps.get_mask(DNNL_ARG_A);
+                    ao_dims_ = cmask_a > 0;
+
+                    // Groups determine supported masks.
+                    if (!attr_zps.has_default_groups(DNNL_ARG_A)) {
+                        VDISPATCH_GEMM(valid_2d_mask(cmask_a, a_ndims),
+                                VERBOSE_UNSUPPORTED_ZP_CFG);
+
+                        const auto idx = DNNL_ARG_WEIGHTS;
+                        auto zp_group_k = attr_zps.get_group(idx, 0);
+                        if (zp_group_k < d->k()) {
+                            wei_zp_2d = true;
+                            ao_dims_ = 2;
+                            wei_q2d_group_k = zp_group_k;
+                        }
+                        const auto wei_q2d_group_n = attr_zps.get_group(idx, 1);
+                        // Non-trivial N group unsupported.
+                        VDISPATCH_GEMM(wei_q2d_group_n == 1,
+                                VERBOSE_UNSUPPORTED_ZP_CFG);
+                        // Zero points with non-trivial groups only supported
+                        // when target tensor is being dequantized.
+                        VDISPATCH_GEMM(!dy_quant_enabled_
+                                        || utils::one_of(d->a_type(), s4, u4)
+                                        || zp_group_k == desc()->k(),
+                                VERBOSE_UNSUPPORTED_ZP_CFG);
+                    } else {
+                        VDISPATCH_GEMM(utils::one_of(cmask_a, 0, mask_per_oc,
+                                               mask_per_ic),
+                                VERBOSE_UNSUPPORTED_ZP_CFG);
+                        // Weights zp can only be performantly enabled during upconversion
+                        // for cases that perform decompression.
+                        VDISPATCH_GEMM(wei_decomp_
+                                        || utils::one_of(d->a_type(), s4, u4)
+                                        || !wei_scales_2d_,
+                                VERBOSE_UNSUPPORTED_ZP_CFG);
+                    }
+                }
+
+                if (!attr_zps.has_default_values(DNNL_ARG_B)) {
+                    const int cmask_b = attr_zps.get_mask(DNNL_ARG_B);
+                    bo_dims_ = cmask_b > 0;
+
+                    // Groups determine supported masks.
+                    if (!attr_zps.has_default_groups(DNNL_ARG_B)) {
+                        VDISPATCH_GEMM(valid_2d_mask(cmask_b, b_ndims),
+                                VERBOSE_UNSUPPORTED_ZP_CFG);
+
+                        const auto idx = DNNL_ARG_SRC;
+                        auto zp_group_k = attr_zps.get_group(idx, 1);
+                        if (zp_group_k < d->k()) {
+                            bo_dims_ = 2;
+                            src_q2d_group_k = zp_group_k;
+                        }
+                        const auto src_q2d_group_m = attr_zps.get_group(idx, 0);
+                        // Non-trivial M group unsupported.
+                        VDISPATCH_GEMM(src_q2d_group_m == 1,
+                                VERBOSE_UNSUPPORTED_ZP_CFG);
+                        // Zero points with non-trivial groups only supported
+                        // when target tensor is being dequantized.
+                        VDISPATCH_GEMM(!dy_quant_enabled_
+                                        || utils::one_of(d->b_type(), s4, u4)
+                                        || zp_group_k == desc()->k(),
+                                VERBOSE_UNSUPPORTED_ZP_CFG);
+                    } else {
+                        VDISPATCH_GEMM(utils::one_of(cmask_b, 0, mask_scalar,
+                                               mask_per_oc | mask_per_ic),
+                                VERBOSE_UNSUPPORTED_ZP_CFG);
+                    }
+                }
+
+                if (!attr_zps.has_default_values(DNNL_ARG_C)) {
+                    const int cmask_c = attr_zps.get_mask(DNNL_ARG_C);
+                    VDISPATCH_GEMM(
+                            utils::one_of(cmask_c, 0, mask_scalar, mask_per_oc),
+                            VERBOSE_UNSUPPORTED_ZP_CFG);
+                }
+
+                if (swap_ab_) std::swap(ao_dims_, bo_dims_);
+            }
 
             for (auto s : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST}) {
-                auto mask = attr()->scales_.get(s).mask_;
+                if (attr()->scales_.has_default_values(s)) continue;
+
+                auto mask = attr()->scales_.get_mask(s);
                 VDISPATCH_GEMM(utils::one_of(mask, 0, mask_scalar, mask_per_oc,
                                        mask_per_ic)
                                 || (s == DNNL_ARG_WEIGHTS && wei_scales_2d_
@@ -322,12 +344,11 @@ struct gen_gemm_t : public gpu_gemm_t {
             }
 
             if (wei_scales_2d_) {
-                auto scales_group_k
-                        = wei_scales.ndims_ > 0 ? wei_scales.group_dims_[0] : 1;
+                auto scales_group_k = wei_scales.get_group(0);
                 if (scales_group_k >= d->k()) {
                     wei_scales_2d_ = false;
                 } else {
-                    wei_scales_type = wei_scales.data_type_;
+                    wei_scales_type = wei_scales.get_data_type();
                     if (!wei_zp_2d)
                         wei_q2d_group_k = scales_group_k;
                     else {
@@ -336,14 +357,13 @@ struct gen_gemm_t : public gpu_gemm_t {
                     }
                 }
                 // Non-trivial N group unsupported.
-                VDISPATCH_GEMM(wei_scales.group_dims_[1] == 1,
+                VDISPATCH_GEMM(wei_scales.get_group(1) == 1,
                         VERBOSE_UNSUPPORTED_SCALES_CFG);
             }
             if (src_scales_2d_) {
-                src_scales_type = src_scales.data_type_;
-                src_po_sc_ = src_scales.mask_ == 2;
-                auto scales_group_k
-                        = src_scales.ndims_ > 0 ? src_scales.group_dims_[1] : 1;
+                src_scales_type = src_scales.get_data_type();
+                src_po_sc_ = src_scales.get_mask() == 2;
+                auto scales_group_k = src_scales.get_group(1);
                 if (scales_group_k >= d->k())
                     src_scales_2d_ = false;
                 else {
@@ -394,6 +414,7 @@ struct gen_gemm_t : public gpu_gemm_t {
                     : data_type::s32;
             if (swap_ab_) std::swap(ao_type, bo_type);
             bool int_acc = utils::one_of(eff_a_type(), s8, u8);
+            int_acc &= !wei_scales_2d_;
             auto co_type = with_bias() ? d->bias_type()
                     : with_sum_ab()    ? d->sum_ab_type
                     : int_acc          ? s32
@@ -520,6 +541,7 @@ struct gen_gemm_t : public gpu_gemm_t {
             auto c_t = d->c_type();
 
             bool is_f16 = utils::everyone_is(f16, a_t, b_t, c_t);
+            bool is_bf16 = utils::everyone_is(bf16, a_t, b_t, c_t);
             bool is_xe_hp_plus = arch_ >= arch_t::xe_hp;
 
             // Rename memory descriptors following column major format.
@@ -570,7 +592,23 @@ struct gen_gemm_t : public gpu_gemm_t {
             auto dotrans = batch ? acb : ba;
             auto notrans = batch ? abc : ab;
 
-            if (is_f16 && is_xe_hp_plus && use_tn) {
+            auto cache_line_align_md = [&](memory_desc_t &md) {
+                auto dim = md.dims[md.ndims - 1];
+                dnnl::impl::dims_t dims;
+                dnnl::impl::utils::array_copy(dims, md.dims, md.ndims);
+                auto kernel_type = convert_dnnl_to_kernel_type(md.data_type);
+                dim_t cache_line_elems = 64 / kernel_type;
+                md.dims[md.ndims - 1] = utils::rnd_up(dim,
+                        std::min((dim_t)cache_line_elems,
+                                utils::rnd_up(dim, 2)));
+                CHECK(memory_desc_init_by_strides(md, nullptr));
+                dnnl::impl::utils::array_copy(md.dims, dims, md.ndims);
+                return status::success;
+            };
+            if (a_any) CHECK(cache_line_align_md(a_desc));
+            if (b_any) CHECK(cache_line_align_md(b_desc));
+
+            if ((is_f16 || is_bf16) && is_xe_hp_plus && use_tn) {
                 if (a_any && b_any) {
                     CHECK(memory_desc_init_by_tag(a_desc, dotrans));
                     CHECK(memory_desc_init_by_tag(b_desc, notrans));
@@ -679,20 +717,22 @@ struct gen_gemm_t : public gpu_gemm_t {
             auto dt = eff_a_type();
             auto align = utils::max_pow2_div(
                     types::elements_to_bytes(dt, eff_lda()));
-            for (int b = 0; b < batch_dims(); b++)
-                align = nstl::min(align,
-                        utils::max_pow2_div(
-                                types::elements_to_bytes(dt, eff_stride_a(b))));
+            for (int b = 0; b < batch_dims(); b++) {
+                auto stride_bytes = utils::max_pow2_div(
+                        types::elements_to_bytes(dt, eff_stride_a(b)));
+                align = (stride_bytes ? nstl::min(align, stride_bytes) : align);
+            }
             return int(align);
         }
         int eff_align_b() const {
             auto dt = eff_b_type();
             auto align = utils::max_pow2_div(
                     types::elements_to_bytes(dt, eff_ldb()));
-            for (int b = 0; b < batch_dims(); b++)
-                align = nstl::min(align,
-                        utils::max_pow2_div(
-                                types::elements_to_bytes(dt, eff_stride_b(b))));
+            for (int b = 0; b < batch_dims(); b++) {
+                auto stride_bytes = utils::max_pow2_div(
+                        types::elements_to_bytes(dt, eff_stride_b(b)));
+                align = (stride_bytes ? nstl::min(align, stride_bytes) : align);
+            }
             return int(align);
         }
         int align_c() const {
@@ -747,7 +787,7 @@ struct gen_gemm_t : public gpu_gemm_t {
 
     gen_gemm_t(const pd_t *apd) : gpu_gemm_t(apd) {}
 
-    ~gen_gemm_t() {
+    ~gen_gemm_t() override {
         if (zero_pool_) release_zero_pool(zero_pool_);
     }
 
@@ -765,7 +805,7 @@ struct gen_gemm_t : public gpu_gemm_t {
         scalar_type_ = kd->scalar_type();
         const auto *info = nocopy_info();
 
-        if (info->fusedBeta() || info->fusedPostOps()) {
+        if (need_zero_pool()) {
             int zg_cl = 0;
             if (info->fusedBeta()) zg_cl++;
             if (info->fusedPostOps()) zg_cl++;
@@ -775,12 +815,10 @@ struct gen_gemm_t : public gpu_gemm_t {
             auto zg_max = pd()->dev_info_->hw_threads(false);
             zero_pool_chunk_size_ = zg_max * 2 * 2 * 64;
 
-#ifndef DNNL_WITH_SYCL
             auto *compute_engine
                     = utils::downcast<compute::compute_engine_t *>(engine);
             CHECK(lookup_zero_pool(compute_engine, nullptr,
                     zero_pool_chunk_size_, &zero_pool_));
-#endif
 
             nocopy_kernel_.save_output_events();
         }
@@ -799,8 +837,8 @@ private:
             const memory_storage_t *b_scales, const memory_storage_t &co,
             const memory_storage_t *c_temp, const memory_storage_t *sround_seed,
             int po_count, const memory_storage_t **po_src, int64_t offset_a,
-            int64_t offset_b, int64_t offset_c, int32_t offset_aq,
-            int32_t offset_bq, int32_t offset_co, int32_t *offset_po_src,
+            int64_t offset_b, int64_t offset_c, int64_t offset_aq,
+            int64_t offset_bq, int64_t offset_co, int64_t *offset_po_src,
             int32_t lda, int32_t ldb, int32_t ldc, int32_t m, int32_t n,
             int32_t k, int32_t k0, float alpha, float beta, int32_t cmask,
             bool last_k_block, bool swapab, bool disable_hilbert) const;
@@ -810,6 +848,10 @@ private:
         return pd()->kernel_desc()->driver_info();
     }
 
+    bool need_zero_pool() const {
+        return nocopy_info()->fusedBeta() || nocopy_info()->fusedPostOps();
+    }
+
     compute::kernel_t nocopy_kernel_;
     compute::scalar_type_t scalar_type_;
     zero_pool_t *zero_pool_ = nullptr;
