@@ -17,6 +17,7 @@
 #include "gpu/intel/jit/gemm/gen_gemm_kernel.hpp"
 #include "common/impl_registration.hpp"
 #include "gpu/intel/compute/device_info.hpp"
+#include "gpu/intel/jit/gemm/gen_gemm_kernel_db.hpp"
 #include "gpu/intel/jit/gemm/include/generator.hpp"
 #include "gpu/intel/jit/gemm/include/strategy_parser.hpp"
 #include "gpu/intel/jit/utils/ngen_type_bridge.hpp"
@@ -28,11 +29,6 @@ namespace gpu {
 namespace intel {
 namespace jit {
 
-#define _CATALOG_ gemm_catalog
-#include "selector/db/kernel.db"
-;
-#undef _CATALOG_
-
 status_t gen_gemm_kernel_desc_t::create_generator(
         const compute::compute_engine_t &engine,
         compute::kernel_t &kernel) const {
@@ -52,6 +48,8 @@ compute::scalar_type_t gen_gemm_kernel_desc_t::scalar_type() const {
         case Type::u32: return compute::scalar_type_t::_uint;
         case Type::s64: return compute::scalar_type_t::_long;
         case Type::u64: return compute::scalar_type_t::_ulong;
+        case Type::f4_e2m1: return compute::scalar_type_t::_f4_e2m1;
+        case Type::f4_e3m0: return compute::scalar_type_t::_f4_e3m0;
         case Type::bf8: return compute::scalar_type_t::_bfloat8;
         case Type::hf8: return compute::scalar_type_t::_hfloat8;
         case Type::bf16: return compute::scalar_type_t::_bfloat16;
@@ -64,12 +62,14 @@ compute::scalar_type_t gen_gemm_kernel_desc_t::scalar_type() const {
 
 status_t gen_gemm_kernel_desc_t::finalize(const char *tags) {
     // Update problem alignments to match catalog entry.
-    if (!isPacked(problem_.A.layout)) {
+    if (!isPacked(problem_.A.layout)
+            && problem_.Ta_ext.paddedSize() >= problem_.Ta.paddedSize()) {
         problem_.A.setAlignment(std::max(
                 problem_.Ta_ext.paddedSize(), entry_->driverInfo.alignment[0]));
     }
 
-    if (!isPacked(problem_.B.layout)) {
+    if (!isPacked(problem_.B.layout)
+            && problem_.Tb_ext.paddedSize() >= problem_.Tb.paddedSize()) {
         problem_.B.setAlignment(std::max(
                 problem_.Tb_ext.paddedSize(), entry_->driverInfo.alignment[1]));
     }
@@ -79,7 +79,7 @@ status_t gen_gemm_kernel_desc_t::finalize(const char *tags) {
                 entry_->restrictions.alignment[2]));
     }
 
-    problem_.CO.setAlignment(problem_.Tco.size());
+    problem_.CO.setAlignment(problem_.Tco.paddedSize());
 
     // Parse strategy string.
     strategy_ = GEMMStrategy(hw_, stepping_);
@@ -382,8 +382,8 @@ status_t gen_gemm_nocopy_kernel_desc_t::select_kernel(compute::gpu_arch_t arch,
     relaxed_acc_ = mode & mode_relaxed_acc;
 
     auto a_type_size = types::data_type_size(a_type);
-    auto b_type_size = types::data_type_size(a_type);
-    auto c_type_size = types::data_type_size(a_type);
+    auto b_type_size = types::data_type_size(b_type);
+    auto c_type_size = types::data_type_size(c_type);
 
     align_a = nstl::max(align_a, int(a_type_size));
     align_b = nstl::max(align_b, int(b_type_size));
@@ -532,8 +532,6 @@ status_t gen_gemm_nocopy_kernel_desc_t::select_kernel(compute::gpu_arch_t arch,
     bool fpmath_tf32 = mode & mode_tf32;
     bool fpmath_bf16 = mode & mode_bf16x1;
     bool fpmath_f16 = mode & mode_f16x1;
-    bool fpmath_strict = !(fpmath_tf32 || fpmath_bf16 || fpmath_f16)
-            && (mode & mode_strict) && (mode & mode_w_decomp);
 
     auto add_mode_matches = [&](bool has_mode, const char *(*match)(Type)) {
         if (!has_mode) return;
@@ -579,17 +577,10 @@ status_t gen_gemm_nocopy_kernel_desc_t::select_kernel(compute::gpu_arch_t arch,
         return nullptr;
     });
 
-    if (fpmath_strict) {
-        if (problem_.Tb.isInt4() && !(fpmath_f16 || fpmath_bf16)) {
-            match_params.emplace_back(match_params[0]);
-            match_params.back().selector.precisions[1]
-                    = match_params.back().selector.precisions[0];
-        } else {
-            match_params.emplace_back(match_params[0]);
-            match_params.back().selector.precisions[0]
-                    = match_params.back().selector.precisions[1];
-        }
-    }
+    add_mode_matches(true, [](Type dt) -> const char * {
+        if (dt.isFP4()) return "E";
+        return nullptr;
+    });
 
     EvaluateParams eval_params;
 
@@ -602,7 +593,7 @@ status_t gen_gemm_nocopy_kernel_desc_t::select_kernel(compute::gpu_arch_t arch,
     eval_params.batch = (batch_dims > 0);
     eval_params.deterministic = (mode & mode_deterministic);
 
-    entry_ = select(gemm_catalog, static_cast<int>(match_params.size()),
+    entry_ = select(catalog(), static_cast<int>(match_params.size()),
             match_params.data(), eval_params, aux_params_);
 
     if (!entry_) return status::unimplemented;
@@ -616,6 +607,7 @@ status_t gen_gemm_nocopy_kernel_desc_t::select_kernel(compute::gpu_arch_t arch,
     auto update_type = [](Type &T, Type T_new, bool sz_change = false) {
         if ((T.bits() != T_new.bits()) && !sz_change) return;
         if (T.isF8() && T_new.isF8()) return;
+        if (T.isF4() && T_new.isF4()) return;
         T = T.isSigned() ? T_new.asSigned() : T_new.asUnsigned();
     };
     update_type(problem_.Ta, Ta_new, true);
@@ -746,7 +738,7 @@ status_t gen_gemm_xe_systolic_kernel_desc_t::select_kernel(
     eval_params.cConvert = (acc_type != c_type);
     eval_params.batch = (batch_dims > 0);
 
-    entry_ = select(gemm_catalog, match_params, eval_params, aux_params_);
+    entry_ = select(catalog(), match_params, eval_params, aux_params_);
 
     if (!entry_) return status::unimplemented;
 
@@ -844,7 +836,7 @@ void gen_gemm_kernel_t::init_interface() {
     if (problem.cOffset != COffset::None || problem.sumA || problem.sumB) {
         interface_.newArgument(
                 "CO", ExternalArgumentType::GlobalPtr, co_access);
-        interface_.newArgument("offset_CO", DataType::d);
+        interface_.newArgument("offset_CO", DataType::q);
         if (problem.cOffset == COffset::Pre)
             interface_.newArgument("ldco", DataType::d);
     }
@@ -864,7 +856,7 @@ void gen_gemm_kernel_t::init_interface() {
         auto bname = "binary" + std::to_string(i);
         interface_.newArgument(bname, ExternalArgumentType::GlobalPtr,
                 strategy.binary[i].getGlobalAccessType());
-        interface_.newArgument("offset_" + bname, DataType::d);
+        interface_.newArgument("offset_" + bname, DataType::q);
         if (problem.binaryRow[i] && problem.binaryCol[i])
             interface_.newArgument("ld" + bname, DataType::d);
     }
@@ -920,16 +912,16 @@ void gen_gemm_kernel_t::init_interface() {
     if (strategy.variableSLM())
         interface_.newArgument("local_mem", ExternalArgumentType::LocalPtr);
     if (problem.aoPtrDims >= 1 || problem.aScale2D)
-        interface_.newArgument("offset_Aq", DataType::d);
+        interface_.newArgument("offset_Aq", DataType::q);
     if (problem.boPtrDims >= 1 || problem.bScale2D)
-        interface_.newArgument("offset_Bq", DataType::d);
+        interface_.newArgument("offset_Bq", DataType::q);
 
     if (desc()->hw_ >= HW::XeHPG) interface_.allowArgumentRearrangement(false);
     interface_.externalName(kernel_name());
 }
 
-xpu::binary_t gen_gemm_kernel_t::get_binary(
-        const ocl::ocl_gpu_engine_t *engine) {
+status_t gen_gemm_kernel_t::get_kernel(
+        compute::kernel_t &kernel, const compute::compute_engine_t *engine) {
     init_interface();
     maybe_print_verbose();
 
@@ -938,7 +930,7 @@ xpu::binary_t gen_gemm_kernel_t::get_binary(
         gemm_kernel_generator_t<ngen::HW::arch> generator; \
         generator.setStepping(desc()->stepping_); \
         generator.gemm(*desc()->problem(), *desc()->strategy(), interface_); \
-        return generator.getBinary(engine->context(), engine->device()); \
+        return generator.get_kernel(kernel, engine); \
         break; \
     }
 
@@ -957,10 +949,9 @@ xpu::binary_t gen_gemm_kernel_t::get_binary(
     } catch (const std::runtime_error &err) {
         VERROR(primitive, gpu, "%s,%s", "jit::gemm", err.what());
     }
-
-    return {};
-
 #undef ARCH_DISPATCH
+
+    return status::runtime_error;
 }
 
 void gen_gemm_kernel_t::maybe_print_verbose() {
