@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -109,7 +109,7 @@ static bool isSubsetOf(DataType dt1, DataType dt2)
 CopyOperand::CopyOperand(RegData rd)
         : grf(rd.getBase()), offset(rd.getLogicalOffset()),
           stride(rd.getHS()), type(rd.getType()), kind(GRF),
-          overwrite(false), overwriteStride(false), neg(rd.getNeg())
+          overwrite(false), overwriteStride(false), neg(rd.getNeg()), abs(rd.getAbs()), inv(false)
 {
     if (rd.getAbs()) stub("Unsupported modifier");
     if (rd.getVS() != 0 || rd.getWidth() != 0)
@@ -124,6 +124,13 @@ CopyOperand CopyOperand::operator-() const
     return clone;
 }
 
+CopyOperand CopyOperand::operator~() const
+{
+    auto clone = *this;
+    clone.inv = !clone.inv;
+    return clone;
+}
+
 // Convert a GRF CopyOperand to an nGEN object.
 RegData CopyOperand::ngen() const
 {
@@ -132,7 +139,9 @@ RegData CopyOperand::ngen() const
     if (kind != GRF || temp) stub("Invalid operation");
 
     RegData rd = ngen::GRF(grf).sub(offset, type)(stride);
+    if (abs) rd = ngen::abs(rd);
     if (neg) rd = -rd;
+    if (inv) rd = ~rd;
 
     return rd;
 }
@@ -168,7 +177,7 @@ void CopyInstruction::moveToIntegerPipe()
     if (asSigned(st) != asSigned(dt)) return;
 
     switch (getBytes(st)) {
-        case 1: st = dt = is4(st) ? DataType::u4 : DataType::ub; break;
+        case 1: st = dt = isInt4(st) ? DataType::u4 : DataType::ub; break;
         case 2: st = dt = DataType::uw; break;
         case 4: st = dt = DataType::ud; break;
         case 8:
@@ -241,6 +250,10 @@ void CopyPlan::transform()
     sort(SortType::PhaseOnly);
 
     legalizeImmediateTypes();
+#ifdef DNNL_DEV_MODE
+    if (get_verbose(verbose_t::debuginfo) > 100)
+        dump();
+#endif
 }
 
 
@@ -586,9 +599,9 @@ void CopyPlan::split2DRegions()
     auto is2D = [](const CopyOperand &op) { return op.inVS || op.inW; };
 
     for (auto &i: insns) {
-        if (is2D(i.dst) || is2D(i.src1) || is2D(i.src2))
+        if ((is2D(i.dst) && !is4Bit(i.dst.type)) || is2D(i.src1) || is2D(i.src2))
             stub("Unsupported 2D region");
-        if (is2D(i.src0)) {
+        if (is2D(i.src0)){
             if (i.flag) stub("Unsupported predication");
             int w = i.src0.inW, vs = i.src0.inVS, hs = i.src0.stride;
             bool splitH = (w * w >= i.simd);
@@ -646,20 +659,43 @@ void CopyPlan::planTypeConversions()
         if (asSigned(st) == asSigned(dt) && st != dt)
             dt = st;
 
-        if (is4(st) && isInt(dt)) {
+        if (isInt4(st) && isInt(dt)) {
             planInt4Upconversion(i);
             rerun = true;
+        } else if (st == Type::ngen_f4_e2m1() && dt == DataType::hf) {
+            planEmulatedF4E2M1ToHF(i);
+            rerun = true;
+        } else if (dt == Type::ngen_f4_e2m1() && st == DataType::hf) {
+            planEmulatedHFToF4E2M1(i);
+            rerun = true;
+        } else if (st == Type::ngen_f4_e3m0() && dt == DataType::hf) {
+            planEmulatedE3M0ToHF(i);
+            rerun = true;
+        } else if (dt == Type::ngen_f4_e3m0() && st == DataType::hf) {
+            planEmulatedHFToE3M0(i);
+            rerun = true;
+        } else if (isFP4(dt)) {
+            copyThrough(i, DataType::hf);
+            rerun = true;
+        } else if (isFP4(st)) {
+            copyThrough(i, DataType::hf);
+            rerun = true;
+        } else if ((isB(st) || isW(st)) && isInt4(dt)) {
+            planBToI4(i);
+            rerun = true;
         } else if (st == DataType::u4 && dt == DataType::hf) {
             copyThrough(i, DataType::uw);
             rerun = true;
         } else if (st == DataType::s4 && dt == DataType::hf) {
             planS4ToHF(i);
             rerun = true;
-        } else if (is4(st) && isFP(dt)) {
+        } else if (isInt4(st) && isFP(dt)) {
             copyThrough(i, DataType::hf, 1);
             rerun = true;
-        } else if (is4(dt))
+        } else if (isInt4(dt))
             stub("Unsupported move to int4");
+        else if (isFP4(dt))
+            stub("Unsupported move to FP4");
         else if (isB(st) && getBytes(dt) == 8)
             copyThrough(i, DataType::w);
         else if (getBytes(st) == 8 && isB(dt))
@@ -734,11 +770,13 @@ void CopyPlan::planTypeConversions()
                     planEmulatedHalveFloat(i);
             }
         } else if (st == DataType::hf8 && dt == DataType::hf) {
-               if (hw < HW::Xe3)
-                    planEmulatedHF8ToHF(i);
+            if (hw < HW::Xe3)
+                planEmulatedHF8ToHF(i);
         } else if (st == DataType::hf && dt == DataType::hf8) {
-               if (hw < HW::Xe3)
-                   planEmulatedHFToHF8(i);
+            if (hw < HW::Xe3)
+                planEmulatedHFToHF8(i);
+        } else if (st == Type::ngen_f8_e8m0() && dt == DataType::hf) {
+                planEmulatedFP8E8M0ToHF(i);
         } else if (st != dt && (isFP8(st) || isFP8(dt))) {
             copyThrough(i, DataType::hf, 1);
             rerun = true;
@@ -847,6 +885,222 @@ void CopyPlan::planS4ToHF(CopyInstruction &i)
     ie[3]->dst.range = ie[0]->src0.type;
 }
 
+// hf->f4 sequence.
+void CopyPlan::planEmulatedHFToF4E2M1(CopyInstruction &i)
+{
+    // Emulation sequence for mov y:f4 x:hf:
+    //        sel (lt)     t1:hf    (abs)x:hf  0x4600:hf     /* Clamp */
+    //        mul          t1:hf    t1:hf      0x400:hf
+    //        add          t1:uw    t1:uw      -0x100
+    //        and (nz)f0   null     t1:uw      0x3ff:uw
+    //   (f0) add          t1:uw    t1:uw      0x200
+    //        shl          t1:uw    t1:uw      3
+    //        bfn.0xCA     t1:uw    t1:uw      x:uw   0x8000 /* copy sign */
+    //        shr          t1:uw    t1:uw      8
+    //        and          t1:uw    t1:uw      0x00f0:uw     /* byte pack */
+    //        shr          t1:uw<2> t1:uw<2>   4
+    //        or           t1:uw    t1.1:uw<2> t1:uw<2>
+    //        mov          t1:ub    t1:ub<2>
+    //        mov          y:uw     t1:ub
+    //
+
+    if (i.src0.neg || i.sat || i.hasCMod()) stub("Unsupported modifier");
+    int simd = i.simd;
+
+    auto ie = splitMultiple<10>(i);
+    auto tmp = newTemp(DataType::ud, simd, 1);
+
+    auto convFlag = newFlag(simd);
+    auto ddst = i.dst;
+    auto ssrc = i.src0;
+
+    if (ssrc.stride > 1) {
+        ie[0]->op = Opcode::mov;
+        ie[0]->dst = ssrc;
+        ie[0]->dst.stride = 1;
+        ie[0]->src0 = ssrc;
+        ssrc.stride = 1;
+    } else {
+        ie[0]->invalidate();
+    }
+
+    // Clamp and round.
+    ie[1]->op = Opcode::sel;
+    ie[1]->flag = convFlag;
+    ie[1]->cmod = ConditionModifier::lt;
+    ie[1]->simd = simd;
+    ie[1]->dst = tmp;
+    ie[1]->dst.type = DataType::hf;
+    ie[1]->src0 = ssrc;
+    ie[1]->src0.type = DataType::hf;
+    ie[1]->src0.abs = true;
+    ie[1]->src1 = Immediate::hf(0x4600);
+
+    ie[2]->op = Opcode::mul;
+    ie[2]->simd = simd;
+    ie[2]->dst = tmp;
+    ie[2]->dst.type = DataType::hf;
+    ie[2]->src0 = tmp;
+    ie[2]->src0.type = DataType::hf;
+    ie[2]->src1 = Immediate::hf(0x0400);
+
+    ie[3]->op = Opcode::add;
+    ie[3]->simd = simd;
+    ie[3]->dst = tmp;
+    ie[3]->dst.type = DataType::uw;
+    ie[3]->src0 = tmp;
+    ie[3]->src0.type = DataType::uw;
+    ie[3]->src1 = Immediate::w(-0x0100);
+
+    ie[4]->op = Opcode::and_;
+    ie[4]->simd = simd;
+    ie[4]->flag = convFlag;
+    ie[4]->cmod = ConditionModifier::nz;
+    ie[4]->dst = CopyOperand();
+    ie[4]->dst.type = DataType::uw;
+    ie[4]->src0 = tmp;
+    ie[4]->src0.type = DataType::uw;
+    ie[4]->src1 = Immediate::uw(0x03ff);
+
+    ie[5]->op = Opcode::add;
+    ie[5]->simd = simd;
+    ie[5]->flag = convFlag;
+    ie[5]->dst = tmp;
+    ie[5]->dst.type = DataType::uw;
+    ie[5]->src0 = tmp;
+    ie[5]->src0.type = DataType::uw;
+    ie[5]->src1 = Immediate::uw(0x0200);
+
+    ie[6]->op = Opcode::shl;
+    ie[6]->simd = simd;
+    ie[6]->dst = tmp;
+    ie[6]->dst.type = DataType::uw;
+    ie[6]->src0 = tmp;
+    ie[6]->src0.type = DataType::uw;
+    ie[6]->src1 = Immediate::uw(3);
+
+    // Restore sign.
+    ie[7]->op = Opcode::bfn;
+    ie[7]->dst = tmp;
+    ie[7]->dst.stride = 1;
+    ie[7]->dst.type = DataType::uw;
+    ie[7]->src0 = tmp;
+    ie[7]->src0.type = DataType::uw;
+    ie[7]->src1 = ssrc;
+    ie[7]->src1.type = DataType::uw;
+    ie[7]->src2 = 0x8000;
+    ie[7]->ctrl = 0xCA;
+
+    ie[8]->op = Opcode::shr;
+    ie[8]->simd = simd;
+    ie[8]->dst = tmp;
+    ie[8]->dst.type = DataType::uw;
+    ie[8]->src0 = tmp;
+    ie[8]->src0.type = DataType::uw;
+    ie[8]->src1 = Immediate::uw(12);
+
+    // Pack into byte.
+
+    ie[9]->op = Opcode::mov;
+    ie[9]->simd = simd;
+    ie[9]->dst = ddst;
+    ie[9]->dst.type = DataType::u4;
+    ie[9]->src0 = tmp;
+    ie[9]->src0.type = DataType::uw;
+
+}
+
+void CopyPlan::planBToI4(CopyInstruction &i)
+{
+    if (i.src0.neg || i.sat || i.hasCMod()) stub("Unsupported modifier");
+    int simd = i.simd;
+
+    auto ie = splitMultiple<6>(i);
+    auto tmp = newTemp(DataType::uw, simd, 1);
+
+    auto ddst = CopyOperand(i.dst);
+    auto ssrc = CopyOperand(i.src0);
+    bool sUw = ssrc.type == DataType::uw;
+    bool sUb = ssrc.type == DataType::ub;
+    bool tempSrc = ((ssrc.stride > 1 && ssrc.type == DataType::uw) || (ssrc.stride == 1 && ssrc.type == DataType::ub));
+    if ((!sUw && !sUb)) stub();
+    int idx = 0;
+
+    if(tempSrc){
+        ie[idx]->op = Opcode::mov;
+        ie[idx]->dst = ssrc;
+        ie[idx]->dst.type = DataType::ub;
+        ie[idx]->dst.stride = 2;
+        ie[idx]->src0 = ssrc;
+        if(sUw){
+           ie[idx]->src0.type = DataType::ub;
+           ie[idx]->src0.stride = ie[idx]->src0.stride * 2;
+        }
+    }else{
+        ie[idx]->invalidate();
+    }
+    ++idx;
+
+    ie[idx]->op = Opcode::mov;
+    ie[idx]->simd = simd/2;
+    ie[idx]->dst = tmp;
+    ie[idx]->dst.type = DataType::uw;
+    ie[idx]->dst.stride = 1;
+    ie[idx]->src0 = ssrc;
+    ie[idx]->src0.type = DataType::uw;
+    ie[idx]->src0.stride = 2;
+    ++idx;
+
+    ie[idx]->op = Opcode::shl;
+    ie[idx]->simd = simd/2;
+    ie[idx]->dst = ssrc;
+    ie[idx]->dst.offset = 0;
+    ie[idx]->dst.stride = 1;
+    ie[idx]->dst.type = DataType::uw;
+    ie[idx]->src0 = ssrc;
+    ie[idx]->src0.type = DataType::uw;
+    ie[idx]->src0.offset = 1;
+    ie[idx]->src0.stride = 2;
+    ie[idx]->src1 = Immediate::uw(0x4);
+    ++idx;
+
+    ie[idx]->op = Opcode::or_;
+    ie[idx]->simd = simd/2;
+    ie[idx]->dst = tmp;
+    ie[idx]->dst.type = DataType::uw;
+    ie[idx]->dst.stride = 1;
+    ie[idx]->src0 = tmp;
+    ie[idx]->src0.type = DataType::uw;
+    ie[idx]->src0.stride = 1;
+    ie[idx]->src1 = ssrc;
+    ie[idx]->src1.type = DataType::uw;
+    ie[idx]->src1.stride = 1;
+    ++idx;
+
+    ie[idx]->op = Opcode::mov;
+    ie[idx]->simd = simd/2;
+    ie[idx]->dst = tmp;
+    ie[idx]->dst.type = DataType::ub;
+    ie[idx]->dst.stride = 1;
+    ie[idx]->src0 = tmp;
+    ie[idx]->src0.stride = 2;
+    ie[idx]->src0.type = DataType::ub;
+    ++idx;
+
+    ie[idx]->op = Opcode::mov;
+    ie[idx]->simd = simd/2;
+    ie[idx]->dst = ddst;
+    ie[idx]->dst.type = DataType::ub;
+    if (ddst.inVS != 0)
+        ie[idx]->dst.stride = ddst.inVS / ddst.inW;
+    if (ie[idx]->dst.offset != 0)
+            ie[idx]->dst.offset /= 2;
+    ie[idx]->src0 = tmp;
+    ie[idx]->src0.stride = 1;
+    ie[idx]->src0.type = DataType::ub;
+    ++idx;
+}
+
 // Emulated f->bf or hf->bf8 sequence.
 void CopyPlan::planEmulatedHalveFloat(CopyInstruction &i)
 {
@@ -907,7 +1161,7 @@ void CopyPlan::planEmulatedHalveFloat(CopyInstruction &i)
 void CopyPlan::planEarlyInt4Upconversions()
 {
     for (auto &i: insns) {
-        if (i.op == Opcode::mov && is4(i.src0.type) && isB(i.dst.type)) {
+        if (i.op == Opcode::mov && isInt4(i.src0.type) && isB(i.dst.type)) {
             bool s4 = (i.src0.type == DataType::s4);
             if (i.src0.inW == 2 && i.src0.stride == 1 && i.dst.stride >= (s4 ? 2 : 4)) {
                 planInt4Upconversion(i);
@@ -1013,6 +1267,113 @@ void CopyPlan::planInt4Upconversion(CopyInstruction &i)
     }
 }
 
+
+// Emulation sequence for fp8 e8m0->hf conversion.
+void CopyPlan::planEmulatedFP8E8M0ToHF(CopyInstruction &i) {
+    if (i.src0.neg || i.sat || i.hasCMod()) stub("Unsupported modifier");
+    if (hw < HW::XeHP) stub("Unsupported HW");
+
+
+    // Emulation sequence for mov y:hf y:e8m0
+    // mov                 y:uw   x:u8                    /* emulated separately
+    // xor                 t1:uw, 0xFF, y:uw
+    // add                 y:w, y:w, -112
+    // csel (ze)           y:w,  0 , y:w, y:w
+    // cmp  (ge)           t0:w, y:w, 31
+    // shr                 y:uw, 10
+    // csel (ge)           y:fp16,  0x7bff, y:fp16, t0:fp16
+    // csel (ze)           y:fp16, NaN:fp16, y:fp16, t1:fp16
+
+    auto ie = splitMultiple<9>(i);
+
+    auto yOrig = i.dst, y = yOrig;
+
+    bool tempY = (y.stride > 1 && multiGRF(hw, i, y));
+    if (tempY)  /* Replace y by temporary if nonunit stride hurts performance */
+        y = newTemp(DataType::uw, i.simd, 1);
+
+    auto yW = y;
+    yW.type = DataType::w;
+
+    auto t0 = i.src0;
+    if (t0.overwrite && t0.overwriteStride
+        && t0.stride == y.stride * 2 && t0.offset == y.offset * 2) {
+        t0.type = DataType::hf;
+        t0.stride /= 2;
+        t0.offset /= 2;
+    } else
+        t0 = newTemp(DataType::hf, i.simd, y.stride, 0, y.offset);
+
+
+    auto t1 = i.src0;
+    t1 = newTemp(DataType::hf, i.simd, y.stride, 0, y.offset);
+
+    auto t0UW = t0;
+    t0UW.type = DataType::uw;
+
+    auto t1W = t1;
+    t1W.type = DataType::w;
+
+    // Copy to u16.
+    ie[0]->src0.type = DataType::ub;
+    ie[0]->dst.type = DataType::uw;
+    ie[0]->dst = yW;
+
+    ie[1]->op = Opcode::xor_;
+    ie[1]->dst =  t0UW;
+    ie[1]->src0 = yW;
+    ie[1]->src1 = 0xFF;
+
+    ie[2]->op = Opcode::add;
+    ie[2]->dst = ie[2]->src0 =yW;
+    ie[2]->src1 = -112;
+
+    ie[3]->op = Opcode::csel;
+    ie[3]->dst = yW;
+    ie[3]->src0 = Immediate::w(0x0);
+    ie[3]->src1 = yW;
+    ie[3]->src2 = yW;
+    ie[3]->cmod = ConditionModifier::le;
+
+    ie[4]->op = Opcode::cmp;
+    ie[4]->dst = t1W;
+    ie[4]->src0 = yW;
+    ie[4]->src1 = Immediate::uw(31);;
+    ie[4]->cmod = ConditionModifier::le;
+    ie[4]->flag = newFlag(ie[4]->simd);
+
+
+    ie[5]->op = Opcode::shl;
+    ie[5]->dst = yW;
+    ie[5]->src0 = yW;
+    ie[5]->src0.type = DataType::uw;
+    ie[5]->dst.type  = DataType::uw;
+    ie[5]->src1 = 10;
+
+    ie[6]->op = Opcode::csel;
+    ie[6]->dst = y;
+    ie[6]->src0 = Immediate::hf(0x7BFF);
+    ie[6]->src1 = y;
+    ie[6]->src2 = t1;
+    ie[6]->cmod = ConditionModifier::ze;
+
+    ie[7]->op = Opcode::csel;
+    ie[7]->dst = y;
+    ie[7]->src0 = Immediate::hf(0x7C01);
+    ie[7]->src1 = y;
+    ie[7]->src2 = t0;
+    ie[7]->cmod = ConditionModifier::ze;
+
+    if (tempY) {
+        ie[8]->op = Opcode::mov;
+        ie[8]->dst = yOrig;
+        ie[8]->dst.type = DataType::uw;
+        ie[8]->src0 = yW;
+    } else
+        ie[8]->invalidate();
+}
+
+
 // Emulation sequence for hf8->hf conversion.
 void CopyPlan::planEmulatedHF8ToHF(CopyInstruction &i)
 {
@@ -1095,6 +1456,300 @@ void CopyPlan::planEmulatedHF8ToHF(CopyInstruction &i)
         ie[7]->invalidate();
 }
 
+// Emulation sequence for f4_e2m1->hf conversion.
+void CopyPlan::planEmulatedF4E2M1ToHF(CopyInstruction &i) {
+
+    // Emulation sequence for mov y:hf x:hf4_E2M1
+    // mov                 t0:uw   x:u4                    /* emulated separately */
+    // shl                 y:uw    t0:uw    9
+    // shl                 t0:uw   t0:uw    12
+    // and                 y:uw    y:uw    0x0E00
+    // mul                 y:hf    y:hf    16384:hf
+    // bfn.0xCA            y:uw    y:uw    t0:uw  0x8000   /* copy sign */
+
+
+    if (i.src0.neg || i.sat || i.hasCMod()) stub("Unsupported modifier");
+
+    auto ie = splitMultiple<8>(i);
+
+    auto yOrig = i.dst, y = yOrig;
+
+    bool tempY = (y.stride > 1 && multiGRF(hw, i, y));
+    if (tempY)  /* Replace y by temporary if nonunit stride hurts performance */
+        y = newTemp(DataType::uw, i.simd, 1);
+
+    auto yUW = y;
+    yUW.type = DataType::uw;
+
+    auto t0 = newTemp(DataType::hf, i.simd, y.stride, 0, y.offset);
+
+    auto t0UW = t0;
+    t0UW.type = DataType::uw;
+    t0UW.stride = 1;
+    t0UW.offset = 0;
+
+    // Copy to u16.
+    ie[0]->src0.type = DataType::u4;
+    ie[0]->dst.type = DataType::uw;
+    ie[0]->dst = t0UW;
+
+    ie[1]->src0 = ie[0]->dst;
+    ie[1]->op = Opcode::shl;
+    ie[1]->dst = yUW;
+    ie[1]->src1 = 9;
+
+    ie[2]->op = Opcode::shl;
+    ie[2]->dst = t0UW;
+    ie[2]->src0 = ie[0]->dst;
+    ie[2]->src1 = 12;
+
+    ie[3]->op = Opcode::and_;
+    ie[3]->dst = ie[3]->src0 = yUW;
+    ie[3]->src1 = 0x0E00;
+
+    ie[4]->op = Opcode::mul;
+    ie[4]->dst = ie[4]->src0 = y;
+    ie[4]->src1 = Immediate::hf(0x7400);
+
+    ie[5]->op = Opcode::and_;
+    ie[5]->dst = ie[5]->src0 = t0UW;
+    ie[5]->src1 = 0x8000;
+
+    ie[6]->op = Opcode::or_;
+    ie[6]->dst = ie[6]->src0 = yUW;
+    ie[6]->src1 = t0UW;
+
+    if (tempY) {
+        ie[7]->op = Opcode::mov;
+        ie[7]->dst = yOrig;
+        ie[7]->dst.type = DataType::uw;
+        ie[7]->src0 = yUW;
+    } else
+        ie[7]->invalidate();
+}
+
+// Emulation sequence for hf->e3m0 conversion.
+void CopyPlan::planEmulatedHFToE3M0(CopyInstruction &i)
+{
+    // // Emulation sequence for  mov y:e3m0 x:hf
+    // sel (lt)     t0:hf   x:hf    0x4c00:hf       /* clamp max val */
+    // mul          t0:hf   t0:hf   0x0c00:hf       /* round up */
+    // add          t0:uw   t0:uw   -0x200          /* subtract 1/2 ULP */
+    // and (nz)f0   null:uw t0:uw   0x7FF
+    // (f0) add     t0:uw   t0:uw   0x400           /* round up */
+    // shl          t0:uw   t0:uw   2
+    // bfn.0xCA     t0:uw   x:uw    t0:uw  0x8000   /* copy sign */
+    // shr          t0:uw   t0:uw   12
+    // mov /* pack halfs into bytes */
+
+    if (i.src0.neg || i.sat || i.hasCMod()) stub("Unsupported modifier");
+
+    auto ie = splitMultiple<10>(i);
+
+    auto x = i.src0;
+    auto y = i.dst;
+
+    bool tempX = (x.stride > 1);
+
+    auto bits = newTemp(DataType::uw, i.simd, 1, 0, 0);
+
+    auto selFlag = newFlag(i.simd);
+    int simd = i.simd;
+    int idx = 0;
+
+    if(tempX){
+        ie[idx]->op = Opcode::mov;
+        ie[idx]->dst = x;
+        ie[idx]->dst.stride = 1;
+        ie[idx]->src0 = x;
+    }else{
+        ie[idx]->invalidate();
+    }
+    ++idx;
+
+    ie[idx]->op = Opcode::sel;
+    ie[idx]->cmod = ConditionModifier::lt;
+    ie[idx]->dst = bits;
+    ie[idx]->dst.type = DataType::hf;
+    ie[idx]->src0 = x;
+    ie[idx]->src0.abs = true;
+    ie[idx]->src0.stride = 1;
+    ie[idx]->src0.type = DataType::hf;
+    ie[idx]->src1 = 0x4c00;
+    ie[idx]->src1.type = DataType::hf;
+    ++idx;
+
+    ie[idx]->op = Opcode::mul;
+    ie[idx]->dst = bits;
+    ie[idx]->dst.stride = 1;
+    ie[idx]->dst.type = DataType::hf;
+    ie[idx]->src0 = bits;
+    ie[idx]->src0.stride = 1;
+    ie[idx]->src0.type = DataType::hf;
+    ie[idx]->src1 = 0x0c00;
+    ie[idx]->src1.type = DataType::hf;
+    ++idx;
+
+    ie[idx]->op = Opcode::add;
+    ie[idx]->dst = bits;
+    ie[idx]->dst.stride = 1;
+    ie[idx]->dst.type = DataType::uw;
+    ie[idx]->src0 = bits;
+    ie[idx]->src0.stride = 1;
+    ie[idx]->src0.type = DataType::uw;
+    ie[idx]->src1 = -0x0200;
+    ie[idx]->src1.type = DataType::w;
+    ++idx;
+
+    ie[idx]->op = Opcode::and_;
+    ie[idx]->flag = selFlag;
+    ie[idx]->cmod = ConditionModifier::nz;
+    ie[idx]->dst = CopyOperand();
+    ie[idx]->dst.stride = 1;
+    ie[idx]->dst.type = DataType::uw;
+    ie[idx]->src0 = bits;
+    ie[idx]->src0.stride = 1;
+    ie[idx]->src0.type = DataType::uw;
+    ie[idx]->src1 = 0x07ff;
+    ie[idx]->src1.type = DataType::uw;
+    ++idx;
+
+    ie[idx]->op = Opcode::add;
+    ie[idx]->flag = selFlag;
+    ie[idx]->dst = bits;
+    ie[idx]->dst.stride = 1;
+    ie[idx]->dst.type = DataType::uw;
+    ie[idx]->src0 = bits;
+    ie[idx]->src0.stride = 1;
+    ie[idx]->src0.type = DataType::uw;
+    ie[idx]->src1 = 0x400;
+    ie[idx]->src1.type = DataType::uw;
+    ++idx;
+
+    ie[idx]->op = Opcode::shl;
+    ie[idx]->dst = bits;
+    ie[idx]->dst.stride = 1;
+    ie[idx]->dst.type = DataType::uw;
+    ie[idx]->src0 = bits;
+    ie[idx]->src0.stride = 1;
+    ie[idx]->src0.type = DataType::uw;
+    ie[idx]->src1 = 0x2;
+    ie[idx]->src1.type = DataType::uw;
+    ++idx;
+
+    ie[idx]->op = Opcode::bfn;
+    ie[idx]->dst = bits;
+    ie[idx]->dst.stride = 1;
+    ie[idx]->dst.type = DataType::uw;
+    ie[idx]->src0 = bits;
+    ie[idx]->src0.stride = 1;
+    ie[idx]->src0.type = DataType::uw;
+    ie[idx]->src1 = x;
+    ie[idx]->src1.stride = 1;
+    ie[idx]->src1.type = DataType::uw;
+    ie[idx]->src2 = 0x8000;
+    ie[idx]->ctrl = 0xCA;
+    ++idx;
+
+    ie[idx]->op = Opcode::shr;
+    ie[idx]->dst = bits;
+    ie[idx]->dst.stride = 1;
+    ie[idx]->dst.type = DataType::uw;
+    ie[idx]->src0 = bits;
+    ie[idx]->src0.stride = 1;
+    ie[idx]->src0.type = DataType::uw;
+    ie[idx]->src1 = 0xC;
+    ie[idx]->src1.type = DataType::uw;
+    ++idx;
+
+    // Pack.
+    ie[idx]->op = Opcode::mov;
+    ie[idx]->simd = simd;
+    ie[idx]->dst = y;
+    ie[idx]->dst.type = DataType::u4;
+    ie[idx]->dst.stride = 1;
+    ie[idx]->src0 = bits;
+    ie[idx]->src0.type = DataType::uw;
+    ie[idx]->src0.stride = 1;
+    ++idx;
+
+}
+
+// Emulation sequence for e3m0->hf conversion.
+void CopyPlan::planEmulatedE3M0ToHF(CopyInstruction &i)
+{
+    // Emulation sequence for mov y:hf x:e3m0: play only on the exponent bits
+    // mov                 y:uw   x:u4                    /* emulated separately */
+    // shl                 t0:uw   t0:uw   12
+    // and                 y:uw    y:uw    0x7
+    // (f1)add             y:uw    y:uw    12
+    // shl                 y:uw    y:uw    10
+    // bfn.0xCA            y:uw    y:uw    t0:uw  0x8000   /* copy sign */
+
+    if (i.src0.neg || i.sat || i.hasCMod()) stub("Unsupported modifier");
+
+    auto ie = splitMultiple<8>(i);
+
+    auto yOrig = i.dst, y = yOrig;
+
+    bool tempY = (y.stride > 1 && multiGRF(hw, i, y));
+    if (tempY)  /* Replace y by temporary if nonunit stride hurts performance */
+        y = newTemp(DataType::uw, i.simd, 1);
+
+    auto yUW = y;
+    yUW.type = DataType::uw;
+
+    auto t0 = newTemp(DataType::hf, i.simd, y.stride, 0, y.offset);
+
+    auto t0UW = t0;
+    t0UW.type = DataType::uw;
+
+    // Copy to u16.
+    ie[0]->src0.type = DataType::u4;
+    ie[0]->dst.type = DataType::uw;
+    ie[0]->dst = yUW;
+
+    ie[1]->op = Opcode::shl;
+    ie[1]->dst = t0UW;
+    ie[1]->src0 = ie[0]->dst;
+    ie[1]->src1 = 12;
+
+    ie[2]->op = Opcode::and_;
+    ie[2]->dst = ie[2]->src0 = yUW;
+    ie[2]->src1 = 0x7;
+    ie[2]->cmod = ConditionModifier::nz;
+    ie[2]->flag = newFlag(ie[2]->simd);
+
+    ie[3]->op = Opcode::add;
+    ie[3]->dst = ie[3]->src0 = yUW;
+    ie[3]->src1 = 0xc;
+    ie[3]->dst.type = DataType::uw;
+    ie[3]->src0.type = DataType::uw;
+    ie[3]->src1.type = DataType::uw;
+    ie[3]->flag = ie[2]->flag;
+
+    ie[4]->src0 = ie[3]->dst;
+    ie[4]->op = Opcode::shl;
+    ie[4]->dst = yUW;
+    ie[4]->src1 = 10;
+
+    ie[5]->op = Opcode::and_;
+    ie[5]->dst = ie[5]->src0 = t0UW;
+    ie[5]->src1 = 0x8000;
+
+    ie[6]->op = Opcode::or_;
+    ie[6]->dst = ie[6]->src0 = yUW;
+    ie[6]->src1 = t0UW;
+
+    if (tempY) {
+        ie[7]->op = Opcode::mov;
+        ie[7]->dst = yOrig;
+        ie[7]->dst.type = DataType::uw;
+        ie[7]->src0 = yUW;
+    } else
+        ie[7]->invalidate();
+}
+
 // Emulation sequence for hf->hf8 conversion.
 void CopyPlan::planEmulatedHFToHF8(CopyInstruction &i)
 {
@@ -1108,7 +1763,7 @@ void CopyPlan::planEmulatedHFToHF8(CopyInstruction &i)
     // mul          t0:hf   t0:hf   2^(-15):hf      /* hf8 underflow */
     // and (ze)f1   null    ~t0:uw  0x7C00          /* nan/inf check */
     // add          t0:uw   t0:uw   -0x40           /* round */
-    // and (nz)f2   null    t0:uw   0x00FF
+    // and (nz)f2   null    t0:uw   0x0FF
     // shl          t0:uw   t0:uw   1               /* move to high byte */
     // (f2) add     t0:uw   t0:uw   0x100
     // (f1) mov     t0:uw   0x7F00
@@ -1145,7 +1800,7 @@ void CopyPlan::planEmulatedHFToHF8(CopyInstruction &i)
     ie[4]->dst = CopyOperand();
     ie[4]->dst.type = DataType::uw;
     ie[4]->src0 = t0UW;
-    ie[4]->src1 = 0x00FF;
+    ie[4]->src1 = 0x0FF;
     ie[4]->cmod = ConditionModifier::nz;
     ie[4]->flag = newFlag(ie[4]->simd);
 
@@ -1182,7 +1837,7 @@ void CopyPlan::checkNoSubbytes()
 {
     for (auto &i: insns)
         if (is4(i.dst.type) || is4(i.src0.type) || is4(i.src1.type) || is4(i.src2.type))
-            stub("Unexpected int4 type");
+            stub("Unexpected 4-bit type");
 }
 
 // Collapse multiple-cnum instructions into single-cnum instructions if possible.
@@ -1315,28 +1970,28 @@ inline bool legalPackedBF(HW hw, const CopyOperand &op)
 }
 
 void   CopyPlan::planFP8SIMD1Mov(CopyInstruction &i){
-		/* Simd 1 not allowed, use following sequence instead:
-		   hf8->hf (analagous sequence will be generated for hf->hf8)
-		   mov(2, t_dst.hf, src<2,2,1>.hf8)
-		   mov(1, dst.uw, t_dst<1,1,1>.uw) */
+    /* Simd 1 not allowed, use following sequence instead:
+       hf8->hf (analagous sequence will be generated for hf->hf8)
+       mov(2, t_dst.hf, src<2,2,1>.hf8)
+       mov(1, dst.uw, t_dst<1,1,1>.uw) */
 
-                auto dt = i.dst.type;
-                auto ie = splitMultiple<2>(i);
-                auto src = i.src0;
-                auto dst = i.dst;
-                auto t_dst = newTemp(dt, 2, 1);
-                t_dst.stride = 1;
+     auto dt = i.dst.type;
+     auto ie = splitMultiple<2>(i);
+     auto src = i.src0;
+     auto dst = i.dst;
+     auto t_dst = newTemp(dt, 2, 1);
+     t_dst.stride = 1;
 
-                ie[0]->op = Opcode::mov;
-                ie[0]->dst = t_dst;
-                ie[0]->src0 = src;
-                ie[0]->src0.stride = 1;
-                ie[0]->simd = 2;
+     ie[0]->op = Opcode::mov;
+     ie[0]->dst = t_dst;
+     ie[0]->src0 = src;
+     ie[0]->src0.stride = 1;
+     ie[0]->simd = 2;
 
-                ie[1]->op = Opcode::mov;
-                ie[1]->dst = dst;
-                ie[1]->src0 = t_dst;
-                ie[1]->moveToIntegerPipe();
+     ie[1]->op = Opcode::mov;
+     ie[1]->dst = dst;
+     ie[1]->src0 = t_dst;
+     ie[1]->moveToIntegerPipe();
 }
 
 // Pass to legalize regions.
@@ -1361,13 +2016,13 @@ void CopyPlan::legalizeRegions()
             if (i.simd == 1 && i.src0.offset == 0 && i.src0.stride == 1){
                 planFP8SIMD1Mov(i);
                 rerun = true;
-	    } else if (i.src0.offset != 0 || i.src0.stride != 1) {
+            } else if (i.src0.offset != 0 || i.src0.stride != 1) {
                 repositionSrc(i, 0, 1, 0);
                 rerun = true;
             } else if (i.dst.offset != 0 || i.dst.stride != 1)
                 repositionDst(i, 1, 0);
             continue;
-	}
+        }
 
         if (dt == DataType::bf || s0t == DataType::bf || s1t == DataType::bf) {
             // bf/f mixed mode: src/dst may be packed unit stride
@@ -1415,14 +2070,15 @@ void CopyPlan::legalizeRegions()
         if (hfIntConvert)
             maxChanOff = 1;     /* special case: integer<->hf only allows .0:hf */
 
-        doRestrideDst |= (channelOffset > maxChanOff);
+        bool badChanOff = (channelOffset >= maxChanOff);
+        doRestrideDst |= badChanOff;
 
         /* For illegal dst, copy through temporary dst */
         if (doRestrideDst) {
             if (i.simd == 1)
                 i.dst.stride = dstMinStride;
             else {
-                restrideDst(i, dstMinStride, hfIntConvert);
+                restrideDst(i, dstMinStride, badChanOff);
                 rerun = true;
                 continue;
             }
@@ -1499,32 +2155,51 @@ void CopyPlan::legalizeRegions()
             int d0o = i.dst.offset;
             int s0s = i.src0.stride;
             int s0o = i.src0.offset;
+            bool strideOK = true, offsetOK = true;
 
             if (!isW(dt)  && !isB(dt))  stub();
             if (!isW(s0t) && !isB(s0t)) stub();
 
             if (isW(s0t)) {
-                canSwizzle &= (s0s <= 2);
+                strideOK &= (s0s <= 2);
                 if (s0s == 2) {
-                    if (isW(dt))
-                        canSwizzle &= (s0o / 2 == d0o % 16);
-                    else
-                        canSwizzle &= (s0o == d0o % 32);
+                    if (isW(dt)) {
+                        offsetOK &= (s0o / 2 == d0o % 16);
+                        d0o = (s0o / 2) % 16;
+                    } else {
+                        offsetOK &= (s0o == d0o % 32);
+                        d0o = s0o % 32;
+                    }
                 }
             } else {
                 if (isW(dt) || d0s > 1)
                     s0s /= 2;
                 if (isW(dt))
                     d0o *= 2;
-                canSwizzle &= (s0s <= 4);
-                if (s0s >= 2)
-                    canSwizzle &= (d0o % (64 / s0s) == s0o / s0s);
+                strideOK &= (s0s <= 4);
+                if (s0s >= 2) {
+                    offsetOK &= (d0o % (64 / s0s) == s0o / s0s);
+                    auto saveD0O = d0o;
+                    d0o = s0o / s0s;
+                    if (isW(dt)) {
+                        if (d0o & 1)
+                            s0o = (saveD0O % (64 / s0s)) * s0s; /* move src rather than dst */
+                        else
+                            d0o /= 2;
+                    }
+                }
             }
 
-            if (!canSwizzle) {
+            if (!strideOK) {
                 int istride = 4 / getBytes(dt);
                 (i.src0.byteStride() < i.dst.byteStride()) ? restrideSrc0(i, istride)
                                                            : restrideDst(i, istride);
+                continue;
+            }
+
+            if (!offsetOK) {
+                (s0o != i.src0.offset) ? repositionSrc(i, 0, i.src0.stride, s0o)
+                                       : repositionDst(i,    i.dst.stride,  d0o);
             }
         }
     }
@@ -1607,6 +2282,8 @@ void CopyPlan::optimizeZip()
                 if (o1.temp && o1.value != o2.value) return false;
                 if (o1.stride & 1) return false;
                 if (o1.neg != o2.neg) return false;
+                if (o1.abs != o2.abs) return false;
+                if (o1.inv != o2.inv) return false;
                 return (o1.offset + (o1.stride >> 1) == o2.offset);
             };
 
@@ -1646,7 +2323,7 @@ static void widen(CopyOperand &op, bool zipping = false)
         case CopyOperand::Null: return;
     }
 
-    if (is4(op.type))      op.type = DataType::ub;
+    if (isInt4(op.type))      op.type = DataType::ub;
     else if (isB(op.type)) op.type = DataType::uw;
     else if (isW(op.type)) op.type = DataType::ud;
     else stub();
@@ -1695,6 +2372,8 @@ void CopyPlan::optimizeZipAdjacent()
             if (o1.temp && o1.value != o2.value) return false;
             if (!widenable(o1, true)) return false;
             if (o1.neg != o2.neg) return false;
+            if (o1.abs != o2.abs) return false;
+            if (o1.inv != o2.inv) return false;
             return (o1.offset + 1 == o2.offset);
         };
 
@@ -1737,7 +2416,7 @@ void CopyPlan::optimizeWidenIntegers()
         while (true) {
             bool doWiden = widenable(i.dst) && widenable(i.src0)
                         && asSigned(i.dst.type) == asSigned(i.src0.type)
-                        && !i.src0.neg && i.simd % 2 == 0;
+                        && !i.src0.neg && !i.src0.inv && !i.src0.abs && i.simd % 2 == 0;
 
             for (auto op: {&i.src1, &i.src2}) if (*op) {
                 doWiden = doWiden && widenable(*op)
@@ -1782,6 +2461,8 @@ void CopyPlan::optimizeConcatenate(bool initial)
                 if (o1.temp != o2.temp) return false;
                 if (o1.temp && (o1.value != o2.value)) return false;
                 if (o1.neg != o2.neg) return false;
+                if (o1.abs != o2.abs) return false;
+                if (o1.inv != o2.inv) return false;
                 auto gap = (o2.absByteOffset(hw) - o1.absByteOffset(hw))
                          - elementsToBytes(o1.stride * i1.simd, o1.type);
                 if (outTooFar)
@@ -1965,7 +2646,7 @@ void CopyPlan::materializeTemps(const GRFAllocator &grfAllocator, const FlagAllo
     cnumOrder.reserve(temps.size());
 
     for (size_t t = 0; t < temps.size(); t++)
-        cnumOrder.push_back(std::make_pair(temps[t].cnumMin, t));
+        cnumOrder.push_back(std::make_pair(temps[t].cnumMin, int(t)));
     std::sort(cnumOrder.begin(), cnumOrder.end());
 
     for (int cnum0 = 0; cnum0 < ncnum; ) {
@@ -2037,5 +2718,108 @@ void CopyPlan::materializeTemps(const GRFAllocator &grfAllocator, const FlagAllo
     temps.clear();
 }
 
+#ifdef DNNL_DEV_MODE
+int CopyPlan::cycleCount() const
+{
+    int count = 0;
+    for (const auto &i: insns)
+        count += (multiGRF(hw, i, i.dst) || multiGRF(hw, i, i.src0) || multiGRF(hw, i, i.src1) || multiGRF(hw, i, i.src2)) ? 2 : 1;
+    return count;
+}
+
+void CopyPlan::dump() const
+{
+    for (const auto &i: insns)
+        i.dump(*this);
+}
+
+void CopyInstruction::dump(const CopyPlan &plan) const
+{
+    if (flag && cmod == ConditionModifier::none) {
+        std::cout << '(';
+        flag.dump();
+        std::cout << ")\t";
+    }
+
+    std::cout << getMnemonic(op, HW::Gen9);
+    if (op == Opcode::bfn)
+        std::cout << '.' << std::hex << int(ctrl) << std::dec;
+    std::cout << " (" << simd << ")\t";
+    if (sat) std::cout << "(sat) ";
+    if (cmod != ConditionModifier::none) {
+        std::cout << '(' << cmod << ')';
+        flag.dump();
+        std::cout << ' ';
+    }
+    dst.dump();
+    std::cout << '\t';
+    src0.dump();
+    if (src1) {
+        std::cout << '\t';
+        src1.dump();
+        if (src2) {
+            std::cout << '\t';
+            src2.dump();
+        }
+    }
+    if (atomic)
+        std::cout << "\t{Atomic}";
+    if (get_verbose(verbose_t::debuginfo))
+        std::cout << "\t\t(phase = " << phase << ", cnum = [" << cnumMin << ", " << cnumMax << "])";
+
+    std::cout << std::endl;
+}
+
+void CopyOperand::dump() const
+{
+    auto outType = [](DataType dt) {
+        if (dt == Type::ngen_f8_e8m0())
+            std::cout << "e8m0";
+        if (dt == Type::ngen_f4_e2m1())
+            std::cout << "e2m1";
+        if (dt == Type::ngen_f4_e3m0())
+            std::cout << "e3m0";
+        else
+            std::cout << dt;
+    };
+
+    if (neg) std::cout << '-';
+    switch (kind) {
+        case Null: std::cout << "null:" << type; break;
+        case GRF:
+            if (temp) {
+                std::cout << 't' << value;
+                if (grf) std::cout << '+' << grf;
+            } else
+                std::cout << 'r' << grf;
+            std::cout << '.' << int(offset) << ':';
+            outType(type);
+            if (range != DataType::invalid && range != type) {
+                std::cout << '[';
+                outType(range);
+                std::cout << ']';
+            }
+            std::cout << '<';
+            if (inVS || inW)
+                std::cout << int(inVS) << ';' << int(inW) << ',';
+            std::cout << int(stride) << '>';
+            break;
+        case Flag:
+            if (temp)
+                std::cout << 't' << value;
+            else
+                std::cout << 'f' << (grf >> 1) << '.' << (grf & 1);
+            if (offset)
+                std::cout << '+' << int(offset);
+            break;
+        case Immediate:
+            LabelManager man;
+            ngenImmediate().outputText(std::cout, PrintDetail::full, man);
+            break;
+    }
+    if (stride > 1 && overwriteStride) std::cout << "!!";
+    else if (overwrite)                std::cout << '!';
+}
+#endif
 
 #include "internal/namespace_end.hxx"
