@@ -244,7 +244,7 @@ bool BLASKernelGenerator<hw>::gemmBinaryOpC(BinaryOp op, bool row, bool column,
     auto globalCM = isLayoutColMajor(state.C_layout);
 
     bool recip = false;
-    if (op == BinaryOp::Div && one_of(Tco, Type::f32, Type::f16)) {
+    if (op == BinaryOp::Div && one_of(Tco, Type::f32)) {
         // Implement div as inv+mul for speed, especially when broadcasting.
         recip = true;
         op = BinaryOp::Mul;
@@ -538,7 +538,7 @@ void BLASKernelGenerator<hw>::gemmApplyPostOps(size_t poMin, size_t poMax, const
         auto &entry = problem.postOps[i];
         switch (entry.kind()) {
             case post_op::kind_t::eltwise: {
-                using Injector = jit_eltwise_injector_f32<hw>;
+                using Injector = eltwise_injector_f32_t<GENERATOR_BASE(hw)>;
                 if (state.Tacc != Type::f32) stub();
 
                 int euCount = 0; /* only used for a DG2 W/A for conv */
@@ -573,7 +573,7 @@ void BLASKernelGenerator<hw>::gemmApplyPostOps(size_t poMin, size_t poMax, const
         }
     }
     if(problem.cStochasticRound){
-        using Injector = jit_eltwise_injector_f32<hw>;
+        using Injector = eltwise_injector_f32_t<GENERATOR_BASE(hw)>;
         int euCount = 0; /* only used for a DG2 W/A for conv */
         Injector injector{this, alg_kind::eltwise_stochastic_round, 0.0, 0.0, 1.0, 
                           euCount, GRFRange(), problem.postOpFwd};
@@ -730,10 +730,10 @@ void BLASKernelGenerator<hw>::gemmApplyABOffset(const GEMMProblem &problem, cons
         if (!(aOffset && bOffset)) return Subregister{};
 
         auto ret = state.ra.alloc_sub(problem.Tc.ngen());
-
-        if (!boVector) mul(1, ret, state.k, state.inputs.bo);
-        else if (Tc.isFP()) mov(1, ret, state.k);
-        else stub();
+        if (!boVector)
+            mul(1, ret, state.k, state.inputs.bo);
+        else
+            mov(1, ret, state.k);
 
         return ret;
     }();
@@ -781,7 +781,7 @@ void BLASKernelGenerator<hw>::gemmApplyABOffset(const GEMMProblem &problem, cons
 
         if (aOffset && bOffset) for (int r = 0; r < state.Bs_regs.getLen(); r++) {
             auto ne = elementsPerGRF(hw, Tc);
-            auto Bs = state.Bs_regs[r];
+            auto Bs = state.Bs_regs[r].retype(Tc.ngen());
             boVector ? emad(ne, Bs, Bs, boData[r].retype(Tc.ngen()), temp, strategy, state)
                      : add(ne, Bs, Bs, temp);
         };
