@@ -177,7 +177,7 @@ void GEMMStrategy::preflight(HW hw, const GEMMProblem &problem)
 
     block2DCRemainder &= !isPacked(problem.C.layout);
     block2DCRemainder &= !isBlock2D(C.accessType);
-    block2DCFull |= (Tc_ext.size() < 4);
+    block2DCFull |= (Tc_ext.paddedSize() < 4);
     block2DCFull &= block2DCRemainder;
 
     extendedAtomicFMA &= !problem.needsASums() && !problem.needsBSums();
@@ -223,6 +223,8 @@ void GEMMStrategy::preflight(HW hw, const GEMMProblem &problem)
         dpasw = true;
     }
 
+    altCRemainder &= (problem.Tc_ext.bits() >= 8);
+
     dpasw &= systolic && fused;
 
     // Accumulator usage: 64-bit emulation, or k chaining, or extra C registers, or storage for r0 header.
@@ -427,10 +429,14 @@ void GEMMStrategy::preflight(HW hw, const GEMMProblem &problem)
     if (blocking[LoopM] <= 0) blocking[LoopM] = defaultMBlock;
     if (blocking[LoopN] <= 0) blocking[LoopN] = defaultNBlock;
     if (blocking[LoopK] <= 0) {
-        int points = 1;
-        if (slmA || (problem.A.layout != MatrixLayout::T)) points++;
-        if (slmB || (problem.B.layout != MatrixLayout::N)) points++;
-        blocking[LoopK] = std::min(2048, (2048 * points) / problem.Ta);
+        if (hw >= HW::XeHPG)
+            blocking[LoopK] = 16777216;
+        else {
+            int points = 1;
+            if (slmA || (problem.A.layout != MatrixLayout::T)) points++;
+            if (slmB || (problem.B.layout != MatrixLayout::N)) points++;
+            blocking[LoopK] = std::min(2048, (2048 * points) / problem.Ta);
+        }
     }
 
     auto defaultBlockAltK = blocking[LoopK];
