@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,5 +16,6 @@
 
 #include "../config.hpp"
 
-#   include "gpu/intel/jit/ngen/ngen_opencl.hpp"
-#   include "gpu/intel/jit/ngen/ngen_register_allocator.hpp"
+#   include "ngen_opencl.hpp"
+#   include "ngen_register_allocator.hpp"
+#   include "ngen_asm.hpp"
