@@ -145,27 +145,20 @@ struct type_info_t {
     IR_DECL_TYPE_ID(class_name) \
     static bool _is_stmt() { return true; };
 
-#define IR_DECL_MUTATE(mutator_template) \
-    object_t _mutate(mutator_template &mutator) const override { \
-        return mutator._mutate(*this); \
-    }
-#define IR_DECL_VISIT(visitor_template) \
-    void _visit(visitor_template &visitor) const override { \
-        visitor._visit(*this); \
-    }
-
 #define IR_DECLARE_TRAVERSERS() \
-    IR_DECL_MUTATE(ir_mutator_t) \
-    IR_DECL_VISIT(ir_visitor_t)
+    object_t _mutate(ir_mutator_t &mutator) const override { \
+        return mutator._mutate(*this); \
+    } \
+    void _visit(ir_visitor_t &visitor) const override { visitor._visit(*this); }
 
 // Defines getter for a function argument.
 #define IR_DEFINE_ARG_GET(name, index) \
     static const expr_t &arg_##name(const func_call_t &c) { \
-        ir_assert(c.func.is<self_type>()) << c; \
+        gpu_assert(c.func.is<self_type>()) << c; \
         return c.args[index]; \
     } \
     static const expr_t &arg_##name(const stmt_t &s) { \
-        ir_assert(s.is<func_call_t>()) << s; \
+        gpu_assert(s.is<func_call_t>()) << s; \
         auto &c = s.as<func_call_t>(); \
         return arg_##name(c); \
     } \
@@ -203,6 +196,8 @@ enum class type_kind_t {
     _bool,
 
     // Integer types.
+    u4,
+    s4,
     u8,
     s8,
     u16,
@@ -213,6 +208,8 @@ enum class type_kind_t {
     s64,
 
     // Floating point types.
+    f4_e3m0,
+    f4_e2m1,
     bf8,
     f8_e5m2 = bf8,
     hf8,
@@ -233,6 +230,8 @@ enum class type_kind_t {
 
 static auto type_kind_names = nstl::to_array({
         make_enum_name(type_kind_t::undef, "undef"),
+        make_enum_name(type_kind_t::u4, "u4"),
+        make_enum_name(type_kind_t::s4, "s4"),
         make_enum_name(type_kind_t::u8, "u8"),
         make_enum_name(type_kind_t::s8, "s8"),
         make_enum_name(type_kind_t::u16, "u16"),
@@ -241,6 +240,8 @@ static auto type_kind_names = nstl::to_array({
         make_enum_name(type_kind_t::s32, "s32"),
         make_enum_name(type_kind_t::u64, "u64"),
         make_enum_name(type_kind_t::s64, "s64"),
+        make_enum_name(type_kind_t::f4_e3m0, "f4_e3m0"),
+        make_enum_name(type_kind_t::f4_e2m1, "f4_e2m1"),
         make_enum_name(type_kind_t::bf8, "bf8"),
         make_enum_name(type_kind_t::hf8, "hf8"),
         make_enum_name(type_kind_t::bf16, "bf16"),
@@ -264,6 +265,8 @@ public:
         return type_t(type_kind_t::_bool, elems);
     }
 
+    static type_t u4(int elems = 1) { return type_t(type_kind_t::u4, elems); }
+    static type_t s4(int elems = 1) { return type_t(type_kind_t::s4, elems); }
     static type_t u8(int elems = 1) { return type_t(type_kind_t::u8, elems); }
     static type_t s8(int elems = 1) { return type_t(type_kind_t::s8, elems); }
     static type_t u16(int elems = 1) { return type_t(type_kind_t::u16, elems); }
@@ -276,11 +279,12 @@ public:
     // Returns unsigned integer type.
     static type_t u(int bits, int elems = 1) {
         switch (bits) {
+            case 4: return u4(elems);
             case 8: return u8(elems);
             case 16: return u16(elems);
             case 32: return u32(elems);
             case 64: return u64(elems);
-            default: ir_error_not_expected();
+            default: gpu_error_not_expected();
         }
         return type_t::undef();
     }
@@ -288,14 +292,22 @@ public:
     // Returns signed integer type.
     static type_t s(int bits, int elems = 1) {
         switch (bits) {
+            case 4: return s4(elems);
             case 8: return s8(elems);
             case 16: return s16(elems);
             case 32: return s32(elems);
             case 64: return s64(elems);
-            default: ir_error_not_expected();
+            default: gpu_error_not_expected();
         }
         return type_t::undef();
     }
+
+    static type_t f4_e3m0(int elems = 1) {
+        return type_t(type_kind_t::f4_e3m0, elems);
+    }
+    static type_t f4_e2m1(int elems = 1) {
+        return type_t(type_kind_t::f4_e2m1, elems);
+    }
     static type_t bf8(int elems = 1) { return type_t(type_kind_t::bf8, elems); }
     static type_t hf8(int elems = 1) { return type_t(type_kind_t::hf8, elems); }
     static type_t bf16(int elems = 1) {
@@ -344,7 +356,7 @@ public:
 
 #undef CASE
 
-        ir_error_not_expected();
+        gpu_error_not_expected();
 
         return undef();
     }
@@ -352,6 +364,8 @@ public:
     template <typename T>
     T max() const {
         switch (kind()) {
+            case type_kind_t::u4:
+            case type_kind_t::s4:
             case type_kind_t::u8:
             case type_kind_t::s8:
             case type_kind_t::u16:
@@ -360,12 +374,12 @@ public:
             case type_kind_t::s32:
             case type_kind_t::u64:
             case type_kind_t::s64: {
-                int bits = 8 * size();
+                int bits = scalar().bitsize();
                 if (is_signed()) bits--;
                 T ret = T(1) << (bits - 1);
                 return ret + (ret - 1);
             }
-            default: ir_error_not_expected();
+            default: gpu_error_not_expected();
         }
         return 0;
     }
@@ -373,6 +387,8 @@ public:
     template <typename T>
     T min() const {
         switch (kind()) {
+            case type_kind_t::u4:
+            case type_kind_t::s4:
             case type_kind_t::u8:
             case type_kind_t::s8:
             case type_kind_t::u16:
@@ -384,7 +400,7 @@ public:
                 if (is_unsigned()) return 0;
                 return -max<T>() - 1;
             }
-            default: ir_error_not_expected();
+            default: gpu_error_not_expected();
         }
         return 0;
     }
@@ -395,30 +411,35 @@ public:
 
     type_t(type_kind_t kind, uint32_t elems = 1) : kind_(kind), elems_(elems) {}
 
-    type_t(const std::string &s) {
-        elems_ = 1;
+    type_t(const std::string &s) : elems_(1) {
 #define CASE(x) \
     if (to_string(type_kind_t::x) == s) { \
         kind_ = type_kind_t::x; \
         return; \
     }
+        CASE(f4_e3m0);
+        CASE(f4_e2m1);
+        CASE(f8_e5m2);
+        CASE(f8_e4m3);
         CASE(bf16);
         CASE(f16);
         CASE(tf32);
         CASE(f32);
         CASE(f64);
-        CASE(s32);
-        CASE(s64);
+
+        CASE(s4);
         CASE(s8);
-        CASE(u8);
         CASE(s16);
         CASE(s32);
         CASE(s64);
+
+        CASE(u4);
+        CASE(u8);
         CASE(u16);
         CASE(u32);
         CASE(u64);
 #undef CASE
-        ir_error_not_expected();
+        gpu_error_not_expected();
     }
 
     // Constructor from dnnl_data_type_t.
@@ -428,6 +449,8 @@ public:
         switch ((int)dt) {
 #define CASE(x) \
     case data_type::x: kind_ = type_kind_t::x; break;
+            CASE(f4_e3m0);
+            CASE(f4_e2m1);
             CASE(f8_e5m2);
             CASE(f8_e4m3);
             CASE(bf16);
@@ -438,8 +461,10 @@ public:
             CASE(s32);
             CASE(s8);
             CASE(u8);
+            CASE(s4);
+            CASE(u4);
 #undef CASE
-            default: ir_error_not_expected();
+            default: gpu_error_not_expected();
         }
     }
 
@@ -465,7 +490,7 @@ public:
     static void init_parse_iface(parse_iface_t<type_t> *iface) {
         iface->add<type_kind_t, &type_t::kind_>();
         iface->set_pre_stringify_func([](const type_t &type) {
-            ir_assert(!type.is_ptr() && (type.is_scalar() || type.is_undef()))
+            gpu_assert(!type.is_ptr() && (type.is_scalar() || type.is_undef()))
                     << "Cannot stringify pointer/non-scalar type.";
         });
     }
@@ -477,15 +502,13 @@ public:
     bool is_bool() const { return kind() == type_kind_t::_bool; }
 
     bool is_fp() const {
-        return utils::one_of(kind(), type_kind_t::bf8, type_kind_t::hf8,
-                type_kind_t::bf16, type_kind_t::f16, type_kind_t::tf32,
-                type_kind_t::f32, type_kind_t::f64);
-    }
-
-    bool is_fp8() const {
-        return utils::one_of(kind(), type_kind_t::bf8, type_kind_t::hf8);
+        return is_fp4() || is_fp8()
+                || utils::one_of(kind(), type_kind_t::bf16, type_kind_t::f16,
+                        type_kind_t::tf32, type_kind_t::f32, type_kind_t::f64);
     }
 
+    bool is_f4_e3m0() const { return kind() == type_kind_t::f4_e3m0; }
+    bool is_f4_e2m1() const { return kind() == type_kind_t::f4_e2m1; }
     bool is_bf8() const { return kind() == type_kind_t::bf8; }
     bool is_hf8() const { return kind() == type_kind_t::hf8; }
     bool is_bf16() const { return kind() == type_kind_t::bf16; }
@@ -494,35 +517,32 @@ public:
     bool is_f32() const { return kind() == type_kind_t::f32; }
     bool is_f64() const { return kind() == type_kind_t::f64; }
 
+    bool is_fp4() const { return is_f4_e3m0() || is_f4_e2m1(); }
+    bool is_fp8() const { return is_bf8() || is_hf8(); }
+
     bool is_int() const {
-        return utils::one_of(kind(), type_kind_t::u8, type_kind_t::s8,
-                type_kind_t::u16, type_kind_t::s16, type_kind_t::u32,
-                type_kind_t::s32, type_kind_t::u64, type_kind_t::s64);
+        return is_x4() || is_x8() || is_x16() || is_x32() || is_x64();
     }
 
+    bool is_s4() const { return kind() == type_kind_t::s4; }
+    bool is_u4() const { return kind() == type_kind_t::u4; }
+    bool is_x4() const { return is_s4() || is_u4(); }
+
     bool is_s8() const { return kind() == type_kind_t::s8; }
     bool is_u8() const { return kind() == type_kind_t::u8; }
-    bool is_x8() const {
-        return utils::one_of(kind(), type_kind_t::s8, type_kind_t::u8);
-    }
+    bool is_x8() const { return is_s8() || is_u8(); }
 
     bool is_s16() const { return kind() == type_kind_t::s16; }
     bool is_u16() const { return kind() == type_kind_t::u16; }
-    bool is_x16() const {
-        return utils::one_of(kind(), type_kind_t::s16, type_kind_t::u16);
-    }
+    bool is_x16() const { return is_s16() || is_u16(); }
 
     bool is_s32() const { return kind() == type_kind_t::s32; }
     bool is_u32() const { return kind() == type_kind_t::u32; }
-    bool is_x32() const {
-        return utils::one_of(kind(), type_kind_t::s32, type_kind_t::u32);
-    }
+    bool is_x32() const { return is_s32() || is_u32(); }
 
     bool is_s64() const { return kind() == type_kind_t::s64; }
     bool is_u64() const { return kind() == type_kind_t::u64; }
-    bool is_x64() const {
-        return utils::one_of(kind(), type_kind_t::s64, type_kind_t::u64);
-    }
+    bool is_x64() const { return is_s64() || is_u64(); }
 
     bool is_byte() const { return kind() == type_kind_t::byte; }
     bool is_dword() const { return kind() == type_kind_t::dword; }
@@ -532,14 +552,14 @@ public:
 
     bool is_signed(int elems = -1) const {
         if (elems != -1 && elems_ != elems) return false;
-        return utils::one_of(kind(), type_kind_t::s8, type_kind_t::s16,
-                type_kind_t::s32, type_kind_t::s64);
+        return utils::one_of(kind(), type_kind_t::s4, type_kind_t::s8,
+                type_kind_t::s16, type_kind_t::s32, type_kind_t::s64);
     }
 
     bool is_unsigned(int elems = -1) const {
         if (elems != -1 && elems_ != elems) return false;
-        return utils::one_of(kind(), type_kind_t::u8, type_kind_t::u16,
-                type_kind_t::u32, type_kind_t::u64);
+        return utils::one_of(kind(), type_kind_t::u4, type_kind_t::u8,
+                type_kind_t::u16, type_kind_t::u32, type_kind_t::u64);
     }
 
     bool is_scalar() const { return elems() == 1; }
@@ -584,6 +604,22 @@ public:
     // Returns size in bytes.
     int size() const;
 
+    // Returns size in bits.
+    int bitsize() const {
+        // 8 elements occupy the same number of bytes that a single element
+        // occupies in bits.
+        constexpr int bits_per_byte = 8;
+        return with_elems(bits_per_byte * elems()).size();
+    }
+
+    // Returns number of elements that fit in `size()` bytes.
+    // The size in bytes of `n` packed elements is
+    //     `div_up(n * size(), packing())`.
+    int packing() const {
+        constexpr int bits_per_byte = 8;
+        return bits_per_byte * size() / bitsize();
+    }
+
     std::string str() const {
         std::ostringstream oss;
         oss << to_string(kind());
@@ -608,6 +644,8 @@ class ref_count_t {
 public:
     ref_count_t() : value_(0) {}
     ref_count_t(const ref_count_t &) = delete;
+    ref_count_t &operator=(const ref_count_t &) = delete;
+    ~ref_count_t() = default;
 
     uint32_t increment() { return ++value_; }
     uint32_t decrement() { return --value_; }
@@ -621,9 +659,12 @@ class object_t;
 class ir_mutator_t;
 class ir_visitor_t;
 
-#define HANDLE_IR_OBJECT(type) class type;
+// clang-tidy doesn't like the semicolon next to the class name.
+#define CLASS_DECLARATION(name) class name
+#define HANDLE_IR_OBJECT(type) CLASS_DECLARATION(type);
 HANDLE_TRAVERSE_TARGETS()
 #undef HANDLE_IR_OBJECT
+#undef CLASS_DECLARATION
 
 // Base class for all IR objects. Implemented as an intrusive pointer, with
 // the reference counter stored inside the object.
@@ -632,6 +673,7 @@ public:
     object_impl_t(type_info_t type_info) : type_info_(type_info) {};
 
     object_impl_t(const object_impl_t &) = delete;
+    object_impl_t &operator=(const object_impl_t &) = delete;
 
     virtual ~object_impl_t() = default;
 
@@ -660,7 +702,7 @@ public:
     //       necessary, and please don't add a non-const variant of the method!
     template <typename T>
     const T &as() const {
-        ir_assert(is<T>());
+        gpu_assert(is<T>());
         return *as_ptr<T>(); // fails on incorrect casts even in Release
     }
 
@@ -751,7 +793,7 @@ public:
 
     template <typename T>
     const T &as() const {
-        ir_assert(impl_);
+        gpu_assert(impl_);
         return impl_->as<T>();
     }
 
@@ -871,6 +913,7 @@ public:
     template <typename T>
     std::vector<T> mutate(const std::vector<T> &v) {
         std::vector<T> new_v;
+        new_v.reserve(v.size());
         for (auto &e : v)
             new_v.push_back(mutate(e));
         return new_v;
@@ -878,7 +921,7 @@ public:
 
     // To catch missing _mutate() handlers in ir_mutator_t.
     object_t _mutate(const object_impl_t &obj) {
-        ir_error_not_expected() << "Can't handle type: " << object_t(&obj);
+        gpu_error_not_expected() << "Can't handle type: " << object_t(&obj);
         return {};
     }
 
@@ -912,7 +955,7 @@ public:
 
     // To catch missing _visit() handlers in ir_visitor_t.
     void _visit(const object_impl_t &obj) {
-        ir_error_not_expected() << "Can't handle type: " << object_t(obj);
+        gpu_error_not_expected() << "Can't handle type: " << object_t(obj);
     }
 
 #define HANDLE_IR_OBJECT(type) virtual void _visit(const type &obj);
@@ -959,7 +1002,7 @@ public:
     expr_t(uint64_t v);
 
     const type_t &type() const {
-        ir_assert(!is_empty());
+        gpu_assert(!is_empty());
         return ((const expr_impl_t *)impl())->type;
     }
 
@@ -982,7 +1025,7 @@ public:
 private:
 #ifdef SANITY_CHECK
     void sanity_check() const override {
-        ir_assert(dynamic_cast<const expr_impl_t *>(impl()) == impl())
+        gpu_assert(dynamic_cast<const expr_impl_t *>(impl()) == impl())
                 << object_t(impl());
     }
 #endif
@@ -1187,7 +1230,7 @@ private:
     cast_t(const type_t &type, const expr_t &expr, bool saturate)
         : expr_impl_t(_type_info(), type), expr(expr), saturate(saturate) {
         if (!is_bool_vec_u16()) {
-            ir_assert(type.elems() == expr.type().elems())
+            gpu_assert(type.elems() == expr.type().elems())
                     << "Number of elements must match.";
         }
     }
@@ -1450,8 +1493,8 @@ private:
         , off(_off)
         , stride(_stride) {
         normalize_ptr(type, buf, off);
-        ir_assert(is_var(buf)) << buf;
-        ir_assert(buf.type().is_ptr()) << buf;
+        gpu_assert(is_var(buf)) << buf;
+        gpu_assert(buf.type().is_ptr()) << buf;
         if (stride == type.scalar().size()) stride = default_stride;
     }
 };
@@ -1544,7 +1587,7 @@ inline const expr_t &get_base(const expr_t &e) {
     if (e.is_empty()) return e;
     if (e.is<var_t>()) return e;
     if (e.is<ptr_t>()) return e.as<ptr_t>().base;
-    ir_error_not_expected() << e;
+    gpu_error_not_expected() << e;
     return e;
 }
 
@@ -1584,7 +1627,7 @@ public:
 
     static expr_t make_broadcast(const expr_t &expr, dim_t elems) {
         if (elems == 1) return expr;
-        ir_assert(expr.type().is_scalar()) << expr;
+        gpu_assert(expr.type().is_scalar()) << expr;
         return make({expr}, std::vector<int>(elems, 0));
     }
 
@@ -1592,9 +1635,9 @@ public:
     // (S[beg], S[beg + 1], ..., S[end - 1]) vector.
     static expr_t make(const expr_t &_shuffle, int beg, int end) {
         auto &shuffle = _shuffle.as<shuffle_t>();
-        ir_assert(beg >= 0 && beg <= shuffle.elems());
-        ir_assert(end >= 0 && end <= shuffle.elems());
-        ir_assert(beg < end);
+        gpu_assert(beg >= 0 && beg <= shuffle.elems());
+        gpu_assert(end >= 0 && end <= shuffle.elems());
+        gpu_assert(beg < end);
         std::vector<expr_t> vec;
         std::vector<int> idx(end - beg, -1);
         for (int i = beg; i < end; i++) {
@@ -1639,19 +1682,19 @@ private:
         : expr_impl_t(_type_info(), shuffle_type(vec, idx))
         , vec(vec)
         , idx(idx) {
-        ir_assert(idx.size() > 1) << "Unexpected empty or scalar shuffle.";
+        gpu_assert(idx.size() > 1) << "Unexpected empty or scalar shuffle.";
     }
 
     static type_t shuffle_type(
             const std::vector<expr_t> &vec, const std::vector<int> &idx) {
-        ir_assert(!vec.empty() && !idx.empty());
+        gpu_assert(!vec.empty() && !idx.empty());
 
         auto elem_type = vec[0].type();
         for (auto &v : vec)
             elem_type = common_type(elem_type, v.type());
 
         for (size_t i = 0; i < idx.size(); i++) {
-            ir_assert(idx[i] >= 0 && idx[i] < int(vec.size()))
+            gpu_assert(idx[i] >= 0 && idx[i] < int(vec.size()))
                     << "Incorrect index.";
             MAYBE_UNUSED(i);
         }
@@ -1708,6 +1751,11 @@ inline expr_t ternary_add3(const expr_t &a, const expr_t &b, const expr_t &c) {
     return ternary_op_t::make(op_kind_t::_add3, a, b, c);
 }
 
+inline expr_t ternary_idiv(
+        const expr_t &a, const expr_t &b, const expr_t &magic) {
+    return ternary_op_t::make(op_kind_t::_idiv, a, b, magic);
+}
+
 // Unary operation: (op a).
 class unary_op_t : public expr_impl_t {
 public:
@@ -1782,7 +1830,7 @@ expr_t to_expr(T value, const type_t &type) {
 
 #undef CASE
 
-    ir_error_not_expected() << type;
+    gpu_error_not_expected() << type;
 
     return expr_t();
 }
@@ -1834,13 +1882,13 @@ inline bool is_var(const expr_t &e) {
 // Convertor from IR expression to C++ constant.
 template <typename T>
 T to_cpp(const expr_t &e) {
-    ir_assert(is_const(e)) << "Expression must be constant.";
+    gpu_assert(is_const(e)) << "Expression must be constant.";
 
     if (e.is<int_imm_t>()) return (T)e.as<int_imm_t>().value;
     if (e.is<float_imm_t>()) return (T)e.as<float_imm_t>().value;
     if (e.is<bool_imm_t>()) return (T)e.as<bool_imm_t>().value;
 
-    ir_error_not_expected();
+    gpu_error_not_expected();
     return 0;
 }
 
@@ -1849,6 +1897,7 @@ inline int to_int(const expr_t &e) {
 }
 
 expr_t operator-(const expr_t &a);
+expr_t div_up(const expr_t &a, const expr_t &b);
 
 #define DECLARE_BINARY_OPERATOR(op, op_kind) \
     expr_t operator op(const expr_t &a, const expr_t &b);
@@ -1907,7 +1956,7 @@ public:
 private:
 #ifdef SANITY_CHECK
     void sanity_check() const override {
-        ir_assert(dynamic_cast<const stmt_impl_t *>(impl()) == impl())
+        gpu_assert(dynamic_cast<const stmt_impl_t *>(impl()) == impl())
                 << object_t(impl());
     }
 #endif
@@ -1944,7 +1993,7 @@ public:
 private:
 #ifdef SANITY_CHECK
     void sanity_check() const override {
-        ir_assert(dynamic_cast<const alloc_attr_impl_t *>(impl()) == impl())
+        gpu_assert(dynamic_cast<const alloc_attr_impl_t *>(impl()) == impl())
                 << object_t(impl());
     }
 #endif
@@ -2069,7 +2118,7 @@ public:
     const T &get_attr() const {
         for (auto &a : attrs)
             if (a.is<T>()) return a.as<T>();
-        ir_error_not_expected() << "Can't find attribute.";
+        gpu_error_not_expected() << "Can't find attribute.";
         return attrs[0].as<T>();
     }
 
@@ -2079,6 +2128,12 @@ public:
                 : 0;
     }
 
+    std::string line_str() const {
+        std::ostringstream out;
+        out << "alloc " << buf.as<var_t>().name << "[" << size << "]";
+        return out.str();
+    }
+
     IR_DECLARE_TRAVERSERS()
 
     expr_t buf;
@@ -2096,7 +2151,7 @@ private:
         , kind(kind)
         , attrs(attrs)
         , body(body) {
-        ir_assert(buf.type().is_ptr()) << buf;
+        gpu_assert(buf.type().is_ptr()) << buf;
     }
 };
 
@@ -2152,6 +2207,17 @@ public:
 
     bool has_default_stride() const { return stride == default_stride; }
 
+    std::string line_str() const {
+        std::ostringstream out;
+        out << load_t::make(value.type(), buf, off, stride);
+        out << " = " << value;
+        if (!mask.is_empty()) {
+            out << ", mask = " << mask.str();
+            if (fill_mask0) out << " [FILL]";
+        }
+        return out.str();
+    }
+
     IR_DECLARE_TRAVERSERS()
 
     static const int default_stride = -1;
@@ -2174,11 +2240,11 @@ private:
         , mask(_mask)
         , fill_mask0(_fill_mask0) {
         normalize_ptr(value.type(), buf, off);
-        ir_assert(is_var(buf)) << buf;
-        ir_assert(buf.type().is_ptr()) << buf;
+        gpu_assert(is_var(buf)) << buf;
+        gpu_assert(buf.type().is_ptr()) << buf;
         if (stride == value.type().scalar().size()) stride = default_stride;
         if (!mask.is_empty())
-            ir_assert(mask.type() == type_t::_bool(value.type().elems()));
+            gpu_assert(mask.type() == type_t::_bool(value.type().elems()));
     }
 };
 
@@ -2211,6 +2277,14 @@ public:
         return ir_utils::get_hash(var, init, bound, body, step, unroll);
     }
 
+    std::string line_str() const {
+        std::ostringstream out;
+        out << "for (" << var << " = " << init << "; " << var << " < " << bound
+            << "; " << var << " += " << step << ") ";
+        if (unroll != 1) out << "[unroll: " << unroll << "] ";
+        return out.str();
+    }
+
     IR_DECLARE_TRAVERSERS()
 
     expr_t var;
@@ -2260,6 +2334,12 @@ public:
         return ir_utils::get_hash(cond, body, else_body);
     }
 
+    std::string line_str() const {
+        std::ostringstream oss;
+        oss << "if (" << cond << ")";
+        return oss.str();
+    }
+
     IR_DECLARE_TRAVERSERS()
 
     expr_t cond;
@@ -2308,6 +2388,12 @@ public:
         return utils::rnd_up(var.type().size(), reg_allocator_t::granularity);
     };
 
+    std::string line_str() const {
+        std::ostringstream out;
+        out << var << "." << var.type() << " = " << value;
+        return out.str();
+    }
+
     IR_DECLARE_TRAVERSERS()
 
     expr_t var;
@@ -2318,7 +2404,7 @@ private:
     let_t(const expr_t &var, const expr_t &value, const stmt_t &body)
         : stmt_impl_t(_type_info()), var(var), value(value), body(body) {
         if (!value.is_empty() && !is_const(value))
-            ir_assert(var.type() == value.type())
+            gpu_assert(var.type() == value.type())
                     << "Variable " << var << " and  value " << value
                     << "have different types. " << var.type()
                     << " != " << value.type() << "\n";
@@ -2389,7 +2475,7 @@ public:
             CASE(prefetch);
             CASE(mul);
 #undef CASE
-            default: ir_error_not_expected();
+            default: gpu_error_not_expected();
         }
         return {};
     }
@@ -2502,6 +2588,12 @@ public:
 
     size_t get_hash() const override { return ir_utils::get_hash(cond, body); }
 
+    std::string line_str() const {
+        std::ostringstream out;
+        out << "while (" << cond << ")";
+        return out.str();
+    }
+
     IR_DECLARE_TRAVERSERS()
 
     expr_t cond;
@@ -2541,7 +2633,8 @@ public:
 private:
 #ifdef SANITY_CHECK
     void sanity_check() const override {
-        ir_assert(dynamic_cast<const func_call_attr_impl_t *>(impl()) == impl())
+        gpu_assert(
+                dynamic_cast<const func_call_attr_impl_t *>(impl()) == impl())
                 << object_t(impl());
     }
 #endif
@@ -2597,12 +2690,12 @@ public:
     func_impl_t(type_info_t type_info) : object_impl_t(type_info) {}
 
     size_t get_hash() const override {
-        ir_error_not_expected() << "get_hash() is not implemented.";
+        gpu_error_not_expected() << "get_hash() is not implemented.";
         return 0;
     }
 
     bool is_equal(const object_impl_t &obj) const override {
-        ir_error_not_expected() << "is_equal() is not implemented.";
+        gpu_error_not_expected() << "is_equal() is not implemented.";
         return false;
     }
 
@@ -2637,7 +2730,7 @@ public:
 private:
 #ifdef SANITY_CHECK
     void sanity_check() const override {
-        ir_assert(dynamic_cast<const func_impl_t *>(impl()) == impl())
+        gpu_assert(dynamic_cast<const func_impl_t *>(impl()) == impl())
                 << object_t(impl());
     }
 #endif
@@ -2663,6 +2756,13 @@ public:
 
     size_t get_hash() const override { return ir_utils::get_hash(args, attr); }
 
+    std::string line_str() const {
+        std::ostringstream out;
+        out << func << "(" << ir_utils::make_seq_print_helper(args) << ")";
+        if (!attr.is_empty()) out << " " << attr;
+        return out.str();
+    }
+
     IR_DECLARE_TRAVERSERS()
 
     func_t func;
@@ -2673,7 +2773,7 @@ private:
     func_call_t(const func_t &func, const std::vector<expr_t> &args,
             const func_call_attr_t &attr)
         : stmt_impl_t(_type_info()), func(func), args(args), attr(attr) {
-        ir_assert(!func.is_empty());
+        gpu_assert(!func.is_empty());
     }
 };
 
@@ -2684,7 +2784,7 @@ inline stmt_t func_impl_t::call(
 
 inline stmt_t func_call_attr_t::apply_to(const stmt_t &s) const {
     auto &c = s.as<func_call_t>();
-    ir_assert(c.attr.is_empty())
+    gpu_assert(c.attr.is_empty())
             << "Merging of attributes is not supported: " << s;
     return func_call_t::make(c.func, c.args, *this);
 }
