@@ -117,7 +117,7 @@ public:
         }
         // Ensure large GRF buffers are aligned to a register boundary.
         if (!entry_ptr->is_slm() && entry_ptr->size > ir_ctx_->grf_size()) {
-            ir_assert(entry_ptr->size % ir_ctx_->grf_size() == 0);
+            gpu_assert(entry_ptr->size % ir_ctx_->grf_size() == 0);
         }
         return entry_ptr->buf;
     }
@@ -129,7 +129,7 @@ public:
     entry_t find(const std::string &name, bool allow_empty = false) const {
         auto it = entries_.find(name);
         if (it != entries_.end()) return it->second;
-        if (!allow_empty) ir_error_not_expected() << "Not found: " << name;
+        if (!allow_empty) gpu_error_not_expected() << "Not found: " << name;
         return entry_t();
     }
 
@@ -139,7 +139,7 @@ public:
 
     const entry_t &find_ref(const std::string &name) const {
         auto it = entries_.find(name);
-        ir_assert(it != entries_.end());
+        gpu_assert(it != entries_.end());
         return it->second;
     }
 
@@ -149,7 +149,7 @@ public:
 
     entry_t &find_ref(const std::string &name) {
         auto it = entries_.find(name);
-        ir_assert(it != entries_.end());
+        gpu_assert(it != entries_.end());
         return it->second;
     }
 
@@ -194,19 +194,19 @@ class alloc_updater_t : public ir_mutator_t {
 public:
     void resize(const expr_t &buf, int new_size) {
         auto ret = resizes_.insert({buf, new_size});
-        ir_assert(ret.second) << buf;
+        gpu_assert(ret.second) << buf;
         MAYBE_UNUSED(ret);
     }
 
     void add_attr(const expr_t &buf, const alloc_attr_t &attr) {
         auto ret = attrs_.insert({buf, attr});
-        ir_assert(ret.second) << buf;
+        gpu_assert(ret.second) << buf;
         MAYBE_UNUSED(ret);
     }
 
     void remove(const expr_t &buf) {
         auto ret = removes_.insert(buf);
-        ir_assert(ret.second) << buf;
+        gpu_assert(ret.second) << buf;
         MAYBE_UNUSED(ret);
     }
 
@@ -223,7 +223,7 @@ public:
             } else if (!new_stmt.is_same(old_stmt)) {
                 auto &new_a = new_stmt.as<alloc_t>();
                 auto &entry = buf_mgr.find_ref(e.buf);
-                ir_assert(entry.attrs.empty());
+                gpu_assert(entry.attrs.empty());
                 entry.size = new_a.size;
                 entry.attrs = new_a.attrs;
                 continue;
@@ -326,6 +326,7 @@ struct expr_cast_helper_t {
 
     static std::vector<T> call(const std::vector<expr_t> &exprs) {
         std::vector<T> ret;
+        ret.reserve(exprs.size());
         for (auto &e : exprs)
             ret.push_back(to_cpp<T>(e));
         return ret;
@@ -345,6 +346,7 @@ struct expr_cast_helper_t<expr_t> {
             = typename std::enable_if<std::is_arithmetic<U>::value>::type>
     static std::vector<expr_t> call(const std::vector<U> &vec) {
         std::vector<expr_t> ret;
+        ret.reserve(vec.size());
         for (auto &v : vec)
             ret.push_back(to_expr(v));
         return ret;
@@ -378,6 +380,7 @@ std::vector<std::pair<KeyT, ValueT>> sort_var_map(
         const std::unordered_map<KeyT, ValueT, HashT, EqualT> &map,
         const CompareT &compare) {
     std::vector<std::pair<KeyT, ValueT>> ret;
+    ret.reserve(map.size());
     for (auto &kv : map)
         ret.emplace_back(kv);
     std::sort(ret.begin(), ret.end(), compare);
@@ -395,6 +398,17 @@ std::vector<std::pair<KeyT, expr_t>> sort_var_map_by_value(
             });
 }
 
+template <typename ValueT, typename HashT, typename EqualT>
+std::vector<std::pair<expr_t, ValueT>> sort_var_map_by_key(
+        const std::unordered_map<expr_t, ValueT, HashT, EqualT> &map) {
+    return sort_var_map(map,
+            [](const std::pair<expr_t, ValueT> &a,
+                    const std::pair<expr_t, ValueT> &b) {
+                return a.first.template as<var_t>().name
+                        < b.first.template as<var_t>().name;
+            });
+}
+
 class alloc_manager_t {
 public:
     alloc_manager_t(const stmt_t &root) {
@@ -415,7 +429,7 @@ public:
             auto &a = _a.as<alloc_t>();
             auto ret = buf2alloc_.insert({a.buf, _a});
             buffers_.push_back(a.buf);
-            ir_assert(ret.second) << "Buffer already exists: " << a.buf;
+            gpu_assert(ret.second) << "Buffer already exists: " << a.buf;
             MAYBE_UNUSED(ret);
         }
         // Sort buffers by name.
@@ -443,13 +457,13 @@ public:
 
     uint32_t alloc_size(const expr_t &buf) const {
         auto *a = find_alloc(buf);
-        ir_assert(a) << buf;
+        gpu_assert(a) << buf;
         return a->size;
     }
 
     alloc_kind_t alloc_kind(const expr_t &buf) const {
         auto *a = find_alloc(buf);
-        ir_assert(a) << buf;
+        gpu_assert(a) << buf;
         return a->kind;
     }
 
@@ -642,7 +656,7 @@ public:
     void pop() { path_.pop_back(); }
 
     const object_impl_t *back() const {
-        ir_assert(!is_empty());
+        gpu_assert(!is_empty());
         return path_.back();
     }
 
@@ -787,7 +801,7 @@ public:
     }
 
     bool implies(const modulus_info_t &other) const {
-        ir_assert(var().is_same(other.var()));
+        gpu_assert(var().is_same(other.var()));
 
         int64_t this_mod = to_cpp<int64_t>(mod());
         int64_t other_mod = to_cpp<int64_t>(other.mod());
@@ -846,7 +860,7 @@ public:
         : relations_(relations) {}
 
     int64_t get_var_bound(const expr_t &e, bool is_low) const override {
-        ir_assert(is_var(e));
+        gpu_assert(is_var(e));
         int64_t def_bound = unlimited_bound(is_low);
         auto it = relations_.find(e);
         if (it == relations_.end()) return def_bound;
@@ -891,7 +905,7 @@ public:
     std::string str() const {
         std::ostringstream oss;
         oss << "relations:" << (relations_.empty() ? " (empty)\n" : "\n");
-        for (auto &r : relations_) {
+        for (auto &r : sort_var_map_by_key(relations_)) {
             oss << "\t" << r.first << ":";
             bool first = true;
             for (auto &s : r.second) {
@@ -903,7 +917,7 @@ public:
 
         oss << "modulus_info:"
             << (modulus_infos_.empty() ? " (empty)\n" : "\n");
-        for (auto &m : modulus_infos_) {
+        for (auto &m : sort_var_map_by_key(modulus_infos_)) {
             oss << "\t" << m.first << ":";
             bool first = true;
             for (auto &s : m.second) {
@@ -958,12 +972,12 @@ private:
             case op_kind_t::_le: break;
             case op_kind_t::_gt:
                 op_kind = op_kind_t::_ge;
-                ir_assert(b < std::numeric_limits<int64_t>::max());
+                gpu_assert(b < std::numeric_limits<int64_t>::max());
                 b += 1;
                 break;
             case op_kind_t::_lt:
                 op_kind = op_kind_t::_le;
-                ir_assert(b > std::numeric_limits<int64_t>::min());
+                gpu_assert(b > std::numeric_limits<int64_t>::min());
                 b -= 1;
                 break;
             default: return false;
