@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -31,28 +31,10 @@ namespace gpu {
 namespace intel {
 namespace jit {
 
-std::ostream &operator<<(std::ostream &out, const send_op_t op) {
-    const char *s = nullptr;
-    switch (op) {
-        case send_op_t::atomic_fadd: s = "atomic_fadd"; break;
-        case send_op_t::atomic_cmpwr: s = "atomic_cmpwr"; break;
-        case send_op_t::load: s = "load"; break;
-        case send_op_t::load_2d: s = "load_2d"; break;
-        case send_op_t::prefetch: s = "prefetch"; break;
-        case send_op_t::prefetch_2d: s = "prefetch_2d"; break;
-        case send_op_t::store: s = "store"; break;
-        case send_op_t::store_2d: s = "store_2d"; break;
-        case send_op_t::undef: s = "undef"; break;
-        default: ir_error_not_expected(); s = "unknown";
-    }
-
-    return out << s;
-}
-
 stmt_t send_t::create_offset_store(const expr_t &header_buf,
         const expr_t &mem_buf, const expr_t &_mem_off,
         bool is_signed_offset) const {
-    ir_assert(is_var(mem_buf));
+    gpu_assert(is_var(mem_buf));
     int header_off = 0;
     int unit_size = 1;
     if (!is_lsc && is_block() && is_slm()) {
@@ -226,9 +208,8 @@ class memory_walker_t {
 public:
     memory_walker_t(const constraint_set_t &cset, const view_t &view)
         : view_(view)
-        , type_size_(view.type().size())
         , mask_tensor_(view.create_mask_tensor(cset).reinterpret(view.type()))
-        , full_size_(view.velems() * type_size_) {
+        , full_size_(utils::div_up(view.velems() * view.type().bitsize(), 8)) {
         init_dense_blocks(cset);
         reset();
     }
@@ -244,7 +225,9 @@ public:
 
     int remaining_size() const { return remaining_size_; }
 
-    int remaining_elems() const { return remaining_size_ / type_size_; }
+    int remaining_elems() const {
+        return remaining_size_ * type().packing() / type().size();
+    }
 
     bool is_dense_and_aligned(int off, int size, int alignment) const {
         if (off + size > remaining_size_) return false;
@@ -262,7 +245,8 @@ public:
             int off = i * slot_size;
             // Overflow is fine, expect it to be handled by proper masking.
             if (off >= remaining_size_) return true;
-            if ((slot_size * slots) % type_size_ != 0) return false;
+            if ((slot_size * slots * type().packing()) % type().size() != 0)
+                return false;
             if (!is_dense_and_aligned(off, slot_size, alignment)) return false;
         }
         return true;
@@ -282,7 +266,7 @@ public:
             return base;
         }
         int block_idx = get_block_index(cur_off_ + off);
-        ir_assert(block_idx >= 0 && block_idx < int(block_offs_.size()));
+        gpu_assert(block_idx >= 0 && block_idx < int(block_offs_.size()));
         base = block_offs_[block_idx];
         auto prev_base = block_offs_[block_idx == 0 ? 0 : block_idx - 1];
         auto get_const_summand = [&](const expr_t &expr) -> int64_t {
@@ -308,30 +292,32 @@ public:
     // Returns a boolean mask expression for the given region to access.
     expr_t get_mask(int off, int size, int mask_size, int nmasks,
             bool allow_fail = false) const {
-        ir_assert(size % mask_size == 0) << "Incompatible mask size.";
+        gpu_assert(size % mask_size == 0) << "Incompatible mask size.";
         auto sub_mask_tensor = create_sub_mask_tensor(off, size);
         sub_mask_tensor = sub_mask_tensor.reinterpret(type_t::u8(mask_size));
         if (sub_mask_tensor.is_empty()) {
             if (allow_fail) return expr_t();
-            ir_error_not_expected();
+            gpu_error_not_expected();
         }
         auto ret = sub_mask_tensor.to_expr(nmasks);
         if (ret.is_empty()) {
             if (allow_fail) return expr_t();
-            ir_error_not_expected() << "Can't create mask.";
+            gpu_error_not_expected() << "Can't create mask.";
         }
         return ret;
     }
 
     // Moves the current position `size` bytes ahead.
     void advance(int size) {
-        ir_assert(size % type_size_ == 0);
+        gpu_assert((size * type().packing()) % type().size() == 0);
         size = std::min(size, remaining_size_);
         cur_off_ += size;
         remaining_size_ -= size;
     }
 
 private:
+    const type_t &type() const { return view_.type(); }
+
     void init_dense_blocks(const constraint_set_t &cset) {
         auto l = view_.create_pseudo_vlayout();
         // Find the maximum innermost dense tile.
@@ -343,7 +329,7 @@ private:
             stride = b.block * b.stride;
         }
         tensor_t tile(dims);
-        dense_block_size_ = tile.elems() * type_size_;
+        dense_block_size_ = tile.elems() * type().size() / type().packing();
         // Split the memory view into dense blocks and precompute block offsets
         // and alignments.
         view_.for_each_tile(tile, [&](const std::vector<dim_t> &start) {
@@ -354,20 +340,21 @@ private:
             int64_t f = get_max_const_factor(off, cset);
             int alignment = f ? ir_utils::max_pow2_divisor(f) : base_alignment;
 
-            block_offs_.push_back(off);
+            block_offs_.push_back(std::move(off));
             block_alignments_.push_back(alignment);
         });
     }
 
     mask_tensor_t create_sub_mask_tensor(int off, int size) const {
-        ir_assert(off % type_size_ == 0);
-        ir_assert(size % type_size_ == 0);
+        gpu_assert((off * type().packing()) % type().size() == 0);
+        gpu_assert((size * type().packing()) % type().size() == 0);
 
-        std::vector<dim_t> sub_dims = {size / type_size_};
-        layout_t sub_layout(view_.type(), 0, sub_dims);
+        std::vector<dim_t> sub_dims
+                = {(dim_t)size * type().packing() / type().size()};
+        layout_t sub_layout(type(), 0, sub_dims);
         mask_tensor_t sub_mask_tensor(sub_layout);
-        int beg = (cur_off_ + off) / type_size_;
-        int end = (cur_off_ + off + size) / type_size_;
+        int beg = (cur_off_ + off) * type().packing() / type().size();
+        int end = (cur_off_ + off + size) * type().packing() / type().size();
         for (int i = beg; i < end; i++) {
             auto mask = (i < mask_tensor_.elems()) ? mask_tensor_.mask(i)
                                                    : expr_t(false);
@@ -385,7 +372,6 @@ private:
     }
 
     view_t view_;
-    int type_size_;
     mask_tensor_t mask_tensor_;
     std::vector<expr_t> block_offs_;
     std::vector<int> block_alignments_;
@@ -399,10 +385,7 @@ class layout_walker_t {
 public:
     layout_walker_t() = default;
     layout_walker_t(const layout_t &layout, int grf_size)
-        : layout_(layout)
-        , grf_size_(grf_size)
-        , type_size_(layout.type().size())
-        , idxs_(layout.blocks().size()) {}
+        : layout_(layout), grf_size_(grf_size), idxs_(layout.blocks().size()) {}
 
     int offset_bytes() const { return off_bytes_; }
 
@@ -419,12 +402,13 @@ public:
     // - The last element must not cross the layout boundary
     bool can_advance(int stride, int elems, bool is_last_region = false) {
         if (is_last_region) elems = std::min(elems, remaining_elems());
+        const auto stride_bytes
+                = utils::div_up(stride * type().size(), type().packing());
         auto cur_idxs = idxs_;
         int cur_off_bytes = off_bytes_;
         for (int i = 0; i < elems - 1; i++) {
             int next_off_bytes = advance(cur_idxs, cur_off_bytes);
-            if (next_off_bytes - cur_off_bytes != stride * type_size_)
-                return false;
+            if (next_off_bytes - cur_off_bytes != stride_bytes) return false;
             cur_off_bytes = next_off_bytes;
         }
         cur_off_bytes = advance(cur_idxs, cur_off_bytes);
@@ -443,6 +427,8 @@ public:
     }
 
 private:
+    const type_t &type() const { return layout_.type(); }
+
     int max_offset_bytes() const {
         return utils::rnd_up((int)layout_.size(), grf_size_);
     }
@@ -459,12 +445,11 @@ private:
             int stride = (int)layout_.blocks()[i].stride;
             off += idxs[i] * stride;
         }
-        return off * type_size_;
+        return utils::div_up(off * type().size(), type().packing());
     }
 
     layout_t layout_;
     int grf_size_;
-    int type_size_;
 
     std::vector<int> idxs_;
     int elems_ = 0;
@@ -516,11 +501,10 @@ void access_builder_t::build() {
     }
     if (!ok && send_op_ == send_op_t::prefetch) {
         // Do not treat as an error, skip prefetch messages during generation.
-        ir_warning() << "Can't generate send decomposition for prefetch."
-                     << std::endl;
+        gpu_warning() << "Can't generate send decomposition for prefetch.";
         return;
     }
-    ir_assert(ok) << "Can't generate send decomposition.";
+    gpu_assert(ok) << "Can't generate send decomposition.";
 }
 
 static bool stride_dimension_ok(const view_t &view, int stride_tidx,
@@ -555,7 +539,7 @@ static expr_t try_scalarize(const expr_t &e) {
         return binary_op_t::make(binary->op_kind, a, b);
     }
 
-    ir_error_not_expected() << e;
+    gpu_error_not_expected() << e;
     return expr_t();
 }
 
@@ -592,7 +576,7 @@ bool access_builder_t::try_build_2d(send_params_t &send_params) {
 
     auto &b0 = blocks[0];
     auto &b1 = blocks[1];
-    ir_assert(b0.dim_idx != b1.dim_idx);
+    gpu_assert(b0.dim_idx != b1.dim_idx);
     if (b0.stride != stride_t(1)) return false;
     if (!b1.stride.is_fixed()) return false;
 
@@ -602,7 +586,7 @@ bool access_builder_t::try_build_2d(send_params_t &send_params) {
             auto &tdim = mem_view_.tdim(i);
             for (dim_idx_t j = 0; j < tdim.nvargs(); j++) {
                 if (tdim.vidx(j) == vdim_idx) {
-                    ir_assert(ret == dim_idx::invalid);
+                    gpu_assert(ret == dim_idx::invalid);
                     stride = (int)tdim.vstride(j);
                     ret = i;
                 }
@@ -900,6 +884,8 @@ bool access_builder_t::check_2d_mask(const tensor_t &tile,
 bool access_builder_t::try_build(
         const layout_t &try_layout, memory_walker_t &mem_walker) {
     auto &try_layout_blocks = try_layout.blocks();
+    const int type_size = mem_type_.size();
+    const int type_packing = mem_type_.packing();
     int reg_stride
             = (try_layout_blocks.empty() ? 0
                                          : (int)try_layout_blocks[0].stride);
@@ -921,16 +907,17 @@ bool access_builder_t::try_build(
             int nmasks = s.nmasks();
             int payload_stride = s.payload_type_stride();
             int access_size = s.access_size();
-            int access_elems = access_size / mem_type_.size();
+            int access_elems = access_size * type_packing / type_size;
             bool is_last_chunk = mem_walker.remaining_size() <= access_size;
 
             if (reg_stride != 1 || payload_stride != slot_size) {
                 // Detected strided GRF layout or strided payload. In this
                 // case require full data type and stride match.
                 if (reg_stride != 0
-                        && payload_stride != reg_stride * mem_type_.size())
+                        && payload_stride * type_packing
+                                != reg_stride * type_size)
                     continue;
-                if (s.type.size() != mem_type_.size()) continue;
+                if (type_packing * s.type.size() != type_size) continue;
             }
             // Prefetches don't have payload so skip these conditions for
             // prefetch.
@@ -965,7 +952,8 @@ bool access_builder_t::try_build(
         send_stmt = try_promote_to_lsc(send_stmt);
         stmt_ = stmt_.append(send_stmt);
 
-        reg_layout_walker_->advance(send.access_size() / mem_type_.size());
+        reg_layout_walker_->advance(
+                send.access_size() * type_packing / type_size);
         mem_walker.advance(send.access_size());
     }
     reg_layout_ = try_layout;
@@ -984,7 +972,8 @@ std::vector<layout_t> access_builder_t::candidate_payload_layouts() const {
     // These payload layouts are to match payload for byte x {1,2} scattered
     // messages (they are dword-strided).
     if (type_size == 2) ret.push_back(vlayout.make_strided(2));
-    if (type_size == 1) ret.push_back(vlayout.make_strided(4));
+    if (type_size == 1 && mem_type_.bitsize() == 8)
+        ret.push_back(vlayout.make_strided(4));
 
     return ret;
 }
@@ -1009,7 +998,7 @@ stmt_t access_builder_t::create_send_stmt(
         } else if (!off_base.is_equal(off_base0)) {
             is_same_base = false;
         }
-        off_vec.push_back(off);
+        off_vec.push_back(std::move(off));
         off_const_vec.emplace_back(off_const - off_const0);
     }
     expr_t off;
@@ -1034,12 +1023,13 @@ stmt_t access_builder_t::create_send_stmt(
 
 static const int any_block = 0;
 
-send_2d_hint_t get_send_2d_hint(send_op_t send_op, const type_t &_type,
+send_2d_hint_t get_send_2d_hint(send_op_t send_op, const type_t &type,
         bool vnni, bool transpose, int w_tile, int h_tile,
         int w_blk = any_block, int h_blk = any_block) {
-    auto type = _type;
+    gpu_assert(!(vnni && transpose)) << "VNNI with transpose is not supported.";
 
-    ir_assert(!(vnni && transpose)) << "VNNI with transpose is not supported.";
+    // Disable sub-byte types for now.
+    if (type.packing() > 1) return send_2d_hint_t();
 
     // XXX: Convert transpose to VNNI when transpose is not
     // supported. This will require additional reorder but
@@ -1092,12 +1082,12 @@ send_2d_hint_t get_send_2d_hint(send_op_t send_op, const type_t &_type,
 
     if (vnni) {
         // TODO: Remove.
-        ir_assert(h_blk > 0);
+        gpu_assert(h_blk > 0);
         h_blk = find_block(h_tile, h_blk, h_max);
     }
     if (transpose && w_blk > 0) {
         // TODO: Remove.
-        ir_assert(w_blk > 0);
+        gpu_assert(w_blk > 0);
         w_blk = find_block(w_tile, w_blk, w_max);
     }
 
