@@ -28,8 +28,8 @@
 #include "gpu/intel/jit/utils/utils.hpp"
 #include "gpu/intel/jit/v2/conv/bridge.hpp"
 #include "gpu/intel/jit/v2/conv/builder.hpp"
+#include "gpu/intel/jit/v2/conv/debug.hpp"
 #include "gpu/intel/jit/v2/conv/kernel.hpp"
-#include "gpu/intel/jit/v2/conv/plan_preset.hpp"
 #include "gpu/intel/jit/v2/conv/plan_registry.hpp"
 
 namespace dnnl {
@@ -53,6 +53,16 @@ void maybe_init_layout(
     md = layout.to_dnnl(md.dims);
 }
 
+status_t init_default_layouts(convolution_pd_t *pd) {
+    auto &src_md = *const_cast<memory_desc_t *>(pd->invariant_src_md());
+    auto &dst_md = *const_cast<memory_desc_t *>(pd->invariant_dst_md());
+    if (src_md.format_kind == format_kind::any)
+        set_default_format(src_md, "axb");
+    if (dst_md.format_kind == format_kind::any)
+        set_default_format(dst_md, "axb");
+    return status::success;
+}
+
 status_t init_layouts(const kernel_desc_t &desc, convolution_pd_t *pd) {
     auto &src_md = *const_cast<memory_desc_t *>(pd->invariant_src_md());
     auto &wei_md = *const_cast<memory_desc_t *>(pd->invariant_wei_md());
@@ -72,10 +82,10 @@ bool has_large_buffers(const convolution_pd_t *pd) {
         gpu_assert(!mdw.format_any());
         return mdw.size() > size_t(std::numeric_limits<int32_t>::max());
     };
-    if (!is_large(*pd->invariant_src_md())) return true;
-    if (!is_large(*pd->invariant_wei_md())) return true;
-    if (!is_large(*pd->invariant_dst_md())) return true;
-    if (!is_large(*pd->invariant_bia_md())) return true;
+    if (is_large(*pd->invariant_src_md())) return true;
+    if (is_large(*pd->invariant_wei_md())) return true;
+    if (is_large(*pd->invariant_dst_md())) return true;
+    if (is_large(*pd->invariant_bia_md())) return true;
     auto &post_ops = pd->attr()->post_ops_;
     for (int i = 0; i < post_ops.len(); i++) {
         auto &e = post_ops.entry_[i];
@@ -97,11 +107,13 @@ public:
             return false;
         // Mixed types are not supported for backward by data.
         if (pd->is_bwd_d()
-                && pd->dst_md()->data_type != pd->diff_src_md()->data_type) {
+                && pd->diff_dst_md()->data_type
+                        != pd->diff_src_md()->data_type) {
             return false;
         }
         using sm = primitive_attr_t::skip_mask_t;
-        auto skip_mask = sm::post_ops | sm::sum_dt;
+        auto skip_mask
+                = sm::post_ops | sm::sum_dt | sm::scales | sm::scales_data_type;
         if (!pd->attr()->has_default_values(skip_mask)) return false;
         return true;
     }
@@ -117,19 +129,21 @@ public:
         if (!pd->set_default_alg_kind(alg_kind::convolution_direct))
             return status::unimplemented;
 
+        CHECK(init_default_layouts(pd));
+
         auto prb = to_problem(pd, engine);
         kernel_desc_t _desc;
-        if (plan_preset_t::instance().is_set()) {
-            _desc = plan_preset_t::instance().get();
+        if (debug_t::init_kernel_desc(_desc)) {
+            _desc.set_defaults();
         } else {
             auto &registry = const_plan_registry();
             _desc = registry.find_best(prb);
             if (_desc.is_empty()) {
-                ir_info() << "Cannot find kernels that can fit the problem.\n";
+                gpu_info() << "Cannot find kernels that can fit the problem.";
                 return status::unimplemented;
             }
         }
-        _desc.spec_strategy = spec_strategy_t::min_dims;
+        _desc.spec.mode = specialization_mode_t::min_dims;
         _desc.fit_to(prb);
         CHECK(init_layouts(_desc, pd));
         CHECK(pd->attr_.set_default_formats(out_md(pd)));
@@ -137,9 +151,9 @@ public:
         // Large buffer support is unimplemented.
         if (has_large_buffers(pd)) return status::unimplemented;
 
-        CHECK(_desc.set_post_ops(pd->attr()->post_ops_, out_md(pd), pd));
-        if (!finalize_conv_desc(_desc, prb)) {
-            ir_info() << "Cannot create kernel descriptor.\n";
+        CHECK(_desc.set_attr(pd, pd->attr(), out_md(pd)));
+        if (!create_conv_plan(_desc, prb)) {
+            gpu_info() << "Cannot create kernel descriptor.\n";
             return status::runtime_error;
         }
         pd->init_plan = std::make_shared<primitive_init_plan_t>();
@@ -166,7 +180,7 @@ private:
         if (pd->is_fwd()) return pd->dst_md();
         if (pd->is_bwd_d()) return pd->diff_src_md();
         if (pd->is_bwd_w()) return pd->diff_weights_md();
-        ir_error_not_expected();
+        gpu_error_not_expected();
         return nullptr;
     }
 
@@ -178,7 +192,7 @@ status_t gen_convolution_fwd_t::pd_t::init(impl::engine_t *engine) {
 }
 
 status_t gen_convolution_fwd_t::init(impl::engine_t *engine) {
-    impl_.reset(new gen_convolution_t());
+    impl_ = std::make_shared<gen_convolution_t>();
     return impl_->init(this, engine);
 }
 
@@ -191,7 +205,7 @@ status_t gen_convolution_bwd_data_t::pd_t::init(impl::engine_t *engine) {
 }
 
 status_t gen_convolution_bwd_data_t::init(impl::engine_t *engine) {
-    impl_.reset(new gen_convolution_t());
+    impl_ = std::make_shared<gen_convolution_t>();
     return impl_->init(this, engine);
 }
 
@@ -205,7 +219,7 @@ status_t gen_convolution_bwd_weights_t::pd_t::init(impl::engine_t *engine) {
 }
 
 status_t gen_convolution_bwd_weights_t::init(impl::engine_t *engine) {
-    impl_.reset(new gen_convolution_t());
+    impl_ = std::make_shared<gen_convolution_t>();
     return impl_->init(this, engine);
 }
 
