@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2024 Intel Corporation
+* Copyright 2024-2025 Intel Corporation
 * Copyright 2024 Codeplay Software Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -57,8 +57,9 @@ struct cublas_lt_params : cublas_base_params {
 
         with_dst_scale_ = !attr->scales_.get(DNNL_ARG_DST).has_default_values();
         if (with_dst_scale_) {
-            auto dst_scale = attr->scales_.get(DNNL_ARG_DST);
-            if (dst_scale.mask_ != 0) { multi_dst_scale_ = true; }
+            if (attr->scales_.get_mask(DNNL_ARG_DST) > 0) {
+                multi_dst_scale_ = true;
+            }
         }
 
         // Initialise flags and variables for the imma case (E.g. imma_case_ flag).
