@@ -39,6 +39,13 @@ namespace impl {
 namespace graph {
 namespace dnnl_impl {
 
+#define VCHECK_OP_EXECUTABLE(cond, msg, ...) \
+    if (!(cond)) { VERROR(graph, op_executable, msg, ##__VA_ARGS__); }
+
+#define VCHECK_PATTERN_UTILS(cond, status, msg, ...) \
+    VCONDCHECK(graph, create, check, pattern, (cond), status, msg, \
+            ##__VA_ARGS__);
+
 const indices_t::type_t input = indices_t::type_t::input;
 const indices_t::type_t output = indices_t::type_t::output;
 
@@ -1086,7 +1093,7 @@ concat_executable_t::desc_t concat_executable_t::create_desc(
     const auto rank = op->get_output_value(0)->get_logical_tensor().ndims;
     const auto res = utils::try_reverse_axis(
             op->get_attr<int64_t>(op_attr::axis), rank);
-    assertm(res.first, "Incorrect axis value.");
+    VCHECK_OP_EXECUTABLE(res.first, "invalid axis for concat");
     const auto axis = res.second;
 
     dnnl::primitive_attr prm_attr;
@@ -1102,9 +1109,8 @@ concat_executable_t::desc_t concat_executable_t::create_desc(
     for (const auto &in_val : op->get_input_values()) {
         const auto tmp_desc
                 = make_dnnl_memory_desc(in_val->get_logical_tensor());
-        src_mds.emplace_back(
-                memory::desc {tmp_desc.get_dims(), tmp_desc.get_data_type(),
-                        get_forced_format_tag(tmp_desc.get_dims())});
+        src_mds.emplace_back(tmp_desc.get_dims(), tmp_desc.get_data_type(),
+                get_forced_format_tag(tmp_desc.get_dims()));
     }
     const auto tmp_desc = make_dnnl_memory_desc(
             op->get_output_value(0)->get_logical_tensor());
@@ -1371,10 +1377,17 @@ softmax_executable_t::desc_t softmax_executable_t::create_desc(
     int64_t axis = op->get_attr<int64_t>(op_attr::axis);
     if (axis < 0) { axis += src.get_ndims(); }
 
-    const dnnl::algorithm algo
-            = op->get_kind() == dnnl_impl::op_kind::dnnl_logsoftmax
-            ? dnnl::algorithm::softmax_log
-            : dnnl::algorithm::softmax_accurate;
+    dnnl::algorithm algo = dnnl::algorithm::undef;
+    if (op->get_kind() == dnnl_impl::op_kind::dnnl_softmax) {
+        const auto mode = op->get_attr<std::string>(op_attr::mode);
+        algo = mode == "inf_as_zero" ? static_cast<dnnl::algorithm>(
+                       dnnl::impl::alg_kind::softmax_accurate_inf_as_zero)
+                                     : dnnl::algorithm::softmax_accurate;
+    } else if (op->get_kind() == dnnl_impl::op_kind::dnnl_logsoftmax) {
+        algo = dnnl::algorithm::softmax_log;
+    } else {
+        assert(!"unexpected op kind");
+    }
 
     dnnl::softmax_forward::primitive_desc pd;
     pd = dnnl::softmax_forward::primitive_desc(p_engine,
@@ -1649,15 +1662,42 @@ bn_folding_t::desc_t bn_folding_t::create_desc(std::shared_ptr<op_t> &op,
     desc.epsilon_desc_ = memory::desc(
             epsilon_dims, memory::data_type::f32, memory::format_tag::a);
 
+#if DNNL_GPU_RUNTIME != DNNL_RUNTIME_NONE \
+        && DNNL_GPU_VENDOR == DNNL_VENDOR_NVIDIA
+    // binary + sqrt post-op fusion is unsupported on NVIDIA GPU
+    if (p_engine.get_kind() == dnnl::engine::kind::gpu) {
+        primitive_attr add_attr;
+        desc.add_pd_
+                = dnnl::binary::primitive_desc(p_engine, algorithm::binary_add,
+                        variance, desc.epsilon_desc_, variance, add_attr);
+
+        primitive_attr sqrt_attr;
+        desc.sqrt_pd_ = dnnl::eltwise_forward::primitive_desc(p_engine,
+                prop_kind::forward, algorithm::eltwise_sqrt, variance, variance,
+                0.0f, 0.0f, sqrt_attr);
+    } else {
+        post_ops add_post_ops;
+        // sqrt_variance = sqrt(temp)
+        add_post_ops.append_eltwise(algorithm::eltwise_sqrt, 0.0f, 0.0f);
+
+        primitive_attr add_attr;
+        add_attr.set_post_ops(add_post_ops);
+        desc.add_pd_
+                = dnnl::binary::primitive_desc(p_engine, algorithm::binary_add,
+                        variance, desc.epsilon_desc_, variance, add_attr);
+    }
+
+#else
     post_ops add_post_ops;
     // sqrt_variance = sqrt(temp)
     add_post_ops.append_eltwise(algorithm::eltwise_sqrt, 0.0f, 0.0f);
 
     primitive_attr add_attr;
-    add_attr.set_post_ops(std::move(add_post_ops));
+    add_attr.set_post_ops(add_post_ops);
     desc.add_pd_ = dnnl::binary::primitive_desc(p_engine, algorithm::binary_add,
             variance, desc.epsilon_desc_, variance, add_attr);
 
+#endif
     // 2. updated_weight = weights * scale / sqrt_variance
 
     // expand 1D scale and variance to same ndims with weights
@@ -1690,7 +1730,7 @@ bn_folding_t::desc_t bn_folding_t::create_desc(std::shared_ptr<op_t> &op,
     mul_post_ops.append_binary(algorithm::binary_div, desc.new_variance_desc_);
 
     primitive_attr mul_attr;
-    mul_attr.set_post_ops(std::move(mul_post_ops));
+    mul_attr.set_post_ops(mul_post_ops);
     desc.mul_pd_ = dnnl::binary::primitive_desc(p_engine, algorithm::binary_mul,
             weights, desc.new_scale_desc_, weights, mul_attr);
 
@@ -1708,14 +1748,26 @@ bn_folding_t::desc_t bn_folding_t::create_desc(std::shared_ptr<op_t> &op,
     sub_post_ops.append_binary(algorithm::binary_add, shift);
 
     primitive_attr sub_attr;
-    sub_attr.set_post_ops(std::move(sub_post_ops));
+    sub_attr.set_post_ops(sub_post_ops);
     desc.sub_pd_ = dnnl::binary::primitive_desc(p_engine, algorithm::binary_sub,
             valid_bias, mean, valid_bias, sub_attr);
 
     memory::dims scratchpad_dims = variance.get_dims();
     // sqrt_variance, zero_bias and others (like epsilon),
     // or no need to alloc bias
+    // binary + sqrt post-op fusion is unsupported on NVIDIA GPU, so we need
+    // one more scratchpad for sqrt
+#if DNNL_GPU_RUNTIME != DNNL_RUNTIME_NONE \
+        && DNNL_GPU_VENDOR == DNNL_VENDOR_NVIDIA
+    size_t factor = 0;
+    if (p_engine.get_kind() == dnnl::engine::kind::gpu) {
+        factor = bias.is_zero() ? 4 : 3;
+    } else {
+        factor = bias.is_zero() ? 3 : 2;
+    }
+#else
     size_t factor = bias.is_zero() ? 3 : 2;
+#endif
     scratchpad_dims[0] *= factor;
     desc.scratchpad_desc_ = memory::desc(
             scratchpad_dims, variance.get_data_type(), memory::format_tag::a);
@@ -1756,7 +1808,8 @@ groupnorm_executable_t::desc_t groupnorm_executable_t::create_desc(
     if (op->has_attr(op_attr::groups)) {
         group_num = op->get_attr<int64_t>(op_attr::groups);
     } else {
-        assertm(false, "group_num is required.");
+        VCHECK_OP_EXECUTABLE(
+                false, "groups attribute is required for groupnorm");
     }
     auto flags = dnnl::normalization_flags::none;
     if (use_affine)
@@ -2406,6 +2459,29 @@ arg_indices_t genindex_executable_t::get_arg_indices(
     return arg_indices;
 }
 
+arg_indices_t sdpa_executable_t::get_arg_indices(
+        const op_t *op, fusion_info_mgr_t &mgr) {
+    UNUSED(mgr);
+
+    arg_indices_t arg_indices;
+    // add input args
+    size_t index = 0;
+    arg_indices.insert({DNNL_ARG_QUERIES, indices_t {input, index++}});
+    arg_indices.insert({DNNL_ARG_KEYS, indices_t {input, index++}});
+    arg_indices.insert({DNNL_ARG_VALUES, indices_t {input, index++}});
+    if (op->get_attr<bool>(dnnl::impl::graph::dnnl_impl::op_attr::with_scale)) {
+        arg_indices.insert({DNNL_ARG_SCALE, indices_t {input, index++}});
+    }
+    if (op->get_attr<bool>(dnnl::impl::graph::dnnl_impl::op_attr::with_mask)) {
+        arg_indices.insert({DNNL_ARG_ATTN_MASK, indices_t {input, index++}});
+    }
+
+    // add output args
+    arg_indices.insert({DNNL_ARG_DST, indices_t {output, 0}});
+    arg_indices.insert({DNNL_ARG_SCRATCHPAD, indices_t {output, 1}});
+    return arg_indices;
+}
+
 } // namespace dnnl_impl
 } // namespace graph
 } // namespace impl
