@@ -646,11 +646,8 @@ bool resolve_node(const binding_t &bind_arg, match_context_t *ctx,
 std::vector<value_t::consumer_t> sort_op_consumers(
         std::shared_ptr<value_t> &op_out_value) {
     auto &cons = op_out_value->get_consumers();
-    std::vector<value_t::consumer_t> sorted_consumers;
     if (cons.empty()) return cons;
-    for (size_t i = 0; i < cons.size(); i++) {
-        sorted_consumers.push_back(cons[i]);
-    }
+    std::vector<value_t::consumer_t> sorted_consumers = cons;
     std::sort(sorted_consumers.begin(), sorted_consumers.end(),
             [&](value_t::consumer_t con_1, value_t::consumer_t con_2) {
                 return con_1.get_op().get_attr<int64_t>(op_attr::op_depth)
@@ -771,8 +768,8 @@ inline std::vector<op_t *> reorder_matched_list(
         }
         auto &outputs = op->get_output_values();
         for (auto it = outputs.begin(); it != outputs.end(); ++it) {
-            auto cons = (*it)->get_consumers();
-            for (auto &con : (*it)->get_consumers()) {
+            const auto &cons = (*it)->get_consumers();
+            for (auto &con : cons) {
                 op_t &con_op = con.get_op();
                 if (std::find(dq.begin(), dq.end(), &con_op) == dq.end()
                         && std::find(fusion_ops.begin(), fusion_ops.end(),
@@ -946,6 +943,9 @@ bool match_alternation(const binding_t &bind_arg, match_context_t *ctx,
     if (bind_arg.bind_kind == BIND_IN) {
         DEBUG(DEBUGINFO_PM, "now doing alt matching");
     }
+    // Coverity: Dynamic cast to pointer can return NULL.
+    if (!alt_nodes) return false;
+
     for (pb_graph_t *alt_node : alt_nodes->get_alternatives()) {
         std::unordered_map<op_t *, pb_op_t *> temp_op_map = matched_op_map;
         binding_t temp_bind = bind_arg;
@@ -1237,7 +1237,7 @@ bool repetition_matcher_t::verify_current_matching_round(
     oport_t oport = pmap_.first;
     op_t *cur_op = local_cached_ctx.out_port_map.at(oport).first;
     size_t cur_op_port = local_cached_ctx.out_port_map.at(oport).second;
-    auto cons = cur_op->get_output_value(cur_op_port)->get_consumers();
+    const auto &cons = cur_op->get_output_value(cur_op_port)->get_consumers();
     if (cons.size() <= 1) return true;
 
     // if current op has more than 1 consumers, while the repetition unit
