@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2024 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -27,6 +27,7 @@
 
 #include <cinttypes>
 #include <functional>
+#include <map>
 #include <string>
 #include <vector>
 #include <unordered_map>
@@ -67,12 +68,12 @@ enum { CRIT = 1, WARN = 2 };
     do { \
         int status__ = (f); \
         if (status__ != OK) { \
-            if (s == CRIT || s == WARN) { \
+            if ((s) == CRIT || (s) == WARN) { \
                 BENCHDNN_PRINT(0, \
                         "Error: Function '%s' at (%s:%d) returned '%d'\n", \
                         __FUNCTION__, __FILE__, __LINE__, status__); \
                 fflush(0); \
-                if (s == CRIT) exit(1); \
+                if ((s) == CRIT) exit(1); \
             } \
             return status__; \
         } \
@@ -107,9 +108,14 @@ extern std::string driver_name;
         } \
     } while (0)
 
+//NOLINTBEGIN(bugprone-macro-parentheses)
+// dnnl_common.hpp:119:5: error: expected ';' at end of declaration list [clang-diagnostic-error]
+//  119 |     BENCHDNN_DISALLOW_COPY_AND_ASSIGN(stream_t);
+//      |     ^
 #define BENCHDNN_DISALLOW_COPY_AND_ASSIGN(T) \
     T(const T &) = delete; \
     T &operator=(const T &) = delete;
+//NOLINTEND(bugprone-macro-parentheses)
 
 /* perf */
 extern double max_ms_per_prb; // max time spend per prb in ms
@@ -136,6 +142,8 @@ struct stat_t {
     int invalid_arguments;
     int listed;
     std::unordered_map<std::string, double[timer::timer_t::mode_t::n_modes]> ms;
+    // Key is the number of the test, value is the repro string.
+    std::map<int, std::string> failed_cases;
 };
 extern stat_t benchdnn_stat;
 
@@ -165,7 +173,7 @@ const char *bool2str(bool value);
 bool match_regex(const char *str, const char *pattern);
 bool skip_start(res_t *res, int idx = benchdnn_stat.tests);
 
-typedef int (*bench_f)(int argc, char **argv);
+using bench_f = int (*)(int, char **);
 std::string locate_file(const std::string &fname);
 int batch(const char *fname, bench_f bench);
 
@@ -203,4 +211,12 @@ void print_dhw(bool &print_d, bool &print_h, bool &print_w, int ndims,
 int benchdnn_getenv_int(const char *name, int default_value);
 std::string benchdnn_getenv_string(const char *name);
 
+// Responsible for printing service information.
+struct summary_t {
+    // Prints up to 10 failed cases reproducers at the end of the run.
+    bool failed_cases = true;
+};
+
+extern summary_t summary;
+
 #endif
