@@ -82,7 +82,7 @@ int str2desc(desc_t *desc, const char *str) {
             ok = 1; \
             s += strlen(prb); \
             char *end_s; \
-            d.c = strtol(s, &end_s, 10); \
+            d.c = strtoll(s, &end_s, 10); \
             if (end_s == s) { \
                 BENCHDNN_PRINT(0, \
                         "ERROR: No value found for `%s` setting. Full " \
@@ -412,6 +412,8 @@ std::string prb_t::set_repro_line() {
 
     if (canonical || dir != def.dir[0]) s << "--dir=" << dir << " ";
     if (canonical || !has_default_dts) s << "--dt=" << dt << " ";
+    if ((canonical || bia_dt_ != def.bia_dt[0]) && !(dir & FLAG_BIA))
+        s << "--bia-dt=" << bia_dt_ << " ";
     if (canonical || stag != def.stag[0]) s << "--stag=" << stag << " ";
     if (canonical || wtag != def.wtag[0]) s << "--wtag=" << wtag << " ";
     if (canonical || dtag != def.dtag[0]) s << "--dtag=" << dtag << " ";
