@@ -139,7 +139,7 @@ void compute_ref_direct_fwd(const prb_t *prb, const args_t &args) {
 
                 conv_res *= wei_scale;
 
-                if (prb->dir & FLAG_BIA) {
+                if (prb->bia_dt() != dnnl_data_type_undef) {
                     const size_t bia_off = bia_off_f(prb, g, oc);
                     conv_res += ((float *)bia_m)[bia_off];
                 }
@@ -323,7 +323,7 @@ void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
                 else
                     ker(conv_res, g, mb, ic, id, ih, iw);
 
-                if (prb->dir & FLAG_BIA) {
+                if (prb->bia_dt() != dnnl_data_type_undef) {
                     const size_t bia_off = (size_t)g * ICG + ic;
                     conv_res += ((float *)bia_m)[bia_off];
                 }
@@ -360,8 +360,8 @@ void compute_ref_bwd_weights(const prb_t *prb, const args_t &args) {
             = [](int64_t I, int64_t O, int64_t k, int64_t S, int64_t P,
                       int64_t D, int64_t &o_s, int64_t &o_e) {
                   const float tmp = P - k * D;
-                  o_s = MAX2(0, ceilf(tmp / S));
-                  o_e = MIN2(O, ceilf((I + tmp) / S));
+                  o_s = MAX2(0, div_up(tmp, S));
+                  o_e = MIN2(O, div_up(I + tmp, S));
               };
 
     auto ker = [&](float &dw, int64_t g, int64_t oc, int64_t ic, int64_t kd,
@@ -429,7 +429,7 @@ void compute_ref_bwd_bias(const prb_t *prb, const args_t &args) {
 
 void compute_ref_direct_bwd_w(const prb_t *prb, const args_t &args) {
     compute_ref_bwd_weights(prb, args);
-    if (!(prb->dir & FLAG_BIA)) return;
+    if (prb->bia_dt() == dnnl_data_type_undef) return;
     compute_ref_bwd_bias(prb, args);
 }
 
