@@ -30,7 +30,7 @@ namespace graph {
 using namespace dnnl::graph;
 using namespace dnnl::impl::graph;
 
-void deserialized_attr::load(utils::json::json_reader_t *reader) {
+void deserialized_attr_t::load(utils::json::json_reader_t *reader) {
     reader->begin_object();
     std::string key_entry;
     std::string value_entry;
@@ -64,7 +64,7 @@ void deserialized_attr::load(utils::json::json_reader_t *reader) {
     reader->next_object_item(&value_entry);
 }
 
-logical_tensor::data_type deserialized_lt::get_data_type() const {
+logical_tensor::data_type deserialized_lt_t::get_data_type() const {
     if (data_type_ == "f32") {
         return logical_tensor::data_type::f32;
     } else if (data_type_ == "f16") {
@@ -92,7 +92,7 @@ logical_tensor::data_type deserialized_lt::get_data_type() const {
     }
 }
 
-logical_tensor::property_type deserialized_lt::get_property_type() const {
+logical_tensor::property_type deserialized_lt_t::get_property_type() const {
     if (property_type_ == "constant") {
         return logical_tensor::property_type::constant;
     } else if (property_type_ == "variable") {
@@ -102,7 +102,7 @@ logical_tensor::property_type deserialized_lt::get_property_type() const {
     }
 }
 
-logical_tensor deserialized_lt::create() const {
+logical_tensor deserialized_lt_t::create() const {
     if (layout_type_ == "any") {
         return logical_tensor(id_, get_data_type(), shape_,
                 logical_tensor::layout_type::any, get_property_type());
@@ -112,7 +112,7 @@ logical_tensor deserialized_lt::create() const {
     }
 }
 
-void deserialized_lt::load(utils::json::json_reader_t *reader) {
+void deserialized_lt_t::load(utils::json::json_reader_t *reader) {
     utils::json::read_helper_t helper;
 
     helper.declare_field("id", &id_);
@@ -124,7 +124,7 @@ void deserialized_lt::load(utils::json::json_reader_t *reader) {
     helper.read_fields(reader);
 }
 
-void deserialized_op::load(utils::json::json_reader_t *reader) {
+void deserialized_op_t::load(utils::json::json_reader_t *reader) {
     utils::json::read_helper_t helper;
 
     helper.declare_field("id", &id_);
@@ -136,14 +136,14 @@ void deserialized_op::load(utils::json::json_reader_t *reader) {
     helper.read_fields(reader);
 }
 
-op deserialized_op::create() const {
+op deserialized_op_t::create() const {
     op aop(id_, opstr2kind(kind_), name_);
     for (auto it = attrs_.begin(); it != attrs_.end(); ++it) {
         const auto &attr = attrstr2kind(it->first);
         const auto &attr_value = it->second;
         const auto &type = attr_value.type_;
         if (type == "string") {
-            const auto value = attr_value.str_value_;
+            const auto &value = attr_value.str_value_;
             aop.set_attr(attr, value);
         } else if (type == "bool") {
             const auto value = attr_value.bool_value_;
@@ -152,13 +152,13 @@ op deserialized_op::create() const {
             const auto value = attr_value.s64_value_;
             aop.set_attr(attr, value);
         } else if (type == "s64[]") {
-            const auto value = attr_value.s64_vector_;
+            const auto &value = attr_value.s64_vector_;
             aop.set_attr(attr, value);
         } else if (type == "f32") {
             const auto value = attr_value.f32_value_;
             aop.set_attr(attr, value);
         } else if (type == "f32[]") {
-            const auto value = attr_value.f32_vector_;
+            const auto &value = attr_value.f32_vector_;
             aop.set_attr(attr, value);
         }
     }
@@ -173,49 +173,49 @@ op deserialized_op::create() const {
     return aop;
 }
 
-bool deserialized_op::get_attr_string(
+bool deserialized_op_t::get_attr_string(
         std::string &attr, const std::string &attr_name) const {
     auto it = attrs_.find(attr_name);
     if (it == attrs_.end()) return false;
     return attr = it->second.str_value_, true;
 }
 
-bool deserialized_op::get_attr_bool(
+bool deserialized_op_t::get_attr_bool(
         bool &attr, const std::string &attr_name) const {
     auto it = attrs_.find(attr_name);
     if (it == attrs_.end()) return false;
     return attr = it->second.bool_value_, true;
 }
 
-bool deserialized_op::get_attr_f32(
+bool deserialized_op_t::get_attr_f32(
         float &attr, const std::string &attr_name) const {
     auto it = attrs_.find(attr_name);
     if (it == attrs_.end()) return false;
     return attr = it->second.f32_value_, true;
 }
 
-bool deserialized_op::get_attr_s64(
+bool deserialized_op_t::get_attr_s64(
         int64_t &attr, const std::string &attr_name) const {
     auto it = attrs_.find(attr_name);
     if (it == attrs_.end()) return false;
     return attr = it->second.s64_value_, true;
 }
 
-bool deserialized_op::get_attr_f32_vector(
+bool deserialized_op_t::get_attr_f32_vector(
         std::vector<float> &attr, const std::string &attr_name) const {
     auto it = attrs_.find(attr_name);
     if (it == attrs_.end()) return false;
     return attr = it->second.f32_vector_, true;
 }
 
-bool deserialized_op::get_attr_s64_vector(
+bool deserialized_op_t::get_attr_s64_vector(
         std::vector<int64_t> &attr, const std::string &attr_name) const {
     auto it = attrs_.find(attr_name);
     if (it == attrs_.end()) return false;
     return attr = it->second.s64_vector_, true;
 }
 
-bool deserialized_op::has_NXC_format() const {
+bool deserialized_op_t::has_NXC_format() const {
     std::string data_format;
     if (get_attr_string(data_format, "data_format")) {
         return data_format == "NXC";
@@ -234,14 +234,14 @@ bool deserialized_op::has_NXC_format() const {
     }
 }
 
-logical_tensor::dims deserialized_op::get_NCX_shape(
+logical_tensor::dims deserialized_op_t::get_NCX_shape(
         size_t idx, bool input) const {
     auto src_dims = input ? in_lts_.at(idx).shape_ : out_lts_.at(idx).shape_;
     if (has_NXC_format()) { change_format_to_ncx(src_dims); }
     return src_dims;
 }
 
-void deserialized_graph::load(const std::string &pass_config_json) {
+void deserialized_graph_t::load(const std::string &pass_config_json) {
     std::ifstream fs(pass_config_json.c_str());
     utils::json::json_reader_t read(&fs);
     utils::json::read_helper_t helper;
@@ -284,7 +284,7 @@ void deserialized_graph::load(const std::string &pass_config_json) {
     }
 
     std::map<size_t, size_t> deg; // record indegree for each op
-    std::map<size_t, deserialized_op> ops_map; // op_id -> op
+    std::map<size_t, deserialized_op_t> ops_map; // op_id -> op
     for (const auto &aop : ops_) {
         ops_map[aop.id_] = aop;
         deg[aop.id_] = 0;
@@ -294,9 +294,8 @@ void deserialized_graph::load(const std::string &pass_config_json) {
         for (const auto &lt : aop.out_lts_) {
             out_lt_2_op_[lt.id_] = aop;
             // collect graph internal and output tensors memory layout
-            std::string mtag
+            lt_2_mtag_[lt.id_]
                     = strides2memory_tag(lt.shape_.size(), lt.stride_, false);
-            lt_2_mtag_[lt.id_] = mtag;
         }
     }
 
@@ -348,8 +347,11 @@ void deserialized_graph::load(const std::string &pass_config_json) {
         }
     }
 
+    // Keep the object out of the call due to recursion inside the call.
+    // Accumulates the state of mb rewrite of nested ops.
+    std::unordered_map<size_t, bool> mb_rewrite_ret;
     for (const auto &graph_in : graph_tensors_) {
-        if (check_tensor_with_mb(graph_in.first)) {
+        if (check_tensor_with_mb(graph_in.first, mb_rewrite_ret)) {
             graph_inputs_with_mb_.push_back(graph_in.first);
         }
     }
@@ -362,13 +364,13 @@ void deserialized_graph::load(const std::string &pass_config_json) {
 }
 
 // Prints the lt in the plain string format: `(id):dt:shape`.
-std::ostream &operator<<(std::ostream &s, const deserialized_lt &dlt) {
+std::ostream &operator<<(std::ostream &s, const deserialized_lt_t &dlt) {
     s << "(" << dlt.id_ << "):" << dlt.data_type_ << ":"
       << lt_dims2str(dlt.shape_);
     return s;
 }
 
-std::string deserialized_lt::get_string() const {
+std::string deserialized_lt_t::get_string() const {
     std::stringstream ss;
     ss << *this;
     return ss.str();
@@ -379,7 +381,7 @@ std::string deserialized_lt::get_string() const {
 //     In: { lt0, lt1, ... }
 //     Out: { lt0, lt1, ... }
 //     Attrs: { Scales: { val0, ... } }  // <-- if any available.
-std::ostream &operator<<(std::ostream &s, const deserialized_op &dop) {
+std::ostream &operator<<(std::ostream &s, const deserialized_op_t &dop) {
     s << "{(" << dop.id_ << ") " << dop.kind_ << "}\n";
 
     s << "    In: { ";
@@ -399,45 +401,65 @@ std::ostream &operator<<(std::ostream &s, const deserialized_op &dop) {
     s << "}\n";
 
     const auto it_attr_scales = dop.attrs_.find("scales");
+    const auto it_attr_group_shape = dop.attrs_.find("group_shape");
     const bool has_scales = it_attr_scales != dop.attrs_.end();
-    if (has_scales) {
+    const bool has_group_shape = it_attr_group_shape != dop.attrs_.end();
+
+    if (has_scales || has_group_shape) {
         s << "    Attrs: { ";
 
-        const auto &scales_v = it_attr_scales->second.f32_vector_;
-        const auto size = scales_v.size();
-        std::string size_str = " (" + std::to_string(size) + ")";
-        s << "Scales" << (size > 1 ? size_str : "") << ": { ";
-        for (size_t i = 0; i < size; i++) {
-            s << scales_v[i];
-            if (i != size - 1) s << ",";
-            s << " ";
+        if (has_scales) {
+            const auto &scales_v = it_attr_scales->second.f32_vector_;
+            const auto size = scales_v.size();
+            std::string size_str = " (" + std::to_string(size) + ")";
+            s << "Scales" << (size > 1 ? size_str : "") << ": { ";
+            for (size_t i = 0; i < size; i++) {
+                s << scales_v[i];
+                if (i != size - 1) s << ",";
+                s << " ";
+            }
+            s << "} "; // Scales
         }
-        s << "} "; // Scales
+
+        if (has_group_shape) {
+            const auto &group_shape_v = it_attr_group_shape->second.s64_vector_;
+            const auto size = group_shape_v.size();
+            std::string size_str = " (" + std::to_string(size) + ")";
+            s << "Group shape:" << (size > 1 ? size_str : "") << ": { ";
+            for (size_t i = 0; i < size; i++) {
+                s << group_shape_v[i];
+                if (i != size - 1) s << ",";
+                s << " ";
+            }
+            s << "} "; // Group Shape
+        }
+
         s << "}\n"; // Attrs
     }
+
     return s;
 }
 
-std::string deserialized_op::get_string() const {
+std::string deserialized_op_t::get_string() const {
     std::stringstream ss;
     ss << *this;
     return ss.str();
 }
 
-std::ostream &operator<<(std::ostream &s, const deserialized_graph &dg) {
+std::ostream &operator<<(std::ostream &s, const deserialized_graph_t &dg) {
     for (const auto &op : dg.ops_) {
         s << op;
     }
     return s;
 }
 
-std::string deserialized_graph::get_string() const {
+std::string deserialized_graph_t::get_string() const {
     std::stringstream ss;
     ss << *this;
     return ss.str();
 }
 
-dnnl::graph::graph deserialized_graph::to_graph(
+dnnl::graph::graph deserialized_graph_t::to_graph(
         const graph_fpmath_mode_t &fpmath_mode) const {
     const auto &engine = get_graph_engine();
     dnnl::graph::graph g(engine.get_kind());
@@ -457,52 +479,71 @@ dnnl::graph::graph deserialized_graph::to_graph(
     return g;
 }
 
-const deserialized_op &deserialized_graph::get_op(size_t id) const {
+const deserialized_op_t &deserialized_graph_t::get_op(size_t id) const {
     for (const auto &op : ops_) {
         if (op.id_ == id) return op;
     }
     assert(!"Given id was not found in the deserialized graph.");
-    static deserialized_op dummy;
+    static deserialized_op_t dummy;
     return dummy;
 }
 
-const deserialized_op &deserialized_graph::get_op_by_out_lt(
+const deserialized_op_t &deserialized_graph_t::get_op_by_out_lt(
         size_t out_lt_id) const {
     for_(const auto &op : ops_)
     for (const auto &out_lt : op.out_lts_) {
         if (out_lt.id_ == out_lt_id) return op;
     }
 
-    static deserialized_op dummy;
+    static deserialized_op_t dummy;
     return dummy;
 }
 
-const deserialized_op &deserialized_graph::get_op_by_in_lt(
+const deserialized_op_t &deserialized_graph_t::get_op_by_in_lt(
         size_t in_lt_id) const {
     for_(const auto &op : ops_)
     for (const auto &in_lt : op.in_lts_) {
         if (in_lt.id_ == in_lt_id) return op;
     }
 
-    static deserialized_op dummy;
+    static deserialized_op_t dummy;
     return dummy;
 }
 
-bool deserialized_graph::check_tensor_with_mb(size_t tensor_id) const {
+bool deserialized_graph_t::check_tensor_with_mb(size_t tensor_id,
+        std::unordered_map<size_t, bool> &mb_rewrite_ret) const {
     if (in_lt_2_ops_.find(tensor_id) == in_lt_2_ops_.end()) return true;
+    if (mb_rewrite_ret.find(tensor_id) != mb_rewrite_ret.end())
+        return mb_rewrite_ret.at(tensor_id);
 
+    bool ret = true;
+    // TODO: initialize with false to avoid multiple re-initialization
     for (const auto &aop : in_lt_2_ops_.at(tensor_id)) {
-        // those unsupport op need rewrite dst_shape / weight_shape also
+        const bool matmul_mb_rewrite = (aop.kind_ == "MatMul")
+                && aop.in_lts_[0].shape_.size() > 2
+                && (tensor_id == aop.in_lts_[0].id_
+                        || tensor_id == aop.in_lts_[1].id_);
+        // The second and third inputs of dynamic dequantize are allowed to
+        // rewrite md only when the sebsequent op supports mb rewriting.
+        const bool dynamicdq_mb_rewrite = (aop.kind_ == "DynamicDequantize")
+                && aop.in_lts_[0].shape_.size() > 2
+                && check_tensor_with_mb(aop.out_lts_[0].id_, mb_rewrite_ret)
+                && (tensor_id == aop.in_lts_[0].id_
+                        || tensor_id == aop.in_lts_[1].id_
+                        || tensor_id == aop.in_lts_[2].id_);
+
         if (std::find(unsupport_mb_rewrite_ops_.begin(),
                     unsupport_mb_rewrite_ops_.end(), aop.kind_)
                 != unsupport_mb_rewrite_ops_.end()) {
-            return false;
-            // bwd ops have multiple inputs with mb
+            // those unsupport op need rewrite dst_shape / weight_shape also
+            ret = false;
         } else if (std::find(bwd_ops_.begin(), bwd_ops_.end(), aop.kind_)
                 != bwd_ops_.end()) {
+            // bwd ops have multiple inputs with mb
+            ret = false;
             if (tensor_id == aop.in_lts_[0].id_
                     || tensor_id == aop.in_lts_[1].id_) {
-                check_tensor_with_mb(aop.out_lts_[0].id_);
+                ret = check_tensor_with_mb(aop.out_lts_[0].id_, mb_rewrite_ret);
                 // deal with LayerNormBackward
             } else if (aop.kind_ == "LayerNormBackward"
                     && ((tensor_id == aop.in_lts_[2].id_
@@ -511,13 +552,12 @@ bool deserialized_graph::check_tensor_with_mb(size_t tensor_id) const {
                             || (tensor_id == aop.in_lts_[3].id_
                                     && aop.in_lts_[3].shape_[0]
                                             == aop.in_lts_[0].shape_[0]))) {
-                check_tensor_with_mb(aop.out_lts_[0].id_);
-            } else {
-                return false;
+                ret = check_tensor_with_mb(aop.out_lts_[0].id_, mb_rewrite_ret);
             }
-            // binary ops need consider rank of 2 inputs
         } else if (std::find(binary_ops_.begin(), binary_ops_.end(), aop.kind_)
                 != binary_ops_.end()) {
+            // binary ops need consider rank of 2 inputs
+            ret = false;
             size_t max_rank_id = aop.in_lts_[0].shape_.size()
                             >= aop.in_lts_[1].shape_.size()
                     ? aop.in_lts_[0].id_
@@ -525,28 +565,29 @@ bool deserialized_graph::check_tensor_with_mb(size_t tensor_id) const {
             if ((aop.in_lts_[0].shape_.size() == aop.in_lts_[1].shape_.size()
                         && aop.in_lts_[1].shape_[0] != 1)
                     || tensor_id == max_rank_id) {
-                check_tensor_with_mb(aop.out_lts_[0].id_);
-            } else {
-                return false;
+                ret = check_tensor_with_mb(aop.out_lts_[0].id_, mb_rewrite_ret);
             }
-            // prelu input1 may has same shape with input0
         } else if (aop.kind_ == "PReLU" && tensor_id == aop.in_lts_[1].id_) {
+            // prelu input1 may has same shape with input0
+            ret = false;
             if (aop.in_lts_[0].shape_.size() == aop.in_lts_[1].shape_.size()) {
-                check_tensor_with_mb(aop.out_lts_[0].id_);
-            } else {
-                return false;
+                ret = check_tensor_with_mb(aop.out_lts_[0].id_, mb_rewrite_ret);
             }
-            // apply mb for all inputs of concat
-        } else if (aop.kind_ != "Concat" && tensor_id != aop.in_lts_[0].id_) {
-            return false;
-            // check consumer ops recursively
+        } else if (!(matmul_mb_rewrite || dynamicdq_mb_rewrite
+                           || aop.kind_ == "Concat")
+                && tensor_id != aop.in_lts_[0].id_) {
+            // Do not rewrite if the given tensor is not the first input of the
+            // op, except matmul, dynamic dequantize and concat.
+            ret = false;
         } else if (aop.kind_ == "End") {
-            return true;
+            ret = true;
         } else {
-            return check_tensor_with_mb(aop.out_lts_[0].id_);
+            ret = check_tensor_with_mb(aop.out_lts_[0].id_, mb_rewrite_ret);
         }
     }
-    return true;
+
+    mb_rewrite_ret.emplace(tensor_id, ret);
+    return ret;
 }
 
 } // namespace graph
