@@ -26,7 +26,171 @@
 
 namespace graph {
 
-void flex_rewrite::rewrite(deserialized_graph &dgraph) {
+void flex_rewrite_t::rewrite_linked_shape_and_attr(
+        deserialized_graph_t &dgraph) {
+    for (auto &aop : dgraph.ops_) {
+        if (aop.kind_ == "DynamicDequantize") {
+            auto &attr = aop.attrs_;
+            if (attr.find("qtype") == attr.end()
+                    || attr["qtype"].str_value_ != "per_group")
+                continue;
+            if (attr.find("group_shape") == attr.end()) {
+                BENCHDNN_PRINT(0,
+                        "Error: missed `group-shape` attribute for "
+                        "per-group quantization for op with id=\'%zu\'\n",
+                        aop.id_);
+                SAFE_V(FAIL);
+            }
+
+            bool input_shape_rewrite = std::any_of(aop.in_lts_.begin(),
+                    aop.in_lts_.end(), [&](const deserialized_lt_t &in_lt) {
+                        return in_shapes_.count(in_lt.id_)
+                                && in_shapes_[in_lt.id_] != "default";
+                    });
+            bool group_shape_rewrite = op_attrs_.count(aop.id_)
+                    && parse_attrs(op_attrs_.at(aop.id_)).count("group_shape");
+
+            auto &group_shape = attr["group_shape"].s64_vector_;
+            const auto &src_lt = aop.in_lts_[0];
+
+            if (!group_shape_rewrite && !input_shape_rewrite) continue;
+            if (input_shape_rewrite && group_shape_rewrite) {
+                // if both input shapes and group_shape are provided, check if
+                // the new shape are valid.
+                auto &scale_lt = aop.in_lts_[1];
+                if (src_lt.shape_.size() != scale_lt.shape_.size()) {
+                    BENCHDNN_PRINT(0,
+                            "Error: the ndims of scale tensor should align "
+                            "with the ndims of input tensor for op with "
+                            "id=\'%zu\'\n",
+                            aop.id_);
+                    SAFE_V(FAIL);
+                }
+
+                if (src_lt.shape_.size() != group_shape.size()) {
+                    BENCHDNN_PRINT(0,
+                            "Error: the ndims of `group-shape` attribute "
+                            "should align with the input ndims for op with "
+                            "id=\'%zu\'\n",
+                            aop.id_);
+                    SAFE_V(FAIL);
+                }
+
+                for (size_t idx = 0; idx < src_lt.shape_.size(); ++idx) {
+                    if (src_lt.shape_[idx]
+                            != scale_lt.shape_[idx] * group_shape[idx]) {
+                        BENCHDNN_PRINT(0,
+                                "Error: the input shape should equal with the "
+                                "product of corresponding dimension of scale "
+                                "shape and group shape, input shape: %lld, "
+                                "scale shape: %lld, group shape: %lld\n",
+                                (long long)src_lt.shape_[idx],
+                                (long long)scale_lt.shape_[idx],
+                                (long long)group_shape[idx]);
+                        SAFE_V(FAIL);
+                    }
+                }
+
+                if (aop.in_lts_.size() > 2) {
+                    auto &zp_lt = aop.in_lts_[2];
+                    if (scale_lt.shape_.size() != zp_lt.shape_.size()) {
+                        BENCHDNN_PRINT(0,
+                                "Error: the ndims of scale tensor should align "
+                                "with the ndims of zero-point tensor for op "
+                                "with id=\'%zu\'\n",
+                                aop.id_);
+                        SAFE_V(FAIL);
+                    }
+
+                    for (size_t idx = 0; idx < scale_lt.shape_.size(); ++idx) {
+                        if (scale_lt.shape_[idx] != zp_lt.shape_[idx]) {
+                            BENCHDNN_PRINT(0,
+                                    "Error: the shape of zero-point tensor "
+                                    "should be the same as the shape of scale "
+                                    "tensor for op with id=\'%zu\'\n",
+                                    aop.id_);
+                            SAFE_V(FAIL);
+                        }
+                    }
+                }
+                continue;
+            }
+
+            if (group_shape_rewrite && !input_shape_rewrite) {
+                // if user only rewrite group shape attribute, update the scale
+                // shape and zps shape (if available) accordingly.
+                dims_t new_group_quant_scale_zps_dims(
+                        src_lt.shape_.size(), DNNL_GRAPH_UNKNOWN_DIM);
+                if (src_lt.shape_.size() != group_shape.size()) {
+                    BENCHDNN_PRINT(0,
+                            "Error: the ndims of `group-shape` attribute "
+                            "should align with the input ndims for op with "
+                            "id=\'%zu\'\n",
+                            aop.id_);
+                    SAFE_V(FAIL);
+                }
+
+                for (size_t idx = 0; idx < src_lt.shape_.size(); ++idx) {
+                    if (src_lt.shape_[idx] % group_shape[idx] != 0) {
+                        BENCHDNN_PRINT(0,
+                                "Error: the dimension of `group-shape` "
+                                "attribute should be divisible by the "
+                                "corresponding dimensions of the input shape, "
+                                "group shape: %lld, input shape: %lld\n",
+                                (long long)group_shape[idx],
+                                (long long)src_lt.shape_[idx]);
+                        SAFE_V(FAIL);
+                    }
+                    new_group_quant_scale_zps_dims[idx]
+                            = src_lt.shape_[idx] / group_shape[idx];
+                }
+
+                auto &scale_lt = aop.in_lts_[1];
+                scale_lt.shape_ = new_group_quant_scale_zps_dims;
+                scale_lt.stride_ = memory_tag2strides(
+                        scale_lt.shape_, dgraph.lt_2_mtag_[scale_lt.id_]);
+                if (aop.in_lts_.size() > 2) {
+                    auto &zp_lt = aop.in_lts_[2];
+                    zp_lt.shape_ = new_group_quant_scale_zps_dims;
+                    zp_lt.stride_ = memory_tag2strides(
+                            zp_lt.shape_, dgraph.lt_2_mtag_[zp_lt.id_]);
+                }
+            } else if (input_shape_rewrite && !group_shape_rewrite) {
+                // if user only rewrites input shapes, update the group-shape
+                // attribute accordingly.
+                auto &scale_lt = aop.in_lts_[1];
+                if (src_lt.shape_.size() != scale_lt.shape_.size()) {
+                    BENCHDNN_PRINT(0,
+                            "Error: the ndims of scale tensor should align "
+                            "with the ndims of input tensor for op with "
+                            "id=\'%zu\'\n",
+                            aop.id_);
+                    SAFE_V(FAIL);
+                }
+
+                std::vector<int64_t> new_group_shape(src_lt.shape_.size(), 1);
+                for (size_t idx = 0; idx < src_lt.shape_.size(); ++idx) {
+                    if (src_lt.shape_[idx] % scale_lt.shape_[idx] != 0) {
+                        BENCHDNN_PRINT(0,
+                                "Error: the dimension of scale  shape should "
+                                "be divisible by the corresponding dimensions "
+                                "of the input  shape, scale shape: %lld, input "
+                                "shape: %lld\n",
+                                (long long)scale_lt.shape_[idx],
+                                (long long)src_lt.shape_[idx]);
+                        SAFE_V(FAIL);
+                    }
+                    new_group_shape[idx]
+                            = src_lt.shape_[idx] / scale_lt.shape_[idx];
+                }
+
+                group_shape = new_group_shape;
+            }
+        }
+    }
+}
+
+void flex_rewrite_t::rewrite(deserialized_graph_t &dgraph) {
     bool change_stride = false;
     inports_shape_rewrite(dgraph, change_stride);
     if (!(op_attrs_.size() == 1 && op_attrs_.count(0)
@@ -35,12 +199,14 @@ void flex_rewrite::rewrite(deserialized_graph &dgraph) {
     }
     infer_output_shape(dgraph, change_stride);
     quantized_graph_rewrite(dgraph);
+    op_kind_rewrite(dgraph);
     graph_attrs_rewrite(dgraph);
+    rewrite_linked_shape_and_attr(dgraph);
     dt_rewrite(dgraph);
     dt_map_rewrite(dgraph);
 }
 
-void flex_rewrite::split_ncx(const std::string &data_format, dims_t &in,
+void flex_rewrite_t::split_ncx(const std::string &data_format, dims_t &in,
         int64_t &n, int64_t &c, dims_t &x) const {
     x.clear();
     n = in[0];
@@ -57,7 +223,7 @@ void flex_rewrite::split_ncx(const std::string &data_format, dims_t &in,
     }
 }
 
-void flex_rewrite::merge_ncx(const std::string &data_format, dims_t &out,
+void flex_rewrite_t::merge_ncx(const std::string &data_format, dims_t &out,
         int64_t n, int64_t c, const dims_t &x) const {
     out.clear();
     out.push_back(n);
@@ -74,7 +240,7 @@ void flex_rewrite::merge_ncx(const std::string &data_format, dims_t &out,
     }
 }
 
-void flex_rewrite::split_oix(const std::string &data_format, dims_t &in,
+void flex_rewrite_t::split_oix(const std::string &data_format, dims_t &in,
         dims_t &oi, dims_t &x) const {
     x.clear();
     if (data_format == "OIX" || data_format == "IOX") {
@@ -113,7 +279,7 @@ std::string stdvec2string(const std::vector<T> &v) {
     return s;
 }
 
-void flex_rewrite::broadcast(const dims_t &x, const dims_t &y, dims_t &z,
+void flex_rewrite_t::broadcast(const dims_t &x, const dims_t &y, dims_t &z,
         const std::string &x_str, const std::string &y_str) const {
     const size_t x_rank = x.size();
     const size_t y_rank = y.size();
@@ -144,8 +310,8 @@ void flex_rewrite::broadcast(const dims_t &x, const dims_t &y, dims_t &z,
     }
 }
 
-void flex_rewrite::cal_pads(dims_t &pads_begin, dims_t &pads_end,
-        const deserialized_op &aop, const dims_t &spatial_dims,
+void flex_rewrite_t::cal_pads(dims_t &pads_begin, dims_t &pads_end,
+        const deserialized_op_t &aop, const dims_t &spatial_dims,
         const dims_t &strides, const dims_t &kernel, bool deconv) const {
     pads_begin.clear();
     pads_end.clear();
@@ -183,8 +349,8 @@ void flex_rewrite::cal_pads(dims_t &pads_begin, dims_t &pads_end,
     }
 }
 
-void flex_rewrite::infer_output_shape(
-        deserialized_graph &dgraph, bool change_stride) {
+void flex_rewrite_t::infer_output_shape(
+        deserialized_graph_t &dgraph, bool change_stride) {
     auto &gi = dgraph.graph_tensors_;
     for (auto &aop : dgraph.ops_) {
         auto kind = opstr2kind(aop.kind_);
@@ -752,7 +918,7 @@ void flex_rewrite::infer_output_shape(
 /// @param msg Error message info when function returns `false` value.
 /// @return `true` if an inport info is valid and `false` otherwise. A message `msg`
 /// describes an error occurred.
-bool flex_rewrite::get_inport_shape_stride(const std::string &in_shape,
+bool flex_rewrite_t::get_inport_shape_stride(const std::string &in_shape,
         std::string &shape, std::string &stride, std::string &msg) {
     assert(shape.empty() && stride.empty());
     if (in_shape == "0" || in_shape == "-") {
@@ -809,16 +975,16 @@ bool flex_rewrite::get_inport_shape_stride(const std::string &in_shape,
     }
 }
 
-void flex_rewrite::inports_shape_rewrite(
-        deserialized_graph &dgraph, bool &change_stride) {
+void flex_rewrite_t::inports_shape_rewrite(
+        deserialized_graph_t &dgraph, bool &change_stride) {
     // reminder mb rewrite status
     if (mb_ != 0 && dgraph.graph_inputs_with_mb_.empty()) {
         BENCHDNN_PRINT(0,
-                "Error: flex_rewrite: can't rewrite mb value with \'%ld\'.\n",
+                "Error: flex_rewrite_t: can't rewrite mb value with \'%ld\'.\n",
                 (long)mb_);
     }
 
-    const auto set_default_deserialized_lt = [](deserialized_lt &lt) {
+    const auto set_default_deserialized_lt_t = [](deserialized_lt_t &lt) {
         auto ndims = lt.shape_.size();
         logical_tensor::dims infer_dim(ndims, -1);
         lt.shape_ = infer_dim;
@@ -850,7 +1016,7 @@ void flex_rewrite::inports_shape_rewrite(
                         lt.id_);
                 SAFE_V(FAIL);
             }
-            set_default_deserialized_lt(lt);
+            set_default_deserialized_lt_t(lt);
             continue;
         }
 
@@ -956,12 +1122,13 @@ void flex_rewrite::inports_shape_rewrite(
 
     for_(auto &aop : dgraph.ops_)
     for (auto &lt : aop.out_lts_) {
-        set_default_deserialized_lt(lt);
+        set_default_deserialized_lt_t(lt);
     }
 }
 
-void flex_rewrite::op_attrs_rewrite(deserialized_graph &dgraph) {
+void flex_rewrite_t::op_attrs_rewrite(deserialized_graph_t &dgraph) {
     std::vector<size_t> op_ids_;
+    op_ids_.reserve(dgraph.ops_.size());
     for (const auto &aop : dgraph.ops_) {
         op_ids_.emplace_back(aop.id_);
     }
@@ -976,15 +1143,15 @@ void flex_rewrite::op_attrs_rewrite(deserialized_graph &dgraph) {
         auto &temp_op = dgraph.ops_[std::distance(op_ids_.begin(), iter)];
         const auto attrs = parse_attrs(temp_attrs.second);
         for (const auto &new_attr : attrs) {
-            auto attr_name = new_attr.first;
+            const auto &attr_name = new_attr.first;
             if (!temp_op.attrs_.count(attr_name)) {
                 BENCHDNN_PRINT(0,
                         "graph: rewrite: no attr name `%s` in op %zd.\n",
                         attr_name.c_str(), temp_attrs.first);
                 SAFE_V(FAIL);
             }
-            auto new_val = new_attr.second;
-            auto attr_type = temp_op.attrs_[attr_name].type_;
+            const auto &new_val = new_attr.second;
+            const auto &attr_type = temp_op.attrs_[attr_name].type_;
             if (attr_type == "string") {
                 temp_op.attrs_[attr_name].str_value_ = new_val;
             } else if (attr_type == "bool") {
@@ -1005,7 +1172,7 @@ void flex_rewrite::op_attrs_rewrite(deserialized_graph &dgraph) {
     }
 }
 
-inline bool is_int8_quantization(const deserialized_op &aop) {
+inline bool is_int8_quantization(const deserialized_op_t &aop) {
     if (aop.kind_ == "Dequantize") {
         const auto dt = aop.in_lts_.front().get_data_type();
         return (dt == logical_tensor::data_type::u8
@@ -1020,7 +1187,7 @@ inline bool is_int8_quantization(const deserialized_op &aop) {
     }
 }
 
-void flex_rewrite::quantized_graph_rewrite(deserialized_graph &dgraph) {
+void flex_rewrite_t::quantized_graph_rewrite(deserialized_graph_t &dgraph) {
     for (auto &aop : dgraph.ops_) {
         if (aop.kind_ != "Dequantize" && aop.kind_ != "Quantize") continue;
 
@@ -1062,7 +1229,7 @@ void flex_rewrite::quantized_graph_rewrite(deserialized_graph &dgraph) {
 }
 
 // Select: only rewrite src_1/src_2/dst as `cond` is always `bool`.
-void dt_rewrite_select(deserialized_op &select, const std::string &dt) {
+void dt_rewrite_select(deserialized_op_t &select, const std::string &dt) {
     select.in_lts_[1].data_type_ = dt;
     select.in_lts_[2].data_type_ = dt;
     select.out_lts_[0].data_type_ = dt;
@@ -1072,7 +1239,7 @@ void dt_rewrite_select(deserialized_op &select, const std::string &dt) {
 // normalization still requires f32 for gamma/beta/etc. This is good for most of
 // the cases. But there is a potential issue if gamma/beta/etc is connected to
 // another op which will rewrite the data type at other places.
-void dt_rewrite_norm(deserialized_op &norm, const std::string &dt) {
+void dt_rewrite_norm(deserialized_op_t &norm, const std::string &dt) {
     if (norm.kind_ == "BatchNormTrainingBackward"
             || norm.kind_ == "LayerNormBackward") {
         // rewrite for src/diff_dst/diff_src.
@@ -1086,7 +1253,7 @@ void dt_rewrite_norm(deserialized_op &norm, const std::string &dt) {
     }
 }
 
-void flex_rewrite::dt_rewrite(deserialized_graph &dgraph) {
+void flex_rewrite_t::dt_rewrite(deserialized_graph_t &dgraph) {
     if (dt_ == dnnl_data_type_undef) return;
 
     // We can only do data type rewriting for pure floating-point graph.
@@ -1163,7 +1330,57 @@ void flex_rewrite::dt_rewrite(deserialized_graph &dgraph) {
     }
 }
 
-void flex_rewrite::dt_map_rewrite(deserialized_graph &dgraph) {
+void flex_rewrite_t::op_kind_rewrite(deserialized_graph_t &dgraph) {
+    // Step 1: check the op kind in the map and whether the given ids are in
+    // the graph.
+    for (const auto &v : op_kind_map_) {
+        if (v.second == "default") return;
+
+        auto target_kind = opstr2kind(v.second);
+        if (target_kind == dnnl::graph::op::kind::LastSymbol) {
+            BENCHDNN_PRINT(0,
+                    "graph: rewrite: invalid target op kind %s is provided\n",
+                    v.second.c_str());
+            SAFE_V(FAIL);
+        }
+
+        // Only support op kind rewrite for binary and eltwise ops.
+        auto target_driver = opkind2driver(target_kind);
+        if (target_driver != dnnl_driver_t::binary
+                && target_driver != dnnl_driver_t::eltwise) {
+            BENCHDNN_PRINT(0,
+                    "graph: rewrite: target op kind %s for id `%zd` is not "
+                    "supported\n",
+                    v.second.c_str(), v.first);
+            SAFE_V(FAIL);
+        }
+
+        auto &aop = dgraph.get_op(v.first);
+        if (aop.empty()) {
+            BENCHDNN_PRINT(0,
+                    "graph: rewrite: ID `%zd` is not found in the graph\n",
+                    v.first);
+            SAFE_V(FAIL);
+        }
+        auto op_driver = opkind2driver(opstr2kind(aop.kind_));
+        if (op_driver != target_driver) {
+            BENCHDNN_PRINT(0,
+                    "graph: rewrite: target op kind `%s` does not "
+                    "match the op kind `%s` in the graph\n",
+                    v.second.c_str(), aop.kind_.c_str());
+            SAFE_V(FAIL);
+        }
+    }
+
+    // Step 2: rewrite the op kinds.
+    for (const auto &v : op_kind_map_) {
+        for (auto &aop : dgraph.ops_) {
+            if (aop.id_ == v.first) { aop.kind_ = v.second; }
+        }
+    }
+}
+
+void flex_rewrite_t::dt_map_rewrite(deserialized_graph_t &dgraph) {
     // check the IDs and data types in dt_map.
     for (const auto &v : dt_map_) {
         if (v.second == dnnl_data_type_undef) return;
@@ -1202,7 +1419,7 @@ void flex_rewrite::dt_map_rewrite(deserialized_graph &dgraph) {
     }
 }
 
-void flex_rewrite::graph_attrs_rewrite(deserialized_graph &dgraph) {
+void flex_rewrite_t::graph_attrs_rewrite(deserialized_graph_t &dgraph) {
 
     // if the fpmath mode is specified by users through cml, replace the fpmath
     // mode from JSON file with the value from cml.
@@ -1221,8 +1438,8 @@ void flex_rewrite::graph_attrs_rewrite(deserialized_graph &dgraph) {
 /// @param dgraph A deserialized graph
 /// @param change_stride A boolean value indicating whether the graph input strides
 /// have been changed.
-void flex_rewrite::update_output_info(
-        deserialized_op &aop, deserialized_graph &dgraph, bool change_stride) {
+void flex_rewrite_t::update_output_info(deserialized_op_t &aop,
+        deserialized_graph_t &dgraph, bool change_stride) {
     auto kind = opstr2kind(aop.kind_);
     auto &gi = dgraph.graph_tensors_;
     // if a input stride is not changed, the output stride should not be changed as well
