@@ -23,7 +23,7 @@
 namespace graph {
 
 partition_data_displacer_t::partition_data_displacer_t(
-        const deserialized_graph &dg, const dnnl::graph::partition &par)
+        const deserialized_graph_t &dg, const dnnl::graph::partition &par)
     : dg_(&dg) {
     const auto &op_ids = par.get_ops();
     op_ids_set_ = std::unordered_set<size_t>(op_ids.begin(), op_ids.end());
@@ -39,7 +39,7 @@ partition_data_displacer_t::partition_data_displacer_t(
     static const std::unordered_set<std::string> f8_main_op_kind {
             "MatMul", "Convolution"};
 
-    // The logic below relies on the assumption that deserialized_graph is
+    // The logic below relies on the assumption that deserialized_graph_t is
     // sorted in the chronological order.
     for (const auto &aop : dg_->ops_) {
         // Skip the check if op is not in the partition.
@@ -157,7 +157,7 @@ partition_data_displacer_t::partition_data_displacer_t(
 
             // Search for an input lt without a parent, this is the one to
             // modify for both explicit and implicit masks.
-            const deserialized_lt *causal_mask_lt = nullptr;
+            const deserialized_lt_t *causal_mask_lt = nullptr;
             size_t offset = SIZE_MAX;
             size_t qk_data_offset = SIZE_MAX;
             // Select condition having a parent or not is the only reliable
@@ -333,7 +333,7 @@ int partition_data_displacer_t::displace_input_data(
             && op_ids_set_.find(parent_op->id_) != op_ids_set_.end()) {
         backward_path_launched = true;
         // generate the reverse op based on OP kind
-        // make a copy of deserialized_op to avoid impact on graph execution
+        // make a copy of deserialized_op_t to avoid impact on graph execution
         // Currently, we support the following OPs' reverse execution:
         // All of the execution need to swap the input lt and output lt first
 
@@ -440,10 +440,10 @@ int partition_data_displacer_t::displace_input_data(
 }
 
 int partition_data_displacer_t::gen_quantize_filling(
-        const ::graph::deserialized_op &main_op, int arg, dnn_mem_t &mem,
+        const ::graph::deserialized_op_t &main_op, int arg, dnn_mem_t &mem,
         const ::std::string &dt, res_t *res) {
     // clone a deserialized op object and modify to specified data type
-    ::graph::deserialized_op op = main_op;
+    ::graph::deserialized_op_t op = main_op;
     auto driver = opkind2driver(opstr2kind(op.kind_));
     bool is_f8_quantization = (dt == "f8_e5m2" || dt == "f8_e4m3");
 
@@ -457,23 +457,23 @@ int partition_data_displacer_t::gen_quantize_filling(
                 // None of them supports u8u8, replace with u8s8.
                 op.in_lts_[1].data_type_ = "s8";
             } else if (dt == "s4" || dt == "u4") {
-                // None of them supports x4x4, replace with f32x4.
-                op.in_lts_[0].data_type_ = "f32";
+                // None of them supports x4x4, replace with f32x4f32 or
+                // xf16x4xf16.
+                op.in_lts_[0].data_type_ = op.out_lts_[0].data_type_;
             }
         }
     }
-    if (driver == dnnl_driver_t::pool || driver == dnnl_driver_t::binary) {
-        // pool does not support x8f32 on cpu
-        // binary does not support x8x8bf16 on gpu
-        // replace output with x8
+    if (driver == dnnl_driver_t::pool || driver == dnnl_driver_t::binary
+            || is_f8_quantization) {
+        // pool does not support x8f32 on cpu, and binary does not support
+        // x8x8bf16 on gpu, hence replace output with x8.
+        // f8 data types needs setting output data type to f8
         op.out_lts_[0].data_type_ = dt;
     } else if (op.out_lts_[0].data_type_ != "bf16") {
         if (op.in_lts_.size() > 1 && op.in_lts_[1].data_type_ == "s8") {
             // Use u8 as output data type for two-input operations to avoid
             // data overflow due to the specific driver logic.
             op.out_lts_[0].data_type_ = "u8";
-        } else if (is_f8_quantization) {
-            op.out_lts_[0].data_type_ = "f8_e5m2";
         } else {
             // Use f32 as output data type since not all primitives support
             // different data types for input and output.
@@ -550,6 +550,9 @@ int partition_data_displacer_t::gen_causal_mask_filling(
     benchdnn_parallel_nd(batch, M, N, [&](int64_t b, int64_t m, int64_t n) {
         int64_t idx = b * M * N + m * N + n;
         float val = m >= n ? 0.f : -INFINITY;
+        // The line below masks out the whole prompt to verify the softmax
+        // output returns zeroes, not NaNs, as expected by PyTorch.
+        if (m == M - 1) val = -INFINITY;
         tmp_mem.set_elem(idx, val);
     });
 
