@@ -73,7 +73,7 @@ void assign_shape_val(int64_t &c, int64_t &w, int64_t &h, int64_t &d,
     d = has_d ? ncx_shape[2] : 1;
 };
 
-bool get_graph_attr(const deserialized_op &base_op_ref,
+bool get_graph_attr(const deserialized_op_t &base_op_ref,
         attr_t::fpmath_mode_t &arg_fpmath_mode) {
 
     const auto &op_kind = base_op_ref.kind_;
@@ -91,8 +91,8 @@ bool get_graph_attr(const deserialized_op &base_op_ref,
     return true;
 }
 
-bool get_driver_tag_by_idx(const deserialized_op &base_op_ref, std::string &tag,
-        int idx = 0, bool from_output = false) {
+bool get_driver_tag_by_idx(const deserialized_op_t &base_op_ref,
+        std::string &tag, int idx = 0, bool from_output = false) {
     logical_tensor::dims strides = from_output
             ? base_op_ref.out_lts_[idx].stride_
             : base_op_ref.in_lts_[idx].stride_;
@@ -104,19 +104,19 @@ bool get_driver_tag_by_idx(const deserialized_op &base_op_ref, std::string &tag,
     return true;
 }
 
-bool get_driver_tag(const deserialized_op &base_op_ref, std::string &tag,
+bool get_driver_tag(const deserialized_op_t &base_op_ref, std::string &tag,
         bool from_output = false) {
     return get_driver_tag_by_idx(base_op_ref, tag, 0, from_output);
 }
 
-bool get_driver_stag_and_dtag(const deserialized_op &base_op_ref,
+bool get_driver_stag_and_dtag(const deserialized_op_t &base_op_ref,
         std::string &stag, std::string &dtag, bool from_output = false) {
     bool ret = get_driver_tag(base_op_ref, stag, from_output);
     dtag = stag;
     return ret;
 }
 
-bool get_driver_axis(const deserialized_op &base_op_ref, int &axis) {
+bool get_driver_axis(const deserialized_op_t &base_op_ref, int &axis) {
     int64_t val = 0;
     base_op_ref.get_attr_s64(val, "axis");
     axis = val >= 0
@@ -125,7 +125,19 @@ bool get_driver_axis(const deserialized_op &base_op_ref, int &axis) {
     return true;
 }
 
-bool get_prb_dims(const deserialized_op &base_op_ref, prb_dims_t &prb_dims) {
+bool get_driver_bia_dt(const deserialized_op_t &base_op_ref,
+        dnnl_data_type_t &bia_dt, const dnnl_data_type_t dt) {
+    if (base_op_ref.in_lts_.size() <= 2)
+        bia_dt = dnnl_data_type_undef;
+    else if (is_integral_dt(dt)) {
+        bia_dt = convert_dt(base_op_ref.in_lts_[2].get_data_type());
+    } else {
+        bia_dt = dt;
+    }
+    return true;
+}
+
+bool get_prb_dims(const deserialized_op_t &base_op_ref, prb_dims_t &prb_dims) {
     prb_dims.dims = base_op_ref.in_lts_.front().shape_;
     prb_dims.ndims = static_cast<int>(prb_dims.dims.size());
     return true;
@@ -133,7 +145,7 @@ bool get_prb_dims(const deserialized_op &base_op_ref, prb_dims_t &prb_dims) {
 
 // extend shape in src to match the ndims
 // if the rank in tensor is less than ndims, we need to insert 1
-void extend_dims(::graph::deserialized_lt &lt, size_t ndims) {
+void extend_dims(::graph::deserialized_lt_t &lt, size_t ndims) {
     size_t nelem = 1;
     for (size_t i = 0; i < lt.shape_.size(); i++) {
         nelem *= lt.shape_[i];
@@ -149,7 +161,7 @@ void extend_dims(::graph::deserialized_lt &lt, size_t ndims) {
 namespace custom {
 
 ::custom::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::custom::settings_t op_setting;
     auto opkind = opstr2kind(base_op_ref.kind_);
     switch (opkind) {
@@ -211,10 +223,10 @@ namespace custom {
 
 namespace binary {
 bool get_binary_prb_vdims(
-        const deserialized_op &base_op_ref, prb_vdims_t &prb_vdims) {
+        const deserialized_op_t &base_op_ref, prb_vdims_t &prb_vdims) {
     // since base_op_ref is a copy from the original
     // it is safe to modify it
-    deserialized_op &base_op = const_cast<deserialized_op &>(base_op_ref);
+    deserialized_op_t &base_op = const_cast<deserialized_op_t &>(base_op_ref);
 
     auto &src0_dims = base_op.in_lts_[0].shape_;
     auto &src1_dims = base_op.in_lts_[1].shape_;
@@ -258,8 +270,8 @@ bool get_binary_prb_vdims(
     return true;
 }
 
-bool get_binary_sdt_and_ddt(
-        const deserialized_op &base_op_ref, ::binary::settings_t &op_setting) {
+bool get_binary_sdt_and_ddt(const deserialized_op_t &base_op_ref,
+        ::binary::settings_t &op_setting) {
     auto sdt0 = convert_dt(base_op_ref.in_lts_[0].get_data_type());
     auto sdt1 = convert_dt(base_op_ref.in_lts_[1].get_data_type());
     auto ddt = convert_dt(base_op_ref.out_lts_[0].get_data_type());
@@ -269,8 +281,8 @@ bool get_binary_sdt_and_ddt(
     return true;
 }
 
-bool get_binary_stag_and_dtag(
-        const deserialized_op &base_op_ref, ::binary::settings_t &op_setting) {
+bool get_binary_stag_and_dtag(const deserialized_op_t &base_op_ref,
+        ::binary::settings_t &op_setting) {
     // src1, src2 and dst could have different tags.
     std::string stag0, stag1, dtag;
     if (!get_driver_tag_by_idx(base_op_ref, dtag, 0, true)
@@ -283,7 +295,8 @@ bool get_binary_stag_and_dtag(
     return true;
 }
 
-bool get_binary_alg(const deserialized_op &base_op_ref, ::binary::alg_t &alg) {
+bool get_binary_alg(
+        const deserialized_op_t &base_op_ref, ::binary::alg_t &alg) {
     static const std::unordered_map<std::string, ::binary::alg_t>
             map_kind_to_alg {{"Add", ::binary::alg_t::ADD},
                     {"BiasAdd", ::binary::alg_t::ADD},
@@ -302,7 +315,7 @@ bool get_binary_alg(const deserialized_op &base_op_ref, ::binary::alg_t &alg) {
 }
 
 ::binary::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::binary::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(
             binary::get_binary_prb_vdims(base_op_ref, op_setting.prb_vdims),
@@ -326,7 +339,7 @@ bool get_binary_alg(const deserialized_op &base_op_ref, ::binary::alg_t &alg) {
 
 namespace bnorm {
 
-bool get_bnorm_desc(const deserialized_op &base_op_ref, ::bnorm::desc_t &d) {
+bool get_bnorm_desc(const deserialized_op_t &base_op_ref, ::bnorm::desc_t &d) {
     const auto &src_ncx_shape = base_op_ref.get_NCX_shape(0, true);
     d.mb = src_ncx_shape[0];
     d.ndims = static_cast<int>(src_ncx_shape.size());
@@ -335,7 +348,7 @@ bool get_bnorm_desc(const deserialized_op &base_op_ref, ::bnorm::desc_t &d) {
     return true;
 }
 
-bool get_bnorm_dir(const deserialized_op &base_op_ref, dir_t &dir) {
+bool get_bnorm_dir(const deserialized_op_t &base_op_ref, dir_t &dir) {
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind == "BatchNormForwardTraining") {
         dir = dir_t::FWD_D;
@@ -356,13 +369,13 @@ bool get_bnorm_dir(const deserialized_op &base_op_ref, dir_t &dir) {
     return true;
 }
 
-bool get_bnorm_dt(const deserialized_op &base_op_ref, dnnl_data_type_t &dt) {
+bool get_bnorm_dt(const deserialized_op_t &base_op_ref, dnnl_data_type_t &dt) {
     dt = convert_dt(base_op_ref.in_lts_.front().get_data_type());
     return true;
 }
 
 bool get_bnorm_flag(
-        const deserialized_op &base_op_ref, ::bnorm::flags_t &flag) {
+        const deserialized_op_t &base_op_ref, ::bnorm::flags_t &flag) {
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind == "BatchNormForwardTraining") {
         if (base_op_ref.in_lts_.size() == 3) {
@@ -390,7 +403,7 @@ bool get_bnorm_flag(
 }
 
 ::bnorm::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::bnorm::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(
             bnorm::get_bnorm_desc(base_op_ref, op_setting.desc), res);
@@ -412,8 +425,9 @@ bool get_bnorm_flag(
 namespace concat {
 
 bool get_concat_prb_vdims(
-        const deserialized_op &base_op_ref, prb_vdims_t &prb_vdims) {
+        const deserialized_op_t &base_op_ref, prb_vdims_t &prb_vdims) {
     std::vector<dims_t> vdims;
+    vdims.reserve(base_op_ref.in_lts_.size());
     for (const auto &in : base_op_ref.in_lts_) {
         vdims.push_back(in.shape_);
     }
@@ -421,16 +435,16 @@ bool get_concat_prb_vdims(
     return true;
 }
 
-bool get_concat_sdt_and_ddt(
-        const deserialized_op &base_op_ref, ::concat::settings_t &op_setting) {
+bool get_concat_sdt_and_ddt(const deserialized_op_t &base_op_ref,
+        ::concat::settings_t &op_setting) {
     const auto &dt = convert_dt(base_op_ref.in_lts_.front().get_data_type());
     op_setting.sdt.front() = dt;
     op_setting.ddt.front() = dt;
     return true;
 }
 
-bool get_concat_stag_and_dtag(
-        const deserialized_op &base_op_ref, ::concat::settings_t &op_setting) {
+bool get_concat_stag_and_dtag(const deserialized_op_t &base_op_ref,
+        ::concat::settings_t &op_setting) {
     size_t in_size = base_op_ref.in_lts_.size();
     std::vector<std::string> stags(in_size);
     std::string dtag;
@@ -445,7 +459,7 @@ bool get_concat_stag_and_dtag(
 }
 
 ::concat::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::concat::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(
             concat::get_concat_prb_vdims(base_op_ref, op_setting.prb_vdims),
@@ -468,7 +482,7 @@ bool get_concat_stag_and_dtag(
 
 namespace conv {
 
-bool get_conv_desc(const deserialized_op &base_op_ref, ::conv::desc_t &d) {
+bool get_conv_desc(const deserialized_op_t &base_op_ref, ::conv::desc_t &d) {
     d.g = 1;
     d.sd = d.sh = d.sw = 1;
     d.pd = d.ph = d.pw = -1;
@@ -544,22 +558,23 @@ bool get_conv_desc(const deserialized_op &base_op_ref, ::conv::desc_t &d) {
     return true;
 }
 
-bool get_conv_dir(const deserialized_op &base_op_ref, dir_t &dir) {
+bool get_conv_dir(const deserialized_op_t &base_op_ref, dir_t &dir) {
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind == "Convolution") {
-        dir = base_op_ref.in_lts_.size() > 2 ? dir_t::FWD_B : dir_t::FWD_I;
+        dir = dir_t::FWD_I;
     } else if (op_kind == "ConvolutionBackwardData") {
         dir = dir_t::BWD_D;
     } else if (op_kind == "ConvolutionBackwardWeights") {
         dir = dir_t::BWD_W;
     } else {
+        assert(!"unexpected op_kind");
         return false;
     }
     return true;
 }
 
-bool get_conv_dt(
-        const deserialized_op &base_op_ref, std::vector<dnnl_data_type_t> &dt) {
+bool get_conv_dt(const deserialized_op_t &base_op_ref,
+        std::vector<dnnl_data_type_t> &dt) {
     dnnl_data_type_t src_dt, wei_dt, dst_dt;
     auto in_lt0_dt = convert_dt(base_op_ref.in_lts_[0].get_data_type());
     auto in_lt1_dt = convert_dt(base_op_ref.in_lts_[1].get_data_type());
@@ -589,7 +604,7 @@ bool get_conv_dt(
     return true;
 }
 
-bool get_conv_wtag(const deserialized_op &base_op_ref, std::string &tag) {
+bool get_conv_wtag(const deserialized_op_t &base_op_ref, std::string &tag) {
     std::string weights_format {};
     if (!base_op_ref.get_attr_string(weights_format, "weights_format"))
         return false;
@@ -627,7 +642,7 @@ bool get_conv_wtag(const deserialized_op &base_op_ref, std::string &tag) {
 }
 
 bool get_conv_stag_and_dtag(
-        const deserialized_op &base_op_ref, ::conv::settings_t &op_setting) {
+        const deserialized_op_t &base_op_ref, ::conv::settings_t &op_setting) {
     std::string stag, dtag;
     if (base_op_ref.kind_ == "Convolution") {
         if (!get_driver_tag_by_idx(base_op_ref, stag, 0)
@@ -653,7 +668,8 @@ bool get_conv_stag_and_dtag(
     return true;
 }
 
-::conv::settings_t get_setting(const deserialized_op &base_op_ref, res_t *res) {
+::conv::settings_t get_setting(
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::conv::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(
             conv::get_conv_desc(base_op_ref, op_setting.desc), res);
@@ -661,6 +677,10 @@ bool get_conv_stag_and_dtag(
             conv::get_conv_dir(base_op_ref, op_setting.dir.front()), res);
     DNN_GRAPH_CHECK_SETTINGS(
             conv::get_conv_dt(base_op_ref, op_setting.dt.front()), res);
+    DNN_GRAPH_CHECK_SETTINGS(
+            get_driver_bia_dt(base_op_ref, op_setting.bia_dt.front(),
+                    op_setting.dt.front()[0]),
+            res);
     DNN_GRAPH_CHECK_SETTINGS(
             conv::get_conv_stag_and_dtag(base_op_ref, op_setting), res);
     DNN_GRAPH_CHECK_SETTINGS(
@@ -675,7 +695,8 @@ bool get_conv_stag_and_dtag(
 
 namespace deconv {
 
-bool get_deconv_desc(const deserialized_op &base_op_ref, ::deconv::desc_t &d) {
+bool get_deconv_desc(
+        const deserialized_op_t &base_op_ref, ::deconv::desc_t &d) {
     d.g = 1;
     d.sd = d.sh = d.sw = 1;
     d.pd = d.ph = d.pw = -1;
@@ -746,27 +767,23 @@ bool get_deconv_desc(const deserialized_op &base_op_ref, ::deconv::desc_t &d) {
     return true;
 }
 
-bool get_deconv_dir(const deserialized_op &base_op_ref, dir_t &dir) {
-    bool ret = false;
+bool get_deconv_dir(const deserialized_op_t &base_op_ref, dir_t &dir) {
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind == "ConvTranspose") {
-        dir = base_op_ref.in_lts_.size() > 2 ? dir_t::FWD_B : dir_t::FWD_I;
-        ret = true;
+        dir = dir_t::FWD_I;
     } else if (op_kind == "ConvTransposeBackwardData") {
         dir = dir_t::BWD_D;
-        ret = true;
     } else if (op_kind == "ConvTransposeBackwardWeights") {
         dir = dir_t::BWD_W;
-        ret = true;
     } else {
         assert(!"unexpected op_kind");
         return false;
     }
-    return ret;
+    return true;
 }
 
-bool get_deconv_dt(
-        const deserialized_op &base_op_ref, std::vector<dnnl_data_type_t> &dt) {
+bool get_deconv_dt(const deserialized_op_t &base_op_ref,
+        std::vector<dnnl_data_type_t> &dt) {
     dnnl_data_type_t src_dt, wei_dt, dst_dt;
     auto in_lt0_dt = convert_dt(base_op_ref.in_lts_[0].get_data_type());
     auto in_lt1_dt = convert_dt(base_op_ref.in_lts_[1].get_data_type());
@@ -795,7 +812,7 @@ bool get_deconv_dt(
     return true;
 }
 
-bool get_deconv_wtag(const deserialized_op &base_op_ref, std::string &tag) {
+bool get_deconv_wtag(const deserialized_op_t &base_op_ref, std::string &tag) {
     std::string weights_format {};
     if (!base_op_ref.get_attr_string(weights_format, "weights_format"))
         return false;
@@ -836,7 +853,7 @@ bool get_deconv_wtag(const deserialized_op &base_op_ref, std::string &tag) {
 }
 
 ::deconv::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::deconv::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(
             deconv::get_deconv_desc(base_op_ref, op_setting.desc), res);
@@ -844,6 +861,10 @@ bool get_deconv_wtag(const deserialized_op &base_op_ref, std::string &tag) {
             deconv::get_deconv_dir(base_op_ref, op_setting.dir.front()), res);
     DNN_GRAPH_CHECK_SETTINGS(
             deconv::get_deconv_dt(base_op_ref, op_setting.dt.front()), res);
+    DNN_GRAPH_CHECK_SETTINGS(
+            get_driver_bia_dt(base_op_ref, op_setting.bia_dt.front(),
+                    op_setting.dt.front()[0]),
+            res);
     DNN_GRAPH_CHECK_SETTINGS(
             get_driver_stag_and_dtag(base_op_ref, op_setting.stag.front(),
                     op_setting.dtag.front(),
@@ -899,13 +920,13 @@ get_eltwise_kind_map() {
     return map_;
 }
 
-bool get_flag_use_dst_for_bwd_compute(const deserialized_op &base_op_ref) {
+bool get_flag_use_dst_for_bwd_compute(const deserialized_op_t &base_op_ref) {
     const auto it = base_op_ref.attrs_.find("use_dst");
     if (it == base_op_ref.attrs_.end()) return false;
     return it->second.bool_value_;
 }
 
-bool get_eltwise_dir(const deserialized_op &base_op_ref, dir_t &dir) {
+bool get_eltwise_dir(const deserialized_op_t &base_op_ref, dir_t &dir) {
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind.rfind("Backward") == std::string::npos) {
         dir = dir_t::FWD_D;
@@ -915,13 +936,14 @@ bool get_eltwise_dir(const deserialized_op &base_op_ref, dir_t &dir) {
     return true;
 }
 
-bool get_eltwise_dt(const deserialized_op &base_op_ref, dnnl_data_type_t &dt) {
+bool get_eltwise_dt(
+        const deserialized_op_t &base_op_ref, dnnl_data_type_t &dt) {
     dt = convert_dt(base_op_ref.in_lts_.front().get_data_type());
     return true;
 }
 
 bool get_eltwise_alg(
-        const deserialized_op &base_op_ref, ::eltwise::alg_t &alg) {
+        const deserialized_op_t &base_op_ref, ::eltwise::alg_t &alg) {
     static const std::unordered_map<std::string, ::eltwise::alg_t>
             map_kind_to_alg_dst {
                     {"ClampBackward", ::eltwise::alg_t::CLIP_V2_DST},
@@ -946,7 +968,7 @@ bool get_eltwise_alg(
     return true;
 }
 
-bool get_eltwise_alpha(const deserialized_op &base_op_ref, float &alpha) {
+bool get_eltwise_alpha(const deserialized_op_t &base_op_ref, float &alpha) {
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind == "Clamp" || op_kind == "ClampBackward") {
         base_op_ref.get_attr_f32(alpha, "min");
@@ -967,7 +989,7 @@ bool get_eltwise_alpha(const deserialized_op &base_op_ref, float &alpha) {
     return true;
 }
 
-bool get_eltwise_beta(const deserialized_op &base_op_ref, float &beta) {
+bool get_eltwise_beta(const deserialized_op_t &base_op_ref, float &beta) {
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind == "Reciprocal") {
         beta = -1; // Reciprocal is pow(-1)
@@ -983,7 +1005,7 @@ bool get_eltwise_beta(const deserialized_op &base_op_ref, float &beta) {
 }
 
 ::eltwise::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::eltwise::settings_t op_setting;
     const auto &map_kind_to_alg = get_eltwise_kind_map();
     DNN_GRAPH_CHECK_SETTINGS(
@@ -1017,7 +1039,7 @@ bool get_eltwise_beta(const deserialized_op &base_op_ref, float &beta) {
 
 namespace gnorm {
 
-bool get_gnorm_desc(const deserialized_op &base_op_ref, ::gnorm::desc_t &d) {
+bool get_gnorm_desc(const deserialized_op_t &base_op_ref, ::gnorm::desc_t &d) {
     auto src_dims = base_op_ref.in_lts_[0].shape_;
     if (base_op_ref.has_NXC_format()) {
         src_dims = base_op_ref.get_NCX_shape(0, true);
@@ -1037,7 +1059,7 @@ bool get_gnorm_desc(const deserialized_op &base_op_ref, ::gnorm::desc_t &d) {
     return true;
 }
 
-bool get_gnorm_dir(const deserialized_op &base_op_ref, dir_t &dir) {
+bool get_gnorm_dir(const deserialized_op_t &base_op_ref, dir_t &dir) {
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind == "GroupNorm") {
         bool keep_stats = false;
@@ -1065,8 +1087,8 @@ bool get_gnorm_dir(const deserialized_op &base_op_ref, dir_t &dir) {
     return true;
 }
 
-bool get_gnorm_dt(
-        const deserialized_op &base_op_ref, std::vector<dnnl_data_type_t> &dt) {
+bool get_gnorm_dt(const deserialized_op_t &base_op_ref,
+        std::vector<dnnl_data_type_t> &dt) {
     auto src_dt = convert_dt(base_op_ref.in_lts_[0].get_data_type());
     auto dst_dt = convert_dt(base_op_ref.out_lts_[0].get_data_type());
     dt = {src_dt, dst_dt};
@@ -1074,7 +1096,7 @@ bool get_gnorm_dt(
 }
 
 bool get_gnorm_flags(
-        const deserialized_op &base_op_ref, ::bnorm::flags_t &flags) {
+        const deserialized_op_t &base_op_ref, ::bnorm::flags_t &flags) {
     bool use_affine = false;
     base_op_ref.get_attr_bool(use_affine, "use_affine");
     const auto &op_kind = base_op_ref.kind_;
@@ -1105,7 +1127,7 @@ bool get_gnorm_flags(
     return true;
 }
 
-bool get_gnorm_stag_and_dtag(const deserialized_op &base_op_ref,
+bool get_gnorm_stag_and_dtag(const deserialized_op_t &base_op_ref,
         std::vector<std::vector<std::string>> &tag) {
     // src and dst may have different tags.
     std::string stag, dtag;
@@ -1119,7 +1141,7 @@ bool get_gnorm_stag_and_dtag(const deserialized_op &base_op_ref,
 }
 
 ::gnorm::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::gnorm::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(get_gnorm_desc(base_op_ref, op_setting.desc), res);
     DNN_GRAPH_CHECK_SETTINGS(
@@ -1139,7 +1161,7 @@ bool get_gnorm_stag_and_dtag(const deserialized_op &base_op_ref,
 
 namespace lnorm {
 
-bool get_lnorm_dir(const deserialized_op &base_op_ref, dir_t &dir) {
+bool get_lnorm_dir(const deserialized_op_t &base_op_ref, dir_t &dir) {
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind == "LayerNorm") {
         bool keep_stats = false;
@@ -1166,13 +1188,13 @@ bool get_lnorm_dir(const deserialized_op &base_op_ref, dir_t &dir) {
     return true;
 }
 
-bool get_lnorm_dt(const deserialized_op &base_op_ref, dnnl_data_type_t &dt) {
+bool get_lnorm_dt(const deserialized_op_t &base_op_ref, dnnl_data_type_t &dt) {
     dt = convert_dt(base_op_ref.in_lts_.front().get_data_type());
     return true;
 }
 
 bool get_lnorm_flags(
-        const deserialized_op &base_op_ref, ::bnorm::flags_t &flags) {
+        const deserialized_op_t &base_op_ref, ::bnorm::flags_t &flags) {
     bool use_affine = false;
     base_op_ref.get_attr_bool(use_affine, "use_affine");
     const auto &op_kind = base_op_ref.kind_;
@@ -1215,7 +1237,7 @@ bool get_lnorm_flags(
 }
 
 ::lnorm::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::lnorm::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(
             get_prb_dims(base_op_ref, op_setting.prb_dims), res);
@@ -1238,9 +1260,9 @@ bool get_lnorm_flags(
 namespace matmul {
 
 bool get_matmul_prb_vdims(
-        const deserialized_op &base_op_ref, prb_vdims_t &prb_vdims) {
+        const deserialized_op_t &base_op_ref, prb_vdims_t &prb_vdims) {
 
-    deserialized_op &base_op = const_cast<deserialized_op &>(base_op_ref);
+    deserialized_op_t &base_op = const_cast<deserialized_op_t &>(base_op_ref);
 
     auto &src_dims = base_op.in_lts_[0].shape_;
     auto &wei_dims = base_op.in_lts_[1].shape_;
@@ -1272,8 +1294,8 @@ bool get_matmul_prb_vdims(
     return true;
 }
 
-bool get_matmul_dt(
-        const deserialized_op &base_op_ref, std::vector<dnnl_data_type_t> &dt) {
+bool get_matmul_dt(const deserialized_op_t &base_op_ref,
+        std::vector<dnnl_data_type_t> &dt) {
     auto src_dt = convert_dt(base_op_ref.in_lts_[0].get_data_type());
     auto wei_dt = convert_dt(base_op_ref.in_lts_[1].get_data_type());
     auto dst_dt = convert_dt(base_op_ref.out_lts_[0].get_data_type());
@@ -1282,7 +1304,7 @@ bool get_matmul_dt(
     return true;
 }
 
-bool get_matmul_tags(const deserialized_op &base_op_ref, std::string &stag,
+bool get_matmul_tags(const deserialized_op_t &base_op_ref, std::string &stag,
         std::string &wtag, std::string &dtag, const int &ndims) {
     logical_tensor::dims src_strides = base_op_ref.in_lts_[0].stride_;
     logical_tensor::dims wei_strides = base_op_ref.in_lts_[1].stride_;
@@ -1303,13 +1325,9 @@ bool get_matmul_tags(const deserialized_op &base_op_ref, std::string &stag,
     return true;
 }
 
-bool get_matmul_bia_dt_mask(const deserialized_op &base_op_ref,
-        dnnl_data_type_t &bia_dt, const dnnl_data_type_t dt, int &bia_mask) {
-    bia_dt = dnnl_data_type_undef;
+bool get_matmul_bia_mask(const deserialized_op_t &base_op_ref, int &bia_mask) {
     if (base_op_ref.in_lts_.size() <= 2) return true;
 
-    // bia_dt is the same as src_dt
-    bia_dt = dt;
     const logical_tensor::dims &bias_shape = base_op_ref.in_lts_[2].shape_;
     const logical_tensor::dims &dst_shape = base_op_ref.out_lts_[0].shape_;
     if (bias_shape.size() != dst_shape.size()) {
@@ -1331,7 +1349,7 @@ bool get_matmul_bia_dt_mask(const deserialized_op &base_op_ref,
 }
 
 ::matmul::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::matmul::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(
             matmul::get_matmul_prb_vdims(base_op_ref, op_setting.prb_vdims),
@@ -1339,9 +1357,11 @@ bool get_matmul_bia_dt_mask(const deserialized_op &base_op_ref,
     DNN_GRAPH_CHECK_SETTINGS(
             matmul::get_matmul_dt(base_op_ref, op_setting.dt.front()), res);
     DNN_GRAPH_CHECK_SETTINGS(
-            matmul::get_matmul_bia_dt_mask(base_op_ref,
-                    op_setting.bia_dt.front(), op_setting.dt.front()[0],
-                    op_setting.bia_mask.front()),
+            get_driver_bia_dt(base_op_ref, op_setting.bia_dt.front(),
+                    op_setting.dt.front()[0]),
+            res);
+    DNN_GRAPH_CHECK_SETTINGS(matmul::get_matmul_bia_mask(
+                                     base_op_ref, op_setting.bia_mask.front()),
             res);
     DNN_GRAPH_CHECK_SETTINGS(
             matmul::get_matmul_tags(base_op_ref, op_setting.stag.front(),
@@ -1358,7 +1378,7 @@ bool get_matmul_bia_dt_mask(const deserialized_op &base_op_ref,
 
 namespace pool {
 
-bool get_pool_desc(const deserialized_op &base_op_ref, ::pool::desc_t &d) {
+bool get_pool_desc(const deserialized_op_t &base_op_ref, ::pool::desc_t &d) {
 
     d.sd = d.sh = d.sw = 1;
     d.pd = d.ph = d.pw = -1;
@@ -1421,7 +1441,7 @@ bool get_pool_desc(const deserialized_op &base_op_ref, ::pool::desc_t &d) {
     return true;
 }
 
-bool get_pool_dir(const deserialized_op &base_op_ref, dir_t &dir) {
+bool get_pool_dir(const deserialized_op_t &base_op_ref, dir_t &dir) {
     bool ret = false;
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind == "MaxPool" || op_kind == "AvgPool") {
@@ -1438,17 +1458,16 @@ bool get_pool_dir(const deserialized_op &base_op_ref, dir_t &dir) {
     return ret;
 }
 
-bool get_pool_dt(
-        const deserialized_op &base_op_ref, std::vector<dnnl_data_type_t> &dt) {
+bool get_pool_dt(const deserialized_op_t &base_op_ref,
+        std::vector<dnnl_data_type_t> &dt) {
     auto src_dt = convert_dt(base_op_ref.in_lts_[0].get_data_type());
     auto dst_dt = convert_dt(base_op_ref.out_lts_[0].get_data_type());
     dt = {src_dt, dst_dt};
     return true;
 }
 
-bool get_pool_alg(const deserialized_op &base_op_ref, ::pool::alg_t &alg) {
-
-    const auto op_kind_ = base_op_ref.kind_;
+bool get_pool_alg(const deserialized_op_t &base_op_ref, ::pool::alg_t &alg) {
+    const auto &op_kind_ = base_op_ref.kind_;
     if (op_kind_ == "MaxPool" || op_kind_ == "MaxPoolBackward") {
         alg = ::pool::alg_t::max;
     } else if (op_kind_ == "AvgPool" || op_kind_ == "AvgPoolBackward") {
@@ -1473,7 +1492,8 @@ bool get_pool_alg(const deserialized_op &base_op_ref, ::pool::alg_t &alg) {
     return true;
 }
 
-::pool::settings_t get_setting(const deserialized_op &base_op_ref, res_t *res) {
+::pool::settings_t get_setting(
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::pool::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(
             pool::get_pool_desc(base_op_ref, op_setting.desc), res);
@@ -1494,7 +1514,7 @@ bool get_pool_alg(const deserialized_op &base_op_ref, ::pool::alg_t &alg) {
 namespace prelu {
 
 bool get_prelu_prb_vdims(
-        const deserialized_op &base_op_ref, prb_vdims_t &prb_vdims) {
+        const deserialized_op_t &base_op_ref, prb_vdims_t &prb_vdims) {
 
     auto src_dims = base_op_ref.in_lts_[0].shape_;
     auto wei_dims = base_op_ref.in_lts_[1].shape_;
@@ -1526,7 +1546,7 @@ bool get_prelu_prb_vdims(
     return true;
 }
 
-bool get_prelu_dir(const deserialized_op &base_op_ref, dir_t &dir) {
+bool get_prelu_dir(const deserialized_op_t &base_op_ref, dir_t &dir) {
     bool ret = false;
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind == "PReLU") {
@@ -1542,15 +1562,15 @@ bool get_prelu_dir(const deserialized_op &base_op_ref, dir_t &dir) {
     return ret;
 }
 
-bool get_prelu_sdt(
-        const deserialized_op &base_op_ref, std::vector<dnnl_data_type_t> &dt) {
+bool get_prelu_sdt(const deserialized_op_t &base_op_ref,
+        std::vector<dnnl_data_type_t> &dt) {
     const auto &_dt = convert_dt(base_op_ref.in_lts_.front().get_data_type());
     dt = {_dt, _dt};
     return true;
 }
 
 bool get_prelu_stag(
-        const deserialized_op &base_op_ref, ::prelu::settings_t &op_setting) {
+        const deserialized_op_t &base_op_ref, ::prelu::settings_t &op_setting) {
     std::string tag0, tag1;
     if (!get_driver_tag_by_idx(base_op_ref, tag0)
             || !get_driver_tag_by_idx(base_op_ref, tag1, 1))
@@ -1560,7 +1580,7 @@ bool get_prelu_stag(
 }
 
 ::prelu::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::prelu::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(
             prelu::get_prelu_prb_vdims(base_op_ref, op_setting.prb_vdims), res);
@@ -1581,7 +1601,7 @@ bool get_prelu_stag(
 namespace reduction {
 
 bool get_reduction_prb_vdims(
-        const deserialized_op &base_op_ref, prb_vdims_t &prb_vdims) {
+        const deserialized_op_t &base_op_ref, prb_vdims_t &prb_vdims) {
     const auto &src_dims = base_op_ref.in_lts_[0].shape_;
     auto dst_dims = base_op_ref.out_lts_[0].shape_;
 
@@ -1606,20 +1626,19 @@ bool get_reduction_prb_vdims(
     }
 
     prb_vdims.vdims = {src_dims, dst_dims};
-    prb_vdims.dst_dims = src_dims;
     prb_vdims.ndims = static_cast<int>(src_dims.size());
     return true;
 }
 
-bool get_reduction_dt(const deserialized_op &base_op_ref, dnnl_data_type_t &sdt,
-        dnnl_data_type_t &ddt) {
+bool get_reduction_dt(const deserialized_op_t &base_op_ref,
+        dnnl_data_type_t &sdt, dnnl_data_type_t &ddt) {
     sdt = convert_dt(base_op_ref.in_lts_.front().get_data_type());
     ddt = convert_dt(base_op_ref.out_lts_.front().get_data_type());
     return true;
 }
 
 bool get_reduction_alg(
-        const deserialized_op &base_op_ref, ::reduction::alg_t &alg) {
+        const deserialized_op_t &base_op_ref, ::reduction::alg_t &alg) {
     static const std::unordered_map<std::string, ::reduction::alg_t>
             map_kind_to_alg {{"ReduceSum", ::reduction::alg_t::sum},
                     {"ReduceProd", ::reduction::alg_t::mul},
@@ -1633,14 +1652,14 @@ bool get_reduction_alg(
     return true;
 }
 
-bool get_reduction_p(const deserialized_op &base_op_ref, float &p) {
+bool get_reduction_p(const deserialized_op_t &base_op_ref, float &p) {
     const auto &op_kind = base_op_ref.kind_;
     p = (op_kind == "ReduceL2") ? 2.f : 1.f;
     return true;
 }
 
 ::reduction::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::reduction::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(reduction::get_reduction_prb_vdims(
                                      base_op_ref, op_setting.prb_vdims),
@@ -1668,7 +1687,7 @@ bool get_reduction_p(const deserialized_op &base_op_ref, float &p) {
 
 namespace reorder {
 
-bool get_reorder_dt(const deserialized_op &base_op_ref, dnnl_data_type_t &sdt,
+bool get_reorder_dt(const deserialized_op_t &base_op_ref, dnnl_data_type_t &sdt,
         dnnl_data_type_t &ddt) {
     sdt = convert_dt(base_op_ref.in_lts_.front().get_data_type());
     ddt = convert_dt(base_op_ref.out_lts_.front().get_data_type());
@@ -1681,7 +1700,7 @@ bool get_reorder_dt(const deserialized_op &base_op_ref, dnnl_data_type_t &sdt,
     return true;
 }
 
-bool get_reorder_stag_and_dtag(const deserialized_op &base_op_ref,
+bool get_reorder_stag_and_dtag(const deserialized_op_t &base_op_ref,
         std::string &stag, std::string &dtag) {
     bool ret = get_driver_stag_and_dtag(base_op_ref, stag, dtag);
     if (!ret) return false;
@@ -1689,7 +1708,7 @@ bool get_reorder_stag_and_dtag(const deserialized_op &base_op_ref,
     return ret;
 }
 
-bool get_reorder_attrs(const deserialized_op &base_op_ref,
+bool get_reorder_attrs(const deserialized_op_t &base_op_ref,
         attr_t::arg_scales_t &arg_scales, attr_t::zero_points_t &zp) {
 
     const auto &op_kind = base_op_ref.kind_;
@@ -1777,9 +1796,9 @@ bool get_reorder_attrs(const deserialized_op &base_op_ref,
 }
 
 ::reorder::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::reorder::settings_t op_setting;
-    const auto op_kind = base_op_ref.kind_;
+    const auto &op_kind = base_op_ref.kind_;
 
     DNN_GRAPH_CHECK_SETTINGS(
             get_prb_dims(base_op_ref, op_setting.prb_dims), res);
@@ -1811,7 +1830,7 @@ bool get_reorder_attrs(const deserialized_op &base_op_ref,
 namespace resampling {
 
 bool get_resampling_desc(
-        const deserialized_op &base_op_ref, ::resampling::desc_t &d) {
+        const deserialized_op_t &base_op_ref, ::resampling::desc_t &d) {
     std::string data_format {};
     base_op_ref.get_attr_string(data_format, "data_format");
 
@@ -1836,7 +1855,7 @@ bool get_resampling_desc(
     return true;
 }
 
-bool get_resampling_dir(const deserialized_op &base_op_ref, dir_t &dir) {
+bool get_resampling_dir(const deserialized_op_t &base_op_ref, dir_t &dir) {
 
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind == "Interpolate") {
@@ -1850,7 +1869,7 @@ bool get_resampling_dir(const deserialized_op &base_op_ref, dir_t &dir) {
     return true;
 }
 
-bool get_resampling_dt(const deserialized_op &base_op_ref,
+bool get_resampling_dt(const deserialized_op_t &base_op_ref,
         dnnl_data_type_t &sdt, dnnl_data_type_t &ddt) {
     sdt = convert_dt(base_op_ref.in_lts_.front().get_data_type());
     ddt = convert_dt(base_op_ref.out_lts_.front().get_data_type());
@@ -1858,7 +1877,7 @@ bool get_resampling_dt(const deserialized_op &base_op_ref,
 }
 
 bool get_resampling_alg(
-        const deserialized_op &base_op_ref, ::resampling::alg_t &alg) {
+        const deserialized_op_t &base_op_ref, ::resampling::alg_t &alg) {
     std::string alg_value {};
     base_op_ref.get_attr_string(alg_value, "mode");
     if (alg_value == "linear" || alg_value == "bilinear"
@@ -1874,7 +1893,7 @@ bool get_resampling_alg(
 }
 
 ::resampling::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::resampling::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(
             resampling::get_resampling_desc(base_op_ref, op_setting.desc), res);
@@ -1898,7 +1917,7 @@ bool get_resampling_alg(
 
 namespace softmax {
 
-bool get_softmax_dir(const deserialized_op &base_op_ref, dir_t &dir) {
+bool get_softmax_dir(const deserialized_op_t &base_op_ref, dir_t &dir) {
     const auto &op_kind = base_op_ref.kind_;
     if (op_kind == "SoftMax" || op_kind == "LogSoftmax") {
         dir = dir_t::FWD_D;
@@ -1912,8 +1931,8 @@ bool get_softmax_dir(const deserialized_op &base_op_ref, dir_t &dir) {
     return true;
 };
 
-bool get_softmax_sdt_and_ddt(
-        const deserialized_op &base_op_ref, ::softmax::settings_t &op_setting) {
+bool get_softmax_sdt_and_ddt(const deserialized_op_t &base_op_ref,
+        ::softmax::settings_t &op_setting) {
     const auto &dt = convert_dt(base_op_ref.in_lts_.front().get_data_type());
     op_setting.sdt.front() = dt;
     op_setting.ddt.front() = dt;
@@ -1921,9 +1940,17 @@ bool get_softmax_sdt_and_ddt(
 }
 
 bool get_softmax_alg(
-        const deserialized_op &base_op_ref, ::softmax::alg_t &alg) {
+        const deserialized_op_t &base_op_ref, ::softmax::alg_t &alg) {
     const auto &op_kind = base_op_ref.kind_;
-    if (op_kind == "SoftMax" || op_kind == "SoftMaxBackward") {
+    if (op_kind == "SoftMax") {
+        alg = ::softmax::alg_t::SOFTMAX;
+
+        std::string mode;
+        if (base_op_ref.get_attr_string(mode, "mode")
+                && mode == "inf_as_zero") {
+            alg = ::softmax::alg_t::softmax_accurate_inf_as_zero;
+        }
+    } else if (op_kind == "SoftMaxBackward") {
         alg = ::softmax::alg_t::SOFTMAX;
     } else if (op_kind == "LogSoftmax" || op_kind == "LogSoftmaxBackward") {
         alg = ::softmax::alg_t::LOGSOFTMAX;
@@ -1935,7 +1962,7 @@ bool get_softmax_alg(
 };
 
 ::softmax::settings_t get_setting(
-        const deserialized_op &base_op_ref, res_t *res) {
+        const deserialized_op_t &base_op_ref, res_t *res) {
     ::softmax::settings_t op_setting;
     DNN_GRAPH_CHECK_SETTINGS(
             get_prb_dims(base_op_ref, op_setting.prb_dims), res);
