@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2021 Intel Corporation
+* Copyright 2018-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -28,14 +28,21 @@
 #include "dnn_types.hpp"
 #include "dnnl_common.hpp"
 #include "dnnl_debug.hpp"
-#include "dnnl_memory.hpp"
-#include "perf_report.hpp"
+#include "utils/perf_report.hpp"
+#include "utils/settings.hpp"
 
 #define AOC array_offset_calculator
 
 namespace rnn {
 
-enum alg_t { VANILLA_RNN, VANILLA_LSTM, VANILLA_GRU, LBR_GRU };
+enum alg_t {
+    VANILLA_RNN,
+    VANILLA_LSTM,
+    VANILLA_GRU,
+    LBR_GRU,
+    VANILLA_AUGRU,
+    LBR_AUGRU
+};
 alg_t str2alg(const char *str);
 const char *alg2str(alg_t alg);
 dnnl_alg_kind_t alg2kind(alg_t alg);
@@ -48,7 +55,7 @@ dnnl_alg_kind_t activation2kind(activation_t alg);
 dnnl_rnn_direction_t str2direction(const char *str);
 const char *direction2str(dnnl_rnn_direction_t direction);
 
-enum data_kind_t {
+enum rnn_data_kind_t {
     SRC_LAYER,
     SRC_ITER,
     SRC_ITER_C,
@@ -80,8 +87,19 @@ enum data_kind_t {
     DIFF_WEIGHTS_PEEPHOLE,
     WEIGHTS_PROJECTION,
     DIFF_WEIGHTS_PROJECTION,
+    // AUGRU requires an addtional argument for attention.
+    AUGRU_ATTENTION,
+    DIFF_AUGRU_ATTENTION,
+    KIND_TOTAL,
 };
-const char *data_kind2str(data_kind_t kind);
+const char *rnn_data_kind2str(rnn_data_kind_t kind);
+
+using flags_t = unsigned;
+// XXX: UNDEF is used in activation_t
+const flags_t NONE = dnnl_rnn_flags_undef;
+const flags_t DIFF_WEIGHTS_OVERWRITE = dnnl_rnn_flags_diff_weights_overwrite;
+flags_t str2flags(const char *str);
+std::string flags2str(flags_t flags);
 
 // Gates indices
 enum {
@@ -103,6 +121,10 @@ template <typename Telem>
 struct array_offset_calculator {
     array_offset_calculator() = default;
 
+    template <typename... Targs>
+    array_offset_calculator(const dnn_mem_t &mem, Targs... dims)
+        : base_ptr_(mem ? (Telem *)mem : nullptr), dims_({dims...}) {}
+
     template <typename... Targs>
     array_offset_calculator(Telem *base_ptr, Targs... dims)
         : base_ptr_(base_ptr), dims_({dims...}) {}
@@ -118,6 +140,7 @@ struct array_offset_calculator {
 
     template <typename... Targs>
     Telem &operator()(Targs... Fargs) const {
+        assert(static_cast<bool>(base_ptr_));
         return *(base_ptr_ + offset(1, Fargs...));
     }
 
@@ -146,7 +169,7 @@ private:
         return offset(d + 1, off * dims_[d] + pos, rem...);
     }
 
-    Telem *base_ptr_;
+    Telem *base_ptr_ = nullptr;
     std::vector<int64_t> dims_;
 };
 
@@ -159,7 +182,7 @@ struct desc_t {
     int64_t mb;
     int64_t n_layer;
     int64_t n_iter;
-    const char *name;
+    std::string name;
 };
 int str2desc(desc_t *desc, const char *str);
 std::ostream &operator<<(std::ostream &s, const desc_t &d);
@@ -188,17 +211,21 @@ struct dt_conf_t {
 
     dt_conf_t(const std::string &str) : str_(str) {}
 
-    virtual const entry_t &operator[](data_kind_t kind) const = 0;
+    virtual const entry_t &operator[](rnn_data_kind_t kind) const = 0;
 
     const std::string &str() const { return str_; }
-    bool is_int8() const { return operator[](SRC_LAYER).dt == dnnl_u8; }
+    bool is_int8() const {
+        return operator[](SRC_LAYER).dt == dnnl_u8
+                || operator[](SRC_LAYER).dt == dnnl_s8;
+    }
+    bool is_s8() const { return operator[](SRC_LAYER).dt == dnnl_s8; }
 
-    static const dt_conf_t &create(const std::string &str);
+    static const dt_conf_t &create(const std::string &str, const attr_t &attr);
 
     std::string str_;
 };
 
-struct settings_t {
+struct settings_t : public base_settings_t {
     settings_t() = default;
 
     // ctor to save certain fields from resetting
@@ -210,46 +237,77 @@ struct settings_t {
 
     std::vector<dir_t> prop {FWD_I};
     std::vector<std::string> cfg {"f32"};
+    std::vector<std::vector<std::string>> tag {{tag::any}};
     std::vector<alg_t> alg {VANILLA_RNN};
     std::vector<dnnl_rnn_direction_t> direction {
             dnnl_unidirectional_left2right};
     std::vector<activation_t> activation {RELU};
     std::vector<bool> skip_nonlinear {false};
-    std::vector<bool> trivial_strides {false};
+    std::vector<bool> trivial_strides {true};
     std::vector<bool> with_peephole {false};
     std::vector<bool> with_projection {false};
-    std::vector<int64_t> n_layer {0}, n_iter {0}, mb {0};
+    std::vector<int64_t> n_layer {0}, n_iter {0};
     std::vector<policy_t> scale_policy {policy_t::COMMON};
     std::vector<policy_t> scale_proj_policy {policy_t::COMMON};
-    std::vector<dnnl_scratchpad_mode_t> scratchpad_mode {
-            dnnl_scratchpad_mode_library};
-    unsigned int flags = 0x0;
+    // The default value is changed for a bitwise mode. The reason is RNN
+    // accumulates diff_weights by default, which is undesired behavior for the
+    // mode.
+    // To properly work, `--mode=B` must be specified before the driver name,
+    // otherwise, the driver settings will use `false` as the mode bit was not
+    // set before entering the driver parsing section.
+    std::vector<flags_t> flags {has_bench_mode_bit(mode_bit_t::bitwise)
+                    ? DIFF_WEIGHTS_OVERWRITE
+                    : NONE};
     float alpha = 0.9f, beta = 0.0f;
 
-    const char *perf_template_csv
-            = "perf,%engine%,%impl%,%name%,%prop%,%cfg%,%alg%,%activation%,%"
-              "direction%"
-              ","
-              "%DESC%,%Gops%,%Gfreq%,%-time%,%-Gflops%,%0time%,%0Gflops%";
-    const char *perf_template_def
-            = "perf,%engine%,%impl%,%name%,%prb%,%Gops%,%Gfreq%,%-time%,%-"
-              "Gflops%,%0time%,%0Gflops%";
-    const char *perf_template = perf_template_def;
+    const char *perf_template_csv() const {
+        static const std::string args
+                = "%prop%,%cfg%,%stag%,%alg%,%activation%,%direction%";
+        return perf_template_csv_base(args);
+    }
 
     void reset() { *this = settings_t(perf_template); }
+
+    bool has_single_setup() const override {
+        return prop.size() == 1 && cfg.size() == 1 && tag.size() == 1
+                && alg.size() == 1 && direction.size() == 1
+                && activation.size() == 1 && skip_nonlinear.size() == 1
+                && trivial_strides.size() == 1 && with_peephole.size() == 1
+                && with_projection.size() == 1 && n_layer.size() == 1
+                && n_iter.size() == 1 && scale_policy.size() == 1
+                && scale_proj_policy.size() == 1
+                && base_settings_t::has_single_setup();
+    }
 };
 
 struct prb_t : public desc_t {
-    prb_t(const desc_t &desc, const dt_conf_t &cfg, dir_t prop, alg_t alg,
+    // A ctor with common interface across all drivers.
+    prb_t(const settings_t &s)
+        : prb_t(s.desc, dt_conf_t::create(s.cfg[0], s.attributes.front()),
+                s.tag[0], s.prop[0], s.alg[0], s.with_peephole[0],
+                s.with_projection[0], s.direction[0], s.scale_policy[0],
+                s.scale_proj_policy[0], s.flags[0], s.activation[0], s.alpha,
+                s.beta, s.skip_nonlinear[0], s.trivial_strides[0], s.n_layer[0],
+                s.n_iter[0], s.mb[0], s.attributes.front(), s.ctx_init[0],
+                s.ctx_exe[0], s.impl_filter) {
+        SAFE_V(s.has_single_setup() ? OK : FAIL);
+    }
+
+    prb_t(const desc_t &desc, const dt_conf_t &cfg,
+            const std::vector<std::string> &tag, dir_t prop, alg_t alg,
             bool with_peephole, bool with_projection,
             dnnl_rnn_direction_t direction, policy_t scale_policy,
             policy_t scale_proj_policy, unsigned int flags,
-            activation_t activation, const attr_t &attr, float alpha,
-            float beta, bool skip_nonlinear, bool trivial_strides,
-            int64_t n_layer, int64_t n_iter, int64_t mb = 0)
+            activation_t activation, float alpha, float beta,
+            bool skip_nonlinear, bool trivial_strides, int64_t n_layer,
+            int64_t n_iter, int64_t mb, const attr_t &attr,
+            const thr_ctx_t &ctx_init, const thr_ctx_t &ctx_exe,
+            const impl_filter_t &impl_filter)
         : desc_t(desc)
         , cfg(cfg)
+        , tag(tag)
         , prop(prop2prop_kind(prop))
+        , dir(prop)
         , alg(alg)
         , with_peephole(with_peephole)
         , with_projection(with_projection)
@@ -258,20 +316,25 @@ struct prb_t : public desc_t {
         , wei_proj_scales_policy(scale_proj_policy)
         , flags(flags)
         , activation(activation)
-        , attr(attr)
-        , user_mb(mb)
         , alpha(alpha)
         , beta(beta)
         , skip_nonlinear(skip_nonlinear)
         , trivial_strides(trivial_strides)
+        , user_mb(mb)
         , ops(0.0)
-        , linear_cscale(0.0f) {
+        , linear_cscale(0.0f)
+        , attr(attr)
+        , ctx_init(ctx_init)
+        , ctx_exe(ctx_exe)
+        , impl_filter(impl_filter) {
 
         if (n_layer) this->n_layer = n_layer;
         if (n_iter) this->n_iter = n_iter;
         if (mb) this->mb = mb;
         count_ops();
 
+        broadcast_vector(this->tag, 3);
+
         wei_scales = nullptr;
         wei_proj_scales = nullptr;
         linear_scales = nullptr;
@@ -291,7 +354,7 @@ struct prb_t : public desc_t {
                 wei_scales_mask = 0x18;
                 wei_nscales = dhc * n_gates();
                 break;
-            default: assert(!"unsupported scaling policy");
+            default: SAFE_V(FAIL);
         }
         wei_scales = (float *)zmalloc(sizeof(float) * wei_nscales, 64);
 
@@ -305,18 +368,19 @@ struct prb_t : public desc_t {
                     wei_proj_scales_mask = 0x0;
                     wei_proj_nscales = 1;
                     break;
-                default: assert(!"unsupported scaling policy");
+                default: SAFE_V(FAIL);
             }
             wei_proj_scales
                     = (float *)zmalloc(sizeof(float) * wei_proj_nscales, 64);
         }
 
         set_qparams(-1., 1.);
+        repro = set_repro_line(); // must be last in ctor to collect right info
     }
     ~prb_t() {
-        if (wei_scales) zfree(wei_scales);
-        if (wei_proj_scales) zfree(wei_proj_scales);
-        if (linear_scales) zfree(linear_scales);
+        zfree(wei_scales);
+        zfree(wei_proj_scales);
+        zfree(linear_scales);
     }
 
     float get_wei_scale(int idx) const {
@@ -347,10 +411,13 @@ struct prb_t : public desc_t {
     int64_t n_gates() const {
         return alg == VANILLA_LSTM
                 ? 4
-                : (alg == VANILLA_GRU || alg == LBR_GRU ? 3 : 1);
+                : (alg == VANILLA_GRU || alg == LBR_GRU || alg == VANILLA_AUGRU
+                                        || alg == LBR_AUGRU
+                                ? 3
+                                : 1);
     }
     int64_t n_bias() const {
-        return alg == LBR_GRU ? n_gates() + 1 : n_gates();
+        return alg == LBR_GRU || alg == LBR_AUGRU ? n_gates() + 1 : n_gates();
     }
 
     int64_t dlc(dlc_type_t type) const {
@@ -361,12 +428,59 @@ struct prb_t : public desc_t {
         return 0;
     }
 
-    bool is_int8() const { return cfg[SRC_LAYER].dt == dnnl_u8; }
+    int ndims(rnn_data_kind_t kind) const {
+        switch (kind) {
+            case SRC_LAYER:
+            case DST_LAYER:
+            case AUGRU_ATTENTION:
+            case DIFF_SRC_LAYER:
+            case DIFF_DST_LAYER:
+            case DIFF_AUGRU_ATTENTION: return 3;
+            case SRC_ITER:
+            case SRC_ITER_C:
+            case WEIGHTS_PEEPHOLE:
+            case WEIGHTS_PROJECTION:
+            case BIAS:
+            case DST_ITER:
+            case DST_ITER_C:
+            case DIFF_SRC_ITER:
+            case DIFF_SRC_ITER_C:
+            case DIFF_WEIGHTS_PEEPHOLE:
+            case DIFF_WEIGHTS_PROJECTION:
+            case DIFF_BIAS:
+            case DIFF_DST_ITER:
+            case DIFF_DST_ITER_C: return 4;
+            case WEIGHTS_LAYER:
+            case WEIGHTS_ITER:
+            case DIFF_WEIGHTS_LAYER:
+            case DIFF_WEIGHTS_ITER: return 5;
+            default: assert(!"unknown data kind");
+        }
+        return 0;
+    }
+
+    bool is_int8() const {
+        return cfg[SRC_LAYER].dt == dnnl_u8 || cfg[SRC_LAYER].dt == dnnl_s8;
+    }
+    bool is_u8() const { return cfg[SRC_LAYER].dt == dnnl_u8; }
+    bool is_s8() const { return cfg[SRC_LAYER].dt == dnnl_s8; }
     bool is_lstm_peephole() const { return with_peephole; }
     bool is_lstm_projection() const { return with_projection; }
+    bool is_augru() const { return alg == VANILLA_AUGRU || alg == LBR_AUGRU; }
+
+    // Used to construct memory desc when dimensions are runtime since such mds
+    // can't be used directly from query and memory objects can't be constructed.
+    benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> get_md(int arg) const {
+        assert(!"No runtime dimensions support for this driver!");
+        return make_benchdnn_dnnl_wrapper<dnnl_memory_desc_t>(nullptr);
+    }
+
+    const char *str() const { return repro.c_str(); }
 
     const dt_conf_t &cfg;
+    std::vector<std::string> tag;
     dnnl_prop_kind_t prop;
+    dir_t dir; // Same as `prop`, for compatibility. TODO: remove me;
     alg_t alg;
     bool with_peephole, with_projection;
     dnnl_rnn_direction_t direction;
@@ -374,10 +488,17 @@ struct prb_t : public desc_t {
     policy_t wei_proj_scales_policy;
     unsigned int flags;
     activation_t activation;
-    attr_t attr;
-    int64_t user_mb;
     float alpha;
     float beta;
+    bool skip_nonlinear;
+    bool trivial_strides;
+    int64_t user_mb;
+    double ops;
+    float linear_cscale;
+    bool inplace = false; // Lacks placement, always considered `false`.
+    attr_t attr;
+    thr_ctx_t ctx_init, ctx_exe;
+    impl_filter_t impl_filter;
 
     float data_scale, data_shift;
 
@@ -389,28 +510,23 @@ struct prb_t : public desc_t {
     int wei_proj_nscales;
     int wei_proj_scales_mask;
 
-    bool skip_nonlinear;
-    bool trivial_strides;
-    double ops;
     float *linear_scales;
-    float linear_cscale;
 
 private:
+    std::string repro;
+
+    std::string set_repro_line();
+
     /* Todo: fused the two functions in set_shifts_scales */
     void set_qparams(float fp_min, float fp_max);
     void set_tparams(float fp_min, float fp_max);
     prb_t(const prb_t &) = delete;
     prb_t &operator=(const prb_t &) = delete;
 };
-std::ostream &operator<<(std::ostream &s, const prb_t &prb);
 
 struct perf_report_t : public base_perf_report_t {
-    using base_perf_report_t::base_perf_report_t;
-
-    void report(const prb_t *prb, const res_t *res, const char *prb_str) {
-        p_ = prb;
-        base_report(res, prb_str);
-    }
+    perf_report_t(const prb_t *prb, const char *perf_template)
+        : base_perf_report_t(perf_template), p_(prb) {}
 
     void dump_alg(std::ostream &s) const override { s << alg2str(p_->alg); }
 
@@ -435,46 +551,47 @@ struct perf_report_t : public base_perf_report_t {
 
     double ops() const override { return p_->ops; }
     const int64_t *user_mb() const override { return &p_->user_mb; }
-    const char *name() const override { return p_->name; }
+    const attr_t *attr() const override { return &p_->attr; }
+    const thr_ctx_t *ctx_init() const override { return &p_->ctx_init; }
+    const thr_ctx_t *ctx_exe() const override { return &p_->ctx_exe; }
+    const std::string *name() const override { return &p_->name; }
     const dnnl_prop_kind_t *prop() const override { return &p_->prop; }
+    const std::vector<std::string> *stag() const override { return &p_->tag; }
 
 private:
-    const prb_t *p_ = nullptr;
+    const prb_t *p_;
 };
 
 void prepare_ws_fwd(const prb_t &prb, std::vector<float> &ws_fwd_buffer,
         AOC<float> &ws_src_layer, AOC<float> &ws_src_iter,
         AOC<float> &ws_src_iter_c, AOC<float> &ws_gates, AOC<float> &ws_ht);
 
-void rnn_linear_fwd(const prb_t &prb, const float *src_layer_,
-        const float *src_iter_, const float *src_iter_c_,
-        const float *weights_layer_, const float *weights_iter_,
-        const float *weights_peephole_, const float *weights_projection_,
-        const float *bias_, float *dst_layer_, float *dst_iter_,
-        float *dst_iter_c_, const AOC<float> &ws_src_layer,
-        const AOC<float> &ws_src_iter, const AOC<float> &ws_src_iter_c,
-        const AOC<float> &ws_gates, const AOC<float> &ws_ht);
+void rnn_linear_fwd(const prb_t &prb, const args_t &args,
+        const AOC<float> &ws_src_layer, const AOC<float> &ws_src_iter,
+        const AOC<float> &ws_src_iter_c, const AOC<float> &ws_gates,
+        const AOC<float> &ws_ht);
 
-void compute_ref_fwd(const prb_t &prb, dnn_mem_t &src_layer_m,
-        dnn_mem_t &src_iter_m, dnn_mem_t &src_iter_c_m,
-        dnn_mem_t &weights_layer_m, dnn_mem_t &weights_iter_m,
-        dnn_mem_t &weights_peephole_m, dnn_mem_t &weights_projection_m,
-        dnn_mem_t &bias_m, dnn_mem_t &dst_layer_m, dnn_mem_t &dst_iter_m,
-        dnn_mem_t &dst_iter_c_m);
+dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args);
+void setup_cmp(compare::compare_t &cmp, const prb_t *prb, data_kind_t kind,
+        const args_t &ref_args);
+std::vector<int> supported_exec_args(dir_t dir);
+int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
+        dnnl_primitive_t prim, const prb_t *prb, res_t *res,
+        dnnl_primitive_t prim_ref = nullptr);
 
-void compute_ref_bwd(const prb_t &prb, dnn_mem_t &src_layer_m,
-        dnn_mem_t &src_iter_m, dnn_mem_t &src_iter_c_m,
-        dnn_mem_t &diff_dst_layer_m, dnn_mem_t &diff_dst_iter_m,
-        dnn_mem_t &diff_dst_iter_c_m, dnn_mem_t &weights_layer_m,
-        dnn_mem_t &weights_iter_m, dnn_mem_t &weights_peephole_m,
-        dnn_mem_t &weights_projection_m, dnn_mem_t &bias_m,
-        dnn_mem_t &dst_layer_m, dnn_mem_t &dst_iter_m, dnn_mem_t &dst_iter_c_m,
-        dnn_mem_t &diff_src_layer_m, dnn_mem_t &diff_src_iter_m,
-        dnn_mem_t &diff_src_iter_c_m, dnn_mem_t &diff_weights_layer_m,
-        dnn_mem_t &diff_weights_iter_m, dnn_mem_t &diff_weights_peephole_m,
-        dnn_mem_t &diff_weights_projection_m, dnn_mem_t &diff_bias_m);
+void skip_unimplemented_prb(const prb_t *prb, res_t *res);
+void skip_invalid_prb(const prb_t *prb, res_t *res);
+void compute_ref(const prb_t *prb, const args_t &args,
+        dnnl_primitive_t prim_ref = nullptr);
+void compute_ref_fwd(const prb_t &prb, const args_t &args);
+void compute_ref_bwd(const prb_t &prb, const args_t &args);
 
-int doit(const prb_t &prb, res_t *res);
+int createit(std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+        const prb_t &prb, res_t *res);
+int checkit(std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+        const prb_t *prb, res_t *res);
+int doit(const std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+        const prb_t &prb, res_t *res);
 int bench(int argc, char **argv);
 
 } // namespace rnn
