@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 * Copyright 2024 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -33,6 +33,10 @@
 
 #include "softmax/softmax.hpp"
 
+// Here only for internal `softmax_accurate_inf_as_zero` alg_kind. Must be
+// removed once alg_kind gets converted to a public value.
+#include "src/common/c_types_map.hpp"
+
 namespace softmax {
 
 dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
@@ -44,7 +48,11 @@ dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
             force_f32_dt ? dnnl_f32 : prb->ddt, prb->dtag);
 
     dnnl_alg_kind_t alg_kind = dnnl_softmax_accurate;
-    if (prb->alg == LOGSOFTMAX) alg_kind = dnnl_softmax_log;
+    if (prb->alg == LOGSOFTMAX)
+        alg_kind = dnnl_softmax_log;
+    else if (prb->alg == SOFTMAX_INF_AS_ZERO)
+        alg_kind = static_cast<dnnl_alg_kind_t>(
+                dnnl::impl::alg_kind::softmax_accurate_inf_as_zero);
 
     attr_args_t attr_args;
     attr_args.prepare_post_ops_mds(prb->attr, prb->ndims, prb->dims.data());
@@ -243,7 +251,7 @@ void setup_cmp(compare::compare_t &cmp, const prb_t *prb, data_kind_t kind,
 #if DNNL_AARCH64 || defined(DNNL_SYCL_HIP) || defined(DNNL_SYCL_CUDA)
     // MIOpen and ACL softmax accumulate in F16, but oneDNN now expects accumulation in
     // F32, this partially reverts 6727bbe8. For more information on ACL softmax, see
-    // https://github.com/oneapi-src/oneDNN/issues/1819
+    // https://github.com/uxlfoundation/oneDNN/issues/1819
     // Similarly, for bf16 on AArch64, the relaxed threshold is necessary due to
     // minor accuracy drops observed compared to f32
     const float trh = trh_f32;
@@ -258,11 +266,6 @@ void setup_cmp(compare::compare_t &cmp, const prb_t *prb, data_kind_t kind,
 #endif
     cmp.set_threshold(trh);
 
-    // LogSoftMax is unstable enough when there are attributes on top.
-    const bool compare_with_norm
-            = (prb->alg == alg_t::LOGSOFTMAX && !prb->attr.is_def());
-    cmp.set_norm_validation_mode(compare_with_norm);
-
     const int64_t axis_size = prb->dims[prb->axis];
     const int64_t n_zeros = (prb->ddt == dnnl_s8 || prb->ddt == dnnl_u8)
             ? (axis_size - 1)
@@ -364,12 +367,14 @@ int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
                 break;
             case DNNL_ARG_DST:
                 if (!is_fwd_prim) {
-                    const bool neg_sign = prb->alg == SOFTMAX ? true : false;
+                    const bool neg_sign = prb->alg == SOFTMAX
+                            || prb->alg == SOFTMAX_INF_AS_ZERO;
                     SAFE(fill_data_bwd(DST, prb, mem, ref_mem, neg_sign), WARN);
                 }
                 break;
             case DNNL_ARG_DIFF_DST: {
-                const bool neg_sign = prb->alg == SOFTMAX ? true : false;
+                const bool neg_sign = prb->alg == SOFTMAX
+                        || prb->alg == SOFTMAX_INF_AS_ZERO;
                 SAFE(fill_data_bwd(DIFF_DST, prb, mem, ref_mem, !neg_sign),
                         WARN);
                 // Need a copy of source data for inplace mode for bitwise
@@ -418,10 +423,15 @@ int createit(std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
     return OK;
 }
 
-int check_cacheit(
-        std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+int checkit(std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
         const prb_t *prb, res_t *res) {
-    return check_caches(v_prim[0], prb, res);
+    if (has_bench_mode_bit(mode_bit_t::exec)) {
+        SAFE(check_total_size(res), WARN);
+    }
+    if (has_bench_mode_bit(mode_bit_t::corr)) {
+        SAFE(check_caches(v_prim[0], prb, res), WARN);
+    }
+    return OK;
 }
 
 int doit(const std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
