@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -30,6 +30,8 @@ alg_t str2alg(const char *str) {
     CASE(softmax_accurate);
     CASE(LOGSOFTMAX);
     CASE(softmax_log);
+    CASE(SOFTMAX_INF_AS_ZERO);
+    CASE(softmax_accurate_inf_as_zero);
 #undef CASE
     assert(!"unknown algorithm");
     return UNDEF;
@@ -38,6 +40,7 @@ alg_t str2alg(const char *str) {
 const char *alg2str(alg_t alg) {
     if (alg == SOFTMAX) return "SOFTMAX";
     if (alg == LOGSOFTMAX) return "LOGSOFTMAX";
+    if (alg == SOFTMAX_INF_AS_ZERO) return "SOFTMAX_INF_AS_ZERO";
     assert(!"unknown algorithm");
     return "UNDEF";
 }
