@@ -17,7 +17,6 @@
 #include <functional>
 #include <random>
 
-#include "oneapi/dnnl/dnnl_graph.hpp"
 #include "gtest/gtest.h"
 
 #include "graph/unit/backend/dnnl/dnnl_test_common.hpp"
@@ -57,7 +56,7 @@ TEST(test_large_partition_execute, Int8Resnet50Stage2Block) {
     graph::engine_t *eng = get_engine();
     graph::stream_t *strm = get_stream();
 
-    utils::id_generator id_gen;
+    utils::id_generator_t id_gen;
     graph::graph_t g(eng->kind());
     utils::construct_int8_resnet50_stage2_block(
             &g, id_gen, 3, false, false, 1.2f, 1, 0);
@@ -93,7 +92,7 @@ TEST(test_large_partition_execute, Int8Resnet50Stage2Block) {
     graph::compiled_partition_t cp(p);
     ASSERT_EQ(p.compile(&cp, inputs, outputs, eng), graph::status::success);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts, ref_outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts, ref_outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -110,8 +109,8 @@ TEST(test_large_partition_execute, Int8Resnet50Stage2Block) {
     ASSERT_EQ(run_graph(g, inputs_ts, ref_outputs_ts, *eng, *strm),
             graph::status::success);
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 
@@ -124,7 +123,7 @@ TEST(test_large_partition_execute, Int8Resnet50Stage2BlockWithZeroZps) {
     graph::engine_t *eng = get_engine();
     graph::stream_t *strm = get_stream();
 
-    utils::id_generator id_gen;
+    utils::id_generator_t id_gen;
     graph::graph_t g(eng->kind());
     int64_t zps_postbinary = eng->kind() == graph::engine_kind::gpu ? 0 : 78;
     utils::construct_int8_resnet50_stage2_block(
@@ -161,7 +160,7 @@ TEST(test_large_partition_execute, Int8Resnet50Stage2BlockWithZeroZps) {
     graph::compiled_partition_t cp(p);
     ASSERT_EQ(p.compile(&cp, inputs, outputs, eng), graph::status::success);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts, ref_outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts, ref_outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -178,8 +177,8 @@ TEST(test_large_partition_execute, Int8Resnet50Stage2BlockWithZeroZps) {
     ASSERT_EQ(run_graph(g, inputs_ts, ref_outputs_ts, *eng, *strm),
             graph::status::success);
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 
@@ -192,7 +191,7 @@ TEST(test_large_partition_execute, Int8Resnet50Stage2BlockWithQuantWei) {
     graph::engine_t *eng = get_engine();
     graph::stream_t *strm = get_stream();
 
-    utils::id_generator id_gen;
+    utils::id_generator_t id_gen;
     graph::graph_t g(eng->kind());
     utils::construct_int8_resnet50_stage2_block(&g, id_gen, 3, true, true);
     g.finalize();
@@ -227,7 +226,7 @@ TEST(test_large_partition_execute, Int8Resnet50Stage2BlockWithQuantWei) {
     graph::compiled_partition_t cp(p);
     ASSERT_EQ(p.compile(&cp, inputs, outputs, eng), graph::status::success);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -240,8 +239,8 @@ TEST(test_large_partition_execute, Int8Resnet50Stage2BlockWithQuantWei) {
         outputs_ts.emplace_back(compiled_output, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -250,7 +249,7 @@ TEST(test_large_partition_execute, F32Resnet50Stage2Block) {
     graph::engine_t *eng = get_engine();
     graph::stream_t *strm = get_stream();
 
-    utils::id_generator id_gen;
+    utils::id_generator_t id_gen;
     graph::graph_t g(eng->kind());
     utils::construct_f32_resnet50_stage2_block(
             &g, id_gen, 3, /* use biasadd */ true);
@@ -263,6 +262,10 @@ TEST(test_large_partition_execute, F32Resnet50Stage2Block) {
     ASSERT_EQ(g.get_num_partitions(), 1U);
     auto part = g.get_partitions()[0];
 
+    // set constant tensor cache capacity as 1GB
+    dnnl_graph_set_constant_tensor_cache_capacity(
+            static_cast<dnnl_engine_kind_t>(eng->kind()), 1024);
+
     // compile
     graph::partition_t p;
     p.init(part);
@@ -290,11 +293,10 @@ TEST(test_large_partition_execute, F32Resnet50Stage2Block) {
 
     std::vector<std::vector<float>> inputs_data;
     std::vector<std::vector<float>> outputs_data, ref_outputs_data;
-    std::vector<test_tensor> inputs_ts, outputs_ts, ref_outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts, ref_outputs_ts;
 
     for (auto &lt : inputs) {
-        inputs_data.emplace_back(
-                std::vector<float>(utils::product(ltw(lt).vdims())));
+        inputs_data.emplace_back(utils::product(ltw(lt).vdims()));
         fill_data(inputs_data.back(), ltw(lt).data_type());
         inputs_ts.emplace_back(*lt, eng, inputs_data.back());
     }
@@ -304,32 +306,30 @@ TEST(test_large_partition_execute, F32Resnet50Stage2Block) {
         cp.query_logical_tensor(lt->id, &compiled_output);
         const std::vector<int64_t> dims = ltw(compiled_output).vdims();
         auto size = utils::product(dims);
-        outputs_data.emplace_back(std::vector<float>(size));
+        outputs_data.emplace_back(size);
         outputs_ts.emplace_back(compiled_output, eng, outputs_data.back());
-        ref_outputs_data.emplace_back(std::vector<float>(size));
+        ref_outputs_data.emplace_back(size);
         ref_outputs_ts.emplace_back(
                 compiled_output, eng, ref_outputs_data.back());
     }
 
-    // set constant tensor cache capacity as 1GB
-    dnnl::graph::set_constant_tensor_cache_capacity(
-            static_cast<engine::kind>(eng->kind()), 1024);
-
     ASSERT_EQ(run_graph(g, inputs_ts, ref_outputs_ts, *eng, *strm),
             graph::status::success);
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     // execute another iteration to test constant cache hit
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     // disable constant tensor cache and then to test constant cache miss
-    dnnl::graph::set_constant_tensor_cache_capacity(
-            static_cast<engine::kind>(eng->kind()), 0);
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    dnnl_graph_set_constant_tensor_cache_capacity(
+            static_cast<dnnl_engine_kind_t>(eng->kind()), 0);
+    graph::compiled_partition_t cp1(p);
+    ASSERT_EQ(p.compile(&cp1, inputs, outputs, eng), graph::status::success);
+    ASSERT_EQ(cp1.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 
@@ -342,7 +342,7 @@ TEST(test_large_partition_execute, ItexInt8Resnet50Stage2Block) {
     graph::engine_t *eng = get_engine();
     graph::stream_t *strm = get_stream();
 
-    utils::id_generator id_gen;
+    utils::id_generator_t id_gen;
     graph::graph_t g(eng->kind());
     utils::construct_itex_int8_resnet50_stage2_block(&g, id_gen, 3);
     g.finalize();
@@ -378,7 +378,7 @@ TEST(test_large_partition_execute, ItexInt8Resnet50Stage2Block) {
     graph::compiled_partition_t cp(p);
     ASSERT_EQ(p.compile(&cp, inputs, outputs, eng), graph::status::success);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -392,12 +392,12 @@ TEST(test_large_partition_execute, ItexInt8Resnet50Stage2Block) {
     }
 
     std::cout << "----------------iter 1----------------\n";
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     std::cout << "----------------iter 2----------------\n";
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -517,7 +517,7 @@ TEST(test_large_partition_execute, Int8Mha_CPU) {
     graph::compiled_partition_t cp(p);
     ASSERT_EQ(p.compile(&cp, inputs, outputs, eng), graph::status::success);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -530,8 +530,8 @@ TEST(test_large_partition_execute, Int8Mha_CPU) {
         outputs_ts.emplace_back(compiled_output, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -574,7 +574,7 @@ TEST(test_large_partition_execute, Int8DistilBertMha) {
     graph::compiled_partition_t cp(p);
     ASSERT_EQ(p.compile(&cp, inputs, outputs, eng), graph::status::success);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -587,8 +587,8 @@ TEST(test_large_partition_execute, Int8DistilBertMha) {
         outputs_ts.emplace_back(compiled_output, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -631,7 +631,7 @@ TEST(test_large_partition_execute, Int8GptMha) {
     graph::compiled_partition_t cp(p);
     ASSERT_EQ(p.compile(&cp, inputs, outputs, eng), graph::status::success);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -644,8 +644,8 @@ TEST(test_large_partition_execute, Int8GptMha) {
         outputs_ts.emplace_back(compiled_output, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -690,7 +690,7 @@ TEST(test_large_partition_execute, F32Mha) {
     graph::compiled_partition_t cp(p);
     ASSERT_EQ(p.compile(&cp, inputs, outputs, eng), graph::status::success);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -703,8 +703,8 @@ TEST(test_large_partition_execute, F32Mha) {
         outputs_ts.emplace_back(compiled_output, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -751,7 +751,7 @@ TEST(test_large_partition_execute, F32DistilBertMha) {
 
     using ltw = graph::logical_tensor_wrapper_t;
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -768,8 +768,8 @@ TEST(test_large_partition_execute, F32DistilBertMha) {
         outputs_ts.emplace_back(compiled_output, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -815,7 +815,7 @@ TEST(test_large_partition_execute, F32GptMha) {
 
     using ltw = graph::logical_tensor_wrapper_t;
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -832,8 +832,8 @@ TEST(test_large_partition_execute, F32GptMha) {
         outputs_ts.emplace_back(compiled_output, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -876,7 +876,7 @@ TEST(test_large_partition_execute, F32JaxMha) {
     graph::compiled_partition_t cp(p);
     ASSERT_EQ(p.compile(&cp, inputs, outputs, eng), graph::status::success);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -889,8 +889,8 @@ TEST(test_large_partition_execute, F32JaxMha) {
         outputs_ts.emplace_back(compiled_output, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -938,7 +938,7 @@ TEST(test_large_partition_execute, F32JaxMqa) {
     // Set back to avoid affecting other tests
     custom_setenv("_ONEDNN_GRAPH_SDPA_FORCE_PRIMITIVE", "0", 1);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -951,8 +951,8 @@ TEST(test_large_partition_execute, F32JaxMqa) {
         outputs_ts.emplace_back(compiled_output, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -1026,13 +1026,13 @@ TEST(test_large_partition_execute, Bf16Mha_CPU) {
     using ltw = graph::logical_tensor_wrapper_t;
 
     std::vector<std::vector<uint16_t>> inputs_data;
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         // set all the input value to 1.f, then the value after softmax should
         // be 1.f/seq_len, and the final output should be 1.f
-        inputs_data.emplace_back(std::vector<uint16_t>(
-                utils::product(ltw(lt).vdims()), f32_to_bf16(1.f)));
+        inputs_data.emplace_back(
+                utils::product(ltw(lt).vdims()), f32_to_bf16(1.f));
         inputs_ts.emplace_back(*lt, eng, inputs_data.back());
     }
 
@@ -1042,8 +1042,8 @@ TEST(test_large_partition_execute, Bf16Mha_CPU) {
         outputs_ts.emplace_back(compiled_output, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 
@@ -1101,7 +1101,7 @@ TEST(test_large_partition_execute, Bf16GptMha) {
 
     using ltw = graph::logical_tensor_wrapper_t;
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -1118,8 +1118,8 @@ TEST(test_large_partition_execute, Bf16GptMha) {
         outputs_ts.emplace_back(compiled_output, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -1172,7 +1172,7 @@ TEST(test_large_partition_execute, Bf16DistilBertMha) {
 
     using ltw = graph::logical_tensor_wrapper_t;
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : inputs) {
         inputs_ts.emplace_back(*lt, eng);
@@ -1189,8 +1189,8 @@ TEST(test_large_partition_execute, Bf16DistilBertMha) {
         outputs_ts.emplace_back(compiled_output, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -1237,7 +1237,7 @@ TEST(test_large_partition_execute, Int8Bf16Mha_CPU) {
     graph::compiled_partition_t cp(p);
     ASSERT_EQ(p.compile(&cp, inputs, outputs, eng), graph::status::success);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : partition_inputs) {
         inputs_ts.emplace_back(lt, eng);
@@ -1247,8 +1247,8 @@ TEST(test_large_partition_execute, Int8Bf16Mha_CPU) {
         outputs_ts.emplace_back(lt, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -1296,7 +1296,7 @@ TEST(test_large_partition_execute, Int8Bf16DistilBertMha_CPU) {
     graph::compiled_partition_t cp(p);
     ASSERT_EQ(p.compile(&cp, inputs, outputs, eng), graph::status::success);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : partition_inputs) {
         inputs_ts.emplace_back(lt, eng);
@@ -1306,8 +1306,8 @@ TEST(test_large_partition_execute, Int8Bf16DistilBertMha_CPU) {
         outputs_ts.emplace_back(lt, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
@@ -1355,7 +1355,7 @@ TEST(test_large_partition_execute, Int8Bf16GptMha_CPU) {
     graph::compiled_partition_t cp(p);
     ASSERT_EQ(p.compile(&cp, inputs, outputs, eng), graph::status::success);
 
-    std::vector<test_tensor> inputs_ts, outputs_ts;
+    std::vector<test_tensor_t> inputs_ts, outputs_ts;
 
     for (auto &lt : partition_inputs) {
         inputs_ts.emplace_back(lt, eng);
@@ -1365,8 +1365,8 @@ TEST(test_large_partition_execute, Int8Bf16GptMha_CPU) {
         outputs_ts.emplace_back(lt, eng);
     }
 
-    ASSERT_EQ(cp.execute(strm, test_tensor::to_graph_tensor(inputs_ts),
-                      test_tensor::to_graph_tensor(outputs_ts)),
+    ASSERT_EQ(cp.execute(strm, test_tensor_t::to_graph_tensor(inputs_ts),
+                      test_tensor_t::to_graph_tensor(outputs_ts)),
             graph::status::success);
     strm->wait();
 }
