@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2023 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -171,10 +171,10 @@ TEST(test_pool_execute_subgraph_int8, PoolAdd) {
         g.add_op(&qout_op);
         g.finalize();
 
-        test_tensor src_s8_ts(src_s8, engine, src_s8_data);
-        test_tensor other_s8_ts(other_s8, engine, other_s8_data);
-        test_tensor case1_dst_s8_ts(dst_s8, engine, case1_dst_s8_data);
-        test_tensor case2_dst_s8_ts(dst_s8, engine, case2_dst_s8_data);
+        test_tensor_t src_s8_ts(src_s8, engine, src_s8_data);
+        test_tensor_t other_s8_ts(other_s8, engine, other_s8_data);
+        test_tensor_t case1_dst_s8_ts(dst_s8, engine, case1_dst_s8_data);
+        test_tensor_t case2_dst_s8_ts(dst_s8, engine, case2_dst_s8_data);
 
         // -------------------------case 1----------------------------------
         ASSERT_EQ(run_graph(g, {src_s8_ts, other_s8_ts}, {case1_dst_s8_ts},
@@ -284,7 +284,7 @@ TEST(test_pool_execute_subgraph_fp32, Pool3Postops) {
         std::vector<graph::op_t> post_ops {};
         for (size_t i = 0; i < post_op_ts.size(); ++i) {
             auto pop_t = post_op_ts[i];
-            post_ops.emplace_back(graph::op_t {i + 1, pop_t, "post op"});
+            post_ops.emplace_back(i + 1, pop_t, "post op");
 
             post_ops.back().add_input(lt_vec[lt_idx]);
             if (std::find(binary_op_ts.begin(), binary_op_ts.end(), pop_t)
@@ -303,13 +303,13 @@ TEST(test_pool_execute_subgraph_fp32, Pool3Postops) {
             g.add_op(&pop);
         g.finalize();
 
-        std::vector<test_tensor> src_tss {};
+        std::vector<test_tensor_t> src_tss {};
         for (size_t i = 0; i < input_lts.size(); ++i)
             src_tss.emplace_back(lt_vec[input_lts[i]], engine, src_datas[i]);
 
         // -------------------------case 1----------------------------------
         std::vector<float> case1_out_data(product(pool_dst_shape));
-        test_tensor case1_dst_ts(lt_vec[lt_idx], engine, case1_out_data);
+        test_tensor_t case1_dst_ts(lt_vec[lt_idx], engine, case1_out_data);
 
         ASSERT_EQ(run_graph(g, src_tss, {case1_dst_ts}, *engine, *strm),
                 graph::status::success);
@@ -335,9 +335,9 @@ TEST(test_pool_execute_subgraph_fp32, Pool3Postops) {
         p.compile(&cp, lt_ins, lt_outs, engine);
 
         std::vector<float> case2_out_data(product(pool_dst_shape));
-        test_tensor case2_dst_ts(lt_vec[lt_idx], engine, case2_out_data);
+        test_tensor_t case2_dst_ts(lt_vec[lt_idx], engine, case2_out_data);
 
-        cp.execute(strm, test_tensor::to_graph_tensor(src_tss),
+        cp.execute(strm, test_tensor_t::to_graph_tensor(src_tss),
                 {case2_dst_ts.get()});
         strm->wait();
 
@@ -400,8 +400,8 @@ TEST(test_pool_execute, AvgPoolExcludePad) {
     cp.query_logical_tensor(dst_lt.id, &lt);
     ASSERT_EQ(lt.layout_type, graph::layout_type::strided);
 
-    test_tensor src_ts(src_lt, eng, src);
-    test_tensor dst_ts(lt, eng, dst);
+    test_tensor_t src_ts(src_lt, eng, src);
+    test_tensor_t dst_ts(lt, eng, dst);
 
     graph::stream_t *strm = get_stream();
     cp.execute(strm, {src_ts.get()}, {dst_ts.get()});
@@ -463,8 +463,8 @@ TEST(test_pool_execute, AvgPoolIncludePad) {
     cp.query_logical_tensor(dst_lt.id, &lt);
     ASSERT_EQ(lt.layout_type, graph::layout_type::strided);
 
-    test_tensor src_ts(src_lt, eng, src);
-    test_tensor dst_ts(lt, eng, dst);
+    test_tensor_t src_ts(src_lt, eng, src);
+    test_tensor_t dst_ts(lt, eng, dst);
 
     graph::stream_t *strm = get_stream();
     cp.execute(strm, {src_ts.get()}, {dst_ts.get()});
@@ -522,8 +522,8 @@ TEST(test_pool_execute, AvgPoolBackwardExcludePad) {
     cp.query_logical_tensor(diff_src_lt.id, &lt);
     ASSERT_EQ(lt.layout_type, graph::layout_type::strided);
 
-    test_tensor diff_dst_ts(diff_dst_lt, eng, diff_dst);
-    test_tensor diff_src_ts(lt, eng, diff_src);
+    test_tensor_t diff_dst_ts(diff_dst_lt, eng, diff_dst);
+    test_tensor_t diff_src_ts(lt, eng, diff_src);
 
     graph::stream_t *strm = get_stream();
     cp.execute(strm, {diff_dst_ts.get()}, {diff_src_ts.get()});
@@ -581,8 +581,8 @@ TEST(test_pool_execute, AvgPoolBackwardIncludePad) {
     cp.query_logical_tensor(diff_src_lt.id, &lt);
     ASSERT_EQ(lt.layout_type, graph::layout_type::strided);
 
-    test_tensor diff_dst_ts(diff_dst_lt, eng, diff_dst);
-    test_tensor diff_src_ts(lt, eng, diff_src);
+    test_tensor_t diff_dst_ts(diff_dst_lt, eng, diff_dst);
+    test_tensor_t diff_src_ts(lt, eng, diff_src);
 
     graph::stream_t *strm = get_stream();
     cp.execute(strm, {diff_dst_ts.get()}, {diff_src_ts.get()});
@@ -689,9 +689,9 @@ TEST(test_pool_execute_subgraph_int8, Avgpool) {
         g.add_op(&qout_op);
         g.finalize();
 
-        test_tensor src_u8_ts(src_u8, engine, src_u8_data);
-        test_tensor dst_u8_ts(dst_u8, engine, case1_out_data);
-        test_tensor dst_u8_case2_ts(dst_u8, engine, case2_out_data);
+        test_tensor_t src_u8_ts(src_u8, engine, src_u8_data);
+        test_tensor_t dst_u8_ts(dst_u8, engine, case1_out_data);
+        test_tensor_t dst_u8_case2_ts(dst_u8, engine, case2_out_data);
 
         // -------------------------case 1----------------------------------
         ASSERT_EQ(run_graph(g, {src_u8_ts}, {dst_u8_ts}, *engine, *strm),
@@ -780,8 +780,8 @@ TEST(test_pool_execute, MaxPool) {
     cp.query_logical_tensor(dst_lt.id, &lt);
     ASSERT_EQ(lt.layout_type, graph::layout_type::strided);
 
-    test_tensor src_ts(src_lt, eng, src);
-    test_tensor dst_ts(lt, eng, dst);
+    test_tensor_t src_ts(src_lt, eng, src);
+    test_tensor_t dst_ts(lt, eng, dst);
 
     graph::stream_t *strm = get_stream();
     cp.execute(strm, {src_ts.get()}, {dst_ts.get()});
@@ -842,8 +842,8 @@ TEST(test_pool_execute, MaxPoolwithCache) {
     cp.query_logical_tensor(dst_lt.id, &lt);
     ASSERT_EQ(lt.layout_type, graph::layout_type::strided);
 
-    test_tensor src_ts(src_lt, eng, src);
-    test_tensor dst_ts(lt, eng, dst);
+    test_tensor_t src_ts(src_lt, eng, src);
+    test_tensor_t dst_ts(lt, eng, dst);
 
     graph::stream_t *strm = get_stream();
     cp.execute(strm, {src_ts.get()}, {dst_ts.get()});
@@ -860,8 +860,8 @@ TEST(test_pool_execute, MaxPoolwithCache) {
     std::vector<float> ref_dst2 {-1.0, 2.0, 4.0, 2.0};
     std::vector<float> dst2(ref_dst2.size(), 0.0);
 
-    test_tensor src_ts2(src_lt, eng, src2);
-    test_tensor dst_ts2(lt, eng, dst2);
+    test_tensor_t src_ts2(src_lt, eng, src2);
+    test_tensor_t dst_ts2(lt, eng, dst2);
 
     cp.execute(strm, {src_ts2.get()}, {dst_ts2.get()});
     strm->wait();
@@ -996,9 +996,9 @@ TEST(test_pool_execute, MaxPoolBackward) {
     cp.query_logical_tensor(diff_src_lt.id, &lt);
     ASSERT_EQ(lt.layout_type, graph::layout_type::strided);
 
-    test_tensor src_ts(src_lt, eng, src);
-    test_tensor diff_dst_ts(diff_dst_lt, eng, diff_dst);
-    test_tensor diff_src_ts(lt, eng, diff_src);
+    test_tensor_t src_ts(src_lt, eng, src);
+    test_tensor_t diff_dst_ts(diff_dst_lt, eng, diff_dst);
+    test_tensor_t diff_src_ts(lt, eng, diff_src);
 
     graph::stream_t *strm = get_stream();
     cp.execute(strm, {src_ts.get(), diff_dst_ts.get()}, {diff_src_ts.get()});
@@ -1059,9 +1059,9 @@ TEST(test_pool_execute, MaxPoolBackwardPlainGrad) {
     std::vector<float> diff_dst(product(output_dims), 1);
     std::vector<float> diff_src(product(input_dims), 1);
 
-    test_tensor src_ts(src_lt, eng, src);
-    test_tensor diff_dst_ts(diff_dst_lt, eng, diff_dst);
-    test_tensor diff_src_ts(lt, eng, diff_src);
+    test_tensor_t src_ts(src_lt, eng, src);
+    test_tensor_t diff_dst_ts(diff_dst_lt, eng, diff_dst);
+    test_tensor_t diff_src_ts(lt, eng, diff_src);
 
     graph::stream_t *strm = get_stream();
     cp.execute(strm, {src_ts.get(), diff_dst_ts.get()}, {diff_src_ts.get()});
@@ -1165,9 +1165,9 @@ TEST(test_pool_execute_subgraph_int8, Maxpool) {
         g.add_op(&qout_op);
         g.finalize();
 
-        test_tensor src_u8_ts(src_u8, engine, src_u8_data);
-        test_tensor dst_u8_ts(dst_u8, engine, case1_out_data);
-        test_tensor dst_u8_case2_ts(dst_u8, engine, case2_out_data);
+        test_tensor_t src_u8_ts(src_u8, engine, src_u8_data);
+        test_tensor_t dst_u8_ts(dst_u8, engine, case1_out_data);
+        test_tensor_t dst_u8_case2_ts(dst_u8, engine, case2_out_data);
 
         // -------------------------case 1----------------------------------
         ASSERT_EQ(run_graph(g, {src_u8_ts}, {dst_u8_ts}, *engine, *strm),
@@ -1281,9 +1281,9 @@ TEST(test_pool_execute_subgraph_int8, MaxpoolAsymmetric) {
     g.add_op(&qout_op);
     g.finalize();
 
-    test_tensor src_u8_ts(src_u8, engine, src_u8_data);
-    test_tensor dst_u8_ts(dst_u8, engine, case1_out_data);
-    test_tensor dst_u8_case2_ts(dst_u8, engine, case2_out_data);
+    test_tensor_t src_u8_ts(src_u8, engine, src_u8_data);
+    test_tensor_t dst_u8_ts(dst_u8, engine, case1_out_data);
+    test_tensor_t dst_u8_case2_ts(dst_u8, engine, case2_out_data);
 
     // -------------------------case 1----------------------------------
     ASSERT_EQ(run_graph(g, {src_u8_ts}, {dst_u8_ts}, *engine, *strm),
@@ -1457,11 +1457,11 @@ public:
             cp.query_logical_tensor(add_dst_lt.id, &lt);
             ASSERT_EQ(lt.layout_type, graph::layout_type::strided);
 
-            test_tensor src_ts(src_lt, eng);
+            test_tensor_t src_ts(src_lt, eng);
             src_ts.fill<bfloat16_t>(4.0);
-            test_tensor post_src_ts(post_src_lt, eng);
+            test_tensor_t post_src_ts(post_src_lt, eng);
             post_src_ts.fill<bfloat16_t>(2.0);
-            test_tensor add_dst_ts(add_dst_lt, eng);
+            test_tensor_t add_dst_ts(add_dst_lt, eng);
 
             graph::stream_t *strm = get_stream();
             ASSERT_EQ(cp.execute(strm, {src_ts.get(), post_src_ts.get()},
@@ -1627,8 +1627,8 @@ TEST(test_pool_execute_subgraph_int8, DequantizePoolReshapeQunatize) {
         g.add_op(&qout_op);
         g.finalize();
 
-        test_tensor src_s8_ts(dq_in_s8, engine, src_s8_data);
-        test_tensor case1_dst_s8_ts(q_out_s8, engine, case1_dst_s8_data);
+        test_tensor_t src_s8_ts(dq_in_s8, engine, src_s8_data);
+        test_tensor_t case1_dst_s8_ts(q_out_s8, engine, case1_dst_s8_data);
         graph::pass::pass_base_ptr apass
                 = get_pass("x8_pool_reshape_transpose");
         apass->run(g);
