@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -33,10 +33,10 @@ void compute_ref_conv_bwd_bias(const test_convolution_sizes_t &c,
 
     const memory::desc bias_d = diff_bias.get_desc();
     const memory::desc dst_d = diff_dst.get_desc();
-    const dnnl::impl::memory_desc_wrapper diff_bias_mdw(bias_d.data);
-    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(dst_d.data);
+    const dnnl::impl::memory_desc_wrapper diff_bias_mdw(bias_d.get());
+    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(dst_d.get());
 
-    auto padded_oc = dst_d.data.padded_dims[1];
+    auto padded_oc = dst_d.get_padded_dims()[1];
 
     dnnl::impl::parallel_nd(
             c.ng, c.oc / c.ng, [&](memory::dim g, memory::dim oc) {
@@ -68,12 +68,12 @@ void compute_ref_conv_bwd_weights(const test_convolution_sizes_t &c,
     const memory::desc src_d = src.get_desc();
     const memory::desc weights_d = diff_weights.get_desc();
     const memory::desc dst_d = diff_dst.get_desc();
-    const dnnl::impl::memory_desc_wrapper src_mdw(src_d.data);
-    const dnnl::impl::memory_desc_wrapper diff_weights_mdw(weights_d.data);
-    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(dst_d.data);
+    const dnnl::impl::memory_desc_wrapper src_mdw(src_d.get());
+    const dnnl::impl::memory_desc_wrapper diff_weights_mdw(weights_d.get());
+    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(dst_d.get());
 
-    auto padded_ic = src_d.data.padded_dims[1];
-    auto padded_oc = dst_d.data.padded_dims[1];
+    auto padded_ic = src_d.get_padded_dims()[1];
+    auto padded_oc = dst_d.get_padded_dims()[1];
 
     dnnl::impl::parallel_nd(c.ng, c.oc / c.ng, c.ic / c.ng, c.kh, c.kw,
             [&](memory::dim g, memory::dim oc, memory::dim ic, memory::dim kh,
@@ -118,10 +118,10 @@ void compute_ref_conv_bwd_weights(const test_convolution_sizes_t &c,
 
 template <typename data_t_src, typename data_t_diff_dst,
         typename data_t_diff_weights, typename data_t_diff_bias>
-class convolution_backward_weights_test
+class convolution_backward_weights_test_t
     : public ::testing::TestWithParam<test_convolution_params_t> {
 protected:
-    virtual void SetUp() {
+    void SetUp() override {
         auto p = ::testing::TestWithParam<
                 test_convolution_params_t>::GetParam();
 
@@ -138,18 +138,62 @@ protected:
                                         memory::format_tag::owi,
                                         memory::format_tag::ohwi,
                                         memory::format_tag::odhwi)))
-                        && data_traits<data_t_src>::data_type
+                        && data_traits_t<data_t_src>::data_type
                                 == memory::data_type::f32
-                        && data_traits<data_t_diff_dst>::data_type
+                        && data_traits_t<data_t_diff_dst>::data_type
                                 == memory::data_type::f32
-                        && data_traits<data_t_diff_weights>::data_type
+                        && data_traits_t<data_t_diff_weights>::data_type
                                 == memory::data_type::f32
                         && check_cuda_alg_format(p.formats.dst_format,
                                 p.formats.weights_format, p.aalgorithm)),
                 "format is not supported.");
 
+        SKIP_IF_HIP(!(hip_check_format_tags(p.formats.src_format)
+                            && hip_check_format_tags(p.formats.dst_format)
+                            && (hip_check_format_tags(p.formats.weights_format)
+                                    || (impl::utils::one_of(
+                                            p.formats.weights_format,
+                                            /* weights formats */
+                                            memory::format_tag::gowi,
+                                            memory::format_tag::gohwi,
+                                            memory::format_tag::godhwi,
+                                            memory::format_tag::owi,
+                                            memory::format_tag::ohwi,
+                                            memory::format_tag::odhwi)))
+                            && data_traits_t<data_t_src>::data_type
+                                    == memory::data_type::f32
+                            && data_traits_t<data_t_diff_dst>::data_type
+                                    == memory::data_type::f32
+                            && data_traits_t<data_t_diff_weights>::data_type
+                                    == memory::data_type::f32
+                            && check_hip_alg_format(p.formats.dst_format,
+                                    p.formats.weights_format, p.aalgorithm)),
+                "Format is not supported.");
+
+        SKIP_IF_GENERIC(
+                !(generic_check_format_tags(p.formats.src_format)
+                        && generic_check_format_tags(p.formats.dst_format)
+                        && (generic_check_format_tags(p.formats.weights_format)
+                                || (impl::utils::one_of(
+                                        p.formats.weights_format,
+                                        memory::format_tag::goiw,
+                                        memory::format_tag::goihw,
+                                        memory::format_tag::goidhw,
+                                        memory::format_tag::oiw,
+                                        memory::format_tag::oihw,
+                                        memory::format_tag::oidhw,
+                                        memory::format_tag::bacd,
+                                        memory::format_tag::bcda,
+                                        memory::format_tag::acbde,
+                                        memory::format_tag::iohw,
+                                        memory::format_tag::hwigo)))
+                        && check_generic_dt<data_t_src>()
+                        && check_generic_dt<data_t_diff_dst>()
+                        && check_generic_dt<data_t_diff_weights>()),
+                "Format is not supported.");
+
         catch_expected_failures(
-                [=]() { Test(); }, p.expect_to_fail, p.expected_status);
+                [&]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
 
     bool cuda_check_format_tags(memory::format_tag tag) {
@@ -160,6 +204,12 @@ protected:
                 memory::format_tag::acdeb);
     }
 
+    bool hip_check_format_tags(memory::format_tag tag) {
+        return impl::utils::one_of(tag, memory::format_tag::ab,
+                memory::format_tag::abc, memory::format_tag::abcd,
+                memory::format_tag::abcde, memory::format_tag::abcdef);
+    }
+
     bool check_cuda_alg_format(memory::format_tag dst_fmt,
             memory::format_tag wei_fmt, algorithm alg) {
         bool res = dst_fmt == wei_fmt;
@@ -173,6 +223,35 @@ protected:
         return res;
     }
 
+    bool check_hip_alg_format(memory::format_tag dst_fmt,
+            memory::format_tag wei_fmt, algorithm alg) {
+        bool res = dst_fmt == wei_fmt;
+        if (alg == dnnl::algorithm::convolution_winograd) {
+            res = res
+                    && impl::utils::one_of(wei_fmt, memory::format_tag::ab,
+                            memory::format_tag::abc, memory::format_tag::abcd,
+                            memory::format_tag::abcde,
+                            memory::format_tag::abcdef);
+        }
+        return res;
+    }
+
+    bool generic_check_format_tags(memory::format_tag tag) {
+        return impl::utils::one_of(tag, memory::format_tag::ab,
+                memory::format_tag::abc, memory::format_tag::abcd,
+                memory::format_tag::abcde, memory::format_tag::abcdef,
+                memory::format_tag::acb, memory::format_tag::acdb,
+                memory::format_tag::acdeb, memory::format_tag::any);
+    }
+
+    template <typename dt>
+    bool check_generic_dt() {
+        return impl::utils::one_of(data_traits_t<dt>::data_type,
+                memory::data_type::f32, memory::data_type::bf16,
+                memory::data_type::f16, memory::data_type::s32,
+                memory::data_type::s8, memory::data_type::u8);
+    }
+
     void Test() {
         auto p = ::testing::TestWithParam<
                 test_convolution_params_t>::GetParam();
@@ -180,13 +259,13 @@ protected:
         ASSERT_EQ(p.aalgorithm, algorithm::convolution_direct);
         auto eng = get_test_engine();
         auto strm = stream(eng);
-        memory::data_type data_type_src = data_traits<data_t_src>::data_type;
+        memory::data_type data_type_src = data_traits_t<data_t_src>::data_type;
         memory::data_type data_type_diff_dst
-                = data_traits<data_t_diff_dst>::data_type;
+                = data_traits_t<data_t_diff_dst>::data_type;
         memory::data_type data_type_diff_weights
-                = data_traits<data_t_diff_weights>::data_type;
+                = data_traits_t<data_t_diff_weights>::data_type;
         memory::data_type data_type_diff_bias
-                = data_traits<data_t_diff_bias>::data_type;
+                = data_traits_t<data_t_diff_bias>::data_type;
 
         test_convolution_sizes_t cd = p.sizes;
 
@@ -233,22 +312,19 @@ protected:
                 right_padding(cd.ih, cd.oh, cd.kh, cd.padh, cd.strh, cd.dilh),
                 right_padding(cd.iw, cd.ow, cd.kw, cd.padw, cd.strw, cd.dilw)};
 
-        auto conv_desc = convolution_forward::desc(prop_kind::forward_training,
-                p.aalgorithm, c_src_desc, c_weights_desc_f, c_diff_bias_desc,
-                c_dst_desc_f, {cd.strh, cd.strw}, {cd.dilh, cd.dilw},
-                {cd.padh, cd.padw}, padR);
-
-        auto conv_bwd_weights_desc = convolution_backward_weights::desc(
-                p.aalgorithm, c_src_desc, c_diff_weights_desc, c_diff_bias_desc,
-                c_diff_dst_desc, {cd.strh, cd.strw}, {cd.dilh, cd.dilw},
-                {cd.padh, cd.padw}, padR);
-
-        auto conv_primitive_desc
-                = convolution_forward::primitive_desc(conv_desc, eng);
+        auto conv_primitive_desc = convolution_forward::primitive_desc(eng,
+                prop_kind::forward_training, p.aalgorithm, c_src_desc,
+                c_weights_desc_f, c_diff_bias_desc, c_dst_desc_f,
+                {cd.strh, cd.strw}, {cd.dilh, cd.dilw}, {cd.padh, cd.padw},
+                padR);
 
         auto conv_bwd_weights_primitive_desc
-                = convolution_backward_weights::primitive_desc(
-                        conv_bwd_weights_desc, eng, conv_primitive_desc);
+                = convolution_backward_weights::primitive_desc(eng,
+                        p.aalgorithm, c_src_desc, c_diff_weights_desc,
+                        c_diff_bias_desc, c_diff_dst_desc, {cd.strh, cd.strw},
+                        {cd.dilh, cd.dilw}, {cd.padh, cd.padw}, padR,
+                        conv_primitive_desc);
+
         conv_bwd_weights_primitive_desc
                 = convolution_backward_weights::primitive_desc(
                         conv_bwd_weights_primitive_desc
@@ -267,6 +343,8 @@ protected:
                             query::exec_arg_md, DNNL_ARG_DIFF_BIAS)
                 == conv_bwd_weights_primitive_desc.diff_bias_desc());
 
+        EXPECT_ANY_THROW(convolution_backward_weights(
+                conv_bwd_weights_primitive_desc, {}));
         convolution_backward_weights(conv_bwd_weights_primitive_desc)
                 .execute(strm,
                         {{DNNL_ARG_DIFF_DST, c_diff_dst.get()},
