@@ -32,10 +32,10 @@ protected:
     engine eng = get_test_engine();
     void SetUp() override {}
 
-    static primitive_attr gen_attr_with_scales() {
+    static primitive_attr gen_attr_with_scales(bool with_wei = true) {
         primitive_attr attr;
         attr.set_scales_mask(DNNL_ARG_SRC, 0);
-        attr.set_scales_mask(DNNL_ARG_WEIGHTS, 0);
+        if (with_wei) attr.set_scales_mask(DNNL_ARG_WEIGHTS, 0);
         attr.set_scales_mask(DNNL_ARG_DST, 0);
         return attr;
     }
@@ -84,14 +84,19 @@ TEST_F(attr_quantization_test_t, TestBNorm) {
                 eng, prop_kind::forward_inference, md, md, 0.1f, flags));
         CHECK_UNIMPL(batch_normalization_forward::primitive_desc(eng,
                 prop_kind::forward_inference, md, md, 0.1f, flags,
-                gen_attr_with_scales()));
+                gen_attr_with_scales(
+                        /* with_wei = false, qunatization is not supported */)));
 
-        for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_BIAS,
-                     DNNL_ARG_MEAN, DNNL_ARG_VARIANCE, DNNL_ARG_DST}) {
+        for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST}) {
             CHECK_UNIMPL(batch_normalization_forward::primitive_desc(eng,
                     prop_kind::forward_inference, md, md, 0.1f, flags,
                     gen_attr_with_zp(arg)));
         }
+        for (auto arg : {DNNL_ARG_BIAS, DNNL_ARG_MEAN, DNNL_ARG_VARIANCE}) {
+            CHECK_INVALID(batch_normalization_forward::primitive_desc(eng,
+                    prop_kind::forward_inference, md, md, 0.1f, flags,
+                    gen_attr_with_zp(arg)));
+        }
     }
 }
 
@@ -106,8 +111,15 @@ TEST_F(attr_quantization_test_t, TestBinary) {
         else
             CHECK_OK(binary::primitive_desc(eng, algorithm::binary_add, md, md,
                     md, gen_attr_with_scales(arg)));
-        CHECK_UNIMPL(binary::primitive_desc(
-                eng, algorithm::binary_add, md, md, md, gen_attr_with_zp(arg)));
+    }
+
+    for (auto arg : {DNNL_ARG_SRC_0, DNNL_ARG_SRC_1, DNNL_ARG_DST}) {
+        if (arg == DNNL_ARG_SRC_1)
+            CHECK_INVALID(binary::primitive_desc(eng, algorithm::binary_add, md,
+                    md, md, gen_attr_with_zp(arg)));
+        else
+            CHECK_UNIMPL(binary::primitive_desc(eng, algorithm::binary_add, md,
+                    md, md, gen_attr_with_zp(arg)));
     }
 }
 
@@ -117,11 +129,10 @@ TEST_F(attr_quantization_test_t, TestConcat) {
     memory::desc md {{1, 16, 3, 3}, data_type::s8, tag::abcd};
     CHECK_OK(concat::primitive_desc(eng, 1, {md, md}));
 
-    for (auto arg :
-            {DNNL_ARG_MULTIPLE_SRC, DNNL_ARG_MULTIPLE_SRC + 1, DNNL_ARG_DST}) {
+    for (auto arg : {DNNL_ARG_MULTIPLE_SRC, DNNL_ARG_MULTIPLE_SRC + 1}) {
         CHECK_OK(concat::primitive_desc(
                 eng, 1, {md, md}, gen_attr_with_scales(arg)));
-        CHECK_UNIMPL(concat::primitive_desc(
+        CHECK_INVALID(concat::primitive_desc(
                 eng, 1, {md, md}, gen_attr_with_zp(arg)));
     }
 }
@@ -359,7 +370,9 @@ TEST_F(attr_quantization_test_t, TestEltwise) {
                 eng, prop_kind::forward, algorithm::eltwise_relu, md, md, 0.f));
 
         CHECK_UNIMPL(eltwise_forward::primitive_desc(eng, prop_kind::forward,
-                algorithm::eltwise_relu, md, md, 0.f, gen_attr_with_scales()));
+                algorithm::eltwise_relu, md, md, 0.f,
+                gen_attr_with_scales(
+                        /* with_wei = false, quantization is not supported */)));
 
         for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_DST}) {
             CHECK_UNIMPL(eltwise_forward::primitive_desc(eng,
@@ -383,8 +396,7 @@ TEST_F(attr_quantization_test_t, TestInnerProduct) {
     CHECK_OK(inner_product_forward::primitive_desc(eng, prop_kind::forward,
             src_md, wei_md, dst_md, gen_attr_with_scales()));
 
-    for (auto arg :
-            {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_BIAS, DNNL_ARG_DST}) {
+    for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST}) {
         CHECK_UNIMPL(
                 inner_product_forward::primitive_desc(eng, prop_kind::forward,
                         src_md, wei_md, dst_md, gen_attr_with_zp(arg)));
@@ -401,16 +413,23 @@ TEST_F(attr_quantization_test_t, TestLNorm) {
 
     CHECK_OK(layer_normalization_forward::primitive_desc(
             eng, prop_kind::forward_inference, md, md, stat_md, 0.1f, flags));
+    CHECK_UNIMPL(layer_normalization_forward::primitive_desc(eng,
+            prop_kind::forward_inference, md, md, stat_md, 0.1f, flags,
+            gen_attr_with_scales(/* WEIGHTS are not supported */)));
     CHECK_OK(layer_normalization_forward::primitive_desc(eng,
             prop_kind::forward_inference, md, md, stat_md, 0.1f, flags,
-            gen_attr_with_scales()));
+            gen_attr_with_scales(/* with_wei = */ false)));
 
-    for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_MEAN, DNNL_ARG_VARIANCE,
-                 DNNL_ARG_WEIGHTS, DNNL_ARG_BIAS, DNNL_ARG_DST}) {
+    for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST}) {
         CHECK_UNIMPL(layer_normalization_forward::primitive_desc(eng,
                 prop_kind::forward_inference, md, md, stat_md, 0.1f, flags,
                 gen_attr_with_zp(arg)));
     }
+    for (auto arg : {DNNL_ARG_MEAN, DNNL_ARG_VARIANCE, DNNL_ARG_BIAS}) {
+        CHECK_INVALID(layer_normalization_forward::primitive_desc(eng,
+                prop_kind::forward_inference, md, md, stat_md, 0.1f, flags,
+                gen_attr_with_zp(arg)));
+    }
 }
 
 TEST_F(attr_quantization_test_t, TestLRN) {
@@ -420,7 +439,9 @@ TEST_F(attr_quantization_test_t, TestLRN) {
                 algorithm::lrn_across_channels, md, md, 5, 1.f, 0.75f, 1.0f));
         CHECK_UNIMPL(lrn_forward::primitive_desc(eng,
                 prop_kind::forward_inference, algorithm::lrn_across_channels,
-                md, md, 5, 1.f, 0.75f, 1.0f, gen_attr_with_scales()));
+                md, md, 5, 1.f, 0.75f, 1.0f,
+                gen_attr_with_scales(
+                        /* with_wei = false, quantization is not supported */)));
 
         for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_DST}) {
             CHECK_UNIMPL(lrn_forward::primitive_desc(eng,
@@ -483,9 +504,9 @@ TEST_F(attr_quantization_test_t, TestMatmul) {
             if (arg == DNNL_ARG_WEIGHTS) {
                 CHECK_OK(matmul::primitive_desc(eng, a_md, b_md, c_md,
                         gen_attr_with_scales(arg, 1 << 1)));
-                CHECK_OK(matmul::primitive_desc(eng, a_md, b_md, c_md,
-                        gen_attr_with_scales(arg, (1 << 1) + (1 << 0))));
                 if (b_dt == data_type::s8) {
+                    CHECK_OK(matmul::primitive_desc(eng, a_md, b_md, c_md,
+                            gen_attr_with_scales(arg, (1 << 1) + (1 << 0))));
                     // Groups non divisible by 32 are not supported.
                     CHECK_UNIMPL(matmul::primitive_desc(eng, a_md, b_md, c_md,
                             gen_attr_with_scales(arg, (1 << 1) + (1 << 0),
@@ -591,10 +612,11 @@ TEST_F(attr_quantization_test_t, TestPool) {
     CHECK_OK(pooling_forward::primitive_desc(eng, prop_kind::forward_inference,
             algorithm::pooling_max, src_md, dst_md, {2, 2}, {2, 2}, {0, 0},
             {0, 0}, {0, 0}));
-    CHECK_UNIMPL(
-            pooling_forward::primitive_desc(eng, prop_kind::forward_inference,
-                    algorithm::pooling_max, src_md, dst_md, {2, 2}, {2, 2},
-                    {0, 0}, {0, 0}, {0, 0}, gen_attr_with_scales()));
+    CHECK_UNIMPL(pooling_forward::primitive_desc(eng,
+            prop_kind::forward_inference, algorithm::pooling_max, src_md,
+            dst_md, {2, 2}, {2, 2}, {0, 0}, {0, 0}, {0, 0},
+            gen_attr_with_scales(
+                    /* with_wei = false, quantization is not supported */)));
 
     for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_DST}) {
         CHECK_UNIMPL(pooling_forward::primitive_desc(eng,
@@ -614,7 +636,9 @@ TEST_F(attr_quantization_test_t, TestPReLU) {
             eng, prop_kind::forward, data_md, weights_md, data_md));
 
     CHECK_UNIMPL(prelu_forward::primitive_desc(eng, prop_kind::forward, data_md,
-            weights_md, data_md, gen_attr_with_scales()));
+            weights_md, data_md,
+            gen_attr_with_scales(
+                    /* with_wei = false, quantization is not supported */)));
 
     for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_DST}) {
         CHECK_UNIMPL(prelu_forward::primitive_desc(eng, prop_kind::forward,
@@ -628,8 +652,10 @@ CPU_TEST_F(attr_quantization_test_t, TestReorder) {
     CHECK_OK(reorder::primitive_desc(eng, src_md, eng, dst_md));
 
     for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_DST}) {
-        CHECK_OK(reorder::primitive_desc(
-                eng, src_md, eng, dst_md, gen_attr_with_scales()));
+        CHECK_UNIMPL(reorder::primitive_desc(eng, src_md, eng, dst_md,
+                gen_attr_with_scales(/* WEIGHTS are not supported */)));
+        CHECK_OK(reorder::primitive_desc(eng, src_md, eng, dst_md,
+                gen_attr_with_scales(/* with_wei = */ false)));
         CHECK_OK(reorder::primitive_desc(
                 eng, src_md, eng, dst_md, gen_attr_with_zp(arg)));
     }
@@ -679,9 +705,8 @@ TEST_F(attr_quantization_test_t, TestRNN) {
                         attr));
     }
 
-    for (auto arg : {DNNL_ARG_SRC_LAYER, DNNL_ARG_SRC_ITER, DNNL_ARG_SRC_ITER_C,
-                 DNNL_ARG_WEIGHTS_LAYER, DNNL_ARG_WEIGHTS_ITER, DNNL_ARG_BIAS,
-                 DNNL_ARG_DST_LAYER, DNNL_ARG_DST_ITER, DNNL_ARG_DST_ITER_C}) {
+    for (auto arg :
+            {DNNL_ARG_SRC_LAYER, DNNL_ARG_WEIGHTS_LAYER, DNNL_ARG_DST_LAYER}) {
         CHECK_UNIMPL(
                 lstm_forward::primitive_desc(eng, prop_kind::forward_inference,
                         rnn_direction::unidirectional_left2right, src_layer_md,
@@ -698,8 +723,10 @@ TEST_F(attr_quantization_test_t, TestShuffle) {
 
     CHECK_OK(shuffle_forward::primitive_desc pd(
             eng, prop_kind::forward, md, md, 1, 4));
-    CHECK_UNIMPL(shuffle_forward::primitive_desc pd(
-            eng, prop_kind::forward, md, md, 1, 4, gen_attr_with_scales()));
+    CHECK_UNIMPL(shuffle_forward::primitive_desc pd(eng, prop_kind::forward, md,
+            md, 1, 4,
+            gen_attr_with_scales(
+                    /* with_wei = false, quantization is not supported */)));
 
     for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_DST}) {
         CHECK_UNIMPL(shuffle_forward::primitive_desc pd(
@@ -729,8 +756,9 @@ TEST_F(attr_quantization_test_t, TestSum) {
     SKIP_IF_HIP(true, "Unsupported operator for HIP");
     memory::desc md {{1, 16, 3, 3}, data_type::s8, tag::abcd};
     CHECK_OK(sum::primitive_desc(eng, {1.f, 1.f}, {md, md}));
-    CHECK_UNIMPL(sum::primitive_desc(
-            eng, {1.f, 1.f}, {md, md}, gen_attr_with_scales()));
+    CHECK_UNIMPL(sum::primitive_desc(eng, {1.f, 1.f}, {md, md},
+            gen_attr_with_scales(
+                    /* with_wei = false, quantization is not supported */)));
 
     for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_DST}) {
         CHECK_UNIMPL(sum::primitive_desc(
